<!DOCTYPE html>
<html lang="en-GB"><!-- Sets the correct page language, IMPORTANT DO NOT REMOVE -->
<head>
	<meta charset="utf-8">
	<meta name="robots" content="noindex, nofollow">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title>Joomla: unsupported PHP version</title><!-- Sets the page title, IMPORTANT DO NOT REMOVE -->
	<style>html{background:#ee2a00;background:-moz-radial-gradient(center,ellipse cover,#f1f1f1 0,#ee2a00 100%);background:-webkit-radial-gradient(center,ellipse cover,#f1f1f1 0,#ee2a00 100%);background:radial-gradient(#f1f1f1 0,#ee2a00 100%) no-repeat fixed}body{color:#555;background-color:#0000;margin:0;padding:0;font:14px/18px sans-serif}ol,ul{margin:0;padding:0;list-style:none}a{color:#0084b4;text-decoration:none}a:hover,a:focus{text-decoration:underline}p a{line-height:inherit}.container{flex-direction:column;justify-content:center;align-items:center;width:100%;height:100vh;margin:0 auto;display:flex;position:relative;overflow:hidden}.alert-main{background:#fff;border:1px solid #0000001a;border-radius:5px;margin:0 20px;padding:20px 60px;display:block;position:relative;box-shadow:0 0 10px #0000000d}svg{z-index:-1;width:400px;position:absolute;bottom:-120px;right:-70px;transform:rotate(10deg)}h1,p{z-index:10;text-align:center;text-rendering:optimizeLegibility;position:relative}h1{text-shadow:0 1px 2px #0003;margin:18px 0 0;font-size:40px;font-weight:200;line-height:1}p,label{color:#777;margin:10px 0 20px;font-size:18px;font-weight:300;line-height:25px}p a{color:#1c3d5c;font-weight:700}.link-help{background-color:#f5f5f5;border:1px solid #0000001a;border-radius:.25rem;padding:.4rem .85rem;font-size:1rem;font-weight:400;text-decoration:none}.link-help:hover{background-color:#eee;text-decoration:none}.footer{text-align:left;margin:8px 20px;font-size:11px}.footer ul{margin-bottom:5px}.footer li{margin:0 5px;line-height:20px;display:inline}.footer li,.footer a{color:#1c3d5c}.footer a:hover{color:#59b0ff}.links{text-align:center;margin-top:4rem;margin-left:auto;margin-right:auto;font-size:1rem;display:block}.links li{margin-top:20px;display:inline-block}@media screen and (width<=480px){.container{height:auto;padding-top:20px;padding-bottom:20px}h1{font-size:30px}.link-help{white-space:nowrap}}</style><!-- Sets the page styling, IMPORTANT DO NOT REMOVE -->
	<script>window.errorLocale={"af-ZA":{"language":"Engels GB","header":"Jammer, jou PHP weergawe word nie ondersteun nie.","text1":"Jou gasheer moet PHP weergawe {{phpversion}} of nuwer hê om hierdie weergawe van Joomla te gebruik.","help-url-text":"Help my om dit op te los"},"ar-AA":{"language":"اللغة العربية بالكود الموحد","header":"عذرًا، إصدار PHP الخاص بك غير مدعوم.","text1":"يحتاج الخادم الخاص بك إلى استخدام إصدار PHP {{phpversion}} أو أحدث لتشغيل هذا الإصدار من جوملا.","help-url-text":"ساعدني في حل هذا"},"be-BY":{"language":"Беларуская BY","header":"Выбачайце, Ваша версія PHP не падтрымліваецца","text1":"Каб запусціць гэтую версію Joomla, Ваш хост мусіць мець PHP {{phpversion}} або навейшы.","help-url-text":"Дапамажыце мне выправіць гэта"},"bg-BG":{"language":"Български","header":"Вашата PHP версия не се поддържа.","text1":"Вашият хост трябва да използва PHP версия {{phpversion}} или по -нова, за да стартира тази версия на Joomla.","help-url-text":"Помогнете ми да разреша това"},"ca-ES":{"language":"Català ca_ES","header":"Ho sentim, la teva versió de PHP no és compatible.","text1":"Cal que el teu servidor utilitzi la versió de PHP {{phpversion}} o posterior per executar aquesta versió de Joomla.","help-url-text":"Ajuda'm a resoldre això"},"cs-CZ":{"language":"Čeština","header":"Omlouváme se, vaše verze PHP není podporována.","text1":"Pro spuštění této verze Joomla musí váš hostitel použít PHP verzi {{phpversion}} nebo novější","help-url-text":"Pomozte mi to vyřešit"},"cy-GB":{"language":"Cymraeg GB","header":"Mae'n ddrwg gennym ond dydi ni ddim yn cefnogi eich fersiwn PHP","text1":"Mae eich lletywr angen defnyddio fersiwn PHP {{phpversion}} neu ddiweddarach i redeg y fersiwn hon o Joomla.","help-url-text":"Helpwch fi i ddatrys hyn"},"da-DK":{"language":"Dansk DK","header":"Beklager, din PHP version er ikke understøttet.","text1":"Din host skal anvende PHP version {{phpversion}} eller nyere for at køre denne version af Joomla.","help-url-text":"Hjælp mig med at løse dette"},"de-AT":{"language":"Deutsch AT","header":"Die verwendete PHP-Version wird leider nicht unterstützt.","text1":"Der Webserver muss die PHP-Version {{phpversion}} oder neuer verwenden, um diese Version von Joomla auszuführen.","help-url-text":"Weitere Informationen sind in der Dokumentation zu finden."},"de-CH":{"language":"Deutsch CH","header":"Die verwendete PHP-Version wird leider nicht unterstützt.","text1":"Der Webserver muss die PHP-Version {{phpversion}} oder neuer verwenden, um diese Version von Joomla auszuführen.","help-url-text":"Weitere Informationen sind in der Dokumentation zu finden."},"de-DE":{"language":"Deutsch DE","header":"Die verwendete PHP-Version wird leider nicht unterstützt.","text1":"Der Webserver muss die PHP-Version {{phpversion}} oder neuer verwenden, um diese Version von Joomla auszuführen.","help-url-text":"Weitere Informationen sind in der Dokumentation zu finden."},"de-LI":{"language":"Deutsch LI","header":"Die verwendete PHP-Version wird leider nicht unterstützt.","text1":"Der Webserver muss die PHP-Version {{phpversion}} oder neuer verwenden, um diese Version von Joomla auszuführen.","help-url-text":"Weitere Informationen sind in der Dokumentation zu finden."},"de-LU":{"language":"Deutsch LU","header":"Die verwendete PHP-Version wird leider nicht unterstützt.","text1":"Der Webserver muss die PHP-Version {{phpversion}} oder neuer verwenden, um diese Version von Joomla auszuführen.","help-url-text":"Weitere Informationen sind in der Dokumentation zu finden."},"el-GR":{"language":"Greek el-GR","header":"Λυπούμαστε, αυτή η έκδοση PHP δεν υποστηρίζεται.","text1":"Ο διακομιστής πρέπει να διαθέτει την έκδοση PHP {{phpversion}} ή νεότερη για την εκτέλεση αυτής της έκδοσης του Joomla.","help-url-text":"Βοήθεια για την επίλυση"},"en-AU":{"language":"English AU","header":"Sorry, your PHP version is not supported.","text1":"Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla.","help-url-text":"Help me resolve this"},"en-CA":{"language":"English CA","header":"Sorry, your PHP version is not supported.","text1":"Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla.","help-url-text":"Help me resolve this"},"en-GB":{"language":"English GB","header":"Sorry, your PHP version is not supported.","text1":"Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla.","help-url-text":"Help me resolve this"},"en-NZ":{"language":"English NZ","header":"Sorry, your PHP version is not supported.","text1":"Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla.","help-url-text":"Help me resolve this"},"en-US":{"language":"English US","header":"Sorry, your PHP version is not supported.","text1":"Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla.","help-url-text":"Help me resolve this"},"es-ES":{"language":"Español ES","header":"Lo sentimos, su versión de PHP no es compatible.","text1":"Para poder usar esta versión de Joomla su hospedaje necesita usar la versión {{phpversion}} de PHP o superior.","help-url-text":"Ayúdeme a resolver esto"},"et-EE":{"language":"Estonian","header":"Vabandust, aga sinu PHP versioon ei ole toetatud.","text1":"Sinu veebimajutuse pakkuja peab kasutama PHP versiooni {{phpversion}} või uuemat, et seda Joomla versiooni jooksutada.","help-url-text":"Aita mul seda lahendada"},"eu-ES":{"language":"Euskara ES","header":"Barka, zure PHP bertsioa ez da bateragarria.","text1":"Zure ostalariak PHP bertsio hau {{phpversion}}  edo berriagoa erabili behar du Joomla bertsio honekin funtzionatzeko.","help-url-text":"Lagundu niri hau konpontzen"},"fa-AF":{"language":"فارسی دری","header":"با عرض پوزش، نسخه‌ی PHP شما پشتیبانی نمی‌شود.","text1":"شما برای اجرای این نسخه از جوملا، به نسخه‌ی PHP {{phpversion}} یا بالاتر احتیاج دارید.","help-url-text":"برای حل این مشکل کمک کن"},"fa-IR":{"language":"Persian IR","header":"متأسفیم، نسخه PHP شما پشتیبانی نمی شود.","text1":"میزبان شما برای اجرای این نسخه از جوملا نیاز به استفاده از نسخه PHP {{phpversion}} یا نسخه جدیدتر دارد.","help-url-text":"به من کمک کنید تا این مسئله را حل کنم"},"fi-FI":{"language":"Suomi","header":"PHP-versiotasi ei tueta.","text1":"Palvelimen täytyy käyttää PHP-versiota {{phpversion}} tai uudempaa jotta voit käyttää tätä Joomlan versiota.","help-url-text":"Auta minua ratkaisemaan tämä"},"fr-CA":{"language":"Français FR","header":"Désolé, votre version de PHP n'est pas prise en charge.","text1":"Votre serveur a besoin d'utiliser la version {{phpversion}} de PHP ou supérieure pour exécuter cette version de Joomla.","help-url-text":"Aidez-moi à résoudre ceci"},"fr-FR":{"language":"Français FR","header":"Désolé, votre version de PHP n'est pas prise en charge.","text1":"Votre serveur a besoin d'utiliser la version {{phpversion}} de PHP ou supérieure pour exécuter cette version de Joomla.","help-url-text":"Aidez-moi à résoudre ceci"},"he-IL":{"language":"אנגלית GB","header":"מצטערים, גרסת ה- PHP שלך אינה נתמכת.","text1":"השרת מארח שלך צריך להשתמש בגרסת PHP {{phpversion}} ואילך כדי להריץ גירסה זו של ג'ומלה.","help-url-text":"עזור לי לפתור זאת"},"hr-HR":{"language":"Hrvatski HR","header":"Nažalost, vaša PHP verzija nije podržana.","text1":"Za pokretanje ove Joomla verzije, vaš host treba koristiti PHP verziju {{phpversion}} ili noviju.","help-url-text":"Pomozite mi riješiti ovo"},"hu-HU":{"language":"Magyar","header":"Sajnáljuk, de a PHP verziója nem támogatott.","text1":"Ennek a Joomla! verziónak a futtatásához a kiszolgálónak PHP {{phpversion}} vagy újabb verziót kell használnia.","help-url-text":"Segíts ezt megoldani"},"it-IT":{"language":"Italiano IT","header":"Siamo spiacenti, la tua versione di PHP non è supportata.","text1":"Il tuo host ha bisogno della versione di PHP {{phpversion}} o una più recente per avviare questa versione di Joomla.","help-url-text":"Aiutami a risolvere questo"},"ja-JP":{"language":"Japanese JP","header":"申し訳ありませんが、お使いのPHPバージョンはサポートされていません。","text1":"このバージョンのJoomlaを実行するには、ホストがPHPバージョン {{phpversion}} 以降を使用する必要があります。","help-url-text":"この問題を解決するには"},"ka-GE":{"language":"ქართული GE","header":"უკაცრავად, თქვენს PHP ვერსიას არა აქვს მხარდაჭერა.","text1":"Joomla! ამ ვერსიის დასაყენებლად თქვენი ჰოსტი უნდა იყენებდეს PHP {{phpversion}} ვერსიას ან უფრო ახალს.","help-url-text":"დამეხმარეთ პრობლემის მოგვარებაში"},"kk-KZ":{"language":"English GB","header":"Кешіріңіз, сіздің PHP нұсқаңызға қолдау көрсетілмейді.","text1":"Хостыңыз PHP {{phpversion}} немесе оданда жоғары нұсқасы болғанда ғана бұл нұсқадағы Joomla-ны іске қоса аласыз.","help-url-text":"Бұл мәселені шешуге көмектес"},"ko-KR":{"language":"Korean KO","header":"죄송합니다. 사용 중인 PHP 버전이 지원되지 않습니다.","text1":"해당 버전의 Joomla를 사용하기 위해서는 PHP 버전 {{phpversion}} 또는 그 상위 버전이 필요합니다.","help-url-text":"이 문제를 해결하도록 도와주세요."},"lt-LT":{"language":"Lietuvių LT","header":"Atsiprašome, Jūsų PHP versija yra nepalaikoma.","text1":"Kad paleisti šios versijos Joomla, Jūsų talpinimo serveris turi palaikyti {{phpversion}} arba vėlesnę PHP versiją.","help-url-text":"Padėkite man tai išspręsti"},"lv-LV":{"language":"Latviešu LV","header":"Diemžēl jūsu PHP versija netiek atbalstīta.","text1":"Lai palaistu šo Joomla versiju, serverim ir jāizmanto {{phpversion}} PHP versija vai jaunāka.","help-url-text":"Palīdziet man to atrisināt"},"mk-MK":{"language":"English GB","header":"За жал, вашата верзија на PHP не е поддржана.","text1":"Вашиот хост треба да користи PHP верзија {{phpversion}} или понова за да ја стартува оваа верзија на Joomla.","help-url-text":"Помогни ми да го решам ова"},"nl-BE":{"language":"Vlaams BE","header":"Sorry, uw PHP-versie wordt niet ondersteund.","text1":"Uw host moet PHP versie {{phpversion}} of nieuwer gebruiken om deze versie van Joomla uit te voeren.","help-url-text":"Help dit probleem op te lossen"},"nl-NL":{"language":"Engels GB","header":"Sorry, uw PHP-versie wordt niet ondersteund.","text1":"Uw host moet PHP versie {{phpversion}} of nieuwer gebruiken om deze versie van Joomla uit te voeren.","help-url-text":"Help dit probleem op te lossen"},"pl-PL":{"language":"Polski PL","header":"Niestety, Twoja wersja PHP nie jest obsługiwana.","text1":"Twój serwer musi używać PHP w wersji {{phpversion}} lub nowszej, aby zainstalować tę wersję Joomla.","help-url-text":"Pomoc w rozwiązaniu tego problemu"},"pt-BR":{"language":"Português do Brasil (pt-BR)","header":"Desculpe, sua versão do PHP não é suportada.","text1":"Seu servidor precisa usar a versão {{phpversion}} do PHP ou uma versão mais recente para executar esta versão do Joomla.","help-url-text":"Ajude-me a resolver isto"},"pt-PT":{"language":"Portuguese (pt-PT)","header":"Lamentamos, mas a sua versão de PHP não é suportada.","text1":"O seu alojamento terá de usar PHP na versão {{phpversion}}, ou mais recente, para poder utilizar esta versão do Joomla.","help-url-text":"Ajude-me a resolver isto"},"ro-RO":{"language":"Română RO","header":"Ne pare rău, versiunea dvs de PHP nu este acceptată.","text1":"Gazda dvs. trebuie să folosească versiunea PHP {{phpversion}} sau una mai nouă pentru a rula această versiune Joomla.","help-url-text":"Ajută-mă să rezolv această problemă"},"ru-RU":{"language":"Русский RU","header":"Используемая версия PHP не поддерживается","text1":"Для установки Joomla необходимо наличие PHP {{phpversion}} или более новой версии.","help-url-text":"Подробнее о решении проблемы"},"sk-SK":{"language":"Slovensky","header":"Prepáčte, ale vaša verzia PHP nie je podporovaná.","text1":"Aby táto verzia Joomla fungovala, váš hosting musí používať PHP vo verzii {{phpversion}} alebo novšej.","help-url-text":"Pomôžte mi vyriešiť to"},"sl-SI":{"language":"Slovenščina SI","header":"Žal vaša PHP različica ni podprta.","text1":"Vaš gostitelj mora uporabljati PHP različico  {{phpversion}} ali novejšo za zagon te različice Joomle.","help-url-text":"Pomagajte mi rešiti to težavo"},"sr-YU":{"language":"Srpski Latinica SRB","header":"Žao nam je vaša PHP verzija nije podržana.","text1":"Vaš host mora da koristi PHP verziju {{phpversion}} ili noviju da bi ova vezija Joomla! radila.","help-url-text":"Kako da rešim ovo"},"sv-SE":{"language":"Svensk SE","header":"Tyvärr, din PHP-version stöds inte.","text1":"Ditt webbhotell måste använda PHP-version {{phpversion}} eller nyare för att köra denna version av Joomla.","help-url-text":"Hjälp mig att lösa detta"},"ta-IN":{"language":"தமிழ் இந்தியா","header":"மன்னிக்கவும், உங்கள் PHP பதிப்பு ஆதரிக்கப்படவில்லை.","text1":"Joomla-வின் இந்தப் பதிப்பை இயக்க, தங்கள் புரவலர்-கணினி (host) ஆனது PHP பதிப்பு {{phpversion}} அல்லது அதற்கும் மேலான பதிப்பைப் பயன்படுத்த வேண்டும்.","help-url-text":"இதனைத் தீர்க்க எனக்கு உதவுக"},"th-TH":{"language":"ภาษาไทย","header":"ขออภัย ไม่รองรับเวอร์ชัน PHP ของคุณ","text1":"โฮสต์ของคุณต้องใช้เวอร์ชัน PHP {{phpversion}} หรือใหม่กว่าเพื่อเรียกใช้ Joomla เวอร์ชันนี้","help-url-text":"ช่วยฉันแก้ปัญหานี้"},"tr-TR":{"language":"Turkish TR","header":"Üzgünüz, PHP sürümünüz desteklenmiyor.","text1":"Joomla'nın bu sürümünü çalıştırmak için sunucunuzun PHP {{phpversion}} sürümünü veya daha yenisini kullanması gerekir.","help-url-text":"Bunu çözmeme yardım et"},"uk-UA":{"language":"Українська (UA)","header":"На жаль, ваша версія PHP не підтримується.","text1":"Ваш хост повинен використовувати версію PHP {{phpversion}} або новішу для запуску цієї версії Joomla.","help-url-text":"Допоможіть мені це вирішити"},"ur-PK":{"language":"انگریزی جی-بی","header":"معذرت، آپ کا پی ایچ پی ورژن تعاون یافتہ نہیں ہے۔","text1":"جملہ کے اس ورژن کو چلانے کے لیے آپ کے میزبان کو پی ایچ پی ورژن {{phpversion}} یا اس سے جدید تر استعمال کرنے کی ضرورت ہے۔","help-url-text":"اس کو حل کرنے میں میری مدد کریں۔"},"vi-VN":{"language":"Vietnamese VN","header":"Xin lỗi, phiên bản PHP của bạn không được hỗ trợ.","text1":"Máy chủ của bạn cần sử dụng PHP phiên bản {{phpversion}} hoặc mới hơn để chạy phiên bản Joomla này.","help-url-text":"Giúp tôi giải quyết vấn đề này"},"zh-CN":{"language":"简体中文","header":"抱歉，PHP 版本不支持。","text1":"需要 PHP {{phpversion}} 或更新的版本运行","help-url-text":"帮助我解决这个问题"},"zh-TW":{"language":"中文 台灣繁體","header":"對不起，你的PHP版本並不支援","text1":"您的主機需要PHP版本{{phpversion}}或以上來執行這個版本的Joomla","help-url-text":"協助我解決這個問題"}};</script><!-- Sets the content of the translated text, IMPORTANT DO NOT REMOVE -->
</head>
<body>
<div class="container">
	<div class="container-main">
		<div class="alert-main">
			<h1 id="headerText">Sorry, your PHP version is not supported</h1><!-- Sets the page header, IMPORTANT DO NOT REMOVE -->
			<p><span id="descText1">Your host needs to use PHP version {{phpversion}} or newer to run this version of Joomla!</span></p><!-- Sets the page description, IMPORTANT DO NOT REMOVE -->
			<p><a id="linkHelp" class="link-help" target="_blank" rel="noreferrer noopener" href="https://docs.joomla.org/Special:MyLanguage/J4.x:Unsupported_PHP_Version">Help me resolve this</a></p><!-- Sets the help page link, IMPORTANT DO NOT REMOVE -->
			<svg version="1.1" xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" viewBox="0 0 74.8 74.8" enable-background="new 0 0 74.8 74.8" xml:space="preserve">
        <g id="brandmark">
          <path id="j-green" fill="#fff" d="M13.5,37.7L12,36.3c-4.5-4.5-5.8-10.8-4.2-16.5c-4.5-1-7.8-5-7.8-9.8c0-5.5,4.5-10,10-10 c5,0,9.1,3.6,9.9,8.4c5.4-1.3,11.3,0.2,15.5,4.4l0.6,0.6l-7.4,7.4l-0.6-0.6c-2.4-2.4-6.3-2.4-8.7,0c-2.4,2.4-2.4,6.3,0,8.7l1.4,1.4 l7.4,7.4l7.8,7.8l-7.4,7.4l-7.8-7.8L13.5,37.7L13.5,37.7z"/>
			<path id="j-orange" fill="#fff" d="M21.8,29.5l7.8-7.8l7.4-7.4l1.4-1.4C42.9,8.4,49.2,7,54.8,8.6C55.5,3.8,59.7,0,64.8,0 c5.5,0,10,4.5,10,10c0,5.1-3.8,9.3-8.7,9.9c1.6,5.6,0.2,11.9-4.2,16.3l-0.6,0.6l-7.4-7.4l0.6-0.6c2.4-2.4,2.4-6.3,0-8.7 c-2.4-2.4-6.3-2.4-8.7,0l-1.4,1.4L37,29l-7.8,7.8L21.8,29.5L21.8,29.5z"/>
			<path id="j-red" fill="#fff" d="M55,66.8c-5.7,1.7-12.1,0.4-16.6-4.1l-0.6-0.6l7.4-7.4l0.6,0.6c2.4,2.4,6.3,2.4,8.7,0 c2.4-2.4,2.4-6.3,0-8.7L53,45.1l-7.4-7.4l-7.8-7.8l7.4-7.4l7.8,7.8l7.4,7.4l1.5,1.5c4.2,4.2,5.7,10.2,4.4,15.7 c4.9,0.7,8.6,4.9,8.6,9.9c0,5.5-4.5,10-10,10C60,74.8,56,71.3,55,66.8L55,66.8z"/>
			<path id="j-blue" fill="#fff" d="M52.2,46l-7.8,7.8L37,61.2l-1.4,1.4c-4.3,4.3-10.3,5.7-15.7,4.4c-1,4.5-5,7.8-9.8,7.8 c-5.5,0-10-4.5-10-10C0,60,3.3,56.1,7.7,55C6.3,49.5,7.8,43.5,12,39.2l0.6-0.6L20,46l-0.6,0.6c-2.4,2.4-2.4,6.3,0,8.7 c2.4,2.4,6.3,2.4,8.7,0l1.4-1.4l7.4-7.4l7.8-7.8L52.2,46L52.2,46z"/>
        </g>
      </svg>
		</div>
		<div class="footer">
			<select id="translatedLanguagesSelect"></select>
			<ul class="links">
				<li><a href="https://www.joomla.org/">Joomla</a></li> -
				<li><a href="https://docs.joomla.org/">Help</a></li>
			</ul>
		</div>
	</div>
</div>
<script>var errorLocale=window.errorLocale||null;(function(e,t){"use strict";if(t){var l=e.getElementById("headerText"),i=e.getElementById("descText1"),g=e.getElementById("linkHelp");Object.keys(t).forEach(function(n){var s=e.getElementById("translatedLanguagesSelect"),u=e.createElement("option");u.text=t[n].language,u.value=n,n==="en-GB"&&u.setAttribute("selected","selected"),e.getElementById("translatedLanguagesSelect").addEventListener("change",function(f){var a=f.target.value;if(a){l&&i&&g&&(l.innerHTML=t[a].header,i.innerHTML=t[a].text1,g.innerText=t[a]["help-url-text"]);var r=e.documentElement;r&&r.setAttribute("lang",a)}}),s.appendChild(u)}),Object.keys(t).forEach(function(n){if(navigator.language===n){e.querySelector('#translatedLanguagesSelect option[value="en-GB"]').removeAttribute("selected"),e.querySelector('#translatedLanguagesSelect option[value="'+n+'"]').setAttribute("selected","selected"),l&&i&&g&&(l.innerHTML=t[n].header,i.innerHTML=t[n].text1,g.innerText=t[n]["help-url-text"]);var s=e.documentElement;s&&s.setAttribute("lang",n)}})}})(document,errorLocale);
</script><!-- Sets the script, IMPORTANT DO NOT REMOVE -->
</body>
</html>
