@charset "UTF-8";
.js-stools {
  flex-wrap: wrap;
  display: flex;
}

.js-stools-container-bar, .js-stools-container-filters {
  margin-bottom: 10px;
}

.js-stools-container-bar {
  margin-inline-start: auto;
}

.js-stools-container-bar .btn-toolbar {
  justify-content: flex-end;
}

.js-stools-container-bar .btn-toolbar > * {
  margin: 5px 0;
}

.js-stools-container-bar .btn-toolbar > * + * {
  margin-inline-start: 8px;
}

.js-stools-container-bar .btn-toolbar .js-stools-btn-clear {
  background-color: var(--template-bg-dark);
}

.js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled], .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled] {
  background: var(--btn-disabled-bg);
  border: var(--js-stools-btn-clear-disabled-border);
  opacity: .5;
}

.js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:hover, .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:active, .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:focus, .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:hover, .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:active, .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:focus {
  cursor: not-allowed;
}

.js-stools-container-bar .ordering-select {
  display: flex;
}

.js-stools-container-filters {
  width: 100%;
  display: none;
}

.js-stools-container-filters-visible {
  grid-gap: 8px;
  background-color: var(--body-bg);
  grid-template-columns: repeat(auto-fill, minmax(220px, 1fr));
  padding: 10px;
  display: grid;
}

.js-stools-field-list + .js-stools-field-list {
  margin-inline-start: 8px;
}

.js-stools-container-selector {
  margin: 5px 0;
  margin-inline-end: 8px;
}

[data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled], [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled] {
  background: var(--body-bg);
  border: var(--btn-secondary-border);
  opacity: .5;
}

[data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:hover, [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:active, [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear[disabled]:focus, [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:hover, [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:active, [data-bs-theme="dark"] .js-stools-container-bar .btn-toolbar .js-stools-btn-clear.dropdown-toggle[disabled]:focus {
  cursor: not-allowed;
}
