@charset "UTF-8";
body {
  color: #22262a;
  background: #fff;
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Helvetica Neue, Arial, Noto Sans, sans-serif, Apple Color Emoji, Segoe UI Emoji, Segoe UI Symbol, Noto Color Emoji;
  font-size: 1rem;
  line-height: 1.3em;
}

h1, h2, h3, h4 {
  font-weight: bold;
}

h1 {
  font-size: 1.857rem;
}

h2 {
  font-size: 1.571rem;
}

h3 {
  font-size: 1.286rem;
}

a:link, a:visited {
  color: #1b57b1;
  font-weight: normal;
  text-decoration: none;
}

a:hover {
  color: #00c;
  font-weight: normal;
  text-decoration: underline;
}

div.caption {
  padding: 0 10px;
}

div.caption img {
  border: 1px solid #ccc;
}

div.caption p {
  color: #666;
  text-align: center;
  font-size: .9em;
}

hr#system-readmore {
  color: red;
  border: 1px dashed red;
}

hr.system-pagebreak {
  color: gray;
  border: 1px dashed gray;
}

span[lang] {
  border: 1px dashed #bbb;
  padding: 2px;
}

span[lang]:after {
  color: red;
  vertical-align: super;
  content: attr(lang);
  font-size: smaller;
}
