<?php return array(
    'root' => array(
        'name' => 'joomla/joomla-cms',
        'pretty_version' => 'dev-5.4-dev',
        'version' => 'dev-5.4-dev',
        'reference' => '97e8dd3854f1d511da7c2eb195894748b1a575a8',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'algo26-matthias/idna-convert' => array(
            'pretty_version' => 'v3.2.1',
            'version' => '3.2.1.0',
            'reference' => '82fe7b60e1a1620c72e68803cc5dcff0b5f59bdc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../algo26-matthias/idna-convert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'brick/math' => array(
            'pretty_version' => '0.12.3',
            'version' => '0.12.3.0',
            'reference' => '866551da34e9a618e64a819ee1e01c20d8a588ba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'composer/ca-bundle' => array(
            'pretty_version' => '1.5.8',
            'version' => '1.5.8.0',
            'reference' => '719026bb30813accb68271fee7e39552a58e9f65',
            'type' => 'library',
            'install_path' => __DIR__ . '/./ca-bundle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'defuse/php-encryption' => array(
            'pretty_version' => 'v2.4.0',
            'version' => '2.4.0.0',
            'reference' => 'f53396c2d34225064647a05ca76c1da9d99e5828',
            'type' => 'library',
            'install_path' => __DIR__ . '/../defuse/php-encryption',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/inflector' => array(
            'pretty_version' => '1.4.4',
            'version' => '1.4.4.0',
            'reference' => '4bd5c1cdfcd00e9e2d8c484f79150f67e5d355d9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/inflector',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dragonmantank/cron-expression' => array(
            'pretty_version' => 'v3.4.0',
            'version' => '3.4.0.0',
            'reference' => '8c784d071debd117328803d86b2097615b457500',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dragonmantank/cron-expression',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'enshrined/svg-sanitize' => array(
            'pretty_version' => '0.22.0',
            'version' => '0.22.0.0',
            'reference' => '0afa95ea74be155a7bcd6c6fb60c276c39984500',
            'type' => 'library',
            'install_path' => __DIR__ . '/../enshrined/svg-sanitize',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fig/link-util' => array(
            'pretty_version' => '1.2.0',
            'version' => '1.2.0.0',
            'reference' => '10e52348a2e9ad4581f2bf3e16458f0861a88c6a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fig/link-util',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'google/recaptcha' => array(
            'pretty_version' => '1.3.1',
            'version' => '1.3.1.0',
            'reference' => '56522c261d2e8c58ba416c90f81a4cd9f2ed89b9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../google/recaptcha',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '2.3.0',
            'version' => '2.3.0.0',
            'reference' => '481557b130ef3790cf82b713667b43030dc9c957',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.12.3',
            'version' => '2.12.3.0',
            'reference' => '7ec62dc3f44aa218487dbed81a9bf9bc647be55d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jakeasmith/http_build_url' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '93c273e77cb1edead0cf8bcf8cd2003428e74e37',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jakeasmith/http_build_url',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-color-output' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => '6c7bf16686cc6a291647fcb87491640a2d5edd20',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-color-output',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-diff' => array(
            'pretty_version' => '6.16.2',
            'version' => '6.16.2.0',
            'reference' => '7f46bcfc582e81769237d0b3f6b8a548efe8799d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-diff',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-mb-string' => array(
            'pretty_version' => '2.0.1',
            'version' => '2.0.1.0',
            'reference' => '8407bfefde47849c9e7c9594e6de2ac85a0f845d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-mb-string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-sequence-matcher' => array(
            'pretty_version' => '4.0.3',
            'version' => '4.0.3.0',
            'reference' => 'd2038ac29627340a7458609072a8ba355e80ec5b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-sequence-matcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/application' => array(
            'pretty_version' => '3.0.4',
            'version' => '3.0.4.0',
            'reference' => '3ab2450f712136f1893337a38bb4c4f951cc853b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../joomla/application',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/archive' => array(
            'pretty_version' => '3.0.4',
            'version' => '3.0.4.0',
            'reference' => '1d506857685f3537c1193f4e77b12f24cc9afe49',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/archive',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/authentication' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => 'ea3a263985a40c231a42d050ccd3f822b7b5e778',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/authentication',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/console' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '839ac3e759a9c8bf962175906480923a7b680c36',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/console',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/crypt' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '0cfd0a3ab2fc64015030319348a16f4fa6e0355a',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/crypt',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/data' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '8eaa08b07b84dce217b1735e5c51d66285ff93ae',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/data',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/database' => array(
            'pretty_version' => '3.4.3',
            'version' => '3.4.3.0',
            'reference' => '6a8aedaea444cd78912153c4d8361eb06ba2a9cd',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/database',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/di' => array(
            'pretty_version' => '3.1.1',
            'version' => '3.1.1.0',
            'reference' => 'fd98d11dddf65adf9c6eff68a4c80f9f2f34b2b9',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/di',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/event' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'd1a5dd4c5c8aa1d1c7a7101da93e677ec5d3ef3f',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/event',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/filesystem' => array(
            'pretty_version' => '3.2.0',
            'version' => '3.2.0.0',
            'reference' => 'be6917c396300c8316ef13ef3e2096732af847fd',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/filesystem',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/filter' => array(
            'pretty_version' => '3.0.6',
            'version' => '3.0.6.0',
            'reference' => 'dde731979af1a34a0b5610281c227274628c1b6f',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/filter',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/http' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => '493dc254d9fa9277c7ab3488573ddb4f3bbfa9bc',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/http',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/input' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'b4ef792308e7adb42fb71a8118025ced899e4386',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/input',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/joomla-cms' => array(
            'pretty_version' => 'dev-5.4-dev',
            'version' => 'dev-5.4-dev',
            'reference' => '97e8dd3854f1d511da7c2eb195894748b1a575a8',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/language' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => '7bd3d468ffad1e9639468a17751ec4324fc57d09',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/language',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/oauth1' => array(
            'pretty_version' => '3.0.1',
            'version' => '3.0.1.0',
            'reference' => '4c1407cc98e0689ff36fa566906673829a8627ec',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/oauth1',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/oauth2' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '318bae49ddfe67f83626bc05e0d0c82d20c365d0',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/oauth2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/registry' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => '4452de9c66abfec59b535e27c62af3528a539311',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/registry',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/router' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => '8aa0013a6ba59ef250496b47855498e9a83156d1',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/router',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/session' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '6bebada5dccf70be0d18db673aa3a68af19b3972',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/session',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/string' => array(
            'pretty_version' => '3.0.4',
            'version' => '3.0.4.0',
            'reference' => '0b3d33564db389e27346f7e275c694897c939434',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/uri' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'ac18b410b58c5d2cb652eac7c13acb4971cb5e0d',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/uri',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'joomla/utilities' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'bec074fa469cfdb30fae168899cf65159bf7890d',
            'type' => 'joomla-package',
            'install_path' => __DIR__ . '/../joomla/utilities',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laminas/laminas-diactoros' => array(
            'pretty_version' => '2.26.0',
            'version' => '2.26.0.0',
            'reference' => '6584d44eb8e477e89d453313b858daac6183cddc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laminas/laminas-diactoros',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'lcobucci/clock' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => '039ef98c6b57b101d10bd11d8fdfda12cbd996dc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../lcobucci/clock',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'lcobucci/jwt' => array(
            'pretty_version' => '4.3.0',
            'version' => '4.3.0.0',
            'reference' => '4d7de2fe0d51a96418c0d04004986e410e87f6b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../lcobucci/jwt',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'maximebf/debugbar' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.2.4',
            ),
        ),
        'mtdowling/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '^1.0',
            ),
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v2.8.2',
            'version' => '2.8.2.0',
            'reference' => 'e30811f7bc69e4b5b6d5783e712c06c8eabf0226',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '9.99.99',
            ),
        ),
        'paragonie/sodium_compat' => array(
            'pretty_version' => 'v1.24.0',
            'version' => '1.24.0.0',
            'reference' => '2cb48f26130919f92f30650bdcc30e6f4ebe45ac',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/sodium_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-debugbar/php-debugbar' => array(
            'pretty_version' => 'v2.2.4',
            'version' => '2.2.4.0',
            'reference' => '3146d04671f51f69ffec2a4207ac3bdcf13a9f35',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-debugbar/php-debugbar',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/async-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-http/client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-tuf/php-tuf' => array(
            'pretty_version' => '1.0.3',
            'version' => '1.0.3.0',
            'reference' => '1fc585dc5a090a4af94823e21470779f31030733',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-tuf/php-tuf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpmailer/phpmailer' => array(
            'pretty_version' => 'v6.10.0',
            'version' => '6.10.0.0',
            'reference' => 'bf74d75a1fde6beaa34a0ddae2ec5fce0f72a144',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpmailer/phpmailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/bcmath_compat' => array(
            'pretty_version' => '2.0.3',
            'version' => '2.0.3.0',
            'reference' => 'ae8f87ea0c96b2ef08ecf0d291d45372d0f7bc5a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/bcmath_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '3.0.55',
            'version' => '3.0.55.0',
            'reference' => 'db9744e6d47e742b1f974e965ad49bdd041105af',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/cache' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => 'aa5030cfa5405eccfdcb1083ce040c2cb8d253bf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/clock' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'e41a24703d4560fd0acb709162f73b8adfc3aa0d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/clock',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/clock-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/container' => array(
            'pretty_version' => '1.1.2',
            'version' => '1.1.2.0',
            'reference' => '513e0666f7216c7459170d56df27dfcefe1689ea',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '~1.0',
            ),
        ),
        'psr/event-dispatcher' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'dbefd12671e8a14ec7f180cab83036ed26714bb0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client' => array(
            'pretty_version' => '1.0.3',
            'version' => '1.0.3.0',
            'reference' => 'bb5906edc1c324c9a05aa0873d40117941e5fa90',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.1',
            'version' => '1.1.0.0',
            'reference' => 'cb6ce4845ce34a8ad9e68117c10ee90a29919eba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/link' => array(
            'pretty_version' => '1.1.1',
            'version' => '1.1.1.0',
            'reference' => '846c25f58a1f02b93a00f2404e3626b6bf9b7807',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/link',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/link-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
                1 => '1.0|2.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0|3.0',
            ),
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spomky-labs/cbor-php' => array(
            'pretty_version' => '3.1.1',
            'version' => '3.1.1.0',
            'reference' => '5404f3e21cbe72f5cf612aa23db2b922fd2f43bf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/cbor-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spomky-labs/pki-framework' => array(
            'pretty_version' => '1.3.0',
            'version' => '1.3.0.0',
            'reference' => 'eced5b5ce70518b983ff2be486e902bbd15135ae',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/pki-framework',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/console' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => '273fd29ff30ba0a88ca5fb83f7cf1ab69306adae',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/console',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => '63afe740e99a13ba87ec199bb07bbdee937a5b62',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/error-handler' => array(
            'pretty_version' => 'v6.4.24',
            'version' => '6.4.24.0',
            'reference' => '30fd0b3cf0e972e82636038ce4db0e4fe777112c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/error-handler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => 'b8e9dce2d8acba3c32af467bb58e0c3656886181',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => '75d7043853a42837e68111812f4d964b01e5101c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-client-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '3.0',
            ),
        ),
        'symfony/ldap' => array(
            'pretty_version' => 'v6.4.24',
            'version' => '6.4.24.0',
            'reference' => '2eda8a14000f804d55adfa189f148f6b1cb020e1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/ldap',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/options-resolver' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => 'd28e7e2db8a73e9511df892d36445f61314bbebe',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/options-resolver',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => 'a3cc8b044a6ea513310cbd48ef7333b384945638',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-iconv' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '5f3b930437ae03ae5dff61269024d8ea1b3774aa',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-iconv',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-grapheme' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '380872130d3a5dd3ace2f4010d95125fde5d5c70',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-grapheme',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '3833d7255cc303546435cb650316bff708a1c75c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '6d857f4d76bd4b343eac26d6b539585d2bc56493',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php70' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php71' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php72' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php73' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php74' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php80' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php81' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'symfony/polyfill-php83' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '17f6f9a6b1735c0f163024d959f700cfbc5155e5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php83',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-uuid' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '21533be36c24be3f4b1669c4725c7d1d2bab4ae2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/service-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => 'f021b05a130d35510bd6b25fe9053c2a8a15d5d4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/service-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/string' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => '7cdec7edfaf2cdd9c18901e35bcf9653d6209ff1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => 'df210c7a2573f1913b2d17cc95f90f53a73d8f7d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/uid' => array(
            'pretty_version' => 'v6.4.24',
            'version' => '6.4.24.0',
            'reference' => '17da16a750541a42cf2183935e0f6008316c23f7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/uid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/validator' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => '9352177c0e937793423053846f80bee805552324',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/validator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/var-dumper' => array(
            'pretty_version' => 'v6.4.25',
            'version' => '6.4.25.0',
            'reference' => 'c6cd92486e9fc32506370822c57bc02353a5a92c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/var-dumper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/web-link' => array(
            'pretty_version' => 'v6.4.24',
            'version' => '6.4.24.0',
            'reference' => '75ffbb304f26a716969863328c8c6a11eadcfa5a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/web-link',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v6.4.41',
            'version' => '6.4.41.0',
            'reference' => 'e8fdf3408c85806198d5826e604ffc6830d33152',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tobscure/json-api' => array(
            'pretty_version' => 'dev-joomla-backports',
            'version' => 'dev-joomla-backports',
            'reference' => '6ce73102280c9c70b41e3407d2504279fa07d158',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tobscure/json-api',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'typo3/phar-stream-wrapper' => array(
            'pretty_version' => 'v3.1.8',
            'version' => '3.1.8.0',
            'reference' => 'a931b28f422a60052db85c0a84a05a366453b2c0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../typo3/phar-stream-wrapper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'voku/portable-ascii' => array(
            'pretty_version' => '2.0.3',
            'version' => '2.0.3.0',
            'reference' => 'b1d923f88091c6bf09699efcd7c8a1b1bfd7351d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../voku/portable-ascii',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'voku/portable-utf8' => array(
            'pretty_version' => 'dev-joomla-5.3',
            'version' => 'dev-joomla-5.3',
            'reference' => 'eeb3d9e390411cd31af808caa7f7de337ea3a24c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../voku/portable-utf8',
            'aliases' => array(
                0 => '6.0.13',
            ),
            'dev_requirement' => false,
        ),
        'wamania/php-stemmer' => array(
            'pretty_version' => 'v4.0.0',
            'version' => '4.0.0.0',
            'reference' => 'd96509294ea843b4b86e4900df27424a6ea0ace8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../wamania/php-stemmer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-auth/cose-lib' => array(
            'pretty_version' => '4.4.2',
            'version' => '4.4.2.0',
            'reference' => 'a93b61c48fb587855f64a9ec11ad7b60e867cb15',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-auth/cose-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-auth/metadata-service' => array(
            'pretty_version' => '4.5.2',
            'version' => '4.5.2.0',
            'reference' => '82b3a517894987db1f8959a9320b9891515af9e7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-auth/metadata-service',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-auth/webauthn-lib' => array(
            'pretty_version' => '4.5.2',
            'version' => '4.5.2.0',
            'reference' => '30c5e891c6418abef6d87a4351132b01b1828a98',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-auth/webauthn-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-library' => array(
            'pretty_version' => '3.4.10',
            'version' => '3.4.10.0',
            'reference' => '8c38010d971c2313406fa1376d763cee745a833b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-library',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => '11cb2199493b2f8a3b53e7f19068fc6aac760991',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'willdurand/negotiation' => array(
            'pretty_version' => '3.1.0',
            'version' => '3.1.0.0',
            'reference' => '68e9ea0553ef6e2ee8db5c1d98829f111e623ec2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../willdurand/negotiation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
