@charset "UTF-8";
.match-custom .control-group .control-label {
  width: auto;
  padding: 0;
}

.match-custom .control-group .controls {
  min-width: 7rem;
  display: inline-block;
}

.match-custom .control-group {
  margin-right: 1.5rem;
  display: inline-block;
}

.match-custom label {
  font-size: small;
}

.match-custom select[multiple] {
  height: 15rem;
}
