tinymce.addI18n('sw',{
"Cut": "Kata",
"Heading 5": "Kichwa 5",
"Header 2": "Kijajuu 2",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "Kisakuzi chako hakiauniwi kufika kwa clipboard moja kwa moja. Tafadhali tumia njia fupi ya kinanda taipu, kama Ctrl+X\/C\/V.",
"Heading 4": "Kichwa 4",
"Div": "Div",
"Heading 2": "Kichwa 2",
"Paste": "Bandika",
"Close": "Funga",
"Font Family": "Jamii ya fonti",
"Pre": "Iliofomatiwa",
"Align right": "Panga kulia",
"New document": "Waraka mpya",
"Blockquote": "Nukuu",
"Numbered list": "Orodha ya nambari",
"Heading 1": "Kichwa 1",
"Headings": "Vichwa",
"Increase indent": "Ongeza jongezo",
"Formats": "Fomati",
"Headers": "Vijajuu",
"Select all": "Chagua zote",
"Header 3": "Kijajuu 3",
"Blocks": "Matofali",
"Undo": "Tengua",
"Strikethrough": "Mkato ulalo",
"Bullet list": "Orodha ya tobo",
"Header 1": "Kijajuu 1",
"Superscript": "Hati juu",
"Clear formatting": "Safisha fomati",
"Font Sizes": "Ukubwa wa fonti",
"Subscript": "Hati chini",
"Header 6": "Kijajuu 6",
"Redo": "Rudia",
"Paragraph": "Aya",
"Ok": "Ok",
"Bold": "Nene",
"Code": "Kodi",
"Italic": "Italiki",
"Align center": "Panga katikati",
"Header 5": "Kijajuu 5",
"Heading 6": "Kichwa 6",
"Heading 3": "Kichwa 3",
"Decrease indent": "Punguza jongezo",
"Header 4": "Kijajuu 4",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Bandika sasa iko katika modi ya matini ghafi. Yaliyomo yatabandikwa kama matini ghafi mpaka utakapo zima chaguo hili.",
"Underline": "Pigia mstari",
"Cancel": "Ghairi",
"Justify": "Sawazisha",
"Inline": "Inline",
"Copy": "Nakili",
"Align left": "Panga kushoto",
"Visual aids": "Misaada ya kuona",
"Lower Greek": "Herufi ndogo za kigiriki",
"Square": "Mraba",
"Default": "Difoti",
"Lower Alpha": "Herufi ndogo",
"Circle": "Duara",
"Disc": "Nukta",
"Upper Alpha": "Herufi kubwa",
"Upper Roman": "Herufi kubwa za kirumi",
"Lower Roman": "Herufi ndogo za kirumi",
"Id should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "Kitambulisho kinapaswa kuanza na herufi kufwatiwa na maherufi, manambari, madeshi, vitone, minukta pacha au vistari chini pekee.",
"Name": "Jina",
"Anchor": "Nanga",
"Id": "Kitambulisho",
"You have unsaved changes are you sure you want to navigate away?": "Uko na mabadilisho ambayo hujayahifadhi. Una hakika unataka kuwacha?",
"Restore last draft": "Rejesha mswadajaribio wa mwisho",
"Special character": "Kikambo maalum",
"Source code": "Chanzo",
"Language": "Lugha",
"Insert\/Edit code sample": "Ingiza\/hariri mfano wa kodi",
"B": "B",
"R": "R",
"G": "G",
"Color": "Rangi",
"Right to left": "Kulia-kushoto",
"Left to right": "Kushoto-kulia",
"Emoticons": "Emoticons",
"Robots": "Roboti",
"Document properties": "Sifa za waraka",
"Title": "Kichwa",
"Keywords": "Maneno misingi",
"Encoding": "Usimbaji",
"Description": "Fafanuo",
"Author": "Mwandishi",
"Fullscreen": "Skrini kamili",
"Horizontal line": "Mstari mlalo",
"Horizontal space": "Nafasi mlalo",
"Insert\/edit image": "Ingiza\/hariri picha",
"General": "Ujumla",
"Advanced": "Hali ya juu",
"Source": "Kodi ya chanzo",
"Border": "Mkingo",
"Constrain proportions": "Lazimisha uwiano",
"Vertical space": "Nafasi wima",
"Image description": "Fafanuo la picha",
"Style": "Mtindo",
"Dimensions": "Kipimo",
"Insert image": "Ingiza picha",
"Image": "Picha",
"Zoom in": "Kuza zaidi",
"Contrast": "Ulinganuzi",
"Back": "Nyuma",
"Gamma": "Gamma",
"Flip horizontally": "Geuza mlalo",
"Resize": "Badilisha ukubwa",
"Sharpen": "Chonga",
"Zoom out": "Fifiza",
"Image options": "Machaguo ya picha",
"Apply": "Omba",
"Brightness": "Mwangaza",
"Rotate clockwise": "Zungusha mwelekeo saa",
"Rotate counterclockwise": "Zungusha mwelekeo kinyume cha saa",
"Edit image": "Hariri",
"Color levels": "Viwango vya rangi",
"Crop": "Puna",
"Orientation": "Uelekeo",
"Flip vertically": "Geuza wima",
"Invert": "Pindua",
"Date\/time": "Tarehe\/saa",
"Insert date\/time": "Ingiza tarehe\/saa",
"Remove link": "Ondoa kiungo",
"Url": "URL",
"Text to display": "Matini ya kuonyesha",
"Anchors": "Mananga",
"Insert link": "Ingiza kiungo",
"Link": "Kiungo",
"New window": "Dirisha jipya",
"None": "Hakuna",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "URL ulioingiza unaonekana ni kama kiungo cha nje. Unataka kuongeza prefix inayohitajika, ambayo ni http:\/\/ ?",
"Paste or type a link": "Bandika au kuchapa kiungo",
"Target": "Lengo",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "URL ulioingiza unaonekana ni kama anwani ya barua pepe. Unataka kuongeza prefix inayohitajika, ambayo ni mailto: ?",
"Insert\/edit link": "Ingiza\/hariri kiungo",
"Insert\/edit video": "Ingiza\/hariri video",
"Media": "Midia",
"Alternative source": "Chanzo cha mbadala",
"Paste your embed code below:": "Bandika kodi yako ya kupachika hapo chini:",
"Insert video": "Ingiza video",
"Poster": "Bando",
"Insert\/edit media": "Ingiza\/hariri midia",
"Embed": "Pachika",
"Nonbreaking space": "Nafasi fungamanishi",
"Page break": "Kigawa kurasa",
"Paste as text": "Bandika kama matini",
"Preview": "Hakikisho",
"Print": "Chapisha",
"Save": "Hifadhi",
"Could not find the specified string.": "Haikuweza kupata tungo lililoelezwa.",
"Replace": "Badilisha",
"Next": "Ifuatayo",
"Whole words": "Maneno kamili",
"Find and replace": "Tafuta na badilisha",
"Replace with": "Badilisha na",
"Find": "Tafuta",
"Replace all": "Badilisha zote",
"Match case": "Tahadhari herufi ndogo\/kubwa",
"Prev": "Iliopita",
"Spellcheck": "Ukaguzi tahajia",
"Finish": "Maliza",
"Ignore all": "Puuza zote",
"Ignore": "Puuza",
"Add to Dictionary": "Ongeza kwa kamusi",
"Insert row before": "Ingiza safumlalo kabla",
"Rows": "Masafumlalo",
"Height": "Urefu",
"Paste row after": "Bandika safumlalo baada",
"Alignment": "Mpangilio",
"Border color": "Rangi ya mkingo",
"Column group": "Kikundi cha safuwima",
"Row": "Safumlalo",
"Insert column before": "Ingiza safuwima kabla",
"Split cell": "Bainisha seli",
"Cell padding": "Pedi ya seli",
"Cell spacing": "Nafasi ya seli",
"Row type": "Aina ya safumlalo",
"Insert table": "Ingiza jedwali",
"Body": "Mwili",
"Caption": "Manukuu",
"Footer": "Kijachini",
"Delete row": "Futa safumlalo",
"Paste row before": "Bandika safumlalo kabla",
"Scope": "Wigo",
"Delete table": "Futa jedwali",
"H Align": "Pangilia mlalo",
"Top": "Juu",
"Header cell": "Seli ya kijajuu",
"Column": "Safuwima",
"Row group": "Kikundi cha safumlalo",
"Cell": "Seli",
"Middle": "Katikati",
"Cell type": "Aina ya seli",
"Copy row": "Nakili safumlalo",
"Row properties": "Mali ya safumlalo",
"Table properties": "Masifa ya jedwali",
"Bottom": "Chini",
"V Align": "Pangilia wima",
"Header": "Kijajuu",
"Right": "Kulia",
"Insert column after": "Ingiza safuwima baada",
"Cols": "Masafuwima",
"Insert row after": "Ingiza safumlalo baada",
"Width": "Upana",
"Cell properties": "Mali ya seli",
"Left": "Kushoto",
"Cut row": "Kata safumlalo",
"Delete column": "Futa safuwima",
"Center": "Katikati",
"Merge cells": "Unganisha maseli",
"Insert template": "Ingiza templeti",
"Templates": "Templeti",
"Background color": "Rangi ya usuli",
"Custom...": "Desturi...",
"Custom color": "Rangi ya desturi",
"No color": "Hakuna rangi",
"Text color": "Rangi ya matini",
"Table of Contents": "Jedwali la yaliyomo",
"Show blocks": "Onyesha matofali",
"Show invisible characters": "Onyesha vibambo visivyoonekana",
"Words: {0}": "Maneno: {0}",
"Insert": "Ingiza",
"File": "Faili",
"Edit": "Hariri picha",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Eneo la matini tondoti. Bofya ALT-F9 kwa menyu. Bofya ALT-F10 kwa upauzana. Bofya ALT-0 kwa usaidizi",
"Tools": "Zana",
"View": "Mtazamo",
"Table": "Jedwali",
"Format": "Fomati",
});