/**
 * @copyright	(C) 2011 Open Source Matters, Inc. <https://www.joomla.org>
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

/* Start Common Styles */

body {
  text-align: right;
}

.TD {
  text-align: left;
}

#errorboxbody {
  text-align: right;
}
#techinfo {
  text-align: right;
}
