@charset "UTF-8";
@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Regular.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Regular.woff?v=178429") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Regular;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Regular.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Regular.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-RegularItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-RegularItalic.woff?v=178429") format("woff");
  font-weight: 400;
  font-style: italic;
}

@font-face {
  font-family: Roboto-RegularItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-RegularItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-RegularItalic.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Light.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Light.woff?v=178429") format("woff");
  font-weight: 300;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Light;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Light.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Light.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-LightItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-LightItalic.woff?v=178429") format("woff");
  font-weight: 300;
  font-style: italic;
}

@font-face {
  font-family: Roboto-LightItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-LightItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-LightItalic.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Thin.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Thin.woff?v=178429") format("woff");
  font-weight: 100;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Thin;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Thin.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Thin.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-ThinItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-ThinItalic.woff?v=178429") format("woff");
  font-weight: 100;
  font-style: italic;
}

@font-face {
  font-family: Roboto-ThinItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-ThinItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-ThinItalic.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Medium.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Medium.woff?v=178429") format("woff");
  font-weight: 500;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Medium;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Medium.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Medium.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-MediumItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-MediumItalic.woff?v=178429") format("woff");
  font-weight: 500;
  font-style: italic;
}

@font-face {
  font-family: Roboto-MediumItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-MediumItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-MediumItalic.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Bold.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Bold.woff?v=178429") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Bold;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Bold.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Bold.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BoldItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BoldItalic.woff?v=178429") format("woff");
  font-weight: 700;
  font-style: italic;
}

@font-face {
  font-family: Roboto-BoldItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BoldItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BoldItalic.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Black.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Black.woff?v=178429") format("woff");
  font-weight: 900;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Black;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Black.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-Black.woff?v=178429") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BlackItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BlackItalic.woff?v=178429") format("woff");
  font-weight: 900;
  font-style: italic;
}

@font-face {
  font-family: Roboto-BlackItalic;
  src: url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BlackItalic.woff2?v=178429") format("woff2"), url("../../../../../vendor/roboto-fontface/fonts/roboto/Roboto-BlackItalic.woff?v=178429") format("woff");
}

:root {
  --cassiopeia-font-family-body: "Roboto", sans-serif;
  --cassiopeia-font-family-headings: "Roboto", sans-serif;
  --cassiopeia-font-weight-headings: 700;
  --cassiopeia-font-weight-normal: 400;
}
