@charset "UTF-8";
div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status {
  font-family: var(--debugbar-font-mono);
  border-bottom: 1px solid var(--debugbar-border);
  color: var(--debugbar-text);
  background-color: var(--debugbar-background-alt);
  padding: 6px;
  font-weight: bold;
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-render-time, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-memory, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-param-count, div.phpdebugbar-widgets-templates a.phpdebugbar-widgets-editor-link, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-type {
  float: right;
  color: var(--debugbar-text);
  margin-left: 8px;
}

div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status span.phpdebugbar-widgets-render-time, div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status span.phpdebugbar-widgets-memory, div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status span.phpdebugbar-widgets-param-count, div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status a.phpdebugbar-widgets-editor-link, div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-status span.phpdebugbar-widgets-type {
  color: var(--debugbar-text);
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-render-time:before, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-memory:before, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-param-count:before, div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-type:before, div.phpdebugbar-widgets-templates a.phpdebugbar-widgets-editor-link:before, div.phpdebugbar-widgets-templates a.phpdebugbar-widgets-editor-link:before {
  margin-right: 4px;
  font-family: PhpDebugbarFontAwesome;
  font-size: 12px;
}

div.phpdebugbar-widgets-templates a.phpdebugbar-widgets-editor-link:hover {
  color: var(--debugbar-hover);
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-render-time:before {
  content: "";
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-memory:before {
  content: "";
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-param-count:before {
  content: "";
}

div.phpdebugbar-widgets-templates span.phpdebugbar-widgets-type:before {
  content: "";
}

div.phpdebugbar-widgets-templates a.phpdebugbar-widgets-editor-link:before {
  content: "";
  margin-left: 4px;
}

div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params {
  border: 1px solid var(--debugbar-border);
  width: 70%;
  font-family: var(--debugbar-font-mono);
  border-collapse: collapse;
  margin: 10px;
  display: none;
}

div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params td {
  border: 1px solid var(--debugbar-border);
  padding: 0 5px;
}

div.phpdebugbar-widgets-templates table.phpdebugbar-widgets-params .phpdebugbar-widgets-name {
  vertical-align: top;
  width: 20%;
  font-weight: bold;
}

div.phpdebugbar[data-theme="dark"] div.phpdebugbar-widgets-templates div.phpdebugbar-widgets-callgraph > pre * {
  background-color: inherit !important;
}
