tinymce.addI18n('ga',{
"Cut": "Gearr",
"Heading 5": "Ceannteideal 5",
"Header 2": "Ceannt\u00e1sc 2",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "N\u00ed f\u00e9idir le do bhrabhs\u00e1la\u00ed teacht go d\u00edreach ar an ngearrthaisce. Bain \u00fas\u00e1id as na haicearra\u00ed Ctrl+X\/C\/V. ",
"Heading 4": "Ceannteideal 4",
"Div": "Deighilt",
"Heading 2": "Ceannteideal 2",
"Paste": "Greamaigh",
"Close": "D\u00fan",
"Font Family": "Cl\u00f3fhoireann",
"Pre": "R\u00e9amh",
"Align right": "Ail\u00ednigh ar dheis",
"New document": "C\u00e1ip\u00e9is nua",
"Blockquote": "Athfhriotal",
"Numbered list": "Liosta Uimhrithe",
"Heading 1": "Ceannteideal 1",
"Headings": "Ceannteidil",
"Increase indent": "M\u00e9adaigh eang",
"Formats": "Form\u00e1id\u00ed",
"Headers": "Ceannt\u00e1sca",
"Select all": "Roghnaigh uile",
"Header 3": "Ceannt\u00e1sc 3",
"Blocks": "Blocanna",
"Undo": "Cealaigh",
"Strikethrough": "L\u00edne tr\u00edd",
"Bullet list": "Liosta Urchar",
"Header 1": "Ceannt\u00e1sc 1",
"Superscript": "Forscript",
"Clear formatting": "Glan form\u00e1idi\u00fa",
"Font Sizes": "Cl\u00f3mh\u00e9ideanna",
"Subscript": "Foscript",
"Header 6": "Ceannt\u00e1sc 6",
"Redo": "Athdh\u00e9an",
"Paragraph": "Alt",
"Ok": "OK",
"Bold": "Trom",
"Code": "C\u00f3d",
"Italic": "Iod\u00e1lach",
"Align center": "Ail\u00ednigh sa l\u00e1r",
"Header 5": "Ceannt\u00e1sc 5",
"Heading 6": "Ceannteideal 6",
"Heading 3": "Ceannteideal 3",
"Decrease indent": "Laghdaigh eang",
"Header 4": "Ceannt\u00e1sc 4",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Sa m\u00f3d gn\u00e1th-th\u00e9acs anois. Gream\u00f3far \u00e1bhar mar ghn\u00e1th-th\u00e9acs go dt\u00ed go m\u00fachfaidh t\u00fa an rogha seo.",
"Underline": "Fol\u00edne",
"Cancel": "Cealaigh",
"Justify": "Comhfhadaigh",
"Inline": "Inl\u00edne",
"Copy": "C\u00f3ipe\u00e1il",
"Align left": "Ail\u00ednigh ar chl\u00e9",
"Visual aids": "\u00c1iseanna amhairc",
"Lower Greek": "Litir Bheag Ghr\u00e9agach",
"Square": "Cearn\u00f3g",
"Default": "R\u00e9amhshocr\u00fa",
"Lower Alpha": "Alfa Beag",
"Circle": "Ciorcal",
"Disc": "Diosca",
"Upper Alpha": "Alfa M\u00f3r",
"Upper Roman": "Litir Mh\u00f3r R\u00f3mh\u00e1nach",
"Lower Roman": "Litir Bheag R\u00f3mh\u00e1nach",
"Id should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "N\u00ed m\u00f3r don aitheantas tos\u00fa le litir, agus gan ach litreacha, uimhreacha, daiseanna, poncanna, idirstadanna, n\u00f3 fostr\u00edoca ina dhiaidh sin.",
"Name": "Ainm",
"Anchor": "Ancaire",
"Id": "Aitheantas",
"You have unsaved changes are you sure you want to navigate away?": "T\u00e1 athruithe gan s\u00e1bh\u00e1il ann. An bhfuil t\u00fa cinnte gur mhaith leat imeacht amach as seo?",
"Restore last draft": "Oscail an dr\u00e9acht is d\u00e9ana\u00ed",
"Special character": "Carachtar speisialta",
"Source code": "C\u00f3d foinseach",
"Language": "Teanga",
"Insert\/Edit code sample": "Cuir sampla c\u00f3id isteach\/in eagar",
"B": "G",
"R": "D",
"G": "U",
"Color": "Dath",
"Right to left": "Deas-go-cl\u00e9",
"Left to right": "Cl\u00e9-go-deas",
"Emoticons": "Straoiseoga",
"Robots": "R\u00f3bait",
"Document properties": "Air\u00edonna na C\u00e1ip\u00e9ise",
"Title": "Teideal",
"Keywords": "Lorgfhocail",
"Encoding": "Ionch\u00f3d\u00fa",
"Description": "Cur S\u00edos",
"Author": "\u00dadar",
"Fullscreen": "L\u00e1nsc\u00e1ile\u00e1n",
"Horizontal line": "L\u00edne chothrom\u00e1nach",
"Horizontal space": "Sp\u00e1s cothrom\u00e1nach",
"Insert\/edit image": "Cuir \u00edomh\u00e1 isteach\/in eagar",
"General": "Ginear\u00e1lta",
"Advanced": "Casta",
"Source": "Foinse",
"Border": "Iml\u00edne",
"Constrain proportions": "Comhr\u00e9ir faoi ghlas",
"Vertical space": "Sp\u00e1s ingearach",
"Image description": "Cur s\u00edos ar an \u00edomh\u00e1",
"Style": "St\u00edl",
"Dimensions": "Tois\u00ed",
"Insert image": "Cuir \u00edomh\u00e1 isteach",
"Image": "\u00cdomh\u00e1",
"Zoom in": "Z\u00fam\u00e1il isteach",
"Contrast": "Codarsnacht",
"Back": "Siar",
"Gamma": "G\u00e1ma",
"Flip horizontally": "Cas go cothrom\u00e1nach",
"Resize": "Athraigh m\u00e9id",
"Sharpen": "G\u00e9araigh",
"Zoom out": "Z\u00fam\u00e1il amach",
"Image options": "Roghanna \u00edomh\u00e1",
"Apply": "Cuir i bhfeidhm",
"Brightness": "Gile",
"Rotate clockwise": "Rothlaigh ar deiseal",
"Rotate counterclockwise": "Rothlaigh ar tuathal",
"Edit image": "Cuir an \u00edomh\u00e1 in eagar",
"Color levels": "Leibh\u00e9il datha",
"Crop": "Bear",
"Orientation": "Treoshu\u00edomh",
"Flip vertically": "Cas go hingearach",
"Invert": "Inbh\u00e9artaigh",
"Date\/time": "D\u00e1ta\/am",
"Insert date\/time": "Cuir d\u00e1ta\/am isteach",
"Remove link": "Bain an nasc",
"Url": "URL",
"Text to display": "T\u00e9acs le taispe\u00e1int",
"Anchors": "Ancair\u00ed",
"Insert link": "Cuir nasc isteach",
"Link": "Nasc",
"New window": "Fuinneog nua",
"None": "Dada",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "Is nasc seachtrach \u00e9 an URL a chuir t\u00fa isteach. An bhfuil fonn ort an r\u00e9im\u00edr riachtanach http:\/\/ a chur leis?",
"Paste or type a link": "Greamaigh n\u00f3 cl\u00f3scr\u00edobh nasc",
"Target": "Sprioc",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "Is seoladh r\u00edomhphoist \u00e9 an URL a chuir t\u00fa isteach. An bhfuil fonn ort an r\u00e9im\u00edr riachtanach mailto: a chur leis?",
"Insert\/edit link": "Cuir nasc isteach\/in eagar",
"Insert\/edit video": "Cuir f\u00edse\u00e1n isteach\/in eagar",
"Media": "Me\u00e1in",
"Alternative source": "Foinse mhalartach",
"Paste your embed code below:": "Greamaigh do ch\u00f3d leabaithe th\u00edos:",
"Insert video": "Cuir f\u00edse\u00e1n isteach",
"Poster": "P\u00f3staer",
"Insert\/edit media": "Cuir me\u00e1n isteach\/in eagar",
"Embed": "Leabaigh",
"Nonbreaking space": "Sp\u00e1s neamhbhristeach",
"Page break": "Briseadh leathanaigh",
"Paste as text": "Greamaigh mar th\u00e9acs",
"Preview": "R\u00e9amhamharc",
"Print": "Priont\u00e1il",
"Save": "S\u00e1bh\u00e1il",
"Could not find the specified string.": "N\u00edor aims\u00edodh an teaghr\u00e1n.",
"Replace": "Ionadaigh",
"Next": "Ar aghaidh",
"Whole words": "Focail ioml\u00e1na",
"Find and replace": "Aimsigh agus ionadaigh",
"Replace with": "Ionadaigh le",
"Find": "Aimsigh",
"Replace all": "Ionadaigh uile",
"Match case": "C\u00e1s-\u00edogair",
"Prev": "Siar",
"Spellcheck": "Seice\u00e1il an litri\u00fa",
"Finish": "Cr\u00edochnaigh",
"Ignore all": "D\u00e9an neamhaird orthu go l\u00e9ir",
"Ignore": "D\u00e9an neamhaird air",
"Add to Dictionary": "Cuir leis an bhFocl\u00f3ir \u00e9",
"Insert row before": "Ions\u00e1igh r\u00f3 os a chionn",
"Rows": "R\u00f3nna",
"Height": "Airde",
"Paste row after": "Greamaigh r\u00f3 faoi",
"Alignment": "Ail\u00edni\u00fa",
"Border color": "Dath na himl\u00edne",
"Column group": "Gr\u00fapa col\u00fan",
"Row": "R\u00f3",
"Insert column before": "Ions\u00e1igh col\u00fan ar chl\u00e9",
"Split cell": "Roinn cill",
"Cell padding": "Stu\u00e1il ceall",
"Cell spacing": "Sp\u00e1s\u00e1il ceall",
"Row type": "Cine\u00e1l an r\u00f3",
"Insert table": "Ions\u00e1igh t\u00e1bla",
"Body": "Corp",
"Caption": "Fotheideal",
"Footer": "Bunt\u00e1sc",
"Delete row": "Scrios an r\u00f3",
"Paste row before": "Greamaigh r\u00f3 os a chionn",
"Scope": "Sc\u00f3ip",
"Delete table": "Scrios an t\u00e1bla",
"H Align": "Ail\u00edni\u00fa C.",
"Top": "Barr",
"Header cell": "Cill cheannt\u00e1isc",
"Column": "Col\u00fan",
"Row group": "Gr\u00fapa r\u00f3nna",
"Cell": "Cill",
"Middle": "L\u00e1r",
"Cell type": "Cine\u00e1l na cille",
"Copy row": "C\u00f3ipe\u00e1il an r\u00f3",
"Row properties": "Air\u00edonna an r\u00f3",
"Table properties": "Air\u00edonna an t\u00e1bla",
"Bottom": "Bun",
"V Align": "Ail\u00edni\u00fa I.",
"Header": "Ceannt\u00e1sc",
"Right": "Ar Dheis",
"Insert column after": "Ions\u00e1igh col\u00fan ar dheis",
"Cols": "Col\u00fain",
"Insert row after": "Ions\u00e1igh r\u00f3 faoi",
"Width": "Leithead",
"Cell properties": "Air\u00edonna na cille",
"Left": "Ar Chl\u00e9",
"Cut row": "Gearr an r\u00f3",
"Delete column": "Scrios an col\u00fan",
"Center": "Sa L\u00e1r",
"Merge cells": "Cumaisc cealla",
"Insert template": "Ions\u00e1igh teimpl\u00e9ad",
"Templates": "Teimpl\u00e9id",
"Background color": "Dath an ch\u00falra",
"Custom...": "Saincheap...",
"Custom color": "Dath saincheaptha",
"No color": "Gan dath",
"Text color": "Dath an t\u00e9acs",
"Table of Contents": "Cl\u00e1r na n\u00c1bhar",
"Show blocks": "Taispe\u00e1in blocanna",
"Show invisible characters": "Taispe\u00e1in carachtair dhofheicthe",
"Words: {0}": "Focail: {0}",
"Insert": "Ions\u00e1ig",
"File": "Comhad",
"Edit": "Eagar",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Limist\u00e9ar M\u00e9ith-Th\u00e9acs. Br\u00faigh ALT-F9 le haghaidh roghchl\u00e1ir, ALT-F10 le haghaidh barra uirlis\u00ed, agus ALT-0 le c\u00fanamh a fh\u00e1il",
"Tools": "Uirlis\u00ed",
"View": "Amharc",
"Table": "T\u00e1bla",
"Format": "Form\u00e1id"
});