@charset "UTF-8";
joomla-alert {
  --jui-alert-min-width: 250px;
  --jui-alert-padding: .5rem 1.25rem;
  --jui-alert-margin: 0 0 1rem 0;
  --jui-alert-border: 1px solid transparent;
  --jui-alert-border-radius: .25rem;
  --jui-alert-animation-duration: .5s;
  --jui-alert-animation-timing-function: ease-in-out;
  --jui-alert-button-color-dark: #000;
  --jui-alert-button-color-light: #fff;
  --jui-alert-success-color: #234423;
  --jui-alert-success-background-color: #d9e6d9;
  --jui-alert-success-border-color: #cadcca;
  --jui-alert-success-link-color: #122212;
  --jui-alert-info-color: #0c5460;
  --jui-alert-info-background-color: #d1ecf1;
  --jui-alert-info-border-color: #bee5eb;
  --jui-alert-info-link-color: #062c33;
  --jui-alert-warning-color: #7d5a29;
  --jui-alert-warning-background-color: #fcefdc;
  --jui-alert-warning-border-color: #fbe8cd;
  --jui-alert-warning-link-color: #573e1c;
  --jui-alert-danger-color: #712b29;
  --jui-alert-danger-background-color: #f7dddc;
  --jui-alert-danger-border-color: #f4cfce;
  --jui-alert-danger-link-color: #4c1d1b;
  min-width: var(--jui-alert-min-width, 250px);
  padding: var(--jui-alert-padding, .5rem 1.25rem);
  margin: var(--jui-alert-margin, 0 0 1rem 0);
  border: var(--jui-alert-border, 1px solid transparent);
  border-radius: var(--jui-alert-border-radius, .25rem);
  animation-duration: var(--jui-alert-animation-duration, .5s);
  animation-timing-function: var(--jui-alert-animation-timing-function, ease-in-out);
  display: block;
}

joomla-alert .joomla-alert--close {
  float: right;
  color: var(--jui-alert-button-color-dark, #000);
  text-shadow: 0 1px 0 var(--jui-alert-button-color-light, #fff);
  opacity: .5;
  background: none;
  border: 0;
  padding: .2rem 1rem;
  font-size: 1.5rem;
  font-weight: 700;
  line-height: 1;
  position: relative;
  top: -.5rem;
  right: -1.25rem;
}

joomla-alert .joomla-alert--close:hover, joomla-alert .joomla-alert--close:focus {
  color: var(--jui-alert-button-color-dark, #000);
  cursor: pointer;
  opacity: .75;
  text-decoration: none;
}

joomla-alert[type="success"] {
  color: var(--jui-alert-success-color, #234423);
  background-color: var(--jui-alert-success-background-color, #d9e6d9);
  border-color: var(--jui-alert-success-border-color, #cadcca);
}

joomla-alert[type="success"] hr {
  border-top-color: var(--jui-alert-success-border-color, #cadcca);
}

joomla-alert[type="success"] .alert-link {
  color: var(--jui-alert-success-link-color, #122212);
}

joomla-alert[type="info"] {
  color: var(--jui-alert-info-color, #0c5460);
  background-color: var(--jui-alert-info-background-color, #d1ecf1);
  border-color: var(--jui-alert-info-border-color, #bee5eb);
}

joomla-alert[type="info"] hr {
  border-top-color: var(--jui-alert-info-border-color, #bee5eb);
}

joomla-alert[type="info"] .alert-link {
  color: var(--jui-alert-info-link-color, #062c33);
}

joomla-alert[type="warning"] {
  color: var(--jui-alert-warning-color, #7d5a29);
  background-color: var(--jui-alert-warning-background-color, #fcefdc);
  border-color: var(--jui-alert-warning-border-color, #fbe8cd);
}

joomla-alert[type="warning"] hr {
  border-top-color: var(--jui-alert-warning-border-color, #fbe8cd);
}

joomla-alert[type="warning"] .alert-link {
  color: var(--jui-alert-warning-link-color, #573e1c);
}

joomla-alert[type="danger"] {
  color: var(--jui-alert-danger-color, #712b29);
  background-color: var(--jui-alert-danger-background-color, #f7dddc);
  border-color: var(--jui-alert-danger-border-color, #f4cfce);
}

joomla-alert[type="danger"] hr {
  border-top-color: var(--jui-alert-danger-border-color, #f4cfce);
}

joomla-alert[type="danger"] .alert-link {
  color: var(--jui-alert-danger-link-color, #4c1d1b);
}

html[dir="rtl"] joomla-alert .joomla-alert--close, html[dir="rtl"] joomla-alert .joomla-alert-button--close {
  float: left;
  left: -1.25rem;
  right: auto;
}

@keyframes joomla-alert-fade-in {
  0% {
    opacity: 0;
  }
}

@keyframes joomla-alert-fade-out {
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
}

@media (prefers-reduced-motion: reduce) {
  joomla-alert {
    animation-duration: 1ms !important;
  }
}

#system-message-container joomla-alert {
  width: 100%;
  min-width: 16rem;
  color: var(--alert-accent-color);
  background-color: var(--alert-bg-color);
  border: 1px solid;
  border-color: var(--alert-border);
  border-radius: .2rem;
  margin-bottom: 1rem;
  padding: 0;
  transition: opacity .15s linear;
  display: flex;
  position: relative;
}

#system-message-container joomla-alert[type="success"], #system-message-container joomla-alert[type="message"] {
  --alert-accent-color: var(--state-success-text);
  --alert-bg-color: var(--state-success-bg);
  --alert-border: var(--state-success-bg-hvr);
  --alert-heading-bg: var(--state-success-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="success"] a:not(.btn), #system-message-container joomla-alert[type="success"] .btn-link, #system-message-container joomla-alert[type="message"] a:not(.btn), #system-message-container joomla-alert[type="message"] .btn-link {
  color: var(--state-success-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="info"], #system-message-container joomla-alert[type="notice"] {
  --alert-accent-color: var(--state-info-text);
  --alert-bg-color: var(--state-info-bg);
  --alert-border: var(--state-info-bg-hvr);
  --alert-heading-bg: var(--state-info-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="info"] a:not(.btn), #system-message-container joomla-alert[type="info"] .btn-link, #system-message-container joomla-alert[type="notice"] a:not(.btn), #system-message-container joomla-alert[type="notice"] .btn-link {
  color: var(--state-info-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="warning"] .joomla-alert--close {
  color: var(--state-warning-text);
}

#system-message-container joomla-alert[type="warning"] {
  --alert-accent-color: var(--state-warning-text);
  --alert-bg-color: var(--state-warning-bg);
  --alert-border: var(--state-warning-border);
  --alert-heading-bg: var(--state-warning-heading-bg);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="warning"] a:not(.btn), #system-message-container joomla-alert[type="warning"] .btn-link {
  color: var(--state-warning-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="error"], #system-message-container joomla-alert[type="danger"] {
  --alert-accent-color: var(--state-danger-text);
  --alert-bg-color: var(--state-danger-bg);
  --alert-border: var(--state-danger-bg-hvr);
  --alert-heading-bg: var(--state-danger-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="error"] a:not(.btn), #system-message-container joomla-alert[type="error"] .btn-link, #system-message-container joomla-alert[type="danger"] a:not(.btn), #system-message-container joomla-alert[type="danger"] .btn-link {
  color: var(--state-danger-text);
  text-decoration: underline;
}

#system-message-container joomla-alert .alert-heading {
  color: #fff;
  background: var(--alert-heading-bg);
  flex-direction: column;
  place-content: center;
  padding: .8rem;
  line-height: normal;
  display: flex;
}

#system-message-container joomla-alert .alert-heading .message:before, #system-message-container joomla-alert .alert-heading .success:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M1299 813l-422 422q-19 19-45 19t-45-19l-294-294q-19-19-19-45t19-45l102-102q19-19 45-19t45 19l147 147 275-275q19-19 45-19t45 19l102 102q19 19 19 45t-19 45zm141 83q0-148-73-273t-198-198-273-73-273 73-198 198-73 273 73 273 198 198 273 73 273-73 198-198 73-273zm224 0q0 209-103 385.5t-279.5 279.5-385.5 103-385.5-103-279.5-279.5-103-385.5 103-385.5 279.5-279.5 385.5-103 385.5 103 279.5 279.5 103 385.5z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .notice:before, #system-message-container joomla-alert .alert-heading .info:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119.043 8 8 119.083 8 256c0 136.997 111.043 248 248 248s248-111.003 248-248C504 119.083 392.957 8 256 8zm0 110c23.196 0 42 18.804 42 42s-18.804 42-42 42-42-18.804-42-42 18.804-42 42-42zm56 254c0 6.627-5.373 12-12 12h-88c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h12v-64h-12c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h64c6.627 0 12 5.373 12 12v100h12c6.627 0 12 5.373 12 12v24z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .warning:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(0, 0, 0, .95)\" d=\"M1024 1375v-190q0-14-9.5-23.5t-22.5-9.5h-192q-13 0-22.5 9.5t-9.5 23.5v190q0 14 9.5 23.5t22.5 9.5h192q13 0 22.5-9.5t9.5-23.5zm-2-374l18-459q0-12-10-19-13-11-24-11h-220q-11 0-24 11-10 7-10 21l17 457q0 10 10 16.5t24 6.5h185q14 0 23.5-6.5t10.5-16.5zm-14-934l768 1408q35 63-2 126-17 29-46.5 46t-63.5 17h-1536q-34 0-63.5-17t-46.5-46q-37-63-2-126l768-1408q17-31 47-49t65-18 65 18 47 49z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .error:before, #system-message-container joomla-alert .alert-heading .danger:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8zm0 448c-110.5 0-200-89.5-200-200S145.5 56 256 56s200 89.5 200 200-89.5 200-200 200zm101.8-262.2L295.6 256l62.2 62.2c4.7 4.7 4.7 12.3 0 17l-22.6 22.6c-4.7 4.7-12.3 4.7-17 0L256 295.6l-62.2 62.2c-4.7 4.7-12.3 4.7-17 0l-22.6-22.6c-4.7-4.7-4.7-12.3 0-17l62.2-62.2-62.2-62.2c-4.7-4.7-4.7-12.3 0-17l22.6-22.6c4.7-4.7 12.3-4.7 17 0l62.2 62.2 62.2-62.2c4.7-4.7 12.3-4.7 17 0l22.6 22.6c4.7 4.7 4.7 12.3 0 17z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-wrapper {
  width: 100%;
}

#system-message-container joomla-alert .alert-link {
  color: var(--states-link-color);
  font-weight: normal;
  text-decoration: underline;
}

#system-message-container joomla-alert .joomla-alert--close, #system-message-container joomla-alert .joomla-alert-button--close {
  top: 0;
  color: var(--alert-accent-color);
  text-shadow: none;
  opacity: 1;
  background: none;
  border: 0;
  padding: .75rem .8rem;
  font-size: 2rem;
  line-height: 1rem;
  position: absolute;
  inset-inline-end: 0;
}

#system-message-container joomla-alert .joomla-alert--close:hover, #system-message-container joomla-alert .joomla-alert--close:focus, #system-message-container joomla-alert .joomla-alert-button--close:hover, #system-message-container joomla-alert .joomla-alert-button--close:focus {
  cursor: pointer;
  opacity: .75;
  text-decoration: none;
}

#system-message-container joomla-alert div {
  font-size: 1rem;
}

#system-message-container joomla-alert div .alert-message {
  padding: .15rem .3rem;
  margin: .5rem;
  padding-inline-end: 2rem;
}

#system-message-container joomla-alert div .alert-message:not(:first-of-type) {
  border-top: 1px solid var(--alert-accent-color);
}
