<?php

/**
 * @package     Joomla.Administrator
 * @subpackage  com_users
 *
 * @copyright   (C) 2008 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Component\Users\Administrator\View\User\HtmlView;

/** @var HtmlView $this */

echo HTMLHelper::_('access.usergroups', 'jform[groups]', $this->groups, true);
