tinymce.addI18n('es',{
"Redo": "Rehacer",
"Undo": "Deshacer",
"Cut": "Cortar",
"Copy": "Copiar",
"Paste": "Pegar",
"Select all": "Seleccionar todo",
"New document": "Nuevo documento",
"Ok": "Ok",
"Cancel": "Cancelar",
"Visual aids": "Ayudas visuales",
"Bold": "Negrita",
"Italic": "Cursiva",
"Underline": "Subrayado",
"Strikethrough": "Tachado",
"Superscript": "Super\u00edndice",
"Subscript": "Sub\u00edndice",
"Clear formatting": "Limpiar formato",
"Align left": "Alinear a la izquierda",
"Align center": "Centrar",
"Align right": "Alinear a la derecha",
"Justify": "Justificar",
"Bullet list": "Lista de vi\u00f1etas",
"Numbered list": "Lista numerada",
"Decrease indent": "Disminuir sangr\u00eda",
"Increase indent": "Aumentar sangr\u00eda",
"Close": "Cerrar",
"Formats": "Formatos",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "Su navegador no soporta el acceso directo al portapapeles. Por favor, use los comandos de teclado Ctrl+X\/C\/V",
"Headers": "Encabezados",
"Header 1": "Encabezado 1",
"Header 2": "Encabezado 2",
"Header 3": "Encabezado 3",
"Header 4": "Encabezado 4",
"Header 5": "Encabezado 5",
"Header 6": "Encabezado 6",
"Headings": "T\u00edtulos",
"Heading 1": "T\u00edtulo 1",
"Heading 2": "T\u00edtulo 2",
"Heading 3": "T\u00edtulo 3",
"Heading 4": "T\u00edtulo 4",
"Heading 5": "T\u00edtulo 5",
"Heading 6": "T\u00edtulo 6",
"Preformatted": "Preformateado",
"Div": "Div",
"Pre": "Pre",
"Code": "C\u00f3digo",
"Paragraph": "P\u00e1rrafo",
"Blockquote": "Cita",
"Inline": "En l\u00ednea",
"Blocks": "Bloques",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Pegar ahora est\u00e1 en modo de texto sin formato. El contenido ahora se pegar\u00e1 como texto sin formato hasta que desactive esta opci\u00f3nn.",
"Fonts": "Fonts",
"Font Sizes": "Tama\u00f1os de fuente",
"Class": "Clase",
"Browse for an image": "Examinar imagen",
"OR": "O",
"Drop an image here": "Arrastrar imagen aqu\u00ed",
"Upload": "Subir",
"Block": "Bloque",
"Align": "Alinear",
"Default": "Predeterminado",
"Circle": "C\u00edrculo",
"Disc": "Disco",
"Square": "Cuadrado",
"Lower Alpha": "Alfa min\u00fascula",
"Lower Greek": "Griega min\u00fascula",
"Lower Roman": "Romana min\u00fascula",
"Upper Alpha": "Alfa may\u00fascula",
"Upper Roman": "Romana may\u00fascula",
"Anchor...": "Ancla...",
"Name": "Nombre",
"Id": "Id",
"Id should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "El ID debe comenzar con una letra, seguida solo de letras, n\u00fameros, guiones, puntos, dos puntos o guiones bajos.",
"You have unsaved changes are you sure you want to navigate away?": "Tiene cambios sin guardar, \u00bfest\u00e1 seguro de que desea salir?",
"Restore last draft": "Restaurar el \u00faltimo borrador",
"Special character...": "Car\u00e1cter especial...",
"Source code": "C\u00f3digo fuente",
"Insert\/Edit code sample": "Insertar\/Editar c\u00f3ndigo de ejemplo",
"Language": "Idioma",
"Code sample...": "C\u00f3ndigo de ejemplo...",
"Color Picker": "Selector de color",
"R": "R",
"G": "G",
"B": "B",
"Left to right": "De derecha a izquierda",
"Right to left": "De izquierda a derecha",
"Emoticons": "Emoticonos",
"Emoticons...": "Emoticonos...",
"Metadata and Document Properties": "Metadatos y Propiedades del documento",
"Title": "T\u00edtulo",
"Keywords": "Palabras clave",
"Description": "Descripci\u00f3nn",
"Robots": "Robots",
"Author": "Author",
"Encoding": "Codificaci\u00f3nn",
"Fullscreen": "Pantalla completa",
"Action": "Acci\u00f3nn",
"Shortcut": "Atajo",
"Help": "Ayuda",
"Address": "Direcci\u00f3nn",
"Focus to menubar": "Enfoque a la barra de men\u00fa",
"Focus to toolbar": "Enfoque a la barra de herramientas",
"Focus to element path": "Enfoque a la ruta del elemento",
"Focus to contextual toolbar": "Enfoque a la barra de herramientas contextual",
"Insert link (if link plugin activated)": "Insertar enlace (Si est\u00e1 activado el plugin de 'Enlaces')",
"Save (if save plugin activated)": "Guardar (Si est\u00e1 activado el plugin de 'Guardar')",
"Find (if searchreplace plugin activated)": "Buscar (Si est\u00e1 activado el plugin 'searchreplace')",
"Plugins installed ({0}):": "Plugins instalados ({0}):",
"Premium plugins:": "Plugins premium:",
"Learn more...": "Saber m\u00e1s...",
"You are using {0}": "Est\u00e1 usando {0}",
"Plugins": "Plugins",
"Handy Shortcuts": "Atajos pr\u00e1cticos",
"Horizontal line": "L\u00ednea horizontal",
"Insert\/edit image": "Insertar\/Editar imagen",
"Alternative description": "Descripci\u00f3n alternativa",
"Accessibility": "Accesibilidad",
"Image is decorative": "La imagen es decorativa",
"Source": "Origen",
"Dimensions": "Dimensiones",
"Constrain proportions": "Restringir las proporciones",
"General": "General",
"Advanced": "Advanzado",
"Style": "Estilo",
"Vertical space": "Espacio vertical",
"Horizontal space": "Espacio horizontal",
"Border": "Borde",
"Insert image": "Insertar imagen",
"Image...": "Imagen...",
"Image list": "Lista de im\u00e1genes",
"Rotate counterclockwise": "Rotar en sentido antihorario",
"Rotate clockwise": "Rotar en sentido horario",
"Flip vertically": "Voltear verticalmente",
"Flip horizontally": "Voltear horizontalmente",
"Edit image": "Editar imagen",
"Image options": "Opciones de imagen",
"Zoom in": "Ampliar",
"Zoom out": "Disminuir",
"Crop": "Recortar",
"Resize": "Redimensionar",
"Orientation": "Orientation",
"Brightness": "Brillo",
"Sharpen": "Agudizar",
"Contrast": "Contraste",
"Color levels": "Niveles de color",
"Gamma": "Gama",
"Invert": "Invertir",
"Apply": "Aplicar",
"Back": "Volver",
"Insert date\/time": "Insertar fecha\/hora",
"Date\/time": "Fecha\/Hora",
"Insert\/edit link": "Insertar\/Editar enlace",
"Text to display": "Texto a mostrar",
"Url": "Url",
"Open link in...": "Abrir el enlace en...",
"Current window": "Misma ventana",
"None": "Ninguna",
"New window": "Nueva ventana",
"Open link": "Abrir enlace",
"Remove link": "Eliminar enlace",
"Anchors": "Anclas",
"Link...": "Enlace...",
"Paste or type a link": "Pegue o escriba un enlace",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "La URL que ha introducido parece ser una direcci\u00f3nn de correo electr\u00f3nnico. \u00bfDesea a\u00f1adir el prefijo requerido 'para:'?",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "La URL que ha introducido parece ser un enlace externo. \u00bfDesea agregar el prefijo requerido 'http:\/\/'?",
"The URL you entered seems to be an external link. Do you want to add the required https:\/\/ prefix?": "La URL que ha introducido parece ser un enlace externo. \u00bfDesea a\u00f1adir el prefijo requerido 'https:\/\/'?",
"Link list": "Lista de enlaces",
"Insert video": "Insertar v\u00eddeo",
"Insert\/edit video": "Insertar\/Editar v\u00eddeo",
"Insert\/edit media": "Insertar\/Editar multimedia",
"Alternative source": "Fuente alternativa",
"Alternative source URL": "Fuente alternativa de la URL",
"Media poster (Image URL)": "Poster multimedia (Imagen URL)",
"Paste your embed code below:": "Pegue su c\u00f3ndigo de inserci\u00f3nn a continuaci\u00f3nn:",
"Embed": "Insertado",
"Media...": "Multimedia...",
"Nonbreaking space": "Espacio irrompible",
"Page break": "Salto de p\u00e1gina",
"Paste as text": "Pegue un texto",
"Preview": "Vista previa",
"Print...": "Imprimir...",
"Save": "Guardar",
"Find": "Buscar",
"Replace with": "Reemplazar con",
"Replace": "Reemplazar",
"Replace all": "Reemplazar todo",
"Previous": "Anterior",
"Next": "Siguiente",
"Find and Replace": "Buscar y Reemplazar",
"Find and replace...": "Buscar y Reemplazar...",
"Could not find the specified string.": "No se pudo encontrar el texto especificado.",
"Match case": "Caso de coincidencia",
"Find whole words only": "Buscar solo por palabras completas",
"Find in selection": "Encontrar en la selecci\u00f3n",
"Spellcheck": "Corrector ortogr\u00e1fico",
"Spellcheck Language": "Idioma del corrector ortogr\u00e1fico",
"No misspellings found.": "No se encontraron errores ortogr\u00e1ficos.",
"Ignore": "Ignorar",
"Ignore all": "Ignorar todo",
"Finish": "Terminar",
"Add to Dictionary": "A\u00f1adir al diccionario",
"Insert table": "Insertar tabla",
"Table properties": "Propiedades de la tabla",
"Delete table": "Borrar tabla",
"Cell": "Celda",
"Row": "Fila",
"Column": "Columna",
"Cell properties": "Propiedades de la celda",
"Merge cells": "Combinar celdas",
"Split cell": "Separar celdas",
"Insert row before": "Insertar fila antes",
"Insert row after": "Insertar fila despu\u00e9s",
"Delete row": "Borrar fila",
"Row properties": "Propiedades de la fila",
"Cut row": "Cortar fila",
"Copy row": "Copiar fila",
"Paste row before": "Pegar fila antes",
"Paste row after": "Paste fila despu\u00e9s",
"Insert column before": "Insertar columna antes",
"Insert column after": "Insertar columna despu\u00e9s",
"Delete column": "Borrar columna",
"Cols": "Columnas",
"Rows": "Filas",
"Width": "Ancho",
"Height": "Alto",
"Cell spacing": "Espaciado de celdas",
"Cell padding": "Relleno de celdas",
"Caption": "T\u00edtulo",
"Show caption": "Mostrar t\u00edtulo",
"Left": "Izquierda",
"Center": "Centro",
"Right": "Derecha",
"Cell type": "Tipo de celda",
"Scope": "\u00c1mbito",
"Alignment": "Alineaci\u00f3nn",
"H Align": "Alineamineto H",
"V Align": "Alineamiento V",
"Top": "Superior",
"Middle": "Mitad",
"Bottom": "Inferior",
"Header cell": "Encabezado de celda",
"Row group": "Grupo de fila",
"Column group": "Grupo de columna",
"Row type": "Tipo de fila",
"Header": "Encabezado",
"Body": "Cuerpo",
"Footer": "Pie",
"Border color": "Color del borde",
"Insert template...": "Insertar plantilla...",
"Templates": "Plantillas",
"Template": "Plantilla",
"Text color": "Color del texto",
"Background color": "Color del fondo",
"Custom...": "Personalizado...",
"Custom color": "Color personalizado",
"No color": "Sin color",
"Remove color": "Quitar color",
"Table of Contents": "Tabla de contenidos",
"Show blocks": "Mostrar bloques",
"Show invisible characters": "Mostrar caracteres invisibles",
"Word count": "Contar palabras",
"Count": "Contar",
"Document": "Documento",
"Selection": "Selecci\u00f3nn",
"Words": "Palabras",
"Words: {0}": "Palabras: {0}",
"{0} words": "{0} palabras",
"File": "Archivo",
"Edit": "Editar",
"Insert": "Insertar",
"View": "Vista",
"Format": "Formato",
"Table": "Tabla",
"Tools": "Herramientas",
"Powered by {0}": "Powered by {0}",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "\u00e1rea de texto enriquecido. Presione ALT-F9 para el men\u00fa. Presione ALT-F10 para la barra de herramientas. Presione ALT-0 para obtener ayuda",
"Image title": "T\u00edtulo de la imagen",
"Border width": "Ancho del borde",
"Border style": "Estilo del borde",
"Error": "Error",
"Warn": "Advertencia",
"Valid": "V\u00e1lido",
"To open the popup, press Shift+Enter": "Para abrir la ventana emergente, presione Shift + Enter",
"Rich Text Area. Press ALT-0 for help.": "\u00e1rea de texto enriquecido. Presione ALT-0 para obtener ayuda.",
"System Font": "Fuente del sistema",
"Failed to upload image: {0}": "No se pudo subir la imagen: {0}",
"Failed to load plugin: {0} from url {1}": "No se pudo cargar el plugin: {0} desde la URL {1}",
"Failed to load plugin url: {0}": "No se pudo cargar la URL del plugin: {0}",
"Failed to initialize plugin: {0}": "No se pudo inicializar el plugin: {0}",
"example": "ejemplo",
"Search": "Buscar",
"All": "Todo",
"Currency": "Moneda",
"Text": "Texto",
"Quotations": "Citas",
"Mathematical": "Matem\u00e1tico",
"Extended Latin": "Lat\u00edn extendido",
"Symbols": "Simbolos",
"Arrows": "Flechas",
"User Defined": "Definido por el usuario",
"dollar sign": "s\u00edmbolo de d\u00f3nlar",
"currency sign": "s\u00edmbolo de moneda",
"euro-currency sign": "s\u00edmbolo de moneda euro",
"colon sign": "s\u00edmbolo de dos puntos",
"cruzeiro sign": "s\u00edmbolo de cruzeiro",
"french franc sign": "s\u00edmbolo del franco franc\u00e9s",
"lira sign": "s\u00edmbolo de lira",
"mill sign": "s\u00edmbolo de mill",
"naira sign": "s\u00edmbolo de naira",
"peseta sign": "s\u00edmbolo de peseta",
"rupee sign": "s\u00edmbolo de rupia",
"won sign": "s\u00edmbolo de won",
"new sheqel sign": "s\u00edmbolo de nuevo sheqel",
"dong sign": "s\u00edmbolo de dong",
"kip sign": "s\u00edmbolo de kip",
"tugrik sign": "s\u00edmbolo de tugrik",
"drachma sign": "s\u00edmbolo de dragma",
"german penny symbol": "s\u00edmbolo de un centavo alem\u00e1n",
"peso sign": "s\u00edmbolo de peso",
"guarani sign": "s\u00edmbolo de guarani",
"austral sign": "s\u00edmbolo de austral",
"hryvnia sign": "s\u00edmbolo de hryvnia",
"cedi sign": "s\u00edmbolo de cedi",
"livre tournois sign": "s\u00edmbolo de livre tournois",
"spesmilo sign": "s\u00edmbolo de spesmilo",
"tenge sign": "s\u00edmbolo de tenge",
"indian rupee sign": "s\u00edmbolo de rupia india",
"turkish lira sign": "s\u00edmbolo de lira turca",
"nordic mark sign": "s\u00edmbolo de marco n\u00f3nrdico",
"manat sign": "s\u00edmbolo de manat",
"ruble sign": "s\u00edmbolo de rublo",
"yen character": "caracter de yen",
"yuan character": "caracter de yuan",
"yuan character, in hong kong and taiwan": "caracter de yuan, en Hong Kong y Taiw\u00e1n",
"yen\/yuan character variant one": "variante uno de caracter yen\/yuan",
"Loading emoticons...": "Cargando emoticonos...",
"Could not load emoticons": "No se pudieron cargar emoticonos",
"People": "Gente",
"Animals and Nature": "Animales y naturaleza",
"Food and Drink": "Comida y bebida",
"Activity": "Actividad",
"Travel and Places": "Viajes y lugares",
"Objects": "Objectos",
"Flags": "Banderas",
"Characters": "Characteres",
"Characters (no spaces)": "Characteres (sin espacios)",
"{0} characters": "{0} characteres",
"Error: Form submit field collision.": "Error: Colisi\u00f3nn con env\u00edo de campo del formulario.",
"Error: No form element found.": "Error: no se encontr\u00f3n ning\u00fan elemento de formulario.",
"Update": "Actualizar",
"Color swatch": "Muestra de color",
"Turquoise": "Turquesa",
"Green": "Verde",
"Blue": "Azul",
"Purple": "Violeta",
"Navy Blue": "Azul marino",
"Dark Turquoise": "Turquesa oscuro",
"Dark Green": "Verde oscuro",
"Medium Blue": "Azul intermedio",
"Medium Purple": "Violeta intermedio",
"Midnight Blue": "Azul intermedio",
"Yellow": "Amarillo",
"Orange": "Naranja",
"Red": "Rojo",
"Light Gray": "Gris claro",
"Gray": "Gris",
"Dark Yellow": "Amarillo oscuro",
"Dark Orange": "Naranja oscuro",
"Dark Red": "Rojo oscuro",
"Medium Gray": "Gris intermedio",
"Dark Gray": "Gris oscuro",
"Light Green": "Verde claro",
"Light Yellow": "Amarillo claro",
"Light Red": "Rojo claro",
"Light Purple": "Violeta claro",
"Light Blue": "Azul claro",
"Dark Purple": "Vileta oscuro",
"Dark Blue": "Azul oscuro",
"Black": "Negro",
"White": "Blanco",
"Switch to or from fullscreen mode": "Cambiar hacia o desde el modo de pantalla completa",
"Open help dialog": "Abrir di\u00e1logo de ayuda",
"history": "historia",
"styles": "estilos",
"formatting": "formateo",
"alignment": "alineaci\u00f3nn",
"indentation": "sangr\u00eda",
"Font": "Fuente",
"Size": "Tama\u00f1o",
"More...": "M\u00e1s...",
"Select...": "Seleccionar...",
"Preferences": "Prefer\u00e9ncias",
"Yes": "S\u00ed",
"No": "No",
"Keyboard Navigation": "Navegaci\u00f3nn por teclado",
"Version": "Versi\u00f3nn",
"Code view": "Vista de c\u00f3digo",
"Open popup menu for split buttons": "Abrir men\u00fa emergente para botones divididos",
"List Properties": "Propiedades de lista",
"List properties...": "Propiedades de lista...",
"Start list at number": "Iniciar lista en el n\u00famero",
"Line height": "Altura de la l\u00ednea",
"comments": "comment\u00e1rios",
"Format Painter": "Pintura de formato",
"Insert\/edit iframe": "Insertar\/editar 'iframe'",
"Capitalization": "Capitalizaci\u00f3nn",
"lowercase": "min\u00f1scula",
"UPPERCASE": "MAY\u00daSCULA",
"Title Case": "Capitalizaci\u00f3nn del t\u00edtulo",
"permanent pen": "pluma permanente",
"Permanent Pen Properties": "Propiedades permanentes de la pluma",
"Permanent pen properties...": "Propiedades permanentes de la pluma...",
"case change": "Cambiar may\u00fasculas y min\u00fasculas",
"page embed": "p\u00e1gina incrustada",
"Advanced sort...": "Orden avanzado...",
"Advanced Sort": "Orden Avanzado",
"Sort table by column ascending": "Ordenar tabla por columna ascendente",
"Sort table by column descending": "Ordenar tabla por columna descendente",
"Sort": "Ordenar",
"Order": "Orden",
"Sort by": "Ordenar por",
"Ascending": "Ascendente",
"Descending": "Descendente",
"Column {0}": "Columna {0}",
"Row {0}": "Fila {0}",
"Spellcheck...": "Corrector...",
"Misspelled word": "Palabra mal escrita",
"Suggestions": "Sugerencias",
"Change": "Cambiar",
"Finding word suggestions": "Encontrar sugerencias de palabras",
"Success": "\u00c9xito",
"Repair": "Reparar",
"Issue {0} of {1}": "Problema {0} de {1}",
"Images must be marked as decorative or have an alternative text description": "Las im\u00e1genes deben estar marcadas como decorativas o tener una descripci\u00f3n de texto alternativo",
"Images must have an alternative text description. Decorative images are not allowed.": "Las im\u00e1genes deben tener una descripci\u00f3n de texto alternativa. No se permiten im\u00e1genes decorativas.",
"Or provide alternative text:": "O proporcione un texto alternativo:",
"Make image decorative:": "Hacer la imagen decorativa:",
"ID attribute must be unique": "El atributo del ID debe ser \u00fanico",
"Make ID unique": "Hacer que el ID sea \u00fanico",
"Keep this ID and remove all others": "Conserve este ID y elimine todos los dem\u00e1s",
"Remove this ID": "Eliminar este ID",
"Remove all IDs": "Eliminar todos los ID",
"Checklist": "Lista de verificaci\u00f3n",
"Anchor": "Ancla",
"Special character": "Car\u00e1cter especial",
"Code sample": "C\u00f3ndigo de ejemplo",
"Color": "Color",
"Document properties": "Propiedades del documento",
"Image description": "Descripci\u00f3nn de la imagen",
"Image": "Imagen",
"Insert link": "Insertar enlace",
"Target": "Destino",
"Link": "Enlace",
"Poster": "Poster",
"Media": "Multimedia",
"Print": "Imprimir",
"Prev": "Previsualizar",
"Find and replace": "Buscar y reemplazar",
"Whole words": "Palabras completas",
"Insert template": "Insertar plantilla"
});
