<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname(dirname($vendorDir));

return array(
    'voku\\' => array($vendorDir . '/voku/portable-utf8/src/voku', $vendorDir . '/voku/portable-ascii/src/voku'),
    'phpseclib3\\' => array($vendorDir . '/phpseclib/phpseclib/phpseclib'),
    'enshrined\\svgSanitize\\' => array($vendorDir . '/enshrined/svg-sanitize/src'),
    'bcmath_compat\\' => array($vendorDir . '/phpseclib/bcmath_compat/src'),
    'Webmozart\\Assert\\' => array($vendorDir . '/webmozart/assert/src'),
    'Webauthn\\MetadataService\\' => array($vendorDir . '/web-auth/metadata-service/src'),
    'Webauthn\\' => array($vendorDir . '/web-auth/webauthn-lib/src'),
    'Wamania\\Snowball\\' => array($vendorDir . '/wamania/php-stemmer/src'),
    'Tuf\\' => array($vendorDir . '/php-tuf/php-tuf/src'),
    'Tobscure\\JsonApi\\' => array($vendorDir . '/tobscure/json-api/src'),
    'TYPO3\\PharStreamWrapper\\' => array($vendorDir . '/typo3/phar-stream-wrapper/src'),
    'Symfony\\Polyfill\\Uuid\\' => array($vendorDir . '/symfony/polyfill-uuid'),
    'Symfony\\Polyfill\\Php83\\' => array($vendorDir . '/symfony/polyfill-php83'),
    'Symfony\\Polyfill\\Mbstring\\' => array($vendorDir . '/symfony/polyfill-mbstring'),
    'Symfony\\Polyfill\\Intl\\Normalizer\\' => array($vendorDir . '/symfony/polyfill-intl-normalizer'),
    'Symfony\\Polyfill\\Intl\\Grapheme\\' => array($vendorDir . '/symfony/polyfill-intl-grapheme'),
    'Symfony\\Polyfill\\Iconv\\' => array($vendorDir . '/symfony/polyfill-iconv'),
    'Symfony\\Polyfill\\Ctype\\' => array($vendorDir . '/symfony/polyfill-ctype'),
    'Symfony\\Contracts\\Translation\\' => array($vendorDir . '/symfony/translation-contracts'),
    'Symfony\\Contracts\\Service\\' => array($vendorDir . '/symfony/service-contracts'),
    'Symfony\\Contracts\\HttpClient\\' => array($vendorDir . '/symfony/http-client-contracts'),
    'Symfony\\Component\\Yaml\\' => array($vendorDir . '/symfony/yaml'),
    'Symfony\\Component\\WebLink\\' => array($vendorDir . '/symfony/web-link'),
    'Symfony\\Component\\VarDumper\\' => array($vendorDir . '/symfony/var-dumper'),
    'Symfony\\Component\\Validator\\' => array($vendorDir . '/symfony/validator'),
    'Symfony\\Component\\Uid\\' => array($vendorDir . '/symfony/uid'),
    'Symfony\\Component\\String\\' => array($vendorDir . '/symfony/string'),
    'Symfony\\Component\\OptionsResolver\\' => array($vendorDir . '/symfony/options-resolver'),
    'Symfony\\Component\\Ldap\\' => array($vendorDir . '/symfony/ldap'),
    'Symfony\\Component\\HttpClient\\' => array($vendorDir . '/symfony/http-client'),
    'Symfony\\Component\\ErrorHandler\\' => array($vendorDir . '/symfony/error-handler'),
    'Symfony\\Component\\Console\\' => array($vendorDir . '/symfony/console'),
    'SpomkyLabs\\Pki\\' => array($vendorDir . '/spomky-labs/pki-framework/src'),
    'ReCaptcha\\' => array($vendorDir . '/google/recaptcha/src/ReCaptcha'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/src'),
    'Psr\\Link\\' => array($vendorDir . '/psr/link/src'),
    'Psr\\Http\\Message\\' => array($vendorDir . '/psr/http-factory/src', $vendorDir . '/psr/http-message/src'),
    'Psr\\Http\\Client\\' => array($vendorDir . '/psr/http-client/src'),
    'Psr\\EventDispatcher\\' => array($vendorDir . '/psr/event-dispatcher/src'),
    'Psr\\Container\\' => array($vendorDir . '/psr/container/src'),
    'Psr\\Clock\\' => array($vendorDir . '/psr/clock/src'),
    'Psr\\Cache\\' => array($vendorDir . '/psr/cache/src'),
    'ParagonIE\\ConstantTime\\' => array($vendorDir . '/paragonie/constant_time_encoding/src'),
    'PHPMailer\\PHPMailer\\' => array($vendorDir . '/phpmailer/phpmailer/src'),
    'Negotiation\\' => array($vendorDir . '/willdurand/negotiation/src/Negotiation'),
    'Lcobucci\\JWT\\' => array($vendorDir . '/lcobucci/jwt/src'),
    'Lcobucci\\Clock\\' => array($vendorDir . '/lcobucci/clock/src'),
    'Laminas\\Diactoros\\' => array($vendorDir . '/laminas/laminas-diactoros/src'),
    'Jose\\Component\\' => array($vendorDir . '/web-token/jwt-library'),
    'Joomla\\Utilities\\' => array($vendorDir . '/joomla/utilities/src'),
    'Joomla\\Uri\\' => array($vendorDir . '/joomla/uri/src'),
    'Joomla\\String\\' => array($vendorDir . '/joomla/string/src'),
    'Joomla\\Session\\' => array($vendorDir . '/joomla/session/src'),
    'Joomla\\Router\\' => array($vendorDir . '/joomla/router/src'),
    'Joomla\\Registry\\' => array($vendorDir . '/joomla/registry/src'),
    'Joomla\\OAuth2\\' => array($vendorDir . '/joomla/oauth2/src'),
    'Joomla\\OAuth1\\' => array($vendorDir . '/joomla/oauth1/src'),
    'Joomla\\Language\\' => array($vendorDir . '/joomla/language/src'),
    'Joomla\\Input\\' => array($vendorDir . '/joomla/input/src'),
    'Joomla\\Http\\' => array($vendorDir . '/joomla/http/src'),
    'Joomla\\Filter\\' => array($vendorDir . '/joomla/filter/src'),
    'Joomla\\Filesystem\\' => array($vendorDir . '/joomla/filesystem/src'),
    'Joomla\\Event\\' => array($vendorDir . '/joomla/event/src'),
    'Joomla\\Database\\' => array($vendorDir . '/joomla/database/src'),
    'Joomla\\Data\\' => array($vendorDir . '/joomla/data/src'),
    'Joomla\\DI\\' => array($vendorDir . '/joomla/di/src'),
    'Joomla\\Crypt\\' => array($vendorDir . '/joomla/crypt/src'),
    'Joomla\\Console\\' => array($vendorDir . '/joomla/console/src'),
    'Joomla\\CMS\\' => array($baseDir . '/libraries/src'),
    'Joomla\\Authentication\\' => array($vendorDir . '/joomla/authentication/src'),
    'Joomla\\Archive\\' => array($vendorDir . '/joomla/archive/src'),
    'Joomla\\Application\\' => array($vendorDir . '/joomla/application/src'),
    'Jfcherng\\Utility\\' => array($vendorDir . '/jfcherng/php-mb-string/src', $vendorDir . '/jfcherng/php-color-output/src'),
    'Jfcherng\\Diff\\' => array($vendorDir . '/jfcherng/php-diff/src', $vendorDir . '/jfcherng/php-sequence-matcher/src'),
    'GuzzleHttp\\Psr7\\' => array($vendorDir . '/guzzlehttp/psr7/src'),
    'GuzzleHttp\\Promise\\' => array($vendorDir . '/guzzlehttp/promises/src'),
    'Fig\\Link\\' => array($vendorDir . '/fig/link-util/src'),
    'Doctrine\\Inflector\\' => array($vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector'),
    'Doctrine\\Common\\Inflector\\' => array($vendorDir . '/doctrine/inflector/lib/Doctrine/Common/Inflector'),
    'Defuse\\Crypto\\' => array($vendorDir . '/defuse/php-encryption/src'),
    'DebugBar\\' => array($vendorDir . '/php-debugbar/php-debugbar/src/DebugBar'),
    'Cron\\' => array($vendorDir . '/dragonmantank/cron-expression/src/Cron'),
    'Cose\\' => array($vendorDir . '/web-auth/cose-lib/src'),
    'Composer\\CaBundle\\' => array($vendorDir . '/composer/ca-bundle/src'),
    'CBOR\\' => array($vendorDir . '/spomky-labs/cbor-php/src'),
    'Brick\\Math\\' => array($vendorDir . '/brick/math/src'),
    'Algo26\\IdnaConvert\\' => array($vendorDir . '/algo26-matthias/idna-convert/src'),
);
