tinymce.addI18n('pt-PT',{
"Cut": "Cortar",
"Heading 5": "T\u00edtulo 5",
"Header 2": "Cabe\u00e7alho 2",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "O seu navegador n\u00e3o suporta acesso direto \u00e0 \u00e1rea de transfer\u00eancia. Por favor use os atalhos Ctrl+X\/C\/V do seu teclado.",
"Heading 4": "T\u00edtulo 4",
"Div": "Div",
"Heading 2": "T\u00edtulo 2",
"Paste": "Colar",
"Close": "Fechar",
"Font Family": "Fonte",
"Pre": "Pre",
"Align right": "Alinhar \u00e0 direita",
"New document": "Novo documento",
"Blockquote": "Cita\u00e7\u00e3o em bloco",
"Numbered list": "Lista numerada",
"Heading 1": "T\u00edtulo 1",
"Headings": "T\u00edtulos",
"Increase indent": "Aumentar avan\u00e7o",
"Formats": "Formatos",
"Headers": "Cabe\u00e7alhos",
"Select all": "Selecionar tudo",
"Header 3": "Cabe\u00e7alho 3",
"Blocks": "Blocos",
"Undo": "Desfazer",
"Strikethrough": "Rasurado",
"Bullet list": "Lista com marcadores",
"Header 1": "Cabe\u00e7alho 1",
"Superscript": "Superior \u00e0 linha",
"Clear formatting": "Limpar formata\u00e7\u00e3o",
"Font Sizes": "Tamanhos",
"Subscript": "Inferior \u00e0 linha",
"Header 6": "Cabe\u00e7alho 6",
"Redo": "Refazer",
"Paragraph": "Par\u00e1grafo",
"Ok": "Ok",
"Bold": "Negrito",
"Code": "C\u00f3digo",
"Italic": "It\u00e1lico",
"Align center": "Alinhar ao centro",
"Header 5": "Cabe\u00e7alho 5",
"Heading 6": "T\u00edtulo 6",
"Heading 3": "T\u00edtulo 3",
"Decrease indent": "Diminuir avan\u00e7o",
"Header 4": "Cabe\u00e7alho 4",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "O comando colar est\u00e1 em modo de texto simples. O conte\u00fado ser\u00e1 colado como texto simples at\u00e9 desativar esta op\u00e7\u00e3o.",
"Underline": "Sublinhado",
"Cancel": "Cancelar",
"Justify": "Justificado",
"Inline": "Na linha",
"Copy": "Copiar",
"Align left": "Alinhar \u00e0 esquerda",
"Visual aids": "Ajuda visual",
"Lower Greek": "\\u03b1. \\u03b2. \\u03b3. ...",
"Square": "Quadrado",
"Default": "Padr\u00e3o",
"Lower Alpha": "a. b. c. ...",
"Circle": "C\u00edrculo",
"Disc": "Disco",
"Upper Alpha": "A. B. C. ...",
"Upper Roman": "I. II. III. ...",
"Lower Roman": "i. ii. iii. ...",
"Id should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "O ID deve come\u00e7ar com uma letra, seguido por letras, n\u00fameros, tra\u00e7os, pontos, dois pontos ou sobtra\u00e7os.",
"Name": "Nome",
"Anchor": "\u00c2ncora",
"Id": "ID",
"You have unsaved changes are you sure you want to navigate away?": "Existem altera\u00e7\u00f5es que ainda n\u00e3o foram guardadas. Tem a certeza que pretende sair?",
"Restore last draft": "Restaurar o \u00faltimo rascunho",
"Special character": "Car\u00e1cter especial",
"Source code": "C\u00f3digo fonte",
"Language": "Idioma",
"Insert\/Edit code sample": "Inserir\/editar amostra de c\u00f3digo",
"B": "B",
"R": "R",
"G": "G",
"Color": "Cor",
"Right to left": "Da direita para a esquerda",
"Left to right": "Da esquerda para a direita",
"Emoticons": "Emo\u00e7\u00f5es",
"Robots": "Rob\u00f4s",
"Document properties": "Propriedades do documento",
"Title": "T\u00edtulo",
"Keywords": "Palavras-chave",
"Encoding": "Codifica\u00e7\u00e3o",
"Description": "Descri\u00e7\u00e3o",
"Author": "Autor",
"Fullscreen": "Ecr\u00e3 completo",
"Horizontal line": "Linha horizontal",
"Horizontal space": "Espa\u00e7amento horizontal",
"Insert\/edit image": "Inserir\/editar imagem",
"General": "Geral",
"Advanced": "Avan\u00e7ado",
"Source": "Localiza\u00e7\u00e3o",
"Border": "Contorno",
"Constrain proportions": "Manter propor\u00e7\u00f5es",
"Vertical space": "Espa\u00e7amento vertical",
"Image description": "Descri\u00e7\u00e3o da imagem",
"Style": "Estilo",
"Dimensions": "Dimens\u00f5es",
"Insert image": "Inserir imagem",
"Image": "Imagem",
"Zoom in": "Mais zoom",
"Contrast": "Contraste",
"Back": "Voltar",
"Gamma": "Gama",
"Flip horizontally": "Inverter horizontalmente",
"Resize": "Redimensionar",
"Sharpen": "Mais nitidez",
"Zoom out": "Menos zoom",
"Image options": "Op\u00e7\u00f5es de imagem",
"Apply": "Aplicar",
"Brightness": "Brilho",
"Rotate clockwise": "Rota\u00e7\u00e3o hor\u00e1ria",
"Rotate counterclockwise": "Rota\u00e7\u00e3o anti-hor\u00e1ria",
"Edit image": "Editar imagem",
"Color levels": "N\u00edveis de cor",
"Crop": "Recortar",
"Orientation": "Orienta\u00e7\u00e3o",
"Flip vertically": "Inverter verticalmente",
"Invert": "Inverter",
"Date\/time": "Data\/hora",
"Insert date\/time": "Inserir data\/hora",
"Remove link": "Remover liga\u00e7\u00e3o",
"Url": "URL",
"Text to display": "Texto a exibir",
"Anchors": "\u00c2ncora",
"Insert link": "Inserir liga\u00e7\u00e3o",
"Link": "Liga\u00e7\u00e3o",
"New window": "Nova janela",
"None": "Nenhum",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "O URL que indicou parece ser um endere\u00e7o web. Quer adicionar o prefixo http:\/\/ tal como necess\u00e1rio?",
"Paste or type a link": "Copiar ou escrever uma hiperliga\u00e7\u00e3o",
"Target": "Alvo",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "O URL que indicou parece ser um endere\u00e7o de email. Quer adicionar o prefixo mailto: tal como necess\u00e1rio?",
"Insert\/edit link": "Inserir\/editar liga\u00e7\u00e3o",
"Insert\/edit video": "Inserir\/editar v\u00eddeo",
"Media": "Media",
"Alternative source": "Localiza\u00e7\u00e3o alternativa",
"Paste your embed code below:": "Colar c\u00f3digo para embeber:",
"Insert video": "Inserir v\u00eddeo",
"Poster": "Autor",
"Insert\/edit media": "Inserir\/editar media",
"Embed": "Embeber",
"Nonbreaking space": "Espa\u00e7o n\u00e3o quebr\u00e1vel",
"Page break": "Quebra de p\u00e1gina",
"Paste as text": "Colar como texto",
"Preview": "Pr\u00e9-visualizar",
"Print": "Imprimir",
"Save": "Guardar",
"Could not find the specified string.": "N\u00e3o foi poss\u00edvel localizar o termo especificado.",
"Replace": "Substituir",
"Next": "Pr\u00f3ximo",
"Whole words": "Palavras completas",
"Find and replace": "Pesquisar e substituir",
"Replace with": "Substituir por",
"Find": "Pesquisar",
"Replace all": "Substituir tudo",
"Match case": "Diferenciar mai\u00fasculas e min\u00fasculas",
"Prev": "Anterior",
"Spellcheck": "Corretor ortogr\u00e1fico",
"Finish": "Concluir",
"Ignore all": "Ignorar tudo",
"Ignore": "Ignorar",
"Add to Dictionary": "Adicionar ao dicion\u00e1rio",
"Insert row before": "Inserir linha antes",
"Rows": "Linhas",
"Height": "Altura",
"Paste row after": "Colar linha depois",
"Alignment": "Alinhamento",
"Border color": "Cor de contorno",
"Column group": "Agrupar coluna",
"Row": "Linha",
"Insert column before": "Inserir coluna antes",
"Split cell": "Dividir c\u00e9lula",
"Cell padding": "Espa\u00e7amento interno da c\u00e9lula",
"Cell spacing": "Espa\u00e7amento entre c\u00e9lulas",
"Row type": "Tipo de linha",
"Insert table": "Inserir tabela",
"Body": "Corpo",
"Caption": "Legenda",
"Footer": "Rodap\u00e9",
"Delete row": "Eliminar linha",
"Paste row before": "Colar linha antes",
"Scope": "Escopo",
"Delete table": "Eliminar tabela",
"H Align": "Alinhamento H",
"Top": "Superior",
"Header cell": "C\u00e9lula de cabe\u00e7alho",
"Column": "Coluna",
"Row group": "Agrupar linha",
"Cell": "C\u00e9lula",
"Middle": "Meio",
"Cell type": "Tipo de c\u00e9lula",
"Copy row": "Copiar linha",
"Row properties": "Propriedades da linha",
"Table properties": "Propriedades da tabela",
"Bottom": "Inferior",
"V Align": "Alinhamento V",
"Header": "Cabe\u00e7alho",
"Right": "Direita",
"Insert column after": "Inserir coluna depois",
"Cols": "Colunas",
"Insert row after": "Inserir linha depois",
"Width": "Largura",
"Cell properties": "Propriedades da c\u00e9lula",
"Left": "Esquerda",
"Cut row": "Cortar linha",
"Delete column": "Eliminar coluna",
"Center": "Centro",
"Merge cells": "Unir c\u00e9lulas",
"Insert template": "Inserir modelo",
"Templates": "Modelos",
"Background color": "Cor de fundo",
"Custom...": "Personalizada...",
"Custom color": "Cor personalizada",
"No color": "Sem cor",
"Text color": "Cor do texto",
"Table of Contents": "\u00cdndice",
"Show blocks": "Mostrar blocos",
"Show invisible characters": "Mostrar caracteres invis\u00edveis",
"Words: {0}": "Palavras: {0}",
"Insert": "Inserir",
"File": "Ficheiro",
"Edit": "Editar",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Caixa de texto formatado. Pressione ALT-F9 para exibir o menu. Pressione ALT-F10 para exibir a barra de ferramentas. Pressione ALT-0 para exibir a ajuda",
"Tools": "Ferramentas",
"View": "Ver",
"Table": "Tabela",
"Format": "Formatar"
});