<?php

/**
 * @package     Joomla.Administrator
 * @subpackage  com_fields
 *
 * @copyright   (C) 2016 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * FieldsHelper
 *
 * @since       3.7.0
 *
 * @deprecated  4.3 will be removed in 6.0
 *              Use \Joomla\Component\Fields\Administrator\Helper\FieldsHelper instead
 */
class FieldsHelper extends \Joomla\Component\Fields\Administrator\Helper\FieldsHelper
{
}
