@charset "UTF-8";
@import "../../../../../../vendor/joomla-custom-elements/css/joomla-alert.css";

#system-message-container:empty {
  margin-top: 0;
  display: none;
}

#system-message-container joomla-alert {
  width: 100%;
  min-width: 16rem;
  color: var(--gray-dark);
  border: 1px solid var(--alert-accent-color, transparent);
  background-color: #fff;
  border-radius: .25rem;
  margin-bottom: 0;
  padding: 0;
  transition: opacity .15s linear;
  display: flex;
  position: relative;
}

#system-message-container joomla-alert + * {
  margin-top: 1rem;
}

#system-message-container joomla-alert .alert-heading {
  color: var(--alert-heading-text);
  background: var(--alert-accent-color, transparent);
  flex-direction: column;
  place-content: center;
  padding: .8rem;
  display: flex;
}

#system-message-container joomla-alert .alert-heading .message:before, #system-message-container joomla-alert .alert-heading .success:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M1299 813l-422 422q-19 19-45 19t-45-19l-294-294q-19-19-19-45t19-45l102-102q19-19 45-19t45 19l147 147 275-275q19-19 45-19t45 19l102 102q19 19 19 45t-19 45zm141 83q0-148-73-273t-198-198-273-73-273 73-198 198-73 273 73 273 198 198 273 73 273-73 198-198 73-273zm224 0q0 209-103 385.5t-279.5 279.5-385.5 103-385.5-103-279.5-279.5-103-385.5 103-385.5 279.5-279.5 385.5-103 385.5 103 279.5 279.5 103 385.5z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .notice:before, #system-message-container joomla-alert .alert-heading .info:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119.043 8 8 119.083 8 256c0 136.997 111.043 248 248 248s248-111.003 248-248C504 119.083 392.957 8 256 8zm0 110c23.196 0 42 18.804 42 42s-18.804 42-42 42-42-18.804-42-42 18.804-42 42-42zm56 254c0 6.627-5.373 12-12 12h-88c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h12v-64h-12c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h64c6.627 0 12 5.373 12 12v100h12c6.627 0 12 5.373 12 12v24z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .warning:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M1024 1375v-190q0-14-9.5-23.5t-22.5-9.5h-192q-13 0-22.5 9.5t-9.5 23.5v190q0 14 9.5 23.5t22.5 9.5h192q13 0 22.5-9.5t9.5-23.5zm-2-374l18-459q0-12-10-19-13-11-24-11h-220q-11 0-24 11-10 7-10 21l17 457q0 10 10 16.5t24 6.5h185q14 0 23.5-6.5t10.5-16.5zm-14-934l768 1408q35 63-2 126-17 29-46.5 46t-63.5 17h-1536q-34 0-63.5-17t-46.5-46q-37-63-2-126l768-1408q17-31 47-49t65-18 65 18 47 49z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .error:before, #system-message-container joomla-alert .alert-heading .danger:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8zm0 448c-110.5 0-200-89.5-200-200S145.5 56 256 56s200 89.5 200 200-89.5 200-200 200zm101.8-262.2L295.6 256l62.2 62.2c4.7 4.7 4.7 12.3 0 17l-22.6 22.6c-4.7 4.7-12.3 4.7-17 0L256 295.6l-62.2 62.2c-4.7 4.7-12.3 4.7-17 0l-22.6-22.6c-4.7-4.7-4.7-12.3 0-17l62.2-62.2-62.2-62.2c-4.7-4.7-4.7-12.3 0-17l22.6-22.6c4.7-4.7 12.3-4.7 17 0l62.2 62.2 62.2-62.2c4.7-4.7 12.3-4.7 17 0l22.6 22.6c4.7 4.7 4.7 12.3 0 17z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-wrapper {
  width: 100%;
}

#system-message-container joomla-alert .alert-link {
  color: var(--success, inherit);
}

#system-message-container joomla-alert[type="success"], #system-message-container joomla-alert[type="message"] {
  --alert-accent-color: var(--success);
  --alert-heading-text: #fffffff2;
  --alert-close-button: var(--success);
  background-color: #fff;
}

#system-message-container joomla-alert[type="info"], #system-message-container joomla-alert[type="notice"] {
  --alert-accent-color: var(--info);
  --alert-heading-text: #fffffff2;
  --alert-close-button: var(--info);
  background-color: #fff;
}

#system-message-container joomla-alert[type="warning"] {
  --alert-accent-color: var(--warning);
  --alert-heading-text: #fffffff2;
  --alert-close-button: var(--warning);
  background-color: #fff;
}

#system-message-container joomla-alert[type="error"], #system-message-container joomla-alert[type="danger"] {
  --alert-accent-color: var(--danger);
  --alert-heading-text: #fffffff2;
  --alert-close-button: var(--danger);
  background-color: #fff;
}

#system-message-container joomla-alert .joomla-alert--close, #system-message-container joomla-alert .joomla-alert-button--close {
  color: var(--alert-close-button);
  opacity: 1;
  background: none;
  border: 0;
  padding: .2rem .8rem;
  font-size: 2rem;
  position: absolute;
  top: 0;
  right: 0;
}

#system-message-container joomla-alert .joomla-alert--close:hover, #system-message-container joomla-alert .joomla-alert--close:focus, #system-message-container joomla-alert .joomla-alert-button--close:hover, #system-message-container joomla-alert .joomla-alert-button--close:focus {
  cursor: pointer;
  opacity: .75;
  text-decoration: none;
}

[dir="rtl"] #system-message-container joomla-alert .joomla-alert--close, [dir="rtl"] #system-message-container joomla-alert .joomla-alert-button--close {
  padding: .2rem .6rem;
  left: 0;
  right: auto;
}

#system-message-container joomla-alert div {
  font-size: 1rem;
}

#system-message-container joomla-alert div .alert-message {
  margin: .5rem;
  padding: .3rem 2rem .3rem .3rem;
}

[dir="rtl"] #system-message-container joomla-alert div .alert-message {
  padding: .3rem .3rem .3rem 2rem;
}

#system-message-container joomla-alert div .alert-message:not(:first-of-type) {
  border-top: 1px solid var(--alert-accent-color);
}
