@charset "UTF-8";
#joomla-tinymce-builder .drop-area-highlight {
  background-color: #d0d0d0;
}

#joomla-tinymce-builder .tox-editor-container > .target {
  min-height: 2.5rem;
}

.tox-mbtn.gu-mirror {
  text-transform: normal;
  box-shadow: none;
  background: #dee0e2;
  border: 0;
  border-radius: 3px;
  outline: none;
  flex: none;
  justify-content: center;
  align-items: center;
  margin: 2px 0 3px;
  padding: 0;
  font-size: 14px;
  font-style: normal;
  font-weight: normal;
  display: flex;
  overflow: hidden;
}
