@charset "UTF-8";
.mce-content-body .mce-item-anchor {
  background: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20width%3D'8'%20height%3D'12'%20xmlns%3D'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg'%3E%3Cpath%20d%3D'M0%200L8%200%208%2012%204.09117821%209%200%2012z'%20fill%3D%22%23cccccc%22%2F%3E%3C%2Fsvg%3E%0A") center no-repeat;
}

.mce-content-body .mce-item-anchor:empty {
  cursor: default;
  -webkit-user-modify: read-only;
  -moz-user-modify: read-only;
  -webkit-user-select: all;
  -moz-user-select: all;
  user-select: all;
  padding: 0 2px;
  display: inline-block;
  width: 8px !important;
  height: 12px !important;
}

.mce-content-body .mce-item-anchor:not(:empty) {
  background-position-x: 2px;
  padding-left: 12px;
  display: inline-block;
}

.mce-content-body .mce-item-anchor[data-mce-selected] {
  outline-offset: 1px;
}

.tox-comments-visible .tox-comment[contenteditable="false"]:not([data-mce-selected]), .tox-comments-visible span.tox-comment img:not([data-mce-selected]), .tox-comments-visible span.tox-comment > audio:not([data-mce-selected]), .tox-comments-visible span.tox-comment > video:not([data-mce-selected]), .tox-comments-visible span.tox-comment span.mce-preview-object:not([data-mce-selected]) {
  outline: 3px solid #ffe89d;
}

.tox-comments-visible .tox-comment[contenteditable="false"][data-mce-annotation-active="true"]:not([data-mce-selected]), .tox-comments-visible span.tox-comment[data-mce-annotation-active="true"] img:not([data-mce-selected]), .tox-comments-visible span.tox-comment[data-mce-annotation-active="true"] > audio:not([data-mce-selected]), .tox-comments-visible span.tox-comment[data-mce-annotation-active="true"] > video:not([data-mce-selected]), .tox-comments-visible span.tox-comment[data-mce-annotation-active="true"] span.mce-preview-object:not([data-mce-selected]) {
  outline: 3px solid #fed635;
}

.tox-comments-visible span.tox-comment:not([data-mce-selected]) {
  background-color: #ffe89d;
  outline: none;
}

.tox-comments-visible span.tox-comment[data-mce-annotation-active="true"]:not([data-mce-selected="inline-boundary"]) {
  background-color: #fed635;
}

.tox-checklist > li:not(.tox-checklist--hidden) {
  margin: .25em 0;
  list-style: none;
}

.tox-checklist > li:not(.tox-checklist--hidden):before {
  content: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2216%22%20height%3D%2216%22%20viewBox%3D%220%200%2016%2016%22%3E%3Cg%20id%3D%22checklist-unchecked%22%20fill%3D%22none%22%20fill-rule%3D%22evenodd%22%3E%3Crect%20id%3D%22Rectangle%22%20width%3D%2215%22%20height%3D%2215%22%20x%3D%22.5%22%20y%3D%22.5%22%20fill-rule%3D%22nonzero%22%20stroke%3D%22%236d737b%22%20rx%3D%222%22%2F%3E%3C%2Fg%3E%3C%2Fsvg%3E%0A");
  cursor: pointer;
  width: 1em;
  height: 1em;
  margin-top: .125em;
  margin-left: -1.5em;
  position: absolute;
}

.tox-checklist li:not(.tox-checklist--hidden).tox-checklist--checked:before {
  content: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2216%22%20height%3D%2216%22%20viewBox%3D%220%200%2016%2016%22%3E%3Cg%20id%3D%22checklist-checked%22%20fill%3D%22none%22%20fill-rule%3D%22evenodd%22%3E%3Crect%20id%3D%22Rectangle%22%20width%3D%2216%22%20height%3D%2216%22%20fill%3D%22%234099FF%22%20fill-rule%3D%22nonzero%22%20rx%3D%222%22%2F%3E%3Cpath%20id%3D%22Path%22%20fill%3D%22%23FFF%22%20fill-rule%3D%22nonzero%22%20d%3D%22M11.5703186%2C3.14417309%20C11.8516238%2C2.73724603%2012.4164781%2C2.62829933%2012.83558%2C2.89774797%20C13.260121%2C3.17069355%2013.3759736%2C3.72932262%2013.0909105%2C4.14168582%20L7.7580587%2C11.8560195%20C7.43776896%2C12.3193404%206.76483983%2C12.3852142%206.35607322%2C11.9948725%20L3.02491697%2C8.8138662%20C2.66090143%2C8.46625845%202.65798871%2C7.89594698%203.01850234%2C7.54483354%20C3.373942%2C7.19866177%203.94940006%2C7.19592841%204.30829608%2C7.5386474%20L6.85276923%2C9.9684299%20L11.5703186%2C3.14417309%20Z%22%2F%3E%3C%2Fg%3E%3C%2Fsvg%3E%0A");
}

[dir="rtl"] .tox-checklist > li:not(.tox-checklist--hidden):before {
  margin-left: 0;
  margin-right: -1.5em;
}

code[class*="language-"], pre[class*="language-"] {
  color: #f8f8f2;
  text-shadow: 0 1px #0000004d;
  text-align: left;
  white-space: pre;
  word-spacing: normal;
  word-break: normal;
  word-wrap: normal;
  -moz-tab-size: 4;
  tab-size: 4;
  -webkit-hyphens: none;
  hyphens: none;
  background: none;
  font-family: Consolas, Monaco, Andale Mono, Ubuntu Mono, monospace;
  line-height: 1.5;
}

pre[class*="language-"] {
  border-radius: .3em;
  margin: .5em 0;
  padding: 1em;
  overflow: auto;
}

:not(pre) > code[class*="language-"], pre[class*="language-"] {
  background: #282a36;
}

:not(pre) > code[class*="language-"] {
  white-space: normal;
  border-radius: .3em;
  padding: .1em;
}

.token.comment, .token.prolog, .token.doctype, .token.cdata {
  color: #6272a4;
}

.token.punctuation {
  color: #f8f8f2;
}

.namespace {
  opacity: .7;
}

.token.property, .token.tag, .token.constant, .token.symbol, .token.deleted {
  color: #ff79c6;
}

.token.boolean, .token.number {
  color: #bd93f9;
}

.token.selector, .token.attr-name, .token.string, .token.char, .token.builtin, .token.inserted {
  color: #50fa7b;
}

.token.operator, .token.entity, .token.url, .language-css .token.string, .style .token.string, .token.variable {
  color: #f8f8f2;
}

.token.atrule, .token.attr-value, .token.function, .token.class-name {
  color: #f1fa8c;
}

.token.keyword {
  color: #8be9fd;
}

.token.regex, .token.important {
  color: #ffb86c;
}

.token.important, .token.bold {
  font-weight: bold;
}

.token.italic {
  font-style: italic;
}

.token.entity {
  cursor: help;
}

.mce-content-body {
  overflow-wrap: break-word;
  word-wrap: break-word;
}

.mce-content-body .mce-visual-caret {
  background-color: currentColor;
  position: absolute;
}

.mce-content-body .mce-visual-caret-hidden {
  display: none;
}

.mce-content-body [data-mce-caret] {
  margin: 0;
  padding: 0;
  position: absolute;
  top: 0;
  left: -1000px;
  right: auto;
}

.mce-content-body .mce-offscreen-selection {
  max-width: 1000000px;
  position: absolute;
  left: -2000000px;
}

.mce-content-body [contentEditable="false"] {
  cursor: default;
}

.mce-content-body [contentEditable="true"] {
  cursor: text;
}

.tox-cursor-format-painter {
  cursor: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%3E%0A%20%20%3Cg%20fill%3D%22none%22%20fill-rule%3D%22evenodd%22%3E%0A%20%20%20%20%3Cpath%20fill%3D%22%23000%22%20fill-rule%3D%22nonzero%22%20d%3D%22M15%2C6%20C15%2C5.45%2014.55%2C5%2014%2C5%20L6%2C5%20C5.45%2C5%205%2C5.45%205%2C6%20L5%2C10%20C5%2C10.55%205.45%2C11%206%2C11%20L14%2C11%20C14.55%2C11%2015%2C10.55%2015%2C10%20L15%2C9%20L16%2C9%20L16%2C12%20L9%2C12%20L9%2C19%20C9%2C19.55%209.45%2C20%2010%2C20%20L11%2C20%20C11.55%2C20%2012%2C19.55%2012%2C19%20L12%2C14%20L18%2C14%20L18%2C7%20L15%2C7%20L15%2C6%20Z%22%2F%3E%0A%20%20%20%20%3Cpath%20fill%3D%22%23000%22%20fill-rule%3D%22nonzero%22%20d%3D%22M1%2C1%20L8.25%2C1%20C8.66421356%2C1%209%2C1.33578644%209%2C1.75%20L9%2C1.75%20C9%2C2.16421356%208.66421356%2C2.5%208.25%2C2.5%20L2.5%2C2.5%20L2.5%2C8.25%20C2.5%2C8.66421356%202.16421356%2C9%201.75%2C9%20L1.75%2C9%20C1.33578644%2C9%201%2C8.66421356%201%2C8.25%20L1%2C1%20Z%22%2F%3E%0A%20%20%3C%2Fg%3E%0A%3C%2Fsvg%3E%0A"), default;
}

div.mce-footnotes hr {
  width: 25%;
  margin-inline: 0 auto;
}

div.mce-footnotes li > a.mce-footnotes-backlink {
  text-decoration: none;
}

@media print {
  sup.mce-footnote a {
    color: #000;
    text-decoration: none;
  }

  div.mce-footnotes {
    break-inside: avoid;
    width: 100%;
  }

  div.mce-footnotes li > a.mce-footnotes-backlink {
    display: none;
  }
}

.mce-content-body figure.align-left {
  float: left;
}

.mce-content-body figure.align-right {
  float: right;
}

.mce-content-body figure.image.align-center {
  margin-left: auto;
  margin-right: auto;
  display: table;
}

.mce-preview-object {
  border: 1px solid gray;
  margin: 0 2px;
  line-height: 0;
  display: inline-block;
  position: relative;
}

.mce-preview-object .mce-shim {
  background: url("data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7");
  width: 100%;
  height: 100%;
  position: absolute;
  top: 0;
  left: 0;
}

.mce-preview-object[data-mce-selected="2"] .mce-shim {
  display: none;
}

.mce-content-body .mce-mergetag {
  -webkit-user-select: none;
  -moz-user-select: none;
  user-select: none;
  cursor: default !important;
}

.mce-content-body .mce-mergetag:hover {
  background-color: #006ce74d;
}

.mce-content-body .mce-mergetag-affix {
  color: #006ce7;
  background-color: #006ce74d;
}

.mce-object {
  background: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%3E%3Cpath%20d%3D%22M4%203h16a1%201%200%200%201%201%201v16a1%201%200%200%201-1%201H4a1%201%200%200%201-1-1V4a1%201%200%200%201%201-1zm1%202v14h14V5H5zm4.79%202.565l5.64%204.028a.5.5%200%200%201%200%20.814l-5.64%204.028a.5.5%200%200%201-.79-.407V7.972a.5.5%200%200%201%20.79-.407z%22%20fill%3D%22%23cccccc%22%2F%3E%3C%2Fsvg%3E%0A") center no-repeat;
  border: 1px dashed #aaa;
}

.mce-pagebreak {
  cursor: default;
  page-break-before: always;
  border: 1px dashed #aaa;
  width: 100%;
  height: 5px;
  margin-top: 15px;
  display: block;
}

@media print {
  .mce-pagebreak {
    border: 0;
  }
}

.tiny-pageembed .mce-shim {
  background: url("data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7");
  width: 100%;
  height: 100%;
  position: absolute;
  top: 0;
  left: 0;
}

.tiny-pageembed[data-mce-selected="2"] .mce-shim {
  display: none;
}

.tiny-pageembed {
  display: inline-block;
  position: relative;
}

.tiny-pageembed--21by9, .tiny-pageembed--16by9, .tiny-pageembed--4by3, .tiny-pageembed--1by1 {
  width: 100%;
  padding: 0;
  display: block;
  position: relative;
  overflow: hidden;
}

.tiny-pageembed--21by9 {
  padding-top: 42.8571%;
}

.tiny-pageembed--16by9 {
  padding-top: 56.25%;
}

.tiny-pageembed--4by3 {
  padding-top: 75%;
}

.tiny-pageembed--1by1 {
  padding-top: 100%;
}

.tiny-pageembed--21by9 iframe, .tiny-pageembed--16by9 iframe, .tiny-pageembed--4by3 iframe, .tiny-pageembed--1by1 iframe {
  border: 0;
  width: 100%;
  height: 100%;
  position: absolute;
  top: 0;
  left: 0;
}

.mce-content-body[data-mce-placeholder] {
  position: relative;
}

.mce-content-body[data-mce-placeholder]:not(.mce-visualblocks):before {
  color: #222f3eb3;
  content: attr(data-mce-placeholder);
  position: absolute;
}

.mce-content-body:not([dir="rtl"])[data-mce-placeholder]:not(.mce-visualblocks):before {
  left: 1px;
}

.mce-content-body[dir="rtl"][data-mce-placeholder]:not(.mce-visualblocks):before {
  right: 1px;
}

.mce-content-body div.mce-resizehandle {
  box-sizing: border-box;
  z-index: 1298;
  background-color: #4099ff;
  border: 1px solid #4099ff;
  width: 10px;
  height: 10px;
  position: absolute;
}

.mce-content-body div.mce-resizehandle:hover {
  background-color: #4099ff;
}

.mce-content-body div.mce-resizehandle:first-of-type {
  cursor: nwse-resize;
}

.mce-content-body div.mce-resizehandle:nth-of-type(2) {
  cursor: nesw-resize;
}

.mce-content-body div.mce-resizehandle:nth-of-type(3) {
  cursor: nwse-resize;
}

.mce-content-body div.mce-resizehandle:nth-of-type(4) {
  cursor: nesw-resize;
}

.mce-content-body .mce-resize-backdrop {
  z-index: 10000;
}

.mce-content-body .mce-clonedresizable {
  cursor: default;
  opacity: .5;
  z-index: 10001;
  outline: 1px dashed #000;
  position: absolute;
}

.mce-content-body .mce-clonedresizable.mce-resizetable-columns th, .mce-content-body .mce-clonedresizable.mce-resizetable-columns td {
  border: 0;
}

.mce-content-body .mce-resize-helper {
  color: #fff;
  white-space: nowrap;
  z-index: 10002;
  background: #000000bf;
  border: 1px;
  border-radius: 3px;
  margin: 5px 10px;
  padding: 5px;
  font-family: sans-serif;
  font-size: 12px;
  line-height: 14px;
  display: none;
  position: absolute;
}

.tox-rtc-user-selection {
  position: relative;
}

.tox-rtc-user-cursor {
  cursor: default;
  width: 2px;
  position: absolute;
  top: 0;
  bottom: 0;
}

.tox-rtc-user-cursor:before {
  background-color: inherit;
  content: "";
  border-radius: 50%;
  width: 8px;
  height: 8px;
  display: block;
  position: absolute;
  top: -3px;
  right: -3px;
}

.tox-rtc-user-cursor:hover:after {
  background-color: inherit;
  box-sizing: border-box;
  color: #fff;
  content: attr(data-user);
  white-space: nowrap;
  z-index: 1000;
  border-radius: 100px;
  min-width: 8px;
  min-height: 8px;
  padding: 0 12px;
  font-size: 12px;
  font-weight: bold;
  display: block;
  position: absolute;
  top: -11px;
  left: -5px;
}

.tox-rtc-user-selection--1 .tox-rtc-user-cursor {
  background-color: #2dc26b;
}

.tox-rtc-user-selection--2 .tox-rtc-user-cursor {
  background-color: #e03e2d;
}

.tox-rtc-user-selection--3 .tox-rtc-user-cursor {
  background-color: #f1c40f;
}

.tox-rtc-user-selection--4 .tox-rtc-user-cursor {
  background-color: #3598db;
}

.tox-rtc-user-selection--5 .tox-rtc-user-cursor {
  background-color: #b96ad9;
}

.tox-rtc-user-selection--6 .tox-rtc-user-cursor {
  background-color: #e67e23;
}

.tox-rtc-user-selection--7 .tox-rtc-user-cursor {
  background-color: #aaa69d;
}

.tox-rtc-user-selection--8 .tox-rtc-user-cursor {
  background-color: #f368e0;
}

.tox-rtc-remote-image {
  background: #eaeaea url("data:image/svg+xml;charset=UTF-8,%3Csvg%20width%3D%2236%22%20height%3D%2212%22%20viewBox%3D%220%200%2036%2012%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%0A%20%20%3Ccircle%20cx%3D%226%22%20cy%3D%226%22%20r%3D%223%22%20fill%3D%22rgba(0%2C%200%2C%200%2C%20.2)%22%3E%0A%20%20%20%20%3Canimate%20attributeName%3D%22r%22%20values%3D%223%3B5%3B3%22%20calcMode%3D%22linear%22%20dur%3D%221s%22%20repeatCount%3D%22indefinite%22%20%2F%3E%0A%20%20%3C%2Fcircle%3E%0A%20%20%3Ccircle%20cx%3D%2218%22%20cy%3D%226%22%20r%3D%223%22%20fill%3D%22rgba(0%2C%200%2C%200%2C%20.2)%22%3E%0A%20%20%20%20%3Canimate%20attributeName%3D%22r%22%20values%3D%223%3B5%3B3%22%20calcMode%3D%22linear%22%20begin%3D%22.33s%22%20dur%3D%221s%22%20repeatCount%3D%22indefinite%22%20%2F%3E%0A%20%20%3C%2Fcircle%3E%0A%20%20%3Ccircle%20cx%3D%2230%22%20cy%3D%226%22%20r%3D%223%22%20fill%3D%22rgba(0%2C%200%2C%200%2C%20.2)%22%3E%0A%20%20%20%20%3Canimate%20attributeName%3D%22r%22%20values%3D%223%3B5%3B3%22%20calcMode%3D%22linear%22%20begin%3D%22.66s%22%20dur%3D%221s%22%20repeatCount%3D%22indefinite%22%20%2F%3E%0A%20%20%3C%2Fcircle%3E%0A%3C%2Fsvg%3E%0A") center no-repeat;
  border: 1px solid #ccc;
  min-width: 320px;
  min-height: 240px;
}

.mce-match-marker {
  color: #fff;
  background: #aaa;
}

.mce-match-marker-selected {
  color: #fff;
  background: #39f;
}

.mce-match-marker-selected::-moz-selection {
  color: #fff;
  background: #39f;
}

.mce-match-marker-selected::selection {
  color: #fff;
  background: #39f;
}

.mce-content-body img[data-mce-selected], .mce-content-body video[data-mce-selected], .mce-content-body audio[data-mce-selected], .mce-content-body object[data-mce-selected], .mce-content-body embed[data-mce-selected], .mce-content-body table[data-mce-selected], .mce-content-body details[data-mce-selected] {
  outline: 3px solid #4099ff;
}

.mce-content-body hr[data-mce-selected] {
  outline-offset: 1px;
  outline: 3px solid #4099ff;
}

.mce-content-body [contentEditable="false"] [contentEditable="true"]:focus, .mce-content-body [contentEditable="false"] [contentEditable="true"]:hover {
  outline: 3px solid #4099ff;
}

.mce-content-body [contentEditable="false"][data-mce-selected] {
  cursor: not-allowed;
  outline: 3px solid #4099ff;
}

.mce-content-body.mce-content-readonly [contentEditable="true"]:focus, .mce-content-body.mce-content-readonly [contentEditable="true"]:hover {
  outline: none;
}

.mce-content-body [data-mce-selected="inline-boundary"] {
  background-color: #4099ff;
}

.mce-content-body .mce-edit-focus {
  outline: 3px solid #4099ff;
}

.mce-content-body td[data-mce-selected], .mce-content-body th[data-mce-selected] {
  position: relative;
}

.mce-content-body td[data-mce-selected]::-moz-selection, .mce-content-body th[data-mce-selected]::-moz-selection {
  background: none;
}

.mce-content-body td[data-mce-selected]::selection, .mce-content-body th[data-mce-selected]::selection {
  background: none;
}

.mce-content-body td[data-mce-selected] *, .mce-content-body th[data-mce-selected] * {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  user-select: none;
  outline: none;
}

.mce-content-body td[data-mce-selected]:after, .mce-content-body th[data-mce-selected]:after {
  content: "";
  mix-blend-mode: lighten;
  background-color: #b4d7ffb3;
  border: 1px solid #0000;
  position: absolute;
  inset: -1px;
}

@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none) {
  .mce-content-body td[data-mce-selected]:after, .mce-content-body th[data-mce-selected]:after {
    border-color: #0054b4b3;
  }
}

.mce-content-body img[data-mce-selected]::-moz-selection {
  background: none;
}

.mce-content-body img[data-mce-selected]::selection {
  background: none;
}

.ephox-snooker-resizer-bar {
  opacity: 0;
  -webkit-user-select: none;
  -moz-user-select: none;
  user-select: none;
  background-color: #4099ff;
}

.ephox-snooker-resizer-cols {
  cursor: col-resize;
}

.ephox-snooker-resizer-rows {
  cursor: row-resize;
}

.ephox-snooker-resizer-bar.ephox-snooker-resizer-bar-dragging {
  opacity: 1;
}

.mce-spellchecker-word {
  cursor: default;
  background-image: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20width%3D'4'%20height%3D'4'%20xmlns%3D'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg'%3E%3Cpath%20stroke%3D'%23ff0000'%20fill%3D'none'%20stroke-linecap%3D'round'%20stroke-opacity%3D'.75'%20d%3D'M0%203L2%201%204%203'%2F%3E%3C%2Fsvg%3E%0A");
  background-position: 0 calc(100% + 1px);
  background-repeat: repeat-x;
  background-size: auto 6px;
  height: 2rem;
}

.mce-spellchecker-grammar {
  cursor: default;
  background-image: url("data:image/svg+xml;charset=UTF-8,%3Csvg%20width%3D'4'%20height%3D'4'%20xmlns%3D'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg'%3E%3Cpath%20stroke%3D'%2300A835'%20fill%3D'none'%20stroke-linecap%3D'round'%20d%3D'M0%203L2%201%204%203'%2F%3E%3C%2Fsvg%3E%0A");
  background-position: 0 calc(100% + 1px);
  background-repeat: repeat-x;
  background-size: auto 6px;
}

.mce-toc {
  border: 1px solid gray;
}

.mce-toc h2 {
  margin: 4px;
}

.mce-toc ul > li {
  list-style-type: none;
}

[data-mce-block] {
  display: block;
}

table[style*="border-width: 0px"], .mce-item-table:not([border]), .mce-item-table[border="0"], table[style*="border-width: 0px"] td, .mce-item-table:not([border]) td, .mce-item-table[border="0"] td, table[style*="border-width: 0px"] th, .mce-item-table:not([border]) th, .mce-item-table[border="0"] th, table[style*="border-width: 0px"] caption, .mce-item-table:not([border]) caption, .mce-item-table[border="0"] caption {
  border: 1px dashed #bbb;
}

.mce-visualblocks p, .mce-visualblocks h1, .mce-visualblocks h2, .mce-visualblocks h3, .mce-visualblocks h4, .mce-visualblocks h5, .mce-visualblocks h6, .mce-visualblocks div:not([data-mce-bogus]), .mce-visualblocks section, .mce-visualblocks article, .mce-visualblocks blockquote, .mce-visualblocks address, .mce-visualblocks pre, .mce-visualblocks figure, .mce-visualblocks figcaption, .mce-visualblocks hgroup, .mce-visualblocks aside, .mce-visualblocks ul, .mce-visualblocks ol, .mce-visualblocks dl {
  background-repeat: no-repeat;
  border: 1px dashed #bbb;
  margin-left: 3px;
  padding-top: 10px;
}

.mce-visualblocks p {
  background-image: url("data:image/gif;base64,R0lGODlhCQAJAJEAAAAAAP///7u7u////yH5BAEAAAMALAAAAAAJAAkAAAIQnG+CqCN/mlyvsRUpThG6AgA7");
}

.mce-visualblocks h1 {
  background-image: url("data:image/gif;base64,R0lGODlhDQAKAIABALu7u////yH5BAEAAAEALAAAAAANAAoAAAIXjI8GybGu1JuxHoAfRNRW3TWXyF2YiRUAOw==");
}

.mce-visualblocks h2 {
  background-image: url("data:image/gif;base64,R0lGODlhDgAKAIABALu7u////yH5BAEAAAEALAAAAAAOAAoAAAIajI8Hybbx4oOuqgTynJd6bGlWg3DkJzoaUAAAOw==");
}

.mce-visualblocks h3 {
  background-image: url("data:image/gif;base64,R0lGODlhDgAKAIABALu7u////yH5BAEAAAEALAAAAAAOAAoAAAIZjI8Hybbx4oOuqgTynJf2Ln2NOHpQpmhAAQA7");
}

.mce-visualblocks h4 {
  background-image: url("data:image/gif;base64,R0lGODlhDgAKAIABALu7u////yH5BAEAAAEALAAAAAAOAAoAAAIajI8HybbxInR0zqeAdhtJlXwV1oCll2HaWgAAOw==");
}

.mce-visualblocks h5 {
  background-image: url("data:image/gif;base64,R0lGODlhDgAKAIABALu7u////yH5BAEAAAEALAAAAAAOAAoAAAIajI8HybbxIoiuwjane4iq5GlW05GgIkIZUAAAOw==");
}

.mce-visualblocks h6 {
  background-image: url("data:image/gif;base64,R0lGODlhDgAKAIABALu7u////yH5BAEAAAEALAAAAAAOAAoAAAIajI8HybbxIoiuwjan04jep1iZ1XRlAo5bVgAAOw==");
}

.mce-visualblocks div:not([data-mce-bogus]) {
  background-image: url("data:image/gif;base64,R0lGODlhEgAKAIABALu7u////yH5BAEAAAEALAAAAAASAAoAAAIfjI9poI0cgDywrhuxfbrzDEbQM2Ei5aRjmoySW4pAAQA7");
}

.mce-visualblocks section {
  background-image: url("data:image/gif;base64,R0lGODlhKAAKAIABALu7u////yH5BAEAAAEALAAAAAAoAAoAAAI5jI+pywcNY3sBWHdNrplytD2ellDeSVbp+GmWqaDqDMepc8t17Y4vBsK5hDyJMcI6KkuYU+jpjLoKADs=");
}

.mce-visualblocks article {
  background-image: url("data:image/gif;base64,R0lGODlhKgAKAIABALu7u////yH5BAEAAAEALAAAAAAqAAoAAAI6jI+pywkNY3wG0GBvrsd2tXGYSGnfiF7ikpXemTpOiJScasYoDJJrjsG9gkCJ0ag6KhmaIe3pjDYBBQA7");
}

.mce-visualblocks blockquote {
  background-image: url("data:image/gif;base64,R0lGODlhPgAKAIABALu7u////yH5BAEAAAEALAAAAAA+AAoAAAJPjI+py+0Knpz0xQDyuUhvfoGgIX5iSKZYgq5uNL5q69asZ8s5rrf0yZmpNkJZzFesBTu8TOlDVAabUyatguVhWduud3EyiUk45xhTTgMBBQA7");
}

.mce-visualblocks address {
  background-image: url("data:image/gif;base64,R0lGODlhLQAKAIABALu7u////yH5BAEAAAEALAAAAAAtAAoAAAI/jI+pywwNozSP1gDyyZcjb3UaRpXkWaXmZW4OqKLhBmLs+K263DkJK7OJeifh7FicKD9A1/IpGdKkyFpNmCkAADs=");
}

.mce-visualblocks pre {
  background-image: url("data:image/gif;base64,R0lGODlhFQAKAIABALu7uwAAACH5BAEAAAEALAAAAAAVAAoAAAIjjI+ZoN0cgDwSmnpz1NCueYERhnibZVKLNnbOq8IvKpJtVQAAOw==");
}

.mce-visualblocks figure {
  background-image: url("data:image/gif;base64,R0lGODlhJAAKAIAAALu7u////yH5BAEAAAEALAAAAAAkAAoAAAI0jI+py+2fwAHUSFvD3RlvG4HIp4nX5JFSpnZUJ6LlrM52OE7uSWosBHScgkSZj7dDKnWAAgA7");
}

.mce-visualblocks figcaption {
  border: 1px dashed #bbb;
}

.mce-visualblocks hgroup {
  background-image: url("data:image/gif;base64,R0lGODlhJwAKAIABALu7uwAAACH5BAEAAAEALAAAAAAnAAoAAAI3jI+pywYNI3uB0gpsRtt5fFnfNZaVSYJil4Wo03Hv6Z62uOCgiXH1kZIIJ8NiIxRrAZNMZAtQAAA7");
}

.mce-visualblocks aside {
  background-image: url("data:image/gif;base64,R0lGODlhHgAKAIABAKqqqv///yH5BAEAAAEALAAAAAAeAAoAAAItjI+pG8APjZOTzgtqy7I3f1yehmQcFY4WKZbqByutmW4aHUd6vfcVbgudgpYCADs=");
}

.mce-visualblocks ul {
  background-image: url("data:image/gif;base64,R0lGODlhDQAKAIAAALu7u////yH5BAEAAAEALAAAAAANAAoAAAIXjI8GybGuYnqUVSjvw26DzzXiqIDlVwAAOw==");
}

.mce-visualblocks ol {
  background-image: url("data:image/gif;base64,R0lGODlhDQAKAIABALu7u////yH5BAEAAAEALAAAAAANAAoAAAIXjI8GybH6HHt0qourxC6CvzXieHyeWQAAOw==");
}

.mce-visualblocks dl {
  background-image: url("data:image/gif;base64,R0lGODlhDQAKAIABALu7u////yH5BAEAAAEALAAAAAANAAoAAAIXjI8GybEOnmOvUoWznTqeuEjNSCqeGRUAOw==");
}

.mce-visualblocks:not([dir="rtl"]) p, .mce-visualblocks:not([dir="rtl"]) h1, .mce-visualblocks:not([dir="rtl"]) h2, .mce-visualblocks:not([dir="rtl"]) h3, .mce-visualblocks:not([dir="rtl"]) h4, .mce-visualblocks:not([dir="rtl"]) h5, .mce-visualblocks:not([dir="rtl"]) h6, .mce-visualblocks:not([dir="rtl"]) div:not([data-mce-bogus]), .mce-visualblocks:not([dir="rtl"]) section, .mce-visualblocks:not([dir="rtl"]) article, .mce-visualblocks:not([dir="rtl"]) blockquote, .mce-visualblocks:not([dir="rtl"]) address, .mce-visualblocks:not([dir="rtl"]) pre, .mce-visualblocks:not([dir="rtl"]) figure, .mce-visualblocks:not([dir="rtl"]) figcaption, .mce-visualblocks:not([dir="rtl"]) hgroup, .mce-visualblocks:not([dir="rtl"]) aside, .mce-visualblocks:not([dir="rtl"]) ul, .mce-visualblocks:not([dir="rtl"]) ol, .mce-visualblocks:not([dir="rtl"]) dl {
  margin-left: 3px;
}

.mce-visualblocks[dir="rtl"] p, .mce-visualblocks[dir="rtl"] h1, .mce-visualblocks[dir="rtl"] h2, .mce-visualblocks[dir="rtl"] h3, .mce-visualblocks[dir="rtl"] h4, .mce-visualblocks[dir="rtl"] h5, .mce-visualblocks[dir="rtl"] h6, .mce-visualblocks[dir="rtl"] div:not([data-mce-bogus]), .mce-visualblocks[dir="rtl"] section, .mce-visualblocks[dir="rtl"] article, .mce-visualblocks[dir="rtl"] blockquote, .mce-visualblocks[dir="rtl"] address, .mce-visualblocks[dir="rtl"] pre, .mce-visualblocks[dir="rtl"] figure, .mce-visualblocks[dir="rtl"] figcaption, .mce-visualblocks[dir="rtl"] hgroup, .mce-visualblocks[dir="rtl"] aside, .mce-visualblocks[dir="rtl"] ul, .mce-visualblocks[dir="rtl"] ol, .mce-visualblocks[dir="rtl"] dl {
  background-position-x: right;
  margin-right: 3px;
}

.mce-nbsp, .mce-shy {
  background: #aaa;
}

.mce-shy:after {
  content: "-";
}

body {
  font-family: sans-serif;
}

table {
  border-collapse: collapse;
}
