@import "../../tools/tools";

// Search tools

.js-stools-container-bar {
  padding: 10px 20px;

  .btn-toolbar {
    justify-content: flex-end;

    > * {
      margin: 4px 0;
      margin-inline-end: 8px;
    }

    .js-stools-btn-clear {
      background-color: $cyan;
      border: 0;
    }
  }

  .ordering-select {
    display: flex;
  }
}

.js-stools-container-filters {
  display: none;
  padding: 0 20px;
  margin-bottom: 20px;

  &-visible {
    display: grid;
    grid-gap: 8px;
    grid-template-columns: repeat(auto-fill, minmax(220px, 1fr));
    padding: 10px;
    background-color: $white;
  }

  > * {
    margin: 4px 0;
    margin-inline-end: 8px;
  }
}

.js-stools-field-list {
  + .js-stools-field-list {
    margin-inline-start: 8px;
  }
}

.js-stools-field-selector {
  .form-select {
    width: auto;
  }
}
