@charset "UTF-8";
.com-content-categories__item {
  border-bottom: 1px solid #dfe3e7;
}

.com-content-categories__item-title-wrapper {
  flex-wrap: wrap;
  justify-content: space-between;
  padding: .5rem 0;
  display: flex;
}

.com-content-categories__children {
  border-top: 1px solid #dfe3e7;
  flex-basis: 100%;
  padding-inline-start: 1.5rem;
}

.com-content-categories__children .com-content-categories__item:last-child {
  border-bottom: none;
}
