@charset "UTF-8";
#finder-filter-window {
  width: 100%;
  margin: 10px 0;
  padding: 0;
  overflow: auto;
}

ul#finder-filter-select-dates {
  margin: 0;
  padding: 0;
  list-style: none;
}

ul#finder-filter-select-dates li.filter-date {
  background: none;
  width: 49%;
  margin: 0;
  padding: 5px 0;
  list-style: none;
}

ul#finder-filter-select-dates li.filter-date select.filter-date-operator {
  margin-inline-end: 10px;
}
