<?php

/**
 * Part of the Joomla Framework Application Package
 *
 * @copyright  (C) 2017 Open Source Matters, Inc. <https://www.joomla.org>
 * @license    GNU General Public License version 2 or later; see LICENSE
 */

namespace Joomla\Application\Exception;

/**
 * Exception thrown when the application can't write to the response body
 *
 * @since  2.0.0
 */
class UnableToWriteBody extends \DomainException
{
}
