@charset "UTF-8";
.mod-articlesnews-horizontal {
  flex-wrap: wrap;
  padding: 0;
  list-style: none;
  display: flex;
}

.mod-articlesnews-horizontal figure img {
  width: 100%;
}

.mod-articlesnews-horizontal li {
  flex-direction: column;
  flex: 300px;
  display: flex;
}

.mod-articlesnews-horizontal li:not(:last-child) {
  margin-inline-end: 2rem;
}

.mod-articlesnews-horizontal .readmore {
  margin-top: auto;
}

@supports (display: grid) {
  .mod-articlesnews-horizontal {
    grid-gap: 2rem;
    grid-template-columns: repeat(auto-fit, minmax(200px, 1fr));
    display: grid;
  }

  .mod-articlesnews-horizontal li:not(:last-child) {
    margin-inline-end: 0;
  }
}
