.field-media-wrapper {
  display: block;
  width: 100%;
  max-width: calc(50vw - 5rem);

  .field-media-preview {
    width: 100%;
    max-width: none;
  }

  @include media-breakpoint-down(lg) {
    min-width: 100%;
  }
}
