// Dragula

.gu-mirror {
  position: fixed !important;
  z-index: 9999 !important;
  margin: 0 !important;
  background-color: hsl(120, 73%, 75%);
  opacity: .8;

  &.table {
    display: table;

    td {
      display: table-cell;
    }

  }

}
