/**
 * @copyright  (C) 2006 Open Source Matters, Inc. <https://www.joomla.org>
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

.outline {
  width: 550px;
  padding: 2px;
  margin: 0 auto;
  background: #fff;
  border: 1px solid #ccc;
}

body {
  height: 100%;
  padding: 0;
  margin: 15px;
  font-family: Arial, Helvetica, Sans Serif;
  font-size: 11px;
  color: #333;
  background: #fff;
}

.frame {
  padding: 8px;
  margin-top: 13px;
  margin-bottom: 25px;
  background-color: #fefcf3;
  border: solid 1px #000;
}

h1 {
  font-size: 18px;
  color: #c33;
}

.table {
  margin-top: 13px;
  border-collapse: collapse;
}

td {
  padding: 3px 5px;
  font-size: 10px;
  border: solid 1px #bbb;
}

.type {
  padding: 3px;
  font-weight: bold;
  color: #fff;
  background-color: #c00;
}
