tinymce.addI18n('lv',{
"Cut": "Izgriezt",
"Header 2": "Otr\u0101 l\u012bme\u0146a virsraksts",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "J\u016bsu p\u0101rl\u016bkprogramma neatbalsta piek\u013cuvi starpliktuvei. L\u016bdzu izmantojiet Ctrl+X\/C\/V klaviat\u016bras sa\u012bsnes.",
"Div": "Div elements",
"Paste": "Iel\u012bm\u0113t",
"Close": "Aizv\u0113rt",
"Font Family": "Font Family",
"Pre": "Pre elements",
"Align right": "L\u012bdzin\u0101t pa labi",
"New document": "Jauns dokuments",
"Blockquote": "Cit\u0101ts",
"Numbered list": "Numur\u0113ts saraksts",
"Increase indent": "Palielin\u0101t atk\u0101pi",
"Formats": "Form\u0101ti",
"Headers": "Virsraksti",
"Select all": "Iez\u012bm\u0113t",
"Header 3": "Tre\u0161\u0101 l\u012bme\u0146a virsraksts",
"Blocks": "Bloka elements",
"Undo": "Atsaukt",
"Strikethrough": "P\u0101rsv\u012btrot",
"Bullet list": "Nenumuer\u0113ts saraksts",
"Header 1": "Pirm\u0101 l\u012bme\u0146a virsraksts",
"Superscript": "Aug\u0161raksts",
"Clear formatting": "No\u0146emt format\u0113jumu",
"Font Sizes": "Font Sizes",
"Subscript": "Apak\u0161raksts",
"Header 6": "Sest\u0101 l\u012bme\u0146a virsraksts",
"Redo": "Atcelt atsauk\u0161anu",
"Paragraph": "Paragr\u0101fs",
"Ok": "Labi",
"Bold": "Treknraksts",
"Code": "Koda elements",
"Italic": "Kurs\u012bvs",
"Align center": "Centr\u0113t",
"Header 5": "Piekt\u0101 l\u012bme\u0146a virsraksts",
"Decrease indent": "Samazin\u0101t atk\u0101pi",
"Header 4": "Ceturt\u0101 l\u012bme\u0146a virsraksts",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Iel\u012bm\u0113\u0161ana tagad ir vienk\u0101r\u0161teksta re\u017e\u012bm\u0101. Saturs tiks iel\u012bm\u0113ts k\u0101 vienk\u0101r\u0161teksts, l\u012bdz \u0161\u012b opcija tiks atsl\u0113gta.",
"Underline": "Pasv\u012btrot",
"Cancel": "Atcelt",
"Justify": "L\u012bdzin\u0101t abas malas",
"Inline": "Rindi\u0146as elements",
"Copy": "Kop\u0113t",
"Align left": "L\u012bdzin\u0101t pa kreisi",
"Visual aids": "Uzskates l\u012bdzek\u013ci",
"Lower Greek": "Grie\u0137u mazie burti",
"Square": "Kvadr\u0101ts",
"Default": "Noklus\u0113juma",
"Lower Alpha": "Lat\u012b\u0146u mazie burti",
"Circle": "Aplis",
"Disc": "Disks",
"Upper Alpha": "Lat\u012b\u0146u lielie burti",
"Upper Roman": "Romie\u0161u lielie burti",
"Lower Roman": "Romie\u0161u mazie burti",
"Name": "V\u0101rds",
"Anchor": "Enkurelements",
"You have unsaved changes are you sure you want to navigate away?": "Jums ir nesaglab\u0101tas izmai\u0146as, esat dro\u0161s, ka v\u0113laties doties prom",
"Restore last draft": "Atjaunot p\u0113d\u0113jo melnrakstu",
"Special character": "\u012apa\u0161ais simbols",
"Source code": "Pirmkods",
"Right to left": "No lab\u0101s uz kreiso",
"Left to right": "No kreis\u0101s uz labo",
"Emoticons": "Emocijas",
"Robots": "Programmas",
"Document properties": "Dokumenta uzst\u0101d\u012bjumi",
"Title": "Nosaukums",
"Keywords": "Atsl\u0113gv\u0101rdi",
"Encoding": "Kod\u0113jums",
"Description": "Apraksts",
"Author": "Autors",
"Fullscreen": "Pilnekr\u0101na re\u017e\u012bms",
"Horizontal line": "Horizont\u0101la l\u012bnija",
"Horizontal space": "Horizont\u0101l\u0101 vieta",
"Insert\/edit image": "Ievietot\/labot att\u0113lu",
"General": "Visp\u0101r\u012bgi",
"Advanced": "Papildus",
"Source": "Avots",
"Border": "Apmale",
"Constrain proportions": "Saglab\u0101t malu attiec\u012bbu",
"Vertical space": "Vertik\u0101l\u0101 vieta",
"Image description": "Att\u0113la apraksts",
"Style": "Stils",
"Dimensions": "Izm\u0113ri",
"Insert image": "Ievietot att\u0113lu",
"Insert date\/time": "Ievietot datumu\/laiku",
"Remove link": "No\u0146emt saiti",
"Url": "Adrese",
"Text to display": "Teksts",
"Anchors": "Enkurelements",
"Insert link": "Ievietot saiti",
"New window": "Jauns logs",
"None": "Nek\u0101",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?",
"Target": "M\u0113r\u0137is",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?",
"Insert\/edit link": "Ievietot\/labot saiti",
"Insert\/edit video": "Ievietot\/redi\u0123\u0113t video",
"Poster": "Att\u0113ls",
"Alternative source": "Alternat\u012bvs avots",
"Paste your embed code below:": "Iekop\u0113jiet embed kodu zem\u0101k:",
"Insert video": "Ievietot video",
"Embed": "Embed",
"Nonbreaking space": "L\u012bnij-nedalo\u0161s atstarpes simbols",
"Page break": "P\u0101rnest jaun\u0101 lap\u0101",
"Paste as text": "Iel\u012bm\u0113t k\u0101 tekstu",
"Preview": "Priek\u0161skat\u012bjums",
"Print": "Print\u0113t",
"Save": "Saglab\u0101t",
"Could not find the specified string.": "Mekl\u0113tais teksts netika atrasts",
"Replace": "Aizvietot",
"Next": "N\u0101ko\u0161ais",
"Whole words": "Pilnus v\u0101rdus",
"Find and replace": "Mekl\u0113t un aizvietot",
"Replace with": "Aizvietot ar",
"Find": "Mekl\u0113t",
"Replace all": "Aizvietot visu",
"Match case": "Re\u0123istrj\u016bt\u012bgs",
"Prev": "Iepriek\u0161\u0113jais",
"Spellcheck": "Pareizrakst\u012bbas p\u0101rbaude",
"Finish": "Beigt",
"Ignore all": "Ignor\u0113t visu",
"Ignore": "Ignor\u0113t",
"Insert row before": "Ievietot rindu pirms",
"Rows": "Rindas",
"Height": "Augstums",
"Paste row after": "Iel\u012bm\u0113t rindu p\u0113c",
"Alignment": "L\u012bdzin\u0101jums",
"Column group": "Kolonnu grupa",
"Row": "Rinda",
"Insert column before": "Ievietot kolonu pirms",
"Split cell": "Sadal\u012bt \u0161\u016bnas",
"Cell padding": "\u0160\u016bnas atstatumi",
"Cell spacing": "\u0160\u016bnu atstarpes",
"Row type": "Rindas tips",
"Insert table": "Ievietot tabulu",
"Body": "\u0136ermenis",
"Caption": "Virsraksts",
"Footer": "K\u0101jene",
"Delete row": "Dz\u0113st rindu",
"Paste row before": "Iel\u012bm\u0113t rindu pirms",
"Scope": "Apgabals",
"Delete table": "Dz\u0113st tabulu",
"Header cell": "Galvenes \u0161\u016bna",
"Column": "Kolona",
"Cell": "\u0160\u016bna",
"Header": "Galvene",
"Cell type": "\u0160\u016bnas tips",
"Copy row": "Kop\u0113t rindu",
"Row properties": "Rindas uzst\u0101d\u012bjumi",
"Table properties": "Tabulas uzst\u0101d\u012bjumi",
"Row group": "Rindu grupa",
"Right": "Pa labi",
"Insert column after": "Ievietot kolonu p\u0113c",
"Cols": "Kolonas",
"Insert row after": "Ievietot rindu p\u0113c",
"Width": "Platums",
"Cell properties": "\u0160\u016bnas uzst\u0101d\u012bjumi",
"Left": "Pa kreisi",
"Cut row": "Izgriezt rindu",
"Delete column": "Dz\u0113st kolonu",
"Center": "Centr\u0113t",
"Merge cells": "Apvienot \u0161\u016bnas",
"Insert template": "Ievietot \u0161ablonu",
"Templates": "\u0160abloni",
"Background color": "Fona kr\u0101sa",
"Text color": "Teksta kr\u0101sa",
"Show blocks": "R\u0101d\u012bt blokus",
"Show invisible characters": "R\u0101d\u012bt neredzam\u0101s rakstz\u012bmes",
"Words: {0}": "V\u0101rdi: {0}",
"Insert": "Ievietot",
"File": "Fails",
"Edit": "Labot",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Vizu\u0101li redi\u0123\u0113jama teksta apgabals. Nospiediet ALT-F9 izv\u0113lnei, ALT-F10 r\u012bkjoslai vai ALT-0 pal\u012bdz\u012bbai.",
"Tools": "R\u012bki",
"View": "Skat\u012bt",
"Table": "Tabula",
"Format": "Form\u0101ts",
"H Align": "Horizont\u0101lais l\u012bdzin\u0101jums",
"V Align": "Vertik\u0101lais l\u012bdzin\u0101jums",
"Top": "Aug\u0161\u0101",
"Middle": "Vid\u016b",
"Bottom": "Apak\u0161\u0101",
"Headings": "Virsraksti",
"Heading 1": "1. virsraksts",
"Heading 2": "2. virsraksts",
"Heading 3": "3. virsraksts",
"Heading 4": "4. virsraksts",
"Heading 5": "5. virsraksts",
"Heading 6": "6. virsraksts",
"Add to Dictionary": "Pievienot v\u0101rdn\u012bcai",
"Border color": "Apmales kr\u0101sa",
"Color": "Kr\u0101sa",
"Custom...": "Piel\u0101gots...",
"Custom color": "Nestandarta kr\u0101sa",
"No color": "Bez kr\u0101sas"
});
