@charset "UTF-8";
joomla-field-mediamore details {
  background: #f5f5f5;
  border: 1px solid #c9c9c9;
  border-radius: .25rem;
  margin: 0 0 0 .5rem;
  position: absolute;
  bottom: 0;
}

joomla-field-mediamore label.input-group-text {
  width: auto;
}

joomla-field-mediamore summary {
  background-color: #e3e3e3;
  border-radius: .25rem;
  padding: 1rem;
  font-weight: 500;
}

joomla-field-mediamore details .form-group {
  margin: 1rem !important;
}
