// Calendar

:root {
  --calendar-week-bg: #{$atum-calendar-week-bg}; // #f4f4f4; rgba(255,255,255,.1);
  --calendar-bg: #{$atum-calendar-bg};

  --calendar-buttons-color: #{$atum-calendar-buttons-color}; // #495057; var(--btn-primary-bg);

  --calendar-select-bg-color: #{$atum-calendar-select-bg}; //$form-select-bg
  --calendar-select-color: var(--body-color);

  --calendar-disabled-color: #{$atum-calendar-disabled-color}; // #999; #4b4b4b;
  --calendar-disabled-bg: #{$atum-calendar-disabled-bg}; // #fff; rgba(0,0,0,.5);

  --calendar-select-bg-url: #{$atum-calendar-select-bg-url} #{"/* rtl:"}$atum-calendar-select-bg-url-rtl#{"*/"};

}

.calendar-container {
  tr.time {
    &:hover, &:focus, &:active {
      background-color: transparent;
    }
  }
}
