.com_content.layout-edit {
  joomla-field-media .field-media-preview {
    height: 10.25rem;
  }

  joomla-tab {

    #attrib-basic[active],
    #editor[active] {
      display: flex;
      flex-wrap: wrap;

      .form-group {
        flex: 0 0 50%;
        max-width: 50%;
        box-shadow: none;

        &:nth-child(1),
        &:nth-child(19) {
          flex: 0 0 100%;
          max-width: 100%;
        }
      }

      .field-spacer {
        flex: 0 0 100%;
        max-width: 100%;
      }
    }
  }
}
