@charset "UTF-8";
body {
  color: #22262a;
  background-color: #fff;
  margin: 1rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
}

h1, h2, h3, h4, h5, h6 {
  margin-top: 0;
  margin-bottom: .5rem;
  font-weight: 700;
  line-height: 1.2;
}

h1 {
  font-size: calc(1.375rem + 1.5vw);
}

h2 {
  font-size: calc(1.325rem + .9vw);
}

h3 {
  font-size: calc(1.3rem + .6vw);
}

h4 {
  font-size: calc(1.275rem + .3vw);
}

h5 {
  font-size: 1.25rem;
}

h6 {
  font-size: 1rem;
}

a {
  text-decoration: none;
}

a:link {
  color: #224faa;
}

a:hover {
  color: #424077;
}

p {
  margin-top: 0;
  margin-bottom: 1rem;
}

hr#system-readmore {
  color: red;
  border: 1px dashed red;
}

span[lang] {
  border: 1px dashed #bbb;
  padding: 2px;
}

span[lang]:after {
  color: red;
  vertical-align: super;
  content: attr(lang);
  font-size: smaller;
}

.float-start {
  float: left;
}

.float-end {
  float: right;
}

.float-none {
  float: none;
}

.mx-auto {
  margin-left: auto !important;
  margin-right: auto !important;
}

.d-block {
  display: block !important;
}
