@charset "UTF-8";
.js-stools-container-bar {
  padding: 10px 20px;
}

.js-stools-container-bar .btn-toolbar {
  justify-content: flex-end;
}

.js-stools-container-bar .btn-toolbar > * {
  margin: 4px 0;
  margin-inline-end: 8px;
}

.js-stools-container-bar .btn-toolbar .js-stools-btn-clear {
  background-color: #30638d;
  border: 0;
}

.js-stools-container-bar .ordering-select {
  display: flex;
}

.js-stools-container-filters {
  margin-bottom: 20px;
  padding: 0 20px;
  display: none;
}

.js-stools-container-filters-visible {
  grid-gap: 8px;
  background-color: #fff;
  grid-template-columns: repeat(auto-fill, minmax(220px, 1fr));
  padding: 10px;
  display: grid;
}

.js-stools-container-filters > * {
  margin: 4px 0;
  margin-inline-end: 8px;
}

.js-stools-field-list + .js-stools-field-list {
  margin-inline-start: 8px;
}

.js-stools-field-selector .form-select {
  width: auto;
}
