@charset "UTF-8";
@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Regular.woff2?v=4596ac") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Regular.woff?v=1cb153") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Condensed-Regular;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Regular.woff2?v=4596ac") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Regular.woff?v=1cb153") format("woff");
}

@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-RegularItalic.woff2?v=1b05ec") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-RegularItalic.woff?v=e271a3") format("woff");
  font-weight: 400;
  font-style: italic;
}

@font-face {
  font-family: Roboto-Condensed-RegularItalic;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-RegularItalic.woff2?v=1b05ec") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-RegularItalic.woff?v=e271a3") format("woff");
}

@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Light.woff2?v=0f24c2") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Light.woff?v=8aaf66") format("woff");
  font-weight: 300;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Condensed-Light;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Light.woff2?v=0f24c2") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Light.woff?v=8aaf66") format("woff");
}

@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-LightItalic.woff2?v=5f05a4") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-LightItalic.woff?v=bdd8fa") format("woff");
  font-weight: 300;
  font-style: italic;
}

@font-face {
  font-family: Roboto-Condensed-LightItalic;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-LightItalic.woff2?v=5f05a4") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-LightItalic.woff?v=bdd8fa") format("woff");
}

@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Bold.woff2?v=412849") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Bold.woff?v=aef455") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Condensed-Bold;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-Bold.woff2?v=412849") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-Bold.woff?v=aef455") format("woff");
}

@font-face {
  font-family: Roboto-Condensed;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-BoldItalic.woff2?v=2f50bd") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-BoldItalic.woff?v=b520ee") format("woff");
  font-weight: 700;
  font-style: italic;
}

@font-face {
  font-family: Roboto-Condensed-BoldItalic;
  src: url("../../fonts/roboto-condensed/Roboto-Condensed-BoldItalic.woff2?v=2f50bd") format("woff2"), url("../../fonts/roboto-condensed/Roboto-Condensed-BoldItalic.woff?v=b520ee") format("woff");
}
