<?php

/**
 * Part of the Joomla Framework Database Package
 *
 * @copyright  Copyright (C) 2005 - 2021 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE
 */

namespace Joomla\Database\Sqlazure;

use Joomla\Database\Sqlsrv\SqlsrvQuery;

/**
 * SQL Azure Query Building Class.
 *
 * @since  1.0
 */
class SqlazureQuery extends SqlsrvQuery
{
}
