// Sidebar

.sidebar-nav {
  padding: 1rem 2rem;

  li {
    font-size: .9rem;
    font-weight: $font-weight-bold;

    &.divider {
      margin: .3rem 0;
    }

    a {
      display: block;
      padding: .25rem;
      font-weight: $font-weight-normal;
      color: $text-normal;
      text-decoration: none;

      &::before {
        margin-inline-end: .5rem;
        font-family: "Font Awesome 6 Free";
        font-weight: 900;
        content: "\f054" #{"/*!rtl:\"\f053\"*/"};
      }
    }

    &.item:hover, &.active {
      background-color: var(--sidebar-item-bg-hvr);

      a {
        color: var(--sidebar-item-color);
      }
    }
  }
}
