// Dragula

.gu-mirror {
  position: fixed !important;
  z-index: $zindex-popover !important;
  display: table;
  margin: 0 !important;
  color: var(--drag-color) !important;
  cursor: move;
  background-color: var(--drag-background-color);
  opacity: .8;

  td, th {
    color: var(--drag-color) !important;
    background-color: var(--drag-background-color);

    a {
      color: var(--drag-color) !important;
    }

    .tbody-icon [class^="icon-"] {
      color: var(--drag-color) !important;
      border-color: var(--drag-color) !important;
    }
  }
}

.js-draggable .sortable-handler {
  cursor: move;
}
