@charset "UTF-8";
joomla-field-simple-color {
  display: block;
}

joomla-field-simple-color .hidden {
  display: none;
}

joomla-field-simple-color .visually-hidden {
  clip: rect(0, 0, 0, 0);
  border: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  padding: 0;
  position: absolute;
  overflow: hidden;
}

joomla-field-simple-color button {
  vertical-align: middle;
  appearance: none;
  background: none;
  border: 1px solid #ccc;
  border-radius: 3px;
  width: 20px;
  height: 20px;
  overflow: hidden;
}

joomla-field-simple-color .btn-close {
  text-align: center;
  vertical-align: middle;
  width: auto;
  margin-top: -4px;
  padding: 0 6px;
  font-size: 1rem;
  line-height: 1.5;
}

joomla-field-simple-color .nocolor {
  background: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAIAAAACUFjqAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3ggRDQENU0dyawAAACZJREFUGNNjPHXqDAMSMDY2ROYyMeAFNJVm/Pv3LzL/7Nnzg8VpAKebCGpIIxHBAAAAAElFTkSuQmCC") !important;
}

joomla-field-simple-color .simplecolors-panel .swatch {
  margin: 0 4px 4px 0;
}

joomla-field-simple-color .swatch:hover, joomla-field-simple-color .swatch.active {
  border-color: #52a8eccc;
  box-shadow: inset 0 1px 1px #00000013, 0 0 8px #52a8ec99;
}

joomla-field-simple-color .simplecolors-panel[data-open=""] {
  display: block;
}

joomla-field-simple-color .simplecolors-panel {
  z-index: 12;
  float: left;
  background-color: #fff;
  background-clip: padding-box;
  border: 1px solid #ddd;
  border-radius: 5px;
  margin: 1px 0 0;
  padding: 6px 2px 2px 6px;
  display: none;
  position: absolute;
  box-shadow: 0 5px 10px #0003;
}
