<?php

/**
 * @package    Joomla.Site
 * @subpackage com_users
 *
 * @copyright  (C) 2022 Open Source Matters, Inc. <https://www.joomla.org>
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\Component\Users\Site\Model;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Multi-factor Authentication Method management model
 *
 * @since 4.2.0
 */
class MethodModel extends \Joomla\Component\Users\Administrator\Model\MethodModel
{
}
