// Bootstrap functions
@import "../../../../vendor/bootstrap/scss/functions";
@import "tools/variables/variables";
@import "../../../../vendor/bootstrap/scss/variables";
@import "../../../../vendor/bootstrap/scss/variables-dark";
@import "../../../../vendor/bootstrap/scss/maps";
@import "../../../../vendor/bootstrap/scss/mixins";
@import "../../../../vendor/bootstrap/scss/utilities";

// Variables, Functions and Mixins
@import "tools/tools";

// Bootstrap
@import "../../../../vendor/bootstrap/scss/bootstrap";

// jQuery Minicolors
@import "../../../../system/scss/jquery-minicolors";

// Blocks
@import "blocks/global"; // Leave this first
@import "blocks/alerts";
@import "blocks/back-to-top";
@import "blocks/banner";
@import "blocks/footer";
@import "blocks/form";
@import "blocks/frontend-edit";
@import "blocks/header";
@import "blocks/icons";
@import "blocks/iframe";
@import "blocks/layout";
@import "blocks/modals";
@import "blocks/modifiers";
@import "blocks/utilities";
@import "blocks/toolbar";
@import "blocks/legacy";
@import "blocks/tags";
@import "blocks/css-grid"; // Last to allow fallback

// Vendor overrides
@import "vendor/awesomplete";
@import "vendor/bootstrap/buttons";
@import "vendor/bootstrap/custom-forms";
@import "vendor/bootstrap/collapse";
@import "vendor/bootstrap/dropdown";
@import "vendor/bootstrap/forms";
@import "vendor/bootstrap/lists";
@import "vendor/bootstrap/modal";
@import "vendor/bootstrap/nav";
@import "vendor/bootstrap/pagination";
@import "vendor/bootstrap/table";
@import "vendor/chosen";
@import "vendor/dragula";
@import "vendor/metismenu/metismenu";
@import "vendor/minicolors";
@import "vendor/tinymce";

// Finally add the cassiopeia colours
:root {
  @each $color, $value in $cassiopeia-colors {
    --#{$color}: #{$value};
  }
}
