@charset "UTF-8";
.content_rating ul {
  margin-bottom: .5em;
  padding-inline-start: 0;
  list-style: none;
}

.content_rating .vote-star, .content_rating .vote-star-empty, .content_rating .vote-star-half {
  display: inline-block;
}

.content_rating .vote-star svg, .content_rating .vote-star-half svg {
  fill: #fd7e14;
  width: 1em;
  height: 1em;
}

.content_rating .vote-star-empty svg {
  fill: #d3d3d3;
  width: 1em;
  height: 1em;
}

.content_rating .vote-star-half {
  margin-inline-start: -1em;
}

[dir="rtl"] .content_rating .vote-star-half {
  transform: scaleX(-1);
}
