CREATE TABLE [#__session] (
  [session_id] VARCHAR(128) NOT NULL,
  [time] INTEGER NOT NULL,
  [data] VARBINARY(MAX) NOT NULL,
  CONSTRAINT [PK_#__session_session_id] PRIMARY KEY CLUSTERED
  (
    [session_id] ASC
  ) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY];
