<?php

/**
 * Joomla! Content Management System
 *
 * @copyright  (C) 2024 Open Source Matters, Inc. <https://www.joomla.org>
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\CMS\TUF;

use Tuf\Exception\TufException;

/**
 * @since  5.1.1
 */
class HttpLoaderException extends TufException
{
}
