<?php

/**
 * @package     Joomla.Administrator
 * @subpackage  com_finder
 *
 * @copyright   (C) 2011 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

\defined('_JEXEC') or die;

/**
 * Finder Indexer Adapter class.
 *
 * @since       2.5
 *
 * @deprecated  4.3 will be removed in 6.0
 *              Use \Joomla\Component\Finder\Administrator\Indexer\Adapter instead
 */
