@charset "UTF-8";
@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Thin.woff2?v=ad538a") format("woff2"), url("../../fonts/roboto/Roboto-Thin.woff?v=d3b473") format("woff");
  font-weight: 100;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Thin;
  src: url("../../fonts/roboto/Roboto-Thin.woff2?v=ad538a") format("woff2"), url("../../fonts/roboto/Roboto-Thin.woff?v=d3b473") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-ThinItalic.woff2?v=5b4a33") format("woff2"), url("../../fonts/roboto/Roboto-ThinItalic.woff?v=8a96ed") format("woff");
  font-weight: 100;
  font-style: italic;
}

@font-face {
  font-family: Roboto-ThinItalic;
  src: url("../../fonts/roboto/Roboto-ThinItalic.woff2?v=5b4a33") format("woff2"), url("../../fonts/roboto/Roboto-ThinItalic.woff?v=8a96ed") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Light.woff2?v=d26871") format("woff2"), url("../../fonts/roboto/Roboto-Light.woff?v=c73eb1") format("woff");
  font-weight: 300;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Light;
  src: url("../../fonts/roboto/Roboto-Light.woff2?v=d26871") format("woff2"), url("../../fonts/roboto/Roboto-Light.woff?v=c73eb1") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-LightItalic.woff2?v=e8eaae") format("woff2"), url("../../fonts/roboto/Roboto-LightItalic.woff?v=13efe6") format("woff");
  font-weight: 300;
  font-style: italic;
}

@font-face {
  font-family: Roboto-LightItalic;
  src: url("../../fonts/roboto/Roboto-LightItalic.woff2?v=e8eaae") format("woff2"), url("../../fonts/roboto/Roboto-LightItalic.woff?v=13efe6") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Regular.woff2?v=73f0a8") format("woff2"), url("../../fonts/roboto/Roboto-Regular.woff?v=35b07e") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Regular;
  src: url("../../fonts/roboto/Roboto-Regular.woff2?v=73f0a8") format("woff2"), url("../../fonts/roboto/Roboto-Regular.woff?v=35b07e") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-RegularItalic.woff2?v=4357be") format("woff2"), url("../../fonts/roboto/Roboto-RegularItalic.woff?v=f5902d") format("woff");
  font-weight: 400;
  font-style: italic;
}

@font-face {
  font-family: Roboto-RegularItalic;
  src: url("../../fonts/roboto/Roboto-RegularItalic.woff2?v=4357be") format("woff2"), url("../../fonts/roboto/Roboto-RegularItalic.woff?v=f5902d") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Medium.woff2?v=90d167") format("woff2"), url("../../fonts/roboto/Roboto-Medium.woff?v=1d6594") format("woff");
  font-weight: 500;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Medium;
  src: url("../../fonts/roboto/Roboto-Medium.woff2?v=90d167") format("woff2"), url("../../fonts/roboto/Roboto-Medium.woff?v=1d6594") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-MediumItalic.woff2?v=13ec0e") format("woff2"), url("../../fonts/roboto/Roboto-MediumItalic.woff?v=83e114") format("woff");
  font-weight: 500;
  font-style: italic;
}

@font-face {
  font-family: Roboto-MediumItalic;
  src: url("../../fonts/roboto/Roboto-MediumItalic.woff2?v=13ec0e") format("woff2"), url("../../fonts/roboto/Roboto-MediumItalic.woff?v=83e114") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Bold.woff2?v=b52fac") format("woff2"), url("../../fonts/roboto/Roboto-Bold.woff?v=50d75e") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Bold;
  src: url("../../fonts/roboto/Roboto-Bold.woff2?v=b52fac") format("woff2"), url("../../fonts/roboto/Roboto-Bold.woff?v=50d75e") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-BoldItalic.woff2?v=94008e") format("woff2"), url("../../fonts/roboto/Roboto-BoldItalic.woff?v=4fe0f7") format("woff");
  font-weight: 700;
  font-style: italic;
}

@font-face {
  font-family: Roboto-BoldItalic;
  src: url("../../fonts/roboto/Roboto-BoldItalic.woff2?v=94008e") format("woff2"), url("../../fonts/roboto/Roboto-BoldItalic.woff?v=4fe0f7") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-Black.woff2?v=59eb36") format("woff2"), url("../../fonts/roboto/Roboto-Black.woff?v=313a65") format("woff");
  font-weight: 900;
  font-style: normal;
}

@font-face {
  font-family: Roboto-Black;
  src: url("../../fonts/roboto/Roboto-Black.woff2?v=59eb36") format("woff2"), url("../../fonts/roboto/Roboto-Black.woff?v=313a65") format("woff");
}

@font-face {
  font-family: Roboto;
  src: url("../../fonts/roboto/Roboto-BlackItalic.woff2?v=f75569") format("woff2"), url("../../fonts/roboto/Roboto-BlackItalic.woff?v=cc2fad") format("woff");
  font-weight: 900;
  font-style: italic;
}

@font-face {
  font-family: Roboto-BlackItalic;
  src: url("../../fonts/roboto/Roboto-BlackItalic.woff2?v=f75569") format("woff2"), url("../../fonts/roboto/Roboto-BlackItalic.woff?v=cc2fad") format("woff");
}
