@charset "UTF-8";
#system-message {
  margin-bottom: 10px;
  padding: 0;
}

#system-message > dt {
  font-weight: bold;
  display: none;
}

#system-message > dd {
  text-indent: 30px;
  margin: 0;
  font-weight: bold;
}

#system-message > dd > ul {
  color: #05b;
  background-position: 4px 0;
  background-repeat: no-repeat;
  border-top: 3px solid #84a7db;
  border-bottom: 3px solid #84a7db;
  margin-bottom: 10px;
  padding: 10px;
  list-style: none;
}

#system-message > dd > ul > li {
  line-height: 1.5em;
}

#system-message > .message > ul {
  background-color: #c3d2e5;
}

#system-message > .error > ul, #system-message > .warning > ul, #system-message > .notice > ul {
  color: #c00;
}

#system-message > .error > ul {
  background-color: #e6c0c0;
  border-color: #de7a7b;
}

#system-message > .warning > ul {
  background-color: #e6c8a6;
  border-color: #fb0;
}

#system-message > .notice > ul {
  background-color: #efe7b8;
  border-color: #f0dc7e;
}
