tinymce.addI18n('ms',{
"Cut": "Potong",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "Pelayar internet anda tidak menyokong akses terus kepada papan klip. Sila gunakan pintasan papan kekunci Ctrl+X\/C\/V.",
"Paste": "Tampal",
"Close": "Tutup",
"Align right": "Alih ke kanan",
"New document": "Dokumen baru",
"Numbered list": "Senarai bernombor",
"Increase indent": "Tingkatkan inden",
"Formats": "Format",
"Select all": "Pilih semua",
"Undo": "Kembali Ke asal",
"Strikethrough": "Batal",
"Bullet list": "Senarai bullet",
"Superscript": "Superskrip",
"Clear formatting": "Hilangkan format",
"Subscript": "Subskrip",
"Redo": "Buat semula",
"Ok": "Ok",
"Bold": "Tebalkan",
"Italic": "Tulisan senget",
"Align center": "Alih ke tengah",
"Decrease indent": "Kurangkan inden",
"Underline": "Garis bawah",
"Cancel": "Batal",
"Justify": "Sama ratakan",
"Copy": "Salin",
"Align left": "Alihkan ke kiri",
"Visual aids": "Visual bantuan",
"Lower Greek": "Greek Kecil",
"Square": "Petak",
"Default": "Asal",
"Lower Alpha": "Alpha Kecil",
"Circle": "Bulat",
"Disc": "Cakera",
"Upper Alpha": "Alpha Besar",
"Upper Roman": "Roman Besar",
"Lower Roman": "Roman Kecik",
"Name": "Nama",
"Anchor": "Pemberat",
"You have unsaved changes are you sure you want to navigate away?": "Anda mempunyai perubahan yang belum disimpan, anda pasti mahu keluar?",
"Restore last draft": "Kembalikan draf lepas",
"Special character": "Karekter unik",
"Source code": "Sumber Kod",
"Right to left": "Kanan Ke kiri",
"Left to right": "Kiri Ke kanan",
"Emoticons": "Emotikon",
"Robots": "Robot",
"Document properties": "Sifat-sifat dokumen",
"Title": "Tajuk",
"Keywords": "Kata kunci",
"Encoding": "Pengekodan",
"Description": "Penerangan",
"Author": "Pengarang",
"Fullscreen": "Skrin penuh",
"Horizontal line": "Garis mendatar",
"Horizontal space": "Ruang mendatar",
"Insert\/edit image": "Masukkan\/Ubah Gambar",
"General": "Am",
"Advanced": "Lanjutan",
"Source": "Sumber",
"Border": "Sempadan",
"Constrain proportions": "Bentuk kekangan",
"Vertical space": "Ruang menegak",
"Image description": "Penerangan gambar",
"Style": "Gaya",
"Dimensions": "Dimensi",
"Insert image": "Masukkan gambar",
"Insert date\/time": "Masukkan tarikh\/masa",
"Remove link": "Padam pautan",
"Url": "Url",
"Text to display": "Teks untuk dipaparkan",
"Insert link": "Masukkan pautan",
"New window": "Tetingkap baru",
"None": "Tiada",
"Target": "Target",
"Insert\/edit link": "Masukkan\/Ubah pautan",
"Insert\/edit video": "Masukkan\/Ubah video",
"Poster": "Poster",
"Alternative source": "Sumber alternatif",
"Paste your embed code below:": "Tampalkan kod embed anda di bawah:",
"Insert video": "Masukkan video",
"Embed": "Embed",
"Nonbreaking space": "Ruang tidak dipisahkan",
"Preview": "Pratonton",
"Print": "Cetak",
"Save": "Simpan",
"Could not find the specified string.": "Tidak dapat mencari untaian yang dinyatakan.",
"Replace": "Ganti",
"Next": "Seterusnya",
"Whole words": "Keseluruhan perkataan",
"Find and replace": "Cari dan gantikan",
"Replace with": "Gantikan dengan",
"Find": "Cari",
"Replace all": "Ganti semua",
"Match case": "Huruf sepadan",
"Prev": "Kembali",
"Spellcheck": "Semakan Ejaan",
"Finish": "Tamat",
"Ignore all": "Biarkan semua",
"Ignore": "Biarkan",
"Insert row before": "Masukkan baris sebelumnya",
"Rows": "Baris-Baris",
"Height": "Tinggi",
"Paste row after": "Tampal baris selepasnya",
"Alignment": "Penjajaran",
"Column group": "Kumpulan lajur",
"Row": "Baris",
"Insert column before": "Masukkan lajur sebelumnya",
"Split cell": "Asingkan sel",
"Cell padding": "Penebalan sel",
"Cell spacing": "Penjarakkan sel",
"Row type": "Jenis baris",
"Insert table": "Masukkan jadual",
"Body": "Badan",
"Caption": "Keterangan",
"Footer": "Pengaki",
"Delete row": "Padam Baris",
"Paste row before": "Tampal baris sebelumnya",
"Scope": "Skop",
"Delete table": "Padam jadual",
"Header cell": "Sel pengepala",
"Column": "Lajur",
"Cell": "Sel",
"Header": "Pengepala",
"Cell type": "Jenis sel",
"Copy row": "Salin baris",
"Row properties": "Sifat-sifat baris",
"Table properties": "Sifat-sifat jadual",
"Row group": "Kumpulan baris",
"Right": "Kanan",
"Insert column after": "Masukkan lajur selepasnya",
"Cols": "Cols",
"Insert row after": "Masukkan baris selepasnya",
"Width": "Lebar",
"Cell properties": "Sifat-sifat sel",
"Left": "Kiri",
"Cut row": "Potong baris",
"Delete column": "Padam lajur",
"Center": "Tengah",
"Merge cells": "Gabung Sel",
"Insert template": "Masukkan templat",
"Templates": "Templat",
"Background color": "Warna latar belakang",
"Text color": "Warna teks",
"Show blocks": "Papar blok",
"Show invisible characters": "Papar karekter tersembunyi",
"Words: {0}": "Perkataan: {0}",
"Insert": "Masukkan",
"File": "Fail",
"Edit": "Ubah",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Ruangan Teks Luas. Tekan ALT-F9 untuk menu. Tekan ALT-F10 untuk bar perkakasan. Tekan ALT-0 untuk bantuan",
"Tools": "Peralatan",
"View": "Pemandangan",
"Table": "Jadual",
"Format": "Format",
"Inline": "Sebaris",
"Blocks": "Blok-blok",
"Edit image": "Ubah gambar",
"Font Family": "Kumpulan fon",
"Font Sizes": "Saiz fon",
"Paragraph": "Perenggan",
"Address": "Alamat",
"Pre": "Pra",
"Code": "Kod",
"Headers": "Pengepala",
"Header 1": "Pengepala 1",
"Header 2": "Pengepala 2",
"Header 3": "Pengepala 3",
"Header 4": "Pengepala 4",
"Header 5": "Pengepala 5",
"Header 6": "Pengepala 6",
"Insert Time": "Masukkan masa",
"Insert nonbreaking space": "Masukkan ruang yang tidak dipisahkan",
"Toggle blockquote": "Togol Ruang Petikan"
});