.media-modal-backdrop {
  position: fixed;
  top: 0;
  left: 0;
  z-index: 1049;
  display: table;
  width: 100%;
  height: 100%;
  background-color: $modal-backdrop-color;

  .modal {
    display: flex;
    align-items: center;
    justify-content: center;
    animation: .5s ease 0s normal none 1 running fadeIn;
  }

  .modal-body {
    width: auto;
    padding: 15px;
  }

  .modal-content {
    box-shadow: $modal-box-shadow;
  }

}

.media-preview-modal {
  color: $modal-preview-text-color;

  .modal {
    display: grid !important;
    align-items: center !important;
    justify-content: center !important;
  }

  .modal-content {
    display: flex;
    flex-direction: column;
    align-items: flex-start;
    background-color: transparent;
    border: 0;
    box-shadow: none;
  }
  .modal-header {
    padding: 0;
    border: 0;
  }
  .modal-body {
    padding: 0;
    background-color: #fff;
    box-shadow: $modal-box-shadow;
    img {
      max-width: 100%;
    }
  }
  .modal-footer {
    display: none;
  }
}

.media-preview-close {
  position: absolute;
  top: -2rem;
  right: 0;
  font-size: 2rem;
  color: var(--light) !important;
  background: none;
  border: 0;
  opacity: .7;
  &:hover {
    cursor: pointer;
    opacity: 1;
  }
}

@media (min-width: 576px) {
  .media-preview-modal {
    .modal-dialog {
      max-width: unset !important;
    }
  }
}

@keyframes fadeInUp {
  from {
    opacity: 0;
    transition: transform .3s ease-out;
    transform: translate(0, -25%);
  }

  to {
    opacity: 1;
    transform: none;
  }
}
