// Collapse

.accordion {

  .card-header {
    display: block;
    font-size: $h5-font-size;
    font-weight: $font-weight-bold;
    line-height: $headings-line-height;
  }

  /**
   * TODO: This seems fairly specifically built for the menu types view and might be better scoped to
   *       that view rather than just being overridden for everything.
   */
  .list-group-item {
    --list-group-color: var(--link-color);
    --list-group-bg: var(--white-offset);
  }

  @if $enable-dark-mode {
    @include color-mode(dark) {
      .list-group-item {
        --list-group-bg: var(--gray-800);
      }
    }
  }
}
