// Override the font path
$fa-font-path: "../../../media/vendor/fontawesome-free/webfonts" !default;
$fa-font-display: block !default;

// Font Awesome 6 Free
@import "../../../../media/vendor/fontawesome-free/scss/fontawesome";
@import "../../../../media/vendor/fontawesome-free/scss/regular";
@import "../../../../media/vendor/fontawesome-free/scss/solid";

// Brands must be imported last
@import "../../../../media/vendor/fontawesome-free/scss/brands";

// B/C for Icomoon
@import "icomoon";
