joomla-field-media .field-media-preview {
  display: flex;
  align-items: center;
  justify-content: center;
  max-width: 356px;
  height: 180px;
  padding: 10px;
  overflow: hidden;
  background-color: #f2f2f2;
  border: 1px solid rgba(0, 0, 0, .15);
  border-width: 1px 1px 0;
  border-radius: .25rem .25rem 0 0;
}

joomla-field-media .field-media-preview-icon {
  width: 7rem;
  height: 7rem;
  background-image: url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 512 512'%3E%3Cpath fill='rgba(0,0,0,.25)' d='M464 64H48C21.49 64 0 85.49 0 112v288c0 26.51 21.49 48 48 48h416c26.51 0 48-21.49 48-48V112c0-26.51-21.49-48-48-48zm-6 336H54a6 6 0 0 1-6-6V118a6 6 0 0 1 6-6h404a6 6 0 0 1 6 6v276a6 6 0 0 1-6 6zM128 152c-22.091 0-40 17.909-40 40s17.909 40 40 40 40-17.909 40-40-17.909-40-40-40zM96 352h320v-80l-87.515-87.515c-4.686-4.686-12.284-4.686-16.971 0L192 304l-39.515-39.515c-4.686-4.686-12.284-4.686-16.971 0L96 304v48z'/%3E%3C/svg%3E");
  background-size: 7rem;
}

joomla-field-media .field-media-input {
  border-top-left-radius: 0;
  border-start-start-radius: 0;
}

joomla-field-media .input-group > .btn.btn-success.button-select {
  border-top-right-radius: 0;
  border-start-end-radius: 0;
}

joomla-field-media .button-clear {
  border-top-right-radius: 0;
  border-start-end-radius: 0;
}

joomla-field-media img {
  max-width: 100%;
  max-height: 100%;
}
