@charset "UTF-8";
/*!
Chosen, a Select Box Enhancer for jQuery and Prototype
by Patrick Filler for Harvest, http://getharvest.com

Version 1.8.7
Full source at https://github.com/harvesthq/chosen
Copyright (c) 2011-2018 Harvest http://getharvest.com

MIT License, https://github.com/harvesthq/chosen/blob/master/LICENSE.md
This file is generated by `grunt build`, do not edit it by hand.
*/
.chosen-container {
  vertical-align: middle;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  font-size: 13px;
  display: inline-block;
  position: relative;
}

.chosen-container * {
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}

.chosen-container .chosen-drop {
  z-index: 1010;
  clip: rect(0, 0, 0, 0);
  -webkit-clip-path: inset(100%);
  clip-path: inset(100%);
  background: #fff;
  border: 1px solid #aaa;
  border-top: 0;
  width: 100%;
  position: absolute;
  top: 100%;
  -webkit-box-shadow: 0 4px 5px #00000026;
  box-shadow: 0 4px 5px #00000026;
}

.chosen-container.chosen-with-drop .chosen-drop {
  clip: auto;
  -webkit-clip-path: none;
  clip-path: none;
}

.chosen-container a {
  cursor: pointer;
}

.chosen-container .search-choice .group-name, .chosen-container .chosen-single .group-name {
  white-space: nowrap;
  text-overflow: ellipsis;
  color: #999;
  margin-right: 4px;
  font-weight: normal;
  overflow: hidden;
}

.chosen-container .search-choice .group-name:after, .chosen-container .chosen-single .group-name:after {
  content: ":";
  vertical-align: top;
  padding-left: 2px;
}

.chosen-container-single .chosen-single {
  color: #444;
  white-space: nowrap;
  background: -webkit-gradient(linear, left top, left bottom, color-stop(.2, #fff), color-stop(.5, #f6f6f6), color-stop(.52, #eee), to(#f4f4f4));
  background: linear-gradient(#fff 20%, #f6f6f6 50%, #eee 52%, #f4f4f4 100%) padding-box padding-box;
  border: 1px solid #aaa;
  border-radius: 5px;
  height: 25px;
  padding: 0 0 0 8px;
  line-height: 24px;
  text-decoration: none;
  display: block;
  position: relative;
  overflow: hidden;
  -webkit-box-shadow: inset 0 0 3px #fff, 0 1px 1px #0000001a;
  box-shadow: inset 0 0 3px #fff, 0 1px 1px #0000001a;
}

.chosen-container-single .chosen-default {
  color: #999;
}

.chosen-container-single .chosen-single span {
  text-overflow: ellipsis;
  white-space: nowrap;
  margin-right: 26px;
  display: block;
  overflow: hidden;
}

.chosen-container-single .chosen-single-with-deselect span {
  margin-right: 38px;
}

.chosen-container-single .chosen-single abbr {
  background: url("chosen-sprite.png?v=8b55a8") -42px 1px no-repeat;
  width: 12px;
  height: 12px;
  font-size: 1px;
  display: block;
  position: absolute;
  top: 6px;
  right: 26px;
}

.chosen-container-single .chosen-single abbr:hover, .chosen-container-single.chosen-disabled .chosen-single abbr:hover {
  background-position: -42px -10px;
}

.chosen-container-single .chosen-single div {
  width: 18px;
  height: 100%;
  display: block;
  position: absolute;
  top: 0;
  right: 0;
}

.chosen-container-single .chosen-single div b {
  background: url("chosen-sprite.png?v=8b55a8") 0 2px no-repeat;
  width: 100%;
  height: 100%;
  display: block;
}

.chosen-container-single .chosen-search {
  z-index: 1010;
  white-space: nowrap;
  margin: 0;
  padding: 3px 4px;
  position: relative;
}

.chosen-container-single .chosen-search input[type="text"] {
  background: url("chosen-sprite.png?v=8b55a8") 100% -20px no-repeat;
  border: 1px solid #aaa;
  border-radius: 0;
  outline: 0;
  width: 100%;
  height: auto;
  margin: 1px 0;
  padding: 4px 20px 4px 5px;
  font-family: sans-serif;
  font-size: 1em;
  line-height: normal;
}

.chosen-container-single .chosen-drop {
  background-clip: padding-box;
  border-radius: 0 0 4px 4px;
  margin-top: -1px;
}

.chosen-container-single.chosen-container-single-nosearch .chosen-search {
  clip: rect(0, 0, 0, 0);
  -webkit-clip-path: inset(100%);
  clip-path: inset(100%);
  position: absolute;
}

.chosen-container .chosen-results {
  color: #444;
  -webkit-overflow-scrolling: touch;
  max-height: 240px;
  margin: 0 4px 4px 0;
  padding: 0 0 0 4px;
  position: relative;
  overflow: hidden auto;
}

.chosen-container .chosen-results li {
  word-wrap: break-word;
  -webkit-touch-callout: none;
  margin: 0;
  padding: 5px 6px;
  line-height: 15px;
  list-style: none;
  display: none;
}

.chosen-container .chosen-results li.active-result {
  cursor: pointer;
  display: list-item;
}

.chosen-container .chosen-results li.disabled-result {
  color: #ccc;
  cursor: default;
  display: list-item;
}

.chosen-container .chosen-results li.highlighted {
  color: #fff;
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(.2, #3875d7), color-stop(.9, #2a62bc));
  background-color: #3875d7;
  background-image: linear-gradient(#3875d7 20%, #2a62bc 90%);
}

.chosen-container .chosen-results li.no-results {
  color: #777;
  background: #f4f4f4;
  display: list-item;
}

.chosen-container .chosen-results li.group-result {
  cursor: default;
  font-weight: bold;
  display: list-item;
}

.chosen-container .chosen-results li.group-option {
  padding-left: 15px;
}

.chosen-container .chosen-results li em {
  font-style: normal;
  text-decoration: underline;
}

.chosen-container-multi .chosen-choices {
  cursor: text;
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(.01, #eee), color-stop(.15, #fff));
  background-color: #fff;
  background-image: linear-gradient(#eee 1%, #fff 15%);
  border: 1px solid #aaa;
  width: 100%;
  height: auto;
  margin: 0;
  padding: 0 5px;
  position: relative;
  overflow: hidden;
}

.chosen-container-multi .chosen-choices li {
  float: left;
  list-style: none;
}

.chosen-container-multi .chosen-choices li.search-field {
  white-space: nowrap;
  margin: 0;
  padding: 0;
}

.chosen-container-multi .chosen-choices li.search-field input[type="text"] {
  height: 25px;
  -webkit-box-shadow: none;
  box-shadow: none;
  color: #999;
  border-radius: 0;
  outline: 0;
  width: 25px;
  margin: 1px 0;
  padding: 0;
  font-family: sans-serif;
  font-size: 100%;
  line-height: normal;
  background: none !important;
  border: 0 !important;
}

.chosen-container-multi .chosen-choices li.search-choice {
  color: #333;
  cursor: default;
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(.2, #f4f4f4), color-stop(.5, #f0f0f0), color-stop(.52, #e8e8e8), to(#eee));
  background-color: #eee;
  background-image: linear-gradient(#f4f4f4 20%, #f0f0f0 50%, #e8e8e8 52%, #eee 100%);
  background-repeat: repeat-x;
  background-size: 100% 19px;
  background-clip: padding-box;
  border: 1px solid #aaa;
  border-radius: 3px;
  max-width: 100%;
  margin: 3px 5px 3px 0;
  padding: 3px 20px 3px 5px;
  line-height: 13px;
  position: relative;
  -webkit-box-shadow: inset 0 0 2px #fff, 0 1px #0000000d;
  box-shadow: inset 0 0 2px #fff, 0 1px #0000000d;
}

.chosen-container-multi .chosen-choices li.search-choice span {
  word-wrap: break-word;
}

.chosen-container-multi .chosen-choices li.search-choice .search-choice-close {
  background: url("chosen-sprite.png?v=8b55a8") -42px 1px no-repeat;
  width: 12px;
  height: 12px;
  font-size: 1px;
  display: block;
  position: absolute;
  top: 4px;
  right: 3px;
}

.chosen-container-multi .chosen-choices li.search-choice .search-choice-close:hover {
  background-position: -42px -10px;
}

.chosen-container-multi .chosen-choices li.search-choice-disabled {
  color: #666;
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(.2, #f4f4f4), color-stop(.5, #f0f0f0), color-stop(.52, #e8e8e8), to(#eee));
  background-color: #e4e4e4;
  background-image: linear-gradient(#f4f4f4 20%, #f0f0f0 50%, #e8e8e8 52%, #eee 100%);
  border: 1px solid #ccc;
  padding-right: 5px;
}

.chosen-container-multi .chosen-choices li.search-choice-focus {
  background: #d4d4d4;
}

.chosen-container-multi .chosen-choices li.search-choice-focus .search-choice-close {
  background-position: -42px -10px;
}

.chosen-container-multi .chosen-results {
  margin: 0;
  padding: 0;
}

.chosen-container-multi .chosen-drop .result-selected {
  color: #ccc;
  cursor: default;
  display: list-item;
}

.chosen-container-active .chosen-single {
  border: 1px solid #5897fb;
  -webkit-box-shadow: 0 0 5px #0000004d;
  box-shadow: 0 0 5px #0000004d;
}

.chosen-container-active.chosen-with-drop .chosen-single {
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(.2, #eee), color-stop(.8, #fff));
  background-image: linear-gradient(#eee 20%, #fff 80%);
  border: 1px solid #aaa;
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
  -webkit-box-shadow: inset 0 1px #fff;
  box-shadow: inset 0 1px #fff;
}

.chosen-container-active.chosen-with-drop .chosen-single div {
  background: none;
  border-left: none;
}

.chosen-container-active.chosen-with-drop .chosen-single div b {
  background-position: -18px 2px;
}

.chosen-container-active .chosen-choices {
  border: 1px solid #5897fb;
  -webkit-box-shadow: 0 0 5px #0000004d;
  box-shadow: 0 0 5px #0000004d;
}

.chosen-container-active .chosen-choices li.search-field input[type="text"] {
  color: #222 !important;
}

.chosen-disabled {
  cursor: default;
  opacity: .5 !important;
}

.chosen-disabled .chosen-single, .chosen-disabled .chosen-choices .search-choice .search-choice-close {
  cursor: default;
}

.chosen-rtl {
  text-align: right;
}

.chosen-rtl .chosen-single {
  padding: 0 8px 0 0;
  overflow: visible;
}

.chosen-rtl .chosen-single span {
  direction: rtl;
  margin-left: 26px;
  margin-right: 0;
}

.chosen-rtl .chosen-single-with-deselect span {
  margin-left: 38px;
}

.chosen-rtl .chosen-single div {
  left: 3px;
  right: auto;
}

.chosen-rtl .chosen-single abbr {
  left: 26px;
  right: auto;
}

.chosen-rtl .chosen-choices li {
  float: right;
}

.chosen-rtl .chosen-choices li.search-field input[type="text"] {
  direction: rtl;
}

.chosen-rtl .chosen-choices li.search-choice {
  margin: 3px 5px 3px 0;
  padding: 3px 5px 3px 19px;
}

.chosen-rtl .chosen-choices li.search-choice .search-choice-close {
  left: 4px;
  right: auto;
}

.chosen-rtl.chosen-container-single .chosen-results {
  margin: 0 0 4px 4px;
  padding: 0 4px 0 0;
}

.chosen-rtl .chosen-results li.group-option {
  padding-left: 0;
  padding-right: 15px;
}

.chosen-rtl.chosen-container-active.chosen-with-drop .chosen-single div {
  border-right: none;
}

.chosen-rtl .chosen-search input[type="text"] {
  direction: rtl;
  background: url("chosen-sprite.png?v=8b55a8") -30px -20px no-repeat;
  padding: 4px 5px 4px 20px;
}

.chosen-rtl.chosen-container-single .chosen-single div b {
  background-position: 6px 2px;
}

.chosen-rtl.chosen-container-single.chosen-with-drop .chosen-single div b {
  background-position: -12px 2px;
}

@media only screen and (-webkit-device-pixel-ratio >= 1.5), only screen and (resolution >= 144dpi), only screen and (resolution >= 1.5x) {
  .chosen-rtl .chosen-search input[type="text"], .chosen-container-single .chosen-single abbr, .chosen-container-single .chosen-single div b, .chosen-container-single .chosen-search input[type="text"], .chosen-container-multi .chosen-choices .search-choice .search-choice-close, .chosen-container .chosen-results-scroll-down span, .chosen-container .chosen-results-scroll-up span {
    background-image: url("chosen-sprite@2x.png?v=614fad") !important;
    background-repeat: no-repeat !important;
    background-size: 52px 37px !important;
  }
}
