<?php

declare(strict_types=1);

namespace Laminas\Diactoros\Exception;

use Throwable;

/**
 * Marker interface for package-specific exceptions.
 */
interface ExceptionInterface extends Throwable
{
}
