@charset "UTF-8";
pre.phpdebugbar-widgets-code-block {
  white-space: pre;
  word-wrap: normal;
  overflow: hidden;
}

pre.phpdebugbar-widgets-code-block code {
  display: block;
  overflow: auto hidden;
}

pre.phpdebugbar-widgets-code-block code.phpdebugbar-widgets-numbered-code {
  padding: 5px;
  line-height: normal;
}

pre.phpdebugbar-widgets-code-block ul li.phpdebugbar-widgets-highlighted-line {
  font-weight: bolder;
  text-decoration: underline;
}

pre.phpdebugbar-widgets-code-block ul li.phpdebugbar-widgets-highlighted-line span {
  background: var(--debugbar-text);
  opacity: .15;
  min-width: calc(100% - 85px);
  margin-left: 10px;
  position: absolute;
}

pre.phpdebugbar-widgets-code-block ul {
  float: left;
  border-right: 1px solid var(--debugbar-header-border);
  text-align: right;
  padding: 5px;
  position: static;
}

.phpdebugbar-widgets-kvlist span.phpdebugbar-widgets-filename, li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-filename, table.phpdebugbar-widgets-tablevar span.phpdebugbar-widgets-filename {
  float: right;
  color: var(--debugbar-link);
  margin-left: 8px;
  font-style: italic;
  display: block;
}

a.phpdebugbar-widgets-editor-link {
  color: var(--debugbar-link);
}

.phpdebugbar-widgets-kvlist span.phpdebugbar-widgets-filename:hover, li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-filename:hover, a.phpdebugbar-widgets-editor-link:hover {
  color: var(--debugbar-hover);
}

a.phpdebugbar-widgets-editor-link:before {
  content: "";
  margin-left: 4px;
  margin-right: 4px;
  font-family: PhpDebugbarFontAwesome;
  font-size: 12px;
  font-style: normal;
}

ul.phpdebugbar-widgets-list {
  font-family: var(--debugbar-font-mono);
  margin: 0;
  padding: 0;
  list-style: none;
}

ul.phpdebugbar-widgets-list li.phpdebugbar-widgets-list-item {
  border-bottom: 1px solid var(--debugbar-border);
  padding: 3px;
  position: relative;
  overflow: hidden;
}

div.phpdebugbar-widgets-messages {
  height: 100%;
  position: relative;
}

div.phpdebugbar-widgets-messages ul.phpdebugbar-widgets-list {
  padding-bottom: 45px;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value:before {
  margin-right: 8px;
  font-family: PhpDebugbarFontAwesome;
  font-size: 11px;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-alert:before {
  content: "";
  color: #cbcf38;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-debug:before {
  content: "";
  color: #78d79a;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-warning:before, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-emergency:before, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-notice:before {
  content: "";
  color: #ecb03d;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-error, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-critical:before {
  color: red;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-error:before, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-value.phpdebugbar-widgets-critical:before {
  content: "";
}

.phpdebugbar-widgets-params .phpdebugbar-widgets-value pre.sf-dump, dl.phpdebugbar-widgets-kvlist dd.phpdebugbar-widgets-value pre.sf-dump, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item pre.sf-dump {
  padding-top: 0;
  padding-bottom: 0;
  display: inline-block !important;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-collector, div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-label {
  float: right;
  color: #888;
  text-shadow: none;
  background: none;
  margin: 0 2px;
  padding: 2px 4px;
  font-size: 12px;
  font-weight: normal;
  text-decoration: none;
}

div.phpdebugbar-widgets-messages li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-collector {
  color: #555;
  font-style: italic;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar {
  background: var(--debugbar-header);
  width: 100%;
  color: var(--debugbar-text);
  border-top: 1px solid var(--debugbar-border);
  border-bottom: 0;
  height: 20px;
  padding: 4px 0;
  position: fixed;
  bottom: 0;
}

div.phpdebugbar-widgets-messages li .phpdebugbar-widgets-label-called-from {
  float: right;
  color: var(--debugbar-text-muted);
  border-bottom: 1px dotted var(--debugbar-border);
  padding-left: 5px;
}

div.phpdebugbar-widgets-messages li .phpdebugbar-widgets-label-called-from:before {
  content: "";
  margin-right: 4px;
  font-family: PhpDebugbarFontAwesome;
  font-size: 12px;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar input {
  width: 30%;
  box-shadow: none;
  border: 0;
  border-radius: 3px;
  height: 15px;
  margin: 0 0 0 7px;
  padding: 2px 6px;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar input:focus {
  outline: none;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter {
  float: right;
  color: var(--debugbar-background);
  background: #7cacd5;
  border-radius: 4px;
  margin: 0 2px;
  padding: 2px 4px;
  font-size: 12px;
  text-decoration: none;
}

div.phpdebugbar-widgets-messages div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded {
  background: var(--debugbar-active);
  color: var(--debugbar-text-muted);
}

dl.phpdebugbar-widgets-kvlist {
  margin: 0;
}

dl.phpdebugbar-widgets-kvlist dt {
  float: left;
  border-top: 1px solid var(--debugbar-border);
  clear: both;
  text-overflow: ellipsis;
  white-space: nowrap;
  width: 150px;
  padding: 5px;
  font-weight: bold;
  overflow: hidden;
}

dl.phpdebugbar-widgets-kvlist dd {
  border-top: 1px solid var(--debugbar-border);
  cursor: pointer;
  min-height: 17px;
  margin-left: 160px;
  padding: 5px;
}

dl.phpdebugbar-widgets-varlist, dl.phpdebugbar-widgets-htmlvarlist {
  font-family: var(--debugbar-font-mono);
}

dl.phpdebugbar-widgets-htmlvarlist dd {
  cursor: initial;
}

ul.phpdebugbar-widgets-timeline {
  margin: 0;
  padding: 0;
  list-style: none;
}

ul.phpdebugbar-widgets-timeline .phpdebugbar-widgets-measure {
  border: none;
  height: 20px;
  display: block;
  position: relative;
}

ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-label, ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-collector {
  font-size: 12px;
  font-family: var(--debugbar-font-mono);
  color: var(--debugbar-text);
  text-shadow: none;
  white-space: pre;
  background: none;
  font-weight: normal;
  position: absolute;
  top: 4px;
  left: 5px;
}

ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-collector {
  left: initial;
  right: 5px;
}

ul.phpdebugbar-widgets-timeline li span.phpdebugbar-widgets-value {
  background-color: var(--debugbar-accent);
  border-bottom: 2px solid var(--debugbar-accent-border);
  border-radius: 3px;
  min-width: 2px;
  height: calc(100% - 4px);
  display: block;
  position: absolute;
  top: 2px;
}

ul.phpdebugbar-widgets-timeline table.phpdebugbar-widgets-params {
  border: 1px solid var(--debugbar-border);
  width: 70%;
  font-family: var(--debugbar-font-mono);
  border-collapse: collapse;
  margin: 10px;
  display: none;
}

ul.phpdebugbar-widgets-timeline table.phpdebugbar-widgets-params td {
  border: 1px solid var(--debugbar-border);
  border-left: none;
  border-right: none;
  padding: 0 5px;
}

ul.phpdebugbar-widgets-timeline table.phpdebugbar-widgets-params .phpdebugbar-widgets-name {
  vertical-align: top;
  width: 20%;
  font-weight: bold;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item {
  cursor: pointer;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-message {
  color: red;
  display: block;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-type {
  font-weight: bold;
  display: block;
  position: absolute;
  top: 4px;
  right: 4px;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item pre.phpdebugbar-widgets-file {
  border: 1px solid var(--debugbar-border);
  font-family: var(--debugbar-font-mono);
  margin: 10px;
  padding: 5px;
  display: none;
}

div.phpdebugbar-widgets-exceptions li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-filename {
  float: none;
}

ul.phpdebugbar-widgets-timeline table.phpdebugbar-widgets-params {
  border: 0;
  width: 99%;
  display: table;
}

div.phpdebugbar[data-theme="dark"] code.phpdebugbar-widgets-sql, div.phpdebugbar[data-theme="dark"] .phpdebugbar-widgets-name, div.phpdebugbar[data-theme="dark"] .phpdebugbar-widgets-key, div.phpdebugbar[data-theme="dark"] .phpdebugbar-widgets-success > pre.sf-dump > .sf-dump-note {
  color: #fdfd96;
}

table.phpdebugbar-widgets-tablevar {
  table-layout: auto;
  width: 100%;
}

table.phpdebugbar-widgets-tablevar td:first-child {
  white-space: nowrap;
  width: 150px;
  font-family: var(--debugbar-font-mono);
}

table.phpdebugbar-widgets-tablevar td.phpdebugbar-widgets-editor {
  white-space: nowrap;
  text-align: right;
  width: 5%;
}

table.phpdebugbar-widgets-tablevar tr.phpdebugbar-widgets-header td, table.phpdebugbar-widgets-tablevar tr.phpdebugbar-widgets-summary td {
  font-weight: bold;
}

table.phpdebugbar-widgets-tablevar td {
  border-bottom: 1px solid var(--debugbar-border);
  text-overflow: ellipsis;
  white-space: nowrap;
  padding: 2px 4px;
  font-weight: normal;
  overflow: hidden;
}

table.phpdebugbar-widgets-tablevar .phpdebugbar-widgets-header span.phpdebugbar-widgets-badge {
  background: var(--debugbar-badge-active);
  color: var(--debugbar-badge-active-text);
  text-shadow: none;
  vertical-align: middle;
  border-radius: 4px;
  margin: 0 5px 0 8px;
  padding: 0 6px;
  font-size: 11px;
  font-weight: normal;
  line-height: 14px;
}
