@charset "UTF-8";
div.phpdebugbar-widgets-sqlqueries .phpdebugbar-widgets-status {
  font-family: var(--debugbar-font-mono);
  border-bottom: 1px solid var(--debugbar-border);
  color: var(--debugbar-text);
  background-color: var(--debugbar-background-alt);
  padding: 6px;
  font-weight: bold;
}

div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item.phpdebugbar-widgets-error {
  color: red;
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-database, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-duration, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-memory, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-row-count, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-copy-clipboard, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-stmt-id {
  float: right;
  color: var(--debugbar-text-muted);
  margin-left: 8px;
}

div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-database, div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-duration, div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-memory, div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-row-count, div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-copy-clipboard, div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-status span.phpdebugbar-widgets-stmt-id {
  color: var(--debugbar-text);
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-database:before, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-duration:before, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-memory:before, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-row-count:before, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-copy-clipboard:before, div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-stmt-id:before, div.phpdebugbar-widgets-sqlqueries a.phpdebugbar-widgets-editor-link:before {
  margin-right: 4px;
  font-family: PhpDebugbarFontAwesome;
  font-size: 12px;
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-database:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-duration:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-memory:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-row-count:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-stmt-id:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-copy-clipboard:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries span.phpdebugbar-widgets-copy-clipboard-check:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries a.phpdebugbar-widgets-editor-link:before {
  content: "";
}

div.phpdebugbar-widgets-sqlqueries a.phpdebugbar-widgets-editor-link {
  color: var(--debugbar-link);
}

div.phpdebugbar-widgets-sqlqueries a.phpdebugbar-widgets-editor-link:hover {
  color: var(--debugbar-hover);
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params {
  border: 1px solid var(--debugbar-border);
  width: 70%;
  font-family: var(--debugbar-font-mono);
  border-collapse: collapse;
  margin: 10px;
  display: none;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params td {
  border: 1px solid var(--debugbar-border);
  text-align: center;
}

div.phpdebugbar-widgets-sqlqueries table.phpdebugbar-widgets-params .phpdebugbar-widgets-name {
  width: 20%;
  font-weight: bold;
}

div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item span.phpdebugbar-widgets-error {
  font-weight: bold;
  display: block;
}

code.phpdebugbar-widgets-sql {
  white-space: pre-wrap;
  overflow-wrap: break-word;
  word-wrap: break-word;
}

div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item.phpdebugbar-widgets-sql-slow {
  background-color: #ffe4e4;
}

div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item.phpdebugbar-widgets-sql-duplicate {
  background-color: #fdfdcd;
}

div.phpdebugbar[data-theme="dark"] div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item.phpdebugbar-widgets-sql-slow {
  background-color: #623100;
}

div.phpdebugbar[data-theme="dark"] div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item.phpdebugbar-widgets-sql-duplicate {
  background-color: #565602;
}

div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-toolbar {
  z-index: 1;
  width: calc(100% - 30px);
  margin-bottom: 5px;
  display: none;
  position: fixed;
  bottom: 0;
}

div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter {
  float: right;
  color: #fff;
  background: #7cacd5;
  border-radius: 4px;
  margin: 0 2px;
  padding: 2px 4px;
  font-size: 12px;
  text-decoration: none;
}

div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-toolbar a.phpdebugbar-widgets-filter.phpdebugbar-widgets-excluded {
  background: var(--debugbar-background);
  color: var(--debugbar-text);
}

div.phpdebugbar-widgets-sqlqueries a.phpdebugbar-widgets-duplicates {
  font-weight: bold;
  text-decoration: underline;
}

div.phpdebugbar-widgets-sqlqueries li.phpdebugbar-widgets-list-item div.phpdebugbar-widgets-bg-measure {
  pointer-events: none;
  width: 100%;
  height: 100%;
  position: absolute;
  top: 0;
  left: 0;
  overflow: hidden;
}

div.phpdebugbar-widgets-sqlqueries div.phpdebugbar-widgets-bg-measure div.phpdebugbar-widgets-value {
  opacity: 1;
  background: var(--debugbar-accent-border);
  height: 1px;
  position: absolute;
  bottom: 0;
}

div.phpdebugbar-widgets-sqlqueries td.phpdebugbar-widgets-value li.phpdebugbar-widgets-table-list-item {
  text-align: left;
  padding-left: 6px;
}

div.phpdebugbar-widgets-sqlqueries .phpdebugbar-text-muted {
  color: var(--debugbar-text-muted);
}
