<?php

/**
 * @package     Joomla.Site
 * @subpackage  mod_users_latest
 *
 * @copyright   (C) 2009 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;
?>
<?php if (!empty($names)) : ?>
    <ul class="mod-userslatest latestusers mod-list">
    <?php foreach ($names as $name) : ?>
        <li>
            <?php echo $name->username; ?>
        </li>
    <?php endforeach; ?>
    </ul>
<?php endif; ?>
