tinymce.addI18n('af',{
"Cut": "Knip",
"Heading 5": "Hooflyn 5",
"Header 2": "Hooflyn 2",
"Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X\/C\/V keyboard shortcuts instead.": "Jou webblaaier ondersteun nie toegang tot die knipbord nie. Gebruik asb. Ctrl+X\/C\/V.",
"Heading 4": "Hooflyn 4",
"Div": "Div",
"Heading 2": "Hooflyn 2",
"Paste": "Plak",
"Close": "Sluit",
"Font Family": "Font Familie",
"Pre": "Pre",
"Align right": "Regs-gerig",
"New document": "Nuwe dokument",
"Blockquote": "Aanhaaling",
"Numbered list": "Genommerde lys",
"Heading 1": "Hooflyn 1",
"Headings": "Koppe",
"Increase indent": "Inkeping vergroot",
"Formats": "Formate",
"Headers": "Hooflyn-tekste",
"Select all": "Alles selekteer",
"Header 3": "Hooflyn 3",
"Blocks": "Blok",
"Undo": "Ongedaan maak",
"Strikethrough": "Deurhaal",
"Bullet list": "Opsommingsteken-lys",
"Header 1": "Hooflyn 1",
"Superscript": "Superskrif",
"Clear formatting": "Herstel Formateering",
"Font Sizes": "Tekengrootte",
"Subscript": "Subskrif",
"Header 6": "Hooflyn 6",
"Redo": "Herdoen",
"Paragraph": "Paragraaf",
"Ok": "OK",
"Bold": "Vetdruk",
"Code": "Kode",
"Italic": "Kursief",
"Align center": "Sentreer",
"Header 5": "Hooflyn 5",
"Heading 6": "Hooflyn 6",
"Heading 3": "Hooflyn 3",
"Decrease indent": "Inkeping verklein",
"Header 4": "Hooflyn 4",
"Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Die plak funksie is nou in plat-teks modus. Teks word ingevoeg sonder enige formateering, todat jy hierdie opsie wissel.",
"Underline": "Onderlyn",
"Cancel": "Kanselleer",
"Justify": "Gerigstelling",
"Inline": "Inlyn",
"Copy": "Kopieer",
"Align left": "Linksgerig",
"Visual aids": "Hulpmiddels",
"Lower Greek": "Griekse letters",
"Square": "Vierkantjie",
"Default": "Verstek",
"Lower Alpha": "Klein letters",
"Circle": "Sirkeltjie",
"Disc": "Balletjie",
"Upper Alpha": "Hoofletters",
"Upper Roman": "Romeinse syfers groot",
"Lower Roman": "Romeinse syfers klein",
"Id should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "Id moet met 'n letter begin en kan slegs deur letters, koppeltekens, syfers, punte en onderstreep-karakters gevolg word",
"Name": "Naam",
"Anchor": "Anker",
"Id": "Id",
"You have unsaved changes are you sure you want to navigate away?": "Jy het ongestoorde wysigings op hierdier bladsy - is jy seker dat jy die bladsy wil verlaat?",
"Restore last draft": "Herstel die laatste konsep",
"Special character": "Spesiaale karakter",
"Source code": "Bronkode",
"Language": "Taal",
"Insert\/Edit code sample": "Voeg\/Redigeer voorbeeld-kode",
"B": "Blou",
"R": "Rooi",
"G": "Groen",
"Color": "Kleur",
"Right to left": "Regs na links",
"Left to right": "Links na regs",
"Emoticons": "Emotikons",
"Robots": "Robotte",
"Document properties": "Dokument eienskappe",
"Title": "Titel",
"Keywords": "Sleutelwoorde",
"Encoding": "Kodering",
"Description": "Beskrywing",
"Author": "Outeur",
"Fullscreen": "Voll-skerm",
"Horizontal line": "Horisontale lyn",
"Horizontal space": "Horisontale ruimte",
"Insert\/edit image": "Afbeelding invoeg\/bewerk",
"General": "Algemeen",
"Advanced": "Gevorderd",
"Source": "Bron",
"Border": "Rand",
"Constrain proportions": "Behou verhoudings",
"Vertical space": "Vertikale ruimte",
"Image description": "Afbeelding bemskrywing",
"Style": "Styl",
"Dimensions": "Afmetings",
"Insert image": "Afbeelding invoeg",
"Image": "Afbeelding",
"Zoom in": "Inzoem",
"Contrast": "Kontras",
"Back": "Terug",
"Gamma": "Gamma",
"Flip horizontally": "Horisontaal weerspie\u00ebl",
"Resize": "Grootte wysig",
"Sharpen": "Verskerp",
"Zoom out": "Uitzoem",
"Image options": "Afbeelding opsies",
"Apply": "Toepas",
"Brightness": "Helderheid",
"Rotate clockwise": "Regsom draai",
"Rotate counterclockwise": "Linksom draai",
"Edit image": "Bewerk afbeelding",
"Color levels": "Kleurvlakke",
"Crop": "Afknip",
"Orientation": "Orienteering",
"Flip vertically": "Vertikaal weerspie\u00ebl",
"Invert": "Omkeer",
"Date\/time": "Datum\/tyd",
"Insert date\/time": "Voeg datum\/tyd in",
"Remove link": "Skakel verwyder",
"Url": "URL",
"Text to display": "Skakelteks",
"Anchors": "Anker",
"Insert link": "Skakel invoeg",
"Link":"Skakel",
"New window": "Nuwe venster",
"None": "Geen",
"The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "Die URL verwys na 'n eksterne adres. Wil jy die \"http:\/\/\" voorvoegsel byvoeg?",
"Paste or type a link": "Plak of tik 'n skalel in",
"Target": "Teiken",
"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "Die URL lyk soos 'n eposadres. Wil jy die \"mailto:\" voorvoegsel byvoeg?",
"Insert\/edit link": "Skakel invoeg\/bewerk",
"Insert\/edit video": "Video invoeg\/bewerk",
"Poster": "Plakaat",
"Insert\/edit media": "Voeg in\/Redigeer media",
"Media": "Media",
"Alternative source": "Alternatiewe bron",
"Paste your embed code below:": "Plak jou ingesluite kode hieronder in:",
"Insert video": "Video invoeg",
"Embed": "Insluit",
"Nonbreaking space": "Vaste spasie invoeg",
"Page break": "Nuwe Bladsy",
"Paste as text": "As teks plak",
"Preview": "Voorbeeld",
"Print": "Druk",
"Save": "Stoor",
"Could not find the specified string.": "Kon nie die gesoekde string vind nie",
"Replace": "Vervang",
"Next": "Volgende",
"Whole words": "Sleg hele woorde",
"Find and replace": "Soek-en-Vervang",
"Replace with": "Vervang met",
"Find": "Soek",
"Replace all": "Alles vervang",
"Match case": "Kassensitief",
"Prev": "Vorige",
"Spellcheck": "Speltoets",
"Finish": "Einde",
"Ignore all": "Alles ignoreer",
"Ignore": "Ignoreer",
"Add to Dictionary": "Aan woordelys byvoeg",
"Insert row before": "Voeg nuwe ry boaan",
"Rows": "Rye",
"Height": "Hoogte",
"Paste row after": "Plak ry onder",
"Alignment": "Gerigdheid",
"Border color": "Randkleur",
"Column group": "Kolomgroep",
"Row": "Ry",
"Insert column before": "Voeg kolom vooraan",
"Split cell": "Split sel",
"Cell padding": "Ruimte binnein sel",
"Cell spacing": "Ruimte rondom sel",
"Row type": "Ry tipe",
"Insert table": "Tabel invoeg",
"Body": "Tabel Inhoud",
"Caption": "Onderskrif",
"Footer": "Voetskrif",
"Delete row": "Verwyder ry",
"Paste row before": "Plak ry vooraan",
"Scope": "Bereik",
"Delete table": "Verwyder tabel",
"H Align": "Horisontaal-gerigdheid",
"Top": "Bo",
"Header cell": "Kop sel",
"Column": "Kolom",
"Row group": "Rygroep",
"Cell": "Sel",
"Middle": "Sentreer",
"Cell type": "Seltipe",
"Copy row": "Kopieer ry",
"Row properties": "Ry eienskappe",
"Table properties": "Tabel eienskappe",
"Bottom": "Onder",
"V Align": "Vertikaal-rerigdheid",
"Header": "Kopteks",
"Right": "Regs",
"Insert column after": "Voeg kolom na",
"Cols": "Kolomme",
"Insert row after": "Voeg ry onderaan",
"Width": "Breedte",
"Cell properties": "Sel eienskappe",
"Left": "Links",
"Cut row": "Knip ry",
"Delete column": "Verwyder kolom",
"Center": "Middel",
"Merge cells": "Selle saamvoeg",
"Insert template": "Sjabloon invoeg",
"Templates": "Sjablone",
"Background color": "Agtergrondkleur",
"Custom...": "Pasgemaakte...",
"Custom color": "Pasgemaakte kleur",
"No color": "Geen kleur",
"Text color": "Tekskleur",
"Table of Contents":"Inhouds-opgawe",
"Show blocks": "Blokke vertoon",
"Show invisible characters": "Onsigbare karakters vertoon",
"Words: {0}": "Woorde: {0}",
"Insert": "Invoeg",
"File": "L\u00eaer",
"Edit": "Bewerk",
"Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Ryk Teks Area. Druk ALT-F9 vir menu, ALT-F10 vir die nutsbalk, ALT-0 vir hulp.",
"Tools": "Gereedskap",
"View": "Beeld",
"Table": "Tabel",
"Format": "Formateering"
});
