.com_tags {

  #fieldset-image-intro,
  #fieldset-image-fulltext {
    > div {
      grid-template-columns: 1fr;
    }

    @include media-breakpoint-up(md) {
      display: inline-block;
      width: calc(50% - 15px);
    }
  }

  #fieldset-image-intro {
    margin-inline-end: 15px;
  }

  #fieldset-image-fulltext {
    margin-inline-start: 15px;
  }
}
