// Awesomplete

.mod-finder__search.input-group {
  flex-wrap: nowrap;
  .awesomplete {
    input {
      @include border-end-radius(0);
    }
  }
  button {
    display: flex;
    align-items: center;
    .icon-search {
      margin-inline-end: .2em;
    }
  }
}

.awesomplete {
  > input {
    max-width: 200px;

    @include media-breakpoint-up(lg) {
      max-width: none;
    }
  }
  > ul {
    z-index: $zindex-dropdown !important;
  }
}
