joomla-field-simple-color {
  display: block;

  .hidden {
    display: none;
  }

  .visually-hidden {
    position: absolute;
    width: 1px;
    height: 1px;
    padding: 0;
    margin: -1px;
    overflow: hidden;
    clip: rect(0,0,0,0);
    border: 0;
  }

  button {
    width: 20px;
    height: 20px;
    overflow: hidden;
    vertical-align: middle;
    background: none;
    border: solid 1px #ccc;
    border-radius: 3px;
    appearance: none;
  }

  .btn-close {
    width: auto;
    padding: 0 6px;
    margin-top: -4px;
    font-size: 1rem;
    line-height: 1.5;
    text-align: center;
    vertical-align: middle;
  }

  .nocolor {
    background: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAIAAAACUFjqAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3ggRDQENU0dyawAAACZJREFUGNNjPHXqDAMSMDY2ROYyMeAFNJVm/Pv3LzL/7Nnzg8VpAKebCGpIIxHBAAAAAElFTkSuQmCC") !important;
  }
  .simplecolors-panel .swatch {
    margin: 0 4px 4px 0;
  }

  .swatch:hover,
  .swatch.active {
    border-color: rgba(82, 168, 236, .8);
    box-shadow: inset 0 1px 1px rgba(0,0,0,.075), 0 0 8px rgba(82, 168, 236, .6);
  }

  .simplecolors-panel[data-open=""] {
    display: block;
  }
  .simplecolors-panel {
    position: absolute;
    z-index: 12;
    display: none;
    float: left;
    padding: 6px 2px 2px 6px;
    margin: 1px 0 0;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid #ddd;
    border-radius: 5px;
    box-shadow: 0 5px 10px rgba(0, 0, 0, .2);
  }
}
