<?php

/**
 * Part of the Joomla Framework Session Package
 *
 * @copyright  Copyright (C) 2005 - 2021 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE
 */

namespace Joomla\Session\Exception;

/**
 * Exception thrown when the database session table cannot be created
 *
 * @since  2.0.0
 */
class CreateSessionTableException extends \RuntimeException
{
}
