<?php

/**
 * @package     Joomla.Site
 * @subpackage  Layout
 *
 * @copyright   (C) 2022 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;

Factory::getApplication()->getDocument()
    ->getWebAssetManager()->useScript('inlinehelp');

echo LayoutHelper::render('joomla.toolbar.standard', $displayData);
