@charset "UTF-8";
/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
.icon-joomla, [class^="icon-"], [class*=" icon-"] {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  font-variant: normal;
  font-style: normal;
  font-weight: normal;
  line-height: 1;
  display: inline-block;
}

@media (prefers-reduced-motion: reduce) {
  .fa-beat, .fa-bounce, .fa-fade, .fa-beat-fade, .fa-flip, .fa-pulse, .icon-pulse, .fa-shake, .fa-spin, .icon-spin, .fa-spin-pulse {
    transition-duration: 0s;
    transition-delay: 0s;
    animation-duration: 1ms;
    animation-iteration-count: 1;
    animation-delay: -1ms;
  }
}

@keyframes fa-beat {
  0%, 90% {
    transform: scale(1);
  }

  45% {
    transform: scale(var(--fa-beat-scale, 1.25));
  }
}

@keyframes fa-bounce {
  0% {
    transform: scale(1)translateY(0);
  }

  10% {
    transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, .9)) translateY(0);
  }

  30% {
    transform: scale(var(--fa-bounce-jump-scale-x, .9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -.5em));
  }

  50% {
    transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, .95)) translateY(0);
  }

  57% {
    transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -.125em));
  }

  64% {
    transform: scale(1)translateY(0);
  }

  100% {
    transform: scale(1)translateY(0);
  }
}

@keyframes fa-fade {
  50% {
    opacity: var(--fa-fade-opacity, .4);
  }
}

@keyframes fa-beat-fade {
  0%, 100% {
    opacity: var(--fa-beat-fade-opacity, .4);
    transform: scale(1);
  }

  50% {
    opacity: 1;
    transform: scale(var(--fa-beat-fade-scale, 1.125));
  }
}

@keyframes fa-flip {
  50% {
    transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
  }
}

@keyframes fa-shake {
  0% {
    transform: rotate(-15deg);
  }

  4% {
    transform: rotate(15deg);
  }

  8%, 24% {
    transform: rotate(-18deg);
  }

  12%, 28% {
    transform: rotate(18deg);
  }

  16% {
    transform: rotate(-22deg);
  }

  20% {
    transform: rotate(22deg);
  }

  32% {
    transform: rotate(-12deg);
  }

  36% {
    transform: rotate(12deg);
  }

  40%, 100% {
    transform: rotate(0);
  }
}

@keyframes fa-spin {
  0% {
    transform: rotate(0);
  }

  100% {
    transform: rotate(360deg);
  }
}

.fa-flip-both, .icon-flip-both, .fa-flip-horizontal.fa-flip-vertical, .fa-flip-horizontal.icon-flip-vertical, .fa-flip-vertical.icon-flip-horizontal, .icon-flip-horizontal.icon-flip-vertical {
  transform: scale(-1);
}

.fa-circle-chevron-right, .fa-chevron-circle-right {
  --fa: "";
}

.fa-trash-can, .fa-trash-alt {
  --fa: "";
}

.fa-user-xmark, .fa-user-times {
  --fa: "";
}

.fa-message, .fa-comment-alt {
  --fa: "";
}

.fa-down-left-and-up-right-to-center, .fa-compress-alt {
  --fa: "";
}

.fa-file-lines, .fa-file-alt, .fa-file-text {
  --fa: "";
}

.fa-calendar-days, .fa-calendar-alt {
  --fa: "";
}

.fa-volleyball, .fa-volleyball-ball {
  --fa: "";
}

.fa-sort-down, .fa-sort-desc {
  --fa: "";
}

.fa-circle-minus, .fa-minus-circle {
  --fa: "";
}

.fa-right-from-bracket, .fa-sign-out-alt {
  --fa: "";
}

.fa-icons, .fa-heart-music-camera-bolt {
  --fa: "";
}

.fa-microphone-lines-slash, .fa-microphone-alt-slash {
  --fa: "";
}

.fa-magnifying-glass-location, .fa-search-location {
  --fa: "";
}

.fa-forward-step, .fa-step-forward {
  --fa: "";
}

.fa-face-smile-beam, .fa-smile-beam {
  --fa: "";
}

.fa-football, .fa-football-ball {
  --fa: "";
}

.fa-angles-down, .fa-angle-double-down {
  --fa: "";
}

.fa-beer-mug-empty, .fa-beer {
  --fa: "";
}

.fa-arrow-up-long, .fa-long-arrow-up {
  --fa: "";
}

.fa-fire-flame-simple, .fa-burn {
  --fa: "";
}

.fa-person, .fa-male {
  --fa: "";
}

.fa-face-grin-stars, .fa-grin-stars {
  --fa: "";
}

.fa-spaghetti-monster-flying, .fa-pastafarianism {
  --fa: "";
}

.fa-spoon, .fa-utensil-spoon {
  --fa: "";
}

.fa-envelopes-bulk, .fa-mail-bulk {
  --fa: "";
}

.fa-circle-h, .fa-hospital-symbol {
  --fa: "";
}

.fa-address-book, .fa-contact-book {
  --fa: "";
}

.fa-pencil, .fa-pencil-alt {
  --fa: "";
}

.fa-paste, .fa-file-clipboard {
  --fa: "";
}

.fa-truck-ramp-box, .fa-truck-loading {
  --fa: "";
}

.fa-scroll-torah, .fa-torah {
  --fa: "";
}

.fa-broom-ball, .fa-quidditch, .fa-quidditch-broom-ball {
  --fa: "";
}

.fa-box-archive, .fa-archive {
  --fa: "";
}

.fa-arrow-down-9-1, .fa-sort-numeric-desc, .fa-sort-numeric-down-alt {
  --fa: "";
}

.fa-face-grin-tongue-squint, .fa-grin-tongue-squint {
  --fa: "";
}

.fa-earth-africa, .fa-globe-africa {
  --fa: "";
}

.fa-tablet-screen-button, .fa-tablet-alt {
  --fa: "";
}

.fa-face-flushed, .fa-flushed {
  --fa: "";
}

.fa-gavel, .fa-legal {
  --fa: "";
}

.fa-bell-concierge, .fa-concierge-bell {
  --fa: "";
}

.fa-pen-ruler, .fa-pencil-ruler {
  --fa: "";
}

.fa-people-arrows, .fa-people-arrows-left-right {
  --fa: "";
}

.fa-square-caret-right, .fa-caret-square-right {
  --fa: "";
}

.fa-scissors, .fa-cut {
  --fa: "";
}

.fa-tachograph-digital, .fa-digital-tachograph {
  --fa: "";
}

.fa-reply, .fa-mail-reply {
  --fa: "";
}

.fa-square-minus, .fa-minus-square {
  --fa: "";
}

.fa-square-caret-down, .fa-caret-square-down {
  --fa: "";
}

.fa-bars, .fa-navicon {
  --fa: "";
}

.fa-hourglass-end, .fa-hourglass-3 {
  --fa: "";
}

.fa-heart-crack, .fa-heart-broken {
  --fa: "";
}

.fa-square-up-right, .fa-external-link-square-alt {
  --fa: "";
}

.fa-face-kiss-beam, .fa-kiss-beam {
  --fa: "";
}

.fa-circle-exclamation, .fa-exclamation-circle {
  --fa: "";
}

.fa-arrow-right-from-bracket, .fa-sign-out {
  --fa: "";
}

.fa-circle-chevron-down, .fa-chevron-circle-down {
  --fa: "";
}

.fa-unlock-keyhole, .fa-unlock-alt {
  --fa: "";
}

.fa-headphones-simple, .fa-headphones-alt {
  --fa: "";
}

.fa-circle-dollar-to-slot, .fa-donate {
  --fa: "";
}

.fa-volume-low, .fa-volume-down {
  --fa: "";
}

.fa-wheat-awn, .fa-wheat-alt {
  --fa: "";
}

.fa-square-check, .fa-check-square {
  --fa: "";
}

.fa-heading, .fa-header {
  --fa: "";
}

.fa-list, .fa-list-squares {
  --fa: "";
}

.fa-square-phone-flip, .fa-phone-square-alt {
  --fa: "";
}

.fa-circle-dot, .fa-dot-circle {
  --fa: "";
}

.fa-face-dizzy, .fa-dizzy {
  --fa: "";
}

.fa-futbol, .fa-futbol-ball, .fa-soccer-ball {
  --fa: "";
}

.fa-paintbrush, .fa-paint-brush {
  --fa: "";
}

.fa-hot-tub-person, .fa-hot-tub {
  --fa: "";
}

.fa-map-location, .fa-map-marked {
  --fa: "";
}

.fa-pen-to-square, .fa-edit {
  --fa: "";
}

.fa-share-nodes, .fa-share-alt {
  --fa: "";
}

.fa-hourglass-half, .fa-hourglass-2 {
  --fa: "";
}

.fa-bag-shopping, .fa-shopping-bag {
  --fa: "";
}

.fa-arrow-down-z-a, .fa-sort-alpha-desc, .fa-sort-alpha-down-alt {
  --fa: "";
}

.fa-hand, .fa-hand-paper {
  --fa: "";
}

.fa-face-kiss, .fa-kiss {
  --fa: "";
}

.fa-face-grin-tongue, .fa-grin-tongue {
  --fa: "";
}

.fa-face-grin-wink, .fa-grin-wink {
  --fa: "";
}

.fa-ear-deaf, .fa-deaf, .fa-deafness, .fa-hard-of-hearing {
  --fa: "";
}

.fa-square-rss, .fa-rss-square {
  --fa: "";
}

.fa-hryvnia-sign, .fa-hryvnia {
  --fa: "";
}

.fa-face-grin-wide, .fa-grin-alt {
  --fa: "";
}

.fa-staff-snake, .fa-rod-asclepius, .fa-rod-snake, .fa-staff-aesculapius {
  --fa: "";
}

.fa-truck-medical, .fa-ambulance {
  --fa: "";
}

.fa-temperature-half, .fa-temperature-2, .fa-thermometer-2, .fa-thermometer-half {
  --fa: "";
}

.fa-poo-storm, .fa-poo-bolt {
  --fa: "";
}

.fa-face-frown-open, .fa-frown-open {
  --fa: "";
}

.fa-folder, .fa-folder-blank {
  --fa: "";
}

.fa-file-waveform, .fa-file-medical-alt {
  --fa: "";
}

.fa-gauge, .fa-dashboard, .fa-gauge-med, .fa-tachometer-alt-average {
  --fa: "";
}

.fa-wand-magic-sparkles, .fa-magic-wand-sparkles {
  --fa: "";
}

.fa-pen-clip, .fa-pen-alt {
  --fa: "";
}

.fa-van-shuttle, .fa-shuttle-van {
  --fa: "";
}

.fa-square-caret-left, .fa-caret-square-left {
  --fa: "";
}

.fa-chart-area, .fa-area-chart {
  --fa: "";
}

.fa-ban, .fa-cancel {
  --fa: "";
}

.fa-spray-can-sparkles, .fa-air-freshener {
  --fa: "";
}

.fa-arrow-pointer, .fa-mouse-pointer {
  --fa: "";
}

.fa-maximize, .fa-expand-arrows-alt {
  --fa: "";
}

.fa-shapes, .fa-triangle-circle-square {
  --fa: "";
}

.fa-shuffle, .fa-random {
  --fa: "";
}

.fa-person-running, .fa-running {
  --fa: "";
}

.fa-computer-mouse, .fa-mouse {
  --fa: "";
}

.fa-arrow-right-to-bracket, .fa-sign-in {
  --fa: "";
}

.fa-shop-slash, .fa-store-alt-slash {
  --fa: "";
}

.fa-hourglass-start, .fa-hourglass-1 {
  --fa: "";
}

.fa-right-to-bracket, .fa-sign-in-alt {
  --fa: "";
}

.fa-thumbtack-slash, .fa-thumb-tack-slash {
  --fa: "";
}

.fa-heart-pulse, .fa-heartbeat {
  --fa: "";
}

.fa-people-carry-box, .fa-people-carry {
  --fa: "";
}

.fa-weight-scale, .fa-weight {
  --fa: "";
}

.fa-user-group, .fa-user-friends {
  --fa: "";
}

.fa-arrow-up-a-z, .fa-sort-alpha-up {
  --fa: "";
}

.fa-face-laugh-squint, .fa-laugh-squint {
  --fa: "";
}

.fa-circle-arrow-up, .fa-arrow-circle-up {
  --fa: "";
}

.fa-person-walking, .fa-walking {
  --fa: "";
}

.fa-bed-pulse, .fa-procedures {
  --fa: "";
}

.fa-shuttle-space, .fa-space-shuttle {
  --fa: "";
}

.fa-face-laugh, .fa-laugh {
  --fa: "";
}

.fa-microphone-lines, .fa-microphone-alt {
  --fa: "";
}

.fa-mars-stroke-up, .fa-mars-stroke-v {
  --fa: "";
}

.fa-champagne-glasses, .fa-glass-cheers {
  --fa: "";
}

.fa-file-arrow-up, .fa-file-upload {
  --fa: "";
}

.fa-wifi, .fa-wifi-3, .fa-wifi-strong {
  --fa: "";
}

.fa-bath, .fa-bathtub {
  --fa: "";
}

.fa-user-pen, .fa-user-edit {
  --fa: "";
}

.fa-border-top-left, .fa-border-style {
  --fa: "";
}

.fa-map-location-dot, .fa-map-marked-alt {
  --fa: "";
}

.fa-square-poll-vertical, .fa-poll {
  --fa: "";
}

.fa-car-battery, .fa-battery-car {
  --fa: "";
}

.fa-mars-stroke-right, .fa-mars-stroke-h {
  --fa: "";
}

.fa-hand-back-fist, .fa-hand-rock {
  --fa: "";
}

.fa-square-caret-up, .fa-caret-square-up {
  --fa: "";
}

.fa-chart-bar, .fa-bar-chart {
  --fa: "";
}

.fa-hands-bubbles, .fa-hands-wash {
  --fa: "";
}

.fa-eye-low-vision, .fa-low-vision {
  --fa: "";
}

.fa-square-plus, .fa-plus-square {
  --fa: "";
}

.fa-martini-glass, .fa-glass-martini-alt {
  --fa: "";
}

.fa-rotate-left, .fa-rotate-back, .fa-rotate-backward, .fa-undo-alt {
  --fa: "";
}

.fa-table-columns, .fa-columns {
  --fa: "";
}

.fa-dolly, .fa-dolly-box {
  --fa: "";
}

.fa-minimize, .fa-compress-arrows-alt {
  --fa: "";
}

.fa-angles-right, .fa-angle-double-right {
  --fa: "";
}

.fa-circle-play, .fa-play-circle {
  --fa: "";
}

.fa-euro-sign, .fa-eur, .fa-euro {
  --fa: "";
}

.fa-circle-check, .fa-check-circle {
  --fa: "";
}

.fa-circle-stop, .fa-stop-circle {
  --fa: "";
}

.fa-compass-drafting, .fa-drafting-compass {
  --fa: "";
}

.fa-face-laugh-beam, .fa-laugh-beam {
  --fa: "";
}

.fa-circle-chevron-up, .fa-chevron-circle-up {
  --fa: "";
}

.fa-sterling-sign, .fa-gbp, .fa-pound-sign {
  --fa: "";
}

.fa-arrow-down-long, .fa-long-arrow-down {
  --fa: "";
}

.fa-reply-all, .fa-mail-reply-all {
  --fa: "";
}

.fa-person-skating, .fa-skating {
  --fa: "";
}

.fa-filter-circle-dollar, .fa-funnel-dollar {
  --fa: "";
}

.fa-circle-arrow-down, .fa-arrow-circle-down {
  --fa: "";
}

.fa-file-import, .fa-arrow-right-to-file {
  --fa: "";
}

.fa-square-arrow-up-right, .fa-external-link-square {
  --fa: "";
}

.fa-temperature-empty, .fa-temperature-0, .fa-thermometer-0, .fa-thermometer-empty {
  --fa: "";
}

.fa-address-card, .fa-contact-card, .fa-vcard {
  --fa: "";
}

.fa-scale-unbalanced-flip, .fa-balance-scale-right {
  --fa: "";
}

.fa-diamond-turn-right, .fa-directions {
  --fa: "";
}

.fa-house-laptop, .fa-laptop-house {
  --fa: "";
}

.fa-face-tired, .fa-tired {
  --fa: "";
}

.fa-cloud-arrow-up, .fa-cloud-upload, .fa-cloud-upload-alt {
  --fa: "";
}

.fa-seedling, .fa-sprout {
  --fa: "";
}

.fa-left-right, .fa-arrows-alt-h {
  --fa: "";
}

.fa-circle-arrow-left, .fa-arrow-circle-left {
  --fa: "";
}

.fa-arrow-down-wide-short, .fa-sort-amount-asc, .fa-sort-amount-down {
  --fa: "";
}

.fa-cloud-bolt, .fa-thunderstorm {
  --fa: "";
}

.fa-text-slash, .fa-remove-format {
  --fa: "";
}

.fa-face-smile-wink, .fa-smile-wink {
  --fa: "";
}

.fa-arrows-left-right, .fa-arrows-h {
  --fa: "";
}

.fa-cloud-arrow-down, .fa-cloud-download, .fa-cloud-download-alt {
  --fa: "";
}

.fa-chalkboard, .fa-blackboard {
  --fa: "";
}

.fa-user-large-slash, .fa-user-alt-slash {
  --fa: "";
}

.fa-handshake-simple-slash, .fa-handshake-alt-slash {
  --fa: "";
}

.fa-arrows-rotate, .fa-refresh, .fa-sync {
  --fa: "";
}

.fa-shield-halved, .fa-shield-alt {
  --fa: "";
}

.fa-book-atlas, .fa-atlas {
  --fa: "";
}

.fa-house-chimney-crack, .fa-house-damage {
  --fa: "";
}

.fa-file-zipper, .fa-file-archive {
  --fa: "";
}

.fa-martini-glass-empty, .fa-glass-martini {
  --fa: "";
}

.fa-person-skiing, .fa-skiing {
  --fa: "";
}

.fa-temperature-arrow-down, .fa-temperature-down {
  --fa: "";
}

.fa-feather-pointed, .fa-feather-alt {
  --fa: "";
}

.fa-rectangle-ad, .fa-ad {
  --fa: "";
}

.fa-circle-arrow-right, .fa-arrow-circle-right {
  --fa: "";
}

.fa-sort, .fa-unsorted {
  --fa: "";
}

.fa-list-ol, .fa-list-1-2, .fa-list-numeric {
  --fa: "";
}

.fa-money-check-dollar, .fa-money-check-alt {
  --fa: "";
}

.fa-face-kiss-wink-heart, .fa-kiss-wink-heart {
  --fa: "";
}

.fa-up-down-left-right, .fa-arrows-alt {
  --fa: "";
}

.fa-star-half-stroke, .fa-star-half-alt {
  --fa: "";
}

.fa-whiskey-glass, .fa-glass-whiskey {
  --fa: "";
}

.fa-arrow-up-right-from-square, .fa-external-link {
  --fa: "";
}

.fa-won-sign, .fa-krw, .fa-won {
  --fa: "";
}

.fa-taxi, .fa-cab {
  --fa: "";
}

.fa-chart-pie, .fa-pie-chart {
  --fa: "";
}

.fa-face-grin-beam, .fa-grin-beam {
  --fa: "";
}

.fa-location-pin, .fa-map-marker {
  --fa: "";
}

.fa-helmet-safety, .fa-hard-hat, .fa-hat-hard {
  --fa: "";
}

.fa-circle-right, .fa-arrow-alt-circle-right {
  --fa: "";
}

.fa-face-rolling-eyes, .fa-meh-rolling-eyes {
  --fa: "";
}

.fa-chart-line, .fa-line-chart {
  --fa: "";
}

.fa-signs-post, .fa-map-signs {
  --fa: "";
}

.fa-screwdriver-wrench, .fa-tools {
  --fa: "";
}

.fa-house-user, .fa-home-user {
  --fa: "";
}

.fa-martini-glass-citrus, .fa-cocktail {
  --fa: "";
}

.fa-face-surprise, .fa-surprise {
  --fa: "";
}

.fa-circle-pause, .fa-pause-circle {
  --fa: "";
}

.fa-apple-whole, .fa-apple-alt {
  --fa: "";
}

.fa-temperature-quarter, .fa-temperature-1, .fa-thermometer-1, .fa-thermometer-quarter {
  --fa: "";
}

.fa-square-poll-horizontal, .fa-poll-h {
  --fa: "";
}

.fa-backward-fast, .fa-fast-backward {
  --fa: "";
}

.fa-basketball, .fa-basketball-ball {
  --fa: "";
}

.fa-circle-up, .fa-arrow-alt-circle-up {
  --fa: "";
}

.fa-mobile-screen-button, .fa-mobile-alt {
  --fa: "";
}

.fa-volume-high, .fa-volume-up {
  --fa: "";
}

.fa-burger, .fa-hamburger {
  --fa: "";
}

.fa-rupee-sign, .fa-rupee {
  --fa: "";
}

.fa-circle-question, .fa-question-circle {
  --fa: "";
}

.fa-phone-flip, .fa-phone-alt {
  --fa: "";
}

.fa-forward-fast, .fa-fast-forward {
  --fa: "";
}

.fa-face-meh-blank, .fa-meh-blank {
  --fa: "";
}

.fa-square-parking, .fa-parking {
  --fa: "";
}

.fa-bars-progress, .fa-tasks-alt {
  --fa: "";
}

.fa-cart-flatbed, .fa-dolly-flatbed {
  --fa: "";
}

.fa-ban-smoking, .fa-smoking-ban {
  --fa: "";
}

.fa-basket-shopping, .fa-shopping-basket {
  --fa: "";
}

.fa-bus-simple, .fa-bus-alt {
  --fa: "";
}

.fa-face-sad-cry, .fa-sad-cry {
  --fa: "";
}

.fa-signal, .fa-signal-5, .fa-signal-perfect {
  --fa: "";
}

.fa-house-chimney, .fa-home-lg {
  --fa: "";
}

.fa-face-frown, .fa-frown {
  --fa: "";
}

.fa-shop, .fa-store-alt {
  --fa: "";
}

.fa-floppy-disk, .fa-save {
  --fa: "";
}

.fa-scale-unbalanced, .fa-balance-scale-left {
  --fa: "";
}

.fa-sort-up, .fa-sort-asc {
  --fa: "";
}

.fa-comment-dots, .fa-commenting {
  --fa: "";
}

.fa-face-grin-squint, .fa-grin-squint {
  --fa: "";
}

.fa-hand-holding-dollar, .fa-hand-holding-usd {
  --fa: "";
}

.fa-hands-praying, .fa-praying-hands {
  --fa: "";
}

.fa-arrow-rotate-right, .fa-arrow-right-rotate, .fa-arrow-rotate-forward, .fa-redo {
  --fa: "";
}

.fa-location-crosshairs, .fa-location {
  --fa: "";
}

.fa-face-grin-tears, .fa-grin-tears {
  --fa: "";
}

.fa-calendar-xmark, .fa-calendar-times {
  --fa: "";
}

.fa-user-gear, .fa-user-cog {
  --fa: "";
}

.fa-arrow-up-1-9, .fa-sort-numeric-up {
  --fa: "";
}

.fa-person-digging, .fa-digging {
  --fa: "";
}

.fa-gauge-simple, .fa-gauge-simple-med, .fa-tachometer-average {
  --fa: "";
}

.fa-quote-right, .fa-quote-right-alt {
  --fa: "";
}

.fa-shirt, .fa-t-shirt, .fa-tshirt {
  --fa: "";
}

.fa-tenge-sign, .fa-tenge {
  --fa: "";
}

.fa-up-right-from-square, .fa-external-link-alt {
  --fa: "";
}

.fa-table-cells, .fa-th {
  --fa: "";
}

.fa-book-bible, .fa-bible {
  --fa: "";
}

.fa-suitcase-medical, .fa-medkit {
  --fa: "";
}

.fa-person-dress, .fa-female {
  --fa: "";
}

.fa-business-time, .fa-briefcase-clock {
  --fa: "";
}

.fa-table-cells-large, .fa-th-large {
  --fa: "";
}

.fa-book-tanakh, .fa-tanakh {
  --fa: "";
}

.fa-phone-volume, .fa-volume-control-phone {
  --fa: "";
}

.fa-cake-candles, .fa-birthday-cake, .fa-cake {
  --fa: "";
}

.fa-angles-up, .fa-angle-double-up {
  --fa: "";
}

.fa-arrow-up-9-1, .fa-sort-numeric-up-alt {
  --fa: "";
}

.fa-hourglass, .fa-hourglass-empty {
  --fa: "";
}

.fa-user-doctor, .fa-user-md {
  --fa: "";
}

.fa-circle-info, .fa-info-circle {
  --fa: "";
}

.fa-camera, .fa-camera-alt {
  --fa: "";
}

.fa-arrow-down-1-9, .fa-sort-numeric-asc, .fa-sort-numeric-down {
  --fa: "";
}

.fa-hand-holding-droplet, .fa-hand-holding-water {
  --fa: "";
}

.fa-prescription-bottle-medical, .fa-prescription-bottle-alt {
  --fa: "";
}

.fa-arrow-down-a-z, .fa-sort-alpha-asc, .fa-sort-alpha-down {
  --fa: "";
}

.fa-arrow-rotate-left, .fa-arrow-left-rotate, .fa-arrow-rotate-back, .fa-arrow-rotate-backward, .fa-undo {
  --fa: "";
}

.fa-hard-drive, .fa-hdd {
  --fa: "";
}

.fa-face-grin-squint-tears, .fa-grin-squint-tears {
  --fa: "";
}

.fa-rectangle-list, .fa-list-alt {
  --fa: "";
}

.fa-person-skiing-nordic, .fa-skiing-nordic {
  --fa: "";
}

.fa-circle-left, .fa-arrow-alt-circle-left {
  --fa: "";
}

.fa-train-subway, .fa-subway {
  --fa: "";
}

.fa-indian-rupee-sign, .fa-indian-rupee, .fa-inr {
  --fa: "";
}

.fa-crop-simple, .fa-crop-alt {
  --fa: "";
}

.fa-money-bill-1, .fa-money-bill-alt {
  --fa: "";
}

.fa-left-long, .fa-long-arrow-alt-left {
  --fa: "";
}

.fa-minus, .fa-subtract {
  --fa: "";
}

.fa-arrow-left-long, .fa-long-arrow-left {
  --fa: "";
}

.fa-hands-asl-interpreting, .fa-american-sign-language-interpreting, .fa-asl-interpreting, .fa-hands-american-sign-language-interpreting {
  --fa: "";
}

.fa-gear, .fa-cog {
  --fa: "";
}

.fa-droplet-slash, .fa-tint-slash {
  --fa: "";
}

.fa-cart-shopping, .fa-shopping-cart {
  --fa: "";
}

.fa-arrow-turn-up, .fa-level-up {
  --fa: "";
}

.fa-square-root-variable, .fa-square-root-alt {
  --fa: "";
}

.fa-clock, .fa-clock-four {
  --fa: "";
}

.fa-backward-step, .fa-step-backward {
  --fa: "";
}

.fa-house-chimney-medical, .fa-clinic-medical {
  --fa: "";
}

.fa-temperature-three-quarters, .fa-temperature-3, .fa-thermometer-3, .fa-thermometer-three-quarters {
  --fa: "";
}

.fa-mobile-screen, .fa-mobile-android-alt {
  --fa: "";
}

.fa-battery-half, .fa-battery-3 {
  --fa: "";
}

.fa-sliders, .fa-sliders-h {
  --fa: "";
}

.fa-ellipsis-vertical, .fa-ellipsis-v {
  --fa: "";
}

.fa-right-long, .fa-long-arrow-alt-right {
  --fa: "";
}

.fa-tty, .fa-teletype {
  --fa: "";
}

.fa-person-hiking, .fa-hiking {
  --fa: "";
}

.fa-cable-car, .fa-tram {
  --fa: "";
}

.fa-face-grin, .fa-grin {
  --fa: "";
}

.fa-delete-left, .fa-backspace {
  --fa: "";
}

.fa-eye-dropper, .fa-eye-dropper-empty, .fa-eyedropper {
  --fa: "";
}

.fa-mobile, .fa-mobile-android, .fa-mobile-phone {
  --fa: "";
}

.fa-face-meh, .fa-meh {
  --fa: "";
}

.fa-book-skull, .fa-book-dead {
  --fa: "";
}

.fa-id-card, .fa-drivers-license {
  --fa: "";
}

.fa-outdent, .fa-dedent {
  --fa: "";
}

.fa-house, .fa-home, .fa-home-alt, .fa-home-lg-alt {
  --fa: "";
}

.fa-arrow-right-arrow-left, .fa-exchange {
  --fa: "";
}

.fa-rotate-right, .fa-redo-alt, .fa-rotate-forward {
  --fa: "";
}

.fa-utensils, .fa-cutlery {
  --fa: "";
}

.fa-arrow-up-wide-short, .fa-sort-amount-up {
  --fa: "";
}

.fa-tower-broadcast, .fa-broadcast-tower {
  --fa: "";
}

.fa-up-long, .fa-long-arrow-alt-up {
  --fa: "";
}

.fa-file-arrow-down, .fa-file-download {
  --fa: "";
}

.fa-bolt, .fa-zap {
  --fa: "";
}

.fa-yen-sign, .fa-cny, .fa-jpy, .fa-rmb, .fa-yen {
  --fa: "";
}

.fa-ruble-sign, .fa-rouble, .fa-rub, .fa-ruble {
  --fa: "";
}

.fa-face-laugh-wink, .fa-laugh-wink {
  --fa: "";
}

.fa-circle-down, .fa-arrow-alt-circle-down {
  --fa: "";
}

.fa-arrow-down-short-wide, .fa-sort-amount-desc, .fa-sort-amount-down-alt {
  --fa: "";
}

.fa-arrow-right-long, .fa-long-arrow-right {
  --fa: "";
}

.fa-ellipsis, .fa-ellipsis-h {
  --fa: "";
}

.fa-kit-medical, .fa-first-aid {
  --fa: "";
}

.fa-credit-card, .fa-credit-card-alt {
  --fa: "";
}

.fa-car, .fa-automobile {
  --fa: "";
}

.fa-book-open-reader, .fa-book-reader {
  --fa: "";
}

.fa-temperature-arrow-up, .fa-temperature-up {
  --fa: "";
}

.fa-square-h, .fa-h-square {
  --fa: "";
}

.fa-temperature-full, .fa-temperature-4, .fa-thermometer-4, .fa-thermometer-full {
  --fa: "";
}

.fa-handshake-angle, .fa-hands-helping {
  --fa: "";
}

.fa-location-dot, .fa-map-marker-alt {
  --fa: "";
}

.fa-person-swimming, .fa-swimmer {
  --fa: "";
}

.fa-droplet, .fa-tint {
  --fa: "";
}

.fa-earth-americas, .fa-earth, .fa-earth-america, .fa-globe-americas {
  --fa: "";
}

.fa-battery-empty, .fa-battery-0 {
  --fa: "";
}

.fa-gauge-high, .fa-tachometer-alt, .fa-tachometer-alt-fast {
  --fa: "";
}

.fa-hospital, .fa-hospital-alt, .fa-hospital-wide {
  --fa: "";
}

.fa-bars-staggered, .fa-reorder, .fa-stream {
  --fa: "";
}

.fa-person-walking-with-cane, .fa-blind {
  --fa: "";
}

.fa-check-to-slot, .fa-vote-yea {
  --fa: "";
}

.fa-boxes-stacked, .fa-boxes, .fa-boxes-alt {
  --fa: "";
}

.fa-link, .fa-chain {
  --fa: "";
}

.fa-ear-listen, .fa-assistive-listening-systems {
  --fa: "";
}

.fa-magnifying-glass, .fa-search {
  --fa: "";
}

.fa-table-tennis-paddle-ball, .fa-ping-pong-paddle-ball, .fa-table-tennis {
  --fa: "";
}

.fa-person-dots-from-line, .fa-diagnoses {
  --fa: "";
}

.fa-trash-can-arrow-up, .fa-trash-restore-alt {
  --fa: "";
}

.fa-file-pen, .fa-file-edit {
  --fa: "";
}

.fa-square-pen, .fa-pen-square, .fa-pencil-square {
  --fa: "";
}

.fa-battery-full, .fa-battery, .fa-battery-5 {
  --fa: "";
}

.fa-list-ul, .fa-list-dots {
  --fa: "";
}

.fa-down-long, .fa-long-arrow-alt-down {
  --fa: "";
}

.fa-landmark-dome, .fa-landmark-alt {
  --fa: "";
}

.fa-tv, .fa-television, .fa-tv-alt {
  --fa: "";
}

.fa-list-check, .fa-tasks {
  --fa: "";
}

.fa-circle-user, .fa-user-circle {
  --fa: "";
}

.fa-car-burst, .fa-car-crash {
  --fa: "";
}

.fa-person-snowboarding, .fa-snowboarding {
  --fa: "";
}

.fa-truck-fast, .fa-shipping-fast {
  --fa: "";
}

.fa-circle-half-stroke, .fa-adjust {
  --fa: "";
}

.fa-circle-radiation, .fa-radiation-alt {
  --fa: "";
}

.fa-baseball, .fa-baseball-ball {
  --fa: "";
}

.fa-diagram-project, .fa-project-diagram {
  --fa: "";
}

.fa-volume-xmark, .fa-volume-mute, .fa-volume-times {
  --fa: "";
}

.fa-grip, .fa-grip-horizontal {
  --fa: "";
}

.fa-share-from-square, .fa-share-square {
  --fa: "";
}

.fa-child-combatant, .fa-child-rifle {
  --fa: "";
}

.fa-square-phone, .fa-phone-square {
  --fa: "";
}

.fa-plus, .fa-add {
  --fa: "+";
}

.fa-xmark, .fa-close, .fa-multiply, .fa-remove, .fa-times {
  --fa: "";
}

.fa-arrows-up-down-left-right, .fa-arrows {
  --fa: "";
}

.fa-chalkboard-user, .fa-chalkboard-teacher {
  --fa: "";
}

.fa-quote-left, .fa-quote-left-alt {
  --fa: "";
}

.fa-trash-arrow-up, .fa-trash-restore {
  --fa: "";
}

.fa-shekel-sign, .fa-ils, .fa-shekel, .fa-sheqel, .fa-sheqel-sign {
  --fa: "";
}

.fa-photo-film, .fa-photo-video {
  --fa: "";
}

.fa-sign-hanging, .fa-sign {
  --fa: "";
}

.fa-tablet, .fa-tablet-android {
  --fa: "";
}

.fa-car-rear, .fa-car-alt {
  --fa: "";
}

.fa-battery-quarter, .fa-battery-2 {
  --fa: "";
}

.fa-baby-carriage, .fa-carriage-baby {
  --fa: "";
}

.fa-percent, .fa-percentage {
  --fa: "%";
}

.fa-face-smile, .fa-smile {
  --fa: "";
}

.fa-thumbtack, .fa-thumb-tack {
  --fa: "";
}

.fa-person-praying, .fa-pray {
  --fa: "";
}

.fa-rotate, .fa-sync-alt {
  --fa: "";
}

.fa-gears, .fa-cogs {
  --fa: "";
}

.fa-face-grin-hearts, .fa-grin-hearts {
  --fa: "";
}

.fa-transgender, .fa-transgender-alt {
  --fa: "";
}

.fa-arrow-turn-down, .fa-level-down {
  --fa: "";
}

.fa-ticket-simple, .fa-ticket-alt {
  --fa: "";
}

.fa-angles-left, .fa-angle-double-left {
  --fa: "";
}

.fa-clock-rotate-left, .fa-history {
  --fa: "";
}

.fa-face-grin-beam-sweat, .fa-grin-beam-sweat {
  --fa: "";
}

.fa-file-export, .fa-arrow-right-from-file {
  --fa: "";
}

.fa-shield, .fa-shield-blank {
  --fa: "";
}

.fa-arrow-up-short-wide, .fa-sort-amount-up-alt {
  --fa: "";
}

.fa-golf-ball-tee, .fa-golf-ball {
  --fa: "";
}

.fa-circle-chevron-left, .fa-chevron-circle-left {
  --fa: "";
}

.fa-wand-magic, .fa-magic {
  --fa: "";
}

.fa-wine-glass-empty, .fa-wine-glass-alt {
  --fa: "";
}

.fa-person-biking, .fa-biking {
  --fa: "";
}

.fa-earth-oceania, .fa-globe-oceania {
  --fa: "";
}

.fa-square-xmark, .fa-times-square, .fa-xmark-square {
  --fa: "";
}

.fa-up-right-and-down-left-from-center, .fa-expand-alt {
  --fa: "";
}

.fa-up-down, .fa-arrows-alt-v {
  --fa: "";
}

.fa-bahai, .fa-haykal {
  --fa: "";
}

.fa-circle-plus, .fa-plus-circle {
  --fa: "";
}

.fa-face-grin-tongue-wink, .fa-grin-tongue-wink {
  --fa: "";
}

.fa-link-slash, .fa-chain-broken, .fa-chain-slash, .fa-unlink {
  --fa: "";
}

.fa-arrow-up-z-a, .fa-sort-alpha-up-alt {
  --fa: "";
}

.fa-fire-flame-curved, .fa-fire-alt {
  --fa: "";
}

.fa-book-quran, .fa-quran {
  --fa: "";
}

.fa-face-angry, .fa-angry {
  --fa: "";
}

.fa-rss, .fa-feed {
  --fa: "";
}

.fa-scale-balanced, .fa-balance-scale {
  --fa: "";
}

.fa-gauge-simple-high, .fa-tachometer, .fa-tachometer-fast {
  --fa: "";
}

.fa-desktop, .fa-desktop-alt {
  --fa: "";
}

.fa-table-list, .fa-th-list {
  --fa: "";
}

.fa-comment-sms, .fa-sms {
  --fa: "";
}

.fa-battery-three-quarters, .fa-battery-4 {
  --fa: "";
}

.fa-hand-fist, .fa-fist-raised {
  --fa: "";
}

.fa-image-portrait, .fa-portrait {
  --fa: "";
}

.fa-earth-europe, .fa-globe-europe {
  --fa: "";
}

.fa-cart-flatbed-suitcase, .fa-luggage-cart {
  --fa: "";
}

.fa-rectangle-xmark, .fa-rectangle-times, .fa-times-rectangle, .fa-window-close {
  --fa: "";
}

.fa-book-journal-whills, .fa-journal-whills {
  --fa: "";
}

.fa-triangle-exclamation, .fa-exclamation-triangle, .fa-warning {
  --fa: "";
}

.fa-share, .fa-mail-forward {
  --fa: "";
}

.fa-right-left, .fa-exchange-alt {
  --fa: "";
}

.fa-money-bill-1-wave, .fa-money-bill-wave-alt {
  --fa: "";
}

.fa-hands, .fa-sign-language, .fa-signing {
  --fa: "";
}

.fa-water-ladder, .fa-ladder-water, .fa-swimming-pool {
  --fa: "";
}

.fa-arrows-up-down, .fa-arrows-v {
  --fa: "";
}

.fa-face-grimace, .fa-grimace {
  --fa: "";
}

.fa-wheelchair-move, .fa-wheelchair-alt {
  --fa: "";
}

.fa-turn-down, .fa-level-down-alt {
  --fa: "";
}

.fa-square-envelope, .fa-envelope-square {
  --fa: "";
}

.fa-bandage, .fa-band-aid {
  --fa: "";
}

.fa-circle-xmark, .fa-times-circle, .fa-xmark-circle {
  --fa: "";
}

.fa-earth-asia, .fa-globe-asia {
  --fa: "";
}

.fa-id-card-clip, .fa-id-card-alt {
  --fa: "";
}

.fa-magnifying-glass-plus, .fa-search-plus {
  --fa: "";
}

.fa-hand-dots, .fa-allergies {
  --fa: "";
}

.fa-mug-saucer, .fa-coffee {
  --fa: "";
}

.fa-magnifying-glass-minus, .fa-search-minus {
  --fa: "";
}

.fa-user-large, .fa-user-alt {
  --fa: "";
}

.fa-note-sticky, .fa-sticky-note {
  --fa: "";
}

.fa-face-sad-tear, .fa-sad-tear {
  --fa: "";
}

.fa-turkish-lira-sign, .fa-try, .fa-turkish-lira {
  --fa: "";
}

.fa-dollar-sign, .fa-dollar, .fa-usd {
  --fa: "$";
}

.fa-magnifying-glass-dollar, .fa-search-dollar {
  --fa: "";
}

.fa-users-gear, .fa-users-cog {
  --fa: "";
}

.fa-building-columns, .fa-bank, .fa-institution, .fa-museum, .fa-university {
  --fa: "";
}

.fa-masks-theater, .fa-theater-masks {
  --fa: "";
}

.fa-handshake-simple, .fa-handshake-alt {
  --fa: "";
}

.fa-jet-fighter, .fa-fighter-jet {
  --fa: "";
}

.fa-square-share-nodes, .fa-share-alt-square {
  --fa: "";
}

.fa-video, .fa-video-camera {
  --fa: "";
}

.fa-graduation-cap, .fa-mortar-board {
  --fa: "";
}

.fa-turn-up, .fa-level-up-alt {
  --fa: "";
}

.sr-only, .fa-sr-only, .sr-only-focusable:not(:focus), .fa-sr-only-focusable:not(:focus) {
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  padding: 0;
  position: absolute;
  overflow: hidden;
}

.fa.fa-envelope-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-remove, .fa.fa-close {
  --fa: "";
}

.fa.fa-trash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-clock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-play-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-repeat, .fa.fa-rotate-right {
  --fa: "";
}

.fa.fa-list-alt {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-picture-o, .fa.fa-photo, .fa.fa-image {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-pencil-square-o, .fa.fa-edit {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eye, .fa.fa-eye-slash {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bar-chart, .fa.fa-bar-chart-o {
  --fa: "";
}

.fa.fa-twitter-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thumbs-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-heart-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-linkedin-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-github-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lemon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bookmark-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-twitter, .fa.fa-facebook {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-f {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hdd-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-files-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-floppy-o, .fa.fa-save {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-navicon, .fa.fa-reorder {
  --fa: "";
}

.fa.fa-pinterest, .fa.fa-pinterest-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linkedin {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tachometer, .fa.fa-dashboard {
  --fa: "";
}

.fa.fa-comment-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comments-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lightbulb-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bell-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-text-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-building-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hospital-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mobile, .fa.fa-mobile-phone {
  --fa: "";
}

.fa.fa-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-smile-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-frown-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-meh-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-keyboard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flag-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-o, .fa.fa-star-half-empty, .fa.fa-star-half-full {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-chain-broken, .fa.fa-unlink {
  --fa: "";
}

.fa.fa-calendar-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-maxcdn, .fa.fa-html5, .fa.fa-css3 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-minus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-compass, .fa.fa-caret-square-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-up, .fa.fa-toggle-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-right, .fa.fa-toggle-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eur, .fa.fa-euro {
  --fa: "";
}

.fa.fa-usd, .fa.fa-dollar {
  --fa: "$";
}

.fa.fa-inr, .fa.fa-rupee {
  --fa: "";
}

.fa.fa-jpy, .fa.fa-cny, .fa.fa-rmb, .fa.fa-yen {
  --fa: "";
}

.fa.fa-rub, .fa.fa-ruble, .fa.fa-rouble {
  --fa: "";
}

.fa.fa-krw, .fa.fa-won {
  --fa: "";
}

.fa.fa-btc, .fa.fa-bitcoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube, .fa.fa-xing, .fa.fa-xing-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-play {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dropbox, .fa.fa-stack-overflow, .fa.fa-instagram, .fa.fa-flickr, .fa.fa-adn, .fa.fa-bitbucket, .fa.fa-bitbucket-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr, .fa.fa-tumblr-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-apple, .fa.fa-windows, .fa.fa-android, .fa.fa-linux, .fa.fa-dribbble, .fa.fa-skype, .fa.fa-foursquare, .fa.fa-trello, .fa.fa-gratipay, .fa.fa-gittip {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-sun-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-moon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vk, .fa.fa-weibo, .fa.fa-renren, .fa.fa-pagelines, .fa.fa-stack-exchange {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-left, .fa.fa-toggle-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dot-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vimeo-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-try, .fa.fa-turkish-lira {
  --fa: "";
}

.fa.fa-plus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slack, .fa.fa-wordpress, .fa.fa-openid {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-institution, .fa.fa-bank {
  --fa: "";
}

.fa.fa-yahoo, .fa.fa-google, .fa.fa-reddit, .fa.fa-reddit-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stumbleupon-circle, .fa.fa-stumbleupon, .fa.fa-delicious, .fa.fa-digg, .fa.fa-pied-piper-pp, .fa.fa-pied-piper-alt, .fa.fa-drupal, .fa.fa-joomla, .fa.fa-behance, .fa.fa-behance-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam, .fa.fa-steam-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-spotify, .fa.fa-deviantart, .fa.fa-soundcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-word-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-excel-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-powerpoint-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-image-o, .fa.fa-file-photo-o, .fa.fa-file-picture-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-archive-o, .fa.fa-file-zip-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-audio-o, .fa.fa-file-sound-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-video-o, .fa.fa-file-movie-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-code-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vine, .fa.fa-codepen, .fa.fa-jsfiddle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-life-bouy, .fa.fa-life-buoy, .fa.fa-life-saver, .fa.fa-support {
  --fa: "";
}

.fa.fa-rebel, .fa.fa-ra {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-resistance {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-empire, .fa.fa-ge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git, .fa.fa-hacker-news, .fa.fa-y-combinator-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tencent-weibo, .fa.fa-qq, .fa.fa-weixin, .fa.fa-wechat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-paper-plane-o, .fa.fa-send-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-thin {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-futbol-o, .fa.fa-soccer-ball-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slideshare, .fa.fa-twitch, .fa.fa-yelp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-paypal, .fa.fa-google-wallet, .fa.fa-cc-visa, .fa.fa-cc-mastercard, .fa.fa-cc-discover, .fa.fa-cc-amex, .fa.fa-cc-paypal, .fa.fa-cc-stripe {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lastfm, .fa.fa-lastfm-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ioxhost, .fa.fa-angellist {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-ils, .fa.fa-shekel, .fa.fa-sheqel {
  --fa: "";
}

.fa.fa-buysellads, .fa.fa-connectdevelop, .fa.fa-dashcube, .fa.fa-forumbee, .fa.fa-leanpub, .fa.fa-sellsy, .fa.fa-shirtsinbulk, .fa.fa-simplybuilt, .fa.fa-skyatlas {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-diamond {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-transgender, .fa.fa-intersex {
  --fa: "";
}

.fa.fa-facebook-official {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-p, .fa.fa-whatsapp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viacoin, .fa.fa-medium, .fa.fa-y-combinator, .fa.fa-yc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-optin-monster, .fa.fa-opencart, .fa.fa-expeditedssl {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-battery-4, .fa.fa-battery {
  --fa: "";
}

.fa.fa-object-group, .fa.fa-object-ungroup, .fa.fa-sticky-note-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-cc-jcb, .fa.fa-cc-diners-club {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hand-rock-o, .fa.fa-hand-grab-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-paper-o, .fa.fa-hand-stop-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-scissors-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-lizard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-spock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-pointer-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-peace-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-creative-commons, .fa.fa-gg, .fa.fa-gg-circle, .fa.fa-odnoklassniki, .fa.fa-odnoklassniki-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-get-pocket, .fa.fa-wikipedia-w, .fa.fa-safari, .fa.fa-chrome, .fa.fa-firefox, .fa.fa-opera, .fa.fa-internet-explorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-contao, .fa.fa-500px, .fa.fa-amazon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-minus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-times-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-check-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-commenting-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-houzz, .fa.fa-vimeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-black-tie, .fa.fa-fonticons, .fa.fa-reddit-alien, .fa.fa-edge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-codiepie, .fa.fa-modx, .fa.fa-fort-awesome, .fa.fa-usb, .fa.fa-product-hunt, .fa.fa-mixcloud, .fa.fa-scribd {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-stop-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bluetooth, .fa.fa-bluetooth-b, .fa.fa-gitlab, .fa.fa-wpbeginner, .fa.fa-wpforms, .fa.fa-envira, .fa.fa-wheelchair-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-question-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-deafness, .fa.fa-hard-of-hearing {
  --fa: "";
}

.fa.fa-glide, .fa.fa-glide-g {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo, .fa.fa-viadeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat, .fa.fa-snapchat-ghost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper, .fa.fa-first-order, .fa.fa-yoast, .fa.fa-themeisle, .fa.fa-google-plus-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-circle {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-font-awesome, .fa.fa-fa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-handshake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-book-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-card-o, .fa.fa-vcard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-id-card-o, .fa.fa-drivers-license-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-quora, .fa.fa-free-code-camp, .fa.fa-telegram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thermometer-4, .fa.fa-thermometer {
  --fa: "";
}

.fa.fa-bathtub, .fa.fa-s15 {
  --fa: "";
}

.fa.fa-window-maximize, .fa.fa-window-restore {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-window-close-o, .fa.fa-times-rectangle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bandcamp, .fa.fa-grav, .fa.fa-etsy, .fa.fa-imdb, .fa.fa-ravelry, .fa.fa-eercast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snowflake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-superpowers, .fa.fa-wpexplorer, .fa.fa-meetup {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-regular-400.woff2?v=896727") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-regular-400.ttf?v=262525") format("truetype");
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 900;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-solid-900.woff2?v=4a6591") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-solid-900.ttf?v=269f97") format("truetype");
}

@font-face {
  font-family: "Font Awesome 6 Brands";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-brands-400.woff2?v=cbcf42") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-brands-400.ttf?v=15d54d") format("truetype");
}

.fa-square-js, .fa-js-square {
  --fa: "";
}

.fa-square-reddit, .fa-reddit-square {
  --fa: "";
}

.fa-square-instagram, .fa-instagram-square {
  --fa: "";
}

.fa-square-hacker-news, .fa-hacker-news-square {
  --fa: "";
}

.fa-square-snapchat, .fa-snapchat-square {
  --fa: "";
}

.fa-square-font-awesome-stroke, .fa-font-awesome-alt {
  --fa: "";
}

.fa-square-viadeo, .fa-viadeo-square {
  --fa: "";
}

.fa-square-dribbble, .fa-dribbble-square {
  --fa: "";
}

.fa-square-twitter, .fa-twitter-square {
  --fa: "";
}

.fa-kickstarter, .fa-square-kickstarter {
  --fa: "";
}

.fa-square-youtube, .fa-youtube-square {
  --fa: "";
}

.fa-wpressr, .fa-rendact {
  --fa: "";
}

.fa-square-steam, .fa-steam-square {
  --fa: "";
}

.fa-square-vimeo, .fa-vimeo-square {
  --fa: "";
}

.fa-font-awesome, .fa-font-awesome-flag, .fa-font-awesome-logo-full {
  --fa: "";
}

.fa-square-github, .fa-github-square {
  --fa: "";
}

.fa-square-gitlab, .fa-gitlab-square {
  --fa: "";
}

.fa-square-odnoklassniki, .fa-odnoklassniki-square {
  --fa: "";
}

.fa-square-pinterest, .fa-pinterest-square {
  --fa: "";
}

.fa-square-google-plus, .fa-google-plus-square {
  --fa: "";
}

.fa-square-xing, .fa-xing-square {
  --fa: "";
}

.fa-42-group, .fa-innosoft {
  --fa: "";
}

.fa-square-pied-piper, .fa-pied-piper-square {
  --fa: "";
}

.fa-square-facebook, .fa-facebook-square {
  --fa: "";
}

.fa-square-lastfm, .fa-lastfm-square {
  --fa: "";
}

.fa-wirsindhandwerk, .fa-wsh {
  --fa: "";
}

.fa-snapchat, .fa-snapchat-ghost {
  --fa: "";
}

.fa-square-behance, .fa-behance-square {
  --fa: "";
}

.fa-square-git, .fa-git-square {
  --fa: "";
}

.fa-square-tumblr, .fa-tumblr-square {
  --fa: "";
}

.fa-telegram, .fa-telegram-plane {
  --fa: "";
}

.fa-square-whatsapp, .fa-whatsapp-square {
  --fa: "";
}

.fa-slack, .fa-slack-hash {
  --fa: "";
}

.fa-medium, .fa-medium-m {
  --fa: "";
}

.fa.fa-envelope-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-remove, .fa.fa-close {
  --fa: "";
}

.fa.fa-trash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-clock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-play-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-repeat, .fa.fa-rotate-right {
  --fa: "";
}

.fa.fa-list-alt {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-picture-o, .fa.fa-photo, .fa.fa-image {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-pencil-square-o, .fa.fa-edit {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eye, .fa.fa-eye-slash {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bar-chart, .fa.fa-bar-chart-o {
  --fa: "";
}

.fa.fa-twitter-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thumbs-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-heart-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-linkedin-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-github-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lemon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bookmark-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-twitter, .fa.fa-facebook {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-f {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hdd-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-files-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-floppy-o, .fa.fa-save {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-navicon, .fa.fa-reorder {
  --fa: "";
}

.fa.fa-pinterest, .fa.fa-pinterest-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linkedin {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tachometer, .fa.fa-dashboard {
  --fa: "";
}

.fa.fa-comment-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comments-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lightbulb-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bell-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-text-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-building-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hospital-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mobile, .fa.fa-mobile-phone {
  --fa: "";
}

.fa.fa-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-smile-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-frown-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-meh-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-keyboard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flag-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-o, .fa.fa-star-half-empty, .fa.fa-star-half-full {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-chain-broken, .fa.fa-unlink {
  --fa: "";
}

.fa.fa-calendar-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-maxcdn, .fa.fa-html5, .fa.fa-css3 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-minus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-compass, .fa.fa-caret-square-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-up, .fa.fa-toggle-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-right, .fa.fa-toggle-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eur, .fa.fa-euro {
  --fa: "";
}

.fa.fa-usd, .fa.fa-dollar {
  --fa: "$";
}

.fa.fa-inr, .fa.fa-rupee {
  --fa: "";
}

.fa.fa-jpy, .fa.fa-cny, .fa.fa-rmb, .fa.fa-yen {
  --fa: "";
}

.fa.fa-rub, .fa.fa-ruble, .fa.fa-rouble {
  --fa: "";
}

.fa.fa-krw, .fa.fa-won {
  --fa: "";
}

.fa.fa-btc, .fa.fa-bitcoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube, .fa.fa-xing, .fa.fa-xing-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-play {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dropbox, .fa.fa-stack-overflow, .fa.fa-instagram, .fa.fa-flickr, .fa.fa-adn, .fa.fa-bitbucket, .fa.fa-bitbucket-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr, .fa.fa-tumblr-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-apple, .fa.fa-windows, .fa.fa-android, .fa.fa-linux, .fa.fa-dribbble, .fa.fa-skype, .fa.fa-foursquare, .fa.fa-trello, .fa.fa-gratipay, .fa.fa-gittip {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-sun-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-moon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vk, .fa.fa-weibo, .fa.fa-renren, .fa.fa-pagelines, .fa.fa-stack-exchange {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-left, .fa.fa-toggle-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dot-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vimeo-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-try, .fa.fa-turkish-lira {
  --fa: "";
}

.fa.fa-plus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slack, .fa.fa-wordpress, .fa.fa-openid {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-institution, .fa.fa-bank {
  --fa: "";
}

.fa.fa-yahoo, .fa.fa-google, .fa.fa-reddit, .fa.fa-reddit-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stumbleupon-circle, .fa.fa-stumbleupon, .fa.fa-delicious, .fa.fa-digg, .fa.fa-pied-piper-pp, .fa.fa-pied-piper-alt, .fa.fa-drupal, .fa.fa-joomla, .fa.fa-behance, .fa.fa-behance-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam, .fa.fa-steam-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-spotify, .fa.fa-deviantart, .fa.fa-soundcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-word-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-excel-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-powerpoint-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-image-o, .fa.fa-file-photo-o, .fa.fa-file-picture-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-archive-o, .fa.fa-file-zip-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-audio-o, .fa.fa-file-sound-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-video-o, .fa.fa-file-movie-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-code-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vine, .fa.fa-codepen, .fa.fa-jsfiddle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-life-bouy, .fa.fa-life-buoy, .fa.fa-life-saver, .fa.fa-support {
  --fa: "";
}

.fa.fa-rebel, .fa.fa-ra {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-resistance {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-empire, .fa.fa-ge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git, .fa.fa-hacker-news, .fa.fa-y-combinator-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tencent-weibo, .fa.fa-qq, .fa.fa-weixin, .fa.fa-wechat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-paper-plane-o, .fa.fa-send-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-thin {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-futbol-o, .fa.fa-soccer-ball-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slideshare, .fa.fa-twitch, .fa.fa-yelp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-paypal, .fa.fa-google-wallet, .fa.fa-cc-visa, .fa.fa-cc-mastercard, .fa.fa-cc-discover, .fa.fa-cc-amex, .fa.fa-cc-paypal, .fa.fa-cc-stripe {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lastfm, .fa.fa-lastfm-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ioxhost, .fa.fa-angellist {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-ils, .fa.fa-shekel, .fa.fa-sheqel {
  --fa: "";
}

.fa.fa-buysellads, .fa.fa-connectdevelop, .fa.fa-dashcube, .fa.fa-forumbee, .fa.fa-leanpub, .fa.fa-sellsy, .fa.fa-shirtsinbulk, .fa.fa-simplybuilt, .fa.fa-skyatlas {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-diamond {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-transgender, .fa.fa-intersex {
  --fa: "";
}

.fa.fa-facebook-official {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-p, .fa.fa-whatsapp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viacoin, .fa.fa-medium, .fa.fa-y-combinator, .fa.fa-yc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-optin-monster, .fa.fa-opencart, .fa.fa-expeditedssl {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-battery-4, .fa.fa-battery {
  --fa: "";
}

.fa.fa-object-group, .fa.fa-object-ungroup, .fa.fa-sticky-note-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-cc-jcb, .fa.fa-cc-diners-club {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hand-rock-o, .fa.fa-hand-grab-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-paper-o, .fa.fa-hand-stop-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-scissors-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-lizard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-spock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-pointer-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-peace-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-creative-commons, .fa.fa-gg, .fa.fa-gg-circle, .fa.fa-odnoklassniki, .fa.fa-odnoklassniki-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-get-pocket, .fa.fa-wikipedia-w, .fa.fa-safari, .fa.fa-chrome, .fa.fa-firefox, .fa.fa-opera, .fa.fa-internet-explorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-contao, .fa.fa-500px, .fa.fa-amazon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-minus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-times-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-check-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-commenting-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-houzz, .fa.fa-vimeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-black-tie, .fa.fa-fonticons, .fa.fa-reddit-alien, .fa.fa-edge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-codiepie, .fa.fa-modx, .fa.fa-fort-awesome, .fa.fa-usb, .fa.fa-product-hunt, .fa.fa-mixcloud, .fa.fa-scribd {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-stop-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bluetooth, .fa.fa-bluetooth-b, .fa.fa-gitlab, .fa.fa-wpbeginner, .fa.fa-wpforms, .fa.fa-envira, .fa.fa-wheelchair-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-question-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-deafness, .fa.fa-hard-of-hearing {
  --fa: "";
}

.fa.fa-glide, .fa.fa-glide-g {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo, .fa.fa-viadeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat, .fa.fa-snapchat-ghost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper, .fa.fa-first-order, .fa.fa-yoast, .fa.fa-themeisle, .fa.fa-google-plus-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-circle {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-font-awesome, .fa.fa-fa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-handshake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-book-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-card-o, .fa.fa-vcard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-id-card-o, .fa.fa-drivers-license-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-quora, .fa.fa-free-code-camp, .fa.fa-telegram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thermometer-4, .fa.fa-thermometer {
  --fa: "";
}

.fa.fa-bathtub, .fa.fa-s15 {
  --fa: "";
}

.fa.fa-window-maximize, .fa.fa-window-restore {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-window-close-o, .fa.fa-times-rectangle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bandcamp, .fa.fa-grav, .fa.fa-etsy, .fa.fa-imdb, .fa.fa-ravelry, .fa.fa-eercast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snowflake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-superpowers, .fa.fa-wpexplorer, .fa.fa-meetup {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.icon-address-book:before, .icon-address:before {
  content: "";
}

.icon-bookmark-2:before, .icon-bookmark:before {
  content: "";
}

.icon-box-add:before, .icon-box-remove:before {
  content: "";
}

.icon-calendar-check:before, .icon-calendar-2:before {
  content: "";
}

.icon-calendar-alt:before, .icon-calendar:before {
  content: "";
}

.icon-checkbox-partial:before, .icon-checkbox-unchecked:before {
  content: "";
}

.icon-checkmark-2:before, .icon-checkmark-circle:before {
  content: "";
}

.icon-cloud-download-alt:before, .icon-cloud-download:before {
  content: "";
}

.icon-credit-2:before, .icon-credit:before {
  content: "";
}

.icon-envelope-opened:before, .icon-envelope:before {
  content: "";
}

.icon-eye-2:before, .icon-eye-blocked:before, .icon-eye-close:before {
  content: "";
}

.icon-flag-3:before, .icon-flag:before {
  content: "";
}

.icon-folder-close:before, .icon-folder-minus:before {
  content: "";
}

.icon-folder-remove:before, .icon-folder:before {
  content: "";
}

.icon-grid-2:before, .icon-grid-view-2:before {
  content: "";
}

.icon-grid-view:before, .icon-grid:before {
  content: "";
}

.icon-heart-2:before, .icon-heart:before {
  content: "";
}

.icon-home-2:before, .icon-home:before {
  content: "";
}

.icon-image:before, .icon-images:before {
  content: "";
}

.icon-info-2:before, .icon-info-circle:before {
  content: "";
}

.icon-list-view:before, .icon-list:before {
  content: "";
}

.icon-lock:before, .icon-locked:before {
  content: "";
}

.icon-mail-2:before, .icon-mail:before {
  content: "";
}

.icon-minus-sign:before, .icon-minus:before {
  content: "";
}

.icon-notification-2:before, .icon-notification-circle:before {
  content: "";
}

.icon-paint-brush:before, .icon-palette:before {
  content: "";
}

.icon-pencil-2:before, .icon-pencil-alt:before {
  content: "";
}

.icon-picture:before, .icon-pictures:before {
  content: "";
}

.icon-print:before, .icon-printer:before {
  content: "";
}

.icon-puzzle-piece:before, .icon-puzzle:before {
  content: "";
}

.icon-question-2:before, .icon-question-circle:before {
  content: "";
}

.icon-question-sign:before, .icon-question:before {
  content: "?";
}

.icon-quote:before, .icon-quotes-left:before {
  content: "";
}

.icon-share-alt:before, .icon-share:before {
  content: "";
}

.icon-shield-alt:before, .icon-shield:before {
  content: "";
}

.icon-smiley-2:before, .icon-smiley-happy-2:before, .icon-smiley-happy:before, .icon-smiley-neutral-2:before, .icon-smiley-neutral:before {
  content: "";
}

.icon-smiley-sad-2:before, .icon-smiley-sad:before {
  content: "";
}

.icon-star-empty:before, .icon-star:before {
  content: "";
}

.icon-tag-2:before, .icon-tag:before {
  content: "";
}

.icon-tags-2:before, .icon-tags:before {
  content: "";
}

.icon-undo-2:before, .icon-undo:before {
  content: "";
}

.icon-universal:before, .icon-universal-access:before {
  content: "";
}

html[dir="rtl"] .float-right {
  float: left;
}

input .icon-check-square:before {
  color: var(--icon-check-square-bg);
  background: var(--icon-check-square-color);
}

input .icon-square:before {
  color: var(--icon-square-bg);
  background: var(--icon-square-color);
}

.fa {
  font-family: var(--fa-style-family, "Font Awesome 6 Free");
  font-weight: var(--fa-style, 900);
}

.fas, [class^="icon-"], [class*=" icon-"], .far, .fab, .icon-joomla, .fa-solid, .fa-regular, .fa-brands, .fa {
  -moz-osx-font-smoothing: grayscale;
  -webkit-font-smoothing: antialiased;
  display: var(--fa-display, inline-block);
  font-variant: normal;
  text-rendering: auto;
  font-style: normal;
  line-height: 1;
}

.fas:before, [class^="icon-"]:before, [class*=" icon-"]:before, .far:before, .fab:before, .icon-joomla:before, .fa-solid:before, .fa-regular:before, .fa-brands:before, .fa:before {
  content: var(--fa);
}

.fa-classic, .fas, [class^="icon-"], [class*=" icon-"], .fa-solid, .far, .fa-regular {
  font-family: "Font Awesome 6 Free";
}

.fa-brands, .fab, .icon-joomla {
  font-family: "Font Awesome 6 Brands";
}

[class^="icon-"], [class*=" icon-"], .icon-joomla {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  font-variant: normal;
  font-style: normal;
  font-weight: normal;
  line-height: 1;
  display: inline-block;
}

.fa-1x {
  font-size: 1em;
}

.fa-2x {
  font-size: 2em;
}

.fa-3x {
  font-size: 3em;
}

.fa-4x {
  font-size: 4em;
}

.fa-5x {
  font-size: 5em;
}

.fa-6x {
  font-size: 6em;
}

.fa-7x {
  font-size: 7em;
}

.fa-8x {
  font-size: 8em;
}

.fa-9x {
  font-size: 9em;
}

.fa-10x {
  font-size: 10em;
}

.fa-2xs {
  vertical-align: .225em;
  font-size: .625em;
  line-height: .1em;
}

.fa-xs, .icon-xs {
  vertical-align: .125em;
  font-size: .75em;
  line-height: .0833333em;
}

.fa-sm, .icon-sm {
  vertical-align: .0535714em;
  font-size: .875em;
  line-height: .0714286em;
}

.fa-lg, .icon-lg {
  vertical-align: -.075em;
  font-size: 1.25em;
  line-height: .05em;
}

.fa-xl {
  vertical-align: -.125em;
  font-size: 1.5em;
  line-height: .0416667em;
}

.fa-2xl {
  vertical-align: -.1875em;
  font-size: 2em;
  line-height: .03125em;
}

.fa-fw, .icon-fw {
  text-align: center;
  width: 1.25em;
}

.fa-ul, .icon-ul {
  margin-left: var(--fa-li-margin, 2.5em);
  padding-left: 0;
  list-style-type: none;
}

.fa-ul > li, .icon-ul > li {
  position: relative;
}

.fa-li, .icon-li {
  left: calc(-1 * var(--fa-li-width, 2em));
  text-align: center;
  width: var(--fa-li-width, 2em);
  line-height: inherit;
  position: absolute;
}

.fa-border, .icon-border {
  border-color: var(--fa-border-color, #eee);
  border-radius: var(--fa-border-radius, .1em);
  border-style: var(--fa-border-style, solid);
  border-width: var(--fa-border-width, .08em);
  padding: var(--fa-border-padding, .2em .25em .15em);
}

.fa-pull-left, .icon-pull-left {
  float: left;
  margin-right: var(--fa-pull-margin, .3em);
}

.fa-pull-right, .icon-pull-right {
  float: right;
  margin-left: var(--fa-pull-margin, .3em);
}

.fa-beat {
  animation-name: fa-beat;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-bounce {
  animation-name: fa-bounce;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(.28, .84, .42, 1));
}

.fa-fade {
  animation-name: fa-fade;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(.4, 0, .6, 1));
}

.fa-beat-fade {
  animation-name: fa-beat-fade;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(.4, 0, .6, 1));
}

.fa-flip {
  animation-name: fa-flip;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-shake {
  animation-name: fa-shake;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin, .icon-spin {
  animation-name: fa-spin;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 2s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin-reverse {
  --fa-animation-direction: reverse;
}

.fa-pulse, .icon-pulse, .fa-spin-pulse {
  animation-name: fa-spin;
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, steps(8));
}

@media (prefers-reduced-motion: reduce) {
  .fa-beat, .fa-bounce, .fa-fade, .fa-beat-fade, .fa-flip, .fa-pulse, .icon-pulse, .fa-shake, .fa-spin, .icon-spin, .fa-spin-pulse {
    transition-duration: 0s;
    transition-delay: 0s;
    animation-duration: 1ms;
    animation-iteration-count: 1;
    animation-delay: -1ms;
  }
}

.fa-rotate-90, .icon-rotate-90 {
  transform: rotate(90deg);
}

.fa-rotate-180, .icon-rotate-180 {
  transform: rotate(180deg);
}

.fa-rotate-270, .icon-rotate-270 {
  transform: rotate(270deg);
}

.fa-flip-horizontal, .icon-flip-horizontal {
  transform: scale(-1, 1);
}

.fa-flip-vertical, .icon-flip-vertical {
  transform: scale(1, -1);
}

.fa-flip-both, .icon-flip-both, .fa-flip-horizontal.fa-flip-vertical, .fa-flip-vertical.icon-flip-horizontal, .fa-flip-horizontal.icon-flip-vertical, .icon-flip-horizontal.icon-flip-vertical {
  transform: scale(-1);
}

.fa-rotate-by {
  transform: rotate(var(--fa-rotate-angle, 0));
}

.fa-stack, .icon-stack {
  vertical-align: middle;
  width: 2.5em;
  height: 2em;
  line-height: 2em;
  display: inline-block;
  position: relative;
}

.fa-stack-1x, .icon-stack-1x, .fa-stack-2x, .icon-stack-2x {
  text-align: center;
  width: 100%;
  z-index: var(--fa-stack-z-index, auto);
  position: absolute;
  left: 0;
}

.fa-stack-1x, .icon-stack-1x {
  line-height: inherit;
}

.fa-stack-2x, .icon-stack-2x {
  font-size: 2em;
}

.fa-inverse, .icon-inverse {
  color: var(--fa-inverse, #fff);
}

.fa-0 {
  --fa: "0";
}

.fa-1 {
  --fa: "1";
}

.fa-2 {
  --fa: "2";
}

.fa-3 {
  --fa: "3";
}

.fa-4 {
  --fa: "4";
}

.fa-5 {
  --fa: "5";
}

.fa-6 {
  --fa: "6";
}

.fa-7 {
  --fa: "7";
}

.fa-8 {
  --fa: "8";
}

.fa-9 {
  --fa: "9";
}

.fa-fill-drip {
  --fa: "";
}

.fa-arrows-to-circle {
  --fa: "";
}

.fa-circle-chevron-right, .fa-chevron-circle-right {
  --fa: "";
}

.fa-at {
  --fa: "@";
}

.fa-trash-can, .fa-trash-alt {
  --fa: "";
}

.fa-text-height {
  --fa: "";
}

.fa-user-xmark, .fa-user-times {
  --fa: "";
}

.fa-stethoscope {
  --fa: "";
}

.fa-message, .fa-comment-alt {
  --fa: "";
}

.fa-info {
  --fa: "";
}

.fa-down-left-and-up-right-to-center, .fa-compress-alt {
  --fa: "";
}

.fa-explosion {
  --fa: "";
}

.fa-file-lines, .fa-file-alt, .fa-file-text {
  --fa: "";
}

.fa-wave-square {
  --fa: "";
}

.fa-ring {
  --fa: "";
}

.fa-building-un {
  --fa: "";
}

.fa-dice-three {
  --fa: "";
}

.fa-calendar-days, .fa-calendar-alt {
  --fa: "";
}

.fa-anchor-circle-check {
  --fa: "";
}

.fa-building-circle-arrow-right {
  --fa: "";
}

.fa-volleyball, .fa-volleyball-ball {
  --fa: "";
}

.fa-arrows-up-to-line {
  --fa: "";
}

.fa-sort-down, .fa-sort-desc {
  --fa: "";
}

.fa-circle-minus, .fa-minus-circle {
  --fa: "";
}

.fa-door-open {
  --fa: "";
}

.fa-right-from-bracket, .fa-sign-out-alt {
  --fa: "";
}

.fa-atom {
  --fa: "";
}

.fa-soap {
  --fa: "";
}

.fa-icons, .fa-heart-music-camera-bolt {
  --fa: "";
}

.fa-microphone-lines-slash, .fa-microphone-alt-slash {
  --fa: "";
}

.fa-bridge-circle-check {
  --fa: "";
}

.fa-pump-medical {
  --fa: "";
}

.fa-fingerprint {
  --fa: "";
}

.fa-hand-point-right {
  --fa: "";
}

.fa-magnifying-glass-location, .fa-search-location {
  --fa: "";
}

.fa-forward-step, .fa-step-forward {
  --fa: "";
}

.fa-face-smile-beam, .fa-smile-beam {
  --fa: "";
}

.fa-flag-checkered {
  --fa: "";
}

.fa-football, .fa-football-ball {
  --fa: "";
}

.fa-school-circle-exclamation {
  --fa: "";
}

.fa-crop {
  --fa: "";
}

.fa-angles-down, .fa-angle-double-down {
  --fa: "";
}

.fa-users-rectangle {
  --fa: "";
}

.fa-people-roof {
  --fa: "";
}

.fa-people-line {
  --fa: "";
}

.fa-beer-mug-empty, .fa-beer {
  --fa: "";
}

.fa-diagram-predecessor {
  --fa: "";
}

.fa-arrow-up-long, .fa-long-arrow-up {
  --fa: "";
}

.fa-fire-flame-simple, .fa-burn {
  --fa: "";
}

.fa-person, .fa-male {
  --fa: "";
}

.fa-laptop {
  --fa: "";
}

.fa-file-csv {
  --fa: "";
}

.fa-menorah {
  --fa: "";
}

.fa-truck-plane {
  --fa: "";
}

.fa-record-vinyl {
  --fa: "";
}

.fa-face-grin-stars, .fa-grin-stars {
  --fa: "";
}

.fa-bong {
  --fa: "";
}

.fa-spaghetti-monster-flying, .fa-pastafarianism {
  --fa: "";
}

.fa-arrow-down-up-across-line {
  --fa: "";
}

.fa-spoon, .fa-utensil-spoon {
  --fa: "";
}

.fa-jar-wheat {
  --fa: "";
}

.fa-envelopes-bulk, .fa-mail-bulk {
  --fa: "";
}

.fa-file-circle-exclamation {
  --fa: "";
}

.fa-circle-h, .fa-hospital-symbol {
  --fa: "";
}

.fa-pager {
  --fa: "";
}

.fa-address-book, .fa-contact-book {
  --fa: "";
}

.fa-strikethrough {
  --fa: "";
}

.fa-k {
  --fa: "K";
}

.fa-landmark-flag {
  --fa: "";
}

.fa-pencil, .fa-pencil-alt {
  --fa: "";
}

.fa-backward {
  --fa: "";
}

.fa-caret-right {
  --fa: "";
}

.fa-comments {
  --fa: "";
}

.fa-paste, .fa-file-clipboard {
  --fa: "";
}

.fa-code-pull-request {
  --fa: "";
}

.fa-clipboard-list {
  --fa: "";
}

.fa-truck-ramp-box, .fa-truck-loading {
  --fa: "";
}

.fa-user-check {
  --fa: "";
}

.fa-vial-virus {
  --fa: "";
}

.fa-sheet-plastic {
  --fa: "";
}

.fa-blog {
  --fa: "";
}

.fa-user-ninja {
  --fa: "";
}

.fa-person-arrow-up-from-line {
  --fa: "";
}

.fa-scroll-torah, .fa-torah {
  --fa: "";
}

.fa-broom-ball, .fa-quidditch, .fa-quidditch-broom-ball {
  --fa: "";
}

.fa-toggle-off {
  --fa: "";
}

.fa-box-archive, .fa-archive {
  --fa: "";
}

.fa-person-drowning {
  --fa: "";
}

.fa-arrow-down-9-1, .fa-sort-numeric-desc, .fa-sort-numeric-down-alt {
  --fa: "";
}

.fa-face-grin-tongue-squint, .fa-grin-tongue-squint {
  --fa: "";
}

.fa-spray-can {
  --fa: "";
}

.fa-truck-monster {
  --fa: "";
}

.fa-w {
  --fa: "W";
}

.fa-earth-africa, .fa-globe-africa {
  --fa: "";
}

.fa-rainbow {
  --fa: "";
}

.fa-circle-notch {
  --fa: "";
}

.fa-tablet-screen-button, .fa-tablet-alt {
  --fa: "";
}

.fa-paw {
  --fa: "";
}

.fa-cloud {
  --fa: "";
}

.fa-trowel-bricks {
  --fa: "";
}

.fa-face-flushed, .fa-flushed {
  --fa: "";
}

.fa-hospital-user {
  --fa: "";
}

.fa-tent-arrow-left-right {
  --fa: "";
}

.fa-gavel, .fa-legal {
  --fa: "";
}

.fa-binoculars {
  --fa: "";
}

.fa-microphone-slash {
  --fa: "";
}

.fa-box-tissue {
  --fa: "";
}

.fa-motorcycle {
  --fa: "";
}

.fa-bell-concierge, .fa-concierge-bell {
  --fa: "";
}

.fa-pen-ruler, .fa-pencil-ruler {
  --fa: "";
}

.fa-people-arrows, .fa-people-arrows-left-right {
  --fa: "";
}

.fa-mars-and-venus-burst {
  --fa: "";
}

.fa-square-caret-right, .fa-caret-square-right {
  --fa: "";
}

.fa-scissors, .fa-cut {
  --fa: "";
}

.fa-sun-plant-wilt {
  --fa: "";
}

.fa-toilets-portable {
  --fa: "";
}

.fa-hockey-puck {
  --fa: "";
}

.fa-table {
  --fa: "";
}

.fa-magnifying-glass-arrow-right {
  --fa: "";
}

.fa-tachograph-digital, .fa-digital-tachograph {
  --fa: "";
}

.fa-users-slash {
  --fa: "";
}

.fa-clover {
  --fa: "";
}

.fa-reply, .fa-mail-reply {
  --fa: "";
}

.fa-star-and-crescent {
  --fa: "";
}

.fa-house-fire {
  --fa: "";
}

.fa-square-minus, .fa-minus-square {
  --fa: "";
}

.fa-helicopter {
  --fa: "";
}

.fa-compass {
  --fa: "";
}

.fa-square-caret-down, .fa-caret-square-down {
  --fa: "";
}

.fa-file-circle-question {
  --fa: "";
}

.fa-laptop-code {
  --fa: "";
}

.fa-swatchbook {
  --fa: "";
}

.fa-prescription-bottle {
  --fa: "";
}

.fa-bars, .fa-navicon {
  --fa: "";
}

.fa-people-group {
  --fa: "";
}

.fa-hourglass-end, .fa-hourglass-3 {
  --fa: "";
}

.fa-heart-crack, .fa-heart-broken {
  --fa: "";
}

.fa-square-up-right, .fa-external-link-square-alt {
  --fa: "";
}

.fa-face-kiss-beam, .fa-kiss-beam {
  --fa: "";
}

.fa-film {
  --fa: "";
}

.fa-ruler-horizontal {
  --fa: "";
}

.fa-people-robbery {
  --fa: "";
}

.fa-lightbulb {
  --fa: "";
}

.fa-caret-left {
  --fa: "";
}

.fa-circle-exclamation, .fa-exclamation-circle {
  --fa: "";
}

.fa-school-circle-xmark {
  --fa: "";
}

.fa-arrow-right-from-bracket, .fa-sign-out {
  --fa: "";
}

.fa-circle-chevron-down, .fa-chevron-circle-down {
  --fa: "";
}

.fa-unlock-keyhole, .fa-unlock-alt {
  --fa: "";
}

.fa-cloud-showers-heavy {
  --fa: "";
}

.fa-headphones-simple, .fa-headphones-alt {
  --fa: "";
}

.fa-sitemap {
  --fa: "";
}

.fa-circle-dollar-to-slot, .fa-donate {
  --fa: "";
}

.fa-memory {
  --fa: "";
}

.fa-road-spikes {
  --fa: "";
}

.fa-fire-burner {
  --fa: "";
}

.fa-flag {
  --fa: "";
}

.fa-hanukiah {
  --fa: "";
}

.fa-feather {
  --fa: "";
}

.fa-volume-low, .fa-volume-down {
  --fa: "";
}

.fa-comment-slash {
  --fa: "";
}

.fa-cloud-sun-rain {
  --fa: "";
}

.fa-compress {
  --fa: "";
}

.fa-wheat-awn, .fa-wheat-alt {
  --fa: "";
}

.fa-ankh {
  --fa: "";
}

.fa-hands-holding-child {
  --fa: "";
}

.fa-asterisk {
  --fa: "*";
}

.fa-square-check, .fa-check-square {
  --fa: "";
}

.fa-peseta-sign {
  --fa: "";
}

.fa-heading, .fa-header {
  --fa: "";
}

.fa-ghost {
  --fa: "";
}

.fa-list, .fa-list-squares {
  --fa: "";
}

.fa-square-phone-flip, .fa-phone-square-alt {
  --fa: "";
}

.fa-cart-plus {
  --fa: "";
}

.fa-gamepad {
  --fa: "";
}

.fa-circle-dot, .fa-dot-circle {
  --fa: "";
}

.fa-face-dizzy, .fa-dizzy {
  --fa: "";
}

.fa-egg {
  --fa: "";
}

.fa-house-medical-circle-xmark {
  --fa: "";
}

.fa-campground {
  --fa: "";
}

.fa-folder-plus {
  --fa: "";
}

.fa-futbol, .fa-futbol-ball, .fa-soccer-ball {
  --fa: "";
}

.fa-paintbrush, .fa-paint-brush {
  --fa: "";
}

.fa-lock {
  --fa: "";
}

.fa-gas-pump {
  --fa: "";
}

.fa-hot-tub-person, .fa-hot-tub {
  --fa: "";
}

.fa-map-location, .fa-map-marked {
  --fa: "";
}

.fa-house-flood-water {
  --fa: "";
}

.fa-tree {
  --fa: "";
}

.fa-bridge-lock {
  --fa: "";
}

.fa-sack-dollar {
  --fa: "";
}

.fa-pen-to-square, .fa-edit {
  --fa: "";
}

.fa-car-side {
  --fa: "";
}

.fa-share-nodes, .fa-share-alt {
  --fa: "";
}

.fa-heart-circle-minus {
  --fa: "";
}

.fa-hourglass-half, .fa-hourglass-2 {
  --fa: "";
}

.fa-microscope {
  --fa: "";
}

.fa-sink {
  --fa: "";
}

.fa-bag-shopping, .fa-shopping-bag {
  --fa: "";
}

.fa-arrow-down-z-a, .fa-sort-alpha-desc, .fa-sort-alpha-down-alt {
  --fa: "";
}

.fa-mitten {
  --fa: "";
}

.fa-person-rays {
  --fa: "";
}

.fa-users {
  --fa: "";
}

.fa-eye-slash {
  --fa: "";
}

.fa-flask-vial {
  --fa: "";
}

.fa-hand, .fa-hand-paper {
  --fa: "";
}

.fa-om {
  --fa: "";
}

.fa-worm {
  --fa: "";
}

.fa-house-circle-xmark {
  --fa: "";
}

.fa-plug {
  --fa: "";
}

.fa-chevron-up {
  --fa: "";
}

.fa-hand-spock {
  --fa: "";
}

.fa-stopwatch {
  --fa: "";
}

.fa-face-kiss, .fa-kiss {
  --fa: "";
}

.fa-bridge-circle-xmark {
  --fa: "";
}

.fa-face-grin-tongue, .fa-grin-tongue {
  --fa: "";
}

.fa-chess-bishop {
  --fa: "";
}

.fa-face-grin-wink, .fa-grin-wink {
  --fa: "";
}

.fa-ear-deaf, .fa-deaf, .fa-deafness, .fa-hard-of-hearing {
  --fa: "";
}

.fa-road-circle-check {
  --fa: "";
}

.fa-dice-five {
  --fa: "";
}

.fa-square-rss, .fa-rss-square {
  --fa: "";
}

.fa-land-mine-on {
  --fa: "";
}

.fa-i-cursor {
  --fa: "";
}

.fa-stamp {
  --fa: "";
}

.fa-stairs {
  --fa: "";
}

.fa-i {
  --fa: "I";
}

.fa-hryvnia-sign, .fa-hryvnia {
  --fa: "";
}

.fa-pills {
  --fa: "";
}

.fa-face-grin-wide, .fa-grin-alt {
  --fa: "";
}

.fa-tooth {
  --fa: "";
}

.fa-v {
  --fa: "V";
}

.fa-bangladeshi-taka-sign {
  --fa: "";
}

.fa-bicycle {
  --fa: "";
}

.fa-staff-snake, .fa-rod-asclepius, .fa-rod-snake, .fa-staff-aesculapius {
  --fa: "";
}

.fa-head-side-cough-slash {
  --fa: "";
}

.fa-truck-medical, .fa-ambulance {
  --fa: "";
}

.fa-wheat-awn-circle-exclamation {
  --fa: "";
}

.fa-snowman {
  --fa: "";
}

.fa-mortar-pestle {
  --fa: "";
}

.fa-road-barrier {
  --fa: "";
}

.fa-school {
  --fa: "";
}

.fa-igloo {
  --fa: "";
}

.fa-joint {
  --fa: "";
}

.fa-angle-right {
  --fa: "";
}

.fa-horse {
  --fa: "";
}

.fa-q {
  --fa: "Q";
}

.fa-g {
  --fa: "G";
}

.fa-notes-medical {
  --fa: "";
}

.fa-temperature-half, .fa-temperature-2, .fa-thermometer-2, .fa-thermometer-half {
  --fa: "";
}

.fa-dong-sign {
  --fa: "";
}

.fa-capsules {
  --fa: "";
}

.fa-poo-storm, .fa-poo-bolt {
  --fa: "";
}

.fa-face-frown-open, .fa-frown-open {
  --fa: "";
}

.fa-hand-point-up {
  --fa: "";
}

.fa-money-bill {
  --fa: "";
}

.fa-bookmark {
  --fa: "";
}

.fa-align-justify {
  --fa: "";
}

.fa-umbrella-beach {
  --fa: "";
}

.fa-helmet-un {
  --fa: "";
}

.fa-bullseye {
  --fa: "";
}

.fa-bacon {
  --fa: "";
}

.fa-hand-point-down {
  --fa: "";
}

.fa-arrow-up-from-bracket {
  --fa: "";
}

.fa-folder, .fa-folder-blank {
  --fa: "";
}

.fa-file-waveform, .fa-file-medical-alt {
  --fa: "";
}

.fa-radiation {
  --fa: "";
}

.fa-chart-simple {
  --fa: "";
}

.fa-mars-stroke {
  --fa: "";
}

.fa-vial {
  --fa: "";
}

.fa-gauge, .fa-dashboard, .fa-gauge-med, .fa-tachometer-alt-average {
  --fa: "";
}

.fa-wand-magic-sparkles, .fa-magic-wand-sparkles {
  --fa: "";
}

.fa-e {
  --fa: "E";
}

.fa-pen-clip, .fa-pen-alt {
  --fa: "";
}

.fa-bridge-circle-exclamation {
  --fa: "";
}

.fa-user {
  --fa: "";
}

.fa-school-circle-check {
  --fa: "";
}

.fa-dumpster {
  --fa: "";
}

.fa-van-shuttle, .fa-shuttle-van {
  --fa: "";
}

.fa-building-user {
  --fa: "";
}

.fa-square-caret-left, .fa-caret-square-left {
  --fa: "";
}

.fa-highlighter {
  --fa: "";
}

.fa-key {
  --fa: "";
}

.fa-bullhorn {
  --fa: "";
}

.fa-globe {
  --fa: "";
}

.fa-synagogue {
  --fa: "";
}

.fa-person-half-dress {
  --fa: "";
}

.fa-road-bridge {
  --fa: "";
}

.fa-location-arrow {
  --fa: "";
}

.fa-c {
  --fa: "C";
}

.fa-tablet-button {
  --fa: "";
}

.fa-building-lock {
  --fa: "";
}

.fa-pizza-slice {
  --fa: "";
}

.fa-money-bill-wave {
  --fa: "";
}

.fa-chart-area, .fa-area-chart {
  --fa: "";
}

.fa-house-flag {
  --fa: "";
}

.fa-person-circle-minus {
  --fa: "";
}

.fa-ban, .fa-cancel {
  --fa: "";
}

.fa-camera-rotate {
  --fa: "";
}

.fa-spray-can-sparkles, .fa-air-freshener {
  --fa: "";
}

.fa-star {
  --fa: "";
}

.fa-repeat {
  --fa: "";
}

.fa-cross {
  --fa: "";
}

.fa-box {
  --fa: "";
}

.fa-venus-mars {
  --fa: "";
}

.fa-arrow-pointer, .fa-mouse-pointer {
  --fa: "";
}

.fa-maximize, .fa-expand-arrows-alt {
  --fa: "";
}

.fa-charging-station {
  --fa: "";
}

.fa-shapes, .fa-triangle-circle-square {
  --fa: "";
}

.fa-shuffle, .fa-random {
  --fa: "";
}

.fa-person-running, .fa-running {
  --fa: "";
}

.fa-mobile-retro {
  --fa: "";
}

.fa-grip-lines-vertical {
  --fa: "";
}

.fa-spider {
  --fa: "";
}

.fa-hands-bound {
  --fa: "";
}

.fa-file-invoice-dollar {
  --fa: "";
}

.fa-plane-circle-exclamation {
  --fa: "";
}

.fa-x-ray {
  --fa: "";
}

.fa-spell-check {
  --fa: "";
}

.fa-slash {
  --fa: "";
}

.fa-computer-mouse, .fa-mouse {
  --fa: "";
}

.fa-arrow-right-to-bracket, .fa-sign-in {
  --fa: "";
}

.fa-shop-slash, .fa-store-alt-slash {
  --fa: "";
}

.fa-server {
  --fa: "";
}

.fa-virus-covid-slash {
  --fa: "";
}

.fa-shop-lock {
  --fa: "";
}

.fa-hourglass-start, .fa-hourglass-1 {
  --fa: "";
}

.fa-blender-phone {
  --fa: "";
}

.fa-building-wheat {
  --fa: "";
}

.fa-person-breastfeeding {
  --fa: "";
}

.fa-right-to-bracket, .fa-sign-in-alt {
  --fa: "";
}

.fa-venus {
  --fa: "";
}

.fa-passport {
  --fa: "";
}

.fa-thumbtack-slash, .fa-thumb-tack-slash {
  --fa: "";
}

.fa-heart-pulse, .fa-heartbeat {
  --fa: "";
}

.fa-people-carry-box, .fa-people-carry {
  --fa: "";
}

.fa-temperature-high {
  --fa: "";
}

.fa-microchip {
  --fa: "";
}

.fa-crown {
  --fa: "";
}

.fa-weight-hanging {
  --fa: "";
}

.fa-xmarks-lines {
  --fa: "";
}

.fa-file-prescription {
  --fa: "";
}

.fa-weight-scale, .fa-weight {
  --fa: "";
}

.fa-user-group, .fa-user-friends {
  --fa: "";
}

.fa-arrow-up-a-z, .fa-sort-alpha-up {
  --fa: "";
}

.fa-chess-knight {
  --fa: "";
}

.fa-face-laugh-squint, .fa-laugh-squint {
  --fa: "";
}

.fa-wheelchair {
  --fa: "";
}

.fa-circle-arrow-up, .fa-arrow-circle-up {
  --fa: "";
}

.fa-toggle-on {
  --fa: "";
}

.fa-person-walking, .fa-walking {
  --fa: "";
}

.fa-l {
  --fa: "L";
}

.fa-fire {
  --fa: "";
}

.fa-bed-pulse, .fa-procedures {
  --fa: "";
}

.fa-shuttle-space, .fa-space-shuttle {
  --fa: "";
}

.fa-face-laugh, .fa-laugh {
  --fa: "";
}

.fa-folder-open {
  --fa: "";
}

.fa-heart-circle-plus {
  --fa: "";
}

.fa-code-fork {
  --fa: "";
}

.fa-city {
  --fa: "";
}

.fa-microphone-lines, .fa-microphone-alt {
  --fa: "";
}

.fa-pepper-hot {
  --fa: "";
}

.fa-unlock {
  --fa: "";
}

.fa-colon-sign {
  --fa: "";
}

.fa-headset {
  --fa: "";
}

.fa-store-slash {
  --fa: "";
}

.fa-road-circle-xmark {
  --fa: "";
}

.fa-user-minus {
  --fa: "";
}

.fa-mars-stroke-up, .fa-mars-stroke-v {
  --fa: "";
}

.fa-champagne-glasses, .fa-glass-cheers {
  --fa: "";
}

.fa-clipboard {
  --fa: "";
}

.fa-house-circle-exclamation {
  --fa: "";
}

.fa-file-arrow-up, .fa-file-upload {
  --fa: "";
}

.fa-wifi, .fa-wifi-3, .fa-wifi-strong {
  --fa: "";
}

.fa-bath, .fa-bathtub {
  --fa: "";
}

.fa-underline {
  --fa: "";
}

.fa-user-pen, .fa-user-edit {
  --fa: "";
}

.fa-signature {
  --fa: "";
}

.fa-stroopwafel {
  --fa: "";
}

.fa-bold {
  --fa: "";
}

.fa-anchor-lock {
  --fa: "";
}

.fa-building-ngo {
  --fa: "";
}

.fa-manat-sign {
  --fa: "";
}

.fa-not-equal {
  --fa: "";
}

.fa-border-top-left, .fa-border-style {
  --fa: "";
}

.fa-map-location-dot, .fa-map-marked-alt {
  --fa: "";
}

.fa-jedi {
  --fa: "";
}

.fa-square-poll-vertical, .fa-poll {
  --fa: "";
}

.fa-mug-hot {
  --fa: "";
}

.fa-car-battery, .fa-battery-car {
  --fa: "";
}

.fa-gift {
  --fa: "";
}

.fa-dice-two {
  --fa: "";
}

.fa-chess-queen {
  --fa: "";
}

.fa-glasses {
  --fa: "";
}

.fa-chess-board {
  --fa: "";
}

.fa-building-circle-check {
  --fa: "";
}

.fa-person-chalkboard {
  --fa: "";
}

.fa-mars-stroke-right, .fa-mars-stroke-h {
  --fa: "";
}

.fa-hand-back-fist, .fa-hand-rock {
  --fa: "";
}

.fa-square-caret-up, .fa-caret-square-up {
  --fa: "";
}

.fa-cloud-showers-water {
  --fa: "";
}

.fa-chart-bar, .fa-bar-chart {
  --fa: "";
}

.fa-hands-bubbles, .fa-hands-wash {
  --fa: "";
}

.fa-less-than-equal {
  --fa: "";
}

.fa-train {
  --fa: "";
}

.fa-eye-low-vision, .fa-low-vision {
  --fa: "";
}

.fa-crow {
  --fa: "";
}

.fa-sailboat {
  --fa: "";
}

.fa-window-restore {
  --fa: "";
}

.fa-square-plus, .fa-plus-square {
  --fa: "";
}

.fa-torii-gate {
  --fa: "";
}

.fa-frog {
  --fa: "";
}

.fa-bucket {
  --fa: "";
}

.fa-image {
  --fa: "";
}

.fa-microphone {
  --fa: "";
}

.fa-cow {
  --fa: "";
}

.fa-caret-up {
  --fa: "";
}

.fa-screwdriver {
  --fa: "";
}

.fa-folder-closed {
  --fa: "";
}

.fa-house-tsunami {
  --fa: "";
}

.fa-square-nfi {
  --fa: "";
}

.fa-arrow-up-from-ground-water {
  --fa: "";
}

.fa-martini-glass, .fa-glass-martini-alt {
  --fa: "";
}

.fa-square-binary {
  --fa: "";
}

.fa-rotate-left, .fa-rotate-back, .fa-rotate-backward, .fa-undo-alt {
  --fa: "";
}

.fa-table-columns, .fa-columns {
  --fa: "";
}

.fa-lemon {
  --fa: "";
}

.fa-head-side-mask {
  --fa: "";
}

.fa-handshake {
  --fa: "";
}

.fa-gem {
  --fa: "";
}

.fa-dolly, .fa-dolly-box {
  --fa: "";
}

.fa-smoking {
  --fa: "";
}

.fa-minimize, .fa-compress-arrows-alt {
  --fa: "";
}

.fa-monument {
  --fa: "";
}

.fa-snowplow {
  --fa: "";
}

.fa-angles-right, .fa-angle-double-right {
  --fa: "";
}

.fa-cannabis {
  --fa: "";
}

.fa-circle-play, .fa-play-circle {
  --fa: "";
}

.fa-tablets {
  --fa: "";
}

.fa-ethernet {
  --fa: "";
}

.fa-euro-sign, .fa-eur, .fa-euro {
  --fa: "";
}

.fa-chair {
  --fa: "";
}

.fa-circle-check, .fa-check-circle {
  --fa: "";
}

.fa-circle-stop, .fa-stop-circle {
  --fa: "";
}

.fa-compass-drafting, .fa-drafting-compass {
  --fa: "";
}

.fa-plate-wheat {
  --fa: "";
}

.fa-icicles {
  --fa: "";
}

.fa-person-shelter {
  --fa: "";
}

.fa-neuter {
  --fa: "";
}

.fa-id-badge {
  --fa: "";
}

.fa-marker {
  --fa: "";
}

.fa-face-laugh-beam, .fa-laugh-beam {
  --fa: "";
}

.fa-helicopter-symbol {
  --fa: "";
}

.fa-universal-access {
  --fa: "";
}

.fa-circle-chevron-up, .fa-chevron-circle-up {
  --fa: "";
}

.fa-lari-sign {
  --fa: "";
}

.fa-volcano {
  --fa: "";
}

.fa-person-walking-dashed-line-arrow-right {
  --fa: "";
}

.fa-sterling-sign, .fa-gbp, .fa-pound-sign {
  --fa: "";
}

.fa-viruses {
  --fa: "";
}

.fa-square-person-confined {
  --fa: "";
}

.fa-user-tie {
  --fa: "";
}

.fa-arrow-down-long, .fa-long-arrow-down {
  --fa: "";
}

.fa-tent-arrow-down-to-line {
  --fa: "";
}

.fa-certificate {
  --fa: "";
}

.fa-reply-all, .fa-mail-reply-all {
  --fa: "";
}

.fa-suitcase {
  --fa: "";
}

.fa-person-skating, .fa-skating {
  --fa: "";
}

.fa-filter-circle-dollar, .fa-funnel-dollar {
  --fa: "";
}

.fa-camera-retro {
  --fa: "";
}

.fa-circle-arrow-down, .fa-arrow-circle-down {
  --fa: "";
}

.fa-file-import, .fa-arrow-right-to-file {
  --fa: "";
}

.fa-square-arrow-up-right, .fa-external-link-square {
  --fa: "";
}

.fa-box-open {
  --fa: "";
}

.fa-scroll {
  --fa: "";
}

.fa-spa {
  --fa: "";
}

.fa-location-pin-lock {
  --fa: "";
}

.fa-pause {
  --fa: "";
}

.fa-hill-avalanche {
  --fa: "";
}

.fa-temperature-empty, .fa-temperature-0, .fa-thermometer-0, .fa-thermometer-empty {
  --fa: "";
}

.fa-bomb {
  --fa: "";
}

.fa-registered {
  --fa: "";
}

.fa-address-card, .fa-contact-card, .fa-vcard {
  --fa: "";
}

.fa-scale-unbalanced-flip, .fa-balance-scale-right {
  --fa: "";
}

.fa-subscript {
  --fa: "";
}

.fa-diamond-turn-right, .fa-directions {
  --fa: "";
}

.fa-burst {
  --fa: "";
}

.fa-house-laptop, .fa-laptop-house {
  --fa: "";
}

.fa-face-tired, .fa-tired {
  --fa: "";
}

.fa-money-bills {
  --fa: "";
}

.fa-smog {
  --fa: "";
}

.fa-crutch {
  --fa: "";
}

.fa-cloud-arrow-up, .fa-cloud-upload, .fa-cloud-upload-alt {
  --fa: "";
}

.fa-palette {
  --fa: "";
}

.fa-arrows-turn-right {
  --fa: "";
}

.fa-vest {
  --fa: "";
}

.fa-ferry {
  --fa: "";
}

.fa-arrows-down-to-people {
  --fa: "";
}

.fa-seedling, .fa-sprout {
  --fa: "";
}

.fa-left-right, .fa-arrows-alt-h {
  --fa: "";
}

.fa-boxes-packing {
  --fa: "";
}

.fa-circle-arrow-left, .fa-arrow-circle-left {
  --fa: "";
}

.fa-group-arrows-rotate {
  --fa: "";
}

.fa-bowl-food {
  --fa: "";
}

.fa-candy-cane {
  --fa: "";
}

.fa-arrow-down-wide-short, .fa-sort-amount-asc, .fa-sort-amount-down {
  --fa: "";
}

.fa-cloud-bolt, .fa-thunderstorm {
  --fa: "";
}

.fa-text-slash, .fa-remove-format {
  --fa: "";
}

.fa-face-smile-wink, .fa-smile-wink {
  --fa: "";
}

.fa-file-word {
  --fa: "";
}

.fa-file-powerpoint {
  --fa: "";
}

.fa-arrows-left-right, .fa-arrows-h {
  --fa: "";
}

.fa-house-lock {
  --fa: "";
}

.fa-cloud-arrow-down, .fa-cloud-download, .fa-cloud-download-alt {
  --fa: "";
}

.fa-children {
  --fa: "";
}

.fa-chalkboard, .fa-blackboard {
  --fa: "";
}

.fa-user-large-slash, .fa-user-alt-slash {
  --fa: "";
}

.fa-envelope-open {
  --fa: "";
}

.fa-handshake-simple-slash, .fa-handshake-alt-slash {
  --fa: "";
}

.fa-mattress-pillow {
  --fa: "";
}

.fa-guarani-sign {
  --fa: "";
}

.fa-arrows-rotate, .fa-refresh, .fa-sync {
  --fa: "";
}

.fa-fire-extinguisher {
  --fa: "";
}

.fa-cruzeiro-sign {
  --fa: "";
}

.fa-greater-than-equal {
  --fa: "";
}

.fa-shield-halved, .fa-shield-alt {
  --fa: "";
}

.fa-book-atlas, .fa-atlas {
  --fa: "";
}

.fa-virus {
  --fa: "";
}

.fa-envelope-circle-check {
  --fa: "";
}

.fa-layer-group {
  --fa: "";
}

.fa-arrows-to-dot {
  --fa: "";
}

.fa-archway {
  --fa: "";
}

.fa-heart-circle-check {
  --fa: "";
}

.fa-house-chimney-crack, .fa-house-damage {
  --fa: "";
}

.fa-file-zipper, .fa-file-archive {
  --fa: "";
}

.fa-square {
  --fa: "";
}

.fa-martini-glass-empty, .fa-glass-martini {
  --fa: "";
}

.fa-couch {
  --fa: "";
}

.fa-cedi-sign {
  --fa: "";
}

.fa-italic {
  --fa: "";
}

.fa-table-cells-column-lock {
  --fa: "";
}

.fa-church {
  --fa: "";
}

.fa-comments-dollar {
  --fa: "";
}

.fa-democrat {
  --fa: "";
}

.fa-z {
  --fa: "Z";
}

.fa-person-skiing, .fa-skiing {
  --fa: "";
}

.fa-road-lock {
  --fa: "";
}

.fa-a {
  --fa: "A";
}

.fa-temperature-arrow-down, .fa-temperature-down {
  --fa: "";
}

.fa-feather-pointed, .fa-feather-alt {
  --fa: "";
}

.fa-p {
  --fa: "P";
}

.fa-snowflake {
  --fa: "";
}

.fa-newspaper {
  --fa: "";
}

.fa-rectangle-ad, .fa-ad {
  --fa: "";
}

.fa-circle-arrow-right, .fa-arrow-circle-right {
  --fa: "";
}

.fa-filter-circle-xmark {
  --fa: "";
}

.fa-locust {
  --fa: "";
}

.fa-sort, .fa-unsorted {
  --fa: "";
}

.fa-list-ol, .fa-list-1-2, .fa-list-numeric {
  --fa: "";
}

.fa-person-dress-burst {
  --fa: "";
}

.fa-money-check-dollar, .fa-money-check-alt {
  --fa: "";
}

.fa-vector-square {
  --fa: "";
}

.fa-bread-slice {
  --fa: "";
}

.fa-language {
  --fa: "";
}

.fa-face-kiss-wink-heart, .fa-kiss-wink-heart {
  --fa: "";
}

.fa-filter {
  --fa: "";
}

.fa-question {
  --fa: "?";
}

.fa-file-signature {
  --fa: "";
}

.fa-up-down-left-right, .fa-arrows-alt {
  --fa: "";
}

.fa-house-chimney-user {
  --fa: "";
}

.fa-hand-holding-heart {
  --fa: "";
}

.fa-puzzle-piece {
  --fa: "";
}

.fa-money-check {
  --fa: "";
}

.fa-star-half-stroke, .fa-star-half-alt {
  --fa: "";
}

.fa-code {
  --fa: "";
}

.fa-whiskey-glass, .fa-glass-whiskey {
  --fa: "";
}

.fa-building-circle-exclamation {
  --fa: "";
}

.fa-magnifying-glass-chart {
  --fa: "";
}

.fa-arrow-up-right-from-square, .fa-external-link {
  --fa: "";
}

.fa-cubes-stacked {
  --fa: "";
}

.fa-won-sign, .fa-krw, .fa-won {
  --fa: "";
}

.fa-virus-covid {
  --fa: "";
}

.fa-austral-sign {
  --fa: "";
}

.fa-f {
  --fa: "F";
}

.fa-leaf {
  --fa: "";
}

.fa-road {
  --fa: "";
}

.fa-taxi, .fa-cab {
  --fa: "";
}

.fa-person-circle-plus {
  --fa: "";
}

.fa-chart-pie, .fa-pie-chart {
  --fa: "";
}

.fa-bolt-lightning {
  --fa: "";
}

.fa-sack-xmark {
  --fa: "";
}

.fa-file-excel {
  --fa: "";
}

.fa-file-contract {
  --fa: "";
}

.fa-fish-fins {
  --fa: "";
}

.fa-building-flag {
  --fa: "";
}

.fa-face-grin-beam, .fa-grin-beam {
  --fa: "";
}

.fa-object-ungroup {
  --fa: "";
}

.fa-poop {
  --fa: "";
}

.fa-location-pin, .fa-map-marker {
  --fa: "";
}

.fa-kaaba {
  --fa: "";
}

.fa-toilet-paper {
  --fa: "";
}

.fa-helmet-safety, .fa-hard-hat, .fa-hat-hard {
  --fa: "";
}

.fa-eject {
  --fa: "";
}

.fa-circle-right, .fa-arrow-alt-circle-right {
  --fa: "";
}

.fa-plane-circle-check {
  --fa: "";
}

.fa-face-rolling-eyes, .fa-meh-rolling-eyes {
  --fa: "";
}

.fa-object-group {
  --fa: "";
}

.fa-chart-line, .fa-line-chart {
  --fa: "";
}

.fa-mask-ventilator {
  --fa: "";
}

.fa-arrow-right {
  --fa: "";
}

.fa-signs-post, .fa-map-signs {
  --fa: "";
}

.fa-cash-register {
  --fa: "";
}

.fa-person-circle-question {
  --fa: "";
}

.fa-h {
  --fa: "H";
}

.fa-tarp {
  --fa: "";
}

.fa-screwdriver-wrench, .fa-tools {
  --fa: "";
}

.fa-arrows-to-eye {
  --fa: "";
}

.fa-plug-circle-bolt {
  --fa: "";
}

.fa-heart {
  --fa: "";
}

.fa-mars-and-venus {
  --fa: "";
}

.fa-house-user, .fa-home-user {
  --fa: "";
}

.fa-dumpster-fire {
  --fa: "";
}

.fa-house-crack {
  --fa: "";
}

.fa-martini-glass-citrus, .fa-cocktail {
  --fa: "";
}

.fa-face-surprise, .fa-surprise {
  --fa: "";
}

.fa-bottle-water {
  --fa: "";
}

.fa-circle-pause, .fa-pause-circle {
  --fa: "";
}

.fa-toilet-paper-slash {
  --fa: "";
}

.fa-apple-whole, .fa-apple-alt {
  --fa: "";
}

.fa-kitchen-set {
  --fa: "";
}

.fa-r {
  --fa: "R";
}

.fa-temperature-quarter, .fa-temperature-1, .fa-thermometer-1, .fa-thermometer-quarter {
  --fa: "";
}

.fa-cube {
  --fa: "";
}

.fa-bitcoin-sign {
  --fa: "";
}

.fa-shield-dog {
  --fa: "";
}

.fa-solar-panel {
  --fa: "";
}

.fa-lock-open {
  --fa: "";
}

.fa-elevator {
  --fa: "";
}

.fa-money-bill-transfer {
  --fa: "";
}

.fa-money-bill-trend-up {
  --fa: "";
}

.fa-house-flood-water-circle-arrow-right {
  --fa: "";
}

.fa-square-poll-horizontal, .fa-poll-h {
  --fa: "";
}

.fa-circle {
  --fa: "";
}

.fa-backward-fast, .fa-fast-backward {
  --fa: "";
}

.fa-recycle {
  --fa: "";
}

.fa-user-astronaut {
  --fa: "";
}

.fa-plane-slash {
  --fa: "";
}

.fa-trademark {
  --fa: "";
}

.fa-basketball, .fa-basketball-ball {
  --fa: "";
}

.fa-satellite-dish {
  --fa: "";
}

.fa-circle-up, .fa-arrow-alt-circle-up {
  --fa: "";
}

.fa-mobile-screen-button, .fa-mobile-alt {
  --fa: "";
}

.fa-volume-high, .fa-volume-up {
  --fa: "";
}

.fa-users-rays {
  --fa: "";
}

.fa-wallet {
  --fa: "";
}

.fa-clipboard-check {
  --fa: "";
}

.fa-file-audio {
  --fa: "";
}

.fa-burger, .fa-hamburger {
  --fa: "";
}

.fa-wrench {
  --fa: "";
}

.fa-bugs {
  --fa: "";
}

.fa-rupee-sign, .fa-rupee {
  --fa: "";
}

.fa-file-image {
  --fa: "";
}

.fa-circle-question, .fa-question-circle {
  --fa: "";
}

.fa-plane-departure {
  --fa: "";
}

.fa-handshake-slash {
  --fa: "";
}

.fa-book-bookmark {
  --fa: "";
}

.fa-code-branch {
  --fa: "";
}

.fa-hat-cowboy {
  --fa: "";
}

.fa-bridge {
  --fa: "";
}

.fa-phone-flip, .fa-phone-alt {
  --fa: "";
}

.fa-truck-front {
  --fa: "";
}

.fa-cat {
  --fa: "";
}

.fa-anchor-circle-exclamation {
  --fa: "";
}

.fa-truck-field {
  --fa: "";
}

.fa-route {
  --fa: "";
}

.fa-clipboard-question {
  --fa: "";
}

.fa-panorama {
  --fa: "";
}

.fa-comment-medical {
  --fa: "";
}

.fa-teeth-open {
  --fa: "";
}

.fa-file-circle-minus {
  --fa: "";
}

.fa-tags {
  --fa: "";
}

.fa-wine-glass {
  --fa: "";
}

.fa-forward-fast, .fa-fast-forward {
  --fa: "";
}

.fa-face-meh-blank, .fa-meh-blank {
  --fa: "";
}

.fa-square-parking, .fa-parking {
  --fa: "";
}

.fa-house-signal {
  --fa: "";
}

.fa-bars-progress, .fa-tasks-alt {
  --fa: "";
}

.fa-faucet-drip {
  --fa: "";
}

.fa-cart-flatbed, .fa-dolly-flatbed {
  --fa: "";
}

.fa-ban-smoking, .fa-smoking-ban {
  --fa: "";
}

.fa-terminal {
  --fa: "";
}

.fa-mobile-button {
  --fa: "";
}

.fa-house-medical-flag {
  --fa: "";
}

.fa-basket-shopping, .fa-shopping-basket {
  --fa: "";
}

.fa-tape {
  --fa: "";
}

.fa-bus-simple, .fa-bus-alt {
  --fa: "";
}

.fa-eye {
  --fa: "";
}

.fa-face-sad-cry, .fa-sad-cry {
  --fa: "";
}

.fa-audio-description {
  --fa: "";
}

.fa-person-military-to-person {
  --fa: "";
}

.fa-file-shield {
  --fa: "";
}

.fa-user-slash {
  --fa: "";
}

.fa-pen {
  --fa: "";
}

.fa-tower-observation {
  --fa: "";
}

.fa-file-code {
  --fa: "";
}

.fa-signal, .fa-signal-5, .fa-signal-perfect {
  --fa: "";
}

.fa-bus {
  --fa: "";
}

.fa-heart-circle-xmark {
  --fa: "";
}

.fa-house-chimney, .fa-home-lg {
  --fa: "";
}

.fa-window-maximize {
  --fa: "";
}

.fa-face-frown, .fa-frown {
  --fa: "";
}

.fa-prescription {
  --fa: "";
}

.fa-shop, .fa-store-alt {
  --fa: "";
}

.fa-floppy-disk, .fa-save {
  --fa: "";
}

.fa-vihara {
  --fa: "";
}

.fa-scale-unbalanced, .fa-balance-scale-left {
  --fa: "";
}

.fa-sort-up, .fa-sort-asc {
  --fa: "";
}

.fa-comment-dots, .fa-commenting {
  --fa: "";
}

.fa-plant-wilt {
  --fa: "";
}

.fa-diamond {
  --fa: "";
}

.fa-face-grin-squint, .fa-grin-squint {
  --fa: "";
}

.fa-hand-holding-dollar, .fa-hand-holding-usd {
  --fa: "";
}

.fa-chart-diagram {
  --fa: "";
}

.fa-bacterium {
  --fa: "";
}

.fa-hand-pointer {
  --fa: "";
}

.fa-drum-steelpan {
  --fa: "";
}

.fa-hand-scissors {
  --fa: "";
}

.fa-hands-praying, .fa-praying-hands {
  --fa: "";
}

.fa-arrow-rotate-right, .fa-arrow-right-rotate, .fa-arrow-rotate-forward, .fa-redo {
  --fa: "";
}

.fa-biohazard {
  --fa: "";
}

.fa-location-crosshairs, .fa-location {
  --fa: "";
}

.fa-mars-double {
  --fa: "";
}

.fa-child-dress {
  --fa: "";
}

.fa-users-between-lines {
  --fa: "";
}

.fa-lungs-virus {
  --fa: "";
}

.fa-face-grin-tears, .fa-grin-tears {
  --fa: "";
}

.fa-phone {
  --fa: "";
}

.fa-calendar-xmark, .fa-calendar-times {
  --fa: "";
}

.fa-child-reaching {
  --fa: "";
}

.fa-head-side-virus {
  --fa: "";
}

.fa-user-gear, .fa-user-cog {
  --fa: "";
}

.fa-arrow-up-1-9, .fa-sort-numeric-up {
  --fa: "";
}

.fa-door-closed {
  --fa: "";
}

.fa-shield-virus {
  --fa: "";
}

.fa-dice-six {
  --fa: "";
}

.fa-mosquito-net {
  --fa: "";
}

.fa-file-fragment {
  --fa: "";
}

.fa-bridge-water {
  --fa: "";
}

.fa-person-booth {
  --fa: "";
}

.fa-text-width {
  --fa: "";
}

.fa-hat-wizard {
  --fa: "";
}

.fa-pen-fancy {
  --fa: "";
}

.fa-person-digging, .fa-digging {
  --fa: "";
}

.fa-trash {
  --fa: "";
}

.fa-gauge-simple, .fa-gauge-simple-med, .fa-tachometer-average {
  --fa: "";
}

.fa-book-medical {
  --fa: "";
}

.fa-poo {
  --fa: "";
}

.fa-quote-right, .fa-quote-right-alt {
  --fa: "";
}

.fa-shirt, .fa-t-shirt, .fa-tshirt {
  --fa: "";
}

.fa-cubes {
  --fa: "";
}

.fa-divide {
  --fa: "";
}

.fa-tenge-sign, .fa-tenge {
  --fa: "";
}

.fa-headphones {
  --fa: "";
}

.fa-hands-holding {
  --fa: "";
}

.fa-hands-clapping {
  --fa: "";
}

.fa-republican {
  --fa: "";
}

.fa-arrow-left {
  --fa: "";
}

.fa-person-circle-xmark {
  --fa: "";
}

.fa-ruler {
  --fa: "";
}

.fa-align-left {
  --fa: "";
}

.fa-dice-d6 {
  --fa: "";
}

.fa-restroom {
  --fa: "";
}

.fa-j {
  --fa: "J";
}

.fa-users-viewfinder {
  --fa: "";
}

.fa-file-video {
  --fa: "";
}

.fa-up-right-from-square, .fa-external-link-alt {
  --fa: "";
}

.fa-table-cells, .fa-th {
  --fa: "";
}

.fa-file-pdf {
  --fa: "";
}

.fa-book-bible, .fa-bible {
  --fa: "";
}

.fa-o {
  --fa: "O";
}

.fa-suitcase-medical, .fa-medkit {
  --fa: "";
}

.fa-user-secret {
  --fa: "";
}

.fa-otter {
  --fa: "";
}

.fa-person-dress, .fa-female {
  --fa: "";
}

.fa-comment-dollar {
  --fa: "";
}

.fa-business-time, .fa-briefcase-clock {
  --fa: "";
}

.fa-table-cells-large, .fa-th-large {
  --fa: "";
}

.fa-book-tanakh, .fa-tanakh {
  --fa: "";
}

.fa-phone-volume, .fa-volume-control-phone {
  --fa: "";
}

.fa-hat-cowboy-side {
  --fa: "";
}

.fa-clipboard-user {
  --fa: "";
}

.fa-child {
  --fa: "";
}

.fa-lira-sign {
  --fa: "";
}

.fa-satellite {
  --fa: "";
}

.fa-plane-lock {
  --fa: "";
}

.fa-tag {
  --fa: "";
}

.fa-comment {
  --fa: "";
}

.fa-cake-candles, .fa-birthday-cake, .fa-cake {
  --fa: "";
}

.fa-envelope {
  --fa: "";
}

.fa-angles-up, .fa-angle-double-up {
  --fa: "";
}

.fa-paperclip {
  --fa: "";
}

.fa-arrow-right-to-city {
  --fa: "";
}

.fa-ribbon {
  --fa: "";
}

.fa-lungs {
  --fa: "";
}

.fa-arrow-up-9-1, .fa-sort-numeric-up-alt {
  --fa: "";
}

.fa-litecoin-sign {
  --fa: "";
}

.fa-border-none {
  --fa: "";
}

.fa-circle-nodes {
  --fa: "";
}

.fa-parachute-box {
  --fa: "";
}

.fa-indent {
  --fa: "";
}

.fa-truck-field-un {
  --fa: "";
}

.fa-hourglass, .fa-hourglass-empty {
  --fa: "";
}

.fa-mountain {
  --fa: "";
}

.fa-user-doctor, .fa-user-md {
  --fa: "";
}

.fa-circle-info, .fa-info-circle {
  --fa: "";
}

.fa-cloud-meatball {
  --fa: "";
}

.fa-camera, .fa-camera-alt {
  --fa: "";
}

.fa-square-virus {
  --fa: "";
}

.fa-meteor {
  --fa: "";
}

.fa-car-on {
  --fa: "";
}

.fa-sleigh {
  --fa: "";
}

.fa-arrow-down-1-9, .fa-sort-numeric-asc, .fa-sort-numeric-down {
  --fa: "";
}

.fa-hand-holding-droplet, .fa-hand-holding-water {
  --fa: "";
}

.fa-water {
  --fa: "";
}

.fa-calendar-check {
  --fa: "";
}

.fa-braille {
  --fa: "";
}

.fa-prescription-bottle-medical, .fa-prescription-bottle-alt {
  --fa: "";
}

.fa-landmark {
  --fa: "";
}

.fa-truck {
  --fa: "";
}

.fa-crosshairs {
  --fa: "";
}

.fa-person-cane {
  --fa: "";
}

.fa-tent {
  --fa: "";
}

.fa-vest-patches {
  --fa: "";
}

.fa-check-double {
  --fa: "";
}

.fa-arrow-down-a-z, .fa-sort-alpha-asc, .fa-sort-alpha-down {
  --fa: "";
}

.fa-money-bill-wheat {
  --fa: "";
}

.fa-cookie {
  --fa: "";
}

.fa-arrow-rotate-left, .fa-arrow-left-rotate, .fa-arrow-rotate-back, .fa-arrow-rotate-backward, .fa-undo {
  --fa: "";
}

.fa-hard-drive, .fa-hdd {
  --fa: "";
}

.fa-face-grin-squint-tears, .fa-grin-squint-tears {
  --fa: "";
}

.fa-dumbbell {
  --fa: "";
}

.fa-rectangle-list, .fa-list-alt {
  --fa: "";
}

.fa-tarp-droplet {
  --fa: "";
}

.fa-house-medical-circle-check {
  --fa: "";
}

.fa-person-skiing-nordic, .fa-skiing-nordic {
  --fa: "";
}

.fa-calendar-plus {
  --fa: "";
}

.fa-plane-arrival {
  --fa: "";
}

.fa-circle-left, .fa-arrow-alt-circle-left {
  --fa: "";
}

.fa-train-subway, .fa-subway {
  --fa: "";
}

.fa-chart-gantt {
  --fa: "";
}

.fa-indian-rupee-sign, .fa-indian-rupee, .fa-inr {
  --fa: "";
}

.fa-crop-simple, .fa-crop-alt {
  --fa: "";
}

.fa-money-bill-1, .fa-money-bill-alt {
  --fa: "";
}

.fa-left-long, .fa-long-arrow-alt-left {
  --fa: "";
}

.fa-dna {
  --fa: "";
}

.fa-virus-slash {
  --fa: "";
}

.fa-minus, .fa-subtract {
  --fa: "";
}

.fa-chess {
  --fa: "";
}

.fa-arrow-left-long, .fa-long-arrow-left {
  --fa: "";
}

.fa-plug-circle-check {
  --fa: "";
}

.fa-street-view {
  --fa: "";
}

.fa-franc-sign {
  --fa: "";
}

.fa-volume-off {
  --fa: "";
}

.fa-hands-asl-interpreting, .fa-american-sign-language-interpreting, .fa-asl-interpreting, .fa-hands-american-sign-language-interpreting {
  --fa: "";
}

.fa-gear, .fa-cog {
  --fa: "";
}

.fa-droplet-slash, .fa-tint-slash {
  --fa: "";
}

.fa-mosque {
  --fa: "";
}

.fa-mosquito {
  --fa: "";
}

.fa-star-of-david {
  --fa: "";
}

.fa-person-military-rifle {
  --fa: "";
}

.fa-cart-shopping, .fa-shopping-cart {
  --fa: "";
}

.fa-vials {
  --fa: "";
}

.fa-plug-circle-plus {
  --fa: "";
}

.fa-place-of-worship {
  --fa: "";
}

.fa-grip-vertical {
  --fa: "";
}

.fa-hexagon-nodes {
  --fa: "";
}

.fa-arrow-turn-up, .fa-level-up {
  --fa: "";
}

.fa-u {
  --fa: "U";
}

.fa-square-root-variable, .fa-square-root-alt {
  --fa: "";
}

.fa-clock, .fa-clock-four {
  --fa: "";
}

.fa-backward-step, .fa-step-backward {
  --fa: "";
}

.fa-pallet {
  --fa: "";
}

.fa-faucet {
  --fa: "";
}

.fa-baseball-bat-ball {
  --fa: "";
}

.fa-s {
  --fa: "S";
}

.fa-timeline {
  --fa: "";
}

.fa-keyboard {
  --fa: "";
}

.fa-caret-down {
  --fa: "";
}

.fa-house-chimney-medical, .fa-clinic-medical {
  --fa: "";
}

.fa-temperature-three-quarters, .fa-temperature-3, .fa-thermometer-3, .fa-thermometer-three-quarters {
  --fa: "";
}

.fa-mobile-screen, .fa-mobile-android-alt {
  --fa: "";
}

.fa-plane-up {
  --fa: "";
}

.fa-piggy-bank {
  --fa: "";
}

.fa-battery-half, .fa-battery-3 {
  --fa: "";
}

.fa-mountain-city {
  --fa: "";
}

.fa-coins {
  --fa: "";
}

.fa-khanda {
  --fa: "";
}

.fa-sliders, .fa-sliders-h {
  --fa: "";
}

.fa-folder-tree {
  --fa: "";
}

.fa-network-wired {
  --fa: "";
}

.fa-map-pin {
  --fa: "";
}

.fa-hamsa {
  --fa: "";
}

.fa-cent-sign {
  --fa: "";
}

.fa-flask {
  --fa: "";
}

.fa-person-pregnant {
  --fa: "";
}

.fa-wand-sparkles {
  --fa: "";
}

.fa-ellipsis-vertical, .fa-ellipsis-v {
  --fa: "";
}

.fa-ticket {
  --fa: "";
}

.fa-power-off {
  --fa: "";
}

.fa-right-long, .fa-long-arrow-alt-right {
  --fa: "";
}

.fa-flag-usa {
  --fa: "";
}

.fa-laptop-file {
  --fa: "";
}

.fa-tty, .fa-teletype {
  --fa: "";
}

.fa-diagram-next {
  --fa: "";
}

.fa-person-rifle {
  --fa: "";
}

.fa-house-medical-circle-exclamation {
  --fa: "";
}

.fa-closed-captioning {
  --fa: "";
}

.fa-person-hiking, .fa-hiking {
  --fa: "";
}

.fa-venus-double {
  --fa: "";
}

.fa-images {
  --fa: "";
}

.fa-calculator {
  --fa: "";
}

.fa-people-pulling {
  --fa: "";
}

.fa-n {
  --fa: "N";
}

.fa-cable-car, .fa-tram {
  --fa: "";
}

.fa-cloud-rain {
  --fa: "";
}

.fa-building-circle-xmark {
  --fa: "";
}

.fa-ship {
  --fa: "";
}

.fa-arrows-down-to-line {
  --fa: "";
}

.fa-download {
  --fa: "";
}

.fa-face-grin, .fa-grin {
  --fa: "";
}

.fa-delete-left, .fa-backspace {
  --fa: "";
}

.fa-eye-dropper, .fa-eye-dropper-empty, .fa-eyedropper {
  --fa: "";
}

.fa-file-circle-check {
  --fa: "";
}

.fa-forward {
  --fa: "";
}

.fa-mobile, .fa-mobile-android, .fa-mobile-phone {
  --fa: "";
}

.fa-face-meh, .fa-meh {
  --fa: "";
}

.fa-align-center {
  --fa: "";
}

.fa-book-skull, .fa-book-dead {
  --fa: "";
}

.fa-id-card, .fa-drivers-license {
  --fa: "";
}

.fa-outdent, .fa-dedent {
  --fa: "";
}

.fa-heart-circle-exclamation {
  --fa: "";
}

.fa-house, .fa-home, .fa-home-alt, .fa-home-lg-alt {
  --fa: "";
}

.fa-calendar-week {
  --fa: "";
}

.fa-laptop-medical {
  --fa: "";
}

.fa-b {
  --fa: "B";
}

.fa-file-medical {
  --fa: "";
}

.fa-dice-one {
  --fa: "";
}

.fa-kiwi-bird {
  --fa: "";
}

.fa-arrow-right-arrow-left, .fa-exchange {
  --fa: "";
}

.fa-rotate-right, .fa-redo-alt, .fa-rotate-forward {
  --fa: "";
}

.fa-utensils, .fa-cutlery {
  --fa: "";
}

.fa-arrow-up-wide-short, .fa-sort-amount-up {
  --fa: "";
}

.fa-mill-sign {
  --fa: "";
}

.fa-bowl-rice {
  --fa: "";
}

.fa-skull {
  --fa: "";
}

.fa-tower-broadcast, .fa-broadcast-tower {
  --fa: "";
}

.fa-truck-pickup {
  --fa: "";
}

.fa-up-long, .fa-long-arrow-alt-up {
  --fa: "";
}

.fa-stop {
  --fa: "";
}

.fa-code-merge {
  --fa: "";
}

.fa-upload {
  --fa: "";
}

.fa-hurricane {
  --fa: "";
}

.fa-mound {
  --fa: "";
}

.fa-toilet-portable {
  --fa: "";
}

.fa-compact-disc {
  --fa: "";
}

.fa-file-arrow-down, .fa-file-download {
  --fa: "";
}

.fa-caravan {
  --fa: "";
}

.fa-shield-cat {
  --fa: "";
}

.fa-bolt, .fa-zap {
  --fa: "";
}

.fa-glass-water {
  --fa: "";
}

.fa-oil-well {
  --fa: "";
}

.fa-vault {
  --fa: "";
}

.fa-mars {
  --fa: "";
}

.fa-toilet {
  --fa: "";
}

.fa-plane-circle-xmark {
  --fa: "";
}

.fa-yen-sign, .fa-cny, .fa-jpy, .fa-rmb, .fa-yen {
  --fa: "";
}

.fa-ruble-sign, .fa-rouble, .fa-rub, .fa-ruble {
  --fa: "";
}

.fa-sun {
  --fa: "";
}

.fa-guitar {
  --fa: "";
}

.fa-face-laugh-wink, .fa-laugh-wink {
  --fa: "";
}

.fa-horse-head {
  --fa: "";
}

.fa-bore-hole {
  --fa: "";
}

.fa-industry {
  --fa: "";
}

.fa-circle-down, .fa-arrow-alt-circle-down {
  --fa: "";
}

.fa-arrows-turn-to-dots {
  --fa: "";
}

.fa-florin-sign {
  --fa: "";
}

.fa-arrow-down-short-wide, .fa-sort-amount-desc, .fa-sort-amount-down-alt {
  --fa: "";
}

.fa-less-than {
  --fa: "<";
}

.fa-angle-down {
  --fa: "";
}

.fa-car-tunnel {
  --fa: "";
}

.fa-head-side-cough {
  --fa: "";
}

.fa-grip-lines {
  --fa: "";
}

.fa-thumbs-down {
  --fa: "";
}

.fa-user-lock {
  --fa: "";
}

.fa-arrow-right-long, .fa-long-arrow-right {
  --fa: "";
}

.fa-anchor-circle-xmark {
  --fa: "";
}

.fa-ellipsis, .fa-ellipsis-h {
  --fa: "";
}

.fa-chess-pawn {
  --fa: "";
}

.fa-kit-medical, .fa-first-aid {
  --fa: "";
}

.fa-person-through-window {
  --fa: "";
}

.fa-toolbox {
  --fa: "";
}

.fa-hands-holding-circle {
  --fa: "";
}

.fa-bug {
  --fa: "";
}

.fa-credit-card, .fa-credit-card-alt {
  --fa: "";
}

.fa-car, .fa-automobile {
  --fa: "";
}

.fa-hand-holding-hand {
  --fa: "";
}

.fa-book-open-reader, .fa-book-reader {
  --fa: "";
}

.fa-mountain-sun {
  --fa: "";
}

.fa-arrows-left-right-to-line {
  --fa: "";
}

.fa-dice-d20 {
  --fa: "";
}

.fa-truck-droplet {
  --fa: "";
}

.fa-file-circle-xmark {
  --fa: "";
}

.fa-temperature-arrow-up, .fa-temperature-up {
  --fa: "";
}

.fa-medal {
  --fa: "";
}

.fa-bed {
  --fa: "";
}

.fa-square-h, .fa-h-square {
  --fa: "";
}

.fa-podcast {
  --fa: "";
}

.fa-temperature-full, .fa-temperature-4, .fa-thermometer-4, .fa-thermometer-full {
  --fa: "";
}

.fa-bell {
  --fa: "";
}

.fa-superscript {
  --fa: "";
}

.fa-plug-circle-xmark {
  --fa: "";
}

.fa-star-of-life {
  --fa: "";
}

.fa-phone-slash {
  --fa: "";
}

.fa-paint-roller {
  --fa: "";
}

.fa-handshake-angle, .fa-hands-helping {
  --fa: "";
}

.fa-location-dot, .fa-map-marker-alt {
  --fa: "";
}

.fa-file {
  --fa: "";
}

.fa-greater-than {
  --fa: ">";
}

.fa-person-swimming, .fa-swimmer {
  --fa: "";
}

.fa-arrow-down {
  --fa: "";
}

.fa-droplet, .fa-tint {
  --fa: "";
}

.fa-eraser {
  --fa: "";
}

.fa-earth-americas, .fa-earth, .fa-earth-america, .fa-globe-americas {
  --fa: "";
}

.fa-person-burst {
  --fa: "";
}

.fa-dove {
  --fa: "";
}

.fa-battery-empty, .fa-battery-0 {
  --fa: "";
}

.fa-socks {
  --fa: "";
}

.fa-inbox {
  --fa: "";
}

.fa-section {
  --fa: "";
}

.fa-gauge-high, .fa-tachometer-alt, .fa-tachometer-alt-fast {
  --fa: "";
}

.fa-envelope-open-text {
  --fa: "";
}

.fa-hospital, .fa-hospital-alt, .fa-hospital-wide {
  --fa: "";
}

.fa-wine-bottle {
  --fa: "";
}

.fa-chess-rook {
  --fa: "";
}

.fa-bars-staggered, .fa-reorder, .fa-stream {
  --fa: "";
}

.fa-dharmachakra {
  --fa: "";
}

.fa-hotdog {
  --fa: "";
}

.fa-person-walking-with-cane, .fa-blind {
  --fa: "";
}

.fa-drum {
  --fa: "";
}

.fa-ice-cream {
  --fa: "";
}

.fa-heart-circle-bolt {
  --fa: "";
}

.fa-fax {
  --fa: "";
}

.fa-paragraph {
  --fa: "";
}

.fa-check-to-slot, .fa-vote-yea {
  --fa: "";
}

.fa-star-half {
  --fa: "";
}

.fa-boxes-stacked, .fa-boxes, .fa-boxes-alt {
  --fa: "";
}

.fa-link, .fa-chain {
  --fa: "";
}

.fa-ear-listen, .fa-assistive-listening-systems {
  --fa: "";
}

.fa-tree-city {
  --fa: "";
}

.fa-play {
  --fa: "";
}

.fa-font {
  --fa: "";
}

.fa-table-cells-row-lock {
  --fa: "";
}

.fa-rupiah-sign {
  --fa: "";
}

.fa-magnifying-glass, .fa-search {
  --fa: "";
}

.fa-table-tennis-paddle-ball, .fa-ping-pong-paddle-ball, .fa-table-tennis {
  --fa: "";
}

.fa-person-dots-from-line, .fa-diagnoses {
  --fa: "";
}

.fa-trash-can-arrow-up, .fa-trash-restore-alt {
  --fa: "";
}

.fa-naira-sign {
  --fa: "";
}

.fa-cart-arrow-down {
  --fa: "";
}

.fa-walkie-talkie {
  --fa: "";
}

.fa-file-pen, .fa-file-edit {
  --fa: "";
}

.fa-receipt {
  --fa: "";
}

.fa-square-pen, .fa-pen-square, .fa-pencil-square {
  --fa: "";
}

.fa-suitcase-rolling {
  --fa: "";
}

.fa-person-circle-exclamation {
  --fa: "";
}

.fa-chevron-down {
  --fa: "";
}

.fa-battery-full, .fa-battery, .fa-battery-5 {
  --fa: "";
}

.fa-skull-crossbones {
  --fa: "";
}

.fa-code-compare {
  --fa: "";
}

.fa-list-ul, .fa-list-dots {
  --fa: "";
}

.fa-school-lock {
  --fa: "";
}

.fa-tower-cell {
  --fa: "";
}

.fa-down-long, .fa-long-arrow-alt-down {
  --fa: "";
}

.fa-ranking-star {
  --fa: "";
}

.fa-chess-king {
  --fa: "";
}

.fa-person-harassing {
  --fa: "";
}

.fa-brazilian-real-sign {
  --fa: "";
}

.fa-landmark-dome, .fa-landmark-alt {
  --fa: "";
}

.fa-arrow-up {
  --fa: "";
}

.fa-tv, .fa-television, .fa-tv-alt {
  --fa: "";
}

.fa-shrimp {
  --fa: "";
}

.fa-list-check, .fa-tasks {
  --fa: "";
}

.fa-jug-detergent {
  --fa: "";
}

.fa-circle-user, .fa-user-circle {
  --fa: "";
}

.fa-user-shield {
  --fa: "";
}

.fa-wind {
  --fa: "";
}

.fa-car-burst, .fa-car-crash {
  --fa: "";
}

.fa-y {
  --fa: "Y";
}

.fa-person-snowboarding, .fa-snowboarding {
  --fa: "";
}

.fa-truck-fast, .fa-shipping-fast {
  --fa: "";
}

.fa-fish {
  --fa: "";
}

.fa-user-graduate {
  --fa: "";
}

.fa-circle-half-stroke, .fa-adjust {
  --fa: "";
}

.fa-clapperboard {
  --fa: "";
}

.fa-circle-radiation, .fa-radiation-alt {
  --fa: "";
}

.fa-baseball, .fa-baseball-ball {
  --fa: "";
}

.fa-jet-fighter-up {
  --fa: "";
}

.fa-diagram-project, .fa-project-diagram {
  --fa: "";
}

.fa-copy {
  --fa: "";
}

.fa-volume-xmark, .fa-volume-mute, .fa-volume-times {
  --fa: "";
}

.fa-hand-sparkles {
  --fa: "";
}

.fa-grip, .fa-grip-horizontal {
  --fa: "";
}

.fa-share-from-square, .fa-share-square {
  --fa: "";
}

.fa-child-combatant, .fa-child-rifle {
  --fa: "";
}

.fa-gun {
  --fa: "";
}

.fa-square-phone, .fa-phone-square {
  --fa: "";
}

.fa-plus, .fa-add {
  --fa: "+";
}

.fa-expand {
  --fa: "";
}

.fa-computer {
  --fa: "";
}

.fa-xmark, .fa-close, .fa-multiply, .fa-remove, .fa-times {
  --fa: "";
}

.fa-arrows-up-down-left-right, .fa-arrows {
  --fa: "";
}

.fa-chalkboard-user, .fa-chalkboard-teacher {
  --fa: "";
}

.fa-peso-sign {
  --fa: "";
}

.fa-building-shield {
  --fa: "";
}

.fa-baby {
  --fa: "";
}

.fa-users-line {
  --fa: "";
}

.fa-quote-left, .fa-quote-left-alt {
  --fa: "";
}

.fa-tractor {
  --fa: "";
}

.fa-trash-arrow-up, .fa-trash-restore {
  --fa: "";
}

.fa-arrow-down-up-lock {
  --fa: "";
}

.fa-lines-leaning {
  --fa: "";
}

.fa-ruler-combined {
  --fa: "";
}

.fa-copyright {
  --fa: "";
}

.fa-equals {
  --fa: "=";
}

.fa-blender {
  --fa: "";
}

.fa-teeth {
  --fa: "";
}

.fa-shekel-sign, .fa-ils, .fa-shekel, .fa-sheqel, .fa-sheqel-sign {
  --fa: "";
}

.fa-map {
  --fa: "";
}

.fa-rocket {
  --fa: "";
}

.fa-photo-film, .fa-photo-video {
  --fa: "";
}

.fa-folder-minus {
  --fa: "";
}

.fa-hexagon-nodes-bolt {
  --fa: "";
}

.fa-store {
  --fa: "";
}

.fa-arrow-trend-up {
  --fa: "";
}

.fa-plug-circle-minus {
  --fa: "";
}

.fa-sign-hanging, .fa-sign {
  --fa: "";
}

.fa-bezier-curve {
  --fa: "";
}

.fa-bell-slash {
  --fa: "";
}

.fa-tablet, .fa-tablet-android {
  --fa: "";
}

.fa-school-flag {
  --fa: "";
}

.fa-fill {
  --fa: "";
}

.fa-angle-up {
  --fa: "";
}

.fa-drumstick-bite {
  --fa: "";
}

.fa-holly-berry {
  --fa: "";
}

.fa-chevron-left {
  --fa: "";
}

.fa-bacteria {
  --fa: "";
}

.fa-hand-lizard {
  --fa: "";
}

.fa-notdef {
  --fa: "";
}

.fa-disease {
  --fa: "";
}

.fa-briefcase-medical {
  --fa: "";
}

.fa-genderless {
  --fa: "";
}

.fa-chevron-right {
  --fa: "";
}

.fa-retweet {
  --fa: "";
}

.fa-car-rear, .fa-car-alt {
  --fa: "";
}

.fa-pump-soap {
  --fa: "";
}

.fa-video-slash {
  --fa: "";
}

.fa-battery-quarter, .fa-battery-2 {
  --fa: "";
}

.fa-radio {
  --fa: "";
}

.fa-baby-carriage, .fa-carriage-baby {
  --fa: "";
}

.fa-traffic-light {
  --fa: "";
}

.fa-thermometer {
  --fa: "";
}

.fa-vr-cardboard {
  --fa: "";
}

.fa-hand-middle-finger {
  --fa: "";
}

.fa-percent, .fa-percentage {
  --fa: "%";
}

.fa-truck-moving {
  --fa: "";
}

.fa-glass-water-droplet {
  --fa: "";
}

.fa-display {
  --fa: "";
}

.fa-face-smile, .fa-smile {
  --fa: "";
}

.fa-thumbtack, .fa-thumb-tack {
  --fa: "";
}

.fa-trophy {
  --fa: "";
}

.fa-person-praying, .fa-pray {
  --fa: "";
}

.fa-hammer {
  --fa: "";
}

.fa-hand-peace {
  --fa: "";
}

.fa-rotate, .fa-sync-alt {
  --fa: "";
}

.fa-spinner {
  --fa: "";
}

.fa-robot {
  --fa: "";
}

.fa-peace {
  --fa: "";
}

.fa-gears, .fa-cogs {
  --fa: "";
}

.fa-warehouse {
  --fa: "";
}

.fa-arrow-up-right-dots {
  --fa: "";
}

.fa-splotch {
  --fa: "";
}

.fa-face-grin-hearts, .fa-grin-hearts {
  --fa: "";
}

.fa-dice-four {
  --fa: "";
}

.fa-sim-card {
  --fa: "";
}

.fa-transgender, .fa-transgender-alt {
  --fa: "";
}

.fa-mercury {
  --fa: "";
}

.fa-arrow-turn-down, .fa-level-down {
  --fa: "";
}

.fa-person-falling-burst {
  --fa: "";
}

.fa-award {
  --fa: "";
}

.fa-ticket-simple, .fa-ticket-alt {
  --fa: "";
}

.fa-building {
  --fa: "";
}

.fa-angles-left, .fa-angle-double-left {
  --fa: "";
}

.fa-qrcode {
  --fa: "";
}

.fa-clock-rotate-left, .fa-history {
  --fa: "";
}

.fa-face-grin-beam-sweat, .fa-grin-beam-sweat {
  --fa: "";
}

.fa-file-export, .fa-arrow-right-from-file {
  --fa: "";
}

.fa-shield, .fa-shield-blank {
  --fa: "";
}

.fa-arrow-up-short-wide, .fa-sort-amount-up-alt {
  --fa: "";
}

.fa-comment-nodes {
  --fa: "";
}

.fa-house-medical {
  --fa: "";
}

.fa-golf-ball-tee, .fa-golf-ball {
  --fa: "";
}

.fa-circle-chevron-left, .fa-chevron-circle-left {
  --fa: "";
}

.fa-house-chimney-window {
  --fa: "";
}

.fa-pen-nib {
  --fa: "";
}

.fa-tent-arrow-turn-left {
  --fa: "";
}

.fa-tents {
  --fa: "";
}

.fa-wand-magic, .fa-magic {
  --fa: "";
}

.fa-dog {
  --fa: "";
}

.fa-carrot {
  --fa: "";
}

.fa-moon {
  --fa: "";
}

.fa-wine-glass-empty, .fa-wine-glass-alt {
  --fa: "";
}

.fa-cheese {
  --fa: "";
}

.fa-yin-yang {
  --fa: "";
}

.fa-music {
  --fa: "";
}

.fa-code-commit {
  --fa: "";
}

.fa-temperature-low {
  --fa: "";
}

.fa-person-biking, .fa-biking {
  --fa: "";
}

.fa-broom {
  --fa: "";
}

.fa-shield-heart {
  --fa: "";
}

.fa-gopuram {
  --fa: "";
}

.fa-earth-oceania, .fa-globe-oceania {
  --fa: "";
}

.fa-square-xmark, .fa-times-square, .fa-xmark-square {
  --fa: "";
}

.fa-hashtag {
  --fa: "#";
}

.fa-up-right-and-down-left-from-center, .fa-expand-alt {
  --fa: "";
}

.fa-oil-can {
  --fa: "";
}

.fa-t {
  --fa: "T";
}

.fa-hippo {
  --fa: "";
}

.fa-chart-column {
  --fa: "";
}

.fa-infinity {
  --fa: "";
}

.fa-vial-circle-check {
  --fa: "";
}

.fa-person-arrow-down-to-line {
  --fa: "";
}

.fa-voicemail {
  --fa: "";
}

.fa-fan {
  --fa: "";
}

.fa-person-walking-luggage {
  --fa: "";
}

.fa-up-down, .fa-arrows-alt-v {
  --fa: "";
}

.fa-cloud-moon-rain {
  --fa: "";
}

.fa-calendar {
  --fa: "";
}

.fa-trailer {
  --fa: "";
}

.fa-bahai, .fa-haykal {
  --fa: "";
}

.fa-sd-card {
  --fa: "";
}

.fa-dragon {
  --fa: "";
}

.fa-shoe-prints {
  --fa: "";
}

.fa-circle-plus, .fa-plus-circle {
  --fa: "";
}

.fa-face-grin-tongue-wink, .fa-grin-tongue-wink {
  --fa: "";
}

.fa-hand-holding {
  --fa: "";
}

.fa-plug-circle-exclamation {
  --fa: "";
}

.fa-link-slash, .fa-chain-broken, .fa-chain-slash, .fa-unlink {
  --fa: "";
}

.fa-clone {
  --fa: "";
}

.fa-person-walking-arrow-loop-left {
  --fa: "";
}

.fa-arrow-up-z-a, .fa-sort-alpha-up-alt {
  --fa: "";
}

.fa-fire-flame-curved, .fa-fire-alt {
  --fa: "";
}

.fa-tornado {
  --fa: "";
}

.fa-file-circle-plus {
  --fa: "";
}

.fa-book-quran, .fa-quran {
  --fa: "";
}

.fa-anchor {
  --fa: "";
}

.fa-border-all {
  --fa: "";
}

.fa-face-angry, .fa-angry {
  --fa: "";
}

.fa-cookie-bite {
  --fa: "";
}

.fa-arrow-trend-down {
  --fa: "";
}

.fa-rss, .fa-feed {
  --fa: "";
}

.fa-draw-polygon {
  --fa: "";
}

.fa-scale-balanced, .fa-balance-scale {
  --fa: "";
}

.fa-gauge-simple-high, .fa-tachometer, .fa-tachometer-fast {
  --fa: "";
}

.fa-shower {
  --fa: "";
}

.fa-desktop, .fa-desktop-alt {
  --fa: "";
}

.fa-m {
  --fa: "M";
}

.fa-table-list, .fa-th-list {
  --fa: "";
}

.fa-comment-sms, .fa-sms {
  --fa: "";
}

.fa-book {
  --fa: "";
}

.fa-user-plus {
  --fa: "";
}

.fa-check {
  --fa: "";
}

.fa-battery-three-quarters, .fa-battery-4 {
  --fa: "";
}

.fa-house-circle-check {
  --fa: "";
}

.fa-angle-left {
  --fa: "";
}

.fa-diagram-successor {
  --fa: "";
}

.fa-truck-arrow-right {
  --fa: "";
}

.fa-arrows-split-up-and-left {
  --fa: "";
}

.fa-hand-fist, .fa-fist-raised {
  --fa: "";
}

.fa-cloud-moon {
  --fa: "";
}

.fa-briefcase {
  --fa: "";
}

.fa-person-falling {
  --fa: "";
}

.fa-image-portrait, .fa-portrait {
  --fa: "";
}

.fa-user-tag {
  --fa: "";
}

.fa-rug {
  --fa: "";
}

.fa-earth-europe, .fa-globe-europe {
  --fa: "";
}

.fa-cart-flatbed-suitcase, .fa-luggage-cart {
  --fa: "";
}

.fa-rectangle-xmark, .fa-rectangle-times, .fa-times-rectangle, .fa-window-close {
  --fa: "";
}

.fa-baht-sign {
  --fa: "";
}

.fa-book-open {
  --fa: "";
}

.fa-book-journal-whills, .fa-journal-whills {
  --fa: "";
}

.fa-handcuffs {
  --fa: "";
}

.fa-triangle-exclamation, .fa-exclamation-triangle, .fa-warning {
  --fa: "";
}

.fa-database {
  --fa: "";
}

.fa-share, .fa-mail-forward {
  --fa: "";
}

.fa-bottle-droplet {
  --fa: "";
}

.fa-mask-face {
  --fa: "";
}

.fa-hill-rockslide {
  --fa: "";
}

.fa-right-left, .fa-exchange-alt {
  --fa: "";
}

.fa-paper-plane {
  --fa: "";
}

.fa-road-circle-exclamation {
  --fa: "";
}

.fa-dungeon {
  --fa: "";
}

.fa-align-right {
  --fa: "";
}

.fa-money-bill-1-wave, .fa-money-bill-wave-alt {
  --fa: "";
}

.fa-life-ring {
  --fa: "";
}

.fa-hands, .fa-sign-language, .fa-signing {
  --fa: "";
}

.fa-calendar-day {
  --fa: "";
}

.fa-water-ladder, .fa-ladder-water, .fa-swimming-pool {
  --fa: "";
}

.fa-arrows-up-down, .fa-arrows-v {
  --fa: "";
}

.fa-face-grimace, .fa-grimace {
  --fa: "";
}

.fa-wheelchair-move, .fa-wheelchair-alt {
  --fa: "";
}

.fa-turn-down, .fa-level-down-alt {
  --fa: "";
}

.fa-person-walking-arrow-right {
  --fa: "";
}

.fa-square-envelope, .fa-envelope-square {
  --fa: "";
}

.fa-dice {
  --fa: "";
}

.fa-bowling-ball {
  --fa: "";
}

.fa-brain {
  --fa: "";
}

.fa-bandage, .fa-band-aid {
  --fa: "";
}

.fa-calendar-minus {
  --fa: "";
}

.fa-circle-xmark, .fa-times-circle, .fa-xmark-circle {
  --fa: "";
}

.fa-gifts {
  --fa: "";
}

.fa-hotel {
  --fa: "";
}

.fa-earth-asia, .fa-globe-asia {
  --fa: "";
}

.fa-id-card-clip, .fa-id-card-alt {
  --fa: "";
}

.fa-magnifying-glass-plus, .fa-search-plus {
  --fa: "";
}

.fa-thumbs-up {
  --fa: "";
}

.fa-user-clock {
  --fa: "";
}

.fa-hand-dots, .fa-allergies {
  --fa: "";
}

.fa-file-invoice {
  --fa: "";
}

.fa-window-minimize {
  --fa: "";
}

.fa-mug-saucer, .fa-coffee {
  --fa: "";
}

.fa-brush {
  --fa: "";
}

.fa-file-half-dashed {
  --fa: "";
}

.fa-mask {
  --fa: "";
}

.fa-magnifying-glass-minus, .fa-search-minus {
  --fa: "";
}

.fa-ruler-vertical {
  --fa: "";
}

.fa-user-large, .fa-user-alt {
  --fa: "";
}

.fa-train-tram {
  --fa: "";
}

.fa-user-nurse {
  --fa: "";
}

.fa-syringe {
  --fa: "";
}

.fa-cloud-sun {
  --fa: "";
}

.fa-stopwatch-20 {
  --fa: "";
}

.fa-square-full {
  --fa: "";
}

.fa-magnet {
  --fa: "";
}

.fa-jar {
  --fa: "";
}

.fa-note-sticky, .fa-sticky-note {
  --fa: "";
}

.fa-bug-slash {
  --fa: "";
}

.fa-arrow-up-from-water-pump {
  --fa: "";
}

.fa-bone {
  --fa: "";
}

.fa-table-cells-row-unlock {
  --fa: "";
}

.fa-user-injured {
  --fa: "";
}

.fa-face-sad-tear, .fa-sad-tear {
  --fa: "";
}

.fa-plane {
  --fa: "";
}

.fa-tent-arrows-down {
  --fa: "";
}

.fa-exclamation {
  --fa: "!";
}

.fa-arrows-spin {
  --fa: "";
}

.fa-print {
  --fa: "";
}

.fa-turkish-lira-sign, .fa-try, .fa-turkish-lira {
  --fa: "";
}

.fa-dollar-sign, .fa-dollar, .fa-usd {
  --fa: "$";
}

.fa-x {
  --fa: "X";
}

.fa-magnifying-glass-dollar, .fa-search-dollar {
  --fa: "";
}

.fa-users-gear, .fa-users-cog {
  --fa: "";
}

.fa-person-military-pointing {
  --fa: "";
}

.fa-building-columns, .fa-bank, .fa-institution, .fa-museum, .fa-university {
  --fa: "";
}

.fa-umbrella {
  --fa: "";
}

.fa-trowel {
  --fa: "";
}

.fa-d {
  --fa: "D";
}

.fa-stapler {
  --fa: "";
}

.fa-masks-theater, .fa-theater-masks {
  --fa: "";
}

.fa-kip-sign {
  --fa: "";
}

.fa-hand-point-left {
  --fa: "";
}

.fa-handshake-simple, .fa-handshake-alt {
  --fa: "";
}

.fa-jet-fighter, .fa-fighter-jet {
  --fa: "";
}

.fa-square-share-nodes, .fa-share-alt-square {
  --fa: "";
}

.fa-barcode {
  --fa: "";
}

.fa-plus-minus {
  --fa: "";
}

.fa-video, .fa-video-camera {
  --fa: "";
}

.fa-graduation-cap, .fa-mortar-board {
  --fa: "";
}

.fa-hand-holding-medical {
  --fa: "";
}

.fa-person-circle-check {
  --fa: "";
}

.fa-turn-up, .fa-level-up-alt {
  --fa: "";
}

.sr-only, .fa-sr-only, .sr-only-focusable:not(:focus), .fa-sr-only-focusable:not(:focus) {
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  padding: 0;
  position: absolute;
  overflow: hidden;
}

.fa.fa-envelope-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-remove, .fa.fa-close {
  --fa: "";
}

.fa.fa-trash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-clock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-play-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-repeat, .fa.fa-rotate-right {
  --fa: "";
}

.fa.fa-list-alt {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-picture-o, .fa.fa-photo, .fa.fa-image {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-pencil-square-o, .fa.fa-edit {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eye, .fa.fa-eye-slash {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bar-chart, .fa.fa-bar-chart-o {
  --fa: "";
}

.fa.fa-twitter-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thumbs-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-heart-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-linkedin-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-github-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lemon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bookmark-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-twitter, .fa.fa-facebook {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-f {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hdd-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-files-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-floppy-o, .fa.fa-save {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-navicon, .fa.fa-reorder {
  --fa: "";
}

.fa.fa-pinterest, .fa.fa-pinterest-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linkedin {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tachometer, .fa.fa-dashboard {
  --fa: "";
}

.fa.fa-comment-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comments-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lightbulb-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bell-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-text-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-building-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hospital-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mobile, .fa.fa-mobile-phone {
  --fa: "";
}

.fa.fa-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-smile-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-frown-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-meh-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-keyboard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flag-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-o, .fa.fa-star-half-empty, .fa.fa-star-half-full {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-chain-broken, .fa.fa-unlink {
  --fa: "";
}

.fa.fa-calendar-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-maxcdn, .fa.fa-html5, .fa.fa-css3 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-minus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-compass, .fa.fa-caret-square-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-up, .fa.fa-toggle-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-right, .fa.fa-toggle-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eur, .fa.fa-euro {
  --fa: "";
}

.fa.fa-usd, .fa.fa-dollar {
  --fa: "$";
}

.fa.fa-inr, .fa.fa-rupee {
  --fa: "";
}

.fa.fa-jpy, .fa.fa-cny, .fa.fa-rmb, .fa.fa-yen {
  --fa: "";
}

.fa.fa-rub, .fa.fa-ruble, .fa.fa-rouble {
  --fa: "";
}

.fa.fa-krw, .fa.fa-won {
  --fa: "";
}

.fa.fa-btc, .fa.fa-bitcoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube, .fa.fa-xing, .fa.fa-xing-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-play {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dropbox, .fa.fa-stack-overflow, .fa.fa-instagram, .fa.fa-flickr, .fa.fa-adn, .fa.fa-bitbucket, .fa.fa-bitbucket-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr, .fa.fa-tumblr-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-apple, .fa.fa-windows, .fa.fa-android, .fa.fa-linux, .fa.fa-dribbble, .fa.fa-skype, .fa.fa-foursquare, .fa.fa-trello, .fa.fa-gratipay, .fa.fa-gittip {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-sun-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-moon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vk, .fa.fa-weibo, .fa.fa-renren, .fa.fa-pagelines, .fa.fa-stack-exchange {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-left, .fa.fa-toggle-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dot-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vimeo-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-try, .fa.fa-turkish-lira {
  --fa: "";
}

.fa.fa-plus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slack, .fa.fa-wordpress, .fa.fa-openid {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-institution, .fa.fa-bank {
  --fa: "";
}

.fa.fa-yahoo, .fa.fa-google, .fa.fa-reddit, .fa.fa-reddit-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stumbleupon-circle, .fa.fa-stumbleupon, .fa.fa-delicious, .fa.fa-digg, .fa.fa-pied-piper-pp, .fa.fa-pied-piper-alt, .fa.fa-drupal, .fa.fa-joomla, .fa.fa-behance, .fa.fa-behance-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam, .fa.fa-steam-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-spotify, .fa.fa-deviantart, .fa.fa-soundcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-word-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-excel-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-powerpoint-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-image-o, .fa.fa-file-photo-o, .fa.fa-file-picture-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-archive-o, .fa.fa-file-zip-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-audio-o, .fa.fa-file-sound-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-video-o, .fa.fa-file-movie-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-code-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vine, .fa.fa-codepen, .fa.fa-jsfiddle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-life-bouy, .fa.fa-life-buoy, .fa.fa-life-saver, .fa.fa-support {
  --fa: "";
}

.fa.fa-rebel, .fa.fa-ra {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-resistance {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-empire, .fa.fa-ge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git, .fa.fa-hacker-news, .fa.fa-y-combinator-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tencent-weibo, .fa.fa-qq, .fa.fa-weixin, .fa.fa-wechat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-paper-plane-o, .fa.fa-send-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-thin {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-futbol-o, .fa.fa-soccer-ball-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slideshare, .fa.fa-twitch, .fa.fa-yelp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-paypal, .fa.fa-google-wallet, .fa.fa-cc-visa, .fa.fa-cc-mastercard, .fa.fa-cc-discover, .fa.fa-cc-amex, .fa.fa-cc-paypal, .fa.fa-cc-stripe {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lastfm, .fa.fa-lastfm-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ioxhost, .fa.fa-angellist {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-ils, .fa.fa-shekel, .fa.fa-sheqel {
  --fa: "";
}

.fa.fa-buysellads, .fa.fa-connectdevelop, .fa.fa-dashcube, .fa.fa-forumbee, .fa.fa-leanpub, .fa.fa-sellsy, .fa.fa-shirtsinbulk, .fa.fa-simplybuilt, .fa.fa-skyatlas {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-diamond {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-transgender, .fa.fa-intersex {
  --fa: "";
}

.fa.fa-facebook-official {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-p, .fa.fa-whatsapp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viacoin, .fa.fa-medium, .fa.fa-y-combinator, .fa.fa-yc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-optin-monster, .fa.fa-opencart, .fa.fa-expeditedssl {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-battery-4, .fa.fa-battery {
  --fa: "";
}

.fa.fa-object-group, .fa.fa-object-ungroup, .fa.fa-sticky-note-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-cc-jcb, .fa.fa-cc-diners-club {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hand-rock-o, .fa.fa-hand-grab-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-paper-o, .fa.fa-hand-stop-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-scissors-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-lizard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-spock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-pointer-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-peace-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-creative-commons, .fa.fa-gg, .fa.fa-gg-circle, .fa.fa-odnoklassniki, .fa.fa-odnoklassniki-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-get-pocket, .fa.fa-wikipedia-w, .fa.fa-safari, .fa.fa-chrome, .fa.fa-firefox, .fa.fa-opera, .fa.fa-internet-explorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-contao, .fa.fa-500px, .fa.fa-amazon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-minus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-times-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-check-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-commenting-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-houzz, .fa.fa-vimeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-black-tie, .fa.fa-fonticons, .fa.fa-reddit-alien, .fa.fa-edge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-codiepie, .fa.fa-modx, .fa.fa-fort-awesome, .fa.fa-usb, .fa.fa-product-hunt, .fa.fa-mixcloud, .fa.fa-scribd {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-stop-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bluetooth, .fa.fa-bluetooth-b, .fa.fa-gitlab, .fa.fa-wpbeginner, .fa.fa-wpforms, .fa.fa-envira, .fa.fa-wheelchair-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-question-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-deafness, .fa.fa-hard-of-hearing {
  --fa: "";
}

.fa.fa-glide, .fa.fa-glide-g {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo, .fa.fa-viadeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat, .fa.fa-snapchat-ghost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper, .fa.fa-first-order, .fa.fa-yoast, .fa.fa-themeisle, .fa.fa-google-plus-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-circle {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-font-awesome, .fa.fa-fa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-handshake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-book-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-card-o, .fa.fa-vcard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-id-card-o, .fa.fa-drivers-license-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-quora, .fa.fa-free-code-camp, .fa.fa-telegram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thermometer-4, .fa.fa-thermometer {
  --fa: "";
}

.fa.fa-bathtub, .fa.fa-s15 {
  --fa: "";
}

.fa.fa-window-maximize, .fa.fa-window-restore {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-window-close-o, .fa.fa-times-rectangle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bandcamp, .fa.fa-grav, .fa.fa-etsy, .fa.fa-imdb, .fa.fa-ravelry, .fa.fa-eercast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snowflake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-superpowers, .fa.fa-wpexplorer, .fa.fa-meetup {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

:root, :host {
  --fa-style-family-classic: "Font Awesome 6 Free";
  --fa-font-regular: normal 400 1em / 1 "Font Awesome 6 Free";
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-regular-400.woff2?v=896727") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-regular-400.ttf?v=262525") format("truetype");
}

.far, .fa-regular {
  font-weight: 400;
}

:root, :host {
  --fa-style-family-classic: "Font Awesome 6 Free";
  --fa-font-solid: normal 900 1em / 1 "Font Awesome 6 Free";
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 900;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-solid-900.woff2?v=4a6591") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-solid-900.ttf?v=269f97") format("truetype");
}

.fas, [class^="icon-"], [class*=" icon-"], .fa-solid {
  font-weight: 900;
}

:root, :host {
  --fa-style-family-brands: "Font Awesome 6 Brands";
  --fa-font-brands: normal 400 1em / 1 "Font Awesome 6 Brands";
}

@font-face {
  font-family: "Font Awesome 6 Brands";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("../../../media/vendor/fontawesome-free/webfonts/fa-brands-400.woff2?v=cbcf42") format("woff2"), url("../../../media/vendor/fontawesome-free/webfonts/fa-brands-400.ttf?v=15d54d") format("truetype");
}

.fab, .icon-joomla, .fa-brands {
  font-weight: 400;
}

.fa-monero {
  --fa: "";
}

.fa-hooli {
  --fa: "";
}

.fa-yelp {
  --fa: "";
}

.fa-cc-visa {
  --fa: "";
}

.fa-lastfm {
  --fa: "";
}

.fa-shopware {
  --fa: "";
}

.fa-creative-commons-nc {
  --fa: "";
}

.fa-aws {
  --fa: "";
}

.fa-redhat {
  --fa: "";
}

.fa-yoast {
  --fa: "";
}

.fa-cloudflare {
  --fa: "";
}

.fa-ups {
  --fa: "";
}

.fa-pixiv {
  --fa: "";
}

.fa-wpexplorer {
  --fa: "";
}

.fa-dyalog {
  --fa: "";
}

.fa-bity {
  --fa: "";
}

.fa-stackpath {
  --fa: "";
}

.fa-buysellads {
  --fa: "";
}

.fa-first-order {
  --fa: "";
}

.fa-modx {
  --fa: "";
}

.fa-guilded {
  --fa: "";
}

.fa-vnv {
  --fa: "";
}

.fa-square-js, .fa-js-square {
  --fa: "";
}

.fa-microsoft {
  --fa: "";
}

.fa-qq {
  --fa: "";
}

.fa-orcid {
  --fa: "";
}

.fa-java {
  --fa: "";
}

.fa-invision {
  --fa: "";
}

.fa-creative-commons-pd-alt {
  --fa: "";
}

.fa-centercode {
  --fa: "";
}

.fa-glide-g {
  --fa: "";
}

.fa-drupal {
  --fa: "";
}

.fa-jxl {
  --fa: "";
}

.fa-dart-lang {
  --fa: "";
}

.fa-hire-a-helper {
  --fa: "";
}

.fa-creative-commons-by {
  --fa: "";
}

.fa-unity {
  --fa: "";
}

.fa-whmcs {
  --fa: "";
}

.fa-rocketchat {
  --fa: "";
}

.fa-vk {
  --fa: "";
}

.fa-untappd {
  --fa: "";
}

.fa-mailchimp {
  --fa: "";
}

.fa-css3-alt {
  --fa: "";
}

.fa-square-reddit, .fa-reddit-square {
  --fa: "";
}

.fa-vimeo-v {
  --fa: "";
}

.fa-contao {
  --fa: "";
}

.fa-square-font-awesome {
  --fa: "";
}

.fa-deskpro {
  --fa: "";
}

.fa-brave {
  --fa: "";
}

.fa-sistrix {
  --fa: "";
}

.fa-square-instagram, .fa-instagram-square {
  --fa: "";
}

.fa-battle-net {
  --fa: "";
}

.fa-the-red-yeti {
  --fa: "";
}

.fa-square-hacker-news, .fa-hacker-news-square {
  --fa: "";
}

.fa-edge {
  --fa: "";
}

.fa-threads {
  --fa: "";
}

.fa-napster {
  --fa: "";
}

.fa-square-snapchat, .fa-snapchat-square {
  --fa: "";
}

.fa-google-plus-g {
  --fa: "";
}

.fa-artstation {
  --fa: "";
}

.fa-markdown {
  --fa: "";
}

.fa-sourcetree {
  --fa: "";
}

.fa-google-plus {
  --fa: "";
}

.fa-diaspora {
  --fa: "";
}

.fa-foursquare {
  --fa: "";
}

.fa-stack-overflow {
  --fa: "";
}

.fa-github-alt {
  --fa: "";
}

.fa-phoenix-squadron {
  --fa: "";
}

.fa-pagelines {
  --fa: "";
}

.fa-algolia {
  --fa: "";
}

.fa-red-river {
  --fa: "";
}

.fa-creative-commons-sa {
  --fa: "";
}

.fa-safari {
  --fa: "";
}

.fa-google {
  --fa: "";
}

.fa-square-font-awesome-stroke, .fa-font-awesome-alt {
  --fa: "";
}

.fa-atlassian {
  --fa: "";
}

.fa-linkedin-in {
  --fa: "";
}

.fa-digital-ocean {
  --fa: "";
}

.fa-nimblr {
  --fa: "";
}

.fa-chromecast {
  --fa: "";
}

.fa-evernote {
  --fa: "";
}

.fa-hacker-news {
  --fa: "";
}

.fa-creative-commons-sampling {
  --fa: "";
}

.fa-adversal {
  --fa: "";
}

.fa-creative-commons {
  --fa: "";
}

.fa-watchman-monitoring {
  --fa: "";
}

.fa-fonticons {
  --fa: "";
}

.fa-weixin {
  --fa: "";
}

.fa-shirtsinbulk {
  --fa: "";
}

.fa-codepen {
  --fa: "";
}

.fa-git-alt {
  --fa: "";
}

.fa-lyft {
  --fa: "";
}

.fa-rev {
  --fa: "";
}

.fa-windows {
  --fa: "";
}

.fa-wizards-of-the-coast {
  --fa: "";
}

.fa-square-viadeo, .fa-viadeo-square {
  --fa: "";
}

.fa-meetup {
  --fa: "";
}

.fa-centos {
  --fa: "";
}

.fa-adn {
  --fa: "";
}

.fa-cloudsmith {
  --fa: "";
}

.fa-opensuse {
  --fa: "";
}

.fa-pied-piper-alt {
  --fa: "";
}

.fa-square-dribbble, .fa-dribbble-square {
  --fa: "";
}

.fa-codiepie {
  --fa: "";
}

.fa-node {
  --fa: "";
}

.fa-mix {
  --fa: "";
}

.fa-steam {
  --fa: "";
}

.fa-cc-apple-pay {
  --fa: "";
}

.fa-scribd {
  --fa: "";
}

.fa-debian {
  --fa: "";
}

.fa-openid {
  --fa: "";
}

.fa-instalod {
  --fa: "";
}

.fa-files-pinwheel {
  --fa: "";
}

.fa-expeditedssl {
  --fa: "";
}

.fa-sellcast {
  --fa: "";
}

.fa-square-twitter, .fa-twitter-square {
  --fa: "";
}

.fa-r-project {
  --fa: "";
}

.fa-delicious {
  --fa: "";
}

.fa-freebsd {
  --fa: "";
}

.fa-vuejs {
  --fa: "";
}

.fa-accusoft {
  --fa: "";
}

.fa-ioxhost {
  --fa: "";
}

.fa-fonticons-fi {
  --fa: "";
}

.fa-app-store {
  --fa: "";
}

.fa-cc-mastercard {
  --fa: "";
}

.fa-itunes-note {
  --fa: "";
}

.fa-golang {
  --fa: "";
}

.fa-kickstarter, .fa-square-kickstarter {
  --fa: "";
}

.fa-grav {
  --fa: "";
}

.fa-weibo {
  --fa: "";
}

.fa-uncharted {
  --fa: "";
}

.fa-firstdraft {
  --fa: "";
}

.fa-square-youtube, .fa-youtube-square {
  --fa: "";
}

.fa-wikipedia-w {
  --fa: "";
}

.fa-wpressr, .fa-rendact {
  --fa: "";
}

.fa-angellist {
  --fa: "";
}

.fa-galactic-republic {
  --fa: "";
}

.fa-nfc-directional {
  --fa: "";
}

.fa-skype {
  --fa: "";
}

.fa-joget {
  --fa: "";
}

.fa-fedora {
  --fa: "";
}

.fa-stripe-s {
  --fa: "";
}

.fa-meta {
  --fa: "";
}

.fa-laravel {
  --fa: "";
}

.fa-hotjar {
  --fa: "";
}

.fa-bluetooth-b {
  --fa: "";
}

.fa-square-letterboxd {
  --fa: "";
}

.fa-sticker-mule {
  --fa: "";
}

.fa-creative-commons-zero {
  --fa: "";
}

.fa-hips {
  --fa: "";
}

.fa-css {
  --fa: "";
}

.fa-behance {
  --fa: "";
}

.fa-reddit {
  --fa: "";
}

.fa-discord {
  --fa: "";
}

.fa-chrome {
  --fa: "";
}

.fa-app-store-ios {
  --fa: "";
}

.fa-cc-discover {
  --fa: "";
}

.fa-wpbeginner {
  --fa: "";
}

.fa-confluence {
  --fa: "";
}

.fa-shoelace {
  --fa: "";
}

.fa-mdb {
  --fa: "";
}

.fa-dochub {
  --fa: "";
}

.fa-accessible-icon {
  --fa: "";
}

.fa-ebay {
  --fa: "";
}

.fa-amazon {
  --fa: "";
}

.fa-unsplash {
  --fa: "";
}

.fa-yarn {
  --fa: "";
}

.fa-square-steam, .fa-steam-square {
  --fa: "";
}

.fa-500px {
  --fa: "";
}

.fa-square-vimeo, .fa-vimeo-square {
  --fa: "";
}

.fa-asymmetrik {
  --fa: "";
}

.fa-font-awesome, .fa-font-awesome-flag, .fa-font-awesome-logo-full {
  --fa: "";
}

.fa-gratipay {
  --fa: "";
}

.fa-apple {
  --fa: "";
}

.fa-hive {
  --fa: "";
}

.fa-gitkraken {
  --fa: "";
}

.fa-keybase {
  --fa: "";
}

.fa-apple-pay {
  --fa: "";
}

.fa-padlet {
  --fa: "";
}

.fa-amazon-pay {
  --fa: "";
}

.fa-square-github, .fa-github-square {
  --fa: "";
}

.fa-stumbleupon {
  --fa: "";
}

.fa-fedex {
  --fa: "";
}

.fa-phoenix-framework {
  --fa: "";
}

.fa-shopify {
  --fa: "";
}

.fa-neos {
  --fa: "";
}

.fa-square-threads {
  --fa: "";
}

.fa-hackerrank {
  --fa: "";
}

.fa-researchgate {
  --fa: "";
}

.fa-swift {
  --fa: "";
}

.fa-angular {
  --fa: "";
}

.fa-speakap {
  --fa: "";
}

.fa-angrycreative {
  --fa: "";
}

.fa-y-combinator {
  --fa: "";
}

.fa-empire {
  --fa: "";
}

.fa-envira {
  --fa: "";
}

.fa-google-scholar {
  --fa: "";
}

.fa-square-gitlab, .fa-gitlab-square {
  --fa: "";
}

.fa-studiovinari {
  --fa: "";
}

.fa-pied-piper {
  --fa: "";
}

.fa-wordpress {
  --fa: "";
}

.fa-product-hunt {
  --fa: "";
}

.fa-firefox {
  --fa: "";
}

.fa-linode {
  --fa: "";
}

.fa-goodreads {
  --fa: "";
}

.fa-square-odnoklassniki, .fa-odnoklassniki-square {
  --fa: "";
}

.fa-jsfiddle {
  --fa: "";
}

.fa-sith {
  --fa: "";
}

.fa-themeisle {
  --fa: "";
}

.fa-page4 {
  --fa: "";
}

.fa-hashnode {
  --fa: "";
}

.fa-react {
  --fa: "";
}

.fa-cc-paypal {
  --fa: "";
}

.fa-squarespace {
  --fa: "";
}

.fa-cc-stripe {
  --fa: "";
}

.fa-creative-commons-share {
  --fa: "";
}

.fa-bitcoin {
  --fa: "";
}

.fa-keycdn {
  --fa: "";
}

.fa-opera {
  --fa: "";
}

.fa-itch-io {
  --fa: "";
}

.fa-umbraco {
  --fa: "";
}

.fa-galactic-senate {
  --fa: "";
}

.fa-ubuntu {
  --fa: "";
}

.fa-draft2digital {
  --fa: "";
}

.fa-stripe {
  --fa: "";
}

.fa-houzz {
  --fa: "";
}

.fa-gg {
  --fa: "";
}

.fa-dhl {
  --fa: "";
}

.fa-square-pinterest, .fa-pinterest-square {
  --fa: "";
}

.fa-xing {
  --fa: "";
}

.fa-blackberry {
  --fa: "";
}

.fa-creative-commons-pd {
  --fa: "";
}

.fa-playstation {
  --fa: "";
}

.fa-quinscape {
  --fa: "";
}

.fa-less {
  --fa: "";
}

.fa-blogger-b {
  --fa: "";
}

.fa-opencart {
  --fa: "";
}

.fa-vine {
  --fa: "";
}

.fa-signal-messenger {
  --fa: "";
}

.fa-paypal {
  --fa: "";
}

.fa-gitlab {
  --fa: "";
}

.fa-typo3 {
  --fa: "";
}

.fa-reddit-alien {
  --fa: "";
}

.fa-yahoo {
  --fa: "";
}

.fa-dailymotion {
  --fa: "";
}

.fa-affiliatetheme {
  --fa: "";
}

.fa-pied-piper-pp {
  --fa: "";
}

.fa-bootstrap {
  --fa: "";
}

.fa-odnoklassniki {
  --fa: "";
}

.fa-nfc-symbol {
  --fa: "";
}

.fa-mintbit {
  --fa: "";
}

.fa-ethereum {
  --fa: "";
}

.fa-speaker-deck {
  --fa: "";
}

.fa-creative-commons-nc-eu {
  --fa: "";
}

.fa-patreon {
  --fa: "";
}

.fa-avianex {
  --fa: "";
}

.fa-ello {
  --fa: "";
}

.fa-gofore {
  --fa: "";
}

.fa-bimobject {
  --fa: "";
}

.fa-brave-reverse {
  --fa: "";
}

.fa-facebook-f {
  --fa: "";
}

.fa-square-google-plus, .fa-google-plus-square {
  --fa: "";
}

.fa-web-awesome {
  --fa: "";
}

.fa-mandalorian {
  --fa: "";
}

.fa-first-order-alt {
  --fa: "";
}

.fa-osi {
  --fa: "";
}

.fa-google-wallet {
  --fa: "";
}

.fa-d-and-d-beyond {
  --fa: "";
}

.fa-periscope {
  --fa: "";
}

.fa-fulcrum {
  --fa: "";
}

.fa-cloudscale {
  --fa: "";
}

.fa-forumbee {
  --fa: "";
}

.fa-mizuni {
  --fa: "";
}

.fa-schlix {
  --fa: "";
}

.fa-square-xing, .fa-xing-square {
  --fa: "";
}

.fa-bandcamp {
  --fa: "";
}

.fa-wpforms {
  --fa: "";
}

.fa-cloudversify {
  --fa: "";
}

.fa-usps {
  --fa: "";
}

.fa-megaport {
  --fa: "";
}

.fa-magento {
  --fa: "";
}

.fa-spotify {
  --fa: "";
}

.fa-optin-monster {
  --fa: "";
}

.fa-fly {
  --fa: "";
}

.fa-square-bluesky {
  --fa: "";
}

.fa-aviato {
  --fa: "";
}

.fa-itunes {
  --fa: "";
}

.fa-cuttlefish {
  --fa: "";
}

.fa-blogger {
  --fa: "";
}

.fa-flickr {
  --fa: "";
}

.fa-viber {
  --fa: "";
}

.fa-soundcloud {
  --fa: "";
}

.fa-digg {
  --fa: "";
}

.fa-tencent-weibo {
  --fa: "";
}

.fa-letterboxd {
  --fa: "";
}

.fa-symfony {
  --fa: "";
}

.fa-maxcdn {
  --fa: "";
}

.fa-etsy {
  --fa: "";
}

.fa-facebook-messenger {
  --fa: "";
}

.fa-audible {
  --fa: "";
}

.fa-think-peaks {
  --fa: "";
}

.fa-bilibili {
  --fa: "";
}

.fa-erlang {
  --fa: "";
}

.fa-x-twitter {
  --fa: "";
}

.fa-cotton-bureau {
  --fa: "";
}

.fa-dashcube {
  --fa: "";
}

.fa-42-group, .fa-innosoft {
  --fa: "";
}

.fa-stack-exchange {
  --fa: "";
}

.fa-elementor {
  --fa: "";
}

.fa-square-pied-piper, .fa-pied-piper-square {
  --fa: "";
}

.fa-creative-commons-nd {
  --fa: "";
}

.fa-palfed {
  --fa: "";
}

.fa-superpowers {
  --fa: "";
}

.fa-resolving {
  --fa: "";
}

.fa-xbox {
  --fa: "";
}

.fa-square-web-awesome-stroke {
  --fa: "";
}

.fa-searchengin {
  --fa: "";
}

.fa-tiktok {
  --fa: "";
}

.fa-square-facebook, .fa-facebook-square {
  --fa: "";
}

.fa-renren {
  --fa: "";
}

.fa-linux {
  --fa: "";
}

.fa-glide {
  --fa: "";
}

.fa-linkedin {
  --fa: "";
}

.fa-hubspot {
  --fa: "";
}

.fa-deploydog {
  --fa: "";
}

.fa-twitch {
  --fa: "";
}

.fa-flutter {
  --fa: "";
}

.fa-ravelry {
  --fa: "";
}

.fa-mixer {
  --fa: "";
}

.fa-square-lastfm, .fa-lastfm-square {
  --fa: "";
}

.fa-vimeo {
  --fa: "";
}

.fa-mendeley {
  --fa: "";
}

.fa-uniregistry {
  --fa: "";
}

.fa-figma {
  --fa: "";
}

.fa-creative-commons-remix {
  --fa: "";
}

.fa-cc-amazon-pay {
  --fa: "";
}

.fa-dropbox {
  --fa: "";
}

.fa-instagram {
  --fa: "";
}

.fa-cmplid {
  --fa: "";
}

.fa-upwork {
  --fa: "";
}

.fa-facebook {
  --fa: "";
}

.fa-gripfire {
  --fa: "";
}

.fa-jedi-order {
  --fa: "";
}

.fa-uikit {
  --fa: "";
}

.fa-fort-awesome-alt {
  --fa: "";
}

.fa-phabricator {
  --fa: "";
}

.fa-ussunnah {
  --fa: "";
}

.fa-earlybirds {
  --fa: "";
}

.fa-trade-federation {
  --fa: "";
}

.fa-autoprefixer {
  --fa: "";
}

.fa-whatsapp {
  --fa: "";
}

.fa-square-upwork {
  --fa: "";
}

.fa-slideshare {
  --fa: "";
}

.fa-google-play {
  --fa: "";
}

.fa-viadeo {
  --fa: "";
}

.fa-line {
  --fa: "";
}

.fa-google-drive {
  --fa: "";
}

.fa-servicestack {
  --fa: "";
}

.fa-simplybuilt {
  --fa: "";
}

.fa-bitbucket {
  --fa: "";
}

.fa-imdb {
  --fa: "";
}

.fa-deezer {
  --fa: "";
}

.fa-raspberry-pi {
  --fa: "";
}

.fa-jira {
  --fa: "";
}

.fa-docker {
  --fa: "";
}

.fa-screenpal {
  --fa: "";
}

.fa-bluetooth {
  --fa: "";
}

.fa-gitter {
  --fa: "";
}

.fa-d-and-d {
  --fa: "";
}

.fa-microblog {
  --fa: "";
}

.fa-cc-diners-club {
  --fa: "";
}

.fa-gg-circle {
  --fa: "";
}

.fa-pied-piper-hat {
  --fa: "";
}

.fa-kickstarter-k {
  --fa: "";
}

.fa-yandex {
  --fa: "";
}

.fa-readme {
  --fa: "";
}

.fa-html5 {
  --fa: "";
}

.fa-sellsy {
  --fa: "";
}

.fa-square-web-awesome {
  --fa: "";
}

.fa-sass {
  --fa: "";
}

.fa-wirsindhandwerk, .fa-wsh {
  --fa: "";
}

.fa-buromobelexperte {
  --fa: "";
}

.fa-salesforce {
  --fa: "";
}

.fa-octopus-deploy {
  --fa: "";
}

.fa-medapps {
  --fa: "";
}

.fa-ns8 {
  --fa: "";
}

.fa-pinterest-p {
  --fa: "";
}

.fa-apper {
  --fa: "";
}

.fa-fort-awesome {
  --fa: "";
}

.fa-waze {
  --fa: "";
}

.fa-bluesky {
  --fa: "";
}

.fa-cc-jcb {
  --fa: "";
}

.fa-snapchat, .fa-snapchat-ghost {
  --fa: "";
}

.fa-fantasy-flight-games {
  --fa: "";
}

.fa-rust {
  --fa: "";
}

.fa-wix {
  --fa: "";
}

.fa-square-behance, .fa-behance-square {
  --fa: "";
}

.fa-supple {
  --fa: "";
}

.fa-webflow {
  --fa: "";
}

.fa-rebel {
  --fa: "";
}

.fa-css3 {
  --fa: "";
}

.fa-staylinked {
  --fa: "";
}

.fa-kaggle {
  --fa: "";
}

.fa-space-awesome {
  --fa: "";
}

.fa-deviantart {
  --fa: "";
}

.fa-cpanel {
  --fa: "";
}

.fa-goodreads-g {
  --fa: "";
}

.fa-square-git, .fa-git-square {
  --fa: "";
}

.fa-square-tumblr, .fa-tumblr-square {
  --fa: "";
}

.fa-trello {
  --fa: "";
}

.fa-creative-commons-nc-jp {
  --fa: "";
}

.fa-get-pocket {
  --fa: "";
}

.fa-perbyte {
  --fa: "";
}

.fa-grunt {
  --fa: "";
}

.fa-weebly {
  --fa: "";
}

.fa-connectdevelop {
  --fa: "";
}

.fa-leanpub {
  --fa: "";
}

.fa-black-tie {
  --fa: "";
}

.fa-themeco {
  --fa: "";
}

.fa-python {
  --fa: "";
}

.fa-android {
  --fa: "";
}

.fa-bots {
  --fa: "";
}

.fa-free-code-camp {
  --fa: "";
}

.fa-hornbill {
  --fa: "";
}

.fa-js {
  --fa: "";
}

.fa-ideal {
  --fa: "";
}

.fa-git {
  --fa: "";
}

.fa-dev {
  --fa: "";
}

.fa-sketch {
  --fa: "";
}

.fa-yandex-international {
  --fa: "";
}

.fa-cc-amex {
  --fa: "";
}

.fa-uber {
  --fa: "";
}

.fa-github {
  --fa: "";
}

.fa-php {
  --fa: "";
}

.fa-alipay {
  --fa: "";
}

.fa-youtube {
  --fa: "";
}

.fa-skyatlas {
  --fa: "";
}

.fa-firefox-browser {
  --fa: "";
}

.fa-replyd {
  --fa: "";
}

.fa-suse {
  --fa: "";
}

.fa-jenkins {
  --fa: "";
}

.fa-twitter {
  --fa: "";
}

.fa-rockrms {
  --fa: "";
}

.fa-pinterest {
  --fa: "";
}

.fa-buffer {
  --fa: "";
}

.fa-npm {
  --fa: "";
}

.fa-yammer {
  --fa: "";
}

.fa-btc {
  --fa: "";
}

.fa-dribbble {
  --fa: "";
}

.fa-stumbleupon-circle {
  --fa: "";
}

.fa-internet-explorer {
  --fa: "";
}

.fa-stubber {
  --fa: "";
}

.fa-telegram, .fa-telegram-plane {
  --fa: "";
}

.fa-old-republic {
  --fa: "";
}

.fa-odysee {
  --fa: "";
}

.fa-square-whatsapp, .fa-whatsapp-square {
  --fa: "";
}

.fa-node-js {
  --fa: "";
}

.fa-edge-legacy {
  --fa: "";
}

.fa-slack, .fa-slack-hash {
  --fa: "";
}

.fa-medrt {
  --fa: "";
}

.fa-usb {
  --fa: "";
}

.fa-tumblr {
  --fa: "";
}

.fa-vaadin {
  --fa: "";
}

.fa-quora {
  --fa: "";
}

.fa-square-x-twitter {
  --fa: "";
}

.fa-reacteurope {
  --fa: "";
}

.fa-medium, .fa-medium-m {
  --fa: "";
}

.fa-amilia {
  --fa: "";
}

.fa-mixcloud {
  --fa: "";
}

.fa-flipboard {
  --fa: "";
}

.fa-viacoin {
  --fa: "";
}

.fa-critical-role {
  --fa: "";
}

.fa-sitrox {
  --fa: "";
}

.fa-discourse {
  --fa: "";
}

.fa-joomla {
  --fa: "";
}

.fa-mastodon {
  --fa: "";
}

.fa-airbnb {
  --fa: "";
}

.fa-wolf-pack-battalion {
  --fa: "";
}

.fa-buy-n-large {
  --fa: "";
}

.fa-gulp {
  --fa: "";
}

.fa-creative-commons-sampling-plus {
  --fa: "";
}

.fa-strava {
  --fa: "";
}

.fa-ember {
  --fa: "";
}

.fa-canadian-maple-leaf {
  --fa: "";
}

.fa-teamspeak {
  --fa: "";
}

.fa-pushed {
  --fa: "";
}

.fa-wordpress-simple {
  --fa: "";
}

.fa-nutritionix {
  --fa: "";
}

.fa-wodu {
  --fa: "";
}

.fa-google-pay {
  --fa: "";
}

.fa-intercom {
  --fa: "";
}

.fa-zhihu {
  --fa: "";
}

.fa-korvue {
  --fa: "";
}

.fa-pix {
  --fa: "";
}

.fa-steam-symbol {
  --fa: "";
}

.fa.fa-glass {
  --fa: "";
}

.fa.fa-envelope-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-remove, .fa.fa-close {
  --fa: "";
}

.fa.fa-gear {
  --fa: "";
}

.fa.fa-trash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-home {
  --fa: "";
}

.fa.fa-file-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-clock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-play-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-repeat, .fa.fa-rotate-right {
  --fa: "";
}

.fa.fa-refresh {
  --fa: "";
}

.fa.fa-list-alt {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dedent {
  --fa: "";
}

.fa.fa-video-camera {
  --fa: "";
}

.fa.fa-picture-o, .fa.fa-photo, .fa.fa-image {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-marker {
  --fa: "";
}

.fa.fa-pencil-square-o, .fa.fa-edit {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-share-square-o {
  --fa: "";
}

.fa.fa-check-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrows {
  --fa: "";
}

.fa.fa-times-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mail-forward {
  --fa: "";
}

.fa.fa-expand {
  --fa: "";
}

.fa.fa-compress {
  --fa: "";
}

.fa.fa-eye, .fa.fa-eye-slash {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-warning {
  --fa: "";
}

.fa.fa-calendar {
  --fa: "";
}

.fa.fa-arrows-v {
  --fa: "";
}

.fa.fa-arrows-h {
  --fa: "";
}

.fa.fa-bar-chart, .fa.fa-bar-chart-o {
  --fa: "";
}

.fa.fa-twitter-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gears {
  --fa: "";
}

.fa.fa-thumbs-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-heart-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-sign-out {
  --fa: "";
}

.fa.fa-linkedin-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thumb-tack {
  --fa: "";
}

.fa.fa-external-link {
  --fa: "";
}

.fa.fa-sign-in {
  --fa: "";
}

.fa.fa-github-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lemon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bookmark-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-twitter, .fa.fa-facebook {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook {
  --fa: "";
}

.fa.fa-facebook-f {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-github {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-credit-card {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-feed {
  --fa: "";
}

.fa.fa-hdd-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-globe {
  --fa: "";
}

.fa.fa-tasks {
  --fa: "";
}

.fa.fa-arrows-alt {
  --fa: "";
}

.fa.fa-group {
  --fa: "";
}

.fa.fa-chain {
  --fa: "";
}

.fa.fa-cut {
  --fa: "";
}

.fa.fa-files-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-floppy-o, .fa.fa-save {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-navicon, .fa.fa-reorder {
  --fa: "";
}

.fa.fa-magic {
  --fa: "";
}

.fa.fa-pinterest, .fa.fa-pinterest-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-square {
  --fa: "";
}

.fa.fa-google-plus-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-money {
  --fa: "";
}

.fa.fa-unsorted {
  --fa: "";
}

.fa.fa-sort-desc {
  --fa: "";
}

.fa.fa-sort-asc {
  --fa: "";
}

.fa.fa-linkedin {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-rotate-left {
  --fa: "";
}

.fa.fa-legal {
  --fa: "";
}

.fa.fa-tachometer, .fa.fa-dashboard {
  --fa: "";
}

.fa.fa-comment-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comments-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flash {
  --fa: "";
}

.fa.fa-clipboard {
  --fa: "";
}

.fa.fa-lightbulb-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-exchange {
  --fa: "";
}

.fa.fa-cloud-download {
  --fa: "";
}

.fa.fa-cloud-upload {
  --fa: "";
}

.fa.fa-bell-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-cutlery {
  --fa: "";
}

.fa.fa-file-text-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-building-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hospital-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-tablet {
  --fa: "";
}

.fa.fa-mobile, .fa.fa-mobile-phone {
  --fa: "";
}

.fa.fa-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mail-reply {
  --fa: "";
}

.fa.fa-github-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-folder-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-smile-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-frown-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-meh-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-keyboard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flag-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-mail-reply-all {
  --fa: "";
}

.fa.fa-star-half-o, .fa.fa-star-half-empty, .fa.fa-star-half-full {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-code-fork {
  --fa: "";
}

.fa.fa-chain-broken, .fa.fa-unlink {
  --fa: "";
}

.fa.fa-calendar-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-maxcdn, .fa.fa-html5, .fa.fa-css3 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-unlock-alt {
  --fa: "";
}

.fa.fa-minus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-level-up {
  --fa: "";
}

.fa.fa-level-down {
  --fa: "";
}

.fa.fa-pencil-square {
  --fa: "";
}

.fa.fa-external-link-square {
  --fa: "";
}

.fa.fa-compass, .fa.fa-caret-square-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-down {
  --fa: "";
}

.fa.fa-toggle-down {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-up, .fa.fa-toggle-up {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-right, .fa.fa-toggle-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eur, .fa.fa-euro {
  --fa: "";
}

.fa.fa-gbp {
  --fa: "";
}

.fa.fa-usd, .fa.fa-dollar {
  --fa: "$";
}

.fa.fa-inr, .fa.fa-rupee {
  --fa: "";
}

.fa.fa-jpy, .fa.fa-cny, .fa.fa-rmb, .fa.fa-yen {
  --fa: "";
}

.fa.fa-rub, .fa.fa-ruble, .fa.fa-rouble {
  --fa: "";
}

.fa.fa-krw, .fa.fa-won {
  --fa: "";
}

.fa.fa-btc, .fa.fa-bitcoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitcoin {
  --fa: "";
}

.fa.fa-file-text {
  --fa: "";
}

.fa.fa-sort-alpha-asc {
  --fa: "";
}

.fa.fa-sort-alpha-desc {
  --fa: "";
}

.fa.fa-sort-amount-asc {
  --fa: "";
}

.fa.fa-sort-amount-desc {
  --fa: "";
}

.fa.fa-sort-numeric-asc {
  --fa: "";
}

.fa.fa-sort-numeric-desc {
  --fa: "";
}

.fa.fa-youtube-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube, .fa.fa-xing, .fa.fa-xing-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-xing-square {
  --fa: "";
}

.fa.fa-youtube-play {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dropbox, .fa.fa-stack-overflow, .fa.fa-instagram, .fa.fa-flickr, .fa.fa-adn, .fa.fa-bitbucket, .fa.fa-bitbucket-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitbucket-square {
  --fa: "";
}

.fa.fa-tumblr, .fa.fa-tumblr-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr-square {
  --fa: "";
}

.fa.fa-long-arrow-down {
  --fa: "";
}

.fa.fa-long-arrow-up {
  --fa: "";
}

.fa.fa-long-arrow-left {
  --fa: "";
}

.fa.fa-long-arrow-right {
  --fa: "";
}

.fa.fa-apple, .fa.fa-windows, .fa.fa-android, .fa.fa-linux, .fa.fa-dribbble, .fa.fa-skype, .fa.fa-foursquare, .fa.fa-trello, .fa.fa-gratipay, .fa.fa-gittip {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gittip {
  --fa: "";
}

.fa.fa-sun-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-moon-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vk, .fa.fa-weibo, .fa.fa-renren, .fa.fa-pagelines, .fa.fa-stack-exchange {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-left, .fa.fa-toggle-left {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dot-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vimeo-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-try, .fa.fa-turkish-lira {
  --fa: "";
}

.fa.fa-plus-square-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slack, .fa.fa-wordpress, .fa.fa-openid {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-institution, .fa.fa-bank {
  --fa: "";
}

.fa.fa-mortar-board {
  --fa: "";
}

.fa.fa-yahoo, .fa.fa-google, .fa.fa-reddit, .fa.fa-reddit-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-reddit-square {
  --fa: "";
}

.fa.fa-stumbleupon-circle, .fa.fa-stumbleupon, .fa.fa-delicious, .fa.fa-digg, .fa.fa-pied-piper-pp, .fa.fa-pied-piper-alt, .fa.fa-drupal, .fa.fa-joomla, .fa.fa-behance, .fa.fa-behance-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-behance-square {
  --fa: "";
}

.fa.fa-steam, .fa.fa-steam-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam-square {
  --fa: "";
}

.fa.fa-automobile {
  --fa: "";
}

.fa.fa-cab {
  --fa: "";
}

.fa.fa-spotify, .fa.fa-deviantart, .fa.fa-soundcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-word-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-excel-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-powerpoint-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-image-o, .fa.fa-file-photo-o, .fa.fa-file-picture-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-archive-o, .fa.fa-file-zip-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-audio-o, .fa.fa-file-sound-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-video-o, .fa.fa-file-movie-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-code-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vine, .fa.fa-codepen, .fa.fa-jsfiddle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-life-bouy, .fa.fa-life-buoy, .fa.fa-life-saver, .fa.fa-support {
  --fa: "";
}

.fa.fa-circle-o-notch {
  --fa: "";
}

.fa.fa-rebel, .fa.fa-ra {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ra {
  --fa: "";
}

.fa.fa-resistance {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-empire, .fa.fa-ge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ge {
  --fa: "";
}

.fa.fa-git-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git, .fa.fa-hacker-news, .fa.fa-y-combinator-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-y-combinator-square {
  --fa: "";
}

.fa.fa-yc-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tencent-weibo, .fa.fa-qq, .fa.fa-weixin, .fa.fa-wechat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wechat {
  --fa: "";
}

.fa.fa-send {
  --fa: "";
}

.fa.fa-paper-plane-o, .fa.fa-send-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-thin {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-header {
  --fa: "";
}

.fa.fa-futbol-o, .fa.fa-soccer-ball-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-slideshare, .fa.fa-twitch, .fa.fa-yelp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-paypal, .fa.fa-google-wallet, .fa.fa-cc-visa, .fa.fa-cc-mastercard, .fa.fa-cc-discover, .fa.fa-cc-amex, .fa.fa-cc-paypal, .fa.fa-cc-stripe {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-trash {
  --fa: "";
}

.fa.fa-copyright {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eyedropper {
  --fa: "";
}

.fa.fa-area-chart {
  --fa: "";
}

.fa.fa-pie-chart {
  --fa: "";
}

.fa.fa-line-chart {
  --fa: "";
}

.fa.fa-lastfm, .fa.fa-lastfm-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lastfm-square {
  --fa: "";
}

.fa.fa-ioxhost, .fa.fa-angellist {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-ils, .fa.fa-shekel, .fa.fa-sheqel {
  --fa: "";
}

.fa.fa-buysellads, .fa.fa-connectdevelop, .fa.fa-dashcube, .fa.fa-forumbee, .fa.fa-leanpub, .fa.fa-sellsy, .fa.fa-shirtsinbulk, .fa.fa-simplybuilt, .fa.fa-skyatlas {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-diamond {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-transgender, .fa.fa-intersex {
  --fa: "";
}

.fa.fa-transgender-alt {
  --fa: "";
}

.fa.fa-facebook-official {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-p, .fa.fa-whatsapp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hotel {
  --fa: "";
}

.fa.fa-viacoin, .fa.fa-medium, .fa.fa-y-combinator, .fa.fa-yc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc {
  --fa: "";
}

.fa.fa-optin-monster, .fa.fa-opencart, .fa.fa-expeditedssl {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-battery-4, .fa.fa-battery {
  --fa: "";
}

.fa.fa-battery-3 {
  --fa: "";
}

.fa.fa-battery-2 {
  --fa: "";
}

.fa.fa-battery-1 {
  --fa: "";
}

.fa.fa-battery-0 {
  --fa: "";
}

.fa.fa-object-group, .fa.fa-object-ungroup, .fa.fa-sticky-note-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-sticky-note-o {
  --fa: "";
}

.fa.fa-cc-jcb, .fa.fa-cc-diners-club {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-clone {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hourglass-o {
  --fa: "";
}

.fa.fa-hourglass-1 {
  --fa: "";
}

.fa.fa-hourglass-2 {
  --fa: "";
}

.fa.fa-hourglass-3 {
  --fa: "";
}

.fa.fa-hand-rock-o, .fa.fa-hand-grab-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-paper-o, .fa.fa-hand-stop-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-scissors-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-lizard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-spock-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-pointer-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-peace-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-registered {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-creative-commons, .fa.fa-gg, .fa.fa-gg-circle, .fa.fa-odnoklassniki, .fa.fa-odnoklassniki-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-odnoklassniki-square {
  --fa: "";
}

.fa.fa-get-pocket, .fa.fa-wikipedia-w, .fa.fa-safari, .fa.fa-chrome, .fa.fa-firefox, .fa.fa-opera, .fa.fa-internet-explorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-television {
  --fa: "";
}

.fa.fa-contao, .fa.fa-500px, .fa.fa-amazon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-minus-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-times-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-check-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-commenting {
  --fa: "";
}

.fa.fa-commenting-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-houzz, .fa.fa-vimeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-vimeo {
  --fa: "";
}

.fa.fa-black-tie, .fa.fa-fonticons, .fa.fa-reddit-alien, .fa.fa-edge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-credit-card-alt {
  --fa: "";
}

.fa.fa-codiepie, .fa.fa-modx, .fa.fa-fort-awesome, .fa.fa-usb, .fa.fa-product-hunt, .fa.fa-mixcloud, .fa.fa-scribd {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-stop-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bluetooth, .fa.fa-bluetooth-b, .fa.fa-gitlab, .fa.fa-wpbeginner, .fa.fa-wpforms, .fa.fa-envira, .fa.fa-wheelchair-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wheelchair-alt {
  --fa: "";
}

.fa.fa-question-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-volume-control-phone {
  --fa: "";
}

.fa.fa-asl-interpreting {
  --fa: "";
}

.fa.fa-deafness, .fa.fa-hard-of-hearing {
  --fa: "";
}

.fa.fa-glide, .fa.fa-glide-g {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-signing {
  --fa: "";
}

.fa.fa-viadeo, .fa.fa-viadeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo-square {
  --fa: "";
}

.fa.fa-snapchat, .fa.fa-snapchat-ghost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-ghost {
  --fa: "";
}

.fa.fa-snapchat-square {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper, .fa.fa-first-order, .fa.fa-yoast, .fa.fa-themeisle, .fa.fa-google-plus-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-official {
  --fa: "";
}

.fa.fa-google-plus-circle {
  --fa: "";
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-font-awesome, .fa.fa-fa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-fa {
  --fa: "";
}

.fa.fa-handshake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-open-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-linode {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-address-book-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vcard {
  --fa: "";
}

.fa.fa-address-card-o, .fa.fa-vcard-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-circle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-id-badge {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-drivers-license {
  --fa: "";
}

.fa.fa-id-card-o, .fa.fa-drivers-license-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-quora, .fa.fa-free-code-camp, .fa.fa-telegram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thermometer-4, .fa.fa-thermometer {
  --fa: "";
}

.fa.fa-thermometer-3 {
  --fa: "";
}

.fa.fa-thermometer-2 {
  --fa: "";
}

.fa.fa-thermometer-1 {
  --fa: "";
}

.fa.fa-thermometer-0 {
  --fa: "";
}

.fa.fa-bathtub, .fa.fa-s15 {
  --fa: "";
}

.fa.fa-window-maximize, .fa.fa-window-restore {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-rectangle {
  --fa: "";
}

.fa.fa-window-close-o, .fa.fa-times-rectangle-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bandcamp, .fa.fa-grav, .fa.fa-etsy, .fa.fa-imdb, .fa.fa-ravelry, .fa.fa-eercast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-eercast {
  --fa: "";
}

.fa.fa-snowflake-o {
  --fa: "";
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-superpowers, .fa.fa-wpexplorer, .fa.fa-meetup {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

[class^="icon-"], [class*=" icon-"] {
  speak: none;
}

[class^="icon-"].disabled, [class*=" icon-"].disabled, [class^="fa-"].disabled, [class*=" fa-"].disabled {
  font-weight: 400;
}

.icon-joomla:before {
  content: "";
}

.icon-accessible:before {
  content: "";
}

.icon-add:before {
  content: "+";
}

.icon-address-book:before, .icon-address:before {
  content: "";
}

.icon-align-justify:before {
  content: "";
}

.icon-angle-double-left:before {
  content: "";
}

.icon-angle-double-right:before {
  content: "";
}

.icon-angle-down:before {
  content: "";
}

.icon-angle-left:before {
  content: "";
}

.icon-angle-right:before {
  content: "";
}

.icon-angle-up:before {
  content: "";
}

.icon-apply:before {
  content: "";
}

.icon-archive:before {
  content: "";
}

.icon-arrow-down-2:before {
  content: "";
}

.icon-arrow-down-3:before {
  content: "";
}

.icon-arrow-down-4:before {
  content: "";
}

.icon-arrow-down:before {
  content: "";
}

.icon-arrow-first:before {
  content: "";
}

.icon-arrow-last:before {
  content: "";
}

.icon-arrow-left-2:before {
  content: "";
}

.icon-arrow-left-3:before {
  content: "";
}

.icon-arrow-left-4:before {
  content: "";
}

.icon-arrow-left:before {
  content: "";
}

.icon-arrow-right-2:before {
  content: "";
}

.icon-arrow-right-3:before {
  content: "";
}

.icon-arrow-right-4:before {
  content: "";
}

.icon-arrow-right:before {
  content: "";
}

.icon-arrow-up-2:before {
  content: "";
}

.icon-arrow-up-3:before {
  content: "";
}

.icon-arrow-up-4:before {
  content: "";
}

.icon-arrow-up:before {
  content: "";
}

.icon-arrows-alt:before {
  content: "";
}

.icon-asterisk:before {
  content: "";
}

.icon-attachment:before {
  content: "";
}

.icon-backward-2:before {
  content: "";
}

.icon-backward-circle:before {
  content: "";
}

.icon-backward:before {
  content: "";
}

.icon-ban-circle:before {
  content: "";
}

.icon-bars:before {
  content: "";
}

.icon-basket:before {
  content: "";
}

.icon-bell:before {
  content: "";
}

.icon-bolt:before {
  content: "";
}

.icon-book:before {
  content: "";
}

.icon-bookmark-2:before, .icon-bookmark:before {
  content: "";
}

.icon-box-add:before, .icon-box-remove:before {
  content: "";
}

.icon-briefcase:before {
  content: "";
}

.icon-broadcast:before {
  content: "";
}

.icon-brush:before {
  content: "";
}

.icon-bubble-quote:before {
  content: "";
}

.icon-bullhorn:before {
  content: "";
}

.icon-calendar-check:before, .icon-calendar-2:before {
  content: "";
}

.icon-calendar-3:before {
  content: "";
}

.icon-calendar-alt:before, .icon-calendar:before {
  content: "";
}

.icon-camera-2:before {
  content: "";
}

.icon-camera:before {
  content: "";
}

.icon-cancel-2:before {
  content: "";
}

.icon-cancel-circle:before {
  content: "";
}

.icon-cancel:before {
  content: "";
}

.icon-caret-down:before {
  content: "";
}

.icon-caret-up:before {
  content: "";
}

.icon-cart:before {
  content: "";
}

.icon-chart:before {
  content: "";
}

.icon-check-circle:before {
  content: "";
}

.icon-check-square:before {
  content: "";
}

.icon-check:before {
  content: "";
}

.icon-checkbox-checked:before {
  content: "";
}

.icon-checkbox-partial:before, .icon-checkbox-unchecked:before {
  content: "";
}

.icon-checkbox:before {
  content: "";
}

.icon-checkedout:before {
  content: "";
}

.icon-checkin:before {
  content: "";
}

.icon-checkmark-2:before, .icon-checkmark-circle:before {
  content: "";
}

.icon-checkmark:before {
  content: "";
}

.icon-chevron-down:before {
  content: "";
}

.icon-chevron-left:before {
  content: "";
}

.icon-chevron-right:before {
  content: "";
}

.icon-chevron-up:before {
  content: "";
}

.icon-circle:before {
  content: "";
}

.icon-clipboard:before {
  content: "";
}

.icon-clock:before {
  content: "";
}

.icon-cloud-download-alt:before, .icon-cloud-download:before {
  content: "";
}

.icon-cloud-upload:before {
  content: "";
}

.icon-cloud:before {
  content: "";
}

.icon-code:before {
  content: "";
}

.icon-code-branch:before {
  content: "";
}

.icon-cog:before {
  content: "";
}

.icon-cogs:before {
  content: "";
}

.icon-collapse:before {
  content: "+";
}

.icon-color-palette:before {
  content: "";
}

.icon-comment-dots:before {
  content: "";
}

.icon-comment:before {
  content: "";
}

.icon-comments-2:before {
  content: "";
}

.icon-comments:before {
  content: "";
}

.icon-compass:before {
  content: "";
}

.icon-connection:before {
  content: "";
}

.icon-contract-2:before {
  content: "";
}

.icon-contract:before {
  content: "";
}

.icon-copy:before {
  content: "";
}

.icon-credit-2:before, .icon-credit:before {
  content: "";
}

.icon-crop:before {
  content: "";
}

.icon-cube:before {
  content: "";
}

.icon-cubes:before {
  content: "";
}

.icon-dashboard:before {
  content: "";
}

.icon-database:before {
  content: "";
}

.icon-default:before {
  content: "";
}

.icon-delete:before {
  content: "";
}

.icon-desktop:before {
  content: "";
}

.icon-downarrow:before {
  content: "";
}

.icon-download:before {
  content: "";
}

.icon-drawer-2:before {
  content: "";
}

.icon-drawer:before {
  content: "";
}

.icon-edit:before {
  content: "";
}

.icon-ellipsis-h:before {
  content: "";
}

.icon-ellipsis-v:before {
  content: "";
}

.icon-enter:before {
  content: "";
}

.icon-envelope-open-text:before {
  content: "";
}

.icon-envelope-opened:before, .icon-envelope:before {
  content: "";
}

.icon-equalizer:before {
  content: "";
}

.icon-error:before {
  content: "!";
}

.icon-exclamation-circle:before {
  content: "";
}

.icon-exclamation-triangle:before {
  content: "";
}

.icon-exclamation:before {
  content: "!";
}

.icon-exit:before {
  content: "";
}

.icon-expand-2:before {
  content: "";
}

.icon-expand:before {
  content: "";
}

.icon-expired:before {
  content: "";
}

.icon-external-link-alt:before {
  content: "";
}

.icon-eye-2:before, .icon-eye-blocked:before, .icon-eye-close:before {
  content: "";
}

.icon-eye-open:before {
  content: "";
}

.icon-eye-slash:before {
  content: "";
}

.icon-eye:before {
  content: "";
}

.icon-fax:before {
  content: "";
}

.icon-featured:before {
  content: "";
}

.icon-feed:before {
  content: "";
}

.icon-file-2:before {
  content: "";
}

.icon-file-add:before {
  content: "+";
}

.icon-file-alt:before {
  content: "";
}

.icon-file-check:before {
  content: "";
}

.icon-file-minus:before {
  content: "";
}

.icon-file-plus:before {
  content: "+";
}

.icon-file-remove:before {
  content: "";
}

.icon-file:before {
  content: "";
}

.icon-filter:before {
  content: "";
}

.icon-first:before {
  content: "";
}

.icon-flag-2:before {
  content: "";
}

.icon-flag-3:before, .icon-flag:before {
  content: "";
}

.icon-flash:before {
  content: "";
}

.icon-folder-2:before {
  content: "";
}

.icon-folder-3:before {
  content: "";
}

.icon-folder-close:before, .icon-folder-minus:before {
  content: "";
}

.icon-folder-open:before {
  content: "";
}

.icon-folder-plus-2:before {
  content: "";
}

.icon-folder-plus:before {
  content: "";
}

.icon-folder-remove:before, .icon-folder:before {
  content: "";
}

.icon-forward-2:before {
  content: "";
}

.icon-forward-circle:before {
  content: "";
}

.icon-forward:before {
  content: "";
}

.icon-generic:before {
  content: "";
}

.icon-globe:before {
  content: "";
}

.icon-grid-2:before, .icon-grid-view-2:before {
  content: "";
}

.icon-grid-view:before, .icon-grid:before {
  content: "";
}

.icon-handshake:before {
  content: "";
}

.icon-health:before {
  content: "";
}

.icon-heart-2:before, .icon-heart:before {
  content: "";
}

.icon-help:before {
  content: "?";
}

.icon-hits:before {
  content: "";
}

.icon-home-2:before, .icon-home:before {
  content: "";
}

.icon-image:before, .icon-images:before {
  content: "";
}

.icon-info-2:before, .icon-info-circle:before {
  content: "";
}

.icon-info:before {
  content: "";
}

.icon-key:before {
  content: "";
}

.icon-lamp:before {
  content: "";
}

.icon-language:before {
  content: "";
}

.icon-last:before {
  content: "";
}

.icon-leftarrow:before {
  content: "";
}

.icon-lightbulb:before {
  content: "";
}

.icon-lightning:before {
  content: "";
}

.icon-link:before {
  content: "";
}

.icon-list-2:before {
  content: "";
}

.icon-list-view:before, .icon-list:before {
  content: "";
}

.icon-loading:before {
  content: "";
}

.icon-location:before {
  content: "";
}

.icon-lock:before, .icon-locked:before {
  content: "";
}

.icon-loop:before {
  content: "";
}

.icon-mail-2:before, .icon-mail:before {
  content: "";
}

.icon-map-signs:before {
  content: "";
}

.icon-menu-2:before {
  content: "";
}

.icon-menu-3:before {
  content: "";
}

.icon-menu:before {
  content: "";
}

.icon-minus-2:before {
  content: "";
}

.icon-minus-circle:before {
  content: "";
}

.icon-minus-sign:before, .icon-minus:before {
  content: "";
}

.icon-mobile:before {
  content: "";
}

.icon-move:before {
  content: "";
}

.icon-music:before {
  content: "";
}

.icon-new-tab-2:before {
  content: "";
}

.icon-new-tab:before {
  content: "";
}

.icon-new:before {
  content: "+";
}

.icon-next:before {
  content: "";
}

.icon-not-ok:before {
  content: "";
}

.icon-notification-2:before, .icon-notification-circle:before {
  content: "";
}

.icon-notification:before {
  content: "!";
}

.icon-ok:before {
  content: "";
}

.icon-open:before {
  content: "";
}

.icon-options:before {
  content: "";
}

.icon-out-2:before {
  content: "";
}

.icon-out-3:before {
  content: "";
}

.icon-out:before {
  content: "";
}

.icon-paint-brush:before, .icon-palette:before {
  content: "";
}

.icon-paperclip:before {
  content: "";
}

.icon-paragraph-center:before {
  content: "";
}

.icon-paragraph-justify:before {
  content: "";
}

.icon-paragraph-left:before {
  content: "";
}

.icon-paragraph-right:before {
  content: "";
}

.icon-pause-circle:before {
  content: "";
}

.icon-pause:before {
  content: "";
}

.icon-pen-square:before {
  content: "";
}

.icon-pencil-2:before, .icon-pencil-alt:before {
  content: "";
}

.icon-pencil:before {
  content: "";
}

.icon-pending:before {
  content: "";
}

.icon-phone-2:before {
  content: "";
}

.icon-phone:before {
  content: "";
}

.icon-picture:before, .icon-pictures:before {
  content: "";
}

.icon-pie:before {
  content: "";
}

.icon-pin:before {
  content: "";
}

.icon-play-2:before {
  content: "";
}

.icon-play-circle:before {
  content: "";
}

.icon-play:before {
  content: "";
}

.icon-plug:before {
  content: "";
}

.icon-plus-2:before {
  content: "+";
}

.icon-plus-circle:before {
  content: "";
}

.icon-plus-square:before {
  content: "";
}

.icon-plus:before {
  content: "+";
}

.icon-power-cord:before {
  content: "";
}

.icon-power-off:before {
  content: "";
}

.icon-previous:before {
  content: "";
}

.icon-print:before, .icon-printer:before {
  content: "";
}

.icon-project-diagram:before {
  content: "";
}

.icon-protected:before {
  content: "";
}

.icon-publish:before {
  content: "";
}

.icon-purge:before {
  content: "";
}

.icon-pushpin:before {
  content: "";
}

.icon-puzzle-piece:before, .icon-puzzle:before {
  content: "";
}

.icon-question-2:before, .icon-question-circle:before {
  content: "";
}

.icon-question-sign:before, .icon-question:before {
  content: "?";
}

.icon-quote-2:before {
  content: "";
}

.icon-quote-3:before {
  content: "";
}

.icon-quote:before, .icon-quotes-left:before {
  content: "";
}

.icon-quotes-right:before {
  content: "";
}

.icon-radio-checked:before {
  content: "";
}

.icon-radio-unchecked:before {
  content: "";
}

.icon-redo-2:before {
  content: "";
}

.icon-redo:before {
  content: "";
}

.icon-refresh:before {
  content: "";
}

.icon-register:before {
  content: "";
}

.icon-remove:before {
  content: "";
}

.icon-reply:before {
  content: "";
}

.icon-rightarrow:before {
  content: "";
}

.icon-rss:before {
  content: "";
}

.icon-save-copy:before {
  content: "";
}

.icon-save-new:before {
  content: "+";
}

.icon-save:before {
  content: "";
}

.icon-scissors:before {
  content: "";
}

.icon-screen:before {
  content: "";
}

.icon-screwdriver:before {
  content: "";
}

.icon-search-minus:before {
  content: "";
}

.icon-search-plus:before {
  content: "";
}

.icon-search:before {
  content: "";
}

.icon-select-file:before {
  content: "";
}

.icon-share-alt:before, .icon-share:before {
  content: "";
}

.icon-shield-alt:before, .icon-shield:before {
  content: "";
}

.icon-shuffle:before {
  content: "";
}

.icon-signup:before {
  content: "";
}

.icon-sliders-h:before {
  content: "";
}

.icon-smiley-2:before, .icon-smiley-happy-2:before, .icon-smiley-happy:before, .icon-smiley-neutral-2:before, .icon-smiley-neutral:before {
  content: "";
}

.icon-smiley-sad-2:before, .icon-smiley-sad:before {
  content: "";
}

.icon-smiley:before {
  content: "";
}

.icon-sort:before {
  content: "";
}

.icon-spinner:before {
  content: "";
}

.icon-square:before {
  content: "";
}

.icon-stack:before {
  content: "";
}

.icon-star-2:before {
  content: "";
}

.icon-star-empty:before, .icon-star:before {
  content: "";
}

.icon-stop-circle:before {
  content: "";
}

.icon-stop:before {
  content: "";
}

.icon-success:before {
  content: "";
}

.icon-support:before {
  content: "";
}

.icon-switch:before {
  content: "";
}

.icon-sync:before {
  content: "";
}

.icon-tablet:before {
  content: "";
}

.icon-tachometer-alt:before {
  content: "";
}

.icon-tag-2:before, .icon-tag:before {
  content: "";
}

.icon-tags-2:before, .icon-tags:before {
  content: "";
}

.icon-tasks:before {
  content: "";
}

.icon-text-width:before {
  content: "";
}

.icon-th:before {
  content: "";
}

.icon-th-large:before {
  content: "";
}

.icon-thumbs-down:before {
  content: "";
}

.icon-thumbs-up:before {
  content: "";
}

.icon-times:before {
  content: "";
}

.icon-toggle-off:before {
  content: "";
}

.icon-toggle-on:before {
  content: "";
}

.icon-tools:before {
  content: "";
}

.icon-trash:before {
  content: "";
}

.icon-tree-2:before {
  content: "";
}

.icon-tree:before {
  content: "";
}

.icon-trophy:before {
  content: "";
}

.icon-unarchive:before {
  content: "";
}

.icon-unblock:before {
  content: "";
}

.icon-undo-2:before, .icon-undo:before {
  content: "";
}

.icon-unfeatured:before {
  content: "";
}

.icon-universal:before, .icon-universal-access:before {
  content: "";
}

.icon-unlock-alt:before {
  content: "";
}

.icon-unlock:before {
  content: "";
}

.icon-unpublish:before {
  content: "";
}

.icon-uparrow:before {
  content: "";
}

.icon-upload:before {
  content: "";
}

.icon-user-circle:before {
  content: "";
}

.icon-user-edit:before {
  content: "";
}

.icon-user-lock:before {
  content: "";
}

.icon-user-tag:before {
  content: "";
}

.icon-user:before {
  content: "";
}

.icon-users-cog:before {
  content: "";
}

.icon-users:before {
  content: "";
}

.icon-vcard:before {
  content: "";
}

.icon-video-2:before {
  content: "";
}

.icon-video:before {
  content: "";
}

.icon-wand:before {
  content: "";
}

.icon-warning-2:before {
  content: "";
}

.icon-warning-circle:before {
  content: "";
}

.icon-warning:before {
  content: "";
}

.icon-wifi:before {
  content: "";
}

.icon-wrench:before {
  content: "";
}

.icon-zoom-in:before {
  content: "";
}

.icon-zoom-out:before {
  content: "";
}

:root, [data-bs-theme="light"] {
  --card-bg: #ffffffb3;
  --card-color: var(--template-bg-dark-60);
  --bluegray: #b2bfcd;
  --lightbluegray: #f6f9fc;
  --toolbar-bg: #fff;
  --success-border: var(--success);
  --info-border: var(--info);
  --warning-border: var(--warning);
  --danger-border: var(--danger);
  --login-main-bg: #0b1c32;
  --border: #cdcdcd;
  --white: #fff;
  --white-offset: #fefefe;
  --focus: #39f;
  --focus-shadow: #eaeaea;
  --toggle-color: #fff;
  --gray-100: #f8f9fa;
  --gray-200: #e8e8e8;
  --gray-300: #dee2e6;
  --gray-400: #cdcdcd;
  --gray-500: #adb5bd;
  --gray-600: #666e76;
  --gray-700: #495057;
  --gray-800: #343a40;
  --gray-900: #212529;
  --primary: #132f53;
  --secondary: #495057;
  --success: #457d54;
  --info: #2a69b8;
  --warning: #ffb514;
  --danger: #c52827;
  --light: #f8f9fa;
  --dark: #212529;
  --action: #132f53;
  --error: #3b0d0c;
  --primary-rgb: 19, 47, 83;
  --secondary-rgb: 73, 80, 87;
  --success-rgb: 69, 125, 84;
  --info-rgb: 42, 105, 184;
  --warning-rgb: 255, 181, 20;
  --danger-rgb: 197, 40, 39;
  --light-rgb: 248, 249, 250;
  --dark-rgb: 33, 37, 41;
  --action-rgb: 19, 47, 83;
  --error-rgb: 59, 13, 12;
  --primary-text-emphasis: #112a49;
  --secondary-text-emphasis: #292c2f;
  --success-text-emphasis: #1c3222;
  --info-text-emphasis: #112a4a;
  --warning-text-emphasis: #60451f;
  --danger-text-emphasis: #4f1010;
  --light-text-emphasis: #495057;
  --dark-text-emphasis: #495057;
  --primary-bg-subtle: #d4e1f1;
  --secondary-bg-subtle: #e0e2e4;
  --success-bg-subtle: #dae5dd;
  --info-bg-subtle: #d4e1f1;
  --warning-bg-subtle: #fcefdc;
  --danger-bg-subtle: #f3d4d4;
  --light-bg-subtle: #fcfcfd;
  --dark-bg-subtle: #cdcdcd;
  --primary-border-subtle: #aac3e2;
  --secondary-border-subtle: #c2c5c8;
  --success-border-subtle: #b5cbbb;
  --info-border-subtle: #aac3e3;
  --warning-border-subtle: #f9deb8;
  --danger-border-subtle: #e8a9a9;
  --light-border-subtle: #e8e8e8;
  --dark-border-subtle: #adb5bd;
  --white-rgb: 255, 255, 255;
  --black-rgb: 0, 0, 0;
  --font-sans-serif: "Roboto", -apple-system, BlinkMacSystemFont, "Segoe UI", "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  --font-monospace: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
  --gradient: linear-gradient(180deg, #ffffff26, #fff0);
  --body-font-family: var(--font-sans-serif);
  --body-font-size: 1rem;
  --body-font-weight: 400;
  --body-line-height: 1.5;
  --body-color: #212529;
  --body-color-rgb: 33, 37, 41;
  --body-bg: #fff;
  --body-bg-rgb: 255, 255, 255;
  --emphasis-color: #000;
  --emphasis-color-rgb: 0, 0, 0;
  --secondary-color: #212529bf;
  --secondary-color-rgb: 33, 37, 41;
  --secondary-bg: #e8e8e8;
  --secondary-bg-rgb: 232, 232, 232;
  --tertiary-color: #21252980;
  --tertiary-color-rgb: 33, 37, 41;
  --tertiary-bg: #f8f9fa;
  --tertiary-bg-rgb: 248, 249, 250;
  --heading-color: var(--template-bg-dark);
  --link-color: #2a69b8;
  --link-color-rgb: 42, 105, 184;
  --link-decoration: none;
  --link-hover-color: #173a65;
  --link-hover-color-rgb: 23, 58, 101;
  --code-color: #971250;
  --highlight-color: #212529;
  --highlight-bg: #fff0d0;
  --border-width: 1px;
  --border-style: solid;
  --border-color: #dee2e6;
  --border-color-translucent: #0000002d;
  --border-radius: .25rem;
  --border-radius-sm: .2rem;
  --border-radius-lg: .3rem;
  --border-radius-xl: .3rem;
  --border-radius-xxl: 2rem;
  --border-radius-2xl: var(--border-radius-xxl);
  --border-radius-pill: 50rem;
  --box-shadow: 0 .5rem 1rem #00000026;
  --box-shadow-sm: 0 .125rem .25rem #00000013;
  --box-shadow-lg: 0 1rem 3rem #0000002d;
  --box-shadow-inset: inset 0 1px 2px #00000013;
  --focus-ring-width: .25rem;
  --focus-ring-opacity: .25;
  --focus-ring-color: #2a69b740;
  --form-valid-color: #457d54;
  --form-valid-border-color: #457d54;
  --form-invalid-color: #c52827;
  --form-invalid-border-color: #c52827;
}

[data-bs-theme="dark"] {
  color-scheme: dark;
  --body-color: #dee2e6;
  --body-color-rgb: 222, 226, 230;
  --body-bg: #212529;
  --body-bg-rgb: 33, 37, 41;
  --emphasis-color: #fff;
  --emphasis-color-rgb: 255, 255, 255;
  --secondary-color: #dee2e6bf;
  --secondary-color-rgb: 222, 226, 230;
  --secondary-bg: #343a40;
  --secondary-bg-rgb: 52, 58, 64;
  --tertiary-color: #dee2e680;
  --tertiary-color-rgb: 222, 226, 230;
  --tertiary-bg: #2b3035;
  --tertiary-bg-rgb: 43, 48, 53;
  --primary-text-emphasis: #7fa5d4;
  --secondary-text-emphasis: #a3a8ad;
  --success-text-emphasis: #8fb198;
  --info-text-emphasis: #7fa5d4;
  --warning-text-emphasis: #f6ce95;
  --danger-text-emphasis: #dc7e7d;
  --light-text-emphasis: #f8f9fa;
  --dark-text-emphasis: #dee2e6;
  --primary-bg-subtle: #081525;
  --secondary-bg-subtle: #141618;
  --success-bg-subtle: #0e1911;
  --info-bg-subtle: #081525;
  --warning-bg-subtle: #302310;
  --danger-bg-subtle: #270808;
  --light-bg-subtle: #343a40;
  --dark-bg-subtle: #1a1d20;
  --primary-border-subtle: #193f6e;
  --secondary-border-subtle: #3d4247;
  --success-border-subtle: #294b32;
  --info-border-subtle: #193f6e;
  --warning-border-subtle: #90682f;
  --danger-border-subtle: #761817;
  --light-border-subtle: #495057;
  --dark-border-subtle: #343a40;
  --heading-color: inherit;
  --link-color: #6fbfdb;
  --link-hover-color: #8ccce2;
  --link-color-rgb: 111, 191, 219;
  --link-hover-color-rgb: 140, 204, 226;
  --code-color: #c17196;
  --highlight-color: #dee2e6;
  --highlight-bg: #664808;
  --border-color: #495057;
  --border-color-translucent: #ffffff26;
  --form-valid-color: #8fb198;
  --form-valid-border-color: #8fb198;
  --form-invalid-color: #dc7e7d;
  --form-invalid-border-color: #dc7e7d;
}

*, :before, :after {
  box-sizing: border-box;
}

@media (prefers-reduced-motion: no-preference) {
  :root {
    scroll-behavior: smooth;
  }
}

body {
  font-family: var(--body-font-family);
  font-size: var(--body-font-size);
  font-weight: var(--body-font-weight);
  line-height: var(--body-line-height);
  color: var(--body-color);
  text-align: var(--body-text-align);
  background-color: var(--body-bg);
  -webkit-text-size-adjust: 100%;
  -webkit-tap-highlight-color: #0000;
  margin: 0;
}

hr {
  color: inherit;
  border: 0;
  border-top: var(--border-width) solid;
  opacity: .25;
  margin: 1rem 0;
}

h6, .h6, h5, .h5, h4, .h4, h3, .h3, h2, .h2, h1, .h1 {
  color: var(--heading-color);
  margin-top: 0;
  margin-bottom: .5rem;
  font-weight: 500;
  line-height: 1.2;
}

h1, .h1 {
  font-size: calc(1.29rem + .48vw);
}

@media (width >= 1200px) {
  h1, .h1 {
    font-size: 1.65rem;
  }
}

h2, .h2 {
  font-size: calc(1.275rem + .3vw);
}

@media (width >= 1200px) {
  h2, .h2 {
    font-size: 1.5rem;
  }
}

h3, .h3 {
  font-size: 1.25rem;
}

h4, .h4 {
  font-size: 1rem;
}

h5, .h5 {
  font-size: .9286rem;
}

h6, .h6 {
  font-size: .8571rem;
}

p {
  margin-top: 0;
  margin-bottom: 1rem;
}

abbr[title] {
  cursor: help;
  text-decoration-skip-ink: none;
  text-decoration: underline dotted;
}

address {
  font-style: normal;
  line-height: inherit;
  margin-bottom: 1rem;
}

ol, ul {
  padding-left: 2rem;
}

ol, ul, dl {
  margin-top: 0;
  margin-bottom: 1rem;
}

ol ol, ul ul, ol ul, ul ol {
  margin-bottom: 0;
}

dt {
  font-weight: 700;
}

dd {
  margin-bottom: .5rem;
  margin-left: 0;
}

blockquote {
  margin: 0 0 1rem;
}

b, strong {
  font-weight: bolder;
}

mark, .mark {
  color: var(--highlight-color);
  background-color: var(--highlight-bg);
  padding: .1875em;
}

sub, sup {
  vertical-align: baseline;
  font-size: .75em;
  line-height: 0;
  position: relative;
}

sub {
  bottom: -.25em;
}

sup {
  top: -.5em;
}

a {
  color: rgba(var(--link-color-rgb), var(--link-opacity, 1));
  text-decoration: none;
}

a:hover {
  --link-color-rgb: var(--link-hover-color-rgb);
}

a:not([href]):not([class]), a:not([href]):not([class]):hover {
  color: inherit;
  text-decoration: none;
}

pre, code, kbd, samp {
  font-family: var(--font-monospace);
  font-size: 1em;
}

pre {
  margin-top: 0;
  margin-bottom: 1rem;
  font-size: .875em;
  display: block;
  overflow: auto;
}

pre code {
  font-size: inherit;
  color: inherit;
  word-break: normal;
}

code {
  color: var(--code-color);
  word-wrap: break-word;
  font-size: .875em;
}

a > code {
  color: inherit;
}

kbd {
  color: var(--body-bg);
  background-color: var(--body-color);
  border-radius: .2rem;
  padding: .1875rem .375rem;
  font-size: .875em;
}

kbd kbd {
  padding: 0;
  font-size: 1em;
}

figure {
  margin: 0 0 1rem;
}

img, svg {
  vertical-align: middle;
}

table {
  caption-side: bottom;
  border-collapse: collapse;
}

caption {
  color: var(--secondary-color);
  text-align: left;
  padding-top: .75rem;
  padding-bottom: .75rem;
}

th {
  text-align: inherit;
  text-align: -webkit-match-parent;
}

thead, tbody, tfoot, tr, td, th {
  border-color: inherit;
  border-style: solid;
  border-width: 0;
}

label {
  display: inline-block;
}

button {
  border-radius: 0;
}

button:focus:not(:focus-visible) {
  outline: 0;
}

input, button, select, optgroup, textarea {
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
  margin: 0;
}

button, select {
  text-transform: none;
}

[role="button"] {
  cursor: pointer;
}

select {
  word-wrap: normal;
}

select:disabled {
  opacity: 1;
}

[list]:not([type="date"]):not([type="datetime-local"]):not([type="month"]):not([type="week"]):not([type="time"])::-webkit-calendar-picker-indicator {
  display: none !important;
}

button, [type="button"], [type="reset"], [type="submit"] {
  -webkit-appearance: button;
}

button:not(:disabled), [type="button"]:not(:disabled), [type="reset"]:not(:disabled), [type="submit"]:not(:disabled) {
  cursor: pointer;
}

::-moz-focus-inner {
  border-style: none;
  padding: 0;
}

textarea {
  resize: vertical;
}

fieldset {
  border: 0;
  min-width: 0;
  margin: 0;
  padding: 0;
}

legend {
  float: left;
  width: 100%;
  line-height: inherit;
  margin-bottom: .5rem;
  padding: 0;
  font-size: calc(1.275rem + .3vw);
}

@media (width >= 1200px) {
  legend {
    font-size: 1.5rem;
  }
}

legend + * {
  clear: left;
}

::-webkit-datetime-edit-fields-wrapper, ::-webkit-datetime-edit-text, ::-webkit-datetime-edit-minute, ::-webkit-datetime-edit-hour-field, ::-webkit-datetime-edit-day-field, ::-webkit-datetime-edit-month-field, ::-webkit-datetime-edit-year-field {
  padding: 0;
}

::-webkit-inner-spin-button {
  height: auto;
}

[type="search"] {
  -webkit-appearance: textfield;
  outline-offset: -2px;
}

[type="search"]::-webkit-search-cancel-button {
  cursor: pointer;
  filter: grayscale();
}

::-webkit-search-decoration {
  -webkit-appearance: none;
}

::-webkit-color-swatch-wrapper {
  padding: 0;
}

::file-selector-button {
  font: inherit;
  -webkit-appearance: button;
}

output {
  display: inline-block;
}

iframe {
  border: 0;
}

summary {
  cursor: pointer;
  display: list-item;
}

progress {
  vertical-align: baseline;
}

[hidden] {
  display: none !important;
}

.lead {
  font-size: 1.25rem;
  font-weight: 300;
}

.display-1 {
  font-size: calc(1.625rem + 4.5vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-1 {
    font-size: 5rem;
  }
}

.display-2 {
  font-size: calc(1.575rem + 3.9vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-2 {
    font-size: 4.5rem;
  }
}

.display-3 {
  font-size: calc(1.525rem + 3.3vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-3 {
    font-size: 4rem;
  }
}

.display-4 {
  font-size: calc(1.475rem + 2.7vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-4 {
    font-size: 3.5rem;
  }
}

.display-5 {
  font-size: calc(1.425rem + 2.1vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-5 {
    font-size: 3rem;
  }
}

.display-6 {
  font-size: calc(1.375rem + 1.5vw);
  font-weight: 300;
  line-height: 1.2;
}

@media (width >= 1200px) {
  .display-6 {
    font-size: 2.5rem;
  }
}

.list-unstyled, .list-inline {
  padding-left: 0;
  list-style: none;
}

.list-inline-item {
  display: inline-block;
}

.list-inline-item:not(:last-child) {
  margin-right: .5rem;
}

.initialism {
  text-transform: uppercase;
  font-size: .875em;
}

.blockquote {
  margin-bottom: 1rem;
  font-size: 1.25rem;
}

.blockquote > :last-child {
  margin-bottom: 0;
}

.blockquote-footer {
  color: #666e76;
  margin-top: -1rem;
  margin-bottom: 1rem;
  font-size: .875em;
}

.blockquote-footer:before {
  content: "— ";
}

.img-fluid {
  max-width: 100%;
  height: auto;
}

.img-thumbnail {
  background-color: var(--body-bg);
  border: var(--border-width) solid var(--border-color);
  border-radius: var(--border-radius);
  max-width: 100%;
  height: auto;
  padding: .25rem;
}

.figure {
  display: inline-block;
}

.figure-img {
  margin-bottom: .5rem;
  line-height: 1;
}

.figure-caption {
  color: var(--secondary-color);
  font-size: .875em;
}

.container, .container-fluid, .container-xxl, .container-xl, .container-lg, .container-md, .container-sm {
  --gutter-x: 2rem;
  --gutter-y: 0;
  width: 100%;
  padding-right: calc(var(--gutter-x) * .5);
  padding-left: calc(var(--gutter-x) * .5);
  margin-left: auto;
  margin-right: auto;
}

@media (width >= 576px) {
  .container-sm, .container {
    max-width: 540px;
  }
}

@media (width >= 768px) {
  .container-md, .container-sm, .container {
    max-width: 720px;
  }
}

@media (width >= 992px) {
  .container-lg, .container-md, .container-sm, .container {
    max-width: 960px;
  }
}

@media (width >= 1200px) {
  .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1140px;
  }
}

@media (width >= 1400px) {
  .container-xxl, .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1320px;
  }
}

:root {
  --breakpoint-xs: 0;
  --breakpoint-sm: 576px;
  --breakpoint-md: 768px;
  --breakpoint-lg: 992px;
  --breakpoint-xl: 1200px;
  --breakpoint-xxl: 1400px;
}

.row {
  --gutter-x: 2rem;
  --gutter-y: 0;
  margin-top: calc(-1 * var(--gutter-y));
  margin-right: calc(-.5 * var(--gutter-x));
  margin-left: calc(-.5 * var(--gutter-x));
  flex-wrap: wrap;
  display: flex;
}

.row > * {
  width: 100%;
  max-width: 100%;
  padding-right: calc(var(--gutter-x) * .5);
  padding-left: calc(var(--gutter-x) * .5);
  margin-top: var(--gutter-y);
  flex-shrink: 0;
}

.col {
  flex: 1 0 0;
}

.row-cols-auto > * {
  flex: none;
  width: auto;
}

.row-cols-1 > * {
  flex: none;
  width: 100%;
}

.row-cols-2 > * {
  flex: none;
  width: 50%;
}

.row-cols-3 > * {
  flex: none;
  width: 33.3333%;
}

.row-cols-4 > * {
  flex: none;
  width: 25%;
}

.row-cols-5 > * {
  flex: none;
  width: 20%;
}

.row-cols-6 > * {
  flex: none;
  width: 16.6667%;
}

.col-auto {
  flex: none;
  width: auto;
}

.col-1 {
  flex: none;
  width: 8.33333%;
}

.col-2 {
  flex: none;
  width: 16.6667%;
}

.col-3 {
  flex: none;
  width: 25%;
}

.col-4 {
  flex: none;
  width: 33.3333%;
}

.col-5 {
  flex: none;
  width: 41.6667%;
}

.col-6 {
  flex: none;
  width: 50%;
}

.col-7 {
  flex: none;
  width: 58.3333%;
}

.col-8 {
  flex: none;
  width: 66.6667%;
}

.col-9 {
  flex: none;
  width: 75%;
}

.col-10 {
  flex: none;
  width: 83.3333%;
}

.col-11 {
  flex: none;
  width: 91.6667%;
}

.col-12 {
  flex: none;
  width: 100%;
}

.offset-1 {
  margin-left: 8.33333%;
}

.offset-2 {
  margin-left: 16.6667%;
}

.offset-3 {
  margin-left: 25%;
}

.offset-4 {
  margin-left: 33.3333%;
}

.offset-5 {
  margin-left: 41.6667%;
}

.offset-6 {
  margin-left: 50%;
}

.offset-7 {
  margin-left: 58.3333%;
}

.offset-8 {
  margin-left: 66.6667%;
}

.offset-9 {
  margin-left: 75%;
}

.offset-10 {
  margin-left: 83.3333%;
}

.offset-11 {
  margin-left: 91.6667%;
}

.g-0, .gx-0 {
  --gutter-x: 0;
}

.g-0, .gy-0 {
  --gutter-y: 0;
}

.g-1, .gx-1 {
  --gutter-x: .25rem;
}

.g-1, .gy-1 {
  --gutter-y: .25rem;
}

.g-2, .gx-2 {
  --gutter-x: .5rem;
}

.g-2, .gy-2 {
  --gutter-y: .5rem;
}

.g-3, .gx-3 {
  --gutter-x: 1rem;
}

.g-3, .gy-3 {
  --gutter-y: 1rem;
}

.g-4, .gx-4 {
  --gutter-x: 1.5rem;
}

.g-4, .gy-4 {
  --gutter-y: 1.5rem;
}

.g-5, .gx-5 {
  --gutter-x: 3rem;
}

.g-5, .gy-5 {
  --gutter-y: 3rem;
}

@media (width >= 576px) {
  .col-sm {
    flex: 1 0 0;
  }

  .row-cols-sm-auto > * {
    flex: none;
    width: auto;
  }

  .row-cols-sm-1 > * {
    flex: none;
    width: 100%;
  }

  .row-cols-sm-2 > * {
    flex: none;
    width: 50%;
  }

  .row-cols-sm-3 > * {
    flex: none;
    width: 33.3333%;
  }

  .row-cols-sm-4 > * {
    flex: none;
    width: 25%;
  }

  .row-cols-sm-5 > * {
    flex: none;
    width: 20%;
  }

  .row-cols-sm-6 > * {
    flex: none;
    width: 16.6667%;
  }

  .col-sm-auto {
    flex: none;
    width: auto;
  }

  .col-sm-1 {
    flex: none;
    width: 8.33333%;
  }

  .col-sm-2 {
    flex: none;
    width: 16.6667%;
  }

  .col-sm-3 {
    flex: none;
    width: 25%;
  }

  .col-sm-4 {
    flex: none;
    width: 33.3333%;
  }

  .col-sm-5 {
    flex: none;
    width: 41.6667%;
  }

  .col-sm-6 {
    flex: none;
    width: 50%;
  }

  .col-sm-7 {
    flex: none;
    width: 58.3333%;
  }

  .col-sm-8 {
    flex: none;
    width: 66.6667%;
  }

  .col-sm-9 {
    flex: none;
    width: 75%;
  }

  .col-sm-10 {
    flex: none;
    width: 83.3333%;
  }

  .col-sm-11 {
    flex: none;
    width: 91.6667%;
  }

  .col-sm-12 {
    flex: none;
    width: 100%;
  }

  .offset-sm-0 {
    margin-left: 0;
  }

  .offset-sm-1 {
    margin-left: 8.33333%;
  }

  .offset-sm-2 {
    margin-left: 16.6667%;
  }

  .offset-sm-3 {
    margin-left: 25%;
  }

  .offset-sm-4 {
    margin-left: 33.3333%;
  }

  .offset-sm-5 {
    margin-left: 41.6667%;
  }

  .offset-sm-6 {
    margin-left: 50%;
  }

  .offset-sm-7 {
    margin-left: 58.3333%;
  }

  .offset-sm-8 {
    margin-left: 66.6667%;
  }

  .offset-sm-9 {
    margin-left: 75%;
  }

  .offset-sm-10 {
    margin-left: 83.3333%;
  }

  .offset-sm-11 {
    margin-left: 91.6667%;
  }

  .g-sm-0, .gx-sm-0 {
    --gutter-x: 0;
  }

  .g-sm-0, .gy-sm-0 {
    --gutter-y: 0;
  }

  .g-sm-1, .gx-sm-1 {
    --gutter-x: .25rem;
  }

  .g-sm-1, .gy-sm-1 {
    --gutter-y: .25rem;
  }

  .g-sm-2, .gx-sm-2 {
    --gutter-x: .5rem;
  }

  .g-sm-2, .gy-sm-2 {
    --gutter-y: .5rem;
  }

  .g-sm-3, .gx-sm-3 {
    --gutter-x: 1rem;
  }

  .g-sm-3, .gy-sm-3 {
    --gutter-y: 1rem;
  }

  .g-sm-4, .gx-sm-4 {
    --gutter-x: 1.5rem;
  }

  .g-sm-4, .gy-sm-4 {
    --gutter-y: 1.5rem;
  }

  .g-sm-5, .gx-sm-5 {
    --gutter-x: 3rem;
  }

  .g-sm-5, .gy-sm-5 {
    --gutter-y: 3rem;
  }
}

@media (width >= 768px) {
  .col-md {
    flex: 1 0 0;
  }

  .row-cols-md-auto > * {
    flex: none;
    width: auto;
  }

  .row-cols-md-1 > * {
    flex: none;
    width: 100%;
  }

  .row-cols-md-2 > * {
    flex: none;
    width: 50%;
  }

  .row-cols-md-3 > * {
    flex: none;
    width: 33.3333%;
  }

  .row-cols-md-4 > * {
    flex: none;
    width: 25%;
  }

  .row-cols-md-5 > * {
    flex: none;
    width: 20%;
  }

  .row-cols-md-6 > * {
    flex: none;
    width: 16.6667%;
  }

  .col-md-auto {
    flex: none;
    width: auto;
  }

  .col-md-1 {
    flex: none;
    width: 8.33333%;
  }

  .col-md-2 {
    flex: none;
    width: 16.6667%;
  }

  .col-md-3 {
    flex: none;
    width: 25%;
  }

  .col-md-4 {
    flex: none;
    width: 33.3333%;
  }

  .col-md-5 {
    flex: none;
    width: 41.6667%;
  }

  .col-md-6 {
    flex: none;
    width: 50%;
  }

  .col-md-7 {
    flex: none;
    width: 58.3333%;
  }

  .col-md-8 {
    flex: none;
    width: 66.6667%;
  }

  .col-md-9 {
    flex: none;
    width: 75%;
  }

  .col-md-10 {
    flex: none;
    width: 83.3333%;
  }

  .col-md-11 {
    flex: none;
    width: 91.6667%;
  }

  .col-md-12 {
    flex: none;
    width: 100%;
  }

  .offset-md-0 {
    margin-left: 0;
  }

  .offset-md-1 {
    margin-left: 8.33333%;
  }

  .offset-md-2 {
    margin-left: 16.6667%;
  }

  .offset-md-3 {
    margin-left: 25%;
  }

  .offset-md-4 {
    margin-left: 33.3333%;
  }

  .offset-md-5 {
    margin-left: 41.6667%;
  }

  .offset-md-6 {
    margin-left: 50%;
  }

  .offset-md-7 {
    margin-left: 58.3333%;
  }

  .offset-md-8 {
    margin-left: 66.6667%;
  }

  .offset-md-9 {
    margin-left: 75%;
  }

  .offset-md-10 {
    margin-left: 83.3333%;
  }

  .offset-md-11 {
    margin-left: 91.6667%;
  }

  .g-md-0, .gx-md-0 {
    --gutter-x: 0;
  }

  .g-md-0, .gy-md-0 {
    --gutter-y: 0;
  }

  .g-md-1, .gx-md-1 {
    --gutter-x: .25rem;
  }

  .g-md-1, .gy-md-1 {
    --gutter-y: .25rem;
  }

  .g-md-2, .gx-md-2 {
    --gutter-x: .5rem;
  }

  .g-md-2, .gy-md-2 {
    --gutter-y: .5rem;
  }

  .g-md-3, .gx-md-3 {
    --gutter-x: 1rem;
  }

  .g-md-3, .gy-md-3 {
    --gutter-y: 1rem;
  }

  .g-md-4, .gx-md-4 {
    --gutter-x: 1.5rem;
  }

  .g-md-4, .gy-md-4 {
    --gutter-y: 1.5rem;
  }

  .g-md-5, .gx-md-5 {
    --gutter-x: 3rem;
  }

  .g-md-5, .gy-md-5 {
    --gutter-y: 3rem;
  }
}

@media (width >= 992px) {
  .col-lg {
    flex: 1 0 0;
  }

  .row-cols-lg-auto > * {
    flex: none;
    width: auto;
  }

  .row-cols-lg-1 > * {
    flex: none;
    width: 100%;
  }

  .row-cols-lg-2 > * {
    flex: none;
    width: 50%;
  }

  .row-cols-lg-3 > * {
    flex: none;
    width: 33.3333%;
  }

  .row-cols-lg-4 > * {
    flex: none;
    width: 25%;
  }

  .row-cols-lg-5 > * {
    flex: none;
    width: 20%;
  }

  .row-cols-lg-6 > * {
    flex: none;
    width: 16.6667%;
  }

  .col-lg-auto {
    flex: none;
    width: auto;
  }

  .col-lg-1 {
    flex: none;
    width: 8.33333%;
  }

  .col-lg-2 {
    flex: none;
    width: 16.6667%;
  }

  .col-lg-3 {
    flex: none;
    width: 25%;
  }

  .col-lg-4 {
    flex: none;
    width: 33.3333%;
  }

  .col-lg-5 {
    flex: none;
    width: 41.6667%;
  }

  .col-lg-6 {
    flex: none;
    width: 50%;
  }

  .col-lg-7 {
    flex: none;
    width: 58.3333%;
  }

  .col-lg-8 {
    flex: none;
    width: 66.6667%;
  }

  .col-lg-9 {
    flex: none;
    width: 75%;
  }

  .col-lg-10 {
    flex: none;
    width: 83.3333%;
  }

  .col-lg-11 {
    flex: none;
    width: 91.6667%;
  }

  .col-lg-12 {
    flex: none;
    width: 100%;
  }

  .offset-lg-0 {
    margin-left: 0;
  }

  .offset-lg-1 {
    margin-left: 8.33333%;
  }

  .offset-lg-2 {
    margin-left: 16.6667%;
  }

  .offset-lg-3 {
    margin-left: 25%;
  }

  .offset-lg-4 {
    margin-left: 33.3333%;
  }

  .offset-lg-5 {
    margin-left: 41.6667%;
  }

  .offset-lg-6 {
    margin-left: 50%;
  }

  .offset-lg-7 {
    margin-left: 58.3333%;
  }

  .offset-lg-8 {
    margin-left: 66.6667%;
  }

  .offset-lg-9 {
    margin-left: 75%;
  }

  .offset-lg-10 {
    margin-left: 83.3333%;
  }

  .offset-lg-11 {
    margin-left: 91.6667%;
  }

  .g-lg-0, .gx-lg-0 {
    --gutter-x: 0;
  }

  .g-lg-0, .gy-lg-0 {
    --gutter-y: 0;
  }

  .g-lg-1, .gx-lg-1 {
    --gutter-x: .25rem;
  }

  .g-lg-1, .gy-lg-1 {
    --gutter-y: .25rem;
  }

  .g-lg-2, .gx-lg-2 {
    --gutter-x: .5rem;
  }

  .g-lg-2, .gy-lg-2 {
    --gutter-y: .5rem;
  }

  .g-lg-3, .gx-lg-3 {
    --gutter-x: 1rem;
  }

  .g-lg-3, .gy-lg-3 {
    --gutter-y: 1rem;
  }

  .g-lg-4, .gx-lg-4 {
    --gutter-x: 1.5rem;
  }

  .g-lg-4, .gy-lg-4 {
    --gutter-y: 1.5rem;
  }

  .g-lg-5, .gx-lg-5 {
    --gutter-x: 3rem;
  }

  .g-lg-5, .gy-lg-5 {
    --gutter-y: 3rem;
  }
}

@media (width >= 1200px) {
  .col-xl {
    flex: 1 0 0;
  }

  .row-cols-xl-auto > * {
    flex: none;
    width: auto;
  }

  .row-cols-xl-1 > * {
    flex: none;
    width: 100%;
  }

  .row-cols-xl-2 > * {
    flex: none;
    width: 50%;
  }

  .row-cols-xl-3 > * {
    flex: none;
    width: 33.3333%;
  }

  .row-cols-xl-4 > * {
    flex: none;
    width: 25%;
  }

  .row-cols-xl-5 > * {
    flex: none;
    width: 20%;
  }

  .row-cols-xl-6 > * {
    flex: none;
    width: 16.6667%;
  }

  .col-xl-auto {
    flex: none;
    width: auto;
  }

  .col-xl-1 {
    flex: none;
    width: 8.33333%;
  }

  .col-xl-2 {
    flex: none;
    width: 16.6667%;
  }

  .col-xl-3 {
    flex: none;
    width: 25%;
  }

  .col-xl-4 {
    flex: none;
    width: 33.3333%;
  }

  .col-xl-5 {
    flex: none;
    width: 41.6667%;
  }

  .col-xl-6 {
    flex: none;
    width: 50%;
  }

  .col-xl-7 {
    flex: none;
    width: 58.3333%;
  }

  .col-xl-8 {
    flex: none;
    width: 66.6667%;
  }

  .col-xl-9 {
    flex: none;
    width: 75%;
  }

  .col-xl-10 {
    flex: none;
    width: 83.3333%;
  }

  .col-xl-11 {
    flex: none;
    width: 91.6667%;
  }

  .col-xl-12 {
    flex: none;
    width: 100%;
  }

  .offset-xl-0 {
    margin-left: 0;
  }

  .offset-xl-1 {
    margin-left: 8.33333%;
  }

  .offset-xl-2 {
    margin-left: 16.6667%;
  }

  .offset-xl-3 {
    margin-left: 25%;
  }

  .offset-xl-4 {
    margin-left: 33.3333%;
  }

  .offset-xl-5 {
    margin-left: 41.6667%;
  }

  .offset-xl-6 {
    margin-left: 50%;
  }

  .offset-xl-7 {
    margin-left: 58.3333%;
  }

  .offset-xl-8 {
    margin-left: 66.6667%;
  }

  .offset-xl-9 {
    margin-left: 75%;
  }

  .offset-xl-10 {
    margin-left: 83.3333%;
  }

  .offset-xl-11 {
    margin-left: 91.6667%;
  }

  .g-xl-0, .gx-xl-0 {
    --gutter-x: 0;
  }

  .g-xl-0, .gy-xl-0 {
    --gutter-y: 0;
  }

  .g-xl-1, .gx-xl-1 {
    --gutter-x: .25rem;
  }

  .g-xl-1, .gy-xl-1 {
    --gutter-y: .25rem;
  }

  .g-xl-2, .gx-xl-2 {
    --gutter-x: .5rem;
  }

  .g-xl-2, .gy-xl-2 {
    --gutter-y: .5rem;
  }

  .g-xl-3, .gx-xl-3 {
    --gutter-x: 1rem;
  }

  .g-xl-3, .gy-xl-3 {
    --gutter-y: 1rem;
  }

  .g-xl-4, .gx-xl-4 {
    --gutter-x: 1.5rem;
  }

  .g-xl-4, .gy-xl-4 {
    --gutter-y: 1.5rem;
  }

  .g-xl-5, .gx-xl-5 {
    --gutter-x: 3rem;
  }

  .g-xl-5, .gy-xl-5 {
    --gutter-y: 3rem;
  }
}

@media (width >= 1400px) {
  .col-xxl {
    flex: 1 0 0;
  }

  .row-cols-xxl-auto > * {
    flex: none;
    width: auto;
  }

  .row-cols-xxl-1 > * {
    flex: none;
    width: 100%;
  }

  .row-cols-xxl-2 > * {
    flex: none;
    width: 50%;
  }

  .row-cols-xxl-3 > * {
    flex: none;
    width: 33.3333%;
  }

  .row-cols-xxl-4 > * {
    flex: none;
    width: 25%;
  }

  .row-cols-xxl-5 > * {
    flex: none;
    width: 20%;
  }

  .row-cols-xxl-6 > * {
    flex: none;
    width: 16.6667%;
  }

  .col-xxl-auto {
    flex: none;
    width: auto;
  }

  .col-xxl-1 {
    flex: none;
    width: 8.33333%;
  }

  .col-xxl-2 {
    flex: none;
    width: 16.6667%;
  }

  .col-xxl-3 {
    flex: none;
    width: 25%;
  }

  .col-xxl-4 {
    flex: none;
    width: 33.3333%;
  }

  .col-xxl-5 {
    flex: none;
    width: 41.6667%;
  }

  .col-xxl-6 {
    flex: none;
    width: 50%;
  }

  .col-xxl-7 {
    flex: none;
    width: 58.3333%;
  }

  .col-xxl-8 {
    flex: none;
    width: 66.6667%;
  }

  .col-xxl-9 {
    flex: none;
    width: 75%;
  }

  .col-xxl-10 {
    flex: none;
    width: 83.3333%;
  }

  .col-xxl-11 {
    flex: none;
    width: 91.6667%;
  }

  .col-xxl-12 {
    flex: none;
    width: 100%;
  }

  .offset-xxl-0 {
    margin-left: 0;
  }

  .offset-xxl-1 {
    margin-left: 8.33333%;
  }

  .offset-xxl-2 {
    margin-left: 16.6667%;
  }

  .offset-xxl-3 {
    margin-left: 25%;
  }

  .offset-xxl-4 {
    margin-left: 33.3333%;
  }

  .offset-xxl-5 {
    margin-left: 41.6667%;
  }

  .offset-xxl-6 {
    margin-left: 50%;
  }

  .offset-xxl-7 {
    margin-left: 58.3333%;
  }

  .offset-xxl-8 {
    margin-left: 66.6667%;
  }

  .offset-xxl-9 {
    margin-left: 75%;
  }

  .offset-xxl-10 {
    margin-left: 83.3333%;
  }

  .offset-xxl-11 {
    margin-left: 91.6667%;
  }

  .g-xxl-0, .gx-xxl-0 {
    --gutter-x: 0;
  }

  .g-xxl-0, .gy-xxl-0 {
    --gutter-y: 0;
  }

  .g-xxl-1, .gx-xxl-1 {
    --gutter-x: .25rem;
  }

  .g-xxl-1, .gy-xxl-1 {
    --gutter-y: .25rem;
  }

  .g-xxl-2, .gx-xxl-2 {
    --gutter-x: .5rem;
  }

  .g-xxl-2, .gy-xxl-2 {
    --gutter-y: .5rem;
  }

  .g-xxl-3, .gx-xxl-3 {
    --gutter-x: 1rem;
  }

  .g-xxl-3, .gy-xxl-3 {
    --gutter-y: 1rem;
  }

  .g-xxl-4, .gx-xxl-4 {
    --gutter-x: 1.5rem;
  }

  .g-xxl-4, .gy-xxl-4 {
    --gutter-y: 1.5rem;
  }

  .g-xxl-5, .gx-xxl-5 {
    --gutter-x: 3rem;
  }

  .g-xxl-5, .gy-xxl-5 {
    --gutter-y: 3rem;
  }
}

.table {
  --table-color-type: initial;
  --table-bg-type: initial;
  --table-color-state: initial;
  --table-bg-state: initial;
  --table-color: var(--emphasis-color);
  --table-bg: transparent;
  --table-border-color: var(--border-color);
  --table-accent-bg: transparent;
  --table-striped-color: var(--emphasis-color);
  --table-striped-bg: rgba(var(--emphasis-color-rgb), .05);
  --table-active-color: var(--emphasis-color);
  --table-active-bg: rgba(var(--emphasis-color-rgb), .1);
  --table-hover-color: var(--emphasis-color);
  --table-hover-bg: rgba(var(--emphasis-color-rgb), .075);
  vertical-align: top;
  border-color: var(--table-border-color);
  width: 100%;
  margin-bottom: 1rem;
}

.table > :not(caption) > * > * {
  color: var(--table-color-state, var(--table-color-type, var(--table-color)));
  background-color: var(--table-bg);
  border-bottom-width: var(--border-width);
  box-shadow: inset 0 0 0 9999px var(--table-bg-state, var(--table-bg-type, var(--table-accent-bg)));
  padding: .75rem 1rem;
}

.table > tbody {
  vertical-align: inherit;
}

.table > thead {
  vertical-align: bottom;
}

.table-group-divider {
  border-top: calc(var(--border-width) * 2) solid #dee2e6;
}

.caption-top {
  caption-side: top;
}

.table-sm > :not(caption) > * > * {
  padding: .3rem;
}

.table-bordered > :not(caption) > * {
  border-width: var(--border-width) 0;
}

.table-bordered > :not(caption) > * > * {
  border-width: 0 var(--border-width);
}

.table-borderless > :not(caption) > * > * {
  border-bottom-width: 0;
}

.table-borderless > :not(:first-child) {
  border-top-width: 0;
}

.table-striped > tbody > tr:nth-of-type(odd) > *, .table-striped-columns > :not(caption) > tr > :nth-child(2n) {
  --table-color-type: var(--table-striped-color);
  --table-bg-type: var(--table-striped-bg);
}

.table-active {
  --table-color-state: var(--table-active-color);
  --table-bg-state: var(--table-active-bg);
}

.table-hover > tbody > tr:hover > * {
  --table-color-state: var(--table-hover-color);
  --table-bg-state: var(--table-hover-bg);
}

.table-primary {
  --table-color: #000;
  --table-bg: #d4e1f1;
  --table-border-color: #aab4c0;
  --table-striped-bg: #cad6e5;
  --table-striped-color: #000;
  --table-active-bg: #bfcbd9;
  --table-active-color: #000;
  --table-hover-bg: #c4d0df;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-secondary {
  --table-color: #000;
  --table-bg: #e0e2e4;
  --table-border-color: #b4b5b6;
  --table-striped-bg: #d5d7d8;
  --table-striped-color: #000;
  --table-active-bg: #cacbcd;
  --table-active-color: #000;
  --table-hover-bg: #d0d1d3;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-success {
  --table-color: #000;
  --table-bg: #dae5dd;
  --table-border-color: #aeb7b1;
  --table-striped-bg: #cfdad2;
  --table-striped-color: #000;
  --table-active-bg: #c4cec7;
  --table-active-color: #000;
  --table-hover-bg: #c9d4cc;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-info {
  --table-color: #000;
  --table-bg: #d4e1f1;
  --table-border-color: #aab4c1;
  --table-striped-bg: #cad6e5;
  --table-striped-color: #000;
  --table-active-bg: #bfcbd9;
  --table-active-color: #000;
  --table-hover-bg: #c4d0df;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-warning {
  --table-color: #000;
  --table-bg: #fcefdc;
  --table-border-color: #cabfb0;
  --table-striped-bg: #efe3d1;
  --table-striped-color: #000;
  --table-active-bg: #e3d7c6;
  --table-active-color: #000;
  --table-hover-bg: #e9ddcb;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-danger {
  --table-color: #000;
  --table-bg: #f3d4d4;
  --table-border-color: #c3aaa9;
  --table-striped-bg: #e7c9c9;
  --table-striped-color: #000;
  --table-active-bg: #dbbfbf;
  --table-active-color: #000;
  --table-hover-bg: #e1c4c4;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-light {
  --table-color: #000;
  --table-bg: #f8f9fa;
  --table-border-color: #c6c7c8;
  --table-striped-bg: #ecedee;
  --table-striped-color: #000;
  --table-active-bg: #dfe0e1;
  --table-active-color: #000;
  --table-hover-bg: #e5e6e7;
  --table-hover-color: #000;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-dark {
  --table-color: #fff;
  --table-bg: #212529;
  --table-border-color: #4d5154;
  --table-striped-bg: #2c3034;
  --table-striped-color: #fff;
  --table-active-bg: #373b3e;
  --table-active-color: #fff;
  --table-hover-bg: #323539;
  --table-hover-color: #fff;
  color: var(--table-color);
  border-color: var(--table-border-color);
}

.table-responsive {
  -webkit-overflow-scrolling: touch;
  overflow-x: auto;
}

@media (width <= 575.98px) {
  .table-responsive-sm {
    -webkit-overflow-scrolling: touch;
    overflow-x: auto;
  }
}

@media (width <= 767.98px) {
  .table-responsive-md {
    -webkit-overflow-scrolling: touch;
    overflow-x: auto;
  }
}

@media (width <= 991.98px) {
  .table-responsive-lg {
    -webkit-overflow-scrolling: touch;
    overflow-x: auto;
  }
}

@media (width <= 1199.98px) {
  .table-responsive-xl {
    -webkit-overflow-scrolling: touch;
    overflow-x: auto;
  }
}

@media (width <= 1399.98px) {
  .table-responsive-xxl {
    -webkit-overflow-scrolling: touch;
    overflow-x: auto;
  }
}

.form-label {
  margin-bottom: .5rem;
}

.col-form-label {
  padding-top: calc(.5rem + var(--border-width));
  padding-bottom: calc(.5rem + var(--border-width));
  font-size: inherit;
  margin-bottom: 0;
  line-height: 1.5;
}

.col-form-label-lg {
  padding-top: calc(.5rem + var(--border-width));
  padding-bottom: calc(.5rem + var(--border-width));
  font-size: 1.25rem;
}

.col-form-label-sm {
  padding-top: calc(.25rem + var(--border-width));
  padding-bottom: calc(.25rem + var(--border-width));
  font-size: .8rem;
}

.form-text {
  color: var(--secondary-color);
  margin-top: .25rem;
  font-size: .875em;
}

.form-control {
  width: 100%;
  color: var(--body-color);
  appearance: none;
  background-color: var(--body-bg);
  border: var(--border-width) solid var(--input-border-color);
  border-radius: var(--border-radius);
  background-clip: padding-box;
  padding: .5rem 1rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  transition: border-color .15s ease-in-out, box-shadow .15s ease-in-out;
  display: block;
}

@media (prefers-reduced-motion: reduce) {
  .form-control {
    transition: none;
  }
}

.form-control[type="file"] {
  overflow: hidden;
}

.form-control[type="file"]:not(:disabled):not([readonly]) {
  cursor: pointer;
}

.form-control:focus {
  color: var(--body-color);
  background-color: var(--body-bg);
  border-color: var(--focus);
  outline: 0;
  box-shadow: 0 0 0 .25rem #2a69b740;
}

.form-control::-webkit-date-and-time-value {
  min-width: 85px;
  height: 1.5em;
  margin: 0;
}

.form-control::-webkit-datetime-edit {
  padding: 0;
  display: block;
}

.form-control::placeholder {
  color: var(--secondary-color);
  opacity: 1;
}

.form-control:disabled {
  background-color: var(--secondary-bg);
  opacity: 1;
}

.form-control::file-selector-button {
  margin: -.5rem -1rem;
  color: var(--white);
  background-color: var(--tertiary-bg);
  pointer-events: none;
  border-color: inherit;
  border-style: solid;
  border-width: 0;
  border-inline-end-width: var(--border-width);
  border-radius: 0;
  margin-inline-end: 1rem;
  padding: .5rem 1rem;
  transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
}

@media (prefers-reduced-motion: reduce) {
  .form-control::file-selector-button {
    transition: none;
  }
}

.form-control:hover:not(:disabled):not([readonly])::file-selector-button {
  background-color: var(--secondary-bg);
}

.form-control-plaintext {
  width: 100%;
  color: var(--body-color);
  border: solid #0000;
  border-width: var(--border-width) 0;
  background-color: #0000;
  margin-bottom: 0;
  padding: .5rem 0;
  line-height: 1.5;
  display: block;
}

.form-control-plaintext:focus {
  outline: 0;
}

.form-control-plaintext.form-control-sm, .form-control-plaintext.form-control-lg {
  padding-left: 0;
  padding-right: 0;
}

.form-control-sm {
  min-height: calc(1.5em + .5rem + calc(var(--border-width) * 2));
  border-radius: var(--border-radius-sm);
  padding: .25rem .5rem;
  font-size: .8rem;
}

.form-control-sm::file-selector-button {
  margin: -.25rem -.5rem;
  margin-inline-end: .5rem;
  padding: .25rem .5rem;
}

.form-control-lg {
  min-height: calc(1.5em + 1rem + calc(var(--border-width) * 2));
  border-radius: var(--border-radius-lg);
  padding: .5rem 1rem;
  font-size: 1.25rem;
}

.form-control-lg::file-selector-button {
  margin: -.5rem -1rem;
  margin-inline-end: 1rem;
  padding: .5rem 1rem;
}

textarea.form-control {
  min-height: calc(1.5em + 1rem + calc(var(--border-width) * 2));
}

textarea.form-control-sm {
  min-height: calc(1.5em + .5rem + calc(var(--border-width) * 2));
}

textarea.form-control-lg {
  min-height: calc(1.5em + 1rem + calc(var(--border-width) * 2));
}

.form-control-color {
  width: 3rem;
  height: calc(1.5em + 1rem + calc(var(--border-width) * 2));
  padding: .5rem;
}

.form-control-color:not(:disabled):not([readonly]) {
  cursor: pointer;
}

.form-control-color::-moz-color-swatch {
  border-radius: var(--border-radius);
  border: 0 !important;
}

.form-control-color::-webkit-color-swatch {
  border-radius: var(--border-radius);
  border: 0 !important;
}

.form-control-color.form-control-sm {
  height: calc(1.5em + .5rem + calc(var(--border-width) * 2));
}

.form-control-color.form-control-lg {
  height: calc(1.5em + 1rem + calc(var(--border-width) * 2));
}

.form-select, .custom-select {
  --form-select-bg-img: url("../images/select-bg.svg?v=5ebba0");
  width: 100%;
  color: var(--body-color);
  appearance: none;
  background-color: var(--template-bg-light);
  background-image: var(--form-select-bg-img), var(--form-select-bg-icon, none);
  border: var(--border-width) solid var(--input-border-color);
  background-position: right 1rem center;
  background-repeat: no-repeat;
  background-size: max(100%, 116rem);
  border-radius: 4px;
  padding: .5rem 3rem .5rem 1rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  transition: border-color .15s ease-in-out, box-shadow .15s ease-in-out;
  display: block;
}

@media (prefers-reduced-motion: reduce) {
  .form-select, .custom-select {
    transition: none;
  }
}

.form-select:focus, .custom-select:focus {
  border-color: var(--focus);
  box-shadow: 0 0 0 .2rem var(--focus-shadow);
  outline: 0;
}

.form-select[multiple], [multiple].custom-select, .form-select[size]:not([size="1"]), [size].custom-select:not([size="1"]) {
  background-image: none;
  padding-right: 1rem;
}

.form-select:disabled, .custom-select:disabled {
  background-color: var(--form-select-disabled-bg);
}

.form-select:-moz-focusring, .custom-select:-moz-focusring {
  color: #0000;
  text-shadow: 0 0 0 var(--body-color);
}

.form-select-sm {
  border-radius: var(--border-radius-sm);
  padding-top: .25rem;
  padding-bottom: .25rem;
  padding-left: .5rem;
  font-size: .8rem;
}

.form-select-lg {
  border-radius: var(--border-radius-lg);
  padding-top: .5rem;
  padding-bottom: .5rem;
  padding-left: 1rem;
  font-size: 1.25rem;
}

[data-bs-theme="dark"] .form-select, [data-bs-theme="dark"] .custom-select {
  --form-select-bg-img: url("../images/select-bg-dark.svg?v=97d2dc");
}

.form-check {
  min-height: 1.5rem;
  margin-bottom: .125rem;
  padding-left: 1.5em;
  display: block;
}

.form-check .form-check-input {
  float: left;
  margin-left: -1.5em;
}

.form-check-reverse {
  text-align: right;
  padding-left: 0;
  padding-right: 1.5em;
}

.form-check-reverse .form-check-input {
  float: right;
  margin-left: 0;
  margin-right: -1.5em;
}

.form-check-input {
  --form-check-bg: var(--body-bg);
  vertical-align: top;
  appearance: none;
  background-color: var(--form-check-bg);
  background-image: var(--form-check-bg-image);
  border: var(--form-check-input-border);
  print-color-adjust: exact;
  background-position: center;
  background-repeat: no-repeat;
  background-size: contain;
  flex-shrink: 0;
  width: 1em;
  height: 1em;
  margin-top: .25em;
}

.form-check-input[type="checkbox"] {
  border-radius: .25em;
}

.form-check-input[type="radio"] {
  border-radius: 50%;
}

.form-check-input:active {
  filter: brightness(90%);
}

.form-check-input:focus {
  border-color: var(--focus);
  outline: 0;
  box-shadow: 0 0 0 .25rem #2a69b740;
}

.form-check-input:checked {
  background-color: var(--form-check-input-checked-bg-color);
  border-color: var(--form-check-input-checked-bg-color);
}

.form-check-input:checked[type="checkbox"] {
  --form-check-bg-image: var(--form-check-input-checked-bg-image);
}

.form-check-input:checked[type="radio"] {
  --form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='2' fill='%23fff'/%3e%3c/svg%3e");
}

.form-check-input[type="checkbox"]:indeterminate {
  --form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'%3e%3cpath fill='none' stroke='%23fff' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='M6 10h8'/%3e%3c/svg%3e");
  background-color: #2a69b7;
  border-color: #2a69b7;
}

.form-check-input:disabled {
  pointer-events: none;
  filter: none;
  opacity: .5;
}

.form-check-input[disabled] ~ .form-check-label, .form-check-input:disabled ~ .form-check-label {
  cursor: default;
  opacity: .5;
}

.form-switch {
  padding-left: 2.5em;
}

.form-switch .form-check-input {
  --form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='rgba%280, 0, 0, 0.25%29'/%3e%3c/svg%3e");
  background-image: var(--form-switch-bg);
  background-position: 0;
  border-radius: 2em;
  width: 2em;
  margin-left: -2.5em;
  transition: background-position .15s ease-in-out;
}

@media (prefers-reduced-motion: reduce) {
  .form-switch .form-check-input {
    transition: none;
  }
}

.form-switch .form-check-input:focus {
  --form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='var%28--focus%29'/%3e%3c/svg%3e");
}

.form-switch .form-check-input:checked {
  --form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%23fff'/%3e%3c/svg%3e");
  background-position: 100%;
}

.form-switch.form-check-reverse {
  padding-left: 0;
  padding-right: 2.5em;
}

.form-switch.form-check-reverse .form-check-input {
  margin-left: 0;
  margin-right: -2.5em;
}

.form-check-inline {
  margin-right: 1rem;
  display: inline-block;
}

.btn-check {
  clip: rect(0, 0, 0, 0);
  pointer-events: none;
  position: absolute;
}

.btn-check[disabled] + .btn, .btn-check:disabled + .btn {
  pointer-events: none;
  filter: none;
  opacity: .4;
}

[data-bs-theme="dark"] .form-switch .form-check-input:not(:checked):not(:focus) {
  --form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='rgba%28255, 255, 255, 0.25%29'/%3e%3c/svg%3e");
}

.form-range {
  appearance: none;
  background-color: #0000;
  width: 100%;
  height: 1.4rem;
  padding: 0;
}

.form-range:focus {
  outline: 0;
}

.form-range:focus::-webkit-slider-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 .25rem #2a69b740;
}

.form-range:focus::-moz-range-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 .25rem #2a69b740;
}

.form-range::-moz-focus-outer {
  border: 0;
}

.form-range::-webkit-slider-thumb {
  appearance: none;
  background-color: #2a69b7;
  border: 0;
  border-radius: 1rem;
  width: 1rem;
  height: 1rem;
  margin-top: -.25rem;
  transition: background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
}

@media (prefers-reduced-motion: reduce) {
  .form-range::-webkit-slider-thumb {
    transition: none;
  }
}

.form-range::-webkit-slider-thumb:active {
  background-color: #bfd2e9;
}

.form-range::-webkit-slider-runnable-track {
  color: #0000;
  cursor: pointer;
  background-color: var(--secondary-bg);
  border-color: #0000;
  border-radius: 1rem;
  width: 100%;
  height: .5rem;
}

.form-range::-moz-range-thumb {
  appearance: none;
  background-color: #2a69b7;
  border: 0;
  border-radius: 1rem;
  width: 1rem;
  height: 1rem;
  transition: background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
}

@media (prefers-reduced-motion: reduce) {
  .form-range::-moz-range-thumb {
    transition: none;
  }
}

.form-range::-moz-range-thumb:active {
  background-color: #bfd2e9;
}

.form-range::-moz-range-track {
  color: #0000;
  cursor: pointer;
  background-color: var(--secondary-bg);
  border-color: #0000;
  border-radius: 1rem;
  width: 100%;
  height: .5rem;
}

.form-range:disabled {
  pointer-events: none;
}

.form-range:disabled::-webkit-slider-thumb {
  background-color: var(--secondary-color);
}

.form-range:disabled::-moz-range-thumb {
  background-color: var(--secondary-color);
}

.form-floating {
  position: relative;
}

.form-floating > .form-control, .form-floating > .form-control-plaintext, .form-floating > .form-select, .form-floating > .custom-select {
  height: calc(3.5rem + calc(var(--border-width) * 2));
  min-height: calc(3.5rem + calc(var(--border-width) * 2));
  line-height: 1.25;
}

.form-floating > label {
  z-index: 2;
  max-width: 100%;
  height: 100%;
  color: rgba(var(--body-color-rgb), .65);
  text-align: start;
  text-overflow: ellipsis;
  white-space: nowrap;
  pointer-events: none;
  border: var(--border-width) solid transparent;
  transform-origin: 0 0;
  padding: 1rem;
  transition: opacity .1s ease-in-out, transform .1s ease-in-out;
  position: absolute;
  top: 0;
  left: 0;
  overflow: hidden;
}

@media (prefers-reduced-motion: reduce) {
  .form-floating > label {
    transition: none;
  }
}

.form-floating > .form-control, .form-floating > .form-control-plaintext {
  padding: 1rem;
}

.form-floating > .form-control::placeholder, .form-floating > .form-control-plaintext::placeholder {
  color: #0000;
}

.form-floating > .form-control:focus, .form-floating > .form-control:not(:placeholder-shown), .form-floating > .form-control-plaintext:focus, .form-floating > .form-control-plaintext:not(:placeholder-shown) {
  padding-top: 1.625rem;
  padding-bottom: .625rem;
}

.form-floating > .form-control:-webkit-autofill, .form-floating > .form-control-plaintext:-webkit-autofill {
  padding-top: 1.625rem;
  padding-bottom: .625rem;
}

.form-floating > .form-select, .form-floating > .custom-select {
  padding-top: 1.625rem;
  padding-bottom: .625rem;
  padding-left: 1rem;
}

.form-floating > .form-control:focus ~ label, .form-floating > .form-control:not(:placeholder-shown) ~ label, .form-floating > .form-control-plaintext ~ label, .form-floating > .form-select ~ label, .form-floating > .custom-select ~ label {
  transform: scale(.85)translateY(-.5rem)translateX(.15rem);
}

.form-floating > .form-control:-webkit-autofill ~ label {
  transform: scale(.85)translateY(-.5rem)translateX(.15rem);
}

.form-floating > textarea:focus ~ label:after, .form-floating > textarea:not(:placeholder-shown) ~ label:after {
  z-index: -1;
  content: "";
  background-color: var(--body-bg);
  border-radius: var(--border-radius);
  height: 1.5em;
  position: absolute;
  inset: 1rem .5rem;
}

.form-floating > textarea:disabled ~ label:after {
  background-color: var(--secondary-bg);
}

.form-floating > .form-control-plaintext ~ label {
  border-width: var(--border-width) 0;
}

.form-floating > :disabled ~ label, .form-floating > .form-control:disabled ~ label {
  color: #666e76;
}

.input-group {
  flex-wrap: wrap;
  align-items: stretch;
  width: 100%;
  display: flex;
  position: relative;
}

.input-group > .form-control, .input-group > .form-select, .input-group > .custom-select, .input-group > .form-floating {
  flex: auto;
  width: 1%;
  min-width: 0;
  position: relative;
}

.input-group > .form-control:focus, .input-group > .form-select:focus, .input-group > .custom-select:focus, .input-group > .form-floating:focus-within {
  z-index: 5;
}

.input-group .btn {
  z-index: 2;
  position: relative;
}

.input-group .btn:focus {
  z-index: 5;
}

.input-group-text {
  color: var(--white);
  text-align: center;
  white-space: nowrap;
  border: var(--border-width) solid var(--template-bg-dark);
  border-radius: var(--border-radius);
  background-color: #132f53;
  align-items: center;
  padding: .5rem 1rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  display: flex;
}

.input-group-lg > .form-control, .input-group-lg > .form-select, .input-group-lg > .custom-select, .input-group-lg > .input-group-text, .input-group-lg > .btn {
  border-radius: var(--border-radius-lg);
  padding: .5rem 1rem;
  font-size: 1.25rem;
}

.input-group-sm > .form-control, .input-group-sm > .form-select, .input-group-sm > .custom-select, .input-group-sm > .input-group-text, .input-group-sm > .btn {
  border-radius: var(--border-radius-sm);
  padding: .25rem .5rem;
  font-size: .8rem;
}

.input-group-lg > .form-select, .input-group-lg > .custom-select, .input-group-sm > .form-select, .input-group-sm > .custom-select {
  padding-right: 4rem;
}

.input-group:not(.has-validation) > :not(:last-child):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating), .input-group:not(.has-validation) > .dropdown-toggle:nth-last-child(n+3), .input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-control, .input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-select, .input-group:not(.has-validation) > .form-floating:not(:last-child) > .custom-select, .input-group.has-validation > :nth-last-child(n+3):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating), .input-group.has-validation > .dropdown-toggle:nth-last-child(n+4), .input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-control, .input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-select, .input-group.has-validation > .form-floating:nth-last-child(n+3) > .custom-select {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.input-group > :not(:first-child):not(.dropdown-menu):not(.valid-tooltip):not(.valid-feedback):not(.invalid-tooltip):not(.invalid-feedback) {
  margin-left: calc(-1 * var(--border-width));
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.input-group > .form-floating:not(:first-child) > .form-control, .input-group > .form-floating:not(:first-child) > .form-select, .input-group > .form-floating:not(:first-child) > .custom-select {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.valid-feedback {
  width: 100%;
  color: var(--form-valid-color);
  margin-top: .25rem;
  font-size: .875em;
  display: none;
}

.valid-tooltip {
  z-index: 5;
  color: #fff;
  background-color: var(--success);
  border-radius: var(--border-radius);
  max-width: 100%;
  margin-top: .1rem;
  padding: .25rem .5rem;
  font-size: .8rem;
  display: none;
  position: absolute;
  top: 100%;
}

.was-validated :valid ~ .valid-feedback, .was-validated :valid ~ .valid-tooltip, .is-valid ~ .valid-feedback, .is-valid ~ .valid-tooltip {
  display: block;
}

.was-validated .form-control:valid, .form-control.is-valid {
  border-color: var(--form-valid-border-color);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23457d54' d='M2.3 6.73.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1'/%3e%3c/svg%3e");
  background-position: right calc(.375em + .25rem) center;
  background-repeat: no-repeat;
  background-size: calc(.75em + .5rem) calc(.75em + .5rem);
  padding-right: calc(1.5em + 1rem);
}

.was-validated .form-control:valid:focus, .form-control.is-valid:focus {
  border-color: var(--form-valid-border-color);
  box-shadow: 0 0 0 .2rem rgba(var(--success-rgb), .25);
}

.was-validated textarea.form-control:valid, textarea.form-control.is-valid {
  background-position: right calc(.375em + .25rem) top calc(.375em + .25rem);
  padding-right: calc(1.5em + 1rem);
}

.was-validated .form-select:valid, .was-validated .custom-select:valid, .form-select.is-valid, .is-valid.custom-select {
  border-color: var(--form-valid-border-color);
}

.was-validated .form-select:valid:not([multiple]):not([size]), .was-validated .custom-select:valid:not([multiple]):not([size]), .was-validated .form-select:valid:not([multiple])[size="1"], .was-validated .custom-select:valid:not([multiple])[size="1"], .form-select.is-valid:not([multiple]):not([size]), .is-valid.custom-select:not([multiple]):not([size]), .form-select.is-valid:not([multiple])[size="1"], .is-valid.custom-select:not([multiple])[size="1"] {
  --form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23457d54' d='M2.3 6.73.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1'/%3e%3c/svg%3e");
  background-position: right 1rem center, right 3rem center;
  background-size: max(100%, 116rem), calc(.75em + .5rem) calc(.75em + .5rem);
  padding-right: 5.5rem;
}

.was-validated .form-select:valid:focus, .was-validated .custom-select:valid:focus, .form-select.is-valid:focus, .is-valid.custom-select:focus {
  border-color: var(--form-valid-border-color);
  box-shadow: 0 0 0 .2rem rgba(var(--success-rgb), .25);
}

.was-validated .form-control-color:valid, .form-control-color.is-valid {
  width: calc(1.5em + 4rem);
}

.was-validated .form-check-input:valid, .form-check-input.is-valid {
  border-color: var(--form-valid-border-color);
}

.was-validated .form-check-input:valid:checked, .form-check-input.is-valid:checked {
  background-color: var(--form-valid-color);
}

.was-validated .form-check-input:valid:focus, .form-check-input.is-valid:focus {
  box-shadow: 0 0 0 .2rem rgba(var(--success-rgb), .25);
}

.was-validated .form-check-input:valid ~ .form-check-label, .form-check-input.is-valid ~ .form-check-label {
  color: var(--form-valid-color);
}

.form-check-inline .form-check-input ~ .valid-feedback {
  margin-left: .5em;
}

.was-validated .input-group > .form-control:not(:focus):valid, .input-group > .form-control:not(:focus).is-valid, .was-validated .input-group > .form-select:not(:focus):valid, .was-validated .input-group > .custom-select:not(:focus):valid, .input-group > .form-select:not(:focus).is-valid, .input-group > .custom-select:not(:focus).is-valid, .was-validated .input-group > .form-floating:not(:focus-within):valid, .input-group > .form-floating:not(:focus-within).is-valid {
  z-index: 3;
}

.invalid-feedback {
  width: 100%;
  color: var(--form-invalid-color);
  margin-top: .25rem;
  font-size: .875em;
  display: none;
}

.invalid-tooltip {
  z-index: 5;
  color: #fff;
  background-color: var(--danger);
  border-radius: var(--border-radius);
  max-width: 100%;
  margin-top: .1rem;
  padding: .25rem .5rem;
  font-size: .8rem;
  display: none;
  position: absolute;
  top: 100%;
}

.was-validated :invalid ~ .invalid-feedback, .was-validated :invalid ~ .invalid-tooltip, .is-invalid ~ .invalid-feedback, .is-invalid ~ .invalid-tooltip {
  display: block;
}

.was-validated .form-control:invalid, .form-control.is-invalid {
  border-color: var(--form-invalid-border-color);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23c52827'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23c52827' stroke='none'/%3e%3c/svg%3e");
  background-position: right calc(.375em + .25rem) center;
  background-repeat: no-repeat;
  background-size: calc(.75em + .5rem) calc(.75em + .5rem);
  padding-right: calc(1.5em + 1rem);
}

.was-validated .form-control:invalid:focus, .form-control.is-invalid:focus {
  border-color: var(--form-invalid-border-color);
  box-shadow: 0 0 0 .2rem rgba(var(--danger-rgb), .25);
}

.was-validated textarea.form-control:invalid, textarea.form-control.is-invalid {
  background-position: right calc(.375em + .25rem) top calc(.375em + .25rem);
  padding-right: calc(1.5em + 1rem);
}

.was-validated .form-select:invalid, .was-validated .custom-select:invalid, .form-select.is-invalid, .is-invalid.custom-select {
  border-color: var(--form-invalid-border-color);
}

.was-validated .form-select:invalid:not([multiple]):not([size]), .was-validated .custom-select:invalid:not([multiple]):not([size]), .was-validated .form-select:invalid:not([multiple])[size="1"], .was-validated .custom-select:invalid:not([multiple])[size="1"], .form-select.is-invalid:not([multiple]):not([size]), .is-invalid.custom-select:not([multiple]):not([size]), .form-select.is-invalid:not([multiple])[size="1"], .is-invalid.custom-select:not([multiple])[size="1"] {
  --form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23c52827'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23c52827' stroke='none'/%3e%3c/svg%3e");
  background-position: right 1rem center, right 3rem center;
  background-size: max(100%, 116rem), calc(.75em + .5rem) calc(.75em + .5rem);
  padding-right: 5.5rem;
}

.was-validated .form-select:invalid:focus, .was-validated .custom-select:invalid:focus, .form-select.is-invalid:focus, .is-invalid.custom-select:focus {
  border-color: var(--form-invalid-border-color);
  box-shadow: 0 0 0 .2rem rgba(var(--danger-rgb), .25);
}

.was-validated .form-control-color:invalid, .form-control-color.is-invalid {
  width: calc(1.5em + 4rem);
}

.was-validated .form-check-input:invalid, .form-check-input.is-invalid {
  border-color: var(--form-invalid-border-color);
}

.was-validated .form-check-input:invalid:checked, .form-check-input.is-invalid:checked {
  background-color: var(--form-invalid-color);
}

.was-validated .form-check-input:invalid:focus, .form-check-input.is-invalid:focus {
  box-shadow: 0 0 0 .2rem rgba(var(--danger-rgb), .25);
}

.was-validated .form-check-input:invalid ~ .form-check-label, .form-check-input.is-invalid ~ .form-check-label {
  color: var(--form-invalid-color);
}

.form-check-inline .form-check-input ~ .invalid-feedback {
  margin-left: .5em;
}

.was-validated .input-group > .form-control:not(:focus):invalid, .input-group > .form-control:not(:focus).is-invalid, .was-validated .input-group > .form-select:not(:focus):invalid, .was-validated .input-group > .custom-select:not(:focus):invalid, .input-group > .form-select:not(:focus).is-invalid, .input-group > .custom-select:not(:focus).is-invalid, .was-validated .input-group > .form-floating:not(:focus-within):invalid, .input-group > .form-floating:not(:focus-within).is-invalid {
  z-index: 4;
}

.btn {
  --btn-padding-x: 1rem;
  --btn-padding-y: .5rem;
  --btn-font-family: ;
  --btn-font-size: 1rem;
  --btn-font-weight: 400;
  --btn-line-height: 1.5;
  --btn-color: var(--body-color);
  --btn-bg: transparent;
  --btn-border-width: var(--border-width);
  --btn-border-color: transparent;
  --btn-border-radius: .25rem;
  --btn-hover-border-color: transparent;
  --btn-box-shadow: inset 0 1px 0 #ffffff26, 0 1px 1px #00000013;
  --btn-disabled-opacity: .4;
  --btn-focus-box-shadow: 0 0 0 .2rem rgba(var(--btn-focus-shadow-rgb), .5);
  padding: var(--btn-padding-y) var(--btn-padding-x);
  font-family: var(--btn-font-family);
  font-size: var(--btn-font-size);
  font-weight: var(--btn-font-weight);
  line-height: var(--btn-line-height);
  color: var(--btn-color);
  text-align: center;
  vertical-align: middle;
  cursor: pointer;
  user-select: none;
  border: var(--btn-border-width) solid var(--btn-border-color);
  border-radius: var(--btn-border-radius);
  background-color: var(--btn-bg);
  transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
  display: inline-block;
}

@media (prefers-reduced-motion: reduce) {
  .btn {
    transition: none;
  }
}

.btn:hover {
  color: var(--btn-hover-color);
  background-color: var(--btn-hover-bg);
  border-color: var(--btn-hover-border-color);
}

.btn-check + .btn:hover {
  color: var(--btn-color);
  background-color: var(--btn-bg);
  border-color: var(--btn-border-color);
}

.btn:focus-visible {
  color: var(--btn-hover-color);
  background-color: var(--btn-hover-bg);
  border-color: var(--btn-hover-border-color);
  box-shadow: var(--btn-focus-box-shadow);
  outline: 0;
}

.btn-check:focus-visible + .btn {
  border-color: var(--btn-hover-border-color);
  box-shadow: var(--btn-focus-box-shadow);
  outline: 0;
}

.btn-check:checked + .btn, :not(.btn-check) + .btn:active, .btn:first-child:active, .btn.active, .btn.show {
  color: var(--btn-active-color);
  background-color: var(--btn-active-bg);
  border-color: var(--btn-active-border-color);
}

.btn-check:checked + .btn:focus-visible, :not(.btn-check) + .btn:active:focus-visible, .btn:first-child:active:focus-visible, .btn.active:focus-visible, .btn.show:focus-visible, .btn-check:checked:focus-visible + .btn {
  box-shadow: var(--btn-focus-box-shadow);
}

.btn:disabled, .btn.disabled, fieldset:disabled .btn {
  color: var(--btn-disabled-color);
  pointer-events: none;
  background-color: var(--btn-disabled-bg);
  border-color: var(--btn-disabled-border-color);
  opacity: var(--btn-disabled-opacity);
}

.btn-primary {
  --btn-color: #fff;
  --btn-bg: #132f53;
  --btn-border-color: #132f53;
  --btn-hover-color: #fff;
  --btn-hover-bg: #102847;
  --btn-hover-border-color: #0f2642;
  --btn-focus-shadow-rgb: 54, 78, 109;
  --btn-active-color: #fff;
  --btn-active-bg: #0f2642;
  --btn-active-border-color: #0e233e;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #132f53;
  --btn-disabled-border-color: #132f53;
}

.btn-secondary {
  --btn-color: #fff;
  --btn-bg: #495057;
  --btn-border-color: #495057;
  --btn-hover-color: #fff;
  --btn-hover-bg: #3e444a;
  --btn-hover-border-color: #3a4046;
  --btn-focus-shadow-rgb: 100, 106, 112;
  --btn-active-color: #fff;
  --btn-active-bg: #3a4046;
  --btn-active-border-color: #373c41;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #495057;
  --btn-disabled-border-color: #495057;
}

.btn-success {
  --btn-color: #fff;
  --btn-bg: #457d54;
  --btn-border-color: #457d54;
  --btn-hover-color: #fff;
  --btn-hover-bg: #3b6a47;
  --btn-hover-border-color: #376443;
  --btn-focus-shadow-rgb: 97, 145, 110;
  --btn-active-color: #fff;
  --btn-active-bg: #376443;
  --btn-active-border-color: #345e3f;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #457d54;
  --btn-disabled-border-color: #457d54;
}

.btn-info {
  --btn-color: #fff;
  --btn-bg: #2a69b8;
  --btn-border-color: #2a69b8;
  --btn-hover-color: #fff;
  --btn-hover-bg: #24599c;
  --btn-hover-border-color: #225493;
  --btn-focus-shadow-rgb: 74, 128, 195;
  --btn-active-color: #fff;
  --btn-active-bg: #225493;
  --btn-active-border-color: #204f8a;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #2a69b8;
  --btn-disabled-border-color: #2a69b8;
}

.btn-warning {
  --btn-color: #000;
  --btn-bg: #ffb514;
  --btn-border-color: #ffb514;
  --btn-hover-color: #000;
  --btn-hover-bg: #ffc037;
  --btn-hover-border-color: #ffbc2c;
  --btn-focus-shadow-rgb: 217, 154, 17;
  --btn-active-color: #000;
  --btn-active-bg: #ffc443;
  --btn-active-border-color: #ffbc2c;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #000;
  --btn-disabled-bg: #ffb514;
  --btn-disabled-border-color: #ffb514;
}

.btn-danger {
  --btn-color: #fff;
  --btn-bg: #c52827;
  --btn-border-color: #c52827;
  --btn-hover-color: #fff;
  --btn-hover-bg: #a72221;
  --btn-hover-border-color: #9e201f;
  --btn-focus-shadow-rgb: 206, 72, 71;
  --btn-active-color: #fff;
  --btn-active-bg: #9e201f;
  --btn-active-border-color: #941e1d;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #c52827;
  --btn-disabled-border-color: #c52827;
}

.btn-light {
  --btn-color: #000;
  --btn-bg: #f8f9fa;
  --btn-border-color: #f8f9fa;
  --btn-hover-color: #000;
  --btn-hover-bg: #d3d4d5;
  --btn-hover-border-color: #c6c7c8;
  --btn-focus-shadow-rgb: 211, 212, 213;
  --btn-active-color: #000;
  --btn-active-bg: #c6c7c8;
  --btn-active-border-color: #babbbc;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #000;
  --btn-disabled-bg: #f8f9fa;
  --btn-disabled-border-color: #f8f9fa;
}

.btn-dark {
  --btn-color: #fff;
  --btn-bg: #212529;
  --btn-border-color: #212529;
  --btn-hover-color: #fff;
  --btn-hover-bg: #424649;
  --btn-hover-border-color: #373b3e;
  --btn-focus-shadow-rgb: 66, 70, 73;
  --btn-active-color: #fff;
  --btn-active-bg: #4d5154;
  --btn-active-border-color: #373b3e;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #212529;
  --btn-disabled-border-color: #212529;
}

.btn-action {
  --btn-color: #fff;
  --btn-bg: #132f53;
  --btn-border-color: #132f53;
  --btn-hover-color: #fff;
  --btn-hover-bg: #102847;
  --btn-hover-border-color: #0f2642;
  --btn-focus-shadow-rgb: 54, 78, 109;
  --btn-active-color: #fff;
  --btn-active-bg: #0f2642;
  --btn-active-border-color: #0e233e;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #132f53;
  --btn-disabled-border-color: #132f53;
}

.btn-error {
  --btn-color: #fff;
  --btn-bg: #3b0d0c;
  --btn-border-color: #3b0d0c;
  --btn-hover-color: #fff;
  --btn-hover-bg: #320b0a;
  --btn-hover-border-color: #2f0a0a;
  --btn-focus-shadow-rgb: 88, 49, 48;
  --btn-active-color: #fff;
  --btn-active-bg: #2f0a0a;
  --btn-active-border-color: #2c0a09;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #fff;
  --btn-disabled-bg: #3b0d0c;
  --btn-disabled-border-color: #3b0d0c;
}

.btn-outline-primary {
  --btn-color: #132f53;
  --btn-border-color: #132f53;
  --btn-hover-color: #fff;
  --btn-hover-bg: #132f53;
  --btn-hover-border-color: #132f53;
  --btn-focus-shadow-rgb: 19, 47, 83;
  --btn-active-color: #fff;
  --btn-active-bg: #132f53;
  --btn-active-border-color: #132f53;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #132f53;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #132f53;
  --gradient: none;
}

.btn-outline-secondary {
  --btn-color: #495057;
  --btn-border-color: #495057;
  --btn-hover-color: #fff;
  --btn-hover-bg: #495057;
  --btn-hover-border-color: #495057;
  --btn-focus-shadow-rgb: 73, 80, 87;
  --btn-active-color: #fff;
  --btn-active-bg: #495057;
  --btn-active-border-color: #495057;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #495057;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #495057;
  --gradient: none;
}

.btn-outline-success {
  --btn-color: #457d54;
  --btn-border-color: #457d54;
  --btn-hover-color: #fff;
  --btn-hover-bg: #457d54;
  --btn-hover-border-color: #457d54;
  --btn-focus-shadow-rgb: 69, 125, 84;
  --btn-active-color: #fff;
  --btn-active-bg: #457d54;
  --btn-active-border-color: #457d54;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #457d54;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #457d54;
  --gradient: none;
}

.btn-outline-info {
  --btn-color: #2a69b8;
  --btn-border-color: #2a69b8;
  --btn-hover-color: #fff;
  --btn-hover-bg: #2a69b8;
  --btn-hover-border-color: #2a69b8;
  --btn-focus-shadow-rgb: 42, 105, 184;
  --btn-active-color: #fff;
  --btn-active-bg: #2a69b8;
  --btn-active-border-color: #2a69b8;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #2a69b8;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #2a69b8;
  --gradient: none;
}

.btn-outline-warning {
  --btn-color: #ffb514;
  --btn-border-color: #ffb514;
  --btn-hover-color: #000;
  --btn-hover-bg: #ffb514;
  --btn-hover-border-color: #ffb514;
  --btn-focus-shadow-rgb: 255, 181, 20;
  --btn-active-color: #000;
  --btn-active-bg: #ffb514;
  --btn-active-border-color: #ffb514;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #ffb514;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #ffb514;
  --gradient: none;
}

.btn-outline-danger {
  --btn-color: #c52827;
  --btn-border-color: #c52827;
  --btn-hover-color: #fff;
  --btn-hover-bg: #c52827;
  --btn-hover-border-color: #c52827;
  --btn-focus-shadow-rgb: 197, 40, 39;
  --btn-active-color: #fff;
  --btn-active-bg: #c52827;
  --btn-active-border-color: #c52827;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #c52827;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #c52827;
  --gradient: none;
}

.btn-outline-light {
  --btn-color: #f8f9fa;
  --btn-border-color: #f8f9fa;
  --btn-hover-color: #000;
  --btn-hover-bg: #f8f9fa;
  --btn-hover-border-color: #f8f9fa;
  --btn-focus-shadow-rgb: 248, 249, 250;
  --btn-active-color: #000;
  --btn-active-bg: #f8f9fa;
  --btn-active-border-color: #f8f9fa;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #f8f9fa;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #f8f9fa;
  --gradient: none;
}

.btn-outline-dark {
  --btn-color: #212529;
  --btn-border-color: #212529;
  --btn-hover-color: #fff;
  --btn-hover-bg: #212529;
  --btn-hover-border-color: #212529;
  --btn-focus-shadow-rgb: 33, 37, 41;
  --btn-active-color: #fff;
  --btn-active-bg: #212529;
  --btn-active-border-color: #212529;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #212529;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #212529;
  --gradient: none;
}

.btn-outline-action {
  --btn-color: #132f53;
  --btn-border-color: #132f53;
  --btn-hover-color: #fff;
  --btn-hover-bg: #132f53;
  --btn-hover-border-color: #132f53;
  --btn-focus-shadow-rgb: 19, 47, 83;
  --btn-active-color: #fff;
  --btn-active-bg: #132f53;
  --btn-active-border-color: #132f53;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #132f53;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #132f53;
  --gradient: none;
}

.btn-outline-error {
  --btn-color: #3b0d0c;
  --btn-border-color: #3b0d0c;
  --btn-hover-color: #fff;
  --btn-hover-bg: #3b0d0c;
  --btn-hover-border-color: #3b0d0c;
  --btn-focus-shadow-rgb: 59, 13, 12;
  --btn-active-color: #fff;
  --btn-active-bg: #3b0d0c;
  --btn-active-border-color: #3b0d0c;
  --btn-active-shadow: inset 0 3px 5px #00000020;
  --btn-disabled-color: #3b0d0c;
  --btn-disabled-bg: transparent;
  --btn-disabled-border-color: #3b0d0c;
  --gradient: none;
}

.btn-link {
  --btn-font-weight: 400;
  --btn-color: var(--link-color);
  --btn-bg: transparent;
  --btn-border-color: transparent;
  --btn-hover-color: var(--link-hover-color);
  --btn-hover-border-color: transparent;
  --btn-active-color: var(--link-hover-color);
  --btn-active-border-color: transparent;
  --btn-disabled-color: #666e76;
  --btn-disabled-border-color: transparent;
  --btn-box-shadow: 0 0 0 #000;
  --btn-focus-shadow-rgb: 74, 128, 195;
  text-decoration: none;
}

.btn-link:focus-visible {
  color: var(--btn-color);
}

.btn-link:hover {
  color: var(--btn-hover-color);
}

.btn-lg, .btn-group-lg > .btn {
  --btn-padding-y: .5rem;
  --btn-padding-x: 1rem;
  --btn-font-size: 1.25rem;
  --btn-border-radius: var(--border-radius-lg);
}

.btn-sm, .btn-group-sm > .btn {
  --btn-padding-y: .25rem;
  --btn-padding-x: .5rem;
  --btn-font-size: .8rem;
  --btn-border-radius: var(--border-radius-sm);
}

.fade {
  transition: opacity .15s linear;
}

@media (prefers-reduced-motion: reduce) {
  .fade {
    transition: none;
  }
}

.fade:not(.show) {
  opacity: 0;
}

.collapse:not(.show) {
  display: none;
}

.collapsing {
  height: 0;
  transition: height .35s;
  overflow: hidden;
}

@media (prefers-reduced-motion: reduce) {
  .collapsing {
    transition: none;
  }
}

.collapsing.collapse-horizontal {
  width: 0;
  height: auto;
  transition: width .35s;
}

@media (prefers-reduced-motion: reduce) {
  .collapsing.collapse-horizontal {
    transition: none;
  }
}

.dropup, .dropend, .dropdown, .dropstart, .dropup-center, .dropdown-center {
  position: relative;
}

.dropdown-toggle {
  white-space: nowrap;
}

.dropdown-toggle:after {
  vertical-align: .255em;
  content: "";
  border: .3em solid #0000;
  border-top-color: currentColor;
  border-bottom: 0;
  margin-left: .255em;
  display: inline-block;
}

.dropdown-toggle:empty:after {
  margin-left: 0;
}

.dropdown-menu {
  --dropdown-zindex: 1000;
  --dropdown-min-width: 10rem;
  --dropdown-padding-x: 0;
  --dropdown-padding-y: 0;
  --dropdown-spacer: 0;
  --dropdown-font-size: 1rem;
  --dropdown-color: var(--body-color);
  --dropdown-bg: var(--body-bg);
  --dropdown-border-color: var(--border-color-translucent);
  --dropdown-border-radius: var(--border-radius);
  --dropdown-border-width: 1px;
  --dropdown-inner-border-radius: calc(var(--border-radius)  - 1px);
  --dropdown-divider-bg: var(--border-color-translucent);
  --dropdown-divider-margin-y: .5rem;
  --dropdown-box-shadow: var(--dropdown-box-shadow);
  --dropdown-link-color: var(--body-color);
  --dropdown-link-hover-color: var(--template-text-dark);
  --dropdown-link-hover-bg: var(--tertiary-bg);
  --dropdown-link-active-color: #fff;
  --dropdown-link-active-bg: #2a69b7;
  --dropdown-link-disabled-color: var(--tertiary-color);
  --dropdown-item-padding-x: .75rem;
  --dropdown-item-padding-y: .5rem;
  --dropdown-header-color: #666e76;
  --dropdown-header-padding-x: .75rem;
  --dropdown-header-padding-y: 0;
  z-index: var(--dropdown-zindex);
  min-width: var(--dropdown-min-width);
  padding: var(--dropdown-padding-y) var(--dropdown-padding-x);
  font-size: var(--dropdown-font-size);
  color: var(--dropdown-color);
  text-align: left;
  background-color: var(--dropdown-bg);
  border: var(--dropdown-border-width) solid var(--dropdown-border-color);
  border-radius: var(--dropdown-border-radius);
  background-clip: padding-box;
  margin: 0;
  list-style: none;
  display: none;
  position: absolute;
}

.dropdown-menu[data-bs-popper] {
  margin-top: var(--dropdown-spacer);
  top: 100%;
  left: 0;
}

.dropdown-menu > .dropdown-item:first-child, .dropdown-menu > li:first-child .dropdown-item {
  border-top-left-radius: var(--dropdown-inner-border-radius);
  border-top-right-radius: var(--dropdown-inner-border-radius);
}

.dropdown-menu > .dropdown-item:last-child, .dropdown-menu > li:last-child .dropdown-item {
  border-bottom-right-radius: var(--dropdown-inner-border-radius);
  border-bottom-left-radius: var(--dropdown-inner-border-radius);
}

.dropdown-menu-start {
  --bs-position: start;
}

.dropdown-menu-start[data-bs-popper] {
  left: 0;
  right: auto;
}

.dropdown-menu-end {
  --bs-position: end;
}

.dropdown-menu-end[data-bs-popper] {
  left: auto;
  right: 0;
}

@media (width >= 576px) {
  .dropdown-menu-sm-start {
    --bs-position: start;
  }

  .dropdown-menu-sm-start[data-bs-popper] {
    left: 0;
    right: auto;
  }

  .dropdown-menu-sm-end {
    --bs-position: end;
  }

  .dropdown-menu-sm-end[data-bs-popper] {
    left: auto;
    right: 0;
  }
}

@media (width >= 768px) {
  .dropdown-menu-md-start {
    --bs-position: start;
  }

  .dropdown-menu-md-start[data-bs-popper] {
    left: 0;
    right: auto;
  }

  .dropdown-menu-md-end {
    --bs-position: end;
  }

  .dropdown-menu-md-end[data-bs-popper] {
    left: auto;
    right: 0;
  }
}

@media (width >= 992px) {
  .dropdown-menu-lg-start {
    --bs-position: start;
  }

  .dropdown-menu-lg-start[data-bs-popper] {
    left: 0;
    right: auto;
  }

  .dropdown-menu-lg-end {
    --bs-position: end;
  }

  .dropdown-menu-lg-end[data-bs-popper] {
    left: auto;
    right: 0;
  }
}

@media (width >= 1200px) {
  .dropdown-menu-xl-start {
    --bs-position: start;
  }

  .dropdown-menu-xl-start[data-bs-popper] {
    left: 0;
    right: auto;
  }

  .dropdown-menu-xl-end {
    --bs-position: end;
  }

  .dropdown-menu-xl-end[data-bs-popper] {
    left: auto;
    right: 0;
  }
}

@media (width >= 1400px) {
  .dropdown-menu-xxl-start {
    --bs-position: start;
  }

  .dropdown-menu-xxl-start[data-bs-popper] {
    left: 0;
    right: auto;
  }

  .dropdown-menu-xxl-end {
    --bs-position: end;
  }

  .dropdown-menu-xxl-end[data-bs-popper] {
    left: auto;
    right: 0;
  }
}

.dropup .dropdown-menu[data-bs-popper] {
  margin-top: 0;
  margin-bottom: var(--dropdown-spacer);
  top: auto;
  bottom: 100%;
}

.dropup .dropdown-toggle:after {
  vertical-align: .255em;
  content: "";
  border: .3em solid #0000;
  border-top: 0;
  border-bottom-color: currentColor;
  margin-left: .255em;
  display: inline-block;
}

.dropup .dropdown-toggle:empty:after {
  margin-left: 0;
}

.dropend .dropdown-menu[data-bs-popper] {
  margin-top: 0;
  margin-left: var(--dropdown-spacer);
  top: 0;
  left: 100%;
  right: auto;
}

.dropend .dropdown-toggle:after {
  vertical-align: .255em;
  content: "";
  border: .3em solid #0000;
  border-left-color: currentColor;
  border-right: 0;
  margin-left: .255em;
  display: inline-block;
}

.dropend .dropdown-toggle:empty:after {
  margin-left: 0;
}

.dropend .dropdown-toggle:after {
  vertical-align: 0;
}

.dropstart .dropdown-menu[data-bs-popper] {
  margin-top: 0;
  margin-right: var(--dropdown-spacer);
  top: 0;
  left: auto;
  right: 100%;
}

.dropstart .dropdown-toggle:after {
  vertical-align: .255em;
  content: "";
  margin-left: .255em;
  display: none;
}

.dropstart .dropdown-toggle:before {
  vertical-align: .255em;
  content: "";
  border-top: .3em solid #0000;
  border-bottom: .3em solid #0000;
  border-right: .3em solid;
  margin-right: .255em;
  display: inline-block;
}

.dropstart .dropdown-toggle:empty:after {
  margin-left: 0;
}

.dropstart .dropdown-toggle:before {
  vertical-align: 0;
}

.dropdown-divider {
  height: 0;
  margin: var(--dropdown-divider-margin-y) 0;
  border-top: 1px solid var(--dropdown-divider-bg);
  opacity: 1;
  overflow: hidden;
}

.dropdown-item {
  width: 100%;
  padding: var(--dropdown-item-padding-y) var(--dropdown-item-padding-x);
  clear: both;
  color: var(--dropdown-link-color);
  text-align: inherit;
  white-space: nowrap;
  border-radius: var(--dropdown-item-border-radius, 0);
  background-color: #0000;
  border: 0;
  font-weight: 400;
  display: block;
}

.dropdown-item:hover, .dropdown-item:focus {
  color: var(--dropdown-link-hover-color);
  background-color: var(--dropdown-link-hover-bg);
}

.dropdown-item.active, .dropdown-item:active {
  color: var(--dropdown-link-active-color);
  background-color: var(--dropdown-link-active-bg);
  text-decoration: none;
}

.dropdown-item.disabled, .dropdown-item:disabled {
  color: var(--dropdown-link-disabled-color);
  pointer-events: none;
  background-color: #0000;
}

.dropdown-menu.show {
  display: block;
}

.dropdown-header {
  padding: var(--dropdown-header-padding-y) var(--dropdown-header-padding-x);
  color: var(--dropdown-header-color);
  white-space: nowrap;
  margin-bottom: 0;
  font-size: .8rem;
  display: block;
}

.dropdown-item-text {
  padding: var(--dropdown-item-padding-y) var(--dropdown-item-padding-x);
  color: var(--dropdown-link-color);
  display: block;
}

.dropdown-menu-dark {
  --dropdown-color: #dee2e6;
  --dropdown-bg: #343a40;
  --dropdown-border-color: var(--border-color-translucent);
  --dropdown-box-shadow: ;
  --dropdown-link-color: #dee2e6;
  --dropdown-link-hover-color: #fff;
  --dropdown-divider-bg: var(--border-color-translucent);
  --dropdown-link-hover-bg: #ffffff26;
  --dropdown-link-active-color: #fff;
  --dropdown-link-active-bg: #2a69b7;
  --dropdown-link-disabled-color: #adb5bd;
  --dropdown-header-color: #adb5bd;
}

.btn-group, .btn-group-vertical {
  vertical-align: middle;
  display: inline-flex;
  position: relative;
}

.btn-group > .btn, .btn-group-vertical > .btn {
  flex: auto;
  position: relative;
}

.btn-group > .btn-check:checked + .btn, .btn-group > .btn-check:focus + .btn, .btn-group > .btn:hover, .btn-group > .btn:focus, .btn-group > .btn:active, .btn-group > .btn.active, .btn-group-vertical > .btn-check:checked + .btn, .btn-group-vertical > .btn-check:focus + .btn, .btn-group-vertical > .btn:hover, .btn-group-vertical > .btn:focus, .btn-group-vertical > .btn:active, .btn-group-vertical > .btn.active {
  z-index: 1;
}

.btn-toolbar {
  flex-wrap: wrap;
  justify-content: flex-start;
  display: flex;
}

.btn-toolbar .input-group {
  width: auto;
}

.btn-group {
  border-radius: .25rem;
}

.btn-group > :not(.btn-check:first-child) + .btn, .btn-group > .btn-group:not(:first-child) {
  margin-left: calc(-1 * var(--border-width));
}

.btn-group > .btn:not(:last-child):not(.dropdown-toggle), .btn-group > .btn.dropdown-toggle-split:first-child, .btn-group > .btn-group:not(:last-child) > .btn {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.btn-group > .btn:nth-child(n+3), .btn-group > :not(.btn-check) + .btn, .btn-group > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.dropdown-toggle-split {
  padding-left: .75rem;
  padding-right: .75rem;
}

.dropdown-toggle-split:after, .dropup .dropdown-toggle-split:after, .dropend .dropdown-toggle-split:after {
  margin-left: 0;
}

.dropstart .dropdown-toggle-split:before {
  margin-right: 0;
}

.btn-sm + .dropdown-toggle-split, .btn-group-sm > .btn + .dropdown-toggle-split {
  padding-left: .375rem;
  padding-right: .375rem;
}

.btn-lg + .dropdown-toggle-split, .btn-group-lg > .btn + .dropdown-toggle-split {
  padding-left: .75rem;
  padding-right: .75rem;
}

.btn-group-vertical {
  flex-direction: column;
  justify-content: center;
  align-items: flex-start;
}

.btn-group-vertical > .btn, .btn-group-vertical > .btn-group {
  width: 100%;
}

.btn-group-vertical > .btn:not(:first-child), .btn-group-vertical > .btn-group:not(:first-child) {
  margin-top: calc(-1 * var(--border-width));
}

.btn-group-vertical > .btn:not(:last-child):not(.dropdown-toggle), .btn-group-vertical > .btn-group:not(:last-child) > .btn {
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}

.btn-group-vertical > .btn:nth-child(n+3), .btn-group-vertical > :not(.btn-check) + .btn, .btn-group-vertical > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.nav {
  --nav-link-padding-x: 1rem;
  --nav-link-padding-y: .5rem;
  --nav-link-font-weight: ;
  --nav-link-color: var(--link-color);
  --nav-link-hover-color: var(--link-hover-color);
  --nav-link-disabled-color: var(--secondary-color);
  flex-wrap: wrap;
  margin-bottom: 0;
  padding-left: 0;
  list-style: none;
  display: flex;
}

.nav-link {
  padding: var(--nav-link-padding-y) var(--nav-link-padding-x);
  font-size: var(--nav-link-font-size);
  font-weight: var(--nav-link-font-weight);
  color: var(--nav-link-color);
  background: none;
  border: 0;
  transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out;
  display: block;
}

@media (prefers-reduced-motion: reduce) {
  .nav-link {
    transition: none;
  }
}

.nav-link:hover, .nav-link:focus {
  color: var(--nav-link-hover-color);
}

.nav-link:focus-visible {
  outline: 0;
  box-shadow: 0 0 0 .25rem #2a69b740;
}

.nav-link.disabled, .nav-link:disabled {
  color: var(--nav-link-disabled-color);
  pointer-events: none;
  cursor: default;
}

.nav-tabs {
  --nav-tabs-border-width: var(--border-width);
  --nav-tabs-border-color: var(--border-color);
  --nav-tabs-border-radius: var(--border-radius);
  --nav-tabs-link-hover-border-color: var(--secondary-bg) var(--secondary-bg) var(--border-color);
  --nav-tabs-link-active-color: var(--emphasis-color);
  --nav-tabs-link-active-bg: var(--body-bg);
  --nav-tabs-link-active-border-color: var(--border-color) var(--border-color) var(--body-bg);
  border-bottom: var(--nav-tabs-border-width) solid var(--nav-tabs-border-color);
}

.nav-tabs .nav-link {
  margin-bottom: calc(-1 * var(--nav-tabs-border-width));
  border: var(--nav-tabs-border-width) solid transparent;
  border-top-left-radius: var(--nav-tabs-border-radius);
  border-top-right-radius: var(--nav-tabs-border-radius);
}

.nav-tabs .nav-link:hover, .nav-tabs .nav-link:focus {
  isolation: isolate;
  border-color: var(--nav-tabs-link-hover-border-color);
}

.nav-tabs .nav-link.active, .nav-tabs .nav-item.show .nav-link {
  color: var(--nav-tabs-link-active-color);
  background-color: var(--nav-tabs-link-active-bg);
  border-color: var(--nav-tabs-link-active-border-color);
}

.nav-tabs .dropdown-menu {
  margin-top: calc(-1 * var(--nav-tabs-border-width));
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.nav-pills {
  --nav-pills-border-radius: var(--border-radius);
  --nav-pills-link-active-color: #fff;
  --nav-pills-link-active-bg: #2a69b7;
}

.nav-pills .nav-link {
  border-radius: var(--nav-pills-border-radius);
}

.nav-pills .nav-link.active, .nav-pills .show > .nav-link {
  color: var(--nav-pills-link-active-color);
  background-color: var(--nav-pills-link-active-bg);
}

.nav-underline {
  --nav-underline-gap: 1rem;
  --nav-underline-border-width: .125rem;
  --nav-underline-link-active-color: var(--emphasis-color);
  gap: var(--nav-underline-gap);
}

.nav-underline .nav-link {
  border-bottom: var(--nav-underline-border-width) solid transparent;
  padding-left: 0;
  padding-right: 0;
}

.nav-underline .nav-link:hover, .nav-underline .nav-link:focus {
  border-bottom-color: currentColor;
}

.nav-underline .nav-link.active, .nav-underline .show > .nav-link {
  color: var(--nav-underline-link-active-color);
  border-bottom-color: currentColor;
  font-weight: 700;
}

.nav-fill > .nav-link, .nav-fill .nav-item {
  text-align: center;
  flex: auto;
}

.nav-justified > .nav-link, .nav-justified .nav-item {
  text-align: center;
  flex-grow: 1;
  flex-basis: 0;
}

.nav-fill .nav-item .nav-link, .nav-justified .nav-item .nav-link {
  width: 100%;
}

.tab-content > .tab-pane {
  display: none;
}

.tab-content > .active {
  display: block;
}

.navbar {
  --navbar-padding-x: 0;
  --navbar-padding-y: .5rem;
  --navbar-color: rgba(var(--emphasis-color-rgb), .65);
  --navbar-hover-color: rgba(var(--emphasis-color-rgb), .8);
  --navbar-disabled-color: rgba(var(--emphasis-color-rgb), .3);
  --navbar-active-color: rgba(var(--emphasis-color-rgb), 1);
  --navbar-brand-padding-y: .3125rem;
  --navbar-brand-margin-end: 1rem;
  --navbar-brand-font-size: 1.25rem;
  --navbar-brand-color: rgba(var(--emphasis-color-rgb), 1);
  --navbar-brand-hover-color: rgba(var(--emphasis-color-rgb), 1);
  --navbar-nav-link-padding-x: .5rem;
  --navbar-toggler-padding-y: .25rem;
  --navbar-toggler-padding-x: .75rem;
  --navbar-toggler-font-size: 1.25rem;
  --navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%2833, 37, 41, 0.75%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
  --navbar-toggler-border-color: rgba(var(--emphasis-color-rgb), .15);
  --navbar-toggler-border-radius: .25rem;
  --navbar-toggler-focus-width: .2rem;
  --navbar-toggler-transition: box-shadow .15s ease-in-out;
  padding: var(--navbar-padding-y) var(--navbar-padding-x);
  flex-wrap: wrap;
  justify-content: space-between;
  align-items: center;
  display: flex;
  position: relative;
}

.navbar > .container, .navbar > .container-fluid, .navbar > .container-sm, .navbar > .container-md, .navbar > .container-lg, .navbar > .container-xl, .navbar > .container-xxl {
  flex-wrap: inherit;
  justify-content: space-between;
  align-items: center;
  display: flex;
}

.navbar-brand {
  padding-top: var(--navbar-brand-padding-y);
  padding-bottom: var(--navbar-brand-padding-y);
  margin-right: var(--navbar-brand-margin-end);
  font-size: var(--navbar-brand-font-size);
  color: var(--navbar-brand-color);
  white-space: nowrap;
}

.navbar-brand:hover, .navbar-brand:focus {
  color: var(--navbar-brand-hover-color);
}

.navbar-nav {
  --nav-link-padding-x: 0;
  --nav-link-padding-y: .5rem;
  --nav-link-font-weight: ;
  --nav-link-color: var(--navbar-color);
  --nav-link-hover-color: var(--navbar-hover-color);
  --nav-link-disabled-color: var(--navbar-disabled-color);
  flex-direction: column;
  margin-bottom: 0;
  padding-left: 0;
  list-style: none;
  display: flex;
}

.navbar-nav .nav-link.active, .navbar-nav .nav-link.show {
  color: var(--navbar-active-color);
}

.navbar-nav .dropdown-menu {
  position: static;
}

.navbar-text {
  color: var(--navbar-color);
  padding-top: .5rem;
  padding-bottom: .5rem;
}

.navbar-text a, .navbar-text a:hover, .navbar-text a:focus {
  color: var(--navbar-active-color);
}

.navbar-collapse {
  flex-grow: 1;
  flex-basis: 100%;
  align-items: center;
}

.navbar-toggler {
  padding: var(--navbar-toggler-padding-y) var(--navbar-toggler-padding-x);
  font-size: var(--navbar-toggler-font-size);
  color: var(--navbar-color);
  border: var(--border-width) solid var(--navbar-toggler-border-color);
  border-radius: var(--navbar-toggler-border-radius);
  transition: var(--navbar-toggler-transition);
  background-color: #0000;
  line-height: 1;
}

@media (prefers-reduced-motion: reduce) {
  .navbar-toggler {
    transition: none;
  }
}

.navbar-toggler:hover {
  text-decoration: none;
}

.navbar-toggler:focus {
  box-shadow: 0 0 0 var(--navbar-toggler-focus-width);
  outline: 0;
  text-decoration: none;
}

.navbar-toggler-icon {
  vertical-align: middle;
  background-image: var(--navbar-toggler-icon-bg);
  background-position: center;
  background-repeat: no-repeat;
  background-size: 100%;
  width: 1.5em;
  height: 1.5em;
  display: inline-block;
}

.navbar-nav-scroll {
  max-height: var(--scroll-height, 75vh);
  overflow-y: auto;
}

@media (width >= 576px) {
  .navbar-expand-sm {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }

  .navbar-expand-sm .navbar-nav {
    flex-direction: row;
  }

  .navbar-expand-sm .navbar-nav .dropdown-menu {
    position: absolute;
  }

  .navbar-expand-sm .navbar-nav .nav-link {
    padding-right: var(--navbar-nav-link-padding-x);
    padding-left: var(--navbar-nav-link-padding-x);
  }

  .navbar-expand-sm .navbar-nav-scroll {
    overflow: visible;
  }

  .navbar-expand-sm .navbar-collapse {
    flex-basis: auto;
    display: flex !important;
  }

  .navbar-expand-sm .navbar-toggler {
    display: none;
  }

  .navbar-expand-sm .offcanvas {
    z-index: auto;
    flex-grow: 1;
    transition: none;
    position: static;
    visibility: visible !important;
    background-color: #0000 !important;
    border: 0 !important;
    width: auto !important;
    height: auto !important;
    transform: none !important;
  }

  .navbar-expand-sm .offcanvas .offcanvas-header {
    display: none;
  }

  .navbar-expand-sm .offcanvas .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
  }
}

@media (width >= 768px) {
  .navbar-expand-md {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }

  .navbar-expand-md .navbar-nav {
    flex-direction: row;
  }

  .navbar-expand-md .navbar-nav .dropdown-menu {
    position: absolute;
  }

  .navbar-expand-md .navbar-nav .nav-link {
    padding-right: var(--navbar-nav-link-padding-x);
    padding-left: var(--navbar-nav-link-padding-x);
  }

  .navbar-expand-md .navbar-nav-scroll {
    overflow: visible;
  }

  .navbar-expand-md .navbar-collapse {
    flex-basis: auto;
    display: flex !important;
  }

  .navbar-expand-md .navbar-toggler {
    display: none;
  }

  .navbar-expand-md .offcanvas {
    z-index: auto;
    flex-grow: 1;
    transition: none;
    position: static;
    visibility: visible !important;
    background-color: #0000 !important;
    border: 0 !important;
    width: auto !important;
    height: auto !important;
    transform: none !important;
  }

  .navbar-expand-md .offcanvas .offcanvas-header {
    display: none;
  }

  .navbar-expand-md .offcanvas .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
  }
}

@media (width >= 992px) {
  .navbar-expand-lg {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }

  .navbar-expand-lg .navbar-nav {
    flex-direction: row;
  }

  .navbar-expand-lg .navbar-nav .dropdown-menu {
    position: absolute;
  }

  .navbar-expand-lg .navbar-nav .nav-link {
    padding-right: var(--navbar-nav-link-padding-x);
    padding-left: var(--navbar-nav-link-padding-x);
  }

  .navbar-expand-lg .navbar-nav-scroll {
    overflow: visible;
  }

  .navbar-expand-lg .navbar-collapse {
    flex-basis: auto;
    display: flex !important;
  }

  .navbar-expand-lg .navbar-toggler {
    display: none;
  }

  .navbar-expand-lg .offcanvas {
    z-index: auto;
    flex-grow: 1;
    transition: none;
    position: static;
    visibility: visible !important;
    background-color: #0000 !important;
    border: 0 !important;
    width: auto !important;
    height: auto !important;
    transform: none !important;
  }

  .navbar-expand-lg .offcanvas .offcanvas-header {
    display: none;
  }

  .navbar-expand-lg .offcanvas .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
  }
}

@media (width >= 1200px) {
  .navbar-expand-xl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }

  .navbar-expand-xl .navbar-nav {
    flex-direction: row;
  }

  .navbar-expand-xl .navbar-nav .dropdown-menu {
    position: absolute;
  }

  .navbar-expand-xl .navbar-nav .nav-link {
    padding-right: var(--navbar-nav-link-padding-x);
    padding-left: var(--navbar-nav-link-padding-x);
  }

  .navbar-expand-xl .navbar-nav-scroll {
    overflow: visible;
  }

  .navbar-expand-xl .navbar-collapse {
    flex-basis: auto;
    display: flex !important;
  }

  .navbar-expand-xl .navbar-toggler {
    display: none;
  }

  .navbar-expand-xl .offcanvas {
    z-index: auto;
    flex-grow: 1;
    transition: none;
    position: static;
    visibility: visible !important;
    background-color: #0000 !important;
    border: 0 !important;
    width: auto !important;
    height: auto !important;
    transform: none !important;
  }

  .navbar-expand-xl .offcanvas .offcanvas-header {
    display: none;
  }

  .navbar-expand-xl .offcanvas .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
  }
}

@media (width >= 1400px) {
  .navbar-expand-xxl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }

  .navbar-expand-xxl .navbar-nav {
    flex-direction: row;
  }

  .navbar-expand-xxl .navbar-nav .dropdown-menu {
    position: absolute;
  }

  .navbar-expand-xxl .navbar-nav .nav-link {
    padding-right: var(--navbar-nav-link-padding-x);
    padding-left: var(--navbar-nav-link-padding-x);
  }

  .navbar-expand-xxl .navbar-nav-scroll {
    overflow: visible;
  }

  .navbar-expand-xxl .navbar-collapse {
    flex-basis: auto;
    display: flex !important;
  }

  .navbar-expand-xxl .navbar-toggler {
    display: none;
  }

  .navbar-expand-xxl .offcanvas {
    z-index: auto;
    flex-grow: 1;
    transition: none;
    position: static;
    visibility: visible !important;
    background-color: #0000 !important;
    border: 0 !important;
    width: auto !important;
    height: auto !important;
    transform: none !important;
  }

  .navbar-expand-xxl .offcanvas .offcanvas-header {
    display: none;
  }

  .navbar-expand-xxl .offcanvas .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
  }
}

.navbar-expand {
  flex-wrap: nowrap;
  justify-content: flex-start;
}

.navbar-expand .navbar-nav {
  flex-direction: row;
}

.navbar-expand .navbar-nav .dropdown-menu {
  position: absolute;
}

.navbar-expand .navbar-nav .nav-link {
  padding-right: var(--navbar-nav-link-padding-x);
  padding-left: var(--navbar-nav-link-padding-x);
}

.navbar-expand .navbar-nav-scroll {
  overflow: visible;
}

.navbar-expand .navbar-collapse {
  flex-basis: auto;
  display: flex !important;
}

.navbar-expand .navbar-toggler {
  display: none;
}

.navbar-expand .offcanvas {
  z-index: auto;
  flex-grow: 1;
  transition: none;
  position: static;
  visibility: visible !important;
  background-color: #0000 !important;
  border: 0 !important;
  width: auto !important;
  height: auto !important;
  transform: none !important;
}

.navbar-expand .offcanvas .offcanvas-header {
  display: none;
}

.navbar-expand .offcanvas .offcanvas-body {
  flex-grow: 0;
  padding: 0;
  display: flex;
  overflow-y: visible;
}

.navbar-dark, .navbar[data-bs-theme="dark"] {
  --navbar-color: #ffffff8c;
  --navbar-hover-color: #ffffffbf;
  --navbar-disabled-color: #ffffff40;
  --navbar-active-color: #fff;
  --navbar-brand-color: #fff;
  --navbar-brand-hover-color: #fff;
  --navbar-toggler-border-color: #ffffff1a;
  --navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%28255, 255, 255, 0.55%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}

[data-bs-theme="dark"] .navbar-toggler-icon {
  --navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%28255, 255, 255, 0.55%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}

.card {
  --card-spacer-y: 1rem;
  --card-spacer-x: 1rem;
  --card-title-spacer-y: .5rem;
  --card-title-color: ;
  --card-subtitle-color: ;
  --card-border-width: 0;
  --card-border-color: transparent;
  --card-border-radius: var(--border-radius);
  --card-box-shadow: ;
  --card-inner-border-radius: calc(var(--border-radius)  - 0);
  --card-cap-padding-y: .5rem;
  --card-cap-padding-x: 1rem;
  --card-cap-bg: rgba(var(--body-color-rgb), .03);
  --card-cap-color: ;
  --card-height: ;
  --card-color: ;
  --card-bg: var(--card-bg);
  --card-img-overlay-padding: 1rem;
  --card-group-margin: 1rem;
  min-width: 0;
  height: var(--card-height);
  color: var(--body-color);
  word-wrap: break-word;
  background-color: var(--card-bg);
  border: var(--card-border-width) solid var(--card-border-color);
  border-radius: var(--card-border-radius);
  background-clip: border-box;
  flex-direction: column;
  display: flex;
  position: relative;
}

.card > hr {
  margin-left: 0;
  margin-right: 0;
}

.card > .list-group {
  border-top: inherit;
  border-bottom: inherit;
}

.card > .list-group:first-child {
  border-top-left-radius: var(--card-inner-border-radius);
  border-top-right-radius: var(--card-inner-border-radius);
  border-top-width: 0;
}

.card > .list-group:last-child {
  border-bottom-right-radius: var(--card-inner-border-radius);
  border-bottom-left-radius: var(--card-inner-border-radius);
  border-bottom-width: 0;
}

.card > .card-header + .list-group, .card > .list-group + .card-footer {
  border-top: 0;
}

.card-body {
  padding: var(--card-spacer-y) var(--card-spacer-x);
  color: var(--card-color);
  flex: auto;
}

.card-title {
  margin-bottom: var(--card-title-spacer-y);
  color: var(--card-title-color);
}

.card-subtitle {
  margin-top: calc(-.5 * var(--card-title-spacer-y));
  color: var(--card-subtitle-color);
  margin-bottom: 0;
}

.card-text:last-child {
  margin-bottom: 0;
}

.card-link + .card-link {
  margin-left: var(--card-spacer-x);
}

.card-header {
  padding: var(--card-cap-padding-y) var(--card-cap-padding-x);
  color: var(--card-cap-color);
  background-color: var(--card-cap-bg);
  border-bottom: var(--card-border-width) solid var(--card-border-color);
  margin-bottom: 0;
}

.card-header:first-child {
  border-radius: var(--card-inner-border-radius) var(--card-inner-border-radius) 0 0;
}

.card-footer {
  padding: var(--card-cap-padding-y) var(--card-cap-padding-x);
  color: var(--card-cap-color);
  background-color: var(--card-cap-bg);
  border-top: var(--card-border-width) solid var(--card-border-color);
}

.card-footer:last-child {
  border-radius: 0 0 var(--card-inner-border-radius) var(--card-inner-border-radius);
}

.card-header-tabs {
  margin-right: calc(-.5 * var(--card-cap-padding-x));
  margin-bottom: calc(-1 * var(--card-cap-padding-y));
  margin-left: calc(-.5 * var(--card-cap-padding-x));
  border-bottom: 0;
}

.card-header-tabs .nav-link.active {
  background-color: var(--card-bg);
  border-bottom-color: var(--card-bg);
}

.card-header-pills {
  margin-right: calc(-.5 * var(--card-cap-padding-x));
  margin-left: calc(-.5 * var(--card-cap-padding-x));
}

.card-img-overlay {
  padding: var(--card-img-overlay-padding);
  border-radius: var(--card-inner-border-radius);
  position: absolute;
  inset: 0;
}

.card-img, .card-img-top, .card-img-bottom {
  width: 100%;
}

.card-img, .card-img-top {
  border-top-left-radius: var(--card-inner-border-radius);
  border-top-right-radius: var(--card-inner-border-radius);
}

.card-img, .card-img-bottom {
  border-bottom-right-radius: var(--card-inner-border-radius);
  border-bottom-left-radius: var(--card-inner-border-radius);
}

.card-group > .card {
  margin-bottom: var(--card-group-margin);
}

@media (width >= 576px) {
  .card-group {
    flex-flow: wrap;
    display: flex;
  }

  .card-group > .card {
    flex: 1 0 0;
    margin-bottom: 0;
  }

  .card-group > .card + .card {
    border-left: 0;
    margin-left: 0;
  }

  .card-group > .card:not(:last-child) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
  }

  .card-group > .card:not(:last-child) > .card-img-top, .card-group > .card:not(:last-child) > .card-header {
    border-top-right-radius: 0;
  }

  .card-group > .card:not(:last-child) > .card-img-bottom, .card-group > .card:not(:last-child) > .card-footer {
    border-bottom-right-radius: 0;
  }

  .card-group > .card:not(:first-child) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
  }

  .card-group > .card:not(:first-child) > .card-img-top, .card-group > .card:not(:first-child) > .card-header {
    border-top-left-radius: 0;
  }

  .card-group > .card:not(:first-child) > .card-img-bottom, .card-group > .card:not(:first-child) > .card-footer {
    border-bottom-left-radius: 0;
  }
}

.accordion {
  --accordion-color: var(--body-color);
  --accordion-bg: var(--body-bg);
  --accordion-transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out, border-radius .15s ease;
  --accordion-border-color: var(--border-color);
  --accordion-border-width: var(--border-width);
  --accordion-border-radius: var(--border-radius);
  --accordion-inner-border-radius: calc(var(--border-radius)  - (var(--border-width)));
  --accordion-btn-padding-x: 1.25rem;
  --accordion-btn-padding-y: 1rem;
  --accordion-btn-color: var(--body-color);
  --accordion-btn-bg: var(--accordion-bg);
  --accordion-btn-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='none' stroke='%23212529' stroke-linecap='round' stroke-linejoin='round'%3e%3cpath d='m2 5 6 6 6-6'/%3e%3c/svg%3e");
  --accordion-btn-icon-width: 1.25rem;
  --accordion-btn-icon-transform: rotate(-180deg);
  --accordion-btn-icon-transition: transform .2s ease-in-out;
  --accordion-btn-active-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='none' stroke='rgb%2816.8, 42, 73.2%29' stroke-linecap='round' stroke-linejoin='round'%3e%3cpath d='m2 5 6 6 6-6'/%3e%3c/svg%3e");
  --accordion-btn-focus-box-shadow: 0 0 0 .25rem #2a69b740;
  --accordion-body-padding-x: 1.25rem;
  --accordion-body-padding-y: 1rem;
  --accordion-active-color: var(--primary-text-emphasis);
  --accordion-active-bg: var(--primary-bg-subtle);
}

.accordion-button {
  width: 100%;
  padding: var(--accordion-btn-padding-y) var(--accordion-btn-padding-x);
  color: var(--accordion-btn-color);
  text-align: left;
  background-color: var(--accordion-btn-bg);
  overflow-anchor: none;
  transition: var(--accordion-transition);
  border: 0;
  border-radius: 0;
  align-items: center;
  font-size: 1rem;
  display: flex;
  position: relative;
}

@media (prefers-reduced-motion: reduce) {
  .accordion-button {
    transition: none;
  }
}

.accordion-button:not(.collapsed) {
  color: var(--accordion-active-color);
  background-color: var(--accordion-active-bg);
  box-shadow: inset 0 calc(-1 * var(--accordion-border-width)) 0 var(--accordion-border-color);
}

.accordion-button:not(.collapsed):after {
  background-image: var(--accordion-btn-active-icon);
  transform: var(--accordion-btn-icon-transform);
}

.accordion-button:after {
  width: var(--accordion-btn-icon-width);
  height: var(--accordion-btn-icon-width);
  content: "";
  background-image: var(--accordion-btn-icon);
  background-repeat: no-repeat;
  background-size: var(--accordion-btn-icon-width);
  transition: var(--accordion-btn-icon-transition);
  flex-shrink: 0;
  margin-left: auto;
}

@media (prefers-reduced-motion: reduce) {
  .accordion-button:after {
    transition: none;
  }
}

.accordion-button:hover {
  z-index: 2;
}

.accordion-button:focus {
  z-index: 3;
  box-shadow: var(--accordion-btn-focus-box-shadow);
  outline: 0;
}

.accordion-header {
  margin-bottom: 0;
}

.accordion-item {
  color: var(--accordion-color);
  background-color: var(--accordion-bg);
  border: var(--accordion-border-width) solid var(--accordion-border-color);
}

.accordion-item:first-of-type {
  border-top-left-radius: var(--accordion-border-radius);
  border-top-right-radius: var(--accordion-border-radius);
}

.accordion-item:first-of-type > .accordion-header .accordion-button {
  border-top-left-radius: var(--accordion-inner-border-radius);
  border-top-right-radius: var(--accordion-inner-border-radius);
}

.accordion-item:not(:first-of-type) {
  border-top: 0;
}

.accordion-item:last-of-type {
  border-bottom-right-radius: var(--accordion-border-radius);
  border-bottom-left-radius: var(--accordion-border-radius);
}

.accordion-item:last-of-type > .accordion-header .accordion-button.collapsed {
  border-bottom-right-radius: var(--accordion-inner-border-radius);
  border-bottom-left-radius: var(--accordion-inner-border-radius);
}

.accordion-item:last-of-type > .accordion-collapse {
  border-bottom-right-radius: var(--accordion-border-radius);
  border-bottom-left-radius: var(--accordion-border-radius);
}

.accordion-body {
  padding: var(--accordion-body-padding-y) var(--accordion-body-padding-x);
}

.accordion-flush > .accordion-item {
  border-left: 0;
  border-right: 0;
  border-radius: 0;
}

.accordion-flush > .accordion-item:first-child {
  border-top: 0;
}

.accordion-flush > .accordion-item:last-child {
  border-bottom: 0;
}

.accordion-flush > .accordion-item > .accordion-collapse, .accordion-flush > .accordion-item > .accordion-header .accordion-button, .accordion-flush > .accordion-item > .accordion-header .accordion-button.collapsed {
  border-radius: 0;
}

[data-bs-theme="dark"] .accordion-button:after {
  --accordion-btn-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='rgb%28127.2, 165, 211.8%29'%3e%3cpath fill-rule='evenodd' d='M1.646 4.646a.5.5 0 0 1 .708 0L8 10.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708'/%3e%3c/svg%3e");
  --accordion-btn-active-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='rgb%28127.2, 165, 211.8%29'%3e%3cpath fill-rule='evenodd' d='M1.646 4.646a.5.5 0 0 1 .708 0L8 10.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708'/%3e%3c/svg%3e");
}

.breadcrumb {
  --breadcrumb-padding-x: 0;
  --breadcrumb-padding-y: 0;
  --breadcrumb-margin-bottom: 1rem;
  --breadcrumb-bg: var(--white);
  --breadcrumb-border-radius: ;
  --breadcrumb-divider-color: var(--secondary-color);
  --breadcrumb-item-padding-x: .5rem;
  --breadcrumb-item-active-color: var(--secondary-color);
  padding: var(--breadcrumb-padding-y) var(--breadcrumb-padding-x);
  margin-bottom: var(--breadcrumb-margin-bottom);
  font-size: var(--breadcrumb-font-size);
  background-color: var(--breadcrumb-bg);
  border-radius: var(--breadcrumb-border-radius);
  flex-wrap: wrap;
  list-style: none;
  display: flex;
}

.breadcrumb-item + .breadcrumb-item {
  padding-left: var(--breadcrumb-item-padding-x);
}

.breadcrumb-item + .breadcrumb-item:before {
  float: left;
  padding-right: var(--breadcrumb-item-padding-x);
  color: var(--breadcrumb-divider-color);
  content: var(--breadcrumb-divider, "/");
}

.breadcrumb-item.active {
  color: var(--breadcrumb-item-active-color);
}

.pagination {
  --pagination-padding-x: .75rem;
  --pagination-padding-y: .375rem;
  --pagination-font-size: 1rem;
  --pagination-color: var(--link-color);
  --pagination-bg: var(--body-bg);
  --pagination-border-width: var(--border-width);
  --pagination-border-color: var(--border-color);
  --pagination-border-radius: var(--border-radius);
  --pagination-hover-color: var(--link-hover-color);
  --pagination-hover-bg: var(--tertiary-bg);
  --pagination-hover-border-color: var(--border-color);
  --pagination-focus-color: var(--link-hover-color);
  --pagination-focus-bg: var(--secondary-bg);
  --pagination-focus-box-shadow: 0 0 0 .25rem #2a69b740;
  --pagination-active-color: #fff;
  --pagination-active-bg: #2a69b7;
  --pagination-active-border-color: #2a69b7;
  --pagination-disabled-color: var(--secondary-color);
  --pagination-disabled-bg: var(--secondary-bg);
  --pagination-disabled-border-color: var(--border-color);
  padding-left: 0;
  list-style: none;
  display: flex;
}

.page-link {
  padding: var(--pagination-padding-y) var(--pagination-padding-x);
  font-size: var(--pagination-font-size);
  color: var(--pagination-color);
  background-color: var(--pagination-bg);
  border: var(--pagination-border-width) solid var(--pagination-border-color);
  transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
  display: block;
  position: relative;
}

@media (prefers-reduced-motion: reduce) {
  .page-link {
    transition: none;
  }
}

.page-link:hover {
  z-index: 2;
  color: var(--pagination-hover-color);
  background-color: var(--pagination-hover-bg);
  border-color: var(--pagination-hover-border-color);
}

.page-link:focus {
  z-index: 3;
  color: var(--pagination-focus-color);
  background-color: var(--pagination-focus-bg);
  box-shadow: var(--pagination-focus-box-shadow);
  outline: 0;
}

.page-link.active, .active > .page-link {
  z-index: 3;
  color: var(--pagination-active-color);
  background-color: var(--pagination-active-bg);
  border-color: var(--pagination-active-border-color);
}

.page-link.disabled, .disabled > .page-link {
  color: var(--pagination-disabled-color);
  pointer-events: none;
  background-color: var(--pagination-disabled-bg);
  border-color: var(--pagination-disabled-border-color);
}

.page-item:not(:first-child) .page-link {
  margin-left: calc(-1 * var(--border-width));
}

.page-item:first-child .page-link {
  border-top-left-radius: var(--pagination-border-radius);
  border-bottom-left-radius: var(--pagination-border-radius);
}

.page-item:last-child .page-link {
  border-top-right-radius: var(--pagination-border-radius);
  border-bottom-right-radius: var(--pagination-border-radius);
}

.pagination-lg {
  --pagination-padding-x: 1.5rem;
  --pagination-padding-y: .75rem;
  --pagination-font-size: 1.25rem;
  --pagination-border-radius: var(--border-radius-lg);
}

.pagination-sm {
  --pagination-padding-x: .5rem;
  --pagination-padding-y: .25rem;
  --pagination-font-size: .8rem;
  --pagination-border-radius: var(--border-radius-sm);
}

.badge {
  --badge-padding-x: .2rem;
  --badge-padding-y: .3rem;
  --badge-font-size: .75rem;
  --badge-font-weight: 700;
  --badge-color: #fff;
  --badge-border-radius: .2rem;
  padding: var(--badge-padding-y) var(--badge-padding-x);
  font-size: var(--badge-font-size);
  font-weight: var(--badge-font-weight);
  color: var(--badge-color);
  text-align: center;
  white-space: nowrap;
  vertical-align: baseline;
  border-radius: var(--badge-border-radius);
  line-height: 1;
  display: inline-block;
}

.badge:empty {
  display: none;
}

.btn .badge {
  position: relative;
  top: -1px;
}

.alert {
  --alert-bg: transparent;
  --alert-padding-x: 1rem;
  --alert-padding-y: 1rem;
  --alert-margin-bottom: 1rem;
  --alert-color: inherit;
  --alert-border-color: transparent;
  --alert-border: var(--border-width) solid var(--alert-border-color);
  --alert-border-radius: var(--border-radius);
  --alert-link-color: inherit;
  padding: var(--alert-padding-y) var(--alert-padding-x);
  margin-bottom: var(--alert-margin-bottom);
  color: var(--alert-color);
  background-color: var(--alert-bg);
  border: var(--alert-border);
  border-radius: var(--alert-border-radius);
  position: relative;
}

.alert-heading {
  color: inherit;
}

.alert-link {
  color: var(--alert-link-color);
  font-weight: 700;
}

.alert-dismissible {
  padding-right: 3rem;
}

.alert-dismissible .btn-close {
  z-index: 2;
  padding: 1.25rem 1rem;
  position: absolute;
  top: 0;
  right: 0;
}

.alert-primary {
  --alert-color: var(--primary-text-emphasis);
  --alert-bg: var(--primary-bg-subtle);
  --alert-border-color: var(--primary-border-subtle);
  --alert-link-color: var(--primary-text-emphasis);
}

.alert-secondary {
  --alert-color: var(--secondary-text-emphasis);
  --alert-bg: var(--secondary-bg-subtle);
  --alert-border-color: var(--secondary-border-subtle);
  --alert-link-color: var(--secondary-text-emphasis);
}

.alert-success {
  --alert-color: var(--success-text-emphasis);
  --alert-bg: var(--success-bg-subtle);
  --alert-border-color: var(--success-border-subtle);
  --alert-link-color: var(--success-text-emphasis);
}

.alert-info {
  --alert-color: var(--info-text-emphasis);
  --alert-bg: var(--info-bg-subtle);
  --alert-border-color: var(--info-border-subtle);
  --alert-link-color: var(--info-text-emphasis);
}

.alert-warning {
  --alert-color: var(--warning-text-emphasis);
  --alert-bg: var(--warning-bg-subtle);
  --alert-border-color: var(--warning-border-subtle);
  --alert-link-color: var(--warning-text-emphasis);
}

.alert-danger {
  --alert-color: var(--danger-text-emphasis);
  --alert-bg: var(--danger-bg-subtle);
  --alert-border-color: var(--danger-border-subtle);
  --alert-link-color: var(--danger-text-emphasis);
}

.alert-light {
  --alert-color: var(--light-text-emphasis);
  --alert-bg: var(--light-bg-subtle);
  --alert-border-color: var(--light-border-subtle);
  --alert-link-color: var(--light-text-emphasis);
}

.alert-dark {
  --alert-color: var(--dark-text-emphasis);
  --alert-bg: var(--dark-bg-subtle);
  --alert-border-color: var(--dark-border-subtle);
  --alert-link-color: var(--dark-text-emphasis);
}

.alert-action {
  --alert-color: var(--action-text-emphasis);
  --alert-bg: var(--action-bg-subtle);
  --alert-border-color: var(--action-border-subtle);
  --alert-link-color: var(--action-text-emphasis);
}

.alert-error {
  --alert-color: var(--error-text-emphasis);
  --alert-bg: var(--error-bg-subtle);
  --alert-border-color: var(--error-border-subtle);
  --alert-link-color: var(--error-text-emphasis);
}

@keyframes progress-bar-stripes {
  0% {
    background-position-x: var(--progress-height);
  }
}

.progress, .progress-stacked {
  --progress-height: 1rem;
  --progress-font-size: .75rem;
  --progress-bg: var(--secondary-bg);
  --progress-border-radius: var(--border-radius);
  --progress-box-shadow: var(--box-shadow-inset);
  --progress-bar-color: #fff;
  --progress-bar-bg: #2a69b7;
  --progress-bar-transition: width .6s ease;
  height: var(--progress-height);
  font-size: var(--progress-font-size);
  background-color: var(--progress-bg);
  border-radius: var(--progress-border-radius);
  display: flex;
  overflow: hidden;
}

.progress-bar {
  color: var(--progress-bar-color);
  text-align: center;
  white-space: nowrap;
  background-color: var(--progress-bar-bg);
  transition: var(--progress-bar-transition);
  flex-direction: column;
  justify-content: center;
  display: flex;
  overflow: hidden;
}

@media (prefers-reduced-motion: reduce) {
  .progress-bar {
    transition: none;
  }
}

.progress-bar-striped {
  background-image: linear-gradient(45deg, #ffffff26 25%, #0000 25% 50%, #ffffff26 50% 75%, #0000 75%, #0000);
  background-size: var(--progress-height) var(--progress-height);
}

.progress-stacked > .progress {
  overflow: visible;
}

.progress-stacked > .progress > .progress-bar {
  width: 100%;
}

.progress-bar-animated {
  animation: 1s linear infinite progress-bar-stripes;
}

@media (prefers-reduced-motion: reduce) {
  .progress-bar-animated {
    animation: none;
  }
}

.list-group {
  --list-group-color: var(--body-color);
  --list-group-bg: var(--body-bg);
  --list-group-border-color: var(--border-color);
  --list-group-border-width: var(--border-width);
  --list-group-border-radius: var(--border-radius);
  --list-group-item-padding-x: 1rem;
  --list-group-item-padding-y: .75rem;
  --list-group-action-color: var(--secondary-color);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--tertiary-bg);
  --list-group-action-active-color: var(--body-color);
  --list-group-action-active-bg: var(--secondary-bg);
  --list-group-disabled-color: var(--secondary-color);
  --list-group-disabled-bg: var(--body-bg);
  --list-group-active-color: #fff;
  --list-group-active-bg: #2a69b7;
  --list-group-active-border-color: #2a69b7;
  border-radius: var(--list-group-border-radius);
  flex-direction: column;
  margin-bottom: 0;
  padding-left: 0;
  display: flex;
}

.list-group-numbered {
  counter-reset: section;
  list-style-type: none;
}

.list-group-numbered > .list-group-item:before {
  content: counters(section, ".") ". ";
  counter-increment: section;
}

.list-group-item {
  padding: var(--list-group-item-padding-y) var(--list-group-item-padding-x);
  color: var(--list-group-color);
  background-color: var(--list-group-bg);
  border: var(--list-group-border-width) solid var(--list-group-border-color);
  display: block;
  position: relative;
}

.list-group-item:first-child {
  border-top-left-radius: inherit;
  border-top-right-radius: inherit;
}

.list-group-item:last-child {
  border-bottom-right-radius: inherit;
  border-bottom-left-radius: inherit;
}

.list-group-item.disabled, .list-group-item:disabled {
  color: var(--list-group-disabled-color);
  pointer-events: none;
  background-color: var(--list-group-disabled-bg);
}

.list-group-item.active {
  z-index: 2;
  color: var(--list-group-active-color);
  background-color: var(--list-group-active-bg);
  border-color: var(--list-group-active-border-color);
}

.list-group-item + .list-group-item {
  border-top-width: 0;
}

.list-group-item + .list-group-item.active {
  margin-top: calc(-1 * var(--list-group-border-width));
  border-top-width: var(--list-group-border-width);
}

.list-group-item-action {
  width: 100%;
  color: var(--list-group-action-color);
  text-align: inherit;
}

.list-group-item-action:not(.active):hover, .list-group-item-action:not(.active):focus {
  z-index: 1;
  color: var(--list-group-action-hover-color);
  background-color: var(--list-group-action-hover-bg);
  text-decoration: none;
}

.list-group-item-action:not(.active):active {
  color: var(--list-group-action-active-color);
  background-color: var(--list-group-action-active-bg);
}

.list-group-horizontal {
  flex-direction: row;
}

.list-group-horizontal > .list-group-item:first-child:not(:last-child) {
  border-bottom-left-radius: var(--list-group-border-radius);
  border-top-right-radius: 0;
}

.list-group-horizontal > .list-group-item:last-child:not(:first-child) {
  border-top-right-radius: var(--list-group-border-radius);
  border-bottom-left-radius: 0;
}

.list-group-horizontal > .list-group-item.active {
  margin-top: 0;
}

.list-group-horizontal > .list-group-item + .list-group-item {
  border-top-width: var(--list-group-border-width);
  border-left-width: 0;
}

.list-group-horizontal > .list-group-item + .list-group-item.active {
  margin-left: calc(-1 * var(--list-group-border-width));
  border-left-width: var(--list-group-border-width);
}

@media (width >= 576px) {
  .list-group-horizontal-sm {
    flex-direction: row;
  }

  .list-group-horizontal-sm > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--list-group-border-radius);
    border-top-right-radius: 0;
  }

  .list-group-horizontal-sm > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--list-group-border-radius);
    border-bottom-left-radius: 0;
  }

  .list-group-horizontal-sm > .list-group-item.active {
    margin-top: 0;
  }

  .list-group-horizontal-sm > .list-group-item + .list-group-item {
    border-top-width: var(--list-group-border-width);
    border-left-width: 0;
  }

  .list-group-horizontal-sm > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--list-group-border-width));
    border-left-width: var(--list-group-border-width);
  }
}

@media (width >= 768px) {
  .list-group-horizontal-md {
    flex-direction: row;
  }

  .list-group-horizontal-md > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--list-group-border-radius);
    border-top-right-radius: 0;
  }

  .list-group-horizontal-md > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--list-group-border-radius);
    border-bottom-left-radius: 0;
  }

  .list-group-horizontal-md > .list-group-item.active {
    margin-top: 0;
  }

  .list-group-horizontal-md > .list-group-item + .list-group-item {
    border-top-width: var(--list-group-border-width);
    border-left-width: 0;
  }

  .list-group-horizontal-md > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--list-group-border-width));
    border-left-width: var(--list-group-border-width);
  }
}

@media (width >= 992px) {
  .list-group-horizontal-lg {
    flex-direction: row;
  }

  .list-group-horizontal-lg > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--list-group-border-radius);
    border-top-right-radius: 0;
  }

  .list-group-horizontal-lg > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--list-group-border-radius);
    border-bottom-left-radius: 0;
  }

  .list-group-horizontal-lg > .list-group-item.active {
    margin-top: 0;
  }

  .list-group-horizontal-lg > .list-group-item + .list-group-item {
    border-top-width: var(--list-group-border-width);
    border-left-width: 0;
  }

  .list-group-horizontal-lg > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--list-group-border-width));
    border-left-width: var(--list-group-border-width);
  }
}

@media (width >= 1200px) {
  .list-group-horizontal-xl {
    flex-direction: row;
  }

  .list-group-horizontal-xl > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--list-group-border-radius);
    border-top-right-radius: 0;
  }

  .list-group-horizontal-xl > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--list-group-border-radius);
    border-bottom-left-radius: 0;
  }

  .list-group-horizontal-xl > .list-group-item.active {
    margin-top: 0;
  }

  .list-group-horizontal-xl > .list-group-item + .list-group-item {
    border-top-width: var(--list-group-border-width);
    border-left-width: 0;
  }

  .list-group-horizontal-xl > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--list-group-border-width));
    border-left-width: var(--list-group-border-width);
  }
}

@media (width >= 1400px) {
  .list-group-horizontal-xxl {
    flex-direction: row;
  }

  .list-group-horizontal-xxl > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--list-group-border-radius);
    border-top-right-radius: 0;
  }

  .list-group-horizontal-xxl > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--list-group-border-radius);
    border-bottom-left-radius: 0;
  }

  .list-group-horizontal-xxl > .list-group-item.active {
    margin-top: 0;
  }

  .list-group-horizontal-xxl > .list-group-item + .list-group-item {
    border-top-width: var(--list-group-border-width);
    border-left-width: 0;
  }

  .list-group-horizontal-xxl > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--list-group-border-width));
    border-left-width: var(--list-group-border-width);
  }
}

.list-group-flush {
  border-radius: 0;
}

.list-group-flush > .list-group-item {
  border-width: 0 0 var(--list-group-border-width);
}

.list-group-flush > .list-group-item:last-child {
  border-bottom-width: 0;
}

.list-group-item-primary {
  --list-group-color: var(--primary-text-emphasis);
  --list-group-bg: var(--primary-bg-subtle);
  --list-group-border-color: var(--primary-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--primary-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--primary-border-subtle);
  --list-group-active-color: var(--primary-bg-subtle);
  --list-group-active-bg: var(--primary-text-emphasis);
  --list-group-active-border-color: var(--primary-text-emphasis);
}

.list-group-item-secondary {
  --list-group-color: var(--secondary-text-emphasis);
  --list-group-bg: var(--secondary-bg-subtle);
  --list-group-border-color: var(--secondary-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--secondary-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--secondary-border-subtle);
  --list-group-active-color: var(--secondary-bg-subtle);
  --list-group-active-bg: var(--secondary-text-emphasis);
  --list-group-active-border-color: var(--secondary-text-emphasis);
}

.list-group-item-success {
  --list-group-color: var(--success-text-emphasis);
  --list-group-bg: var(--success-bg-subtle);
  --list-group-border-color: var(--success-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--success-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--success-border-subtle);
  --list-group-active-color: var(--success-bg-subtle);
  --list-group-active-bg: var(--success-text-emphasis);
  --list-group-active-border-color: var(--success-text-emphasis);
}

.list-group-item-info {
  --list-group-color: var(--info-text-emphasis);
  --list-group-bg: var(--info-bg-subtle);
  --list-group-border-color: var(--info-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--info-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--info-border-subtle);
  --list-group-active-color: var(--info-bg-subtle);
  --list-group-active-bg: var(--info-text-emphasis);
  --list-group-active-border-color: var(--info-text-emphasis);
}

.list-group-item-warning {
  --list-group-color: var(--warning-text-emphasis);
  --list-group-bg: var(--warning-bg-subtle);
  --list-group-border-color: var(--warning-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--warning-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--warning-border-subtle);
  --list-group-active-color: var(--warning-bg-subtle);
  --list-group-active-bg: var(--warning-text-emphasis);
  --list-group-active-border-color: var(--warning-text-emphasis);
}

.list-group-item-danger {
  --list-group-color: var(--danger-text-emphasis);
  --list-group-bg: var(--danger-bg-subtle);
  --list-group-border-color: var(--danger-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--danger-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--danger-border-subtle);
  --list-group-active-color: var(--danger-bg-subtle);
  --list-group-active-bg: var(--danger-text-emphasis);
  --list-group-active-border-color: var(--danger-text-emphasis);
}

.list-group-item-light {
  --list-group-color: var(--light-text-emphasis);
  --list-group-bg: var(--light-bg-subtle);
  --list-group-border-color: var(--light-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--light-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--light-border-subtle);
  --list-group-active-color: var(--light-bg-subtle);
  --list-group-active-bg: var(--light-text-emphasis);
  --list-group-active-border-color: var(--light-text-emphasis);
}

.list-group-item-dark {
  --list-group-color: var(--dark-text-emphasis);
  --list-group-bg: var(--dark-bg-subtle);
  --list-group-border-color: var(--dark-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--dark-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--dark-border-subtle);
  --list-group-active-color: var(--dark-bg-subtle);
  --list-group-active-bg: var(--dark-text-emphasis);
  --list-group-active-border-color: var(--dark-text-emphasis);
}

.list-group-item-action {
  --list-group-color: var(--action-text-emphasis);
  --list-group-bg: var(--action-bg-subtle);
  --list-group-border-color: var(--action-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--action-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--action-border-subtle);
  --list-group-active-color: var(--action-bg-subtle);
  --list-group-active-bg: var(--action-text-emphasis);
  --list-group-active-border-color: var(--action-text-emphasis);
}

.list-group-item-error {
  --list-group-color: var(--error-text-emphasis);
  --list-group-bg: var(--error-bg-subtle);
  --list-group-border-color: var(--error-border-subtle);
  --list-group-action-hover-color: var(--emphasis-color);
  --list-group-action-hover-bg: var(--error-border-subtle);
  --list-group-action-active-color: var(--emphasis-color);
  --list-group-action-active-bg: var(--error-border-subtle);
  --list-group-active-color: var(--error-bg-subtle);
  --list-group-active-bg: var(--error-text-emphasis);
  --list-group-active-border-color: var(--error-text-emphasis);
}

.btn-close {
  --btn-close-color: #000;
  --btn-close-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23000'%3e%3cpath d='M.293.293a1 1 0 0 1 1.414 0L8 6.586 14.293.293a1 1 0 1 1 1.414 1.414L9.414 8l6.293 6.293a1 1 0 0 1-1.414 1.414L8 9.414l-6.293 6.293a1 1 0 0 1-1.414-1.414L6.586 8 .293 1.707a1 1 0 0 1 0-1.414'/%3e%3c/svg%3e");
  --btn-close-opacity: .5;
  --btn-close-hover-opacity: .75;
  --btn-close-focus-shadow: 0 0 0 .25rem #2a69b740;
  --btn-close-focus-opacity: 1;
  --btn-close-disabled-opacity: .25;
  box-sizing: content-box;
  width: 1em;
  height: 1em;
  color: var(--btn-close-color);
  background: transparent var(--btn-close-bg) center / 1em auto no-repeat;
  filter: var(--btn-close-filter);
  opacity: var(--btn-close-opacity);
  border: 0;
  border-radius: .25rem;
  padding: .25em;
}

.btn-close:hover {
  color: var(--btn-close-color);
  opacity: var(--btn-close-hover-opacity);
  text-decoration: none;
}

.btn-close:focus {
  box-shadow: var(--btn-close-focus-shadow);
  opacity: var(--btn-close-focus-opacity);
  outline: 0;
}

.btn-close:disabled, .btn-close.disabled {
  pointer-events: none;
  user-select: none;
  opacity: var(--btn-close-disabled-opacity);
}

.btn-close-white {
  --btn-close-filter: invert(1) grayscale(100%) brightness(200%);
}

:root, [data-bs-theme="light"] {
  --btn-close-filter: ;
}

[data-bs-theme="dark"] {
  --btn-close-filter: invert(1) grayscale(100%) brightness(200%);
}

.toast {
  --toast-zindex: 1090;
  --toast-padding-x: .75rem;
  --toast-padding-y: .5rem;
  --toast-spacing: 2rem;
  --toast-max-width: 350px;
  --toast-font-size: .875rem;
  --toast-color: ;
  --toast-bg: rgba(var(--body-bg-rgb), .85);
  --toast-border-width: var(--border-width);
  --toast-border-color: var(--border-color-translucent);
  --toast-border-radius: var(--border-radius);
  --toast-box-shadow: var(--box-shadow);
  --toast-header-color: var(--secondary-color);
  --toast-header-bg: rgba(var(--body-bg-rgb), .85);
  --toast-header-border-color: var(--border-color-translucent);
  width: var(--toast-max-width);
  max-width: 100%;
  font-size: var(--toast-font-size);
  color: var(--toast-color);
  pointer-events: auto;
  background-color: var(--toast-bg);
  border: var(--toast-border-width) solid var(--toast-border-color);
  box-shadow: var(--toast-box-shadow);
  border-radius: var(--toast-border-radius);
  background-clip: padding-box;
}

.toast.showing {
  opacity: 0;
}

.toast:not(.show) {
  display: none;
}

.toast-container {
  --toast-zindex: 1090;
  z-index: var(--toast-zindex);
  pointer-events: none;
  width: max-content;
  max-width: 100%;
  position: absolute;
}

.toast-container > :not(:last-child) {
  margin-bottom: var(--toast-spacing);
}

.toast-header {
  padding: var(--toast-padding-y) var(--toast-padding-x);
  color: var(--toast-header-color);
  background-color: var(--toast-header-bg);
  border-bottom: var(--toast-border-width) solid var(--toast-header-border-color);
  border-top-left-radius: calc(var(--toast-border-radius)  - var(--toast-border-width));
  border-top-right-radius: calc(var(--toast-border-radius)  - var(--toast-border-width));
  background-clip: padding-box;
  align-items: center;
  display: flex;
}

.toast-header .btn-close {
  margin-right: calc(-.5 * var(--toast-padding-x));
  margin-left: var(--toast-padding-x);
}

.toast-body {
  padding: var(--toast-padding-x);
  word-wrap: break-word;
}

.modal {
  --modal-zindex: 1050;
  --modal-width: 500px;
  --modal-padding: 1rem;
  --modal-margin: .5rem;
  --modal-color: var(--body-color);
  --modal-bg: var(--body-bg);
  --modal-border-color: var(--border-color-translucent);
  --modal-border-width: var(--border-width);
  --modal-border-radius: var(--border-radius-lg);
  --modal-box-shadow: var(--box-shadow-sm);
  --modal-inner-border-radius: calc(var(--border-radius-lg)  - (var(--border-width)));
  --modal-header-padding-x: 1rem;
  --modal-header-padding-y: 1rem;
  --modal-header-padding: 1rem 1rem;
  --modal-header-border-color: var(--border-color);
  --modal-header-border-width: var(--border-width);
  --modal-title-line-height: 1.5;
  --modal-footer-gap: .5rem;
  --modal-footer-bg: ;
  --modal-footer-border-color: var(--border-color);
  --modal-footer-border-width: var(--border-width);
  z-index: var(--modal-zindex);
  outline: 0;
  width: 100%;
  height: 100%;
  display: none;
  position: fixed;
  top: 0;
  left: 0;
  overflow: hidden auto;
}

.modal-dialog {
  width: auto;
  margin: var(--modal-margin);
  pointer-events: none;
  position: relative;
}

.modal.fade .modal-dialog {
  transition: transform .3s ease-out;
  transform: translate(0, -50px);
}

@media (prefers-reduced-motion: reduce) {
  .modal.fade .modal-dialog {
    transition: none;
  }
}

.modal.show .modal-dialog {
  transform: none;
}

.modal.modal-static .modal-dialog {
  transform: scale(1.02);
}

.modal-dialog-scrollable {
  height: calc(100% - var(--modal-margin) * 2);
}

.modal-dialog-scrollable .modal-content {
  max-height: 100%;
  overflow: hidden;
}

.modal-dialog-scrollable .modal-body {
  overflow-y: auto;
}

.modal-dialog-centered {
  min-height: calc(100% - var(--modal-margin) * 2);
  align-items: center;
  display: flex;
}

.modal-content {
  width: 100%;
  color: var(--modal-color);
  pointer-events: auto;
  background-color: var(--modal-bg);
  border: var(--modal-border-width) solid var(--modal-border-color);
  border-radius: var(--modal-border-radius);
  background-clip: padding-box;
  outline: 0;
  flex-direction: column;
  display: flex;
  position: relative;
}

.modal-backdrop {
  --backdrop-zindex: 1040;
  --backdrop-bg: #000;
  --backdrop-opacity: .5;
  z-index: var(--backdrop-zindex);
  background-color: var(--backdrop-bg);
  width: 100vw;
  height: 100vh;
  position: fixed;
  top: 0;
  left: 0;
}

.modal-backdrop.fade {
  opacity: 0;
}

.modal-backdrop.show {
  opacity: var(--backdrop-opacity);
}

.modal-header {
  padding: var(--modal-header-padding);
  border-bottom: var(--modal-header-border-width) solid var(--modal-header-border-color);
  border-top-left-radius: var(--modal-inner-border-radius);
  border-top-right-radius: var(--modal-inner-border-radius);
  flex-shrink: 0;
  align-items: center;
  display: flex;
}

.modal-header .btn-close {
  padding: calc(var(--modal-header-padding-y) * .5) calc(var(--modal-header-padding-x) * .5);
  margin-top: calc(-.5 * var(--modal-header-padding-y));
  margin-right: calc(-.5 * var(--modal-header-padding-x));
  margin-bottom: calc(-.5 * var(--modal-header-padding-y));
  margin-left: auto;
}

.modal-title {
  line-height: var(--modal-title-line-height);
  margin-bottom: 0;
}

.modal-body {
  padding: var(--modal-padding);
  flex: auto;
  position: relative;
}

.modal-footer {
  padding: calc(var(--modal-padding)  - var(--modal-footer-gap) * .5);
  background-color: var(--modal-footer-bg);
  border-top: var(--modal-footer-border-width) solid var(--modal-footer-border-color);
  border-bottom-right-radius: var(--modal-inner-border-radius);
  border-bottom-left-radius: var(--modal-inner-border-radius);
  flex-wrap: wrap;
  flex-shrink: 0;
  justify-content: flex-end;
  align-items: center;
  display: flex;
}

.modal-footer > * {
  margin: calc(var(--modal-footer-gap) * .5);
}

@media (width >= 576px) {
  .modal {
    --modal-margin: 1.75rem;
    --modal-box-shadow: var(--box-shadow);
  }

  .modal-dialog {
    max-width: var(--modal-width);
    margin-left: auto;
    margin-right: auto;
  }

  .modal-sm {
    --modal-width: 300px;
  }
}

@media (width >= 992px) {
  .modal-lg, .modal-xl {
    --modal-width: 800px;
  }
}

@media (width >= 1200px) {
  .modal-xl {
    --modal-width: 1140px;
  }
}

.modal-fullscreen {
  width: 100vw;
  max-width: none;
  height: 100%;
  margin: 0;
}

.modal-fullscreen .modal-content {
  border: 0;
  border-radius: 0;
  height: 100%;
}

.modal-fullscreen .modal-header, .modal-fullscreen .modal-footer {
  border-radius: 0;
}

.modal-fullscreen .modal-body {
  overflow-y: auto;
}

@media (width <= 575.98px) {
  .modal-fullscreen-sm-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }

  .modal-fullscreen-sm-down .modal-content {
    border: 0;
    border-radius: 0;
    height: 100%;
  }

  .modal-fullscreen-sm-down .modal-header, .modal-fullscreen-sm-down .modal-footer {
    border-radius: 0;
  }

  .modal-fullscreen-sm-down .modal-body {
    overflow-y: auto;
  }
}

@media (width <= 767.98px) {
  .modal-fullscreen-md-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }

  .modal-fullscreen-md-down .modal-content {
    border: 0;
    border-radius: 0;
    height: 100%;
  }

  .modal-fullscreen-md-down .modal-header, .modal-fullscreen-md-down .modal-footer {
    border-radius: 0;
  }

  .modal-fullscreen-md-down .modal-body {
    overflow-y: auto;
  }
}

@media (width <= 991.98px) {
  .modal-fullscreen-lg-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }

  .modal-fullscreen-lg-down .modal-content {
    border: 0;
    border-radius: 0;
    height: 100%;
  }

  .modal-fullscreen-lg-down .modal-header, .modal-fullscreen-lg-down .modal-footer {
    border-radius: 0;
  }

  .modal-fullscreen-lg-down .modal-body {
    overflow-y: auto;
  }
}

@media (width <= 1199.98px) {
  .modal-fullscreen-xl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }

  .modal-fullscreen-xl-down .modal-content {
    border: 0;
    border-radius: 0;
    height: 100%;
  }

  .modal-fullscreen-xl-down .modal-header, .modal-fullscreen-xl-down .modal-footer {
    border-radius: 0;
  }

  .modal-fullscreen-xl-down .modal-body {
    overflow-y: auto;
  }
}

@media (width <= 1399.98px) {
  .modal-fullscreen-xxl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }

  .modal-fullscreen-xxl-down .modal-content {
    border: 0;
    border-radius: 0;
    height: 100%;
  }

  .modal-fullscreen-xxl-down .modal-header, .modal-fullscreen-xxl-down .modal-footer {
    border-radius: 0;
  }

  .modal-fullscreen-xxl-down .modal-body {
    overflow-y: auto;
  }
}

.tooltip {
  --tooltip-zindex: 1070;
  --tooltip-max-width: 200px;
  --tooltip-padding-x: .5rem;
  --tooltip-padding-y: .25rem;
  --tooltip-margin: ;
  --tooltip-font-size: .8rem;
  --tooltip-color: var(--body-bg);
  --tooltip-bg: var(--emphasis-color);
  --tooltip-border-radius: var(--border-radius);
  --tooltip-opacity: .9;
  --tooltip-arrow-width: .8rem;
  --tooltip-arrow-height: .4rem;
  z-index: var(--tooltip-zindex);
  margin: var(--tooltip-margin);
  font-family: var(--font-sans-serif);
  text-align: left;
  text-align: start;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  font-size: var(--tooltip-font-size);
  word-wrap: break-word;
  opacity: 0;
  text-decoration: none;
  display: block;
}

.tooltip.show {
  opacity: var(--tooltip-opacity);
}

.tooltip .tooltip-arrow {
  width: var(--tooltip-arrow-width);
  height: var(--tooltip-arrow-height);
  display: block;
}

.tooltip .tooltip-arrow:before {
  content: "";
  border-style: solid;
  border-color: #0000;
  position: absolute;
}

.bs-tooltip-top .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^="top"] .tooltip-arrow {
  bottom: calc(-1 * var(--tooltip-arrow-height));
}

.bs-tooltip-top .tooltip-arrow:before, .bs-tooltip-auto[data-popper-placement^="top"] .tooltip-arrow:before {
  border-width: var(--tooltip-arrow-height) calc(var(--tooltip-arrow-width) * .5) 0;
  border-top-color: var(--tooltip-bg);
  top: -1px;
}

.bs-tooltip-end .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^="right"] .tooltip-arrow {
  left: calc(-1 * var(--tooltip-arrow-height));
  width: var(--tooltip-arrow-height);
  height: var(--tooltip-arrow-width);
}

.bs-tooltip-end .tooltip-arrow:before, .bs-tooltip-auto[data-popper-placement^="right"] .tooltip-arrow:before {
  border-width: calc(var(--tooltip-arrow-width) * .5) var(--tooltip-arrow-height) calc(var(--tooltip-arrow-width) * .5) 0;
  border-right-color: var(--tooltip-bg);
  right: -1px;
}

.bs-tooltip-bottom .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^="bottom"] .tooltip-arrow {
  top: calc(-1 * var(--tooltip-arrow-height));
}

.bs-tooltip-bottom .tooltip-arrow:before, .bs-tooltip-auto[data-popper-placement^="bottom"] .tooltip-arrow:before {
  border-width: 0 calc(var(--tooltip-arrow-width) * .5) var(--tooltip-arrow-height);
  border-bottom-color: var(--tooltip-bg);
  bottom: -1px;
}

.bs-tooltip-start .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^="left"] .tooltip-arrow {
  right: calc(-1 * var(--tooltip-arrow-height));
  width: var(--tooltip-arrow-height);
  height: var(--tooltip-arrow-width);
}

.bs-tooltip-start .tooltip-arrow:before, .bs-tooltip-auto[data-popper-placement^="left"] .tooltip-arrow:before {
  border-width: calc(var(--tooltip-arrow-width) * .5) 0 calc(var(--tooltip-arrow-width) * .5) var(--tooltip-arrow-height);
  border-left-color: var(--tooltip-bg);
  left: -1px;
}

.tooltip-inner {
  max-width: var(--tooltip-max-width);
  padding: var(--tooltip-padding-y) var(--tooltip-padding-x);
  color: var(--tooltip-color);
  text-align: center;
  background-color: var(--tooltip-bg);
  border-radius: var(--tooltip-border-radius);
}

.popover {
  --popover-zindex: 1060;
  --popover-max-width: 276px;
  --popover-font-size: .8rem;
  --popover-bg: var(--body-bg);
  --popover-border-width: var(--border-width);
  --popover-border-color: var(--border-color-translucent);
  --popover-border-radius: var(--border-radius-lg);
  --popover-inner-border-radius: calc(var(--border-radius-lg)  - var(--border-width));
  --popover-box-shadow: var(--box-shadow);
  --popover-header-padding-x: 1rem;
  --popover-header-padding-y: .5rem;
  --popover-header-font-size: 1rem;
  --popover-header-color: var(--template-bg-dark);
  --popover-header-bg: var(--secondary-bg);
  --popover-body-padding-x: 1rem;
  --popover-body-padding-y: 1rem;
  --popover-body-color: var(--body-color);
  --popover-arrow-width: 1rem;
  --popover-arrow-height: .5rem;
  --popover-arrow-border: var(--popover-border-color);
  z-index: var(--popover-zindex);
  max-width: var(--popover-max-width);
  font-family: var(--font-sans-serif);
  text-align: left;
  text-align: start;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  font-size: var(--popover-font-size);
  word-wrap: break-word;
  background-color: var(--popover-bg);
  border: var(--popover-border-width) solid var(--popover-border-color);
  border-radius: var(--popover-border-radius);
  background-clip: padding-box;
  text-decoration: none;
  display: block;
}

.popover .popover-arrow {
  width: var(--popover-arrow-width);
  height: var(--popover-arrow-height);
  display: block;
}

.popover .popover-arrow:before, .popover .popover-arrow:after {
  content: "";
  border: 0 solid #0000;
  display: block;
  position: absolute;
}

.bs-popover-top > .popover-arrow, .bs-popover-auto[data-popper-placement^="top"] > .popover-arrow {
  bottom: calc(-1 * (var(--popover-arrow-height))  - var(--popover-border-width));
}

.bs-popover-top > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="top"] > .popover-arrow:before, .bs-popover-top > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="top"] > .popover-arrow:after {
  border-width: var(--popover-arrow-height) calc(var(--popover-arrow-width) * .5) 0;
}

.bs-popover-top > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="top"] > .popover-arrow:before {
  border-top-color: var(--popover-arrow-border);
  bottom: 0;
}

.bs-popover-top > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="top"] > .popover-arrow:after {
  bottom: var(--popover-border-width);
  border-top-color: var(--popover-bg);
}

.bs-popover-end > .popover-arrow, .bs-popover-auto[data-popper-placement^="right"] > .popover-arrow {
  left: calc(-1 * (var(--popover-arrow-height))  - var(--popover-border-width));
  width: var(--popover-arrow-height);
  height: var(--popover-arrow-width);
}

.bs-popover-end > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="right"] > .popover-arrow:before, .bs-popover-end > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="right"] > .popover-arrow:after {
  border-width: calc(var(--popover-arrow-width) * .5) var(--popover-arrow-height) calc(var(--popover-arrow-width) * .5) 0;
}

.bs-popover-end > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="right"] > .popover-arrow:before {
  border-right-color: var(--popover-arrow-border);
  left: 0;
}

.bs-popover-end > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="right"] > .popover-arrow:after {
  left: var(--popover-border-width);
  border-right-color: var(--popover-bg);
}

.bs-popover-bottom > .popover-arrow, .bs-popover-auto[data-popper-placement^="bottom"] > .popover-arrow {
  top: calc(-1 * (var(--popover-arrow-height))  - var(--popover-border-width));
}

.bs-popover-bottom > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="bottom"] > .popover-arrow:before, .bs-popover-bottom > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="bottom"] > .popover-arrow:after {
  border-width: 0 calc(var(--popover-arrow-width) * .5) var(--popover-arrow-height);
}

.bs-popover-bottom > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="bottom"] > .popover-arrow:before {
  border-bottom-color: var(--popover-arrow-border);
  top: 0;
}

.bs-popover-bottom > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="bottom"] > .popover-arrow:after {
  top: var(--popover-border-width);
  border-bottom-color: var(--popover-bg);
}

.bs-popover-bottom .popover-header:before, .bs-popover-auto[data-popper-placement^="bottom"] .popover-header:before {
  width: var(--popover-arrow-width);
  margin-left: calc(-.5 * var(--popover-arrow-width));
  content: "";
  border-bottom: var(--popover-border-width) solid var(--popover-header-bg);
  display: block;
  position: absolute;
  top: 0;
  left: 50%;
}

.bs-popover-start > .popover-arrow, .bs-popover-auto[data-popper-placement^="left"] > .popover-arrow {
  right: calc(-1 * (var(--popover-arrow-height))  - var(--popover-border-width));
  width: var(--popover-arrow-height);
  height: var(--popover-arrow-width);
}

.bs-popover-start > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="left"] > .popover-arrow:before, .bs-popover-start > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="left"] > .popover-arrow:after {
  border-width: calc(var(--popover-arrow-width) * .5) 0 calc(var(--popover-arrow-width) * .5) var(--popover-arrow-height);
}

.bs-popover-start > .popover-arrow:before, .bs-popover-auto[data-popper-placement^="left"] > .popover-arrow:before {
  border-left-color: var(--popover-arrow-border);
  right: 0;
}

.bs-popover-start > .popover-arrow:after, .bs-popover-auto[data-popper-placement^="left"] > .popover-arrow:after {
  right: var(--popover-border-width);
  border-left-color: var(--popover-bg);
}

.popover-header {
  padding: var(--popover-header-padding-y) var(--popover-header-padding-x);
  font-size: var(--popover-header-font-size);
  color: var(--popover-header-color);
  background-color: var(--popover-header-bg);
  border-bottom: var(--popover-border-width) solid var(--popover-border-color);
  border-top-left-radius: var(--popover-inner-border-radius);
  border-top-right-radius: var(--popover-inner-border-radius);
  margin-bottom: 0;
}

.popover-header:empty {
  display: none;
}

.popover-body {
  padding: var(--popover-body-padding-y) var(--popover-body-padding-x);
  color: var(--popover-body-color);
}

.carousel {
  position: relative;
}

.carousel.pointer-event {
  touch-action: pan-y;
}

.carousel-inner {
  width: 100%;
  position: relative;
  overflow: hidden;
}

.carousel-inner:after {
  clear: both;
  content: "";
  display: block;
}

.carousel-item {
  float: left;
  backface-visibility: hidden;
  width: 100%;
  margin-right: -100%;
  transition: transform .6s ease-in-out;
  display: none;
  position: relative;
}

@media (prefers-reduced-motion: reduce) {
  .carousel-item {
    transition: none;
  }
}

.carousel-item.active, .carousel-item-next, .carousel-item-prev {
  display: block;
}

.carousel-item-next:not(.carousel-item-start), .active.carousel-item-end {
  transform: translateX(100%);
}

.carousel-item-prev:not(.carousel-item-end), .active.carousel-item-start {
  transform: translateX(-100%);
}

.carousel-fade .carousel-item {
  opacity: 0;
  transition-property: opacity;
  transform: none;
}

.carousel-fade .carousel-item.active, .carousel-fade .carousel-item-next.carousel-item-start, .carousel-fade .carousel-item-prev.carousel-item-end {
  z-index: 1;
  opacity: 1;
}

.carousel-fade .active.carousel-item-start, .carousel-fade .active.carousel-item-end {
  z-index: 0;
  opacity: 0;
  transition: opacity 0s .6s;
}

@media (prefers-reduced-motion: reduce) {
  .carousel-fade .active.carousel-item-start, .carousel-fade .active.carousel-item-end {
    transition: none;
  }
}

.carousel-control-prev, .carousel-control-next {
  z-index: 1;
  color: #fff;
  text-align: center;
  width: 15%;
  filter: var(--carousel-control-icon-filter);
  opacity: .5;
  background: none;
  border: 0;
  justify-content: center;
  align-items: center;
  padding: 0;
  transition: opacity .15s;
  display: flex;
  position: absolute;
  top: 0;
  bottom: 0;
}

@media (prefers-reduced-motion: reduce) {
  .carousel-control-prev, .carousel-control-next {
    transition: none;
  }
}

.carousel-control-prev:hover, .carousel-control-prev:focus, .carousel-control-next:hover, .carousel-control-next:focus {
  color: #fff;
  opacity: .9;
  outline: 0;
  text-decoration: none;
}

.carousel-control-prev {
  left: 0;
}

.carousel-control-next {
  right: 0;
}

.carousel-control-prev-icon, .carousel-control-next-icon {
  background-position: 50%;
  background-repeat: no-repeat;
  background-size: 100% 100%;
  width: 2rem;
  height: 2rem;
  display: inline-block;
}

.carousel-control-prev-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M11.354 1.646a.5.5 0 0 1 0 .708L5.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0'/%3e%3c/svg%3e");
}

.carousel-control-next-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M4.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L10.293 8 4.646 2.354a.5.5 0 0 1 0-.708'/%3e%3c/svg%3e");
}

.carousel-indicators {
  z-index: 2;
  justify-content: center;
  margin-bottom: 1rem;
  margin-left: 15%;
  margin-right: 15%;
  padding: 0;
  display: flex;
  position: absolute;
  bottom: 0;
  left: 0;
  right: 0;
}

.carousel-indicators [data-bs-target] {
  box-sizing: content-box;
  text-indent: -999px;
  cursor: pointer;
  background-color: var(--carousel-indicator-active-bg);
  opacity: .5;
  background-clip: padding-box;
  border: 10px solid #0000;
  border-left: 0;
  border-right: 0;
  flex: 0 auto;
  width: 30px;
  height: 3px;
  margin-left: 3px;
  margin-right: 3px;
  padding: 0;
  transition: opacity .6s;
}

@media (prefers-reduced-motion: reduce) {
  .carousel-indicators [data-bs-target] {
    transition: none;
  }
}

.carousel-indicators .active {
  opacity: 1;
}

.carousel-caption {
  color: var(--carousel-caption-color);
  text-align: center;
  padding-top: 1.25rem;
  padding-bottom: 1.25rem;
  position: absolute;
  bottom: 1.25rem;
  left: 15%;
  right: 15%;
}

.carousel-dark {
  --carousel-indicator-active-bg: #000;
  --carousel-caption-color: #000;
  --carousel-control-icon-filter: invert(1) grayscale(100);
}

:root, [data-bs-theme="light"] {
  --carousel-indicator-active-bg: #fff;
  --carousel-caption-color: #fff;
  --carousel-control-icon-filter: ;
}

[data-bs-theme="dark"] {
  --carousel-indicator-active-bg: #000;
  --carousel-caption-color: #000;
  --carousel-control-icon-filter: invert(1) grayscale(100);
}

.spinner-grow, .spinner-border {
  width: var(--spinner-width);
  height: var(--spinner-height);
  vertical-align: var(--spinner-vertical-align);
  animation: var(--spinner-animation-speed) linear infinite var(--spinner-animation-name);
  border-radius: 50%;
  flex-shrink: 0;
  display: inline-block;
}

@keyframes spinner-border {
  to {
    transform: rotate(360deg);
  }
}

.spinner-border {
  --spinner-width: 2rem;
  --spinner-height: 2rem;
  --spinner-vertical-align: -.125em;
  --spinner-border-width: .25em;
  --spinner-animation-speed: .75s;
  --spinner-animation-name: spinner-border;
  border: var(--spinner-border-width) solid currentcolor;
  border-right-color: #0000;
}

.spinner-border-sm {
  --spinner-width: 1rem;
  --spinner-height: 1rem;
  --spinner-border-width: .2em;
}

@keyframes spinner-grow {
  0% {
    transform: scale(0);
  }

  50% {
    opacity: 1;
    transform: none;
  }
}

.spinner-grow {
  --spinner-width: 2rem;
  --spinner-height: 2rem;
  --spinner-vertical-align: -.125em;
  --spinner-animation-speed: .75s;
  --spinner-animation-name: spinner-grow;
  opacity: 0;
  background-color: currentColor;
}

.spinner-grow-sm {
  --spinner-width: 1rem;
  --spinner-height: 1rem;
}

@media (prefers-reduced-motion: reduce) {
  .spinner-border, .spinner-grow {
    --spinner-animation-speed: 1.5s;
  }
}

.offcanvas, .offcanvas-xxl, .offcanvas-xl, .offcanvas-lg, .offcanvas-md, .offcanvas-sm {
  --offcanvas-zindex: 1045;
  --offcanvas-width: 400px;
  --offcanvas-height: 30vh;
  --offcanvas-padding-x: 1rem;
  --offcanvas-padding-y: 1rem;
  --offcanvas-color: var(--body-color);
  --offcanvas-bg: var(--body-bg);
  --offcanvas-border-width: var(--border-width);
  --offcanvas-border-color: var(--border-color-translucent);
  --offcanvas-box-shadow: var(--box-shadow-sm);
  --offcanvas-transition: transform .3s ease-in-out;
  --offcanvas-title-line-height: 1.5;
}

@media (width <= 575.98px) {
  .offcanvas-sm {
    z-index: var(--offcanvas-zindex);
    max-width: 100%;
    color: var(--offcanvas-color);
    visibility: hidden;
    background-color: var(--offcanvas-bg);
    transition: var(--offcanvas-transition);
    background-clip: padding-box;
    outline: 0;
    flex-direction: column;
    display: flex;
    position: fixed;
    bottom: 0;
  }
}

@media (width <= 575.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-sm {
    transition: none;
  }
}

@media (width <= 575.98px) {
  .offcanvas-sm.offcanvas-start {
    width: var(--offcanvas-width);
    border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    left: 0;
    transform: translateX(-100%);
  }

  .offcanvas-sm.offcanvas-end {
    width: var(--offcanvas-width);
    border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    right: 0;
    transform: translateX(100%);
  }

  .offcanvas-sm.offcanvas-top {
    height: var(--offcanvas-height);
    border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    top: 0;
    left: 0;
    right: 0;
    transform: translateY(-100%);
  }

  .offcanvas-sm.offcanvas-bottom {
    height: var(--offcanvas-height);
    border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    left: 0;
    right: 0;
    transform: translateY(100%);
  }

  .offcanvas-sm.showing, .offcanvas-sm.show:not(.hiding) {
    transform: none;
  }

  .offcanvas-sm.showing, .offcanvas-sm.hiding, .offcanvas-sm.show {
    visibility: visible;
  }
}

@media (width >= 576px) {
  .offcanvas-sm {
    --offcanvas-height: auto;
    --offcanvas-border-width: 0;
    background-color: #0000 !important;
  }

  .offcanvas-sm .offcanvas-header {
    display: none;
  }

  .offcanvas-sm .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
    background-color: #0000 !important;
  }
}

@media (width <= 767.98px) {
  .offcanvas-md {
    z-index: var(--offcanvas-zindex);
    max-width: 100%;
    color: var(--offcanvas-color);
    visibility: hidden;
    background-color: var(--offcanvas-bg);
    transition: var(--offcanvas-transition);
    background-clip: padding-box;
    outline: 0;
    flex-direction: column;
    display: flex;
    position: fixed;
    bottom: 0;
  }
}

@media (width <= 767.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-md {
    transition: none;
  }
}

@media (width <= 767.98px) {
  .offcanvas-md.offcanvas-start {
    width: var(--offcanvas-width);
    border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    left: 0;
    transform: translateX(-100%);
  }

  .offcanvas-md.offcanvas-end {
    width: var(--offcanvas-width);
    border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    right: 0;
    transform: translateX(100%);
  }

  .offcanvas-md.offcanvas-top {
    height: var(--offcanvas-height);
    border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    top: 0;
    left: 0;
    right: 0;
    transform: translateY(-100%);
  }

  .offcanvas-md.offcanvas-bottom {
    height: var(--offcanvas-height);
    border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    left: 0;
    right: 0;
    transform: translateY(100%);
  }

  .offcanvas-md.showing, .offcanvas-md.show:not(.hiding) {
    transform: none;
  }

  .offcanvas-md.showing, .offcanvas-md.hiding, .offcanvas-md.show {
    visibility: visible;
  }
}

@media (width >= 768px) {
  .offcanvas-md {
    --offcanvas-height: auto;
    --offcanvas-border-width: 0;
    background-color: #0000 !important;
  }

  .offcanvas-md .offcanvas-header {
    display: none;
  }

  .offcanvas-md .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
    background-color: #0000 !important;
  }
}

@media (width <= 991.98px) {
  .offcanvas-lg {
    z-index: var(--offcanvas-zindex);
    max-width: 100%;
    color: var(--offcanvas-color);
    visibility: hidden;
    background-color: var(--offcanvas-bg);
    transition: var(--offcanvas-transition);
    background-clip: padding-box;
    outline: 0;
    flex-direction: column;
    display: flex;
    position: fixed;
    bottom: 0;
  }
}

@media (width <= 991.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-lg {
    transition: none;
  }
}

@media (width <= 991.98px) {
  .offcanvas-lg.offcanvas-start {
    width: var(--offcanvas-width);
    border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    left: 0;
    transform: translateX(-100%);
  }

  .offcanvas-lg.offcanvas-end {
    width: var(--offcanvas-width);
    border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    right: 0;
    transform: translateX(100%);
  }

  .offcanvas-lg.offcanvas-top {
    height: var(--offcanvas-height);
    border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    top: 0;
    left: 0;
    right: 0;
    transform: translateY(-100%);
  }

  .offcanvas-lg.offcanvas-bottom {
    height: var(--offcanvas-height);
    border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    left: 0;
    right: 0;
    transform: translateY(100%);
  }

  .offcanvas-lg.showing, .offcanvas-lg.show:not(.hiding) {
    transform: none;
  }

  .offcanvas-lg.showing, .offcanvas-lg.hiding, .offcanvas-lg.show {
    visibility: visible;
  }
}

@media (width >= 992px) {
  .offcanvas-lg {
    --offcanvas-height: auto;
    --offcanvas-border-width: 0;
    background-color: #0000 !important;
  }

  .offcanvas-lg .offcanvas-header {
    display: none;
  }

  .offcanvas-lg .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
    background-color: #0000 !important;
  }
}

@media (width <= 1199.98px) {
  .offcanvas-xl {
    z-index: var(--offcanvas-zindex);
    max-width: 100%;
    color: var(--offcanvas-color);
    visibility: hidden;
    background-color: var(--offcanvas-bg);
    transition: var(--offcanvas-transition);
    background-clip: padding-box;
    outline: 0;
    flex-direction: column;
    display: flex;
    position: fixed;
    bottom: 0;
  }
}

@media (width <= 1199.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xl {
    transition: none;
  }
}

@media (width <= 1199.98px) {
  .offcanvas-xl.offcanvas-start {
    width: var(--offcanvas-width);
    border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    left: 0;
    transform: translateX(-100%);
  }

  .offcanvas-xl.offcanvas-end {
    width: var(--offcanvas-width);
    border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    right: 0;
    transform: translateX(100%);
  }

  .offcanvas-xl.offcanvas-top {
    height: var(--offcanvas-height);
    border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    top: 0;
    left: 0;
    right: 0;
    transform: translateY(-100%);
  }

  .offcanvas-xl.offcanvas-bottom {
    height: var(--offcanvas-height);
    border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    left: 0;
    right: 0;
    transform: translateY(100%);
  }

  .offcanvas-xl.showing, .offcanvas-xl.show:not(.hiding) {
    transform: none;
  }

  .offcanvas-xl.showing, .offcanvas-xl.hiding, .offcanvas-xl.show {
    visibility: visible;
  }
}

@media (width >= 1200px) {
  .offcanvas-xl {
    --offcanvas-height: auto;
    --offcanvas-border-width: 0;
    background-color: #0000 !important;
  }

  .offcanvas-xl .offcanvas-header {
    display: none;
  }

  .offcanvas-xl .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
    background-color: #0000 !important;
  }
}

@media (width <= 1399.98px) {
  .offcanvas-xxl {
    z-index: var(--offcanvas-zindex);
    max-width: 100%;
    color: var(--offcanvas-color);
    visibility: hidden;
    background-color: var(--offcanvas-bg);
    transition: var(--offcanvas-transition);
    background-clip: padding-box;
    outline: 0;
    flex-direction: column;
    display: flex;
    position: fixed;
    bottom: 0;
  }
}

@media (width <= 1399.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xxl {
    transition: none;
  }
}

@media (width <= 1399.98px) {
  .offcanvas-xxl.offcanvas-start {
    width: var(--offcanvas-width);
    border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    left: 0;
    transform: translateX(-100%);
  }

  .offcanvas-xxl.offcanvas-end {
    width: var(--offcanvas-width);
    border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    top: 0;
    right: 0;
    transform: translateX(100%);
  }

  .offcanvas-xxl.offcanvas-top {
    height: var(--offcanvas-height);
    border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    top: 0;
    left: 0;
    right: 0;
    transform: translateY(-100%);
  }

  .offcanvas-xxl.offcanvas-bottom {
    height: var(--offcanvas-height);
    border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
    max-height: 100%;
    left: 0;
    right: 0;
    transform: translateY(100%);
  }

  .offcanvas-xxl.showing, .offcanvas-xxl.show:not(.hiding) {
    transform: none;
  }

  .offcanvas-xxl.showing, .offcanvas-xxl.hiding, .offcanvas-xxl.show {
    visibility: visible;
  }
}

@media (width >= 1400px) {
  .offcanvas-xxl {
    --offcanvas-height: auto;
    --offcanvas-border-width: 0;
    background-color: #0000 !important;
  }

  .offcanvas-xxl .offcanvas-header {
    display: none;
  }

  .offcanvas-xxl .offcanvas-body {
    flex-grow: 0;
    padding: 0;
    display: flex;
    overflow-y: visible;
    background-color: #0000 !important;
  }
}

.offcanvas {
  z-index: var(--offcanvas-zindex);
  max-width: 100%;
  color: var(--offcanvas-color);
  visibility: hidden;
  background-color: var(--offcanvas-bg);
  transition: var(--offcanvas-transition);
  background-clip: padding-box;
  outline: 0;
  flex-direction: column;
  display: flex;
  position: fixed;
  bottom: 0;
}

@media (prefers-reduced-motion: reduce) {
  .offcanvas {
    transition: none;
  }
}

.offcanvas.offcanvas-start {
  width: var(--offcanvas-width);
  border-right: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
  top: 0;
  left: 0;
  transform: translateX(-100%);
}

.offcanvas.offcanvas-end {
  width: var(--offcanvas-width);
  border-left: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
  top: 0;
  right: 0;
  transform: translateX(100%);
}

.offcanvas.offcanvas-top {
  height: var(--offcanvas-height);
  border-bottom: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
  max-height: 100%;
  top: 0;
  left: 0;
  right: 0;
  transform: translateY(-100%);
}

.offcanvas.offcanvas-bottom {
  height: var(--offcanvas-height);
  border-top: var(--offcanvas-border-width) solid var(--offcanvas-border-color);
  max-height: 100%;
  left: 0;
  right: 0;
  transform: translateY(100%);
}

.offcanvas.showing, .offcanvas.show:not(.hiding) {
  transform: none;
}

.offcanvas.showing, .offcanvas.hiding, .offcanvas.show {
  visibility: visible;
}

.offcanvas-backdrop {
  z-index: 1040;
  background-color: #000;
  width: 100vw;
  height: 100vh;
  position: fixed;
  top: 0;
  left: 0;
}

.offcanvas-backdrop.fade {
  opacity: 0;
}

.offcanvas-backdrop.show {
  opacity: .5;
}

.offcanvas-header {
  padding: var(--offcanvas-padding-y) var(--offcanvas-padding-x);
  align-items: center;
  display: flex;
}

.offcanvas-header .btn-close {
  padding: calc(var(--offcanvas-padding-y) * .5) calc(var(--offcanvas-padding-x) * .5);
  margin-top: calc(-.5 * var(--offcanvas-padding-y));
  margin-right: calc(-.5 * var(--offcanvas-padding-x));
  margin-bottom: calc(-.5 * var(--offcanvas-padding-y));
  margin-left: auto;
}

.offcanvas-title {
  line-height: var(--offcanvas-title-line-height);
  margin-bottom: 0;
}

.offcanvas-body {
  padding: var(--offcanvas-padding-y) var(--offcanvas-padding-x);
  flex-grow: 1;
  overflow-y: auto;
}

.placeholder {
  vertical-align: middle;
  cursor: wait;
  opacity: .5;
  background-color: currentColor;
  min-height: 1em;
  display: inline-block;
}

.placeholder.btn:before {
  content: "";
  display: inline-block;
}

.placeholder-xs {
  min-height: .6em;
}

.placeholder-sm {
  min-height: .8em;
}

.placeholder-lg {
  min-height: 1.2em;
}

.placeholder-glow .placeholder {
  animation: 2s ease-in-out infinite placeholder-glow;
}

@keyframes placeholder-glow {
  50% {
    opacity: .2;
  }
}

.placeholder-wave {
  animation: 2s linear infinite placeholder-wave;
  mask-image: linear-gradient(130deg, #000 55%, #000c 75%, #000 95%);
  mask-size: 200% 100%;
}

@keyframes placeholder-wave {
  100% {
    mask-position: -200% 0;
  }
}

.clearfix:after {
  clear: both;
  content: "";
  display: block;
}

.text-bg-primary {
  color: #fff !important;
  background-color: RGBA(var(--primary-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-secondary {
  color: #fff !important;
  background-color: RGBA(var(--secondary-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-success {
  color: #fff !important;
  background-color: RGBA(var(--success-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-info {
  color: #fff !important;
  background-color: RGBA(var(--info-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-warning {
  color: #000 !important;
  background-color: RGBA(var(--warning-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-danger {
  color: #fff !important;
  background-color: RGBA(var(--danger-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-light {
  color: #000 !important;
  background-color: RGBA(var(--light-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-dark {
  color: #fff !important;
  background-color: RGBA(var(--dark-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-action {
  color: #fff !important;
  background-color: RGBA(var(--action-rgb), var(--bg-opacity, 1)) !important;
}

.text-bg-error {
  color: #fff !important;
  background-color: RGBA(var(--error-rgb), var(--bg-opacity, 1)) !important;
}

.link-primary {
  color: RGBA(var(--primary-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--primary-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-primary:hover, .link-primary:focus {
  color: RGBA(15, 38, 66, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(15, 38, 66, var(--link-underline-opacity, 1)) !important;
}

.link-secondary {
  color: RGBA(var(--secondary-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--secondary-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-secondary:hover, .link-secondary:focus {
  color: RGBA(58, 64, 70, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(58, 64, 70, var(--link-underline-opacity, 1)) !important;
}

.link-success {
  color: RGBA(var(--success-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--success-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-success:hover, .link-success:focus {
  color: RGBA(55, 100, 67, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(55, 100, 67, var(--link-underline-opacity, 1)) !important;
}

.link-info {
  color: RGBA(var(--info-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--info-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-info:hover, .link-info:focus {
  color: RGBA(34, 84, 147, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(34, 84, 147, var(--link-underline-opacity, 1)) !important;
}

.link-warning {
  color: RGBA(var(--warning-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--warning-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-warning:hover, .link-warning:focus {
  color: RGBA(255, 196, 67, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(255, 196, 67, var(--link-underline-opacity, 1)) !important;
}

.link-danger {
  color: RGBA(var(--danger-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--danger-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-danger:hover, .link-danger:focus {
  color: RGBA(158, 32, 31, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(158, 32, 31, var(--link-underline-opacity, 1)) !important;
}

.link-light {
  color: RGBA(var(--light-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--light-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-light:hover, .link-light:focus {
  color: RGBA(249, 250, 251, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(249, 250, 251, var(--link-underline-opacity, 1)) !important;
}

.link-dark {
  color: RGBA(var(--dark-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--dark-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-dark:hover, .link-dark:focus {
  color: RGBA(26, 30, 33, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(26, 30, 33, var(--link-underline-opacity, 1)) !important;
}

.link-action {
  color: RGBA(var(--action-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--action-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-action:hover, .link-action:focus {
  color: RGBA(15, 38, 66, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(15, 38, 66, var(--link-underline-opacity, 1)) !important;
}

.link-error {
  color: RGBA(var(--error-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--error-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-error:hover, .link-error:focus {
  color: RGBA(47, 10, 10, var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(47, 10, 10, var(--link-underline-opacity, 1)) !important;
}

.link-body-emphasis {
  color: RGBA(var(--emphasis-color-rgb), var(--link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--emphasis-color-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-body-emphasis:hover, .link-body-emphasis:focus {
  color: RGBA(var(--emphasis-color-rgb), var(--link-opacity, .75)) !important;
  text-decoration-color: RGBA(var(--emphasis-color-rgb), var(--link-underline-opacity, .75)) !important;
}

.focus-ring:focus {
  box-shadow: var(--focus-ring-x, 0) var(--focus-ring-y, 0) var(--focus-ring-blur, 0) var(--focus-ring-width) var(--focus-ring-color);
  outline: 0;
}

.icon-link {
  text-decoration-color: rgba(var(--link-color-rgb), var(--link-opacity, .5));
  text-underline-offset: .25em;
  backface-visibility: hidden;
  align-items: center;
  gap: .375rem;
  display: inline-flex;
}

.icon-link > .bi {
  fill: currentColor;
  flex-shrink: 0;
  width: 1em;
  height: 1em;
  transition: transform .2s ease-in-out;
}

@media (prefers-reduced-motion: reduce) {
  .icon-link > .bi {
    transition: none;
  }
}

.icon-link-hover:hover > .bi, .icon-link-hover:focus-visible > .bi {
  transform: var(--icon-link-transform, translate3d(.25em, 0, 0));
}

.ratio {
  width: 100%;
  position: relative;
}

.ratio:before {
  padding-top: var(--aspect-ratio);
  content: "";
  display: block;
}

.ratio > * {
  width: 100%;
  height: 100%;
  position: absolute;
  top: 0;
  left: 0;
}

.ratio-1x1 {
  --aspect-ratio: 100%;
}

.ratio-4x3 {
  --aspect-ratio: 75%;
}

.ratio-16x9 {
  --aspect-ratio: 56.25%;
}

.ratio-21x9 {
  --aspect-ratio: 42.8571%;
}

.fixed-top {
  z-index: 1030;
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
}

.fixed-bottom {
  z-index: 1030;
  position: fixed;
  bottom: 0;
  left: 0;
  right: 0;
}

.sticky-top {
  z-index: 1020;
  position: sticky;
  top: 0;
}

.sticky-bottom {
  z-index: 1020;
  position: sticky;
  bottom: 0;
}

@media (width >= 576px) {
  .sticky-sm-top {
    z-index: 1020;
    position: sticky;
    top: 0;
  }

  .sticky-sm-bottom {
    z-index: 1020;
    position: sticky;
    bottom: 0;
  }
}

@media (width >= 768px) {
  .sticky-md-top {
    z-index: 1020;
    position: sticky;
    top: 0;
  }

  .sticky-md-bottom {
    z-index: 1020;
    position: sticky;
    bottom: 0;
  }
}

@media (width >= 992px) {
  .sticky-lg-top {
    z-index: 1020;
    position: sticky;
    top: 0;
  }

  .sticky-lg-bottom {
    z-index: 1020;
    position: sticky;
    bottom: 0;
  }
}

@media (width >= 1200px) {
  .sticky-xl-top {
    z-index: 1020;
    position: sticky;
    top: 0;
  }

  .sticky-xl-bottom {
    z-index: 1020;
    position: sticky;
    bottom: 0;
  }
}

@media (width >= 1400px) {
  .sticky-xxl-top {
    z-index: 1020;
    position: sticky;
    top: 0;
  }

  .sticky-xxl-bottom {
    z-index: 1020;
    position: sticky;
    bottom: 0;
  }
}

.hstack {
  flex-direction: row;
  align-self: stretch;
  align-items: center;
  display: flex;
}

.vstack {
  flex-direction: column;
  flex: auto;
  align-self: stretch;
  display: flex;
}

.visually-hidden, .sr-only, .visually-hidden-focusable:not(:focus):not(:focus-within) {
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
  width: 1px !important;
  height: 1px !important;
  margin: -1px !important;
  padding: 0 !important;
  overflow: hidden !important;
}

.visually-hidden:not(caption), .sr-only:not(caption), .visually-hidden-focusable:not(:focus):not(:focus-within):not(caption) {
  position: absolute !important;
}

.visually-hidden *, .sr-only *, .visually-hidden-focusable:not(:focus):not(:focus-within) * {
  overflow: hidden !important;
}

.stretched-link:after {
  z-index: 1;
  content: "";
  position: absolute;
  inset: 0;
}

.text-truncate {
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
}

.vr {
  width: var(--border-width);
  opacity: .25;
  background-color: currentColor;
  align-self: stretch;
  min-height: 1em;
  display: inline-block;
}

.align-baseline {
  vertical-align: baseline !important;
}

.align-top {
  vertical-align: top !important;
}

.align-middle {
  vertical-align: middle !important;
}

.align-bottom {
  vertical-align: bottom !important;
}

.align-text-bottom {
  vertical-align: text-bottom !important;
}

.align-text-top {
  vertical-align: text-top !important;
}

.float-start {
  float: left !important;
}

.float-end {
  float: right !important;
}

.float-none {
  float: none !important;
}

.object-fit-contain {
  object-fit: contain !important;
}

.object-fit-cover {
  object-fit: cover !important;
}

.object-fit-fill {
  object-fit: fill !important;
}

.object-fit-scale {
  object-fit: scale-down !important;
}

.object-fit-none {
  object-fit: none !important;
}

.opacity-0 {
  opacity: 0 !important;
}

.opacity-25 {
  opacity: .25 !important;
}

.opacity-50 {
  opacity: .5 !important;
}

.opacity-75 {
  opacity: .75 !important;
}

.opacity-100 {
  opacity: 1 !important;
}

.overflow-auto {
  overflow: auto !important;
}

.overflow-hidden {
  overflow: hidden !important;
}

.overflow-visible {
  overflow: visible !important;
}

.overflow-scroll {
  overflow: scroll !important;
}

.overflow-x-auto {
  overflow-x: auto !important;
}

.overflow-x-hidden {
  overflow-x: hidden !important;
}

.overflow-x-visible {
  overflow-x: visible !important;
}

.overflow-x-scroll {
  overflow-x: scroll !important;
}

.overflow-y-auto {
  overflow-y: auto !important;
}

.overflow-y-hidden {
  overflow-y: hidden !important;
}

.overflow-y-visible {
  overflow-y: visible !important;
}

.overflow-y-scroll {
  overflow-y: scroll !important;
}

.d-inline {
  display: inline !important;
}

.d-inline-block {
  display: inline-block !important;
}

.d-block {
  display: block !important;
}

.d-grid {
  display: grid !important;
}

.d-inline-grid {
  display: inline-grid !important;
}

.d-table {
  display: table !important;
}

.d-table-row {
  display: table-row !important;
}

.d-table-cell {
  display: table-cell !important;
}

.d-flex {
  display: flex !important;
}

.d-inline-flex {
  display: inline-flex !important;
}

.d-none {
  display: none !important;
}

.shadow {
  box-shadow: var(--box-shadow) !important;
}

.shadow-sm {
  box-shadow: var(--box-shadow-sm) !important;
}

.shadow-lg {
  box-shadow: var(--box-shadow-lg) !important;
}

.shadow-none {
  box-shadow: none !important;
}

.focus-ring-primary {
  --focus-ring-color: rgba(var(--primary-rgb), var(--focus-ring-opacity));
}

.focus-ring-secondary {
  --focus-ring-color: rgba(var(--secondary-rgb), var(--focus-ring-opacity));
}

.focus-ring-success {
  --focus-ring-color: rgba(var(--success-rgb), var(--focus-ring-opacity));
}

.focus-ring-info {
  --focus-ring-color: rgba(var(--info-rgb), var(--focus-ring-opacity));
}

.focus-ring-warning {
  --focus-ring-color: rgba(var(--warning-rgb), var(--focus-ring-opacity));
}

.focus-ring-danger {
  --focus-ring-color: rgba(var(--danger-rgb), var(--focus-ring-opacity));
}

.focus-ring-light {
  --focus-ring-color: rgba(var(--light-rgb), var(--focus-ring-opacity));
}

.focus-ring-dark {
  --focus-ring-color: rgba(var(--dark-rgb), var(--focus-ring-opacity));
}

.focus-ring-action {
  --focus-ring-color: rgba(var(--action-rgb), var(--focus-ring-opacity));
}

.focus-ring-error {
  --focus-ring-color: rgba(var(--error-rgb), var(--focus-ring-opacity));
}

.position-static {
  position: static !important;
}

.position-relative {
  position: relative !important;
}

.position-absolute {
  position: absolute !important;
}

.position-fixed {
  position: fixed !important;
}

.position-sticky {
  position: sticky !important;
}

.top-0 {
  top: 0 !important;
}

.top-50 {
  top: 50% !important;
}

.top-100 {
  top: 100% !important;
}

.bottom-0 {
  bottom: 0 !important;
}

.bottom-50 {
  bottom: 50% !important;
}

.bottom-100 {
  bottom: 100% !important;
}

.start-0 {
  left: 0 !important;
}

.start-50 {
  left: 50% !important;
}

.start-100 {
  left: 100% !important;
}

.end-0 {
  right: 0 !important;
}

.end-50 {
  right: 50% !important;
}

.end-100 {
  right: 100% !important;
}

.translate-middle {
  transform: translate(-50%, -50%) !important;
}

.translate-middle-x {
  transform: translateX(-50%) !important;
}

.translate-middle-y {
  transform: translateY(-50%) !important;
}

.border {
  border: var(--border-width) var(--border-style) var(--border-color) !important;
}

.border-0 {
  border: 0 !important;
}

.border-top {
  border-top: var(--border-width) var(--border-style) var(--border-color) !important;
}

.border-top-0 {
  border-top: 0 !important;
}

.border-end {
  border-right: var(--border-width) var(--border-style) var(--border-color) !important;
}

.border-end-0 {
  border-right: 0 !important;
}

.border-bottom {
  border-bottom: var(--border-width) var(--border-style) var(--border-color) !important;
}

.border-bottom-0 {
  border-bottom: 0 !important;
}

.border-start {
  border-left: var(--border-width) var(--border-style) var(--border-color) !important;
}

.border-start-0 {
  border-left: 0 !important;
}

.border-primary {
  --border-opacity: 1;
  border-color: rgba(var(--primary-rgb), var(--border-opacity)) !important;
}

.border-secondary {
  --border-opacity: 1;
  border-color: rgba(var(--secondary-rgb), var(--border-opacity)) !important;
}

.border-success {
  --border-opacity: 1;
  border-color: rgba(var(--success-rgb), var(--border-opacity)) !important;
}

.border-info {
  --border-opacity: 1;
  border-color: rgba(var(--info-rgb), var(--border-opacity)) !important;
}

.border-warning {
  --border-opacity: 1;
  border-color: rgba(var(--warning-rgb), var(--border-opacity)) !important;
}

.border-danger {
  --border-opacity: 1;
  border-color: rgba(var(--danger-rgb), var(--border-opacity)) !important;
}

.border-light {
  --border-opacity: 1;
  border-color: rgba(var(--light-rgb), var(--border-opacity)) !important;
}

.border-dark {
  --border-opacity: 1;
  border-color: rgba(var(--dark-rgb), var(--border-opacity)) !important;
}

.border-action {
  --border-opacity: 1;
  border-color: rgba(var(--action-rgb), var(--border-opacity)) !important;
}

.border-error {
  --border-opacity: 1;
  border-color: rgba(var(--error-rgb), var(--border-opacity)) !important;
}

.border-black {
  --border-opacity: 1;
  border-color: rgba(var(--black-rgb), var(--border-opacity)) !important;
}

.border-white {
  --border-opacity: 1;
  border-color: rgba(var(--white-rgb), var(--border-opacity)) !important;
}

.border-primary-subtle {
  border-color: var(--primary-border-subtle) !important;
}

.border-secondary-subtle {
  border-color: var(--secondary-border-subtle) !important;
}

.border-success-subtle {
  border-color: var(--success-border-subtle) !important;
}

.border-info-subtle {
  border-color: var(--info-border-subtle) !important;
}

.border-warning-subtle {
  border-color: var(--warning-border-subtle) !important;
}

.border-danger-subtle {
  border-color: var(--danger-border-subtle) !important;
}

.border-light-subtle {
  border-color: var(--light-border-subtle) !important;
}

.border-dark-subtle {
  border-color: var(--dark-border-subtle) !important;
}

.border-1 {
  border-width: 1px !important;
}

.border-2 {
  border-width: 2px !important;
}

.border-3 {
  border-width: 3px !important;
}

.border-4 {
  border-width: 4px !important;
}

.border-5 {
  border-width: 5px !important;
}

.border-opacity-10 {
  --border-opacity: .1;
}

.border-opacity-25 {
  --border-opacity: .25;
}

.border-opacity-50 {
  --border-opacity: .5;
}

.border-opacity-75 {
  --border-opacity: .75;
}

.border-opacity-100 {
  --border-opacity: 1;
}

.w-25 {
  width: 25% !important;
}

.w-50 {
  width: 50% !important;
}

.w-75 {
  width: 75% !important;
}

.w-100 {
  width: 100% !important;
}

.w-auto {
  width: auto !important;
}

.mw-100 {
  max-width: 100% !important;
}

.vw-100 {
  width: 100vw !important;
}

.min-vw-100 {
  min-width: 100vw !important;
}

.h-25 {
  height: 25% !important;
}

.h-50 {
  height: 50% !important;
}

.h-75 {
  height: 75% !important;
}

.h-100 {
  height: 100% !important;
}

.h-auto {
  height: auto !important;
}

.mh-100 {
  max-height: 100% !important;
}

.vh-100 {
  height: 100vh !important;
}

.min-vh-100 {
  min-height: 100vh !important;
}

.flex-fill {
  flex: auto !important;
}

.flex-row {
  flex-direction: row !important;
}

.flex-column {
  flex-direction: column !important;
}

.flex-row-reverse {
  flex-direction: row-reverse !important;
}

.flex-column-reverse {
  flex-direction: column-reverse !important;
}

.flex-grow-0 {
  flex-grow: 0 !important;
}

.flex-grow-1 {
  flex-grow: 1 !important;
}

.flex-shrink-0 {
  flex-shrink: 0 !important;
}

.flex-shrink-1 {
  flex-shrink: 1 !important;
}

.flex-wrap {
  flex-wrap: wrap !important;
}

.flex-nowrap {
  flex-wrap: nowrap !important;
}

.flex-wrap-reverse {
  flex-wrap: wrap-reverse !important;
}

.justify-content-start {
  justify-content: flex-start !important;
}

.justify-content-end {
  justify-content: flex-end !important;
}

.justify-content-center {
  justify-content: center !important;
}

.justify-content-between {
  justify-content: space-between !important;
}

.justify-content-around {
  justify-content: space-around !important;
}

.justify-content-evenly {
  justify-content: space-evenly !important;
}

.align-items-start {
  align-items: flex-start !important;
}

.align-items-end {
  align-items: flex-end !important;
}

.align-items-center {
  align-items: center !important;
}

.align-items-baseline {
  align-items: baseline !important;
}

.align-items-stretch {
  align-items: stretch !important;
}

.align-content-start {
  align-content: flex-start !important;
}

.align-content-end {
  align-content: flex-end !important;
}

.align-content-center {
  align-content: center !important;
}

.align-content-between {
  align-content: space-between !important;
}

.align-content-around {
  align-content: space-around !important;
}

.align-content-stretch {
  align-content: stretch !important;
}

.align-self-auto {
  align-self: auto !important;
}

.align-self-start {
  align-self: flex-start !important;
}

.align-self-end {
  align-self: flex-end !important;
}

.align-self-center {
  align-self: center !important;
}

.align-self-baseline {
  align-self: baseline !important;
}

.align-self-stretch {
  align-self: stretch !important;
}

.order-first {
  order: -1 !important;
}

.order-0 {
  order: 0 !important;
}

.order-1 {
  order: 1 !important;
}

.order-2 {
  order: 2 !important;
}

.order-3 {
  order: 3 !important;
}

.order-4 {
  order: 4 !important;
}

.order-5 {
  order: 5 !important;
}

.order-last {
  order: 6 !important;
}

.m-0 {
  margin: 0 !important;
}

.m-1 {
  margin: .25rem !important;
}

.m-2 {
  margin: .5rem !important;
}

.m-3 {
  margin: 1rem !important;
}

.m-4 {
  margin: 1.5rem !important;
}

.m-5 {
  margin: 3rem !important;
}

.m-auto {
  margin: auto !important;
}

.mx-0 {
  margin-left: 0 !important;
  margin-right: 0 !important;
}

.mx-1 {
  margin-left: .25rem !important;
  margin-right: .25rem !important;
}

.mx-2 {
  margin-left: .5rem !important;
  margin-right: .5rem !important;
}

.mx-3 {
  margin-left: 1rem !important;
  margin-right: 1rem !important;
}

.mx-4 {
  margin-left: 1.5rem !important;
  margin-right: 1.5rem !important;
}

.mx-5 {
  margin-left: 3rem !important;
  margin-right: 3rem !important;
}

.mx-auto {
  margin-left: auto !important;
  margin-right: auto !important;
}

.my-0 {
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}

.my-1 {
  margin-top: .25rem !important;
  margin-bottom: .25rem !important;
}

.my-2 {
  margin-top: .5rem !important;
  margin-bottom: .5rem !important;
}

.my-3 {
  margin-top: 1rem !important;
  margin-bottom: 1rem !important;
}

.my-4 {
  margin-top: 1.5rem !important;
  margin-bottom: 1.5rem !important;
}

.my-5 {
  margin-top: 3rem !important;
  margin-bottom: 3rem !important;
}

.my-auto {
  margin-top: auto !important;
  margin-bottom: auto !important;
}

.mt-0 {
  margin-top: 0 !important;
}

.mt-1 {
  margin-top: .25rem !important;
}

.mt-2 {
  margin-top: .5rem !important;
}

.mt-3 {
  margin-top: 1rem !important;
}

.mt-4 {
  margin-top: 1.5rem !important;
}

.mt-5 {
  margin-top: 3rem !important;
}

.mt-auto {
  margin-top: auto !important;
}

.me-0 {
  margin-right: 0 !important;
}

.me-1 {
  margin-right: .25rem !important;
}

.me-2 {
  margin-right: .5rem !important;
}

.me-3 {
  margin-right: 1rem !important;
}

.me-4 {
  margin-right: 1.5rem !important;
}

.me-5 {
  margin-right: 3rem !important;
}

.me-auto {
  margin-right: auto !important;
}

.mb-0 {
  margin-bottom: 0 !important;
}

.mb-1 {
  margin-bottom: .25rem !important;
}

.mb-2 {
  margin-bottom: .5rem !important;
}

.mb-3 {
  margin-bottom: 1rem !important;
}

.mb-4 {
  margin-bottom: 1.5rem !important;
}

.mb-5 {
  margin-bottom: 3rem !important;
}

.mb-auto {
  margin-bottom: auto !important;
}

.ms-0 {
  margin-left: 0 !important;
}

.ms-1 {
  margin-left: .25rem !important;
}

.ms-2 {
  margin-left: .5rem !important;
}

.ms-3 {
  margin-left: 1rem !important;
}

.ms-4 {
  margin-left: 1.5rem !important;
}

.ms-5 {
  margin-left: 3rem !important;
}

.ms-auto {
  margin-left: auto !important;
}

.p-0 {
  padding: 0 !important;
}

.p-1 {
  padding: .25rem !important;
}

.p-2 {
  padding: .5rem !important;
}

.p-3 {
  padding: 1rem !important;
}

.p-4 {
  padding: 1.5rem !important;
}

.p-5 {
  padding: 3rem !important;
}

.px-0 {
  padding-left: 0 !important;
  padding-right: 0 !important;
}

.px-1 {
  padding-left: .25rem !important;
  padding-right: .25rem !important;
}

.px-2 {
  padding-left: .5rem !important;
  padding-right: .5rem !important;
}

.px-3 {
  padding-left: 1rem !important;
  padding-right: 1rem !important;
}

.px-4 {
  padding-left: 1.5rem !important;
  padding-right: 1.5rem !important;
}

.px-5 {
  padding-left: 3rem !important;
  padding-right: 3rem !important;
}

.py-0 {
  padding-top: 0 !important;
  padding-bottom: 0 !important;
}

.py-1 {
  padding-top: .25rem !important;
  padding-bottom: .25rem !important;
}

.py-2 {
  padding-top: .5rem !important;
  padding-bottom: .5rem !important;
}

.py-3 {
  padding-top: 1rem !important;
  padding-bottom: 1rem !important;
}

.py-4 {
  padding-top: 1.5rem !important;
  padding-bottom: 1.5rem !important;
}

.py-5 {
  padding-top: 3rem !important;
  padding-bottom: 3rem !important;
}

.pt-0 {
  padding-top: 0 !important;
}

.pt-1 {
  padding-top: .25rem !important;
}

.pt-2 {
  padding-top: .5rem !important;
}

.pt-3 {
  padding-top: 1rem !important;
}

.pt-4 {
  padding-top: 1.5rem !important;
}

.pt-5 {
  padding-top: 3rem !important;
}

.pe-0 {
  padding-right: 0 !important;
}

.pe-1 {
  padding-right: .25rem !important;
}

.pe-2 {
  padding-right: .5rem !important;
}

.pe-3 {
  padding-right: 1rem !important;
}

.pe-4 {
  padding-right: 1.5rem !important;
}

.pe-5 {
  padding-right: 3rem !important;
}

.pb-0 {
  padding-bottom: 0 !important;
}

.pb-1 {
  padding-bottom: .25rem !important;
}

.pb-2 {
  padding-bottom: .5rem !important;
}

.pb-3 {
  padding-bottom: 1rem !important;
}

.pb-4 {
  padding-bottom: 1.5rem !important;
}

.pb-5 {
  padding-bottom: 3rem !important;
}

.ps-0 {
  padding-left: 0 !important;
}

.ps-1 {
  padding-left: .25rem !important;
}

.ps-2 {
  padding-left: .5rem !important;
}

.ps-3 {
  padding-left: 1rem !important;
}

.ps-4 {
  padding-left: 1.5rem !important;
}

.ps-5 {
  padding-left: 3rem !important;
}

.gap-0 {
  gap: 0 !important;
}

.gap-1 {
  gap: .25rem !important;
}

.gap-2 {
  gap: .5rem !important;
}

.gap-3 {
  gap: 1rem !important;
}

.gap-4 {
  gap: 1.5rem !important;
}

.gap-5 {
  gap: 3rem !important;
}

.row-gap-0 {
  row-gap: 0 !important;
}

.row-gap-1 {
  row-gap: .25rem !important;
}

.row-gap-2 {
  row-gap: .5rem !important;
}

.row-gap-3 {
  row-gap: 1rem !important;
}

.row-gap-4 {
  row-gap: 1.5rem !important;
}

.row-gap-5 {
  row-gap: 3rem !important;
}

.column-gap-0 {
  column-gap: 0 !important;
}

.column-gap-1 {
  column-gap: .25rem !important;
}

.column-gap-2 {
  column-gap: .5rem !important;
}

.column-gap-3 {
  column-gap: 1rem !important;
}

.column-gap-4 {
  column-gap: 1.5rem !important;
}

.column-gap-5 {
  column-gap: 3rem !important;
}

.font-monospace {
  font-family: var(--font-monospace) !important;
}

.fs-1 {
  font-size: calc(1.29rem + .48vw) !important;
}

.fs-2 {
  font-size: calc(1.275rem + .3vw) !important;
}

.fs-3 {
  font-size: 1.25rem !important;
}

.fs-4 {
  font-size: 1rem !important;
}

.fs-5 {
  font-size: .9286rem !important;
}

.fs-6 {
  font-size: .8571rem !important;
}

.fst-italic {
  font-style: italic !important;
}

.fst-normal {
  font-style: normal !important;
}

.fw-lighter {
  font-weight: lighter !important;
}

.fw-light {
  font-weight: 300 !important;
}

.fw-normal {
  font-weight: 400 !important;
}

.fw-medium {
  font-weight: 500 !important;
}

.fw-semibold {
  font-weight: 600 !important;
}

.fw-bold {
  font-weight: 700 !important;
}

.fw-bolder {
  font-weight: bolder !important;
}

.lh-1 {
  line-height: 1 !important;
}

.lh-sm {
  line-height: 1.25 !important;
}

.lh-base {
  line-height: 1.5 !important;
}

.lh-lg {
  line-height: 2 !important;
}

.text-start {
  text-align: left !important;
}

.text-end {
  text-align: right !important;
}

.text-center {
  text-align: center !important;
}

.text-decoration-none {
  text-decoration: none !important;
}

.text-decoration-underline {
  text-decoration: underline !important;
}

.text-decoration-line-through {
  text-decoration: line-through !important;
}

.text-lowercase {
  text-transform: lowercase !important;
}

.text-uppercase {
  text-transform: uppercase !important;
}

.text-capitalize {
  text-transform: capitalize !important;
}

.text-wrap {
  white-space: normal !important;
}

.text-nowrap {
  white-space: nowrap !important;
}

.text-break {
  word-wrap: break-word !important;
  word-break: break-word !important;
}

.text-primary {
  --text-opacity: 1;
  color: rgba(var(--primary-rgb), var(--text-opacity)) !important;
}

.text-secondary {
  --text-opacity: 1;
  color: rgba(var(--secondary-rgb), var(--text-opacity)) !important;
}

.text-success {
  --text-opacity: 1;
  color: rgba(var(--success-rgb), var(--text-opacity)) !important;
}

.text-info {
  --text-opacity: 1;
  color: rgba(var(--info-rgb), var(--text-opacity)) !important;
}

.text-warning {
  --text-opacity: 1;
  color: rgba(var(--warning-rgb), var(--text-opacity)) !important;
}

.text-danger {
  --text-opacity: 1;
  color: rgba(var(--danger-rgb), var(--text-opacity)) !important;
}

.text-light {
  --text-opacity: 1;
  color: rgba(var(--light-rgb), var(--text-opacity)) !important;
}

.text-dark {
  --text-opacity: 1;
  color: rgba(var(--dark-rgb), var(--text-opacity)) !important;
}

.text-action {
  --text-opacity: 1;
  color: rgba(var(--action-rgb), var(--text-opacity)) !important;
}

.text-error {
  --text-opacity: 1;
  color: rgba(var(--error-rgb), var(--text-opacity)) !important;
}

.text-black {
  --text-opacity: 1;
  color: rgba(var(--black-rgb), var(--text-opacity)) !important;
}

.text-white {
  --text-opacity: 1;
  color: rgba(var(--white-rgb), var(--text-opacity)) !important;
}

.text-body {
  --text-opacity: 1;
  color: rgba(var(--body-color-rgb), var(--text-opacity)) !important;
}

.text-muted {
  --text-opacity: 1;
  color: var(--secondary-color) !important;
}

.text-black-50 {
  --text-opacity: 1;
  color: #00000080 !important;
}

.text-white-50 {
  --text-opacity: 1;
  color: #ffffff80 !important;
}

.text-body-secondary {
  --text-opacity: 1;
  color: var(--secondary-color) !important;
}

.text-body-tertiary {
  --text-opacity: 1;
  color: var(--tertiary-color) !important;
}

.text-body-emphasis {
  --text-opacity: 1;
  color: var(--emphasis-color) !important;
}

.text-reset {
  --text-opacity: 1;
  color: inherit !important;
}

.text-opacity-25 {
  --text-opacity: .25;
}

.text-opacity-50 {
  --text-opacity: .5;
}

.text-opacity-75 {
  --text-opacity: .75;
}

.text-opacity-100 {
  --text-opacity: 1;
}

.text-primary-emphasis {
  color: var(--primary-text-emphasis) !important;
}

.text-secondary-emphasis {
  color: var(--secondary-text-emphasis) !important;
}

.text-success-emphasis {
  color: var(--success-text-emphasis) !important;
}

.text-info-emphasis {
  color: var(--info-text-emphasis) !important;
}

.text-warning-emphasis {
  color: var(--warning-text-emphasis) !important;
}

.text-danger-emphasis {
  color: var(--danger-text-emphasis) !important;
}

.text-light-emphasis {
  color: var(--light-text-emphasis) !important;
}

.text-dark-emphasis {
  color: var(--dark-text-emphasis) !important;
}

.link-opacity-10, .link-opacity-10-hover:hover {
  --link-opacity: .1;
}

.link-opacity-25, .link-opacity-25-hover:hover {
  --link-opacity: .25;
}

.link-opacity-50, .link-opacity-50-hover:hover {
  --link-opacity: .5;
}

.link-opacity-75, .link-opacity-75-hover:hover {
  --link-opacity: .75;
}

.link-opacity-100, .link-opacity-100-hover:hover {
  --link-opacity: 1;
}

.link-offset-1, .link-offset-1-hover:hover {
  text-underline-offset: .125em !important;
}

.link-offset-2, .link-offset-2-hover:hover {
  text-underline-offset: .25em !important;
}

.link-offset-3, .link-offset-3-hover:hover {
  text-underline-offset: .375em !important;
}

.link-underline-primary {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--primary-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-secondary {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--secondary-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-success {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--success-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-info {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--info-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-warning {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--warning-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-danger {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--danger-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-light {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--light-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-dark {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--dark-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-action {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--action-rgb), var(--link-underline-opacity)) !important;
}

.link-underline-error {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--error-rgb), var(--link-underline-opacity)) !important;
}

.link-underline {
  --link-underline-opacity: 1;
  text-decoration-color: rgba(var(--link-color-rgb), var(--link-underline-opacity, 1)) !important;
}

.link-underline-opacity-0, .link-underline-opacity-0-hover:hover {
  --link-underline-opacity: 0;
}

.link-underline-opacity-10, .link-underline-opacity-10-hover:hover {
  --link-underline-opacity: .1;
}

.link-underline-opacity-25, .link-underline-opacity-25-hover:hover {
  --link-underline-opacity: .25;
}

.link-underline-opacity-50, .link-underline-opacity-50-hover:hover {
  --link-underline-opacity: .5;
}

.link-underline-opacity-75, .link-underline-opacity-75-hover:hover {
  --link-underline-opacity: .75;
}

.link-underline-opacity-100, .link-underline-opacity-100-hover:hover {
  --link-underline-opacity: 1;
}

.bg-primary {
  --bg-opacity: 1;
  background-color: rgba(var(--primary-rgb), var(--bg-opacity)) !important;
}

.bg-secondary {
  --bg-opacity: 1;
  background-color: rgba(var(--secondary-rgb), var(--bg-opacity)) !important;
}

.bg-success {
  --bg-opacity: 1;
  background-color: rgba(var(--success-rgb), var(--bg-opacity)) !important;
}

.bg-info {
  --bg-opacity: 1;
  background-color: rgba(var(--info-rgb), var(--bg-opacity)) !important;
}

.bg-warning {
  --bg-opacity: 1;
  background-color: rgba(var(--warning-rgb), var(--bg-opacity)) !important;
}

.bg-danger {
  --bg-opacity: 1;
  background-color: rgba(var(--danger-rgb), var(--bg-opacity)) !important;
}

.bg-light {
  --bg-opacity: 1;
  background-color: rgba(var(--light-rgb), var(--bg-opacity)) !important;
}

.bg-dark {
  --bg-opacity: 1;
  background-color: rgba(var(--dark-rgb), var(--bg-opacity)) !important;
}

.bg-action {
  --bg-opacity: 1;
  background-color: rgba(var(--action-rgb), var(--bg-opacity)) !important;
}

.bg-error {
  --bg-opacity: 1;
  background-color: rgba(var(--error-rgb), var(--bg-opacity)) !important;
}

.bg-black {
  --bg-opacity: 1;
  background-color: rgba(var(--black-rgb), var(--bg-opacity)) !important;
}

.bg-white {
  --bg-opacity: 1;
  background-color: rgba(var(--white-rgb), var(--bg-opacity)) !important;
}

.bg-body {
  --bg-opacity: 1;
  background-color: rgba(var(--body-bg-rgb), var(--bg-opacity)) !important;
}

.bg-transparent {
  --bg-opacity: 1;
  background-color: #0000 !important;
}

.bg-body-secondary {
  --bg-opacity: 1;
  background-color: rgba(var(--secondary-bg-rgb), var(--bg-opacity)) !important;
}

.bg-body-tertiary {
  --bg-opacity: 1;
  background-color: rgba(var(--tertiary-bg-rgb), var(--bg-opacity)) !important;
}

.bg-opacity-10 {
  --bg-opacity: .1;
}

.bg-opacity-25 {
  --bg-opacity: .25;
}

.bg-opacity-50 {
  --bg-opacity: .5;
}

.bg-opacity-75 {
  --bg-opacity: .75;
}

.bg-opacity-100 {
  --bg-opacity: 1;
}

.bg-primary-subtle {
  background-color: var(--primary-bg-subtle) !important;
}

.bg-secondary-subtle {
  background-color: var(--secondary-bg-subtle) !important;
}

.bg-success-subtle {
  background-color: var(--success-bg-subtle) !important;
}

.bg-info-subtle {
  background-color: var(--info-bg-subtle) !important;
}

.bg-warning-subtle {
  background-color: var(--warning-bg-subtle) !important;
}

.bg-danger-subtle {
  background-color: var(--danger-bg-subtle) !important;
}

.bg-light-subtle {
  background-color: var(--light-bg-subtle) !important;
}

.bg-dark-subtle {
  background-color: var(--dark-bg-subtle) !important;
}

.bg-gradient {
  background-image: var(--gradient) !important;
}

.user-select-all {
  user-select: all !important;
}

.user-select-auto {
  user-select: auto !important;
}

.user-select-none {
  user-select: none !important;
}

.pe-none {
  pointer-events: none !important;
}

.pe-auto {
  pointer-events: auto !important;
}

.rounded {
  border-radius: var(--border-radius) !important;
}

.rounded-0 {
  border-radius: 0 !important;
}

.rounded-1 {
  border-radius: var(--border-radius-sm) !important;
}

.rounded-2 {
  border-radius: var(--border-radius) !important;
}

.rounded-3 {
  border-radius: var(--border-radius-lg) !important;
}

.rounded-4 {
  border-radius: var(--border-radius-xl) !important;
}

.rounded-5 {
  border-radius: var(--border-radius-xxl) !important;
}

.rounded-circle {
  border-radius: 50% !important;
}

.rounded-pill {
  border-radius: var(--border-radius-pill) !important;
}

.rounded-top {
  border-top-left-radius: var(--border-radius) !important;
  border-top-right-radius: var(--border-radius) !important;
}

.rounded-top-0 {
  border-top-left-radius: 0 !important;
  border-top-right-radius: 0 !important;
}

.rounded-top-1 {
  border-top-left-radius: var(--border-radius-sm) !important;
  border-top-right-radius: var(--border-radius-sm) !important;
}

.rounded-top-2 {
  border-top-left-radius: var(--border-radius) !important;
  border-top-right-radius: var(--border-radius) !important;
}

.rounded-top-3 {
  border-top-left-radius: var(--border-radius-lg) !important;
  border-top-right-radius: var(--border-radius-lg) !important;
}

.rounded-top-4 {
  border-top-left-radius: var(--border-radius-xl) !important;
  border-top-right-radius: var(--border-radius-xl) !important;
}

.rounded-top-5 {
  border-top-left-radius: var(--border-radius-xxl) !important;
  border-top-right-radius: var(--border-radius-xxl) !important;
}

.rounded-top-circle {
  border-top-left-radius: 50% !important;
  border-top-right-radius: 50% !important;
}

.rounded-top-pill {
  border-top-left-radius: var(--border-radius-pill) !important;
  border-top-right-radius: var(--border-radius-pill) !important;
}

.rounded-end {
  border-top-right-radius: var(--border-radius) !important;
  border-bottom-right-radius: var(--border-radius) !important;
}

.rounded-end-0 {
  border-top-right-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
}

.rounded-end-1 {
  border-top-right-radius: var(--border-radius-sm) !important;
  border-bottom-right-radius: var(--border-radius-sm) !important;
}

.rounded-end-2 {
  border-top-right-radius: var(--border-radius) !important;
  border-bottom-right-radius: var(--border-radius) !important;
}

.rounded-end-3 {
  border-top-right-radius: var(--border-radius-lg) !important;
  border-bottom-right-radius: var(--border-radius-lg) !important;
}

.rounded-end-4 {
  border-top-right-radius: var(--border-radius-xl) !important;
  border-bottom-right-radius: var(--border-radius-xl) !important;
}

.rounded-end-5 {
  border-top-right-radius: var(--border-radius-xxl) !important;
  border-bottom-right-radius: var(--border-radius-xxl) !important;
}

.rounded-end-circle {
  border-top-right-radius: 50% !important;
  border-bottom-right-radius: 50% !important;
}

.rounded-end-pill {
  border-top-right-radius: var(--border-radius-pill) !important;
  border-bottom-right-radius: var(--border-radius-pill) !important;
}

.rounded-bottom {
  border-bottom-right-radius: var(--border-radius) !important;
  border-bottom-left-radius: var(--border-radius) !important;
}

.rounded-bottom-0 {
  border-bottom-right-radius: 0 !important;
  border-bottom-left-radius: 0 !important;
}

.rounded-bottom-1 {
  border-bottom-right-radius: var(--border-radius-sm) !important;
  border-bottom-left-radius: var(--border-radius-sm) !important;
}

.rounded-bottom-2 {
  border-bottom-right-radius: var(--border-radius) !important;
  border-bottom-left-radius: var(--border-radius) !important;
}

.rounded-bottom-3 {
  border-bottom-right-radius: var(--border-radius-lg) !important;
  border-bottom-left-radius: var(--border-radius-lg) !important;
}

.rounded-bottom-4 {
  border-bottom-right-radius: var(--border-radius-xl) !important;
  border-bottom-left-radius: var(--border-radius-xl) !important;
}

.rounded-bottom-5 {
  border-bottom-right-radius: var(--border-radius-xxl) !important;
  border-bottom-left-radius: var(--border-radius-xxl) !important;
}

.rounded-bottom-circle {
  border-bottom-right-radius: 50% !important;
  border-bottom-left-radius: 50% !important;
}

.rounded-bottom-pill {
  border-bottom-right-radius: var(--border-radius-pill) !important;
  border-bottom-left-radius: var(--border-radius-pill) !important;
}

.rounded-start {
  border-bottom-left-radius: var(--border-radius) !important;
  border-top-left-radius: var(--border-radius) !important;
}

.rounded-start-0 {
  border-top-left-radius: 0 !important;
  border-bottom-left-radius: 0 !important;
}

.rounded-start-1 {
  border-bottom-left-radius: var(--border-radius-sm) !important;
  border-top-left-radius: var(--border-radius-sm) !important;
}

.rounded-start-2 {
  border-bottom-left-radius: var(--border-radius) !important;
  border-top-left-radius: var(--border-radius) !important;
}

.rounded-start-3 {
  border-bottom-left-radius: var(--border-radius-lg) !important;
  border-top-left-radius: var(--border-radius-lg) !important;
}

.rounded-start-4 {
  border-bottom-left-radius: var(--border-radius-xl) !important;
  border-top-left-radius: var(--border-radius-xl) !important;
}

.rounded-start-5 {
  border-bottom-left-radius: var(--border-radius-xxl) !important;
  border-top-left-radius: var(--border-radius-xxl) !important;
}

.rounded-start-circle {
  border-top-left-radius: 50% !important;
  border-bottom-left-radius: 50% !important;
}

.rounded-start-pill {
  border-bottom-left-radius: var(--border-radius-pill) !important;
  border-top-left-radius: var(--border-radius-pill) !important;
}

.visible {
  visibility: visible !important;
}

.invisible {
  visibility: hidden !important;
}

.z-n1 {
  z-index: -1 !important;
}

.z-0 {
  z-index: 0 !important;
}

.z-1 {
  z-index: 1 !important;
}

.z-2 {
  z-index: 2 !important;
}

.z-3 {
  z-index: 3 !important;
}

@media (width >= 576px) {
  .float-sm-start {
    float: left !important;
  }

  .float-sm-end {
    float: right !important;
  }

  .float-sm-none {
    float: none !important;
  }

  .object-fit-sm-contain {
    object-fit: contain !important;
  }

  .object-fit-sm-cover {
    object-fit: cover !important;
  }

  .object-fit-sm-fill {
    object-fit: fill !important;
  }

  .object-fit-sm-scale {
    object-fit: scale-down !important;
  }

  .object-fit-sm-none {
    object-fit: none !important;
  }

  .d-sm-inline {
    display: inline !important;
  }

  .d-sm-inline-block {
    display: inline-block !important;
  }

  .d-sm-block {
    display: block !important;
  }

  .d-sm-grid {
    display: grid !important;
  }

  .d-sm-inline-grid {
    display: inline-grid !important;
  }

  .d-sm-table {
    display: table !important;
  }

  .d-sm-table-row {
    display: table-row !important;
  }

  .d-sm-table-cell {
    display: table-cell !important;
  }

  .d-sm-flex {
    display: flex !important;
  }

  .d-sm-inline-flex {
    display: inline-flex !important;
  }

  .d-sm-none {
    display: none !important;
  }

  .flex-sm-fill {
    flex: auto !important;
  }

  .flex-sm-row {
    flex-direction: row !important;
  }

  .flex-sm-column {
    flex-direction: column !important;
  }

  .flex-sm-row-reverse {
    flex-direction: row-reverse !important;
  }

  .flex-sm-column-reverse {
    flex-direction: column-reverse !important;
  }

  .flex-sm-grow-0 {
    flex-grow: 0 !important;
  }

  .flex-sm-grow-1 {
    flex-grow: 1 !important;
  }

  .flex-sm-shrink-0 {
    flex-shrink: 0 !important;
  }

  .flex-sm-shrink-1 {
    flex-shrink: 1 !important;
  }

  .flex-sm-wrap {
    flex-wrap: wrap !important;
  }

  .flex-sm-nowrap {
    flex-wrap: nowrap !important;
  }

  .flex-sm-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }

  .justify-content-sm-start {
    justify-content: flex-start !important;
  }

  .justify-content-sm-end {
    justify-content: flex-end !important;
  }

  .justify-content-sm-center {
    justify-content: center !important;
  }

  .justify-content-sm-between {
    justify-content: space-between !important;
  }

  .justify-content-sm-around {
    justify-content: space-around !important;
  }

  .justify-content-sm-evenly {
    justify-content: space-evenly !important;
  }

  .align-items-sm-start {
    align-items: flex-start !important;
  }

  .align-items-sm-end {
    align-items: flex-end !important;
  }

  .align-items-sm-center {
    align-items: center !important;
  }

  .align-items-sm-baseline {
    align-items: baseline !important;
  }

  .align-items-sm-stretch {
    align-items: stretch !important;
  }

  .align-content-sm-start {
    align-content: flex-start !important;
  }

  .align-content-sm-end {
    align-content: flex-end !important;
  }

  .align-content-sm-center {
    align-content: center !important;
  }

  .align-content-sm-between {
    align-content: space-between !important;
  }

  .align-content-sm-around {
    align-content: space-around !important;
  }

  .align-content-sm-stretch {
    align-content: stretch !important;
  }

  .align-self-sm-auto {
    align-self: auto !important;
  }

  .align-self-sm-start {
    align-self: flex-start !important;
  }

  .align-self-sm-end {
    align-self: flex-end !important;
  }

  .align-self-sm-center {
    align-self: center !important;
  }

  .align-self-sm-baseline {
    align-self: baseline !important;
  }

  .align-self-sm-stretch {
    align-self: stretch !important;
  }

  .order-sm-first {
    order: -1 !important;
  }

  .order-sm-0 {
    order: 0 !important;
  }

  .order-sm-1 {
    order: 1 !important;
  }

  .order-sm-2 {
    order: 2 !important;
  }

  .order-sm-3 {
    order: 3 !important;
  }

  .order-sm-4 {
    order: 4 !important;
  }

  .order-sm-5 {
    order: 5 !important;
  }

  .order-sm-last {
    order: 6 !important;
  }

  .m-sm-0 {
    margin: 0 !important;
  }

  .m-sm-1 {
    margin: .25rem !important;
  }

  .m-sm-2 {
    margin: .5rem !important;
  }

  .m-sm-3 {
    margin: 1rem !important;
  }

  .m-sm-4 {
    margin: 1.5rem !important;
  }

  .m-sm-5 {
    margin: 3rem !important;
  }

  .m-sm-auto {
    margin: auto !important;
  }

  .mx-sm-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .mx-sm-1 {
    margin-left: .25rem !important;
    margin-right: .25rem !important;
  }

  .mx-sm-2 {
    margin-left: .5rem !important;
    margin-right: .5rem !important;
  }

  .mx-sm-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }

  .mx-sm-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }

  .mx-sm-5 {
    margin-left: 3rem !important;
    margin-right: 3rem !important;
  }

  .mx-sm-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }

  .my-sm-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }

  .my-sm-1 {
    margin-top: .25rem !important;
    margin-bottom: .25rem !important;
  }

  .my-sm-2 {
    margin-top: .5rem !important;
    margin-bottom: .5rem !important;
  }

  .my-sm-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }

  .my-sm-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }

  .my-sm-5 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }

  .my-sm-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }

  .mt-sm-0 {
    margin-top: 0 !important;
  }

  .mt-sm-1 {
    margin-top: .25rem !important;
  }

  .mt-sm-2 {
    margin-top: .5rem !important;
  }

  .mt-sm-3 {
    margin-top: 1rem !important;
  }

  .mt-sm-4 {
    margin-top: 1.5rem !important;
  }

  .mt-sm-5 {
    margin-top: 3rem !important;
  }

  .mt-sm-auto {
    margin-top: auto !important;
  }

  .me-sm-0 {
    margin-right: 0 !important;
  }

  .me-sm-1 {
    margin-right: .25rem !important;
  }

  .me-sm-2 {
    margin-right: .5rem !important;
  }

  .me-sm-3 {
    margin-right: 1rem !important;
  }

  .me-sm-4 {
    margin-right: 1.5rem !important;
  }

  .me-sm-5 {
    margin-right: 3rem !important;
  }

  .me-sm-auto {
    margin-right: auto !important;
  }

  .mb-sm-0 {
    margin-bottom: 0 !important;
  }

  .mb-sm-1 {
    margin-bottom: .25rem !important;
  }

  .mb-sm-2 {
    margin-bottom: .5rem !important;
  }

  .mb-sm-3 {
    margin-bottom: 1rem !important;
  }

  .mb-sm-4 {
    margin-bottom: 1.5rem !important;
  }

  .mb-sm-5 {
    margin-bottom: 3rem !important;
  }

  .mb-sm-auto {
    margin-bottom: auto !important;
  }

  .ms-sm-0 {
    margin-left: 0 !important;
  }

  .ms-sm-1 {
    margin-left: .25rem !important;
  }

  .ms-sm-2 {
    margin-left: .5rem !important;
  }

  .ms-sm-3 {
    margin-left: 1rem !important;
  }

  .ms-sm-4 {
    margin-left: 1.5rem !important;
  }

  .ms-sm-5 {
    margin-left: 3rem !important;
  }

  .ms-sm-auto {
    margin-left: auto !important;
  }

  .p-sm-0 {
    padding: 0 !important;
  }

  .p-sm-1 {
    padding: .25rem !important;
  }

  .p-sm-2 {
    padding: .5rem !important;
  }

  .p-sm-3 {
    padding: 1rem !important;
  }

  .p-sm-4 {
    padding: 1.5rem !important;
  }

  .p-sm-5 {
    padding: 3rem !important;
  }

  .px-sm-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  .px-sm-1 {
    padding-left: .25rem !important;
    padding-right: .25rem !important;
  }

  .px-sm-2 {
    padding-left: .5rem !important;
    padding-right: .5rem !important;
  }

  .px-sm-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }

  .px-sm-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }

  .px-sm-5 {
    padding-left: 3rem !important;
    padding-right: 3rem !important;
  }

  .py-sm-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }

  .py-sm-1 {
    padding-top: .25rem !important;
    padding-bottom: .25rem !important;
  }

  .py-sm-2 {
    padding-top: .5rem !important;
    padding-bottom: .5rem !important;
  }

  .py-sm-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }

  .py-sm-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }

  .py-sm-5 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }

  .pt-sm-0 {
    padding-top: 0 !important;
  }

  .pt-sm-1 {
    padding-top: .25rem !important;
  }

  .pt-sm-2 {
    padding-top: .5rem !important;
  }

  .pt-sm-3 {
    padding-top: 1rem !important;
  }

  .pt-sm-4 {
    padding-top: 1.5rem !important;
  }

  .pt-sm-5 {
    padding-top: 3rem !important;
  }

  .pe-sm-0 {
    padding-right: 0 !important;
  }

  .pe-sm-1 {
    padding-right: .25rem !important;
  }

  .pe-sm-2 {
    padding-right: .5rem !important;
  }

  .pe-sm-3 {
    padding-right: 1rem !important;
  }

  .pe-sm-4 {
    padding-right: 1.5rem !important;
  }

  .pe-sm-5 {
    padding-right: 3rem !important;
  }

  .pb-sm-0 {
    padding-bottom: 0 !important;
  }

  .pb-sm-1 {
    padding-bottom: .25rem !important;
  }

  .pb-sm-2 {
    padding-bottom: .5rem !important;
  }

  .pb-sm-3 {
    padding-bottom: 1rem !important;
  }

  .pb-sm-4 {
    padding-bottom: 1.5rem !important;
  }

  .pb-sm-5 {
    padding-bottom: 3rem !important;
  }

  .ps-sm-0 {
    padding-left: 0 !important;
  }

  .ps-sm-1 {
    padding-left: .25rem !important;
  }

  .ps-sm-2 {
    padding-left: .5rem !important;
  }

  .ps-sm-3 {
    padding-left: 1rem !important;
  }

  .ps-sm-4 {
    padding-left: 1.5rem !important;
  }

  .ps-sm-5 {
    padding-left: 3rem !important;
  }

  .gap-sm-0 {
    gap: 0 !important;
  }

  .gap-sm-1 {
    gap: .25rem !important;
  }

  .gap-sm-2 {
    gap: .5rem !important;
  }

  .gap-sm-3 {
    gap: 1rem !important;
  }

  .gap-sm-4 {
    gap: 1.5rem !important;
  }

  .gap-sm-5 {
    gap: 3rem !important;
  }

  .row-gap-sm-0 {
    row-gap: 0 !important;
  }

  .row-gap-sm-1 {
    row-gap: .25rem !important;
  }

  .row-gap-sm-2 {
    row-gap: .5rem !important;
  }

  .row-gap-sm-3 {
    row-gap: 1rem !important;
  }

  .row-gap-sm-4 {
    row-gap: 1.5rem !important;
  }

  .row-gap-sm-5 {
    row-gap: 3rem !important;
  }

  .column-gap-sm-0 {
    column-gap: 0 !important;
  }

  .column-gap-sm-1 {
    column-gap: .25rem !important;
  }

  .column-gap-sm-2 {
    column-gap: .5rem !important;
  }

  .column-gap-sm-3 {
    column-gap: 1rem !important;
  }

  .column-gap-sm-4 {
    column-gap: 1.5rem !important;
  }

  .column-gap-sm-5 {
    column-gap: 3rem !important;
  }

  .text-sm-start {
    text-align: left !important;
  }

  .text-sm-end {
    text-align: right !important;
  }

  .text-sm-center {
    text-align: center !important;
  }
}

@media (width >= 768px) {
  .float-md-start {
    float: left !important;
  }

  .float-md-end {
    float: right !important;
  }

  .float-md-none {
    float: none !important;
  }

  .object-fit-md-contain {
    object-fit: contain !important;
  }

  .object-fit-md-cover {
    object-fit: cover !important;
  }

  .object-fit-md-fill {
    object-fit: fill !important;
  }

  .object-fit-md-scale {
    object-fit: scale-down !important;
  }

  .object-fit-md-none {
    object-fit: none !important;
  }

  .d-md-inline {
    display: inline !important;
  }

  .d-md-inline-block {
    display: inline-block !important;
  }

  .d-md-block {
    display: block !important;
  }

  .d-md-grid {
    display: grid !important;
  }

  .d-md-inline-grid {
    display: inline-grid !important;
  }

  .d-md-table {
    display: table !important;
  }

  .d-md-table-row {
    display: table-row !important;
  }

  .d-md-table-cell {
    display: table-cell !important;
  }

  .d-md-flex {
    display: flex !important;
  }

  .d-md-inline-flex {
    display: inline-flex !important;
  }

  .d-md-none {
    display: none !important;
  }

  .flex-md-fill {
    flex: auto !important;
  }

  .flex-md-row {
    flex-direction: row !important;
  }

  .flex-md-column {
    flex-direction: column !important;
  }

  .flex-md-row-reverse {
    flex-direction: row-reverse !important;
  }

  .flex-md-column-reverse {
    flex-direction: column-reverse !important;
  }

  .flex-md-grow-0 {
    flex-grow: 0 !important;
  }

  .flex-md-grow-1 {
    flex-grow: 1 !important;
  }

  .flex-md-shrink-0 {
    flex-shrink: 0 !important;
  }

  .flex-md-shrink-1 {
    flex-shrink: 1 !important;
  }

  .flex-md-wrap {
    flex-wrap: wrap !important;
  }

  .flex-md-nowrap {
    flex-wrap: nowrap !important;
  }

  .flex-md-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }

  .justify-content-md-start {
    justify-content: flex-start !important;
  }

  .justify-content-md-end {
    justify-content: flex-end !important;
  }

  .justify-content-md-center {
    justify-content: center !important;
  }

  .justify-content-md-between {
    justify-content: space-between !important;
  }

  .justify-content-md-around {
    justify-content: space-around !important;
  }

  .justify-content-md-evenly {
    justify-content: space-evenly !important;
  }

  .align-items-md-start {
    align-items: flex-start !important;
  }

  .align-items-md-end {
    align-items: flex-end !important;
  }

  .align-items-md-center {
    align-items: center !important;
  }

  .align-items-md-baseline {
    align-items: baseline !important;
  }

  .align-items-md-stretch {
    align-items: stretch !important;
  }

  .align-content-md-start {
    align-content: flex-start !important;
  }

  .align-content-md-end {
    align-content: flex-end !important;
  }

  .align-content-md-center {
    align-content: center !important;
  }

  .align-content-md-between {
    align-content: space-between !important;
  }

  .align-content-md-around {
    align-content: space-around !important;
  }

  .align-content-md-stretch {
    align-content: stretch !important;
  }

  .align-self-md-auto {
    align-self: auto !important;
  }

  .align-self-md-start {
    align-self: flex-start !important;
  }

  .align-self-md-end {
    align-self: flex-end !important;
  }

  .align-self-md-center {
    align-self: center !important;
  }

  .align-self-md-baseline {
    align-self: baseline !important;
  }

  .align-self-md-stretch {
    align-self: stretch !important;
  }

  .order-md-first {
    order: -1 !important;
  }

  .order-md-0 {
    order: 0 !important;
  }

  .order-md-1 {
    order: 1 !important;
  }

  .order-md-2 {
    order: 2 !important;
  }

  .order-md-3 {
    order: 3 !important;
  }

  .order-md-4 {
    order: 4 !important;
  }

  .order-md-5 {
    order: 5 !important;
  }

  .order-md-last {
    order: 6 !important;
  }

  .m-md-0 {
    margin: 0 !important;
  }

  .m-md-1 {
    margin: .25rem !important;
  }

  .m-md-2 {
    margin: .5rem !important;
  }

  .m-md-3 {
    margin: 1rem !important;
  }

  .m-md-4 {
    margin: 1.5rem !important;
  }

  .m-md-5 {
    margin: 3rem !important;
  }

  .m-md-auto {
    margin: auto !important;
  }

  .mx-md-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .mx-md-1 {
    margin-left: .25rem !important;
    margin-right: .25rem !important;
  }

  .mx-md-2 {
    margin-left: .5rem !important;
    margin-right: .5rem !important;
  }

  .mx-md-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }

  .mx-md-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }

  .mx-md-5 {
    margin-left: 3rem !important;
    margin-right: 3rem !important;
  }

  .mx-md-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }

  .my-md-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }

  .my-md-1 {
    margin-top: .25rem !important;
    margin-bottom: .25rem !important;
  }

  .my-md-2 {
    margin-top: .5rem !important;
    margin-bottom: .5rem !important;
  }

  .my-md-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }

  .my-md-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }

  .my-md-5 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }

  .my-md-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }

  .mt-md-0 {
    margin-top: 0 !important;
  }

  .mt-md-1 {
    margin-top: .25rem !important;
  }

  .mt-md-2 {
    margin-top: .5rem !important;
  }

  .mt-md-3 {
    margin-top: 1rem !important;
  }

  .mt-md-4 {
    margin-top: 1.5rem !important;
  }

  .mt-md-5 {
    margin-top: 3rem !important;
  }

  .mt-md-auto {
    margin-top: auto !important;
  }

  .me-md-0 {
    margin-right: 0 !important;
  }

  .me-md-1 {
    margin-right: .25rem !important;
  }

  .me-md-2 {
    margin-right: .5rem !important;
  }

  .me-md-3 {
    margin-right: 1rem !important;
  }

  .me-md-4 {
    margin-right: 1.5rem !important;
  }

  .me-md-5 {
    margin-right: 3rem !important;
  }

  .me-md-auto {
    margin-right: auto !important;
  }

  .mb-md-0 {
    margin-bottom: 0 !important;
  }

  .mb-md-1 {
    margin-bottom: .25rem !important;
  }

  .mb-md-2 {
    margin-bottom: .5rem !important;
  }

  .mb-md-3 {
    margin-bottom: 1rem !important;
  }

  .mb-md-4 {
    margin-bottom: 1.5rem !important;
  }

  .mb-md-5 {
    margin-bottom: 3rem !important;
  }

  .mb-md-auto {
    margin-bottom: auto !important;
  }

  .ms-md-0 {
    margin-left: 0 !important;
  }

  .ms-md-1 {
    margin-left: .25rem !important;
  }

  .ms-md-2 {
    margin-left: .5rem !important;
  }

  .ms-md-3 {
    margin-left: 1rem !important;
  }

  .ms-md-4 {
    margin-left: 1.5rem !important;
  }

  .ms-md-5 {
    margin-left: 3rem !important;
  }

  .ms-md-auto {
    margin-left: auto !important;
  }

  .p-md-0 {
    padding: 0 !important;
  }

  .p-md-1 {
    padding: .25rem !important;
  }

  .p-md-2 {
    padding: .5rem !important;
  }

  .p-md-3 {
    padding: 1rem !important;
  }

  .p-md-4 {
    padding: 1.5rem !important;
  }

  .p-md-5 {
    padding: 3rem !important;
  }

  .px-md-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  .px-md-1 {
    padding-left: .25rem !important;
    padding-right: .25rem !important;
  }

  .px-md-2 {
    padding-left: .5rem !important;
    padding-right: .5rem !important;
  }

  .px-md-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }

  .px-md-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }

  .px-md-5 {
    padding-left: 3rem !important;
    padding-right: 3rem !important;
  }

  .py-md-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }

  .py-md-1 {
    padding-top: .25rem !important;
    padding-bottom: .25rem !important;
  }

  .py-md-2 {
    padding-top: .5rem !important;
    padding-bottom: .5rem !important;
  }

  .py-md-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }

  .py-md-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }

  .py-md-5 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }

  .pt-md-0 {
    padding-top: 0 !important;
  }

  .pt-md-1 {
    padding-top: .25rem !important;
  }

  .pt-md-2 {
    padding-top: .5rem !important;
  }

  .pt-md-3 {
    padding-top: 1rem !important;
  }

  .pt-md-4 {
    padding-top: 1.5rem !important;
  }

  .pt-md-5 {
    padding-top: 3rem !important;
  }

  .pe-md-0 {
    padding-right: 0 !important;
  }

  .pe-md-1 {
    padding-right: .25rem !important;
  }

  .pe-md-2 {
    padding-right: .5rem !important;
  }

  .pe-md-3 {
    padding-right: 1rem !important;
  }

  .pe-md-4 {
    padding-right: 1.5rem !important;
  }

  .pe-md-5 {
    padding-right: 3rem !important;
  }

  .pb-md-0 {
    padding-bottom: 0 !important;
  }

  .pb-md-1 {
    padding-bottom: .25rem !important;
  }

  .pb-md-2 {
    padding-bottom: .5rem !important;
  }

  .pb-md-3 {
    padding-bottom: 1rem !important;
  }

  .pb-md-4 {
    padding-bottom: 1.5rem !important;
  }

  .pb-md-5 {
    padding-bottom: 3rem !important;
  }

  .ps-md-0 {
    padding-left: 0 !important;
  }

  .ps-md-1 {
    padding-left: .25rem !important;
  }

  .ps-md-2 {
    padding-left: .5rem !important;
  }

  .ps-md-3 {
    padding-left: 1rem !important;
  }

  .ps-md-4 {
    padding-left: 1.5rem !important;
  }

  .ps-md-5 {
    padding-left: 3rem !important;
  }

  .gap-md-0 {
    gap: 0 !important;
  }

  .gap-md-1 {
    gap: .25rem !important;
  }

  .gap-md-2 {
    gap: .5rem !important;
  }

  .gap-md-3 {
    gap: 1rem !important;
  }

  .gap-md-4 {
    gap: 1.5rem !important;
  }

  .gap-md-5 {
    gap: 3rem !important;
  }

  .row-gap-md-0 {
    row-gap: 0 !important;
  }

  .row-gap-md-1 {
    row-gap: .25rem !important;
  }

  .row-gap-md-2 {
    row-gap: .5rem !important;
  }

  .row-gap-md-3 {
    row-gap: 1rem !important;
  }

  .row-gap-md-4 {
    row-gap: 1.5rem !important;
  }

  .row-gap-md-5 {
    row-gap: 3rem !important;
  }

  .column-gap-md-0 {
    column-gap: 0 !important;
  }

  .column-gap-md-1 {
    column-gap: .25rem !important;
  }

  .column-gap-md-2 {
    column-gap: .5rem !important;
  }

  .column-gap-md-3 {
    column-gap: 1rem !important;
  }

  .column-gap-md-4 {
    column-gap: 1.5rem !important;
  }

  .column-gap-md-5 {
    column-gap: 3rem !important;
  }

  .text-md-start {
    text-align: left !important;
  }

  .text-md-end {
    text-align: right !important;
  }

  .text-md-center {
    text-align: center !important;
  }
}

@media (width >= 992px) {
  .float-lg-start {
    float: left !important;
  }

  .float-lg-end {
    float: right !important;
  }

  .float-lg-none {
    float: none !important;
  }

  .object-fit-lg-contain {
    object-fit: contain !important;
  }

  .object-fit-lg-cover {
    object-fit: cover !important;
  }

  .object-fit-lg-fill {
    object-fit: fill !important;
  }

  .object-fit-lg-scale {
    object-fit: scale-down !important;
  }

  .object-fit-lg-none {
    object-fit: none !important;
  }

  .d-lg-inline {
    display: inline !important;
  }

  .d-lg-inline-block {
    display: inline-block !important;
  }

  .d-lg-block {
    display: block !important;
  }

  .d-lg-grid {
    display: grid !important;
  }

  .d-lg-inline-grid {
    display: inline-grid !important;
  }

  .d-lg-table {
    display: table !important;
  }

  .d-lg-table-row {
    display: table-row !important;
  }

  .d-lg-table-cell {
    display: table-cell !important;
  }

  .d-lg-flex {
    display: flex !important;
  }

  .d-lg-inline-flex {
    display: inline-flex !important;
  }

  .d-lg-none {
    display: none !important;
  }

  .flex-lg-fill {
    flex: auto !important;
  }

  .flex-lg-row {
    flex-direction: row !important;
  }

  .flex-lg-column {
    flex-direction: column !important;
  }

  .flex-lg-row-reverse {
    flex-direction: row-reverse !important;
  }

  .flex-lg-column-reverse {
    flex-direction: column-reverse !important;
  }

  .flex-lg-grow-0 {
    flex-grow: 0 !important;
  }

  .flex-lg-grow-1 {
    flex-grow: 1 !important;
  }

  .flex-lg-shrink-0 {
    flex-shrink: 0 !important;
  }

  .flex-lg-shrink-1 {
    flex-shrink: 1 !important;
  }

  .flex-lg-wrap {
    flex-wrap: wrap !important;
  }

  .flex-lg-nowrap {
    flex-wrap: nowrap !important;
  }

  .flex-lg-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }

  .justify-content-lg-start {
    justify-content: flex-start !important;
  }

  .justify-content-lg-end {
    justify-content: flex-end !important;
  }

  .justify-content-lg-center {
    justify-content: center !important;
  }

  .justify-content-lg-between {
    justify-content: space-between !important;
  }

  .justify-content-lg-around {
    justify-content: space-around !important;
  }

  .justify-content-lg-evenly {
    justify-content: space-evenly !important;
  }

  .align-items-lg-start {
    align-items: flex-start !important;
  }

  .align-items-lg-end {
    align-items: flex-end !important;
  }

  .align-items-lg-center {
    align-items: center !important;
  }

  .align-items-lg-baseline {
    align-items: baseline !important;
  }

  .align-items-lg-stretch {
    align-items: stretch !important;
  }

  .align-content-lg-start {
    align-content: flex-start !important;
  }

  .align-content-lg-end {
    align-content: flex-end !important;
  }

  .align-content-lg-center {
    align-content: center !important;
  }

  .align-content-lg-between {
    align-content: space-between !important;
  }

  .align-content-lg-around {
    align-content: space-around !important;
  }

  .align-content-lg-stretch {
    align-content: stretch !important;
  }

  .align-self-lg-auto {
    align-self: auto !important;
  }

  .align-self-lg-start {
    align-self: flex-start !important;
  }

  .align-self-lg-end {
    align-self: flex-end !important;
  }

  .align-self-lg-center {
    align-self: center !important;
  }

  .align-self-lg-baseline {
    align-self: baseline !important;
  }

  .align-self-lg-stretch {
    align-self: stretch !important;
  }

  .order-lg-first {
    order: -1 !important;
  }

  .order-lg-0 {
    order: 0 !important;
  }

  .order-lg-1 {
    order: 1 !important;
  }

  .order-lg-2 {
    order: 2 !important;
  }

  .order-lg-3 {
    order: 3 !important;
  }

  .order-lg-4 {
    order: 4 !important;
  }

  .order-lg-5 {
    order: 5 !important;
  }

  .order-lg-last {
    order: 6 !important;
  }

  .m-lg-0 {
    margin: 0 !important;
  }

  .m-lg-1 {
    margin: .25rem !important;
  }

  .m-lg-2 {
    margin: .5rem !important;
  }

  .m-lg-3 {
    margin: 1rem !important;
  }

  .m-lg-4 {
    margin: 1.5rem !important;
  }

  .m-lg-5 {
    margin: 3rem !important;
  }

  .m-lg-auto {
    margin: auto !important;
  }

  .mx-lg-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .mx-lg-1 {
    margin-left: .25rem !important;
    margin-right: .25rem !important;
  }

  .mx-lg-2 {
    margin-left: .5rem !important;
    margin-right: .5rem !important;
  }

  .mx-lg-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }

  .mx-lg-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }

  .mx-lg-5 {
    margin-left: 3rem !important;
    margin-right: 3rem !important;
  }

  .mx-lg-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }

  .my-lg-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }

  .my-lg-1 {
    margin-top: .25rem !important;
    margin-bottom: .25rem !important;
  }

  .my-lg-2 {
    margin-top: .5rem !important;
    margin-bottom: .5rem !important;
  }

  .my-lg-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }

  .my-lg-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }

  .my-lg-5 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }

  .my-lg-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }

  .mt-lg-0 {
    margin-top: 0 !important;
  }

  .mt-lg-1 {
    margin-top: .25rem !important;
  }

  .mt-lg-2 {
    margin-top: .5rem !important;
  }

  .mt-lg-3 {
    margin-top: 1rem !important;
  }

  .mt-lg-4 {
    margin-top: 1.5rem !important;
  }

  .mt-lg-5 {
    margin-top: 3rem !important;
  }

  .mt-lg-auto {
    margin-top: auto !important;
  }

  .me-lg-0 {
    margin-right: 0 !important;
  }

  .me-lg-1 {
    margin-right: .25rem !important;
  }

  .me-lg-2 {
    margin-right: .5rem !important;
  }

  .me-lg-3 {
    margin-right: 1rem !important;
  }

  .me-lg-4 {
    margin-right: 1.5rem !important;
  }

  .me-lg-5 {
    margin-right: 3rem !important;
  }

  .me-lg-auto {
    margin-right: auto !important;
  }

  .mb-lg-0 {
    margin-bottom: 0 !important;
  }

  .mb-lg-1 {
    margin-bottom: .25rem !important;
  }

  .mb-lg-2 {
    margin-bottom: .5rem !important;
  }

  .mb-lg-3 {
    margin-bottom: 1rem !important;
  }

  .mb-lg-4 {
    margin-bottom: 1.5rem !important;
  }

  .mb-lg-5 {
    margin-bottom: 3rem !important;
  }

  .mb-lg-auto {
    margin-bottom: auto !important;
  }

  .ms-lg-0 {
    margin-left: 0 !important;
  }

  .ms-lg-1 {
    margin-left: .25rem !important;
  }

  .ms-lg-2 {
    margin-left: .5rem !important;
  }

  .ms-lg-3 {
    margin-left: 1rem !important;
  }

  .ms-lg-4 {
    margin-left: 1.5rem !important;
  }

  .ms-lg-5 {
    margin-left: 3rem !important;
  }

  .ms-lg-auto {
    margin-left: auto !important;
  }

  .p-lg-0 {
    padding: 0 !important;
  }

  .p-lg-1 {
    padding: .25rem !important;
  }

  .p-lg-2 {
    padding: .5rem !important;
  }

  .p-lg-3 {
    padding: 1rem !important;
  }

  .p-lg-4 {
    padding: 1.5rem !important;
  }

  .p-lg-5 {
    padding: 3rem !important;
  }

  .px-lg-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  .px-lg-1 {
    padding-left: .25rem !important;
    padding-right: .25rem !important;
  }

  .px-lg-2 {
    padding-left: .5rem !important;
    padding-right: .5rem !important;
  }

  .px-lg-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }

  .px-lg-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }

  .px-lg-5 {
    padding-left: 3rem !important;
    padding-right: 3rem !important;
  }

  .py-lg-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }

  .py-lg-1 {
    padding-top: .25rem !important;
    padding-bottom: .25rem !important;
  }

  .py-lg-2 {
    padding-top: .5rem !important;
    padding-bottom: .5rem !important;
  }

  .py-lg-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }

  .py-lg-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }

  .py-lg-5 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }

  .pt-lg-0 {
    padding-top: 0 !important;
  }

  .pt-lg-1 {
    padding-top: .25rem !important;
  }

  .pt-lg-2 {
    padding-top: .5rem !important;
  }

  .pt-lg-3 {
    padding-top: 1rem !important;
  }

  .pt-lg-4 {
    padding-top: 1.5rem !important;
  }

  .pt-lg-5 {
    padding-top: 3rem !important;
  }

  .pe-lg-0 {
    padding-right: 0 !important;
  }

  .pe-lg-1 {
    padding-right: .25rem !important;
  }

  .pe-lg-2 {
    padding-right: .5rem !important;
  }

  .pe-lg-3 {
    padding-right: 1rem !important;
  }

  .pe-lg-4 {
    padding-right: 1.5rem !important;
  }

  .pe-lg-5 {
    padding-right: 3rem !important;
  }

  .pb-lg-0 {
    padding-bottom: 0 !important;
  }

  .pb-lg-1 {
    padding-bottom: .25rem !important;
  }

  .pb-lg-2 {
    padding-bottom: .5rem !important;
  }

  .pb-lg-3 {
    padding-bottom: 1rem !important;
  }

  .pb-lg-4 {
    padding-bottom: 1.5rem !important;
  }

  .pb-lg-5 {
    padding-bottom: 3rem !important;
  }

  .ps-lg-0 {
    padding-left: 0 !important;
  }

  .ps-lg-1 {
    padding-left: .25rem !important;
  }

  .ps-lg-2 {
    padding-left: .5rem !important;
  }

  .ps-lg-3 {
    padding-left: 1rem !important;
  }

  .ps-lg-4 {
    padding-left: 1.5rem !important;
  }

  .ps-lg-5 {
    padding-left: 3rem !important;
  }

  .gap-lg-0 {
    gap: 0 !important;
  }

  .gap-lg-1 {
    gap: .25rem !important;
  }

  .gap-lg-2 {
    gap: .5rem !important;
  }

  .gap-lg-3 {
    gap: 1rem !important;
  }

  .gap-lg-4 {
    gap: 1.5rem !important;
  }

  .gap-lg-5 {
    gap: 3rem !important;
  }

  .row-gap-lg-0 {
    row-gap: 0 !important;
  }

  .row-gap-lg-1 {
    row-gap: .25rem !important;
  }

  .row-gap-lg-2 {
    row-gap: .5rem !important;
  }

  .row-gap-lg-3 {
    row-gap: 1rem !important;
  }

  .row-gap-lg-4 {
    row-gap: 1.5rem !important;
  }

  .row-gap-lg-5 {
    row-gap: 3rem !important;
  }

  .column-gap-lg-0 {
    column-gap: 0 !important;
  }

  .column-gap-lg-1 {
    column-gap: .25rem !important;
  }

  .column-gap-lg-2 {
    column-gap: .5rem !important;
  }

  .column-gap-lg-3 {
    column-gap: 1rem !important;
  }

  .column-gap-lg-4 {
    column-gap: 1.5rem !important;
  }

  .column-gap-lg-5 {
    column-gap: 3rem !important;
  }

  .text-lg-start {
    text-align: left !important;
  }

  .text-lg-end {
    text-align: right !important;
  }

  .text-lg-center {
    text-align: center !important;
  }
}

@media (width >= 1200px) {
  .float-xl-start {
    float: left !important;
  }

  .float-xl-end {
    float: right !important;
  }

  .float-xl-none {
    float: none !important;
  }

  .object-fit-xl-contain {
    object-fit: contain !important;
  }

  .object-fit-xl-cover {
    object-fit: cover !important;
  }

  .object-fit-xl-fill {
    object-fit: fill !important;
  }

  .object-fit-xl-scale {
    object-fit: scale-down !important;
  }

  .object-fit-xl-none {
    object-fit: none !important;
  }

  .d-xl-inline {
    display: inline !important;
  }

  .d-xl-inline-block {
    display: inline-block !important;
  }

  .d-xl-block {
    display: block !important;
  }

  .d-xl-grid {
    display: grid !important;
  }

  .d-xl-inline-grid {
    display: inline-grid !important;
  }

  .d-xl-table {
    display: table !important;
  }

  .d-xl-table-row {
    display: table-row !important;
  }

  .d-xl-table-cell {
    display: table-cell !important;
  }

  .d-xl-flex {
    display: flex !important;
  }

  .d-xl-inline-flex {
    display: inline-flex !important;
  }

  .d-xl-none {
    display: none !important;
  }

  .flex-xl-fill {
    flex: auto !important;
  }

  .flex-xl-row {
    flex-direction: row !important;
  }

  .flex-xl-column {
    flex-direction: column !important;
  }

  .flex-xl-row-reverse {
    flex-direction: row-reverse !important;
  }

  .flex-xl-column-reverse {
    flex-direction: column-reverse !important;
  }

  .flex-xl-grow-0 {
    flex-grow: 0 !important;
  }

  .flex-xl-grow-1 {
    flex-grow: 1 !important;
  }

  .flex-xl-shrink-0 {
    flex-shrink: 0 !important;
  }

  .flex-xl-shrink-1 {
    flex-shrink: 1 !important;
  }

  .flex-xl-wrap {
    flex-wrap: wrap !important;
  }

  .flex-xl-nowrap {
    flex-wrap: nowrap !important;
  }

  .flex-xl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }

  .justify-content-xl-start {
    justify-content: flex-start !important;
  }

  .justify-content-xl-end {
    justify-content: flex-end !important;
  }

  .justify-content-xl-center {
    justify-content: center !important;
  }

  .justify-content-xl-between {
    justify-content: space-between !important;
  }

  .justify-content-xl-around {
    justify-content: space-around !important;
  }

  .justify-content-xl-evenly {
    justify-content: space-evenly !important;
  }

  .align-items-xl-start {
    align-items: flex-start !important;
  }

  .align-items-xl-end {
    align-items: flex-end !important;
  }

  .align-items-xl-center {
    align-items: center !important;
  }

  .align-items-xl-baseline {
    align-items: baseline !important;
  }

  .align-items-xl-stretch {
    align-items: stretch !important;
  }

  .align-content-xl-start {
    align-content: flex-start !important;
  }

  .align-content-xl-end {
    align-content: flex-end !important;
  }

  .align-content-xl-center {
    align-content: center !important;
  }

  .align-content-xl-between {
    align-content: space-between !important;
  }

  .align-content-xl-around {
    align-content: space-around !important;
  }

  .align-content-xl-stretch {
    align-content: stretch !important;
  }

  .align-self-xl-auto {
    align-self: auto !important;
  }

  .align-self-xl-start {
    align-self: flex-start !important;
  }

  .align-self-xl-end {
    align-self: flex-end !important;
  }

  .align-self-xl-center {
    align-self: center !important;
  }

  .align-self-xl-baseline {
    align-self: baseline !important;
  }

  .align-self-xl-stretch {
    align-self: stretch !important;
  }

  .order-xl-first {
    order: -1 !important;
  }

  .order-xl-0 {
    order: 0 !important;
  }

  .order-xl-1 {
    order: 1 !important;
  }

  .order-xl-2 {
    order: 2 !important;
  }

  .order-xl-3 {
    order: 3 !important;
  }

  .order-xl-4 {
    order: 4 !important;
  }

  .order-xl-5 {
    order: 5 !important;
  }

  .order-xl-last {
    order: 6 !important;
  }

  .m-xl-0 {
    margin: 0 !important;
  }

  .m-xl-1 {
    margin: .25rem !important;
  }

  .m-xl-2 {
    margin: .5rem !important;
  }

  .m-xl-3 {
    margin: 1rem !important;
  }

  .m-xl-4 {
    margin: 1.5rem !important;
  }

  .m-xl-5 {
    margin: 3rem !important;
  }

  .m-xl-auto {
    margin: auto !important;
  }

  .mx-xl-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .mx-xl-1 {
    margin-left: .25rem !important;
    margin-right: .25rem !important;
  }

  .mx-xl-2 {
    margin-left: .5rem !important;
    margin-right: .5rem !important;
  }

  .mx-xl-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }

  .mx-xl-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }

  .mx-xl-5 {
    margin-left: 3rem !important;
    margin-right: 3rem !important;
  }

  .mx-xl-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }

  .my-xl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }

  .my-xl-1 {
    margin-top: .25rem !important;
    margin-bottom: .25rem !important;
  }

  .my-xl-2 {
    margin-top: .5rem !important;
    margin-bottom: .5rem !important;
  }

  .my-xl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }

  .my-xl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }

  .my-xl-5 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }

  .my-xl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }

  .mt-xl-0 {
    margin-top: 0 !important;
  }

  .mt-xl-1 {
    margin-top: .25rem !important;
  }

  .mt-xl-2 {
    margin-top: .5rem !important;
  }

  .mt-xl-3 {
    margin-top: 1rem !important;
  }

  .mt-xl-4 {
    margin-top: 1.5rem !important;
  }

  .mt-xl-5 {
    margin-top: 3rem !important;
  }

  .mt-xl-auto {
    margin-top: auto !important;
  }

  .me-xl-0 {
    margin-right: 0 !important;
  }

  .me-xl-1 {
    margin-right: .25rem !important;
  }

  .me-xl-2 {
    margin-right: .5rem !important;
  }

  .me-xl-3 {
    margin-right: 1rem !important;
  }

  .me-xl-4 {
    margin-right: 1.5rem !important;
  }

  .me-xl-5 {
    margin-right: 3rem !important;
  }

  .me-xl-auto {
    margin-right: auto !important;
  }

  .mb-xl-0 {
    margin-bottom: 0 !important;
  }

  .mb-xl-1 {
    margin-bottom: .25rem !important;
  }

  .mb-xl-2 {
    margin-bottom: .5rem !important;
  }

  .mb-xl-3 {
    margin-bottom: 1rem !important;
  }

  .mb-xl-4 {
    margin-bottom: 1.5rem !important;
  }

  .mb-xl-5 {
    margin-bottom: 3rem !important;
  }

  .mb-xl-auto {
    margin-bottom: auto !important;
  }

  .ms-xl-0 {
    margin-left: 0 !important;
  }

  .ms-xl-1 {
    margin-left: .25rem !important;
  }

  .ms-xl-2 {
    margin-left: .5rem !important;
  }

  .ms-xl-3 {
    margin-left: 1rem !important;
  }

  .ms-xl-4 {
    margin-left: 1.5rem !important;
  }

  .ms-xl-5 {
    margin-left: 3rem !important;
  }

  .ms-xl-auto {
    margin-left: auto !important;
  }

  .p-xl-0 {
    padding: 0 !important;
  }

  .p-xl-1 {
    padding: .25rem !important;
  }

  .p-xl-2 {
    padding: .5rem !important;
  }

  .p-xl-3 {
    padding: 1rem !important;
  }

  .p-xl-4 {
    padding: 1.5rem !important;
  }

  .p-xl-5 {
    padding: 3rem !important;
  }

  .px-xl-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  .px-xl-1 {
    padding-left: .25rem !important;
    padding-right: .25rem !important;
  }

  .px-xl-2 {
    padding-left: .5rem !important;
    padding-right: .5rem !important;
  }

  .px-xl-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }

  .px-xl-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }

  .px-xl-5 {
    padding-left: 3rem !important;
    padding-right: 3rem !important;
  }

  .py-xl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }

  .py-xl-1 {
    padding-top: .25rem !important;
    padding-bottom: .25rem !important;
  }

  .py-xl-2 {
    padding-top: .5rem !important;
    padding-bottom: .5rem !important;
  }

  .py-xl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }

  .py-xl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }

  .py-xl-5 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }

  .pt-xl-0 {
    padding-top: 0 !important;
  }

  .pt-xl-1 {
    padding-top: .25rem !important;
  }

  .pt-xl-2 {
    padding-top: .5rem !important;
  }

  .pt-xl-3 {
    padding-top: 1rem !important;
  }

  .pt-xl-4 {
    padding-top: 1.5rem !important;
  }

  .pt-xl-5 {
    padding-top: 3rem !important;
  }

  .pe-xl-0 {
    padding-right: 0 !important;
  }

  .pe-xl-1 {
    padding-right: .25rem !important;
  }

  .pe-xl-2 {
    padding-right: .5rem !important;
  }

  .pe-xl-3 {
    padding-right: 1rem !important;
  }

  .pe-xl-4 {
    padding-right: 1.5rem !important;
  }

  .pe-xl-5 {
    padding-right: 3rem !important;
  }

  .pb-xl-0 {
    padding-bottom: 0 !important;
  }

  .pb-xl-1 {
    padding-bottom: .25rem !important;
  }

  .pb-xl-2 {
    padding-bottom: .5rem !important;
  }

  .pb-xl-3 {
    padding-bottom: 1rem !important;
  }

  .pb-xl-4 {
    padding-bottom: 1.5rem !important;
  }

  .pb-xl-5 {
    padding-bottom: 3rem !important;
  }

  .ps-xl-0 {
    padding-left: 0 !important;
  }

  .ps-xl-1 {
    padding-left: .25rem !important;
  }

  .ps-xl-2 {
    padding-left: .5rem !important;
  }

  .ps-xl-3 {
    padding-left: 1rem !important;
  }

  .ps-xl-4 {
    padding-left: 1.5rem !important;
  }

  .ps-xl-5 {
    padding-left: 3rem !important;
  }

  .gap-xl-0 {
    gap: 0 !important;
  }

  .gap-xl-1 {
    gap: .25rem !important;
  }

  .gap-xl-2 {
    gap: .5rem !important;
  }

  .gap-xl-3 {
    gap: 1rem !important;
  }

  .gap-xl-4 {
    gap: 1.5rem !important;
  }

  .gap-xl-5 {
    gap: 3rem !important;
  }

  .row-gap-xl-0 {
    row-gap: 0 !important;
  }

  .row-gap-xl-1 {
    row-gap: .25rem !important;
  }

  .row-gap-xl-2 {
    row-gap: .5rem !important;
  }

  .row-gap-xl-3 {
    row-gap: 1rem !important;
  }

  .row-gap-xl-4 {
    row-gap: 1.5rem !important;
  }

  .row-gap-xl-5 {
    row-gap: 3rem !important;
  }

  .column-gap-xl-0 {
    column-gap: 0 !important;
  }

  .column-gap-xl-1 {
    column-gap: .25rem !important;
  }

  .column-gap-xl-2 {
    column-gap: .5rem !important;
  }

  .column-gap-xl-3 {
    column-gap: 1rem !important;
  }

  .column-gap-xl-4 {
    column-gap: 1.5rem !important;
  }

  .column-gap-xl-5 {
    column-gap: 3rem !important;
  }

  .text-xl-start {
    text-align: left !important;
  }

  .text-xl-end {
    text-align: right !important;
  }

  .text-xl-center {
    text-align: center !important;
  }
}

@media (width >= 1400px) {
  .float-xxl-start {
    float: left !important;
  }

  .float-xxl-end {
    float: right !important;
  }

  .float-xxl-none {
    float: none !important;
  }

  .object-fit-xxl-contain {
    object-fit: contain !important;
  }

  .object-fit-xxl-cover {
    object-fit: cover !important;
  }

  .object-fit-xxl-fill {
    object-fit: fill !important;
  }

  .object-fit-xxl-scale {
    object-fit: scale-down !important;
  }

  .object-fit-xxl-none {
    object-fit: none !important;
  }

  .d-xxl-inline {
    display: inline !important;
  }

  .d-xxl-inline-block {
    display: inline-block !important;
  }

  .d-xxl-block {
    display: block !important;
  }

  .d-xxl-grid {
    display: grid !important;
  }

  .d-xxl-inline-grid {
    display: inline-grid !important;
  }

  .d-xxl-table {
    display: table !important;
  }

  .d-xxl-table-row {
    display: table-row !important;
  }

  .d-xxl-table-cell {
    display: table-cell !important;
  }

  .d-xxl-flex {
    display: flex !important;
  }

  .d-xxl-inline-flex {
    display: inline-flex !important;
  }

  .d-xxl-none {
    display: none !important;
  }

  .flex-xxl-fill {
    flex: auto !important;
  }

  .flex-xxl-row {
    flex-direction: row !important;
  }

  .flex-xxl-column {
    flex-direction: column !important;
  }

  .flex-xxl-row-reverse {
    flex-direction: row-reverse !important;
  }

  .flex-xxl-column-reverse {
    flex-direction: column-reverse !important;
  }

  .flex-xxl-grow-0 {
    flex-grow: 0 !important;
  }

  .flex-xxl-grow-1 {
    flex-grow: 1 !important;
  }

  .flex-xxl-shrink-0 {
    flex-shrink: 0 !important;
  }

  .flex-xxl-shrink-1 {
    flex-shrink: 1 !important;
  }

  .flex-xxl-wrap {
    flex-wrap: wrap !important;
  }

  .flex-xxl-nowrap {
    flex-wrap: nowrap !important;
  }

  .flex-xxl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }

  .justify-content-xxl-start {
    justify-content: flex-start !important;
  }

  .justify-content-xxl-end {
    justify-content: flex-end !important;
  }

  .justify-content-xxl-center {
    justify-content: center !important;
  }

  .justify-content-xxl-between {
    justify-content: space-between !important;
  }

  .justify-content-xxl-around {
    justify-content: space-around !important;
  }

  .justify-content-xxl-evenly {
    justify-content: space-evenly !important;
  }

  .align-items-xxl-start {
    align-items: flex-start !important;
  }

  .align-items-xxl-end {
    align-items: flex-end !important;
  }

  .align-items-xxl-center {
    align-items: center !important;
  }

  .align-items-xxl-baseline {
    align-items: baseline !important;
  }

  .align-items-xxl-stretch {
    align-items: stretch !important;
  }

  .align-content-xxl-start {
    align-content: flex-start !important;
  }

  .align-content-xxl-end {
    align-content: flex-end !important;
  }

  .align-content-xxl-center {
    align-content: center !important;
  }

  .align-content-xxl-between {
    align-content: space-between !important;
  }

  .align-content-xxl-around {
    align-content: space-around !important;
  }

  .align-content-xxl-stretch {
    align-content: stretch !important;
  }

  .align-self-xxl-auto {
    align-self: auto !important;
  }

  .align-self-xxl-start {
    align-self: flex-start !important;
  }

  .align-self-xxl-end {
    align-self: flex-end !important;
  }

  .align-self-xxl-center {
    align-self: center !important;
  }

  .align-self-xxl-baseline {
    align-self: baseline !important;
  }

  .align-self-xxl-stretch {
    align-self: stretch !important;
  }

  .order-xxl-first {
    order: -1 !important;
  }

  .order-xxl-0 {
    order: 0 !important;
  }

  .order-xxl-1 {
    order: 1 !important;
  }

  .order-xxl-2 {
    order: 2 !important;
  }

  .order-xxl-3 {
    order: 3 !important;
  }

  .order-xxl-4 {
    order: 4 !important;
  }

  .order-xxl-5 {
    order: 5 !important;
  }

  .order-xxl-last {
    order: 6 !important;
  }

  .m-xxl-0 {
    margin: 0 !important;
  }

  .m-xxl-1 {
    margin: .25rem !important;
  }

  .m-xxl-2 {
    margin: .5rem !important;
  }

  .m-xxl-3 {
    margin: 1rem !important;
  }

  .m-xxl-4 {
    margin: 1.5rem !important;
  }

  .m-xxl-5 {
    margin: 3rem !important;
  }

  .m-xxl-auto {
    margin: auto !important;
  }

  .mx-xxl-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .mx-xxl-1 {
    margin-left: .25rem !important;
    margin-right: .25rem !important;
  }

  .mx-xxl-2 {
    margin-left: .5rem !important;
    margin-right: .5rem !important;
  }

  .mx-xxl-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }

  .mx-xxl-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }

  .mx-xxl-5 {
    margin-left: 3rem !important;
    margin-right: 3rem !important;
  }

  .mx-xxl-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }

  .my-xxl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }

  .my-xxl-1 {
    margin-top: .25rem !important;
    margin-bottom: .25rem !important;
  }

  .my-xxl-2 {
    margin-top: .5rem !important;
    margin-bottom: .5rem !important;
  }

  .my-xxl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }

  .my-xxl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }

  .my-xxl-5 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }

  .my-xxl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }

  .mt-xxl-0 {
    margin-top: 0 !important;
  }

  .mt-xxl-1 {
    margin-top: .25rem !important;
  }

  .mt-xxl-2 {
    margin-top: .5rem !important;
  }

  .mt-xxl-3 {
    margin-top: 1rem !important;
  }

  .mt-xxl-4 {
    margin-top: 1.5rem !important;
  }

  .mt-xxl-5 {
    margin-top: 3rem !important;
  }

  .mt-xxl-auto {
    margin-top: auto !important;
  }

  .me-xxl-0 {
    margin-right: 0 !important;
  }

  .me-xxl-1 {
    margin-right: .25rem !important;
  }

  .me-xxl-2 {
    margin-right: .5rem !important;
  }

  .me-xxl-3 {
    margin-right: 1rem !important;
  }

  .me-xxl-4 {
    margin-right: 1.5rem !important;
  }

  .me-xxl-5 {
    margin-right: 3rem !important;
  }

  .me-xxl-auto {
    margin-right: auto !important;
  }

  .mb-xxl-0 {
    margin-bottom: 0 !important;
  }

  .mb-xxl-1 {
    margin-bottom: .25rem !important;
  }

  .mb-xxl-2 {
    margin-bottom: .5rem !important;
  }

  .mb-xxl-3 {
    margin-bottom: 1rem !important;
  }

  .mb-xxl-4 {
    margin-bottom: 1.5rem !important;
  }

  .mb-xxl-5 {
    margin-bottom: 3rem !important;
  }

  .mb-xxl-auto {
    margin-bottom: auto !important;
  }

  .ms-xxl-0 {
    margin-left: 0 !important;
  }

  .ms-xxl-1 {
    margin-left: .25rem !important;
  }

  .ms-xxl-2 {
    margin-left: .5rem !important;
  }

  .ms-xxl-3 {
    margin-left: 1rem !important;
  }

  .ms-xxl-4 {
    margin-left: 1.5rem !important;
  }

  .ms-xxl-5 {
    margin-left: 3rem !important;
  }

  .ms-xxl-auto {
    margin-left: auto !important;
  }

  .p-xxl-0 {
    padding: 0 !important;
  }

  .p-xxl-1 {
    padding: .25rem !important;
  }

  .p-xxl-2 {
    padding: .5rem !important;
  }

  .p-xxl-3 {
    padding: 1rem !important;
  }

  .p-xxl-4 {
    padding: 1.5rem !important;
  }

  .p-xxl-5 {
    padding: 3rem !important;
  }

  .px-xxl-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  .px-xxl-1 {
    padding-left: .25rem !important;
    padding-right: .25rem !important;
  }

  .px-xxl-2 {
    padding-left: .5rem !important;
    padding-right: .5rem !important;
  }

  .px-xxl-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }

  .px-xxl-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }

  .px-xxl-5 {
    padding-left: 3rem !important;
    padding-right: 3rem !important;
  }

  .py-xxl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }

  .py-xxl-1 {
    padding-top: .25rem !important;
    padding-bottom: .25rem !important;
  }

  .py-xxl-2 {
    padding-top: .5rem !important;
    padding-bottom: .5rem !important;
  }

  .py-xxl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }

  .py-xxl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }

  .py-xxl-5 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }

  .pt-xxl-0 {
    padding-top: 0 !important;
  }

  .pt-xxl-1 {
    padding-top: .25rem !important;
  }

  .pt-xxl-2 {
    padding-top: .5rem !important;
  }

  .pt-xxl-3 {
    padding-top: 1rem !important;
  }

  .pt-xxl-4 {
    padding-top: 1.5rem !important;
  }

  .pt-xxl-5 {
    padding-top: 3rem !important;
  }

  .pe-xxl-0 {
    padding-right: 0 !important;
  }

  .pe-xxl-1 {
    padding-right: .25rem !important;
  }

  .pe-xxl-2 {
    padding-right: .5rem !important;
  }

  .pe-xxl-3 {
    padding-right: 1rem !important;
  }

  .pe-xxl-4 {
    padding-right: 1.5rem !important;
  }

  .pe-xxl-5 {
    padding-right: 3rem !important;
  }

  .pb-xxl-0 {
    padding-bottom: 0 !important;
  }

  .pb-xxl-1 {
    padding-bottom: .25rem !important;
  }

  .pb-xxl-2 {
    padding-bottom: .5rem !important;
  }

  .pb-xxl-3 {
    padding-bottom: 1rem !important;
  }

  .pb-xxl-4 {
    padding-bottom: 1.5rem !important;
  }

  .pb-xxl-5 {
    padding-bottom: 3rem !important;
  }

  .ps-xxl-0 {
    padding-left: 0 !important;
  }

  .ps-xxl-1 {
    padding-left: .25rem !important;
  }

  .ps-xxl-2 {
    padding-left: .5rem !important;
  }

  .ps-xxl-3 {
    padding-left: 1rem !important;
  }

  .ps-xxl-4 {
    padding-left: 1.5rem !important;
  }

  .ps-xxl-5 {
    padding-left: 3rem !important;
  }

  .gap-xxl-0 {
    gap: 0 !important;
  }

  .gap-xxl-1 {
    gap: .25rem !important;
  }

  .gap-xxl-2 {
    gap: .5rem !important;
  }

  .gap-xxl-3 {
    gap: 1rem !important;
  }

  .gap-xxl-4 {
    gap: 1.5rem !important;
  }

  .gap-xxl-5 {
    gap: 3rem !important;
  }

  .row-gap-xxl-0 {
    row-gap: 0 !important;
  }

  .row-gap-xxl-1 {
    row-gap: .25rem !important;
  }

  .row-gap-xxl-2 {
    row-gap: .5rem !important;
  }

  .row-gap-xxl-3 {
    row-gap: 1rem !important;
  }

  .row-gap-xxl-4 {
    row-gap: 1.5rem !important;
  }

  .row-gap-xxl-5 {
    row-gap: 3rem !important;
  }

  .column-gap-xxl-0 {
    column-gap: 0 !important;
  }

  .column-gap-xxl-1 {
    column-gap: .25rem !important;
  }

  .column-gap-xxl-2 {
    column-gap: .5rem !important;
  }

  .column-gap-xxl-3 {
    column-gap: 1rem !important;
  }

  .column-gap-xxl-4 {
    column-gap: 1.5rem !important;
  }

  .column-gap-xxl-5 {
    column-gap: 3rem !important;
  }

  .text-xxl-start {
    text-align: left !important;
  }

  .text-xxl-end {
    text-align: right !important;
  }

  .text-xxl-center {
    text-align: center !important;
  }
}

@media (width >= 1200px) {
  .fs-1 {
    font-size: 1.65rem !important;
  }

  .fs-2 {
    font-size: 1.5rem !important;
  }
}

@media print {
  .d-print-inline {
    display: inline !important;
  }

  .d-print-inline-block {
    display: inline-block !important;
  }

  .d-print-block {
    display: block !important;
  }

  .d-print-grid {
    display: grid !important;
  }

  .d-print-inline-grid {
    display: inline-grid !important;
  }

  .d-print-table {
    display: table !important;
  }

  .d-print-table-row {
    display: table-row !important;
  }

  .d-print-table-cell {
    display: table-cell !important;
  }

  .d-print-flex {
    display: flex !important;
  }

  .d-print-inline-flex {
    display: inline-flex !important;
  }

  .d-print-none {
    display: none !important;
  }
}

html {
  height: 100%;
}

html.a11y_font {
  font-size: 18px;
}

body {
  text-align: start;
  flex-direction: column;
  min-height: 100%;
  margin: 0;
  padding: 0;
  display: flex;
}

body.admin, body#subhead-container {
  background-color: var(--admin-background);
}

body.monochrome {
  filter: grayscale();
}

body.monochrome:after {
  z-index: -1;
  content: "";
  background: var(--template-bg-dark-5);
  width: 100%;
  height: 100%;
  position: fixed;
  top: 0;
  left: 0;
}

body.a11y_contrast {
  filter: contrast(115%);
}

body.monochrome.a11y_contrast {
  filter: grayscale() contrast(115%);
}

body.a11y_highlight a, body.a11y_highlight .header-item-content {
  border: 1px dotted red;
  padding: 3px;
  text-decoration: underline !important;
}

body.a11y_highlight .joomlaversion {
  border: none;
  padding: 0;
  text-decoration: none !important;
}

body.a11y_highlight a.page-link {
  padding: .375rem .75rem;
}

h1, .h1, h2, .h2, h3, .h3, h4, .h4, h5, .h5, h6, .h6 {
  font-weight: 700;
}

h1, .h1 {
  font-weight: 400;
}

small, .small {
  font-size: .8rem;
}

legend {
  font-size: 1.2rem;
  font-weight: 700;
}

.js-focus-visible :focus:not(.focus-visible) {
  outline: none;
}

a.focus-visible, button.focus-visible, input.focus-visible, textarea.focus-visible {
  outline: auto 2px var(--focus);
}

.break-word {
  word-break: break-all;
  word-wrap: break-word;
}

label, caption {
  text-align: start;
}

.logo img path {
  fill: var(--template-text-dark);
}

.container-main, .system-debug {
  padding-bottom: 50px;
}

body .container-main {
  background: var(--main-bg);
  order: 1;
  padding-left: 0;
  padding-right: 0;
  position: relative;
}

.content {
  padding: 1rem;
}

@media (width >= 768px) {
  .content {
    padding: 0 2rem;
  }

  .subhead + .content {
    padding-top: 0;
  }
}

body:not(.contentpane) .main-card {
  background: var(--main-card-bg);
  box-shadow: var(--atum-box-shadow);
  border-radius: .25rem;
}

.row-selected {
  background-color: var(--toolbar-bg);
}

.input-group input {
  min-width: 220px;
}

.item-associations {
  padding: 0;
}

.item-associations li {
  margin-bottom: 3px;
  list-style: none;
  display: inline-block;
}

.message-alert {
  text-align: end !important;
}

a[target="_blank"]:before {
  content: "";
  padding-inline-end: 3px;
  font-family: "Font Awesome 6 Free";
  font-size: 14px;
  font-weight: 900;
}

@media (width <= 575.98px) {
  #wrapper.d-flex {
    display: block !important;
  }
}

.text-muted {
  color: var(--text-muted-color);
  opacity: .7;
}

.card-columns {
  grid-gap: 0 15px;
  grid-template-columns: 1fr;
  display: grid;
}

@media (width >= 768px) {
  .card-columns {
    grid-template-columns: 1fr 1fr;
  }

  .cpanel-help .card-columns, .cpanel-system .card-columns {
    grid-template-columns: 1fr 1fr 1fr;
  }
}

.tiny {
  line-height: 1.4rem;
}

details {
  background: var(--details-bg);
  border: var(--details-border);
  border-radius: .25rem;
  margin: 0 0 2rem;
  padding: .5rem 1rem;
}

details summary {
  color: var(--template-link-color);
}

details summary ~ * {
  margin-top: 1rem;
}

meter {
  width: 100%;
}

@media (width >= 768px) {
  joomla-tab[orientation="horizontal"]:not([view="accordion"]) section > .row {
    --gutter-x: 4rem;
  }

  joomla-tab[orientation="horizontal"]:not([view="accordion"]) section > .row > * + * {
    border-inline-start: 1px solid var(--template-bg-dark-10);
  }
}

.header {
  z-index: 1020;
  color: #fff;
  background: var(--header-bg);
}

.header-inside {
  min-height: 54px;
  display: flex;
}

.header a {
  color: #fff;
}

.header a:before {
  display: none;
}

.header .logo {
  background-color: var(--header-logo-bg);
  flex-flow: row;
  align-items: center;
  width: 18rem;
  height: 100%;
  padding: 12px 5px;
  transition: all .3s ease-in-out;
  display: flex;
  overflow: hidden;
}

.header .logo.small {
  justify-content: center;
  width: 3rem;
  transition: all .3s ease-in-out;
}

.header .logo.small svg:not(.logo-collapsed), .header .logo.small img:not(.logo-collapsed) {
  display: none;
}

.header .logo.small svg.logo-collapsed, .header .logo.small img.logo-collapsed {
  object-fit: contain;
  object-position: center center;
  width: 20px;
  height: 20px;
  display: inline-block;
}

.header .logo svg, .header .logo img {
  object-fit: contain;
  object-position: left center;
  width: 150px;
  height: 30px;
  margin: 0 .35rem;
}

.header .logo svg.logo-collapsed, .header .logo img.logo-collapsed {
  display: none;
}

.header .page-title {
  color: #fff;
  text-overflow: ellipsis;
  white-space: nowrap;
  margin: 0;
  padding: 0 1rem;
  font-size: 1.2rem;
  overflow: hidden;
}

.header .page-title > span {
  margin-inline-end: .25rem;
}

.header .dropdown-menu {
  font-size: .85rem;
}

.header .dropdown-header, .header .dropdown-item {
  color: #fff;
  background-color: var(--dropdown-item-bg);
  padding: .82rem .75rem;
}

.header .dropdown-header > span, .header .dropdown-item > span {
  margin-inline-end: .5rem;
}

.header .dropdown-header:hover, .header .dropdown-item:hover {
  background-color: var(--dropdown-item-bg-hvr);
}

.header .dropdown-header {
  font-size: inherit;
  background-color: var(--dropdown-header-bg);
  padding: .75rem;
}

.header-items {
  justify-content: flex-end;
  align-items: center;
  width: 50%;
  margin: 15px;
  margin-inline-start: auto;
}

.header-items .dropdown-toggle {
  background-color: #0000;
  border: 0;
}

.header-items .dropdown-toggle:after {
  display: none;
}

.header-item {
  margin: 0 4px;
  position: relative;
}

.header-item-content {
  color: #fff;
  background-color: var(--header-item-content-bg);
  border-radius: var(--header-item-content-border-radius);
  border: none;
  align-items: center;
  padding-inline-end: 4px;
  line-height: 1rem;
  display: flex;
}

.header-item-content a, .header-item-content button {
  color: #fff;
  text-decoration: none;
}

.header-item-content a {
  display: flex;
}

.header-item-content:not(.no-link):not(.joomlaversion):hover {
  background-color: var(--header-item-content-bg-hvr);
}

.header-item-content.joomlaversion {
  color: var(--header-item-content-jversion-color);
  background-color: var(--header-item-content-jversion-bg);
}

.header-item-content.joomlaversion .header-item-text {
  padding-inline-end: 12px;
}

.header-item-icon > * {
  color: #fff;
  background-color: var(--template-bg-dark);
  border-radius: 16px;
  justify-content: center;
  align-items: center;
  width: 28px;
  min-width: 28px;
  height: 28px;
  margin: 4px;
  display: flex;
}

.header-item-icon span {
  margin: 4px;
}

.header-item-count {
  margin-inline-end: .5rem;
}

.header-item-text {
  white-space: nowrap;
  padding: 0 8px 0 4px;
  font-size: .75rem;
}

.header-more-btn {
  margin: 0 5px;
  font-size: 1.7rem;
}

.header-dd-items .header-item-content {
  background: none;
}

.header-dd-items .header-item-text {
  font-size: .75rem;
}

.header-dd-items .dropdown-item {
  padding: .5rem .75rem;
}

.header-dd-item {
  padding: 3px 6px;
}

@media (width <= 767.98px) {
  .header-items {
    background: var(--template-bg-dark);
    flex-direction: row-reverse;
    width: 100%;
    margin: 0;
    padding: 10px 0;
    position: fixed;
    bottom: 0;
  }

  .header-items .icon-angle-down, .header-items .fa-angle-down {
    transform: rotate(180deg);
  }
}

[data-bs-theme="dark"] .header-item-icon > * {
  background: var(--header-bg);
}

.form-select, .custom-select {
  cursor: pointer;
  background: var(--form-select-background);
  border: var(--form-select-border);
  max-width: 100%;
}

[dir="rtl"] .form-select, [dir="rtl"] .custom-select {
  background: var(--form-select-background-rtl);
}

.form-select[multiple], [multiple].custom-select {
  padding: 0;
}

.form-select[multiple] option, [multiple].custom-select option {
  padding: .3rem 1rem;
}

.form-select[multiple] option:checked, [multiple].custom-select option:checked {
  color: var(--template-text-light);
  background-color: var(--template-contrast) !important;
}

.form-select.form-select-success, .form-select-success.custom-select, .form-select.custom-select-success, .custom-select-success.custom-select {
  color: var(--success-text);
  background-color: var(--success);
  border-color: var(--success);
}

.form-select.form-select-success option, .form-select-success.custom-select option, .form-select.custom-select-success option, .custom-select-success.custom-select option {
  color: var(--body-color);
  background-color: var(--form-select-success-bg);
}

.form-select.form-select-danger, .form-select-danger.custom-select, .form-select.custom-select-danger, .custom-select-danger.custom-select {
  color: var(--danger-text);
  background-color: var(--danger);
  border-color: var(--danger);
}

.form-select.form-select-danger option, .form-select-danger.custom-select option, .form-select.custom-select-danger option, .custom-select-danger.custom-select option {
  color: var(--body-color);
  background-color: var(--form-select-danger-bg);
}

.form-select:disabled, .custom-select:disabled {
  cursor: default;
  background-color: var(--form-select-disabled-bg);
  box-shadow: none;
  background-image: none;
  border: 0;
}

.form-select optgroup, .custom-select optgroup, .form-select option, .custom-select option {
  color: var(--form-select-option);
  background-color: var(--body-bg);
}

.form-select optgroup:disabled, .custom-select optgroup:disabled, .form-select option:disabled, .custom-select option:disabled {
  background-color: var(--form-select-option-disabled-bg);
}

[data-bs-theme="dark"] .form-select:disabled, [data-bs-theme="dark"] .custom-select:disabled {
  opacity: .5;
  border: 1px solid #ffffff1a;
}

joomla-alert {
  --jui-alert-min-width: 250px;
  --jui-alert-padding: .5rem 1.25rem;
  --jui-alert-margin: 0 0 1rem 0;
  --jui-alert-border: 1px solid transparent;
  --jui-alert-border-radius: .25rem;
  --jui-alert-animation-duration: .5s;
  --jui-alert-animation-timing-function: ease-in-out;
  --jui-alert-button-color-dark: #000;
  --jui-alert-button-color-light: #fff;
  --jui-alert-success-color: #234423;
  --jui-alert-success-background-color: #d9e6d9;
  --jui-alert-success-border-color: #cadcca;
  --jui-alert-success-link-color: #122212;
  --jui-alert-info-color: #0c5460;
  --jui-alert-info-background-color: #d1ecf1;
  --jui-alert-info-border-color: #bee5eb;
  --jui-alert-info-link-color: #062c33;
  --jui-alert-warning-color: #7d5a29;
  --jui-alert-warning-background-color: #fcefdc;
  --jui-alert-warning-border-color: #fbe8cd;
  --jui-alert-warning-link-color: #573e1c;
  --jui-alert-danger-color: #712b29;
  --jui-alert-danger-background-color: #f7dddc;
  --jui-alert-danger-border-color: #f4cfce;
  --jui-alert-danger-link-color: #4c1d1b;
  min-width: var(--jui-alert-min-width, 250px);
  padding: var(--jui-alert-padding, .5rem 1.25rem);
  margin: var(--jui-alert-margin, 0 0 1rem 0);
  border: var(--jui-alert-border, 1px solid transparent);
  border-radius: var(--jui-alert-border-radius, .25rem);
  animation-duration: var(--jui-alert-animation-duration, .5s);
  animation-timing-function: var(--jui-alert-animation-timing-function, ease-in-out);
  display: block;
}

joomla-alert .joomla-alert--close {
  float: right;
  color: var(--jui-alert-button-color-dark, #000);
  text-shadow: 0 1px 0 var(--jui-alert-button-color-light, #fff);
  opacity: .5;
  background: none;
  border: 0;
  padding: .2rem 1rem;
  font-size: 1.5rem;
  font-weight: 700;
  line-height: 1;
  position: relative;
  top: -.5rem;
  right: -1.25rem;
}

joomla-alert .joomla-alert--close:hover, joomla-alert .joomla-alert--close:focus {
  color: var(--jui-alert-button-color-dark, #000);
  cursor: pointer;
  opacity: .75;
  text-decoration: none;
}

joomla-alert[type="success"] {
  color: var(--jui-alert-success-color, #234423);
  background-color: var(--jui-alert-success-background-color, #d9e6d9);
  border-color: var(--jui-alert-success-border-color, #cadcca);
}

joomla-alert[type="success"] hr {
  border-top-color: var(--jui-alert-success-border-color, #cadcca);
}

joomla-alert[type="success"] .alert-link {
  color: var(--jui-alert-success-link-color, #122212);
}

joomla-alert[type="info"] {
  color: var(--jui-alert-info-color, #0c5460);
  background-color: var(--jui-alert-info-background-color, #d1ecf1);
  border-color: var(--jui-alert-info-border-color, #bee5eb);
}

joomla-alert[type="info"] hr {
  border-top-color: var(--jui-alert-info-border-color, #bee5eb);
}

joomla-alert[type="info"] .alert-link {
  color: var(--jui-alert-info-link-color, #062c33);
}

joomla-alert[type="warning"] {
  color: var(--jui-alert-warning-color, #7d5a29);
  background-color: var(--jui-alert-warning-background-color, #fcefdc);
  border-color: var(--jui-alert-warning-border-color, #fbe8cd);
}

joomla-alert[type="warning"] hr {
  border-top-color: var(--jui-alert-warning-border-color, #fbe8cd);
}

joomla-alert[type="warning"] .alert-link {
  color: var(--jui-alert-warning-link-color, #573e1c);
}

joomla-alert[type="danger"] {
  color: var(--jui-alert-danger-color, #712b29);
  background-color: var(--jui-alert-danger-background-color, #f7dddc);
  border-color: var(--jui-alert-danger-border-color, #f4cfce);
}

joomla-alert[type="danger"] hr {
  border-top-color: var(--jui-alert-danger-border-color, #f4cfce);
}

joomla-alert[type="danger"] .alert-link {
  color: var(--jui-alert-danger-link-color, #4c1d1b);
}

html[dir="rtl"] joomla-alert .joomla-alert--close, html[dir="rtl"] joomla-alert .joomla-alert-button--close {
  float: left;
  left: -1.25rem;
  right: auto;
}

@keyframes joomla-alert-fade-in {
  0% {
    opacity: 0;
  }
}

@keyframes joomla-alert-fade-out {
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
}

@media (prefers-reduced-motion: reduce) {
  joomla-alert {
    animation-duration: 1ms !important;
  }
}

#system-message-container joomla-alert {
  width: 100%;
  min-width: 16rem;
  color: var(--alert-accent-color);
  background-color: var(--alert-bg-color);
  border: 1px solid;
  border-color: var(--alert-border);
  border-radius: .2rem;
  margin-bottom: 1rem;
  padding: 0;
  transition: opacity .15s linear;
  display: flex;
  position: relative;
}

#system-message-container joomla-alert[type="success"], #system-message-container joomla-alert[type="message"] {
  --alert-accent-color: var(--state-success-text);
  --alert-bg-color: var(--state-success-bg);
  --alert-border: var(--state-success-bg-hvr);
  --alert-heading-bg: var(--state-success-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="success"] a:not(.btn), #system-message-container joomla-alert[type="success"] .btn-link, #system-message-container joomla-alert[type="message"] a:not(.btn), #system-message-container joomla-alert[type="message"] .btn-link {
  color: var(--state-success-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="info"], #system-message-container joomla-alert[type="notice"] {
  --alert-accent-color: var(--state-info-text);
  --alert-bg-color: var(--state-info-bg);
  --alert-border: var(--state-info-bg-hvr);
  --alert-heading-bg: var(--state-info-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="info"] a:not(.btn), #system-message-container joomla-alert[type="info"] .btn-link, #system-message-container joomla-alert[type="notice"] a:not(.btn), #system-message-container joomla-alert[type="notice"] .btn-link {
  color: var(--state-info-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="warning"] .joomla-alert--close {
  color: var(--state-warning-text);
}

#system-message-container joomla-alert[type="warning"] {
  --alert-accent-color: var(--state-warning-text);
  --alert-bg-color: var(--state-warning-bg);
  --alert-border: var(--state-warning-border);
  --alert-heading-bg: var(--state-warning-heading-bg);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="warning"] a:not(.btn), #system-message-container joomla-alert[type="warning"] .btn-link {
  color: var(--state-warning-text);
  text-decoration: underline;
}

#system-message-container joomla-alert[type="error"], #system-message-container joomla-alert[type="danger"] {
  --alert-accent-color: var(--state-danger-text);
  --alert-bg-color: var(--state-danger-bg);
  --alert-border: var(--state-danger-bg-hvr);
  --alert-heading-bg: var(--state-danger-bg-hvr);
  --alert-link-color: var(--state-success-link-color, var(--states-link-color));
}

#system-message-container joomla-alert[type="error"] a:not(.btn), #system-message-container joomla-alert[type="error"] .btn-link, #system-message-container joomla-alert[type="danger"] a:not(.btn), #system-message-container joomla-alert[type="danger"] .btn-link {
  color: var(--state-danger-text);
  text-decoration: underline;
}

#system-message-container joomla-alert .alert-heading {
  color: #fff;
  background: var(--alert-heading-bg);
  flex-direction: column;
  place-content: center;
  padding: .8rem;
  line-height: normal;
  display: flex;
}

#system-message-container joomla-alert .alert-heading .message:before, #system-message-container joomla-alert .alert-heading .success:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M1299 813l-422 422q-19 19-45 19t-45-19l-294-294q-19-19-19-45t19-45l102-102q19-19 45-19t45 19l147 147 275-275q19-19 45-19t45 19l102 102q19 19 19 45t-19 45zm141 83q0-148-73-273t-198-198-273-73-273 73-198 198-73 273 73 273 198 198 273 73 273-73 198-198 73-273zm224 0q0 209-103 385.5t-279.5 279.5-385.5 103-385.5-103-279.5-279.5-103-385.5 103-385.5 279.5-279.5 385.5-103 385.5 103 279.5 279.5 103 385.5z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .notice:before, #system-message-container joomla-alert .alert-heading .info:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119.043 8 8 119.083 8 256c0 136.997 111.043 248 248 248s248-111.003 248-248C504 119.083 392.957 8 256 8zm0 110c23.196 0 42 18.804 42 42s-18.804 42-42 42-42-18.804-42-42 18.804-42 42-42zm56 254c0 6.627-5.373 12-12 12h-88c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h12v-64h-12c-6.627 0-12-5.373-12-12v-24c0-6.627 5.373-12 12-12h64c6.627 0 12 5.373 12 12v100h12c6.627 0 12 5.373 12 12v24z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .warning:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 1792 1792\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(0, 0, 0, .95)\" d=\"M1024 1375v-190q0-14-9.5-23.5t-22.5-9.5h-192q-13 0-22.5 9.5t-9.5 23.5v190q0 14 9.5 23.5t22.5 9.5h192q13 0 22.5-9.5t9.5-23.5zm-2-374l18-459q0-12-10-19-13-11-24-11h-220q-11 0-24 11-10 7-10 21l17 457q0 10 10 16.5t24 6.5h185q14 0 23.5-6.5t10.5-16.5zm-14-934l768 1408q35 63-2 126-17 29-46.5 46t-63.5 17h-1536q-34 0-63.5-17t-46.5-46q-37-63-2-126l768-1408q17-31 47-49t65-18 65 18 47 49z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-heading .error:before, #system-message-container joomla-alert .alert-heading .danger:before {
  content: "";
  background-image: url("data:image/svg+xml;utf8,<svg width=\"1792\" height=\"1792\" viewBox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path fill=\"rgba(255, 255, 255, .95)\" d=\"M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8zm0 448c-110.5 0-200-89.5-200-200S145.5 56 256 56s200 89.5 200 200-89.5 200-200 200zm101.8-262.2L295.6 256l62.2 62.2c4.7 4.7 4.7 12.3 0 17l-22.6 22.6c-4.7 4.7-12.3 4.7-17 0L256 295.6l-62.2 62.2c-4.7 4.7-12.3 4.7-17 0l-22.6-22.6c-4.7-4.7-4.7-12.3 0-17l62.2-62.2-62.2-62.2c-4.7-4.7-4.7-12.3 0-17l22.6-22.6c4.7-4.7 12.3-4.7 17 0l62.2 62.2 62.2-62.2c4.7-4.7 12.3-4.7 17 0l22.6 22.6c4.7 4.7 4.7 12.3 0 17z\"/></svg>");
  background-size: 100%;
  width: 1em;
  height: 1em;
  display: inline-block;
}

#system-message-container joomla-alert .alert-wrapper {
  width: 100%;
}

#system-message-container joomla-alert .alert-link {
  color: var(--states-link-color);
  font-weight: normal;
  text-decoration: underline;
}

#system-message-container joomla-alert .joomla-alert--close, #system-message-container joomla-alert .joomla-alert-button--close {
  top: 0;
  color: var(--alert-accent-color);
  text-shadow: none;
  opacity: 1;
  background: none;
  border: 0;
  padding: .75rem .8rem;
  font-size: 2rem;
  line-height: 1rem;
  position: absolute;
  inset-inline-end: 0;
}

#system-message-container joomla-alert .joomla-alert--close:hover, #system-message-container joomla-alert .joomla-alert--close:focus, #system-message-container joomla-alert .joomla-alert-button--close:hover, #system-message-container joomla-alert .joomla-alert-button--close:focus {
  cursor: pointer;
  opacity: .75;
  text-decoration: none;
}

#system-message-container joomla-alert div {
  font-size: 1rem;
}

#system-message-container joomla-alert div .alert-message {
  margin: .5rem;
  padding: .15rem .3rem;
  padding-inline-end: 2rem;
}

#system-message-container joomla-alert div .alert-message:not(:first-of-type) {
  border-top: 1px solid var(--alert-accent-color);
}

th {
  text-align: -webkit-match-parent;
}

:root {
  --template-sidebar-bg: var(--template-bg-dark-80);
  --template-sidebar-font-color: #fff;
  --template-sidebar-link-color: #fff;
  --template-bg-light: #f0f4fb;
  --template-link-color: var(--link-color);
  --template-link-hover-color: var(--link-hover-color);
  --template-text-light: #fff;
  --template-text-dark: #495057;
  --template-special-color: #001b4c;
  --template-contrast: #2a69b8;
  --template-quickicon-color: hsl(var(--hue), 30%, 40%);
  --template-bg-dark: hsl(var(--hue), 40%, 20%);
  --template-bg-dark-3: hsl(var(--hue), 40%, 97%);
  --template-bg-dark-5: hsl(var(--hue), 40%, 95%);
  --template-bg-dark-7: hsl(var(--hue), 40%, 93%);
  --template-bg-dark-10: hsl(var(--hue), 40%, 90%);
  --template-bg-dark-15: hsl(var(--hue), 40%, 85%);
  --template-bg-dark-20: hsl(var(--hue), 40%, 80%);
  --template-bg-dark-30: hsl(var(--hue), 40%, 70%);
  --template-bg-dark-40: hsl(var(--hue), 40%, 60%);
  --template-bg-dark-50: hsl(var(--hue), 40%, 50%);
  --template-bg-dark-60: hsl(var(--hue), 40%, 40%);
  --template-bg-dark-65: hsl(var(--hue), 40%, 35%);
  --template-bg-dark-70: hsl(var(--hue), 40%, 30%);
  --template-bg-dark-75: hsl(var(--hue), 40%, 25%);
  --template-bg-dark-80: hsl(var(--hue), 40%, 20%);
  --template-bg-dark-90: hsl(var(--hue), 40%, 10%);
  --media-manager-overlay-header-bg: var(--template-bg-dark-5);
  --media-manager-infobar-dt-color: #0000008a;
  --media-manager-content-bg: var(--body-bg);
  --media-manager-file-icon-color: hsl(var(--hue), 20%, 60%);
  --media-manager-content-color: #2a69b8;
  --media-manager-overlay-bg: var(--template-bg-dark-3);
  --media-manager-disk-name-color: var(--template-text-dark);
  --media-manager-tree-item-hover-bg: #e1e1e1;
  --media-manager-toolbar-icon-color: var(--template-bg-dark-60);
  --media-manager-border: 1px solid var(--template-bg-dark-7);
  --media-browser-file-bg: hsl(var(--hue), 20%, 97%);
  --media-drive-bg: var(--com-media-manager-content-bg, #fff);
  --media-toolbar-bg: var(--com-media-manager-content-bg, #fff);
  --media-toolbar-icon-bg: transparent;
  --media-toolbar-icon-color: var(--com-media-manager-toolbar-icon-color, var(--template-bg-dark-60));
  --media-toolbar-icon-bg-hvr: #f0f0f0;
  --media-tree-item-color: var(--template-bg-dark-60);
  --media-manager-breadcrumb-item-bg: var(--com-media-manager-overlay-bg, var(--template-bg-dark-3));
  --media-tree-active-icon-color: #2a69b8;
  --image-bg: #fff;
  --image-bg-image: linear-gradient(45deg, hsl(var(--hue), 20%, 97%) 25%, transparent 25%, transparent 75%, #eee 75%, hsl(var(--hue), 20%, 97%) 100%), linear-gradient(45deg, #eee 25%, transparent 25%, transparent 75%, hsl(var(--hue), 20%, 97%) 75%, hsl(var(--hue), 20%, 97%) 100%);
  --template-quickicon-bg: hsl(var(--hue), 60%, 97%);
  --template-quickicon-bg-hvr: var(--template-bg-dark);
  --form-control-bg: var(--template-text-light);
  --form-control-bg-disabled: var(--gray-200);
  --form-control-border: var(--border-width) solid var(--template-bg-dark-20);
  --form-check-input-checked-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'%3e%3cpath fill='none' stroke='%23fff' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='m6 10 3 3 6-6'/%3e%3c/svg%3e");
  --form-check-input-checked-bg-color: #2a69b7;
  --form-check-input-border: var(--border-width) solid var(--template-bg-dark-20);
  --form-select-background: var(--template-bg-light) url("../images/select-bg.svg?v=5ebba0") no-repeat right center / max(100%, 116rem);
  --form-select-background-rtl: var(--template-bg-light) url("../images/select-bg-rtl.svg?v=e50af2") no-repeat left center / max(100%, 116rem);
  --form-select-border: var(--form-control-border);
  --form-select-success-bg: var(--white-offset);
  --form-select-danger-bg: var(--white-offset);
  --form-select-disabled-bg: var(--gray-200);
  --form-select-option: var(--template-text-dark);
  --form-select-option-disabled-bg: var(--template-bg-dark-5);
  --file-selector-btn-bg: var(--template-bg-dark-70);
  --file-selector-btn-bg-hvr: var(--template-bg-dark-50);
  --bg-normal: var(--template-bg-light);
  --bg-reverted: var(--template-bg-dark-60);
  --text-normal: var(--template-text-dark);
  --text-reverted: var(--template-text-light);
  --quickicon-bg: var(--white);
  --quickicon-color: var(--template-quickicon-color);
  --quickicon-color-hvr: var(--template-quickicon-color);
  --quickicon-border: 1px solid hsl(var(--hue), 50%, 93%);
  --quickicon-border-radius: 4px;
  --quickicon-link-hvr: #fff;
  --quickicon-linkadd-bg: hsl(var(--hue), 50%, 93%);
  --quickicon-linkadd-bg-hvr: var(--template-bg-dark);
  --quickicon-linkadd-color: var(--template-quickicon-color);
  --quickicon-amount-bg: hsl(var(--hue), 50%, 93%);
  --new-modules-color: var(--template-quickicon-color);
  --new-modules-bg: hsl(var(--hue), 60%, 97%);
  --new-modules-bg-hvr: var(--template-bg-dark);
  --new-modules-icon-color: var(--template-quickicon-color);
  --new-modules-icon-color-hvr: hsl(var(--hue), 50%, 93%);
  --new-module-color: var(--text-color);
  --new-module-bg: var(--template-quickicon-bg);
  --new-module-border: 1px solid hsl(var(--hue), 50%, 93%);
  --new-module-link-bg: hsl(var(--hue), 50%, 93%);
  --new-module-link-span: var(--quickicon-color);
  --new-module-link-hvr: var(--new-modules-bg-hvr);
  --new-module-link-span-hvr: var(--new-modules-icon-color-hvr);
  --sidebarwrapper-bg: var(--template-sidebar-bg);
  --sidebar-item-color: #fff;
  --sidebar-item-color-hvr: var(--template-text-light);
  --sidebar-item-bg-hvr: var(--template-bg-dark-65);
  --sidebar-toggle-bg: var(--template-bg-dark-60);
  --sidebar-toggle-link: #fff;
  --main-nav-ul-bg: var(--template-bg-dark-75);
  --main-nav-divider-bg: var(--template-bg-dark-60);
  --main-nav-item-title: var(--template-bg-dark-30);
  --main-nav-item-title-hvr-bg: var(--template-bg-dark-60);
  --main-nav-mm-active-bg: var(--template-bg-dark-70);
  --main-nav-mm-active-quicktask-bg: var(--template-bg-dark-60);
  --main-nav-badge: var(--template-bg-dark-60);
  --mobile-menu-collapse-bg: var(--template-bg-dark-50);
  --header-bg: var(--template-bg-dark);
  --header-item-content-bg: var(--template-bg-dark-60);
  --header-item-content-bg-hvr: var(--template-bg-dark-50);
  --header-item-content-border-radius: 22px;
  --header-item-content-jversion-color: var(--bluegray);
  --header-item-content-jversion-bg: transparent;
  --header-logo-bg: var(--template-bg-dark-70);
  --table-color: var(--body-color);
  --table-border-top-nfc: 2px solid #dee2e6;
  --table-link-color: var(--template-link-color);
  --table-link-font-weight: 500;
  --table-link-font-weight-sorted: 500;
  --card-header-color: var(--template-bg-dark);
  --card-bg: #ffffffb3;
  --card-header-bg: var(--card-bg);
  --card-body-bg: var(--card-bg);
  --main-card-bg: var(--body-bg);
  --main-card-col-border-left: 1px solid var(--template-bg-dark-10);
  --module-actions-color: var(--body-color);
  --atum-list-group-border-color: var(--template-bg-dark-15);
  --atum-box-shadow: 0 2px 10px -8px var(--template-bg-dark-50);
  --dropdown-box-shadow: 0 1px 1px #00000026;
  --quickicon-box-shadow-success: 0 0 3px 0 var(--success);
  --quickicon-box-shadow-danger: 0 0 3px 0 var(--danger);
  --quickicon-box-shadow-warning: 0 0 3px 0 var(--warning);
  --login-box-shadow: 0 4px 20px -10px var(--template-bg-dark-50);
  --login-focus-box-shadow: inset 0 0 1px 1px var(--template-contrast);
  --modal-btn-box-shadow: 1px 1px 1px 0 #00000040;
  --modal-joomla-dialog-box-shadow: 0 .5rem 1rem #00000026;
  --form-aria-grabbed-box-shadow: 0 0 2px 1px var(--template-bg-dark);
  --choices__button_joomla-focus-box-shadow: 0 0 0 2px #00bcd4;
  --joomla-tab-accordion-box-shadow: 0 1px #fff inset, 0 0 3px #0000000a;
  --joomla-tab-btn-hvr: var(--template-bg-dark-60);
  --joomla-tab-btn-aria-exp-bg: var(--template-bg-dark-3);
  --joomla-tab-btn-region-border: 1px solid var(--template-bg-dark-3);
  --joomla-tablist-btn-aria-exp: var(--template-text-light);
  --joomla-tablist-btn-aria-exp-bg: var(--template-bg-dark-60);
  --joomla-tablist-btn-aria-exp-aft-bg: var(--template-bg-dark);
  --joomla-tablist-border-bottom: 1px solid var(--template-bg-dark-10);
  --joomla-tab-tablist-border: 1px solid #dee2e6;
  --joomla-tab-vert-border-bottom: 1px solid #dee2e6;
  --admin-background: var(--template-bg-light);
  --options-form-color: var(--template-text-dark);
  --options-form-bg: unset;
  --options-form-border: 1px solid var(--template-bg-dark-20);
  --options-form-legend-color: var(--template-text-dark);
  --options-form-legend-bg: inherit;
  --details-bg: var(--template-bg-dark-3);
  --details-border: 1px solid var(--template-bg-dark-10);
  --nav-header-color: var(--template-bg-dark);
  --choices-input-border-color: var(--body-bg);
  --choices-input-bg: var(--body-bg);
  --choices-input-placeholder: #495057;
  --choices-list-multiple-item-bg: var(--template-bg-dark);
  --choices-list-multiple-item: #fff;
  --switcher-toggle-inside-bg: #fff;
  --switcher-toggle-outside-off-bg: #d3d3d3;
  --switcher-toggle-outside-on-bg: #2f7d32;
  --input-border-color: var(--template-bg-dark-20);
  --featured-icon-color: #c78800;
  --users-method-image: var(--light);
  --users-methods-reset-cont-bg: var(--light);
  --users-method-header-inact-bg: var(--light);
  --login-label-color: #001b4c;
  --icon-success: var(--success);
  --icon-success-border: var(--success);
  --icon-checkedout-color: var(--template-text-dark);
  --btn-primary-color: var(--template-text-light);
  --btn-primary-bg: var(--template-bg-dark-60);
  --btn-primary-border: 1px solid var(--template-bg-dark-60);
  --btn-primary-bg-hvr: var(--template-bg-dark-70);
  --btn-primary-border-hvr: 1px solid var(--template-bg-dark-90);
  --btn-primary-toggle-bg: var(--template-bg-dark);
  --btn-primary-toggle-border: var(--template-bg-dark);
  --btn-secondary-bg: var(--template-bg-dark-60);
  --btn-secondary-border: 1px solid var(--template-bg-dark-60);
  --btn-secondary-color: var(--template-text-light);
  --btn-secondary-bg-hvr: var(--template-bg-dark-70);
  --btn-secondary-color-hvr: var(--template-text-light);
  --btn-info-color: var(--template-text-light);
  --btn-info-bg: #2a69b8;
  --btn-info-border: 1px solid #2a69b8;
  --btn-info-bg-hvr: #21518e;
  --btn-info-border-hvr: 1px solid #21518e;
  --btn-outline-secondary-color: gray-700;
  --btn-outline-secondary-bg: transparent;
  --btn-outline-secondary-border: var(--btn-secondary-border);
  --btn-dark-border: 1px solid #000000d9;
  --btn-dark-bg: #000000e6;
  --btn-dark-bg-hvr: #000;
  --btn-dark-color: #fff;
  --btn-dark-color-hvr: #fff;
  --btn-light-bg: #f8f9fa;
  --btn-light-border: 1px solid #f8f9fa;
  --btn-light-bg-hvr: #d3d4d5;
  --input-group-text-bg: var(--template-bg-dark);
  --input-group-text-border: 1px solid var(--template-bg-dark);
  --js-stools-btn-clear-bg: var(--template-bg-dark);
  --js-stools-btn-clear-disabled-bg: var(--btn-disabled-bg);
  --js-stools-btn-clear-disabled-border: #dee2e6cc;
  --cpanel-addmodule-hover: var(--template-bg-dark);
  --dropdown-header-bg: var(--template-bg-dark);
  --dropdown-item-bg: var(--template-bg-dark-70);
  --dropdown-item-bg-hvr: var(--template-bg-dark);
  --dropdown-item-border-bottom: 1px solid #0000001a;
  --atum-dropdown-link-hover-bg: var(--tertiary-bg);
  --atum-dropdown-link-hover-color: var(--body-color);
  --states-link-color: var(--template-link-color);
  --states-btn-primary-bg: var(--btn-primary-bg);
  --states-btn-primary-bg-hover: var(--btn-primary-bg-hvr);
  --states-btn-primary-color: var(--btn-primary-color);
  --states-btn-primary-border: var(--btn-primary-bg);
  --state-success-text: #457d54;
  --state-success-text-hvr: #f2f8f4;
  --state-success-bg: #f2f8f4;
  --state-success-bg-hvr: #457d54;
  --state-success-border: hsl(var(--hue), 50%, 93%);
  --state-success-btn-border: 1px solid #457d54;
  --state-success-icon: #55a258;
  --state-success-icon-hvr: var(--state-success-icon);
  --state-success-heading-bg: #457d54;
  --state-success-link-color: #457d54;
  --state-info-text: var(--template-bg-dark-70);
  --state-info-text-hvr: var(--body-bg);
  --state-info-bg: #dbe4f0;
  --state-info-bg-hvr: var(--template-bg-dark-70);
  --state-info-border: var(--template-bg-dark-70);
  --state-info-icon: var(--template-bg-dark-70);
  --state-info-heading-bg: var(--template-bg-dark-70);
  --state-info-link-color: var(--template-bg-dark-70);
  --state-warning-text: #996901;
  --state-warning-text-hvr: #fffcf4;
  --state-warning-bg: #fffcf4;
  --state-warning-bg-hvr: #996900;
  --state-warning-border: #ffb512;
  --state-warning-icon: #ffb514;
  --state-warning-heading-bg: #ffb512;
  --state-warning-link-color: var(--warning-text-emphasis);
  --state-danger-text: var(--danger-text-emphasis);
  --state-danger-text-hvr: #fef8f8;
  --state-danger-bg: #f3d4d4;
  --state-danger-bg-hvr: #c52827;
  --state-danger-border: var(--danger-border-subtle);
  --state-danger-icon: #ce8484;
  --state-danger-heading-bg: #c52827;
  --state-danger-link-color: var(--danger-text-emphasis);
  --state-error-text: #c52827;
  --state-error-text-hvr: #c52827;
  --state-error-bg: #fef8f8;
  --state-error-bg-hvr: #fef8f8;
  --state-error-border: 1px solid #c52827;
  --state-error-icon: #e3706f;
  --state-error-heading-bg: #c52827;
  --alert-info-color: var(--template-bg-dark);
  --alert-info-bg: var(--template-bg-dark-10);
  --alert-info-border: 1px solid var(--template-bg-dark-20);
  --alert-link-text-color: var(--info-text-emphasis);
  --alert-accent-color: #fff;
  --atum-subhead-color: var(--template-text-dark);
  --atum-subhead-bg: #fff;
  --atum-subhead-bg-img: linear-gradient(var(--toolbar-bg), var(--template-bg-dark-3));
  --atum-btn: #495057;
  --atum-btn-icon: #495057;
  --atum-btn-icon-hvr: var(--template-text-light);
  --atum-btn-bg: var(--body-bg);
  --atum-btn-bg-hvr: #495057;
  --atum-btn-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-hvr: var(--body-bg);
  --atum-btn-success: var(--success);
  --atum-btn-success-icon: var(--atum-btn-success-bg-hvr);
  --atum-btn-success-icon-hvr: var(--template-text-light);
  --atum-btn-success-bg: var(--body-bg);
  --atum-btn-success-bg-hvr: var(--success);
  --atum-btn-success-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-success-hvr: var(--body-bg);
  --atum-btn-danger: #c52827;
  --atum-btn-danger-icon: #c52827;
  --atum-btn-danger-icon-hvr: var(--template-text-light);
  --atum-btn-danger-bg: var(--body-bg);
  --atum-btn-danger-bg-hvr: #c52827;
  --atum-btn-danger-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-danger-hvr: var(--body-bg);
  --atum-btn-info: #495057;
  --atum-btn-info-color-hvr: var(--template-text-light);
  --atum-btn-info-icon: #2a69b8;
  --atum-btn-info-icon-hvr: var(--atum-btn-hvr);
  --atum-btn-info-bg: var(--body-bg);
  --atum-btn-info-bg-hvr: #2a69b8;
  --atum-btn-info-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-info-hvr: var(--body-bg);
  --atum-btn-primary: var(--template-link-color);
  --atum-btn-primary-icon: var(--template-link-color);
  --atum-btn-primary-icon-hvr: var(--template-text-light);
  --atum-btn-primary-bg: var(--body-bg);
  --atum-btn-primary-bg-hvr: var(--template-link-color);
  --atum-btn-primary-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-primary-hvr: var(--body-bg);
  --atum-btn-action: var(--template-text-light);
  --atum-btn-action-icon: var(--template-bg-dark);
  --atum-btn-action-icon-hvr: var(--template-text-light);
  --atum-btn-action-bg: var(--body-bg);
  --atum-btn-action-bg-hvr: var(--template-bg-dark);
  --atum-btn-action-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-action-hvr: var(--body-bg);
  --atum-list-group-bg: var(--body-bg);
  --atum-calendar-week-bg: #f4f4f4;
  --atum-calendar-bg: var(--body-bg);
  --atum-calendar-buttons-color: #495057;
  --atum-calendar-disabled-color: #999;
  --atum-calendar-disabled-bg: #fff;
  --atum-calendar-select-bg: var(--template-bg-light);
  --atum-calendar-select-bg-url: url("../../images/select-bg.svg?v=178429");
  --atum-calendar-select-bg-url-rtl: url("../../images/select-bg-rtl.svg?v=178429");
  --permissions-sliders-oddCol-bg: var(--template-bg-light);
  --respTable-border-bottom: 1em solid var(--template-bg-dark-80);
  --treeselect-line-color: hsl(var(--hue), 40%, 93%);
  --treeselect-dropdown-toggle: var(--template-text-dark);
  --icon-check-square-bg: #212529;
  --icon-square-bg: #212529;
  --icon-check-square-color: var(--template-text-light);
  --icon-square-color: var(--template-text-light);
  --drag-color: #151515;
  --drag-background-color: #20c997;
  --success-text: #457d54;
  --success: #457d54;
  --successhvr: #2a4c33;
  --danger-text: #c52827;
  --danger: #c52827;
  --dangerhvr: #851b1a;
  --darkdanger: #c52827;
  --darkdangerhvr: #851b1a;
  --warning-text: #000;
  --warning: #ffb514;
  --warninghvr: #c78800;
  --text-muted-color: var(--template-text-dark) !important;
  --text-muted-hvr-color: var(--template-text-light) !important;
  --badge-bg-info: var(--info) !important;
}

[data-bs-theme="dark"] {
  --body-bg: #0f151d;
  --body-color: #ffffffd9;
  --template-bg-dark: #0c1a2b;
  --main-bg: #171c24;
  --admin-background: var(--body-bg);
  --button-and-icon-color: #007db0;
  --template-quickicon-color: var(--template-bg-dark-5);
  --focus-shadow: var(--gray-800);
  --media-manager-bg: var(--body-bg);
  --media-manager-content-bg: var(--body-bg);
  --media-manager-overlay-bg: var(--template-bg-dark-90);
  --media-manager-infobar-dt-color: #ffffff8a;
  --media-manager-content-color: #17bcff;
  --media-manager-overlay-header-bg: var(--template-bg-dark-80);
  --media-manager-disk-name-color: var(--template-text-light);
  --media-manager-border: 1px solid var(--gray-800);
  --media-manager-file-icon-color: #007db0;
  --media-manager-tree-item-hover-bg: var(--gray-600);
  --media-manager-toolbar-icon-color: #17bcff;
  --media-manager-breadcrumb-item-bg: var(--gray-700);
  --media-tree-item-color: #007db0;
  --media-tree-active-icon-color: #00597d;
  --media-browser-file-bg: var(--media-manager-bg);
  --media-drive-bg: var(--media-manager-bg);
  --media-toolbar-bg: var(--media-manager-bg);
  --media-toolbar-icon-bg: var(--media-manager-bg);
  --media-toolbar-icon-color: #00a1e3;
  --media-toolbar-icon-bg-hvr: var(--gray-800);
  --image-bg: #000;
  --image-bg-image: linear-gradient(45deg, var(--body-bg) 25%, transparent 25%, transparent 75%, #111 75%, var(--body-bg) 100%), linear-gradient(45deg, #111 25%, transparent 25%, transparent 75%, var(--body-bg) 75%, var(--body-bg) 100%);
  --success-text: #418d52;
  --success: #3d854d;
  --successhvr: #25512f;
  --danger-text: #e0615c;
  --danger: #c62a25;
  --dangerhvr: #861c19;
  --darkdanger: #941f1c;
  --darkdangerhvr: #541210;
  --error: #941f1c;
  --errorhvr: #be2824;
  --warning-text: #000;
  --warning: #f0ad4e;
  --warninghvr: #df8a13;
  --dark-mode-text-light: #ffffffd9;
  --template-link-color: #6fbfdb;
  --link-color-rgb: 36, 158, 200;
  --link-color: #6fbfdb;
  --template-link-hover-color: #98d1e5;
  --link-color-rgb-hvr: var(--template-link-hover-color);
  --template-quickicon-bg: #ffffff1a;
  --template-quickicon-bg-hvr: #ffffff26;
  --options-form-color: var(--template-text-light);
  --options-form-bg: transparent;
  --options-form-border: 1px solid #ffffff0d;
  --options-form-legend-color: var(--template-text-light);
  --options-form-legend-bg: var(--admin-background);
  --form-control-bg: var(--template-sidebar-bg);
  --form-control-bg-disabled: var(--gray-800);
  --form-control-border: 1px solid #fff3;
  --form-control-border-disabled: 1px solid #ffffff1a;
  --form-check-input-checked-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'%3e%3cpath fill='none' stroke='%23eee' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='m6 10 3 3 6-6'/%3e%3c/svg%3e");
  --form-check-input-checked-bg-color: #007db0;
  --form-check-input-border: 1px solid #fff3;
  --form-select-background: #007db0 url("../images/select-bg-dark.svg?v=97d2dc") no-repeat right center / max(100%, 116rem);
  --form-select-background-rtl: #007db0 url("../images/select-bg-rtl-dark.svg?v=5f59e1") no-repeat left center / max(100%, 116rem);
  --form-select-border: var(--form-control-border);
  --form-select-success-bg: var(--template-sidebar-bg);
  --form-select-danger-bg: var(--template-sidebar-bg);
  --form-select-disabled-bg: var(--template-sidebar-bg);
  --form-select-option: var(--template-text-light);
  --form-select-option-disabled-bg: var(--template-bg-dark-90);
  --file-selector-btn-bg: #007db0;
  --file-selector-btn-bg-hvr: #00597d;
  --bg-normal: var(--template-bg-dark);
  --bg-reverted: var(--template-bg-dark-20);
  --text-normal: var(--template-text-light);
  --text-reverted: black;
  --quickicon-bg: transparent;
  --quickicon-border: none;
  --quickicon-border-radius: 4px;
  --quickicon-link-hvr: #fff;
  --quickicon-linkadd-bg: #ffffff0a;
  --quickicon-linkadd-bg-hvr: var(--btn-primary-bg-hvr);
  --quickicon-linkadd-color: #fff;
  --quickicon-amount-bg: #6b6f77;
  --quickicon-color-hvr: var(--quickicon-link-hvr);
  --new-modules-color: var(--template-quickicon-color);
  --new-modules-bg: var(--template-bg-dark-65);
  --new-modules-bg-hvr: var(--btn-primary-bg-hvr);
  --new-modules-icon-color: var(--template-bg-dark-80);
  --new-modules-icon-color-hvr: var(--template-quickicon-color);
  --new-module-color: var(--dark-mode-text-light);
  --new-module-bg: var(--template-quickicon-bg);
  --new-module-border: none;
  --new-module-link-bg: #ffffff0d;
  --new-module-link-span: var(--quickicon-linkadd-color);
  --new-module-link-hvr: var(--new-modules-bg-hvr);
  --new-module-link-span-hvr: var(--icon-color-hvr);
  --sidebarwrapper-bg: var(--template-sidebar-bg);
  --sidebar-item-color: #fff;
  --sidebar-item-color-hvr: var(--template-text-light);
  --sidebar-item-bg-hvr: #007db0;
  --sidebar-toggle-bg: #20262d;
  --sidebar-toggle-link: #fff;
  --main-nav-ul-bg: #20262d;
  --main-nav-divider-bg: #495057;
  --main-nav-item-title: var(--template-bg-dark-30);
  --main-nav-mm-active-bg: #007db0;
  --main-nav-mm-active-quicktask-bg: #00597d;
  --main-nav-badge: var(--template-bg-dark-60);
  --main-nav-item-title-hvr-bg: #007db0;
  --mobile-menu-collapse-bg: var(--template-bg-dark-50);
  --sidebar-bg: #0f151d;
  --template-sidebar-bg: #0f151d;
  --sidebar-accent-bg: #2667bb;
  --header-bg: #0a0e13;
  --header-item-content-bg: #007db0;
  --header-item-content-bg-hvr: #00597d;
  --header-item-content-border-radius: 22px;
  --header-item-content-jversion-color: var(--dark-mode-text-light);
  --header-item-content-jversion-bg: transparent;
  --header-logo-bg: var(--header-bg);
  --alert-accent-color: #479a59;
  --alert-bg-color: #f2f8f4;
  --alert-link-text-color: #6fbfdb;
  --table-color: #ffffffe6;
  --table-border-top-nfc: 1px solid #969696;
  --table-link-color: #6fbfdb;
  --table-link-font-weight: 400;
  --table-link-font-weight-sorted: 400;
  --card-bg-dark: var(--template-sidebar-bg);
  --card-header-color: var(--dark-mode-text-light);
  --card-header-bg: var(--template-sidebar-bg);
  --card-bg: var(--template-sidebar-bg);
  --card-body-bg: var(--template-sidebar-bg);
  --main-card-bg: var(--template-sidebar-bg);
  --main-card-col-border-left: 1px solid var(--gray-800);
  --module-actions-color: var(--body-color);
  --atum-list-group-border-color: #ffffff1a;
  --border-color: #ffffff1a;
  --atum-box-shadow: none;
  --dropdown-box-shadow: none;
  --quickicon-box-shadow-success: none;
  --quickicon-box-shadow-danger: none;
  --quickicon-box-shadow-warning: none;
  --login-box-shadow: none;
  --login-focus-box-shadow: none;
  --modal-btn-box-shadow: none;
  --modal-joomla-dialog-box-shadow: none;
  --form-aria-grabbed-box-shadow: none;
  --choices__button_joomla-focus-box-shadow: none;
  --joomla-tab-accordion-box-shadow: none;
  --joomla-tab-btn-hvr: var(--template-text-light);
  --joomla-tab-btn-aria-exp-bg: var(--atum-btn-bg);
  --joomla-tab-btn-region-border: var(--atum-btn-bg);
  --joomla-tablist-btn-aria-exp: var(--template-text-light);
  --joomla-tablist-btn-aria-exp-bg: #00597d;
  --joomla-tablist-btn-aria-exp-aft-bg: #007db0;
  --joomla-tablist-border-bottom: 1px solid var(--gray-800);
  --joomla-tab-tablist-border: 1px solid var(--gray-800);
  --joomla-tab-vert-border-bottom: 1px solid var(--gray-800);
  --details-bg: #0006;
  --details-border: 1px solid var(--details-bg);
  --nav-header-color: var(--body-color);
  --choices-input-border-color: var(--btn-primary-bg);
  --choices-input-bg: var(--body-bg);
  --choices-input-placeholder: var(--gray-200);
  --choices-list-multiple-item-bg: var(--btn-primary-bg);
  --choices-list-multiple-item: var(--template-text-light);
  --switcher-toggle-inside-bg: var(--body-bg);
  --switcher-toggle-outside-off-bg: var(--gray-700);
  --switcher-toggle-outside-on-bg: var(--success);
  --input-border-color: #495057;
  --featured-icon-color: #f0ad4e;
  --users-method-image: var(--light);
  --users-methods-reset-cont-bg: var(--body-bg);
  --users-method-header-inact-bg: var(--body-bg);
  --login-label-color: var(--white);
  --icon-success: #418d52;
  --icon-success-border: #418d52;
  --icon-checkedout-color: var(--gray-400);
  --btn-primary-color: var(--text-normal);
  --btn-primary-bg: #007db0;
  --primary: #007db0;
  --btn-primary-border: 1px solid #00a1e3;
  --btn-primary-bg-hvr: #00597d;
  --btn-primary-border-hvr: 1px solid #00a1e3;
  --btn-primary-toggle-bg: var(--template-bg-dark);
  --btn-primary-toggle-border: var(--template-bg-dark);
  --btn-secondary-color: var(--template-text-light);
  --btn-secondary-bg: #33383f;
  --btn-secondary-border: 1px solid #4a515b;
  --btn-secondary-bg-hvr: #1c1f23;
  --btn-secondary-color-hvr: var(--template-text-light);
  --btn-secondary-border-hvr: 1px solid #4a515b;
  --btn-dark-border: 1px solid #000000d9;
  --btn-dark-bg: #000000e6;
  --btn-dark-bg-hvr: #000;
  --btn-dark-color: #fff;
  --btn-dark-color-hvr: #fff;
  --btn-light-bg: #007db0;
  --btn-light-border: 1px solid #00a1e3;
  --btn-light-bg-hvr: var(--btn-primary-bg-hvr);
  --btn-outline-secondary-color: var(--template-text-light);
  --btn-outline-secondary-bg: #64676c;
  --btn-outline-secondary-border: 1px solid #7d8086;
  --btn-info-color: var(--template-text-light);
  --btn-info-bg: #33383f;
  --btn-info-border: 1px solid #4a515b;
  --btn-info-bg-hvr: #1c1f23;
  --btn-info-border-hvr: 1px solid #1c1f23;
  --dropdown-header-bg: #00597d;
  --dropdown-divider-bg: 1px solid #fff;
  --dropdown-item-bg: #007db0;
  --dropdown-item-bg-hvr: #00597d;
  --dropdown-item-border-bottom: 1px solid #ffffff1a;
  --atum-dropdown-link-hover-bg: var(--gray-700);
  --input-group-text-bg: var(--btn-primary-bg);
  --input-group-text-border: 1px solid var(--btn-primary-border);
  --input-group-text-color: var(--btn-primary-color);
  --js-stools-btn-clear-bg: var(--body-bg);
  --js-stools-btn-clear-disabled-bg: var(--body-bg);
  --js-stools-btn-clear-disabled-border: var(--form-select-border);
  --states-link-color: #fff;
  --states-btn-primary-bg: #00000080;
  --states-btn-primary-bg-hover: #000000b3;
  --states-btn-primary-color: var(--template-text-light);
  --states-btn-primary-border: #000c;
  --state-success-text: var(--text-normal);
  --state-success-text-hvr: var(--text-normal);
  --state-success-bg: #3d854d;
  --state-success-bg-hvr: #25512f;
  --state-success-border: var(--state-success-bg);
  --state-success-btn-border: var(--state-success-bg);
  --state-success-icon: var(--text-normal);
  --state-success-icon-hvr: var(--text-normal);
  --state-success-heading-bg: #3d854d;
  --state-info-text: var(--text-normal);
  --state-info-text-hvr: var(--text-normal);
  --state-info-bg: #004764;
  --state-info-bg-hvr: #002231;
  --state-info-border: transparent;
  --state-info-icon: var(--state-info-text);
  --state-info-heading-bg: #006b97;
  --state-warning-text: #000;
  --state-warning-text-hvr: #000;
  --state-warning-bg: #f0ad4e;
  --state-warning-bg-hvr: #df8a13;
  --state-warning-border: #df8a13;
  --state-warning-icon: #000;
  --state-warning-heading-bg: #df8a13;
  --state-danger-text: var(--text-normal);
  --state-danger-text-hvr: var(--text-normal);
  --state-danger-bg: #c62a25;
  --state-danger-bg-hvr: #861c19;
  --state-danger-border: var(--dangerhvr);
  --state-danger-icon: var(--text-normal);
  --state-danger-heading-bg: var(--dangerhvr);
  --state-error-text: var(--text-normal);
  --state-error-bg: var(--error);
  --state-error-border: var(--error);
  --state-error-icon: var(--text-normal);
  --state-error-heading-bg: var(--errorlight);
  --alert-info-color: var(--text-normal);
  --alert-info-bg: var(--body-bg);
  --alert-info-border: var(--body-bg);
  --atum-subhead-color: var(--template-text-dark);
  --atum-subhead-bg: var(--body-bg);
  --atum-subhead-bg-img: linear-gradient(var(--body-bg), var(--dark-bg));
  --atum-btn: var(--template-text-light);
  --atum-btn-icon: var(--template-text-light);
  --atum-btn-icon-hvr: #495057;
  --atum-btn-bg: #007db0;
  --atum-btn-bg-hvr: #004764;
  --atum-btn-border: 1px solid hsl(var(--hue), 20%, 80%);
  --atum-btn-hvr: var(--template-text-light);
  --atum-btn-success: var(--atum-btn-success-hvr);
  --atum-btn-success-icon: var(--template-text-light);
  --atum-btn-success-icon-hvr: var(--atum-btn-success-hvr);
  --atum-btn-success-bg: #3d854d;
  --atum-btn-success-bg-hvr: #25512f;
  --atum-btn-success-border: 1px solid #4da861;
  --atum-btn-success-hvr: var(--template-text-light);
  --atum-btn-danger: var(--dangerhvr);
  --atum-btn-danger-color: var(--template-text-light);
  --atum-btn-danger-icon: var(--template-text-light);
  --atum-btn-danger-icon-hvr: var(--atum-btn-danger-hvr);
  --atum-btn-danger-bg: #c52827;
  --atum-btn-danger-bg-hvr: #851b1a;
  --atum-btn-danger-border: 1px solid #da4645;
  --atum-btn-danger-hvr: var(--template-text-light);
  --atum-btn-info: #b1b7c0;
  --atum-btn-info-icon: var(--template-text-light);
  --atum-btn-info-icon-hvr: var(--template-text-dark);
  --atum-btn-info-bg: #33383f;
  --atum-btn-info-bg-hvr: #111215;
  --atum-btn-info-border: 1px solid #4a515b;
  --atum-btn-info-hvr: var(--template-text-dark);
  --atum-btn-primary: #31c3ff;
  --atum-btn-primary-icon: var(--template-text-light);
  --atum-btn-primary-icon-hvr: var(--template-text-dark);
  --atum-btn-primary-bg: #007db0;
  --atum-btn-primary-bg-hvr: #004764;
  --atum-btn-primary-border: 1px solid #00a1e3;
  --atum-btn-primary-hvr: var(--template-text-light);
  --atum-btn-secondary: #f0f4fb;
  --atum-btn-action: #1c1f23;
  --atum-btn-action-icon: var(--template-text-light);
  --atum-btn-action-icon-hvr: var(--atum-btn-action-hvr);
  --atum-btn-action-bg: #33383f;
  --atum-btn-action-bg-hvr: #111215;
  --atum-btn-action-border: 1px solid #4a515b;
  --atum-list-group-bg: var(--template-sidebar-bg);
  --atum-calendar-week-bg: #4b4b4b;
  --atum-calendar-bg: var(--body-bg);
  --atum-calendar-buttons-color: var(--btn-primary-bg);
  --atum-calendar-disabled-color: #4b4b4b;
  --atum-calendar-disabled-bg: #00000080;
  --atum-calendar-select-bg: #007db0;
  --atum-calendar-select-bg-url: url("../../images/select-bg-dark.svg?v=178429");
  --atum-calendar-select-bg-url-rtl: url("../../images/select-bg-dark-rtl.svg?v=178429");
  --permissions-sliders-oddCol-bg: var(--template-bg-dark);
  --respTable-border-bottom: 1em solid #007db0;
  --treeselect-line-color: #fff6;
  --treeselect-dropdown-toggle: var(--template-text-light);
  --icon-check-square-bg: #007db0;
  --icon-square-bg: #007db0;
  --icon-check-square-color: radial-gradient(white 50%, transparent 50%);
  --icon-square-color: radial-gradient(white 50%, transparent 50%);
  --drag-color: #151515;
  --drag-background-color: #c2d049;
  --text-muted-color: var(--template-text-light) !important;
  --text-muted-hvr-color: var(--template-text-dark) !important;
  --badge-bg-info: #fff3 !important;
  --info: #fff3 !important;
}

.border-primary {
  border-color: var(--primary) !important;
}

.border-secondary {
  border-color: var(--secondary) !important;
}

.border-success {
  border-color: var(--success) !important;
}

.border-info {
  border-color: var(--info) !important;
}

.border-warning {
  border-color: var(--warning) !important;
}

.border-danger {
  border-color: var(--danger) !important;
}

.border-light {
  border-color: var(--light) !important;
}

.border-dark {
  border-color: var(--dark) !important;
}

.border-white {
  border-color: var(--white) !important;
}

.text-primary {
  color: var(--primary) !important;
}

.text-secondary {
  color: var(--secondary) !important;
}

.text-success {
  color: var(--success) !important;
}

.text-info {
  color: var(--info) !important;
}

.text-warning {
  color: var(--warning) !important;
}

.text-danger {
  color: var(--danger) !important;
}

.text-light {
  color: var(--light) !important;
}

.text-dark {
  color: var(--dark) !important;
}

.text-black {
  color: var(--black) !important;
}

.text-white {
  color: var(--white) !important;
}

.text-body {
  color: var(--body-color) !important;
}

.bg-primary {
  background-color: var(--primary) !important;
}

.bg-secondary {
  background-color: var(--secondary) !important;
}

.bg-success {
  background-color: var(--success) !important;
}

.bg-info {
  background-color: var(--info) !important;
}

.bg-warning {
  background-color: var(--warning) !important;
}

.bg-danger {
  background-color: var(--danger) !important;
}

.bg-light {
  background-color: var(--light) !important;
}

.bg-dark {
  background-color: var(--dark) !important;
}

.bg-black {
  background-color: var(--black) !important;
}

.bg-white {
  background-color: var(--white) !important;
}

.bg-body {
  background-color: var(--body-bg) !important;
}

*, :after, :before {
  box-sizing: border-box;
}

body {
  background: var(--body-bg);
}

[data-bs-theme="dark"] body {
  background: var(--main-bg);
}

.header .h2 {
  margin-bottom: 0;
}

.j-install {
  flex-direction: column;
  min-height: 100vh;
  display: flex;
}

.j-install .logo {
  max-width: 270px;
}

.j-install img.logo-small, .j-install svg.logo-small {
  width: 2rem;
  height: 2rem;
}

.j-install .icon-question {
  color: #fff;
}

.j-install .icon-question:hover {
  color: var(--link-color);
}

.j-container {
  width: 100%;
  max-width: 40rem;
  margin: 1rem auto;
}

.j-container h1, .j-container .h1 {
  color: #001b4c;
  text-align: center;
}

.j-install-step {
  background-color: #fff;
  margin-bottom: 20px;
  display: none;
  box-shadow: 0 0 2px #343a431a, 0 2px 5px #343a4314, 0 5px 15px #343a4314;
}

.j-install-step.active {
  display: block;
}

.j-install-step select {
  width: 100%;
  margin-inline-start: 0 !important;
}

.j-install-step-header {
  color: #001b4c;
  background-color: #fff;
  border-bottom: 1px solid #efefef;
  padding: 10px 25px;
  font-size: 1.1rem;
  line-height: 2.4rem;
  position: relative;
}

.j-install-step-header span {
  margin-inline-end: 5px;
  font-size: 1.2rem;
  position: relative;
}

[data-bs-theme="dark"] .j-install-step {
  background-color: var(--secondary-bg);
}

[data-bs-theme="dark"] .j-install-step-header {
  color: #fff;
  background-color: var(--secondary-bg);
}

.j-install-step-form {
  padding: .65rem 1.2rem;
}

.j-install-step-form .control-group {
  margin-bottom: 0;
}

.j-install-step-form .control-group .control-label {
  padding: 0;
}

.languageForm {
  padding: 0 0 30px;
}

.languageForm .form-select, .languageForm .custom-select {
  width: 100%;
}

#system-message-container joomla-alert {
  margin-top: 25px;
}

.alert-heading {
  font-size: 1rem;
}

.hidden, [hidden="hidden"] {
  display: none;
}

.btn-primary {
  color: var(--btn-primary-color);
  background-color: var(--btn-primary-bg);
  border: var(--btn-primary-border);
}

.btn-primary:hover, .btn-primary:focus, .btn-primary:active {
  background-color: var(--btn-primary-bg-hvr);
  border: var(--btn-primary-border-hvr);
}

.btn-success {
  color: #fff;
  background-color: #75b544;
  border-color: #75b544;
}

.form-control.input-full {
  max-width: 100%;
}

[data-bs-theme="dark"] .form-control {
  background-color: var(--form-control-bg);
}

label {
  margin-bottom: .5rem;
}

.control-group {
  margin-bottom: 18px;
}

.control-group:after {
  clear: both;
  content: "";
  display: table;
}

.control-group .control-label {
  padding-top: 5px;
  text-align: start;
  padding-inline-end: 5px;
}

.spacer hr {
  width: 380px;
}

.card .form-select, .card .custom-select {
  width: 100%;
}

td .form-control {
  width: auto;
  display: inline-block;
}

legend {
  margin-bottom: 1.1rem;
}

.checkboxes {
  padding-top: 5px;
}

.checkboxes .checkbox input {
  margin-inline-start: 0;
  position: static;
}

.form-check {
  margin-bottom: 0;
  padding-top: 5px;
}

#jform_db_prefix {
  text-align: start;
  direction: ltr;
}

.form-select[multiple], [multiple].custom-select {
  background: none;
  height: auto;
}

.form-control-feedback {
  color: #c52827;
  display: block;
}

[data-bs-theme="dark"] .form-control-feedback {
  color: #d55;
}

caption {
  caption-side: top;
  margin: 0 1rem 0 0;
  padding: 0;
}

[data-bs-theme="dark"] caption {
  color: #fff;
}

.bg-warning {
  color: #000;
}

.footer {
  z-index: 1020;
  background-color: #fff;
  position: fixed;
  bottom: 0;
  left: 0;
  box-shadow: 0 0 1px 1px #00000040;
}

[data-bs-theme="dark"] .footer {
  background: var(--header-bg);
  box-shadow: none;
}

[data-bs-theme="dark"] .footer a {
  color: var(--table-link-color);
}

.table > :not(:last-child) > :last-child > * {
  border-bottom-color: var(--border-color);
}

.progresslabel {
  width: 100%;
}

progress {
  background: linear-gradient(#ccc 0%, #eee 50% 50%, #ccc 100%);
  border: 0;
  border-radius: 4px;
  width: 100%;
  height: 25px;
  margin: 1rem auto;
  display: block;
}

.j-header {
  color: #fff;
  background-color: #0b3257;
  flex-direction: row;
  display: flex;
}

.j-header-logo {
  width: 100%;
  max-width: 240px;
  margin: 20px;
}

.j-header-help {
  margin: 20px 20px 20px auto;
}

.j-header-help a {
  color: #fff;
  padding: 1rem;
  font-size: 2rem;
}

.j-footer {
  color: #fff;
  text-align: right;
  background-color: #0b3257;
  margin-top: auto;
  padding: 10px 20px;
  font-size: .8rem;
}

.alert {
  background: #fafafa;
  border: 1px solid #0000001a;
  margin: 0 0 20px;
  display: flex;
  position: relative;
  box-shadow: 0 0 10px #0000000d;
}

.alert p {
  margin-bottom: 0;
}

.alert-icon {
  opacity: .6;
  justify-content: center;
  align-items: center;
  width: 25%;
  margin-bottom: 0;
  font-size: 3rem;
  display: flex;
}

.alert-icon .fa {
  text-align: center;
  width: 100%;
}

.alert-text {
  padding: 10px 20px;
}

.j-installation-error {
  background-color: #1c3d5c;
  background-image: linear-gradient(90deg, #1c3d5c 0%, #2a5c8b 100%);
}
